local bie3 = GameInfoTypes.BELIEF_ITG_ENHANCER_3;
local bie4 = GameInfoTypes.BELIEF_ITG_ENHANCER_4;
local bie5 = GameInfoTypes.BELIEF_ITG_ENHANCER_5;
local bie11 = GameInfoTypes.BELIEF_ITG_ENHANCER_11;

function ITGBeliefRemoved(iPlayer, iRel, iBelief)
	if iBelief == GameInfoTypes.BELIEF_ITG_ENHANCER_11 then
		ClearReligionPressureFromFollowers(iRel)
	end
	ITGBeliefAddedToPlayer(iPlayer, iRel, iBelief, -1, false)
end

function ITGBeliefAddedToPlayer(iPlayer, iRel, iBelief, iAdd, bAdd)
	if iBelief == bie4  then
		gT.Players[iPlayer].RelBonusMod = gT.Players[iPlayer].RelBonusMod - 0.15 * iAdd;
	elseif iBelief == bie3 then
		Players[iPlayer]:ChangePlayerCombatModifierNearHolySite(20 * iAdd);
	elseif iBelief == bie5 then
		for iCity in Players[iPlayer]:Cities() do
			iCity:ChangeBaseReligionPressure(iRel, 6 * iAdd)
		end
	end
end

function ITGBeliefAdded(iPlayer, iRel, iBelief)
	if iBelief == bie11 then
		UpdateAllCitiesPressureFromFollowers()
	end
	ITGBeliefAddedToPlayer(iPlayer, iRel, iBelief, 1, true)
end

function CityAcquiredBeliefBonuses(iCity, oldPlayer, newPlayer)
	local oldRel = Players[oldPlayer]:GetReligionCreatedByPlayer();
	local newRel = Players[newPlayer]:GetReligionCreatedByPlayer();
	if IsReligionHaveBelief(oldRel, GameInfoTypes.BELIEF_ITG_ENHANCER_5) then
		iCity:ChangeBaseReligionPressure(oldRel, -6)
	end
	if IsReligionHaveBelief(newRel, GameInfoTypes.BELIEF_ITG_ENHANCER_5) then
		iCity:ChangeBaseReligionPressure(newRel, 6)
	end
end

function ITGNewReligion(iPlayer, iRel, bPantheon)
	for i, belief in pairs(Game.GetBeliefsInReligion(iRel)) do
		if bPantheon or (not GameInfo.Beliefs[belief].Pantheon) then
			ITGBeliefAdded(iPlayer, iRel, belief)
		end
	end
end

function ITGNewPantheon(player,  pantheonbelief)
	ITGBeliefAdded(iPlayer, nil, pantheonbelief)
	if gT.Players[player].HarmonyPantheon then
		if Players[player]:GetReligionCreatedByPlayer() > 0 or #Game.GetBeliefsInReligion(0, player) > 4 then
			gT.Players[player].HarmonyPantheon = nil;
		end
	end
	for l, s in pairs(gT.Leagues) do
		if s.Resolutions[37] then
			if s.Members[player] then
				gT.Leagues[l].Pantheons[pantheonbelief] = 1;
			end
		end
	end
	if gT.Globals.PantheonCost then
		gT.Globals.PantheonCost  = gT.Globals.PantheonCost + 5;
		if #Game.GetBeliefsInReligion(0, player) > 4 then
			if not gT.Players[player].AfterRelTresholdScale then
				gT.Players[player].AfterRelTresholdScale = true;
				gT.Players[player].ReligionRewardTreshold = math.ceil(gT.Globals.PantheonCost / 100)
			end
		end
	end
	if gT.Globals.GameGenerated then
		DoBoostUnitOfTypeCS(player, GameInfoTypes.UNIT_ITG_AOT_WARRIOR_UU_0, 1)
	end
end

function IsValidToGainExtraRangedStrikeRel(iCity, iPlayer)
	local iRel = iCity:GetReligiousMajority();
	if IsReligionHaveBelief(iRel, GameInfoTypes.BELIEF_ITG_ENHANCER_13) then
		local rPlayer = Game.GetFounder(iRel, -1);
		if rPlayer and Players[rPlayer] then
			if not Teams[Players[iPlayer]:GetTeam()]:IsAtWar(Players[rPlayer]:GetTeam()) then
				return true;
			end
		end
	end
	return false;
end

function IsValidToLooseExtraRangedStrikeRel(iCity, iPlayer)
	local iRel = iCity:GetReligiousMajority();
	if IsReligionHaveBelief(iRel, GameInfoTypes.BELIEF_ITG_ENHANCER_14) then
		local rPlayer = Game.GetFounder(iRel, -1);
		if rPlayer and Players[rPlayer] then
			if Teams[Players[iPlayer]:GetTeam()]:IsAtWar(Players[rPlayer]:GetTeam()) then
				return true;
			end
		end
	end
	return false;
end

function DoTurnBeliefBonuses(iPlayer)
	local iRel = Players[iPlayer]:GetReligionCreatedByPlayer()
	if IsReligionHaveBelief(iRel, GameInfoTypes.BELIEF_ITG_ENHANCER_9) then
		local cRg = (gT.Players[iPlayer].EnhancerBelief9 or LastRandom(0,5)) + 1;
		if cRg >= 8 then
			DoRandomMissionaryConversion(iPlayer, iRel)
			cRg = 0;
		end
		gT.Players[iPlayer].EnhancerBelief9 = 0;
	end
	for iCity in Players[iPlayer]:Cities() do
		local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
		if IsValidToGainExtraRangedStrikeRel(iCity, iPlayer) then
			if LastRandom(1,20) <= 3 then
				gT.Cities[cID].NumAttacks = gT.Cities[cID].NumAttacks + 1;
			end
		end
		if IsValidToLooseExtraRangedStrikeRel(iCity, iPlayer) then
			if LastRandom(1,4) == 3 then
				SpendAttackCityRangedStrike(iPlayer, iCity, cID)
			end
		end
		if IsBeliefPresentInACity(iCity, GameInfoTypes.BELIEF_ITG_PANTHEON_16) then
			if LastRandom(1,20) == 3 then
				gT.Cities[cID].NumAttacks = gT.Cities[cID].NumAttacks + 1;
			end
		end
	end
	UpdatePlayerCitiesPressureFromFollowers(iPlayer)
end

function DoRandomMissionaryConversion(iPlayer, iRel)
	local hTab = {}
	local iTeam = Players[iPlayer]:Cities();
	for i, v in pairs(Players) do
		for iCity in v:Cities() do
			if iCity:Plot():IsRevealed(iTeam) then
				if not iCity:IsSpreadImmune() then
					table.insert(hTab, iCity)
				end
			end
		end
	end
	if #hTab > 0 then
		local lCity = hTab[LastRandom(1,#hTab)];
		gT.Players[iPlayer].FreeMissioniares = gT.Players[iPlayer].FreeMissioniares + 1;
		DoMissionaryConversionOnCity(iPlayer, lCity)	
	end
end

function PrepareValidReligionsPressureFromFollowers()
	local hTab = {}
	for i, v in pairs(Players) do
		local iRel = v:GetReligionCreatedByPlayer();
		if iRel > 0 then
			if IsReligionHaveBelief(iRel, GameInfoTypes.BELIEF_ITG_ENHANCER_11) then
				table.insert(hTab, iRel)
			end
		end
	end
	return hTab;
end

function UpdateAllCitiesPressureFromFollowers(bTab)
	local gTab = bTab or PrepareValidReligionsPressureFromFollowers()
	for i,v in pairs(Players) do
		UpdatePlayerCitiesPressureFromFollowers(i, gTab)
	end
end

function UpdatePlayerCitiesPressureFromFollowers(iPlayer, bTab)
	local gTab = bTab or PrepareValidReligionsPressureFromFollowers()
	for iCity in Players[iPlayer]:Cities() do
		UpdateCityPressureFromFollowers(iCity, gTab)
	end
end

function UpdateCityPressureFromFollowers(iCity, bTab)
	local gTab = bTab or PrepareValidReligionsPressureFromFollowers()
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
	for i, iRel in pairs(gTab) do
		local cBonus = gT.Cities[cID].RelPressureFromFollowers[iRel] or 0;
		local sBonus = iCity:GetNumFollowers(iRel);
		if sBonus ~= cBonus then
			iCity:ChangeBaseReligionPressure(iRel, sBonus - cBonus)
			gT.Cities[cID].RelPressureFromFollowers[iRel] = sBonus;
		end
	end
end

function ClearReligionPressureFromFollowers(iRel)
	for i, v in pairs(Players) do
		for iCity in v:Cities() do
			local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
			if gT.Cities[cID].RelPressureFromFollowers[iRel] then
				iCity:ChangeBaseReligionPressure(iRel, -gT.Cities[cID].RelPressureFromFollowers[iRel])
				gT.Cities[cID].RelPressureFromFollowers[iRel] = 0;
			end
		end
	end
end

function ShouldHaveBeliefPromotion20NearHolyCity(iUnit, iPlayer)
	if IsPlayerHaveBelief(iPlayer, GameInfoTypes.BELIEF_ITG_ENHANCER_4) then
		local iRel = Players[iPlayer]:GetReligionCreatedByPlayer();
		for iplotIndex, data in pairs(gT.HolySites) do
			if data.ActiveReligions[iRel] then
				local hPlot = Map.GetPlotByIndex(iplotIndex);
				if Map.PlotDistance(hPlot:GetX(), hPlot:GetY(), iUnit:GetX(), iUnit:GetY()) <= 2 then
					return true;
				end
			end
		end
	end
	return false;
end

function IsBeliefPresentInACity(iCity, iBelief)
	local iRel = iCity:GetReligiousMajority();
	if iRel == -1 then
		return false;
	elseif iRel == 0 then
		if Players[iCity:GetOwner()]:HasCreatedPantheon() then
			if IsReligionHaveBelief(0, iBelief, iCity:GetOwner()) then
				return true;
			end
		end
	elseif IsReligionHaveBelief(iRel, iBelief) then
		return true;
	end
	if iBelief == iCity:GetSecondaryReligionPantheonBelief() then
		return true;
	end
	return false;
end

function IsWithinProperDistanceForKillingBeliefBonus(unitX, unitY, iBelief, iPlayer, dis)
	for iCity in Players[iPlayer]:Cities() do
		if IsBeliefPresentInACity(iCity, iBelief) then
			if Map.PlotDistance(unitX, unitY, iCity:GetX(), iCity:GetY()) <= dis then
				return true;
			end
		end
	end
	return false;
end

function IsWithinProperDistanceForHappinessBeliefBonus(unitX, unitY, iBelief, iPlayer, dis)
	local iCity;
	local mindis = 4;
	for tCity in Players[iPlayer]:Cities() do
		if IsBeliefPresentInACity(tCity, iBelief) then
			if Map.PlotDistance(unitX, unitY, tCity:GetX(), tCity:GetY()) <= mindis then
				mindis = Map.PlotDistance(unitX, unitY, tCity:GetX(), tCity:GetY());
				iCity = tCity;
			end
		end
	end
	if iCity then
		ChangeITGHappinessInTheCity(iCity, 1, nil, "okokokce")
		return true;
	else
		return false;
	end
end