local PromTableTER = {}
local ClassToLogPromotions = {}
local FeatureToLogPromotions = {
[fMarsh] = 1,
[fJungle] = 2,
[fForest] = 3
}
local TerrainToLogPromotions = {
[tTundra] = 5,
[tPlains] = 6,
[tGrass] = 7,
[tSnow] = 8,
[tDesert] = 9
}
local DomainToLogPromotions = 
{
[dSea] = 25,
[dLand] = 26,
[dAir] = 27,
}

function DoCacheVeteranPromotionData()
	local promtoVetID = {}
	local VetTypetoVetID = {}
	for row in DB.Query("SELECT * FROM VeteranPromotions") do
		local index = row.ID;
		PromTableTER[index] = {UnitCombatTypes = {}}
		for a, b in pairs(row) do
			PromTableTER[index][a] = b
		end
		promtoVetID[PromTableTER[index].PromotionType] = index;
		VetTypetoVetID[PromTableTER[index].Type] = index;
		local unitpromotionID = GameInfo.UnitPromotions[PromTableTER[index].PromotionType].ID
		PromTableTER[index].PromotionType = unitpromotionID;
	end
	for row in DB.Query("SELECT * FROM VeteranPromotionScenarios") do
		local vetID = VetTypetoVetID[row.VeteranPromotionType];
		if not PromTableTER[vetID].Scenarios then
			PromTableTER[vetID].Scenarios = {};
		end
		PromTableTER[vetID].Scenarios[ITG_ScenariosTypes[row.ScenarioType]] = true;
	end
	for row in GameInfo.UnitPromotions_UnitClasses() do
		local promvetid = promtoVetID[row.PromotionType];
		if promvetid then
			ClassToLogPromotions[GameInfo.UnitClasses[row.UnitClassType].ID] = promvetid;
		end
	end
	for row in GameInfo.UnitPromotions_UnitCombats() do
		local promvetid = promtoVetID[row.PromotionType];
		if promvetid then
			PromTableTER[promvetid].UnitCombatTypes[GameInfoTypes[row.UnitCombatType]] = true;
		end
	end
end
DoCacheVeteranPromotionData()

local tRagingFire = {}
local tAch112 = 0;
local asiainno2 = GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ASIA_2;

GameEvents.CanHavePromotion.Add(function(ownerId, unitId, ePromotion)
	print("CanHavePromotion still fires", ownerId, unitId, ePromotion)
	return true;
end)

function AwardWithRandomVeteranProms(iUnit, amount)
	local tab = GetAvailableVeteranPromotion(iUnit:GetOwner(), iUnit)
	tab = Shuffle(tab)
	for i = 1, amount do
		if tab[i] then
			iUnit:SetHasPromotion(tab[i], true)
			UnitITGTookPromotionVeteran(iUnit:GetOwner(), iUnit, tab[i], true)
		else
			break;
		end
	end
end

function ChangeAILogPromotionCount(g, domain, type)
	Players[g]:ChangeVetDomainLog(type, domain, 1)
end

function DoAILogForVeteranPromotionAfterCombat(g, aUnit, iPlot, domain)
	if (not gT.Players[g]) then
		return
	end
	if iPlot:IsHills() then
		ChangeAILogPromotionCount(g, domain, 4)
	end
	local tType = iPlot:GetTerrainType();
	if TerrainToLogPromotions[tType] then
		ChangeAILogPromotionCount(g, domain, TerrainToLogPromotions[tType])
	end
	local fType = iPlot:GetFeatureType();
	if FeatureToLogPromotions[fType] then
		ChangeAILogPromotionCount(g, domain, FeatureToLogPromotions[fType])
	end
	if aUnit then
		local cClass = aUnit:GetUnitClassType();
		if ClassToLogPromotions[cClass] then
			ChangeAILogPromotionCount(g, domain, ClassToLogPromotions[cClass])
		end
		local dDomain = aUnit:GetDomainType();
		if DomainToLogPromotions[dDomain] then
			ChangeAILogPromotionCount(g, domain, DomainToLogPromotions[dDomain])
		end
	end
end

function CanUnitAcquireVeteranPromsAtAll(iUnit, iPlayer)
	if ((not gT.Players[iPlayer]) or (not gT.Players[iPlayer].UnitClassXP[iUnit:GetUnitClassType()])) then
		return false;
	end
	local uilevel = iUnit:GetLevel();
	if uilevel < 4 then
		return false;
	end
	return ((uilevel % 2) == 0)
end

function GetNumVeterancyPromotion(iUnit)
	local c = 0;
	for i, data in pairs(PromTableTER) do
		if iUnit:IsHasPromotion(data.PromotionType) then
			c = c + 1;
		end
	end
	return c;
end

function DoUpdateHPInstantlyOnVeteranProm(iPlayer, iVal)
	for iUnit in Players[iPlayer]:Units() do
		local numhp = iVal * 7 * GetNumVeterancyPromotion(iUnit)
		if numhp ~= 0 then
			iUnit:ChangeMaxHitPoints(numhp, true)
		end
	end
end

local techinnomeso8 = GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_MESOAMERICAN_8
function UnitITGTookPromotionVeteran(iPlayer, iUnit, iPromotion, bAsBonus)
	if HasPlayerTech(iPlayer, asiainno2, "adddddddddddqwe") then
		iUnit:ChangeMaxHitPoints(7, true)
	end
	if not bAsBonus then
		if HasPlayerTech(iPlayer, techinnomeso8, "dawewqec") then
			if Players[iPlayer]:GetReligionCreatedByPlayer() > 0 then -- otherwise bad implications
				gT.Players[iPlayer].FreeMissioniares = gT.Players[iPlayer].FreeMissioniares + 1;
				if iPlayer == Game.GetActivePlayer() then
					Events.OpenInfoCorner();
				end
			end
		end
 		gT.Players[iPlayer].LHNumVetProms = gT.Players[iPlayer].LHNumVetProms + 1;
		VictoryTaskCompletion(iPlayer, 42)
		TakeCareOfEventMorale(iPlayer, 14)
		DoLogDominationEvent(-1, iPlayer, nil, 10)
		if Players[iPlayer]:IsHuman() then
			LuaEvents.ITG_AchievementCompleted(53)
		end
	end
end

GameEvents.UnitPromoted.Add(function(ownerId, unitId, ePromotion)
	print("UnitPromoted should no longer fire", ownerId, unitId, ePromotion)
end)

GameEvents.UnitPromotionReady.Add(function(ownerId, unitId)
	DoTestITGVeteranPromotion(ownerId, unitId)
end)

function IsVeteranPromotionValid(iPlayer, iUnit, eVetID)
	if PromTableTER[eVetID].Disabled then
		return false;
	end
	if PromTableTER[eVetID].Scenarios and (not PromTableTER[eVetID].Scenarios[gT.Globals.ScenarioType]) then
		return false;
	end
	if not PromTableTER[eVetID].UnitCombatTypes[iUnit:GetUnitCombatType()] then
		return false;
	end
	if iUnit:IsHasPromotion(PromTableTER[eVetID].PromotionType) then
		return false;
	end
	return true;
end

function GetAvailableVeteranPromotion(iPlayer, iUnit, bBest)
	local domain = iUnit:GetDomainType();
	local hTab = {}
	local max = 0;
	for i, data in pairs(PromTableTER) do
		if IsVeteranPromotionValid(iPlayer, iUnit, i) then
			if bBest then
				local flav = math.max(data.BaseAIeval, data.AIeval * Players[iPlayer]:GetVetDomainLog(i, domain));
				if flav > max then
					max = flav;
					hTab = {};
				end
				if flav == max then
					table.insert(hTab, data.PromotionType)
				end
			else
				table.insert(hTab, data.PromotionType)
			end
		end
	end
	return hTab;
end

function UI_RequestVetProm(iPlayer, unitId)
	local iUnit = Players[iPlayer]:GetUnitByID(unitId);
	DoTestITGVeteranPromotion(iPlayer, unitId)
	if iUnit:GetExperience() >= iUnit:ExperienceNeeded() then
		MapModData.ITGSG = GetAvailableVeteranPromotion(iPlayer, iUnit);
	end
end
LuaEvents.ITGSG_RequestVetUnitPromData.Add(UI_RequestVetProm)

LuaEvents.ITG_PlayerChoosesVetPromotion.Add(function(iUnit, ePromotion)
	local iPlayer = iUnit:GetOwner();
	iUnit:SetHasPromotion(ePromotion, true)
	DoUnitLevelUp(iPlayer, iUnit)
	UnitITGTookPromotionVeteran(iPlayer, iUnit, ePromotion)
end)

function DoTestITGVeteranPromotion(iPlayer, unitId)
	local iUnit = Players[iPlayer]:GetUnitByID(unitId);
	if iUnit:GetExperience() >= iUnit:ExperienceNeeded() then
		local bLevelUP = true;
		if CanUnitAcquireVeteranPromsAtAll(iUnit, iPlayer) then
			local avTab = GetAvailableVeteranPromotion(iPlayer, iUnit, true)
			if #avTab > 0 then
				if (not Players[iPlayer]:IsHuman()) or (gT.Players[iPlayer].MasterOfArmy) then
					avTab = Shuffle(avTab)
					iUnit:SetHasPromotion(avTab[1], true)
					UnitITGTookPromotionVeteran(iPlayer, iUnit, avTab[1])
				else
					bLevelUP = false; -- human has to interact with UI
				end
			end
		end
		if bLevelUP then
			DoUnitLevelUp(iPlayer, iUnit)
			DoTestITGVeteranPromotion(iPlayer, unitId)
		end
	end
end

function DoUnitLevelUp(iPlayer, iUnit)
	iUnit:ChangeLevel(1)
	if gT.Players[iPlayer] then
		local uID = iUnit:GetID() .. "," .. iPlayer .. "," .. iUnit:GetGameTurnCreated();
		AdjustCombatBonusFromLevel(iUnit, uID, iPlayer)
		if gT.Globals.GameGenerated then
	 		gT.Players[iPlayer].LHNumUnitLevelsUP = gT.Players[iPlayer].LHNumUnitLevelsUP + 1;
		end
		if Players[iPlayer]:IsHuman() then
			if iUnit:GetLevel() == 10 then
				LuaEvents.ITG_AchievementCompleted(124)
			end
		end
	end
end

function AIGoThroughUnitsVeteranProm(iPlayer)
	-- this fires only for human AI check
	for iUnit in Players[iPlayer]:Units() do
		DoTestITGVeteranPromotion(iPlayer, iUnit:GetID())
	end
end

GameEvents.UnitPillagedImprovement.Add(function(unitID, iPlayer, x, y, improvement, goldearned)
	local iPlot = Map.GetPlot(x, y)
	local indexPlot = iPlot:GetPlotIndex();
	if gT.Manufactories[indexPlot] then
		ManufactoryPillaged(indexPlot, iPlayer)
	end
	if gT.Players[iPlayer] then
		BasicCSMissionCompleteFunc(iPlayer, 44, 1);
		local iUnit = Players[iPlayer]:GetUnitByID(unitID)
		local unitType = iUnit:GetUnitType();
		local iActivePlayer = Game.GetActivePlayer();
		local uID = unitID .. "," .. iPlayer .. "," .. iUnit:GetGameTurnCreated();
		gT.Players[iPlayer].UnitsByID[uID].LastPillage = GetITGGameTurn();
		if gT.Players[iPlayer].CultBonusOn[195] then
			if LastRandom(1, 20) == 3 then
				local iOwner = iPlot:GetOwner();
				if gT.Players[iOwner] and (iOwner ~= iPlayer) then
					TakeCareOfEventMorale(iPlayer, 25, {-1, "Pillaged Improvemnt"})
					if iOwner == iActivePlayer then
						Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(x, y))), "-1 [ICON_ITG_MORALE]", 0.5)
					end
				end
			end
			local iCity = GetNearestCityOfPlayerToPlot(x, y, iPlayer, 999)
			if iCity then
				local bFood = LastRandom(8, 20);
				local bWorkrate = LastRandom(25, 55);
				iCity:ChangeFood(bFood);
				ChangeITGWorkRate(iCity, bWorkrate)
				if iPlayer == Game.GetActivePlayer() then
					Events.GameplayAlertMessage("You have plundered " .. bFood .. " [ICON_FOOD] Food from " .. Locale.Lookup(GameInfo.Improvements[improvement].Description) .. " to " .. iCity:GetName() .. ".")
					Events.GameplayAlertMessage("You have plundered " .. bWorkrate .. " [ICON_WORKER] Work Rate from " .. Locale.Lookup(GameInfo.Improvements[improvement].Description) .. " to " .. iCity:GetName() .. ".")
				end
			end
		end
		if unitType == GameInfoTypes.UNIT_ITG_AOT_HORSEMAN_UU_5 then
			iUnit:ChangeMaxHitPoints(4, true)
		elseif unitType == GameInfoTypes.UNIT_ITG_AOT_CHARIOT_UU_0 then
			BasicRangedStrikeAtTurnStart(iUnit, iPlayer, 2, false, nil, true)
		elseif unitType== GameInfoTypes.UNIT_ITG_AOT_SWORDSMAN_UU_6 then
			tRagingFire = {}
			tAch112 = 1;
			DoPillageSpreadFrom(Players[iPlayer]:GetTeam(), Map.GetPlot(x, y), 20)
			if tAch112 >= 5 then
				if Players[iPlayer]:IsHuman() then
					LuaEvents.ITG_AchievementCompleted(112)
				end
			end
		elseif unitType == GameInfoTypes.UNIT_ITG_IND_CUIRASS_UU_2 then
			iUnit:ChangeExperience(7)
			if LastRandom(1,100) <= 17 then
				local tPlayer = iPlot:GetOwner();
				if gT.Players[tPlayer] and (tPlayer ~= iPlayer) then
					TakeCareOfEventMorale(tPlayer, 25, {-1, "Improvement Pillaged"})
				end
			end
		elseif unitType == GameInfoTypes.UNIT_ITG_TRIAL_HORSEMAN_5 then
			DoDamageAdjacentPlots(iPlayer, 10, x, y, true)
		elseif unitType == GameInfoTypes.UNIT_ITG_NW_NATIVE_KNIGHT then
			DemoralizeAdjacentEnemiesWithChance(x, y, iPlayer, 20)
		elseif unitType == GameInfoTypes.UNIT_ITG_TANK_UU_1 then
			local lbl = LastRandom(1, 4);
			if lbl == 1 then
				iUnit:ChangeMoves(180);
			elseif lbl == 2 then
				iUnit:ChangeDamage(-15);
			elseif lbl == 3 then
				iUnit:ChangeExperience(8)
			elseif lbl == 4 then
				iUnit:ChangeTemporaryCombatStrength(8);	
			end
		elseif unitType == GameInfoTypes.UNIT_ITG_IND_LANCER_UU_4 then
			if not gT.Players[iPlayer].UnitsByID[uID].TempSipahi then
				ChangeITGBaseCombatStrength(iUnit, 4, nil)
				gT.Players[iPlayer].UnitsByID[uID].TempSipahi = true;
			end
		end
		if HasPlayerTech(iPlayer, GameInfoTypes.TECH_UNIQUE_ITG_MODERN_30, "kofoewkfoe") then
			if iUnit:GetUnitClassType() == GameInfoTypes.UNITCLASS_PARATROOPER then
				DoDamageAdjacentPlots(iPlayer, 10, x, y, true)
			end
		end
		local sResGain = GetResourceStoredFromPillage(iPlayer, iPlot);
		if sResGain and (iPlayer == iActivePlayer) then
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(x, y))), sResGain, 0.5)
		end


		gT.Players[iPlayer].PillagedImpros = gT.Players[iPlayer].PillagedImpros + 1;
		if HasPlayerTech(iPlayer, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_PAGAN_5 ,"Daed") then
			if gT.Players[iPlayer].PillagedImpros % 8 == 0 then -- this after count is updated!
				ChangeExtraSupply(iPlayer, 1)
			end
			if LastChance(15) then
				TakeCareOfEventMorale(iPlayer, 25, {1, "Improvement Pillaged"})
			end
			local iCity = GetNearestCityOfPlayerToPlot(x, y, iPlayer, 999)
			if iCity and (goldearned > 0) then
				iCity:ChangeFood(goldearned);
				iCity:ChangeProduction(goldearned);
				if iPlayer == Game.GetActivePlayer() then
					Events.GameplayAlertMessage("You have plundered " .. goldearned .. " [ICON_FOOD] Food from " .. Locale.Lookup(GameInfo.Improvements[improvement].Description) .. " to " .. iCity:GetName() .. ".")
					Events.GameplayAlertMessage("You have plundered " .. goldearned .. " [ICON_PRODUCTION] Production from " .. Locale.Lookup(GameInfo.Improvements[improvement].Description) .. " to " .. iCity:GetName() .. ".")
				end
			end
		end
		
		gT.Players[iPlayer].PillagedImprosGold = gT.Players[iPlayer].PillagedImprosGold + goldearned;
		if gT.Academies[indexPlot] then
			if gT.Academies[indexPlot].Rank == 1 and (Players[iPlayer]:IsHuman()) then
				LuaEvents.ITG_AchievementCompleted(107)
			end
		end
	end
end)

function GetTabOfVeteranPromAll(cv)
	local tab = {}
	for i, v in pairs(PromTableTER) do
		if not v.Disabled then
			table.insert(tab, v.PromotionType)
		end
	end
	return tab;
end

function DoPillageSpreadFrom(iTeam, iPlot, chance)
	tRagingFire[iPlot:GetPlotIndex()] = true;
	for j = 0, 5 do
		local jPlot = Map.GetPlotByIndex(iPlot:GetX(), iPlot:GetY(), j);
		if jPlot and (jPlot:GetOwner() ~= -1) and (Teams[Players[jPlot:GetOwner()]:GetTeam()]:IsAtWar(iTeam)) then
			if (not tRagingFire[jPlot:GetPlotIndex()]) and (LastRandom(1, 100) <= chance) then
				if jPlot:IsCity() then
					jPlot:GetPlotCity():ChangeDamage(25);
					DoPillageSpreadFrom(iTeam, jPlot, chance)
				elseif jPlot:GetImprovementType() ~= -1 and (not jPlot:IsImprovementPillaged()) then
					jPlot:SetImprovementPillaged(true)
					tAch112 = tAch112 + 1;
					DoPillageSpreadFrom(iTeam, jPlot, chance)
				end
			end
		end
	end
end