local typesOfEpidemics = {"Plague", "Typhus", "Smallpox", "Measles", "Malaria", "Yellow Fever", "Influenza", "Dengue Fever", "Cholera", "Leprosy"}
local tImmuneInno = GameInfoTypes.TECH_UNIQUE_ITG_INDUSTRIAL_2;
local arabtrial4 = GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ARABIA_4;

function CityLethalityModifier(iCity, cID, iPlayer)
	local base = gT.Cities[cID].Health
	base = base + GetNumCityAcademiesWithAbility(iCity, 28)
	if HasPlayerNatureWonder(iPlayer, GameInfoTypes.FEATURE_FOUNTAIN_YOUTH) then
		base = base + 7;
	end
	return base;
end

function GreatOverTurnPlague()
	local gTurn = GetITGGameTurn();
--existing plagues
	for g, h in pairs(gT.Players) do	
		for iCity in Players[g]:Cities() do
			local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
			if iCity:IsPlague() then
				for epid, data in pairs(gT.Cities[cID].Epidemies) do
					if data.Active then
						if iCity:GetPopulation() > 1 then
							if LastRandom(1,100) <= gT.Globals.PlaguesByID[epid].Lethal - CityLethalityModifier(iCity, cID, g) then
								iCity:ChangePopulation(-1, true)
								ChangeITGCityInteger(iCity, "CI_PLAGUE_POP_LOST", 1)
								gT.Globals.PlaguesByID[epid].NumKilled = gT.Globals.PlaguesByID[epid].NumKilled + 1;
								DoTriggerGreatPeopleKillFromEpidemy(iCity, (100/(iCity:GetPopulation() + 1)))
								UpdateHealthInSingleCity(iCity, g)
							end
						end
						if data.End <= gTurn then
							gT.Cities[cID].Epidemies[epid].Active = false;
							gT.Cities[cID].HealthFromEpidemy = gT.Cities[cID].HealthFromEpidemy + 1;
							if h.CultBonusOn[80] then
								gT.Cities[cID].HealthFromEpidemy = gT.Cities[cID].HealthFromEpidemy + 1;
							end
							CheckToRemoveEpidemy(iCity, cID)
							UpdateHealthInSingleCity(iCity, g)
						end
					end
					if data.EndContagious and (data.EndContagious >= gTurn) then
						if data.Outbreak and (data.Outbreak ~= gTurn) then
							DoSpreadPlagueOnTurn(iCity, cID, g, epid, data.Outbreak == (gTurn - 1))
						end
					end
				end 
			end
		end
	end
-- eventual new plagues
	if gT.Globals.PlaguesOutbreaks[gTurn] then
		for i = 1, gT.Globals.PlaguesOutbreaks[gTurn] do
			CreateNewPlagueIfPossible(gTurn)
		end
	end
end

function PlaguesInitialData()
	gT.Globals.PlaguesByID = {}
	gT.Globals.PlaguesOutbreaks = {}
	local numPlagues = DBScenario.EstPlagueOutbreaks + LastRandom(-1, 2);
	--one plague per 8 cities;
	local nCities = 0;
	for g, gd in pairs(gT.Players) do
		nCities = nCities + Players[g]:GetNumCities();
	end
	local numPlaguesPerCity = math.ceil(nCities / EST_PLAGUE_PER_CITIES);
	if numPlaguesPerCity < numPlagues then
		numPlagues = numPlaguesPerCity;
	elseif numPlaguesPerCity > numPlagues + 2 then
		numPlagues = numPlagues + LastRandom(0, 1);
	end
	if numPlagues < 1 then
		numPlagues = 1;
	end
	for i = 1, numPlagues do
		local turn = LastRandom(3, 97);
		gT.Globals.PlaguesOutbreaks[turn] = (gT.Globals.PlaguesOutbreaks[turn] or 0) + 1;
	end
	InitialCitiesNearbyCache()
end

function StartWeakPlagueInCity(iCity)
	StartPlagueInCity(iCity, GetITGGameTurn(), {Contagion = LastRandom(5, 25), Duration = LastRandom(1, 2), Lethal = LastRandom(1, 5)})
end

function StartPlagueInCity(iCity, gTurn, exData)
	local fData = exData or {}
	local badTypes = {}
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
	for id, epidemydata in pairs(gT.Cities[cID].Epidemies) do
		badTypes[epidemydata.Type] = 1;
	end
	local randomtype = {}
	for i, string in pairs(typesOfEpidemics) do
		if not badTypes[string] then
			table.insert(randomtype, string)
		end
	end
	if #randomtype == 0 then
		return;
	end
	local type = RandomFromTab(randomtype);
	local id = 1 + #gT.Globals.PlaguesByID;
	local contagious = fData.Contagion or (33 + LastRandom(0, 17) + LastRandom(1,5) * DBScenario.PlagueStrength);
	local duration = fData.Duration or (1 + LastRandom(0, 1) + LastRandom(1, math.floor(DBScenario.PlagueStrength/3)));
	local lethal = fData.Lethal or (5 + LastRandom(4, 7) * DBScenario.PlagueStrength);
	table.insert(gT.Globals.PlaguesByID, {Origin = iCity:GetName(), Outbreak = gTurn, Type = type, NumKilled = 0, NumCities = 0, Contagious = contagious, Duration = duration, Lethal = lethal})
--initial strike :)
	DoOutbreakOfEpidemy(iCity, id, true)	
end

function CreateNewPlagueIfPossible(gTurn)
--choose city
	local cityTab = {}
	for g, gd in pairs(gT.Players) do
		for iCity in Players[g]:Cities() do
			local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
			local base = 15 - gT.Cities[cID].Health;
			if gT.Cities[cID].HealthLevel < 0 then
				base = base + math.pow(2, math.ceil(- gT.Cities[cID].HealthLevel));
			end
			if gd.CultBonusOn[193] then
				base = math.floor(0.75 * base)
			end
			table.insert(cityTab, {iCity, math.max(1, base)})
		end
	end
	local iCity = GetRandomFromBiasedChanceTable(cityTab)
	StartPlagueInCity(iCity, gTurn)
end

function DoOutbreakOfEpidemy(iCity, EpidemyID, bTruelyNew, sReason, sCity)	
	local iPlayer = iCity:GetOwner();
	gT.Globals.PlaguesByID[EpidemyID].NumCities = gT.Globals.PlaguesByID[EpidemyID].NumCities + 1;
	if not bTruelyNew then
		gT.Globals.PlaguesByID[EpidemyID].Contagious = gT.Globals.PlaguesByID[EpidemyID].Contagious - 1; -- weaker and weaker :)
	end
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
	if not iCity:IsPlague() then
		DoStartPlagueInCityDLL(iCity)
	end
	gT.Cities[cID].Tplagues = gT.Cities[cID].Tplagues + 1
	--so contagious of population * (lethal - health)
	local numCitizensAffected = math.min(iCity:GetPopulation() - 1, iCity:GetPopulation() * (gT.Globals.PlaguesByID[EpidemyID].Contagious + LastRandom(-5, 5)) / 100)
	local ck = iCity:GetFood()/ iCity:GrowthThreshold();
	local sp = iCity:GetPopulation();
	iCity:SetFood(0)
	DoIncreasementStockExchange(5)
	local remc = 0;
	local chanceToDie = gT.Globals.PlaguesByID[EpidemyID].Lethal - CityLethalityModifier(iCity, cID, iCity:GetOwner());
	for i = 1, numCitizensAffected do
		if LastRandom(1, 100) < chanceToDie then
			remc = remc + 1;
		end
	end
	if HasPlayerTech(iPlayer, arabtrial4, "arabtrial4") then
		remc = math.min(1, remc);
	end
	if remc > 0 then
		iCity:ChangePopulation(-remc, true)
		gT.Globals.PlaguesByID[EpidemyID].NumKilled = gT.Globals.PlaguesByID[EpidemyID].NumKilled + remc;	
		ChangeITGCityInteger(iCity, "CI_PLAGUE_POP_LOST", remc)
	end
	local dur = 0;
	for i = 1, gT.Globals.PlaguesByID[EpidemyID].Duration do
		if LastRandom(1, gT.Globals.PlaguesByID[EpidemyID].Duration) <= i then
			dur = i;
			break;
		end
	end
	local lastturn = GetITGGameTurn() + dur;
	local lastContturn = lastturn + math.ceil(gT.Globals.PlaguesByID[EpidemyID].Duration / 3);
	gT.Cities[cID].Epidemies[EpidemyID] = {Active = true, Outbreak = GetITGGameTurn(), Type = gT.Globals.PlaguesByID[EpidemyID].Type, End = lastturn, EndContagious = lastContturn}
	if gT.Players[iPlayer].CultBonusOn[194] then
		for dCity in Players[iPlayer]:Cities() do
			if dCity:GetID() ~= iCity:GetID() then
				if LastRandom(1, 2) == 2 then
					local dcID = dCity:Plot():GetPlotIndex() .. "," .. dCity:GetGameTurnFounded();
					if not gT.Cities[dcID].Epidemies[EpidemyID] then
						gT.Cities[dcID].Epidemies[EpidemyID] = {Active = false, Type = gT.Globals.PlaguesByID[EpidemyID].Type, EndContagious = -1, Outbreak = -1}
					end
				end
			end
		end
	end
	
	--notification
	local bGreatPeopleChance = math.floor((ck + remc) / sp * 100);
	DoTriggerGreatPeopleKillFromEpidemy(iCity, bGreatPeopleChance)
	UpdateHealthInSingleCity(iCity, iCity:GetOwner())
	if LastRandom(1, 100) <= 30 then
		TakeCareOfEventMorale(iCity:GetOwner(), 25, {-1, "Epidemy"})
	end
	if bTruelyNew then
		for jj, iHuman in pairs(gT.Globals.HumanPlayers) do
			if iCity:Plot():IsRevealed(Players[iHuman]:GetTeam()) then
				Players[iHuman]:AddNotification(94343293, "The outbreak of new " .. gT.Globals.PlaguesByID[EpidemyID].Type .. "'s epidemy killed about " .. math.floor((ck + remc) / sp * 100) .. "% of citizens in the " .. iCity:GetName() .. ".", "New Epidemy", iCity:GetX(), iCity:GetY(), 7)
			end
		end
	elseif Players[iPlayer]:IsHuman() then
		local shu = ""
		if sReason == "Contagion" then
			shu = " Contagion's source is believed to be " .. sCity:GetName() .. ".";
		elseif sReason == "Pilgrims" then
			shu = " Contagion's source is traced to pilgrims travelling from " .. sCity:GetName() .. ".";
		elseif sReason == "Trade" then
			shu = " Contagion's source is traced to traders that arrived from " .. sCity:GetName() .. ".";
		elseif sReason == "Migration" then
			shu = " Contagion's source seems to be migrants escaping from epidemy in " .. sCity:GetName() .. ".";
		end
		Players[iPlayer]:AddNotification(94343293, "The outbreak of " .. gT.Globals.PlaguesByID[EpidemyID].Type .. "'s epidemy killed about " .. math.floor((ck + remc) / sp * 100) .. "% of citizens in the " .. iCity:GetName() .. "." .. shu, "Epidemy", iCity:GetX(), iCity:GetY(), 7)
	end
end

function DoStartPlagueInCityDLL(iCity)
	iCity:SetPlague(true)
	--disable heal around :) not true/false but integer
	for i, plot in pairs(GetPlotsInRadiusOfPlot(iCity:Plot(), 2)) do
		plot:ChangePlague(1)
	end
	UpdateUnitCostFromQuarantine(iCity:GetOwner())
	Events.SpecificCityInfoDirty(iCity:GetOwner(), iCity:GetID(), CityUpdateTypes.CITY_UPDATE_TYPE_BANNER)
end

function DoRemovePlagueInCityDLL(iCity)
	iCity:SetPlague(false)
	--enable heal around;
	for i, plot in pairs(GetPlotsInRadiusOfPlot(iCity:Plot(), 2)) do
		plot:ChangePlague(-1)
	end
	UpdateUnitCostFromQuarantine(iCity:GetOwner())
end

function CheckToRemoveEpidemy(iCity, cID)
	for epid, sth in pairs(gT.Cities[cID].Epidemies) do
		if sth.Active then
			return;
		end
	end
	if gT.Players[iCity:GetOwner()].CultBonusOn[136] then
		if GetITGCityInteger(iCity, "CI_CULT_136") == 0 then
			iCity:ChangeYieldRateModifier(yProduction, 100)
			iCity:ChangeYieldRateModifier(yFood, 100)
		end
		SetITGCityInteger(iCity, "CI_CULT_136", GetITGGameTurn() + 5);
	end
	DoRemovePlagueInCityDLL(iCity)
end

function GetNumSpreadsFromSingleCity(epid, gTurn, bFirstCitySpread)
	if gTurn - gT.Globals.PlaguesByID[epid].Outbreak < 2 then
		return 15;
	end
	if bFirstCitySpread then
		return 4;
	end
	if gT.Globals.PlaguesByID[epid].Contagious >= 50 then
		return 2;
	end
	return 1;
end

local bcHarbor = GameInfoTypes.BUILDINGCLASS_HARBOR;
function DoSpreadPlagueOnTurn(iCity, cID, g, epid, bFirstSpread)
	local gTurn = GetITGGameTurn();
	local cRtp = gTurn - gT.Cities[cID].Epidemies[epid].Outbreak - 1;
	if cRtp ~= 0 then
		if LastRandom(1, 100) > gT.Globals.PlaguesByID[epid].Contagious - cRtp * 2 then
			return;
		end
	end
	local buildingHarbor = Players[g]:GetPlayerUniqueBuilding(bcHarbor);
	local lFHarbor = (buildingHarbor ~= -1) and (iCity:IsHasBuilding(buildingHarbor));
	local roadtab;
	local citiesByHealth = {}
	for h, sIndex in pairs(GetNearCities(cID, 10)) do
		local sCity = Map.GetPlotByIndex(sIndex):GetPlotCity();
		if not IsCityAlreadyImmuneToEpidemy(sCity, gT.Globals.PlaguesByID[epid].Type) then
			if sCity:Plot():IsRevealed(Players[g]:GetTeam()) then
				local cID = sCity:Plot():GetPlotIndex() .. "," .. sCity:GetGameTurnFounded();
				table.insert(citiesByHealth, {sIndex, gT.Cities[cID].Health + gT.Cities[cID].PlagueDefense})
			end
		end
	end
	table.sort(citiesByHealth, SortFunctionForLowestMapDistance)
	local iSpreadsLeft = GetNumSpreadsFromSingleCity(epid, gTurn, bFirstSpread)
	for h, cbhData in pairs(citiesByHealth) do
		local sipindex = cbhData[1];
		local sCity = Map.GetPlotByIndex(sipindex):GetPlotCity();
		if not IsCityAlreadyImmuneToEpidemy(sCity, gT.Globals.PlaguesByID[epid].Type) then
			local lChance = GetChanceForPlagueSpread(sCity, epid, iCity) / 2
			if gT.Cities[cID].RoadConnected[sipindex] or (lFHarbor and AreConnectedWithHarbor(sCity, iCity)) then
				lChance = lChance * 2;
			elseif (not gT.Cities[cID].DoNotCheckRoads[sipindex]) and (roadtab or (GetContID(iCity:Plot(), 1244263) == GetContID(sCity:Plot(), 45654))) then
				if not roadtab then
					roadtab = RoadConnections(iCity:Plot(), nil)
				end
				if roadtab[sCity:Plot():GetPlotIndex()] then
					lChance = lChance * 2;
					gT.Cities[cID].RoadConnected[sipindex] = true;
				end
			end
			lChance = lChance - 8 - cRtp * 3;
			if gT.Players[sCity:GetOwner()].CultBonusOn[193] then
				lChance = math.floor(0.75 * lChance)
			end
			if LastRandom(1, 100) < lChance then
				DoOutbreakOfEpidemy(sCity, epid, false, "Contagion", iCity)
				iSpreadsLeft = iSpreadsLeft - 1;
				if iSpreadsLeft == 0 then
					return;
				end
			end
		end
	end
end

function AreConnectedWithHarbor(sCity, iCity)
	local buildingHarbor = Players[sCity:GetOwner()]:GetPlayerUniqueBuilding(bcHarbor);
	if (buildingHarbor ~= -1) and (sCity:IsHasBuilding(buildingHarbor)) then
		local rTab = ReverseTable(GetCoastOfOceanTab(iCity:Plot()))
		for i, oc in pairs(GetCoastOfOceanTab(sCity:Plot())) do
			if rTab[oc] then
				return true;
			end
		end
	end
	return false;
end

function IsCityAlreadyImmuneToEpidemy(iCity, pname)
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
	if HasPlayerTech(iCity:GetOwner(), tImmuneInno, "teinnosade") then
		return true;
	end
	for id, epidemydata in pairs(gT.Cities[cID].Epidemies) do
		if epidemydata.Type == pname then
			return true;
		end
	end
	return false;
end

function DoTriggerGreatPeopleKillFromEpidemy(iCity, iGreatPeopleChance)
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
	local rChance = math.ceil(iGreatPeopleChance / 2) - gT.Cities[cID].Health;
	--iterate
	local cpIndex = iCity:Plot():GetPlotIndex();
	for gpID, hhh in pairs(gT.Players[iCity:GetOwner()].GreatPeople) do
		local gd = gT.GreatPeople[gpID];
		if gd.Alive and (not gd.Military) then
			if gd.City == cpIndex then
				if LastRandom(1, 100) <= rChance then
		--			print("Great Person died with plague chance", rChance)
					KillGreatPerson(gpID, true)
					if iCity:GetOwner() == Game.GetActivePlayer() then
						local strTitle = "Great Person died!";
						local string = GetGreatPersonName(gpID) .. " died during an epidemy in " .. iCity:GetName() .. "!";
						Players[iCity:GetOwner()]:AddNotification(13551235, string, strTitle, -1, -1, gT.GreatPeople[gpID].Type, 7)
					end
				end
			end
		end
	end
end

function DoTradeEpidemy(iCity, sCity, sReason, bSending)
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
	local gTurn = GetITGGameTurn();
	if (not sReason) and bSending then
		if iCity:GetOwner() ~= sCity:GetOwner() then
			if HasPlayerCHIncomingTrait(sCity:GetOwner(), 1) then
				return;
			end
		end
	end
	for epid, data in pairs(gT.Cities[cID].Epidemies) do
		if data.EndContagious and (data.EndContagious >= gTurn) then
			if not IsCityAlreadyImmuneToEpidemy(sCity, data.Type) then
				local lChance = GetChanceForPlagueSpread(sCity, epid, iCity);
				if gT.Players[sCity:GetOwner()].CultBonusOn[193] then
					lChance = math.floor(0.75 * lChance)
				end
				if LastRandom(1, 100) <= lChance then
					DoOutbreakOfEpidemy(sCity, epid, false, sReason or "Trade", iCity)
					if (not sReason) and bSending then
						if Players[iCity:GetOwner()]:IsHuman() then
							LuaEvents.ITG_AchievementCompleted(93)
						end
					end
				end
			end
		end
	end 
end

function GetChanceForPlagueSpread(sCity, epid, iCity)
	local chance = gT.Globals.PlaguesByID[epid].Contagious + LastRandom(-5, 5);
	local cID = sCity:Plot():GetPlotIndex() .. "," .. sCity:GetGameTurnFounded();
	chance = chance - gT.Cities[cID].Health - gT.Cities[cID].PlagueDefense;
	chance = chance - 5 * GetNumCityAcademiesWithAbility(iCity, 27)
	if Players[sCity:GetOwner()]:IsGoldenAge() then
		chance = chance / 2;
	end
	return chance;
end

function DoSpreadEpidemyMigrants(iCity, sCity)
	DoTradeEpidemy(iCity, sCity, "Migration")
end

LuaEvents.ITGSG_RequestPlagueString.Add(function(iCity)
	local sText = "There is an epidemy of " .. GetEpidemyString(iCity) .. " in the city. Units within two tiles cannot heal. Epidemy can passively spread upon nearby cities or through Trade Routes and migrations.";
	MapModData.ITGSG = sText;
end)

function GetEpidemyString(iCity)
	local bFirst = true;
	local s = ""
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
	for epid, data in pairs(gT.Cities[cID].Epidemies) do
		if data.Active then
			if bFirst then
				s = "[COLOR_CITY_GREEN]" .. data.Type .. "[ENDCOLOR]";
				bFirst = false;
			else
				s = s .. ", [COLOR_CITY_GREEN]" .. data.Type .. "[ENDCOLOR]";
			end
		end
	end
	return s;
end