local dbTogTCultural = {
"FoodFromFish", "BaseSettlementStrength", "InnovationChance", "CanDoReformation", "CanSettleNewCity"
}
local bCultLink = (PreGame.GetGameOption("GAMEOPTION_ITG_CULTURALLY_LINKED") == 1);
local CultCivLinked;
function InitializeDBCultCivLinked()
	if not bCultLink then
		return;
	end
	if CultCivLinked then
		return;
	end
	CultCivLinked = {};
	for row in DB.Query("SELECT * FROM ITG_CivilizationCultures") do
		if not CultCivLinked[row.CivilizationType] then
			CultCivLinked[row.CivilizationType] = {};
		end
		CultCivLinked[row.CivilizationType][ITG_ScenariosTypes[row.ScenarioType]] = ITG_CultureTypes[row.CultureType] or (ITG_SubcultureTypes[row.CultureType] + 1000) 
	end
end

local g_cultureTechs;
local g_cultureFreeBuildingClasses;
local g_cultureOptBuildingClasses;
local g_cultureStartingPop;
local g_cultureGoverments;

function InitializeTechData()
	g_cultureTechs = {}
	for row in DB.Query("SELECT * FROM ITG_CultureTechs") do
		if not g_cultureTechs[row.CultureType] then
			g_cultureTechs[row.CultureType] = {Must = {}}
		end
		g_cultureTechs[row.CultureType].Must[GameInfo.Technologies[row.TechType].ID] = true;
	end
	for row in DB.Query("SELECT * FROM ITG_CultureOptionalTechs") do
		if not g_cultureTechs[row.CultureType] then
			g_cultureTechs[row.CultureType] = {}
		end
		if not g_cultureTechs[row.CultureType].Optional then
			g_cultureTechs[row.CultureType].Optional = {}
		end
		g_cultureTechs[row.CultureType].Optional[GameInfo.Technologies[row.TechType].ID] = row.Chance;
	end
end
function GetPlayerInitTechData(iPlayer)
	if not g_cultureTechs then
		InitializeTechData()
	end
	local tMust, tOptional = {}, {}
	local cTypes = ITG_CultureData[gT.Players[iPlayer].CultureType].Type;
	if g_cultureTechs[cTypes] then
		if g_cultureTechs[cTypes].Must then
			for a, b in pairs(g_cultureTechs[cTypes].Must) do
				tMust[a] = b;
			end
		end
		if g_cultureTechs[cTypes].Optional then
			for a, b in pairs(g_cultureTechs[cTypes].Optional) do
				tOptional[a] = b;
			end
		end
	end
	if gT.Players[iPlayer].SubcultureType then
		local scTypes = ITG_SubcultureData[gT.Players[iPlayer].SubcultureType].Type;
		if g_cultureTechs[scTypes] then
			if g_cultureTechs[scTypes].Must then
				tMust = {}
				for a, b in pairs(g_cultureTechs[scTypes].Must) do
					tMust[a] = b;
				end
			end
			if g_cultureTechs[scTypes].Optional then
				tOptional = {}
				for a, b in pairs(g_cultureTechs[scTypes].Optional) do
					tOptional[a] = b;
				end
			end
		end
	end
	return tMust, tOptional;
end
function ClearInitTechData()
	g_cultureTechs = nil;
end

function InitializeFreeBuildingClasses()
	g_cultureFreeBuildingClasses = {}
	g_cultureOptBuildingClasses = {}
	for row in DB.Query("SELECT * FROM ITG_CultureFreeBuildingClasses") do
		if not g_cultureFreeBuildingClasses[row.CultureType] then
			g_cultureFreeBuildingClasses[row.CultureType] = {}
		end
		table.insert(g_cultureFreeBuildingClasses[row.CultureType], GameInfo.BuildingClasses[row.BuildingClassType].ID)
	end
	for row in DB.Query("SELECT * FROM ITG_CultureOptionalBuildingClasses") do
		if not g_cultureOptBuildingClasses[row.CultureType] then
			g_cultureOptBuildingClasses[row.CultureType] = {}
		end
		table.insert(g_cultureOptBuildingClasses[row.CultureType], {GameInfo.BuildingClasses[row.BuildingClassType].ID, row.Population, row.Chance})
	end
end

function InitializeStartingPopDB()
	g_cultureStartingPop = {};
	for row in DB.Query("SELECT * FROM ITG_CulturePopRef") do
		if not g_cultureStartingPop[row.CultureType] then
			g_cultureStartingPop[row.CultureType] = {}
		end
		table.insert(g_cultureStartingPop[row.CultureType], {row.Min, row.Max})
	end
end

function GetDBStartingPopVar(iPlayer)
	if not g_cultureStartingPop then
		InitializeStartingPopDB()
	end
	if gT.Players[iPlayer].SubcultureType then
		local sType = ITG_SubcultureData[gT.Players[iPlayer].SubcultureType].Type;
		if g_cultureStartingPop[sType] then
			return RandomFromTab(g_cultureStartingPop[sType])
		end
	end
	local cType = ITG_CultureData[gT.Players[iPlayer].CultureType].Type
	if g_cultureStartingPop[cType] then
		return RandomFromTab(g_cultureStartingPop[cType]);
	end
	print("DB error init pop!", iPlayer)
	return {1,2};
end

function GetOptionalBuildingClassesOnCityFound(iPlayer)
	if not g_cultureOptBuildingClasses then
		InitializeFreeBuildingClasses()
	end
	if gT.Players[iPlayer].SubcultureType then
		if g_cultureOptBuildingClasses[ITG_SubcultureData[gT.Players[iPlayer].SubcultureType].Type] then
			return g_cultureOptBuildingClasses[ITG_SubcultureData[gT.Players[iPlayer].SubcultureType].Type]
		end
	end
	if g_cultureOptBuildingClasses[ITG_CultureData[gT.Players[iPlayer].CultureType].Type] then
		return g_cultureOptBuildingClasses[ITG_CultureData[gT.Players[iPlayer].CultureType].Type];
	end
	return {};
end

function GetFreeBuildingClassesOnCityFound(iPlayer)
	if not g_cultureFreeBuildingClasses then
		InitializeFreeBuildingClasses()
	end
	if gT.Players[iPlayer].SubcultureType then
		if g_cultureFreeBuildingClasses[ITG_SubcultureData[gT.Players[iPlayer].SubcultureType].Type] then
			return g_cultureFreeBuildingClasses[ITG_SubcultureData[gT.Players[iPlayer].SubcultureType].Type]
		end
	end
	if g_cultureFreeBuildingClasses[ITG_CultureData[gT.Players[iPlayer].CultureType].Type] then
		return g_cultureFreeBuildingClasses[ITG_CultureData[gT.Players[iPlayer].CultureType].Type];
	end
	return {};
end

local g_CultureResRevealed;
local g_CultureInnovations;
local g_CultureFreeResources;
function InitTempDBTablesForCultures(bAdd)
	if bAdd then
		g_CultureResRevealed = {};
		for row in DB.Query("SELECT * FROM ITG_CultureResourceRevealed") do
			if not g_CultureResRevealed[row.CultureType] then
				g_CultureResRevealed[row.CultureType] = {}
			end
			table.insert(g_CultureResRevealed[row.CultureType], GameInfo.Resources[row.ResourceType].ID)
		end
		g_CultureInnovations = {}
		for row in DB.Query("SELECT * FROM ITG_CultureInnovations") do
			if not g_CultureInnovations[row.CultureType] then
				g_CultureInnovations[row.CultureType] = {}
			end
			table.insert(g_CultureInnovations[row.CultureType], GameInfo.Technologies[row.TechType].ID)
		end
		g_CultureFreeResources = {};
		for row in DB.Query("SELECT * FROM ITG_CultureFreeResources") do
			if not g_CultureFreeResources[row.CultureType] then
				g_CultureFreeResources[row.CultureType] = {}
			end
			table.insert(g_CultureFreeResources[row.CultureType], {GameInfo.Resources[row.ResourceType].ID, row.Amount})
		end
	else
		g_CultureResRevealed = nil;
		g_CultureInnovations = nil;
		g_CultureFreeResources = nil;
	end
end

function DoAddDBDataFromCultures() -- this fires once all cultures and subcultures are assigned;
	InitTempDBTablesForCultures(true)
	for g, h in pairs(gT.Players) do
		local gTeam = Teams[Players[g]:GetTeam()];
		gT.Players[g].Innovations = {};
		Players[g]:SetFakeGameTurnsElapsed(LastRandom(GetDBCulturePlayerInfo(g,"MinFTurnsElapsed"), GetDBCulturePlayerInfo(g,"MaxFTurnsElapsed")))
		Players[g]:SetCityConnectionGoldPenaltyFree(GetDBCulturePlayerInfo(g,"UnmoddedCityConnection"));
		if GetDBCulturePlayerInfo(g, "CantCityConquer") and (not Players[g]:IsMinorCiv()) then
			Players[g]:SetCannotConquerCities(true)
			gT.Players[g].OccupyLimit = 0;
		else
			Players[g]:SetCannotConquerCities(false)
		end
		gT.Players[g].CultureString = Locale.Lookup(GetDBCulturePlayerInfo(g,"Description"));
		gT.Players[g].LastCultureTechPrereq = GameInfo.Technologies[GetDBCulturePlayerInfo(g, "TechCultureStyles")].ID
		HandleBenefitsFromCultureType(g, 1, true, 1)
		local cType = ITG_CultureData[h.CultureType].Type;
		for i, resid in pairs(g_CultureResRevealed[cType] or {}) do
			gTeam:SetForceRevealedResource(resid, true)
		end
		local sType;
		if h.SubcultureType then
			sType = ITG_SubcultureData[h.SubcultureType].Type;
			for i, resid in pairs(g_CultureResRevealed[sType] or {}) do
				gTeam:SetForceRevealedResource(resid, true)
			end
		end
		for i, techID in pairs(sType and g_CultureInnovations[sType] or g_CultureInnovations[cType]) do
			table.insert(gT.Players[g].Innovations, techID)
		end
		for i, resD in pairs(sType and g_CultureFreeResources[sType] or g_CultureFreeResources[cType] or {}) do
			ChangeITGExtraResources(g, resD[1], resD[2])
		end
		for i, hString in pairs(dbTogTCultural) do
			gT.Players[g][hString] = GetDBCulturePlayerInfo(g,hString)
		end
		local iGarDefModInit = GetDBCulturePlayerInfo(g,"GarrisonDefMod");
		if iGarDefModInit ~= 0 then
			Players[g]:ChangeGarrisonDefMod(iGarDefModInit)
		end
	end
	InitTempDBTablesForCultures(false)
end

function AssignCultureTypeToPlayer(iPlayer, type, bFsubtype)
	gT.Players[iPlayer].CultureType = type;
	if #ITG_CultureSubcultures[type] > 0 then
		if bFsubtype then
			AddCulturalSubtype(iPlayer, bFsubtype) -- this one is passing still incorrectly
		else
			if type == 6 then
				if ShouldBecomeNomadicBasedOnStart(iPlayer) then
					AddCulturalSubtype(iPlayer, ITG_SubcultureTypes["SUBCULTURE_PAG_NOMADIC"])
				elseif ShouldBecomeNordicBasedOnStart(iPlayer) then
					AddCulturalSubtype(iPlayer, ITG_SubcultureTypes["SUBCULTURE_PAG_VIKINGS"])
				end
			elseif type == 8 then
				AddCulturalSubtype(iPlayer, GetProperSubcultureForCityState(iPlayer))
			else
				local subTabs = {}
				for i, subcultureIndex in pairs(ITG_CultureSubcultures[type]) do
					if LastChance(ITG_SubcultureData[subcultureIndex].Chance or 0) then
						table.insert(subTabs, subcultureIndex)
					end
				end
				if #subTabs > 0 then
					AddCulturalSubtype(iPlayer, RandomFromTab(subTabs))
				end
			end
		end
	end
end

function AddCulturalSubtype(iPlayer, sType)
	if sType == -1 then
		return;
	end
	gT.Players[iPlayer].SubcultureType = sType;
end

function StartingPlotValidForMaritimeCS(iPlot)
	return GetSizeOfOceanAdjacentToPlot(iPlot) >= 20;
end

function ModernAssignRelations(minTab, majTab)
	for g, gh in pairs(gT.Players) do
		local tteam = Players[g]:GetTeam()
		for j, jh in pairs(gT.Players) do
			if j > g then
				Teams[tteam]:Meet(Players[j]:GetTeam())
			end
		end
	end
	for jt, minor in pairs(minTab) do
		local lPlayer = majTab[LastRandom(1, #majTab)];
		Players[minor]:ChangeMinorCivFriendshipWithMajor(lPlayer, LastRandom(40,180))
	end
end

function IndustrialAssignRelations(minTab, majTab)
--meet all
	for g, gh in pairs(gT.Players) do
		local tteam = Players[g]:GetTeam()
		for j, jh in pairs(gT.Players) do
			if j > g then
				Teams[tteam]:Meet(Players[j]:GetTeam())
			end
		end
	end
	local nummucho = math.min(math.floor(#minTab / 15), 2)
	local numdecent = math.min(math.floor(#minTab / 8), 4)
	local tt = numdecent + nummucho
	if numdecent > 0 then
		for jt, majorID in pairs(majTab) do
			minTab = Shuffle(minTab);
			for i = 1, tt do
				if i <= nummucho then
					Players[minTab[i]]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(100,180))
				else
					Players[minTab[i]]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(35,55))
				end
			end
		end
	end
end

function NewWorldAssignRelations(minTab, majTab)
	for it, minorID in pairs(minTab) do
		if Players[minorID]:IsAlive() then
			local cType = gT.Players[minorID].CultureType;
			for jt, majorID in pairs(majTab) do
				if Teams[Players[majorID]:GetTeam()]:IsHasMet(Players[minorID]:GetTeam()) then
					if gT.Players[majorID].CultureType == cType then
						if cType == 1 then
							if LastRandom(1,10) < 9 then
								if LastRandom(1,10) < 6 then
									if LastRandom(1,3) == 2 then
										Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(120,180))
									else
										Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(60,150))
									end
								else
									Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(20,50))
								end
							end
						else
							if LastRandom(1,10) < 7 then
								if LastRandom(1,5) < 4 then
									Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(30,125))
								else
									Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(30,65))
								end
							end
						end
					elseif cType == 1 then
						if LastRandom(1,10) < 4 then
							Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(5,20))
						end	
					else
						if LastRandom(1,10) < 8 then
							if LastRandom(1,4) == 3 then
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(30,125))
							else
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(25,45))
							end
						end
					end
				end
			end
		end
	end
end

function AncientAssignRelations(minTab, majTab)
	for it, minorID in pairs(minTab) do
		if Players[minorID]:IsAlive() then
			local cType = gT.Players[minorID].CultureType;
			for jt, majorID in pairs(majTab) do
				if Teams[Players[majorID]:GetTeam()]:IsHasMet(Players[minorID]:GetTeam()) then
					if gT.Players[majorID].CultureType == cType then
						if cType == 8 then
							if LastRandom(1,10) < 9 then
								if LastRandom(1,10) < 6 then
									if LastRandom(1,3) == 2 then
										Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(120,180))
									else
										Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(60,150))
									end
								else
									Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(20,50))
								end
							end
						else
							if LastRandom(1,10) < 7 then
								if LastRandom(1,5) < 4 then
									Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(30,125))
								else
									Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(30,65))
								end
							end
						end
					elseif cType == 7 then
						if LastRandom(1,10) < 6 then
							Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(5,70))
						end	
					elseif cType == 8 then
						if LastRandom(1,10) < 8 then
							if LastRandom(1,4) == 3 then
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(30,70))
							else
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(25,45))
							end
						end
					end
				end
			end
		end
	end
end

function MedievalAssignRelations(minTab, majTab)
	for it, minorID in pairs(minTab) do
		if Players[minorID]:IsAlive() then
			local cType = gT.Players[minorID].CultureType;
			for jt, majorID in pairs(majTab) do
				if Teams[Players[majorID]:GetTeam()]:IsHasMet(Players[minorID]:GetTeam()) then
					if gT.Players[majorID].CultureType == cType then
						if cType == 6 then
							if LastRandom(1, 10) == 1 then
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(40,75))
							elseif LastRandom(1,10) < 5 then
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(20,55))
							end
						elseif cType == 3 then
							if LastRandom(1, 10) == 1 then
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(120,155))
							elseif LastRandom(1, 10) == 1 then
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(60,90))
							elseif LastRandom(1, 10) < 5 then
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(25,50))
							end
						elseif cType == 4 then
							if LastRandom(1, 10) == 1 then
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(125,185))
							elseif LastRandom(1, 10) == 1 then
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(70,100))
							elseif LastRandom(1, 10) < 3 then
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(25,70))
							end
						else
							if LastRandom(1, 10) == 1 then
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(60,100))
							elseif LastRandom(1, 10) == 1 then
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(40,60))
							elseif LastRandom(1, 10) < 5 then
								Players[minorID]:ChangeMinorCivFriendshipWithMajor(majorID, LastRandom(10,40))
							end
						end
					else
					--nothing, other cultures have no influence
					end
				end
			end
		end
	end
end

function GetPredeterminedCulturalType(player)
	local tVal = Modding.OpenSaveData().GetValue("ITG_pgd_PlayerCultureType" .. player)
	if tVal then
		Modding.OpenSaveData().SetValue("ITG_pgd_PlayerCultureType" .. player, nil) -- for further eras
	end
	if tVal and (tVal ~= -1) then
		return tVal;
	end
	if bCultLink then
		if not Players[player]:IsMinorCiv() then
			local civType = GameInfo.Civilizations[Players[player]:GetCivilizationType()].Type;
			if CultCivLinked[civType] and CultCivLinked[civType][gT.Globals.ScenarioType] then
				return CultCivLinked[civType][gT.Globals.ScenarioType];
			end
		else
			--maybe one day :] //update: not today
		end
	end
	return nil;
end

function AncientAssignCultures(minTab, majTab)
	local hTab = ITG_ScenarioCultures[gT.Globals.ScenarioType];
	local nMtotal = #majTab;
	local nEmpires = math.min( LastRandom(math.ceil(nMtotal / 5), math.ceil(nMtotal / 4)), 4)
	local nTribes = math.ceil((nMtotal - nEmpires) / 100 * LastRandom(30,55))
	print("AAC", nMtotal, nEmpires, nTribes)
	local preMaritime = {}
	for i, player in pairs(majTab) do 
		local irValue = GetPredeterminedCulturalType(player)
		if irValue == ITG_CultureTypes["CULTURE_EMPIRE"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_EMPIRE"], -1)
			nEmpires = nEmpires - 1
		elseif irValue == ITG_SubcultureTypes["SUBCULTURE_CS_MERCANTILE"] + 1000 then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_CITY_STATE"], ITG_SubcultureTypes["SUBCULTURE_CS_MERCANTILE"])
		elseif irValue == ITG_CultureTypes["CULTURE_CITY_STATE"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_CITY_STATE"], nil)
			if gT.Players[player].SubcultureType == ITG_SubcultureTypes["SUBCULTURE_CS_MARITIME"] then 
				table.insert(preMaritime, player)
			end
		elseif irValue == ITG_CultureTypes["CULTURE_TRIBE"] then
			nTribes = nTribes - 1;
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_TRIBE"], -1)
		elseif irValue == ITG_SubcultureTypes["SUBCULTURE_CS_MILITARISTIC"] + 1000 then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_CITY_STATE"], ITG_SubcultureTypes["SUBCULTURE_CS_MILITARISTIC"])
		elseif irValue == ITG_SubcultureTypes["SUBCULTURE_CS_RELIGIOUS"] + 1000 then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_CITY_STATE"], ITG_SubcultureTypes["SUBCULTURE_CS_RELIGIOUS"])
		elseif irValue == ITG_SubcultureTypes["SUBCULTURE_CS_CULTURAL"] + 1000 then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_CITY_STATE"], ITG_SubcultureTypes["SUBCULTURE_CS_CULTURAL"])
		elseif irValue == ITG_SubcultureTypes["SUBCULTURE_CS_MARITIME"] + 1000 then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_CITY_STATE"], ITG_SubcultureTypes["SUBCULTURE_CS_MARITIME"])
			table.insert(preMaritime, player)
		end
		--REINVENTING THE WHEEL BONUS HERE APPLIED
		gT.Players[player].NextInnoChance = gT.Players[player].NextInnoChance + 300;
	end
	local tLuckTab = {}
	for it, player in pairs(majTab) do
		if not gT.Players[player].CultureType then
			table.insert(tLuckTab, player)
		end
	end
	for i = 1, nEmpires do
		if #tLuckTab > 0 then
			local lKey = LastRandom(1, #tLuckTab);
			AssignCultureTypeToPlayer(tLuckTab[lKey], ITG_CultureTypes["CULTURE_EMPIRE"])
			table.remove(tLuckTab, lKey)
		end
	end
	local yetUnassigned = {}
	local yetBarbarian = {}
	local yetCS = {}
	local yetMedian = {}
	for g, gd in pairs(gT.Players) do
		if not gd.CultureType then
			table.insert(yetUnassigned, g)
			table.insert(yetMedian, {g, 0})
		elseif gd.CultureType == 8 then
			table.insert(yetCS, g)
			table.insert(yetMedian, {g, 0})
		elseif gd.CultureType == 9 then
			table.insert(yetBarbarian, g)
		end
	end
	local hmin = 99999;
	if #yetMedian > 0 then
		for i, dd in pairs(yetMedian) do
			local iv = 0;
			local sp = Players[dd[1]]:GetStartingPlot()
			for h, bb in pairs(yetMedian) do
				if h ~= i then
					local cp = Players[bb[1]]:GetStartingPlot()
					iv = iv + Map.PlotDistance(sp:GetX(), sp:GetY(), cp:GetX(), cp:GetY())
				end
			end
			yetMedian[i][2] = iv;
			hmin = math.min(hmin, iv);
		end
		local lkey;
		for i, dd in pairs(yetMedian) do
			if dd[2] == hmin then
				lkey = dd[1];
				break;
			end
		end
		if not lkey then
			print("Heresy 99/23/553/")
		end
		local csFar = {}
		local middleofcsworld = Players[lkey]:GetStartingPlot();
		local majorFar = {}
		for i, p in pairs(yetUnassigned) do
			local sp = Players[p]:GetStartingPlot();
			local dis = Map.PlotDistance(sp:GetX(), sp:GetY(), middleofcsworld:GetX(), middleofcsworld:GetY())
			if Players[p]:IsMinorCiv() then
				table.insert(csFar, {p, dis})
			else
				table.insert(majorFar, {p, dis})
			end
		end
		table.sort(csFar, SortFunctionForGreaterInteger2)
		table.sort(majorFar, SortFunctionForGreaterInteger2)
		for i = 1, nTribes do
			if majorFar[i] then
				AssignCultureTypeToPlayer(majorFar[i][1], ITG_CultureTypes["CULTURE_TRIBE"])
			end
		end
		local ncst = math.ceil(#minTab / 100 * LastRandom(30,60))
		for i = 1, ncst do
			if csFar[i] then
				AssignCultureTypeToPlayer(csFar[i][1], ITG_CultureTypes["CULTURE_TRIBE"])
			else
				print("Should not happen 1266/3431/435/")
			end
		end
		local finalCS = {}
		local fdoneCS = {}
		for i, data in pairs(gT.Players) do
			if not data.CultureType then
				table.insert(finalCS, i)
			elseif data.CultureType == 8 then
				table.insert(finalCS, i)
			end
		end
		--	1. city-state merge closely :]
		local numtop = #finalCS;
		GeneratePossibleCityLocations(nil, true)
		local clc = GetCityLocations();
		ClearCityLocations();
		local scc = {}
		for i, data in pairs(gT.Players) do
			if data.CultureType == 8 then
				scc[Players[i]:GetStartingPlot():GetPlotIndex()] = i;
			else
				clc[Players[i]:GetStartingPlot():GetPlotIndex()] = nil;
				if Players[i]:IsMinorCiv() then
					if Players[i]:GetMinorCivTrait() == GameInfoTypes.MINOR_TRAIT_MARITIME then
						table.insert(preMaritime, i)
					end
				end
			end
		end
		local pTs = {}
		for hhIndex, jj in pairs(clc) do
			local hh = Map.GetPlotByIndex(hhIndex)
			if GetSizeOfCont(GetContID(hh,8123123)) > 15 then
				table.insert(pTs, {hhIndex, Map.PlotDistance(hh:GetX(), hh:GetY(), middleofcsworld:GetX(), middleofcsworld:GetY())})
			end
		end
		table.sort(pTs, SortFunctionForLowestMapDistance)
		local function GetBestMaritimePosition()
			local bNplot; 
			for jkj, bjjd in pairs(pTs) do
				if not scc[bjjd[1]] then
					if StartingPlotValidForMaritimeCS(Map.GetPlotByIndex(bjjd[1])) then
						print("Found valid maritime plot on position", jkj, " out of ", numtop, " city-states.")
						bNplot = Map.GetPlotByIndex(bjjd[1]);
						break;
					end
				end
			end
			return bNplot;
		end
		for i = 1, numtop do
			if scc[pTs[i][1]] then
				if gT.Players[scc[pTs[i][1]]].SubcultureType == 7 then
					if not StartingPlotValidForMaritimeCS(Map.GetPlotByIndex(pTs[i][1])) then
					--maritime bad; let's find him best
					--it assumes that maritime cs are already properly assigned by base game (only pre-defined major)
						local bNplot = GetBestMaritimePosition();
						if bNplot then
							scc[bNplot:GetPlotIndex()] = 1;
							scc[pTs[i][1]] = nil;
							DoMoveStartingLocation(scc[pTs[i][1]], bNplot)
						else
							print("Not a single valid plot for preset maritime city-state.")
						end
					end
				end
				fdoneCS[scc[pTs[i][1]]] = 1; --stay where you started
			end
		end
		--let's handle preMaritime with bad starting positions first;
		for ijk, maritimePlayer in pairs(preMaritime) do
			if not fdoneCS[maritimePlayer] then
				local bNplot = GetBestMaritimePosition();
				if bNplot then
					scc[bNplot:GetPlotIndex()] = 1;
					fdoneCS[maritimePlayer] = true;
					DoMoveStartingLocation(maritimePlayer, bNplot)
				else
					print("Not a single valid plot for preset maritime city-state x2.")
				end
			end
		end
		finalCS = Shuffle(finalCS)
		for i = 1, numtop do
			if not scc[pTs[i][1]] then
				for j, d in pairs(finalCS) do
					if not fdoneCS[d] then
						fdoneCS[d] = 1;
						DoMoveStartingLocation(d, Map.GetPlotByIndex(pTs[i][1]))
						break;
					end
				end
			end
		end
		for i, data in pairs(gT.Players) do
			if not data.CultureType then
				AssignCultureTypeToPlayer(i, ITG_CultureTypes["CULTURE_CITY_STATE"]); -- now maritime should not be an option for major anymore;
			end
		end
	end
	for i, data in pairs(gT.Players) do
		if not data.CultureType then
			print("Very bad 123231", i)
			AssignCultureTypeToPlayer(i, RandomFromTab(hTab))
		end
	end
end

function DoMoveStartingLocation(player, plot)
	local osplot = Players[player]:GetStartingPlot();
	gT.Players[player].StartingPlot = plot:GetPlotIndex()
	Players[player]:SetStartingPlot(plot)
	gT.Players[player].MovedLocation = true;
	for iUnit in Players[player]:Units() do
		iUnit:SetXY(plot:GetX(), plot:GetY())
	end
	local iTeam = Players[player]:GetTeam();
	for ix = -5, 5 do
		for iy = -5, 5 do
			local nPlot = Map.GetPlot(osplot:GetX() +ix, osplot:GetY() + iy);
			if nPlot then
				if not nPlot:IsVisible(iTeam) then
					nPlot:SetRevealed(iTeam, false);
				end
			end
		end	
	end
end

function MedievalAssignCultures(minTab, majTab)
	local hTab = ITG_ScenarioCultures[gT.Globals.ScenarioType];
	local nMtotal = #majTab;
	local vAdded = {0, 0, 0, 0}
	for i, player in pairs(majTab) do 
		local irValue = GetPredeterminedCulturalType(player)
		if irValue == ITG_CultureTypes["CULTURE_CHRISTIANS"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_CHRISTIANS"], -1)
			vAdded[1] = vAdded[1] + 1;
		elseif irValue == ITG_CultureTypes["CULTURE_MUSLIMS"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_MUSLIMS"], -1)
			vAdded[2] = vAdded[2] + 1;
		elseif irValue == ITG_CultureTypes["CULTURE_PAGANS"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_PAGANS"], -1)
			vAdded[4] = vAdded[4] + 1;
		elseif irValue == ITG_CultureTypes["CULTURE_FAR_EAST"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_FAR_EAST"], -1)
			vAdded[3] = vAdded[3] + 1;
		elseif irValue == 1000 + ITG_SubcultureTypes["SUBCULTURE_PAG_NOMADIC"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_PAGANS"], ITG_SubcultureTypes["SUBCULTURE_PAG_NOMADIC"])
			vAdded[4] = vAdded[4] + 1;
		elseif irValue == 1000 + ITG_SubcultureTypes["SUBCULTURE_PAG_VIKINGS"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_PAGANS"], ITG_SubcultureTypes["SUBCULTURE_PAG_VIKINGS"])
			vAdded[4] = vAdded[4] + 1;
		elseif irValue == 1000 + ITG_SubcultureTypes["SUBCULTURE_M_AFRICAN"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_MUSLIMS"], ITG_SubcultureTypes["SUBCULTURE_M_AFRICAN"])
			vAdded[2] = vAdded[2] + 1;
		end
	end
	local tMajorPerOne = 100 / nMtotal;
	local exPerc = {35, 30, 18, 17}
	if LastRandom(1,3) == 2 then
		exPerc = {30, 25, 10, 35}
	end
	local vHelpT = {};
	local leftOvers = {};
	for i = 1, 4 do
		vHelpT[i] = math.floor(exPerc[i] / tMajorPerOne);
		leftOvers[i] = {i, math.floor(100 * (exPerc[i] - vHelpT[i] * tMajorPerOne))};
	end

	for i = 1, nMtotal - vHelpT[1] - vHelpT[2] - vHelpT[3] - vHelpT[4] do
		local gLucky = GetRandomFromBiasedChanceTable(leftOvers);
		vHelpT[gLucky] = vHelpT[gLucky] + 1
	end
	local orderOfdone = {{1, vAdded[1]}, {2, vAdded[2]}, {3, vAdded[3]}, {4, vAdded[4]}};
	table.sort(orderOfdone, SortFunctionForGreaterInteger2)
	local gCultType = {ITG_CultureTypes["CULTURE_CHRISTIANS"], ITG_CultureTypes["CULTURE_MUSLIMS"], ITG_CultureTypes["CULTURE_FAR_EAST"], ITG_CultureTypes["CULTURE_PAGANS"]}
	for i = 1, 4 do
		local nextID = orderOfdone[i][1];
		if vHelpT[nextID] > vAdded[nextID] then
		--	if vHelpT[nextID] > 0 then
				local numTryToAdd = vHelpT[nextID] - vAdded[nextID];
				for j = 1, numTryToAdd do
					if vAdded[nextID] == 0 then
						local tLuckTab = {}
						for it, player in pairs(majTab) do
							if not gT.Players[player].CultureType then
								table.insert(tLuckTab, player)
							end
						end
						if #tLuckTab > 0 then
							vAdded[nextID] = vAdded[nextID] + 1;
							local lP = tLuckTab[LastRandom(1,#tLuckTab)]
							AssignCultureTypeToPlayer(lP, gCultType[nextID])
						end
					else
						local minDis = 9999;
						local bestPlayer;
						for it, player in pairs(majTab) do
							if not gT.Players[player].CultureType then
								local dis = 0
								local plot = Players[player]:GetStartingPlot()
								for hit, hplayer in pairs(majTab) do
									if gT.Players[hplayer].CultureType == gCultType[nextID]  then
										local hplot = Players[hplayer]:GetStartingPlot()
										dis = dis + Map.PlotDistance(hplot:GetX(), hplot:GetY(), plot:GetX(), plot:GetY())
									end
								end
								if dis < minDis then
									minDis = dis;
									bestPlayer = player;
								end
							end
						end
						if bestPlayer then
							vAdded[nextID] = vAdded[nextID] + 1;
							AssignCultureTypeToPlayer(bestPlayer, gCultType[nextID])
						end
					end
				end
		--	end
		end
	end
	--minors
	if #minTab > 0 then
		local tMinorPerOne = 100 / #minTab;
		vHelpT[1] = math.floor(38 / tMinorPerOne);
		vHelpT[2] = math.floor(32 / tMinorPerOne);
		vHelpT[3] = math.floor(18 / tMinorPerOne);
		vHelpT[4] = math.floor(12 / tMinorPerOne);
		leftOvers = {{1, math.floor(100 * (38 - vHelpT[1] * tMinorPerOne))},
		{2, math.floor(100 * (32 - vHelpT[2] * tMinorPerOne))},
		{3, math.floor(100 * (18 - vHelpT[3] * tMinorPerOne))},
		{4, math.floor(100 * (12 - vHelpT[4] * tMinorPerOne))}	}
		for i = 1, #minTab - vHelpT[1] - vHelpT[2] - vHelpT[3] - vHelpT[4] do
			local gLucky = GetRandomFromBiasedChanceTable(leftOvers);
			vHelpT[gLucky] = vHelpT[gLucky] + 1
		end
		orderOfdone = {{1, vAdded[1]}, {2, vAdded[2]}, {3, vAdded[3]}, {4, vAdded[4]}};
		table.sort(orderOfdone, SortFunctionForGreaterInteger2)
		for i = 1, 4 do
			local nextID = orderOfdone[i][1];
			if vHelpT[nextID] > 0 then
			--	if vHelpT[nextID] > 0 then
					for j = 1, vHelpT[nextID] do
						minTab = Shuffle(minTab)
						local minDis = 9999;
						local bestPlayer;
						for it, player in pairs(minTab) do
							if not gT.Players[player].CultureType then
								local dis = 0
								local plot = Players[player]:GetStartingPlot()
								for hit, hplayer in pairs(majTab) do
									if gT.Players[hplayer].CultureType == gCultType[nextID]  then
										local hplot = Players[hplayer]:GetStartingPlot()
										dis = dis + Map.PlotDistance(hplot:GetX(), hplot:GetY(), plot:GetX(), plot:GetY())
									end
								end
								if dis < minDis then
									minDis = dis;
									bestPlayer = player;
								end
							end
						end
						if bestPlayer then
							vAdded[nextID] = vAdded[nextID] + 1;
							AssignCultureTypeToPlayer(bestPlayer, gCultType[nextID])
						end
					end
			--	end
			end
		end
	end
	for i, data in pairs(gT.Players) do
		if not gT.Players[i].CultureType then
			print("Very bad 123231", i)
			AssignCultureTypeToPlayer(i, hTab[math.random(1,#hTab)])
		end
	end
end

function ModernAssignCultures(minTab, majTab)
	for i, player in pairs(majTab) do 
		local irValue = GetPredeterminedCulturalType(player)
		if irValue == ITG_CultureTypes["CULTURE_MODERN"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_MODERN"], -1)
		end
	end
	for i, data in pairs(gT.Players) do
		if not gT.Players[i].CultureType then
			AssignCultureTypeToPlayer(i, ITG_CultureTypes["CULTURE_MODERN"])
		end
	end
	for g, gh in pairs(gT.Players) do
		if not Players[g]:IsMinorCiv() then
			gT.Globals.NoFreeManContinents[GetContID(Players[g]:GetStartingPlot(), 8129931)] = 1;
		end
	end
end

function IndustrialAssignCultures(minTab, majTab)
	local inducivM = math.max(1, math.floor(0.5 * #majTab))
	local inducivm = math.max(1, math.floor(0.5 * #minTab))
	local pcolcivM = math.max(1, math.floor(0.3 * #majTab))
	local pcolcivm = math.max(1, math.floor(0.3 * #minTab))
	local porientcivM = math.max(1, #majTab - pcolcivM - inducivM)
	local porientcivm = math.max(1, #minTab - pcolcivm - inducivm)
	local hTab = ITG_ScenarioCultures[gT.Globals.ScenarioType];
	for i, player in pairs(majTab) do 
		local irValue = GetPredeterminedCulturalType(player)
		if irValue == ITG_CultureTypes["CULTURE_INDUSTRIAL"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_INDUSTRIAL"], -1)
			inducivM = inducivM - 1;
		elseif irValue == 1000 + ITG_SubcultureTypes["SUBCULTURE_IND_ORIENTAL"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_INDUSTRIAL"], ITG_SubcultureTypes["SUBCULTURE_IND_ORIENTAL"])
			porientcivM = porientcivM - 1;
		elseif irValue == 1000 + ITG_SubcultureTypes["SUBCULTURE_IND_POSTCOLONIAL"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_INDUSTRIAL"], ITG_SubcultureTypes["SUBCULTURE_IND_POSTCOLONIAL"])
			pcolcivM = pcolcivM - 1;
		end
	end
	majTab = Shuffle(majTab);
	minTab = Shuffle(minTab);
	for i, player in pairs(majTab) do
		if not gT.Players[player].CultureType then
			local hTab = {}
			if pcolcivM > 0 then
				table.insert(hTab, 2)
			end
			if porientcivM > 0 then
				table.insert(hTab, 1)
			end
			if inducivM > 0 then
				table.insert(hTab, 0)
			end
			if #hTab > 0 then
				local lV = hTab[LastRandom(1,#hTab)]
				if lV == 0 then
					inducivM = inducivM - 1;
					AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_INDUSTRIAL"], -1)
				elseif lV == 1 then
					porientcivM = porientcivM - 1;
					AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_INDUSTRIAL"], ITG_SubcultureTypes["SUBCULTURE_IND_ORIENTAL"])
				else
					pcolcivM = pcolcivM - 1;
					AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_INDUSTRIAL"], ITG_SubcultureTypes["SUBCULTURE_IND_POSTCOLONIAL"])
				end
			else
				print("Logic error 9324924124/12412312#")
			end
		end
	end
	for i, player in pairs(minTab) do
		if not gT.Players[player].CultureType then
			local hTab = {}
			if pcolcivm > 0 then
				table.insert(hTab, 2)
			end
			if porientcivm > 0 then
				table.insert(hTab, 1)
			end
			if inducivm > 0 then
				table.insert(hTab, 0)
			end
			if #hTab > 0 then
				local lV = hTab[LastRandom(1,#hTab)]
				if lV == 0 then
					inducivm = inducivm - 1;
					AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_INDUSTRIAL"], -1)
				elseif lV == 1 then
					porientcivm = porientcivm - 1;
					AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_INDUSTRIAL"], ITG_SubcultureTypes["SUBCULTURE_IND_ORIENTAL"])
				else
					pcolcivm = pcolcivm - 1;
					AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_INDUSTRIAL"], ITG_SubcultureTypes["SUBCULTURE_IND_POSTCOLONIAL"])
				end
			else
				print("Logic error 12412664632/12412312#")
			end
		end
	end
--ratio 20% oriental 30% postcolonial 50% industrial
	for i, data in pairs(gT.Players) do
		if not gT.Players[i].CultureType then
			AssignCultureTypeToPlayer(i, hTab[math.random(1,#hTab)])
		end
	end
	for g, gh in pairs(gT.Players) do
		if not Players[g]:IsMinorCiv() then
			gT.Globals.NoFreeManContinents[GetContID(Players[g]:GetStartingPlot(), 971237)] = 1;
		end
	end
end

function NewWorldAssignCultures(minTab, majTab)
	local hTab = ITG_ScenarioCultures[gT.Globals.ScenarioType];
	local numcolonists = math.max(1, #majTab/3 + math.random(1,2) * math.pow(-1, math.random(1,2)))
	local tabOfLocoLocos = {}
	local nmColonial = 0;
	for i, player in pairs(majTab) do 
		local irValue = GetPredeterminedCulturalType(player)
		if irValue == ITG_CultureTypes["CULTURE_COLONIALISTS"] then
			nmColonial = nmColonial + 1;
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_COLONIALISTS"], -1)
		elseif irValue == 1000 + ITG_SubcultureTypes["SUBCULTURE_NF_ASIAN"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_NATIVE_FORCES"], ITG_SubcultureTypes["SUBCULTURE_NF_ASIAN"])
		elseif irValue == ITG_CultureTypes["CULTURE_NATIVE_FORCES"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_NATIVE_FORCES"], -1)
		elseif irValue == 1000 + ITG_SubcultureTypes["SUBCULTURE_COL_ARABIAN"] then
			nmColonial = nmColonial + 1;
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_COLONIALISTS"], ITG_SubcultureTypes["SUBCULTURE_COL_ARABIAN"])
		elseif irValue == 1000 + ITG_SubcultureTypes["SUBCULTURE_NF_AFRICAN"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_NATIVE_FORCES"], ITG_SubcultureTypes["SUBCULTURE_NF_AFRICAN"])
		elseif irValue == 1000 + ITG_SubcultureTypes["SUBCULTURE_COL_TROPICAL"] then
			AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_COLONIALISTS"], ITG_SubcultureTypes["SUBCULTURE_COL_TROPICAL"])
			table.insert(tabOfLocoLocos, player)
		end
	end
	numcolonists = math.max(numcolonists, 0)
	local contTab = {}
	for it, majPlayer in pairs(majTab) do
		local contID = GetContID(Map.GetPlotByIndex(gT.Players[majPlayer].StartingPlot), 4134124);
		contTab[contID] = (contTab[contID] or 0) + 1;
	end
	local chosenCont;
	for contID, sth in pairs(contTab) do
		if sth >= numcolonists then
			if not chosenCont then
				chosenCont = contID;
			else
				if math.abs(sth - numcolonists) < math.abs(contTab[chosenCont] - numcolonists) then
					if math.random(1,4) < 4 then
						chosenCont = contID;
					end
				elseif math.random(1,4) < 2 then
					chosenCont = contID;
				end
			end
		end
	end
	local hval, cTab, center = 999, {}, nil;
	for it, majPlayer in pairs(majTab) do
		local contID = GetContID(Map.GetPlotByIndex(gT.Players[majPlayer].StartingPlot), 124124);
		if (not chosenCont) or (contID == chosenCont) then
			local val, pTab = GetNumberValueOfCenterArea(majPlayer, majTab, chosenCont, numcolonists)
			if hval > val then
				hval = val;
				cTab = pTab;
				center = majPlayer;
			end
		end
	end
	if #cTab == 0 then
		print("Much bad, error 19424")
		for i, data in pairs(gT.Players) do
			if not gT.Players[player].CultureType then
				AssignCultureTypeToPlayer(i, hTab[math.random(1,#hTab)])
			end
		end
	else
		if PreGame.GetGameOption("GAMEOPTION_ITG_MAP_MODIFICATIONS") == 1 then
			if PreGame.GetGameOption("GAMEOPTION_ITG_SUBCULTURE_TROPICAL") == 1 then
				local navcolon = {};
				for it, player in pairs(cTab) do
					if not gT.Players[player].CultureType then
						table.insert(navcolon, player)
					end
				end
				if #navcolon + nmColonial > LastRandom(2, 4) then
					if (#navcolon > 0) and (#tabOfLocoLocos == 0) then
						if LastChance(50) then -- mystery code above; added this to reduce appearance of tropico guy
							local vlplayer = navcolon[LastRandom(1,#navcolon)];
							AssignCultureTypeToPlayer(vlplayer, ITG_CultureTypes["CULTURE_COLONIALISTS"], ITG_SubcultureTypes["SUBCULTURE_COL_TROPICAL"])
							table.insert(tabOfLocoLocos, vlplayer)
						end
					end
				end
			end
		end
		if #tabOfLocoLocos > 0 then
			GeneratePossibleCityLocations(nil, false, 7)
			local clc = GetCityLocations();
			ClearCityLocations();
			local grandestOcean = GetSizeOfLargestOcean(true);
			local fineLocations = {}
			for iPlotIndex, hc in pairs(clc) do
				local iPlot = Map.GetPlotByIndex(iPlotIndex)
				if IsCoastOfOceanOrOcean(iPlot, grandestOcean) then
					if iPlot:GetTerrainType() ~= tTundra then
						local contSize = GetSizeOfCont(GetContID(iPlot, 891313))
						if contSize > 5 then
							table.insert(fineLocations, {iPlot, math.max(10, contSize)})
						end
					end
				end
			end
			print("We are trying to move loco locos", #tabOfLocoLocos, #fineLocations)
			if #tabOfLocoLocos <= #fineLocations then
				fineLocations = Shuffle(fineLocations);
				table.sort(fineLocations, SortFunctionForLowestMapDistance)
				for i, player in pairs(tabOfLocoLocos) do
					DoMoveStartingLocation(player, fineLocations[i][1])
				end
			end
		end
		for it, player in pairs(cTab) do
			if not gT.Players[player].CultureType then
				AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_COLONIALISTS"])
			end
		end
		for it, player in pairs(majTab) do
			if not gT.Players[player].CultureType then
				AssignCultureTypeToPlayer(player, ITG_CultureTypes["CULTURE_NATIVE_FORCES"])
			end
		end
		local joseTab = {}
		local numminorcolonists = math.max(1, #minTab/3 + math.random(1,2) * math.pow(-1, math.random(1,2)))
		local iPlot = Map.GetPlotByIndex(gT.Players[center].StartingPlot)
		gT.Globals.NoFreeManContinents[GetContID(iPlot, 54366)] = 1;
		for i, minor in pairs(minTab) do
			if gT.Globals.TreasuryCont == GetContID(Players[minor]:GetStartingPlot()) then
				AssignCultureTypeToPlayer(minor, ITG_CultureTypes["CULTURE_NATIVE_FORCES"])
			end
		end
		for i, minor in pairs(minTab) do
			if not gT.Players[minor].CultureType then
				local mPlot = Map.GetPlotByIndex(gT.Players[minor].StartingPlot)
				table.insert(joseTab, {minor, Map.PlotDistance(mPlot:GetX(), mPlot:GetY(), iPlot:GetX(), iPlot:GetY())})
			end
		end
		table.sort(joseTab, SortFunctionForGreaterInteger2)
		for i, data in pairs(joseTab) do
			if i <= numminorcolonists then
				AssignCultureTypeToPlayer(data[1], ITG_CultureTypes["CULTURE_COLONIALISTS"])
			else
				AssignCultureTypeToPlayer(data[1], ITG_CultureTypes["CULTURE_NATIVE_FORCES"])
			end
		end
	end
	for i, data in pairs(gT.Players) do
		if not gT.Players[i].CultureType then
			print("Very bad 123231")
			AssignCultureTypeToPlayer(i, hTab[math.random(1,#hTab)])
		end
	end
end

function GetScenarioTypeOfNewGame()
	local irValue = Modding.OpenSaveData().GetValue("ITG_pgd_ScenarioEraStart")
	if irValue ~= -1 then
		return irValue
	end
	local tab = {}
	for scType, scIndex in pairs(ITG_ScenariosTypes) do
		table.insert(tab, scIndex);
	end
	return RandomFromTab(tab);
end

function AssignCultureFunction(hMinorTab, hMajorTab)
	InitializeDBCultCivLinked()
	if gT.Globals.ScenarioType == ITG_ScenariosTypes["SCENARIO_NEW_WORLD"] then
		NewWorldAssignCultures(hMinorTab, hMajorTab)
	elseif gT.Globals.ScenarioType == ITG_ScenariosTypes["SCENARIO_TRIAL"] then
		MedievalAssignCultures(hMinorTab, hMajorTab)
	elseif gT.Globals.ScenarioType == ITG_ScenariosTypes["SCENARIO_AGE_OF_TITANS"] then
		AncientAssignCultures(hMinorTab, hMajorTab)
	elseif gT.Globals.ScenarioType == ITG_ScenariosTypes["SCENARIO_LIGHT_OF_HOPE"] then
		IndustrialAssignCultures(hMinorTab, hMajorTab)
	elseif gT.Globals.ScenarioType == ITG_ScenariosTypes["SCENARIO_SLEEP_OF_REASON"] then
		ModernAssignCultures(hMinorTab, hMajorTab)
	else
		print("New scenario? assign culture function")
	end
	DoAddDBDataFromCultures()
end

function GenerateBaseRelationsOnStart(hMinorTab, hMajorTab)
	if gT.Globals.ScenarioType == ITG_ScenariosTypes["SCENARIO_NEW_WORLD"] then
		NewWorldAssignRelations(hMinorTab, hMajorTab)
	elseif gT.Globals.ScenarioType == ITG_ScenariosTypes["SCENARIO_TRIAL"] then
		MedievalAssignRelations(hMinorTab, hMajorTab)
	elseif gT.Globals.ScenarioType == ITG_ScenariosTypes["SCENARIO_AGE_OF_TITANS"] then
		AncientAssignRelations(hMinorTab, hMajorTab)
	elseif gT.Globals.ScenarioType == ITG_ScenariosTypes["SCENARIO_LIGHT_OF_HOPE"] then
		IndustrialAssignRelations(hMinorTab, hMajorTab)
	elseif gT.Globals.ScenarioType == ITG_ScenariosTypes["SCENARIO_SLEEP_OF_REASON"] then
		ModernAssignRelations(hMinorTab, hMajorTab)
	else
		print("New scenario? generate base releations function")
	end
end

function PopulateGlobalInfos()
	gT.Globals.ResourcesInScenario = {}
	for row in DB.Query("SELECT ResourceType FROM ITG_ScenarioResources WHERE ScenarioType = '" .. DBScenario.Type .. "'") do
		gT.Globals.ResourcesInScenario[GameInfo.Resources[row.ResourceType].ID] = true;
	end
	if DBScenario.GlobalizationTech then
		gT.Globals.GlobalizationTech = {};
	end
	gT.Globals.FreeTechForBackwardCiv = {};
	if (DBScenario.BackwardFreeTech or 0) > 0 then
		local h = DBScenario.BackwardFreeTech;
		while h < 100 do
			gT.Globals.FreeTechForBackwardCiv[h] = true;
			h = h + DBScenario.BackwardFreeTech;
		end
	end
	gT.Globals.AcademyRewards = {DBScenario.AcaReward1,DBScenario.AcaReward2,DBScenario.AcaReward3}

	gT.Globals.GoodEras = {}
	for row in DB.Query("SELECT EraType FROM ITG_ScenarioEras WHERE ScenarioType = '" .. DBScenario.Type .. "'") do
		gT.Globals.GoodEras[row.EraType] = true;
	end
end

function RequestCultureTypeGoverments(cType)
	if not g_cultureGoverments then
		g_cultureGoverments = {};
		for row in DB.Query("SELECT * FROM ITG_CultureGoverments") do
			if not g_cultureGoverments[row.CultureType] then
				g_cultureGoverments[row.CultureType] = {}
			end
			table.insert(g_cultureGoverments[row.CultureType], {ITG_GovermentTypes[row.GovermentType], row.Chance})
		end
	end
	return g_cultureGoverments[ITG_CultureData[cType].Type]
end

function GetDBCulturePlayerInfo(iPlayer, string)
	if not gT.Players[iPlayer] then
		print("GetDBCulturePlayerInfo", iPlayer, string);
	end
	if not string then
		print("GetDBCulturePlayerInfo lack of key!") 
	end
	return GetDBSubCultureInfo(gT.Players[iPlayer].CultureType, gT.Players[iPlayer].SubcultureType, string)
end

function GetDBSubCultureInfo(cType, subType, string)
	if subType then
		if ITG_SubcultureData[subType][string] ~= nil then
			return ITG_SubcultureData[subType][string];
		end
	end
	return ITG_CultureData[cType][string];
end
MapModData.GetDBCulturePlayerInfo = GetDBCulturePlayerInfo

function GetPlayerDisabledTechs(iPlayer)
	if gT.Players[iPlayer].SubcultureType then
		return ITG_CultureDisabledTechs[ITG_SubcultureData[gT.Players[iPlayer].SubcultureType].Type] or {};
	end
	return ITG_CultureDisabledTechs[ITG_CultureData[gT.Players[iPlayer].CultureType].Type] or {};
end

LuaEvents.ITGSG_RequestDisabledTechs.Add(function(iPlayer)
	MapModData.ITGSG = GetPlayerDisabledTechs(iPlayer)
end)

LuaEvents.ITGSG_Civilopedia_PopulateList_Units.Add(function()
	local tab = {}
	for type, scIndex in pairs(ITG_ScenariosTypes) do
		local sTypes = {}
		for i, v in pairs(ITG_ScenarioCultures[scIndex]) do
			for j, c in pairs(ITG_CultureSubcultures[v]) do
				table.insert(sTypes, c)
			end
		end
		tab[scIndex] = GetAllPossibleUnitsFromClasses(scIndex, ITG_ScenarioCultures[scIndex], sTypes)
		tab[scIndex] = GetExtraSpecialUnits(scIndex, tab[scIndex])
	end
	table.insert(tab, GetSpecialUnitsCivilopedia())
	for a, b in pairs(tab) do
		tab[a] = FinalizeCivilopediaUnits(tab[a], "Units", "Class", "UnitClasses")
	end
	MapModData.ITGSG = tab;
end)

LuaEvents.ITGSG_Civilopedia_PopulateList_Buildings.Add(function()
	local tab = {}
	tab[0] = GetReligiousBuildingsForCivilopedia();
	for type, scIndex in pairs(ITG_ScenariosTypes) do
		local sTypes = {}
		for i, v in pairs(ITG_ScenarioCultures[scIndex]) do
			for j, c in pairs(ITG_CultureSubcultures[v]) do
				table.insert(sTypes, c)
			end
		end
		tab[scIndex] = GetAllPossibleBuildingsFromClasses(scIndex, ITG_ScenarioCultures[scIndex], sTypes)
	end
	for a, b in pairs(tab) do
		tab[a] = FinalizeCivilopediaUnits(tab[a], "Buildings", "BuildingClass", "BuildingClasses")
	end
	MapModData.ITGSG = tab;
end)

LuaEvents.ITGSG_RequestScenarioNames.Add(function()
	local tab = {{}, {3,2,1,4,5}, gT.Globals.ScenarioType}
	for row in DB.Query("SELECT ID, Description FROM ITG_Scenarios") do
		tab[1][row.ID] = Locale.Lookup(row.Description)
	end
	MapModData.ITGSG = tab;
end)

LuaEvents.ITGSG_Civilopedia_PopulateList_Wonders.Add(function()
	local tab = {}
	for row in DB.Query("SELECT ID, IFNULL(ActiveWonderTag, 'NO_WONDER') AS wTag FROM ITG_Scenarios") do
		tab[row.ID] = GetWondersCivilopediaSc(row.wTag)
	end
	MapModData.ITGSG = tab;
end)

LuaEvents.ITGCivilopediaRequestPromData.Add(function()
	local gData = {}
	--12 headings here
	--veteran
	--general
	--admiral
	--mercenary
	--other (all unused without unvisibility)
	table.insert(gData, {Proms = GetClassPromsOfType(1), Name = "Land Melee"})
	table.insert(gData, {Proms = GetClassPromsOfType(2), Name = "Land Mounted"})
	table.insert(gData, {Proms = GetClassPromsOfType(5), Name = "Land Ranged"})
	table.insert(gData, {Proms = GetClassPromsOfType(4), Name = "Land Siege"})
	table.insert(gData, {Proms = GetClassPromsOfType(3), Name = "Naval Melee"})
	table.insert(gData, {Proms = GetClassPromsOfType(6), Name = "Naval Ranged"})
	table.insert(gData, {Proms = GetClassPromsOfType(11), Name = "Bombers"})
	table.insert(gData, {Proms = GetClassPromsOfType(9), Name = "Carriers"})
	table.insert(gData, {Proms = GetClassPromsOfType(10), Name = "Fighters"})
	table.insert(gData, {Proms = GetClassPromsOfType(12), Name = "Submarines"})
	table.insert(gData, {Proms = GetClassPromsOfType(8), Name = "Tanks"})
	table.insert(gData, {Proms = GetAdmiralPromTabAll(1), Name = "Admiral"})
	table.insert(gData, {Proms = GetGeneralPromTabAll(2), Name = "General"})
	table.insert(gData, {Proms = GetTabOfMercenaryPromAll(3), Name = "Mercenary"})
	table.insert(gData, {Proms = GetTabOfVeteranPromAll(4), Name = "Veteran"})

	local nTto = {}
	for j, v in pairs(gData) do
		for i, prom in pairs(v.Proms) do
			nTto[prom] = true;
		end
	end
	local other = {}
	for unitPromotion in GameInfo.UnitPromotions() do
		if (not nTto[unitPromotion.ID]) and unitPromotion.ShowIcon then
			table.insert(other, unitPromotion.ID)
		end
	end
	table.insert(gData, {Proms = other, Name = "Other"}) 
	MapModData.ITGSG = gData
end)

LuaEvents.ITGCivilopediaRequestAvNowInnos.Add(function(iPlayer)
	local tab = {}
	if gT.Players[iPlayer] then
		for i, inno in pairs(gT.Players[iPlayer].Innovations) do
			tab[inno] = true;
		end
	end
	MapModData.ITGSG = tab;
end)

LuaEvents.ITGCivilopediaRequestTechs.Add(function()
	local tab = {}
	for type, i in pairs(ITG_ScenariosTypes) do
		local tGeras = {}
		for row in DB.Query("SELECT EraType FROM ITG_ScenarioEras WHERE ScenarioType = '" .. type .. "'") do
			tGeras[row.EraType] = true;
		end
		tab[i] = {Index = i, Techs = {}}
		for row in GameInfo.Technologies() do
			if not row.IsInnovation then
				if tGeras[row.Era] then
					table.insert(tab[i].Techs, row.ID)
				end
			end
		end
	end
	local hh = {Index = #tab + 1, Techs = {}}
	for row in GameInfo.Technologies() do
		if row.IsInnovation then
			table.insert(hh.Techs, row.ID)
		end
	end
	table.insert(tab, hh)
	MapModData.ITGSG = tab;
end)