
-------------------------------------------------
-- Choose Pantheon Popup
-------------------------------------------------

include( "IconSupport" );
include( "InstanceManager" );
include( "TechButtonInclude" );
include( "TechHelpInclude" );

local maxTechNameLength = 22 - Locale.Length(Locale.Lookup("TXT_KEY_TURNS"));
local maxSmallButtons = 5;

local fTime = true;
local needConfirm = false;
local isVisible = false;
local currentID = -1;
local dataTable = {}
local confirminfo1, confirminfo2, confirminfo3, confirminfo4;
local doubleDataInfo;
local doubleData;
local wasAlreadyUpdated = false;
local specialType;

-- Used for Piano Keys
local ltBlue = {19/255,32/255,46/255,120/255};
local dkBlue = {12/255,22/255,30/255,120/255};
local ltRed = {45/255,19/255,19/255,120/255};
local dkRed = {28/255,11/255,11/255,120/255};

local g_ItemManager = InstanceManager:new( "ItemInstance", "Button", Controls.ItemStack );
local g_ItemManagerUnit = InstanceManager:new( "UniqueUnitIstance", "Button", Controls.ItemStack );
local g_TechInstanceManager = InstanceManager:new( "TechButtonInstance", "HelpGridID",  Controls.ItemStack );
local bHidden = true;
local g_bPantheons = true;

local screenSizeX, screenSizeY = UIManager:GetScreenSizeVal()
local spWidth, spHeight = Controls.ItemScrollPanel:GetSizeVal();

-- Original UI designed at 1050px 
local heightOffset = screenSizeY - 1020;

spHeight = spHeight + heightOffset;
Controls.ItemScrollPanel:SetSizeVal(spWidth, spHeight); 
Controls.ItemScrollPanel:CalculateInternalSize();
Controls.ItemScrollPanel:ReprocessAnchoring();

local bpWidth, bpHeight = Controls.BottomPanel:GetSizeVal();
--bpHeight = bpHeight * heightRatio;
bpHeight = bpHeight + heightOffset 

Controls.BottomPanel:SetSizeVal(bpWidth, bpHeight);
Controls.BottomPanel:ReprocessAnchoring();

-------------------------------------------------
-------------------------------------------------
function OnPopupMessage()

   	UIManager:QueuePopup( ContextPtr, PopupPriority.InGameUtmost );
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function OnClose()
--    UIManager:DequeuePopup(ContextPtr);
	if doubleDataInfo then
		doubleDataInfo = nil;
	elseif currentID == #dataTable then
		currentID = 1;
	else
		currentID = currentID + 1;
	end
	RefreshList()
end
Controls.CloseButton:RegisterCallback( Mouse.eLClick, OnClose );


function OnRealClose()
	dataTable = {}
	isVisible = false;
	UIManager:DequeuePopup(ContextPtr)
end
Controls.RealCloseButton:RegisterCallback( Mouse.eLClick, OnRealClose );

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function OnConfirmed()
	wasAlreadyUpdated = false;
	if doubleDataInfo then
		doubleData = confirminfo3;
		RefreshList()
	else
		if specialType then
			if specialType == "Action" then
				LuaEvents.ITGSG_HumanChooseLegalAction(Game.GetActivePlayer(), confirminfo1, confirminfo2, confirminfo3, confirminfo4)
			elseif specialType == "GovRelocation" then
				LuaEvents.ITGSG_HumanChooseGovRelocation(Game.GetActivePlayer(), confirminfo1, confirminfo2, confirminfo3, true)
			elseif specialType == "Embargo" then
				LuaEvents.ITGSG_HumanChooseEmbargoInCH(Game.GetActivePlayer(), confirminfo1, confirminfo2, confirminfo3, confirminfo4)
			end
		else
			LuaEvents.ITGSG_HumanChooseSimpleBonus(Game.GetActivePlayer(), confirminfo1, confirminfo2, confirminfo3, confirminfo4)
		end
		if MapModData.ITGSG then
			Events.AudioPlay2DSound(MapModData.ITGSG);
			MapModData.ITGSG = nil;
		else
			Events.AudioPlay2DSound("AS2D_INTERFACE_POLICY");
		end
		if not wasAlreadyUpdated then
			local hid = currentID;
			if #dataTable == 1 then
				dataTable = {}
				isVisible = false;
				UIManager:DequeuePopup(ContextPtr)
			else
				if hid == #dataTable then
					currentID = 1;
				end
				table.remove(dataTable, hid)
				RefreshList()
			end
		end
	end
end

function OnYes( )
	Controls.ChooseConfirm:SetHide(true);
	OnConfirmed();
end
Controls.Yes:RegisterCallback( Mouse.eLClick, OnYes );

function OnNo( )
	Controls.ChooseConfirm:SetHide(true);
end
Controls.No:RegisterCallback( Mouse.eLClick, OnNo );



function UpdateLowButton()
	if doubleDataInfo then
		Controls.NextButton:SetText("Back")
		Controls.CloseButton:SetHide(false)
		Controls.NextButton:SetDisabled(false)
	else
		Controls.NextButton:SetText("Next (" .. currentID .. "/" .. #dataTable .. ")")
		Controls.CloseButton:SetHide(#dataTable == 1)
		Controls.NextButton:SetDisabled(2 > #dataTable)
	end
	Controls.ChooseConfirm:SetHide(true);
end

function SelectTheBonus(type, id, exd1, exd2, isDoubleDataInfo)
	confirminfo1, confirminfo2, confirminfo3, confirminfo4, doubleDataInfo = type, id, exd1, exd2, isDoubleDataInfo;
	if needConfirm and (not isDoubleDataInfo) then
		Controls.ChooseConfirm:SetHide(false);
	else
		OnConfirmed();
	end
end

function RefreshList()
	g_ItemManager:ResetInstances();
	g_ItemManagerUnit:ResetInstances();
	g_TechInstanceManager:ResetInstances();
		
	local pPlayer = Players[Game.GetActivePlayer()];
	CivIconHookup( pPlayer:GetID(), 64, Controls.CivIcon, Controls.CivIconBG, Controls.CivIconShadow, false, true );
	
	specialType = dataTable[currentID].SpecialType;
	if doubleDataInfo then
		if doubleDataInfo == "Reformation" then
			Controls.PanelTitle:SetText("Choose belief to be replaced.")
			for i, belief in pairs(doubleData) do
				local itemInstance = g_ItemManagerUnit:GetInstance();
				itemInstance.Name:SetText(Locale.Lookup(GameInfo.Beliefs[belief].ShortDescription));
				itemInstance.Description:SetText("[NEWLINE]" .. Locale.Lookup(GameInfo.Beliefs[belief].Description));
				itemInstance.Button:SetDisabled(false)

				itemInstance.Frame:SetHide(true)
				itemInstance.ProdImage:SetHide(true)		
				itemInstance.Button:RegisterCallback(Mouse.eLClick, function() SelectTheBonus(dataTable[currentID].BonusType, confirminfo2, belief, nil, nil); end);
	
				if(bTickTock == false) then
					itemInstance.Box:SetColorVal(unpack(ltBlue));
				else
					itemInstance.Box:SetColorVal(unpack(dkBlue));
				end
				local buttonWidth, buttonHeight = itemInstance.Button:GetSizeVal();
				local descWidth, descHeight = itemInstance.Description:GetSizeVal();
				local newHeight = descHeight + 40;	
				itemInstance.Button:SetSizeVal(buttonWidth, newHeight);
				itemInstance.Box:SetSizeVal(buttonWidth + 20, newHeight);
				itemInstance.BounceAnim:SetSizeVal(buttonWidth + 20, newHeight + 5);
				itemInstance.BounceGrid:SetSizeVal(buttonWidth + 20, newHeight + 5);
				bTickTock = not bTickTock;
			end
		elseif doubleDataInfo == "SocialDevelopment" then
			Controls.PanelTitle:SetText("Choose Social Bonus to be replaced.")
			for i, social in pairs(doubleData) do
				local itemInstance = g_ItemManagerUnit:GetInstance();
				itemInstance.Name:SetText(social.Name);
				itemInstance.Description:SetText("[NEWLINE]" .. social.Help);
				itemInstance.Button:SetDisabled(false)
				itemInstance.Frame:SetHide(false)
				itemInstance.ProdImage:SetHide(false)

				IconHookup( social.PortraitIndex, 64, social.IconAtlas, itemInstance.ProdImage )		
				itemInstance.Button:RegisterCallback(Mouse.eLClick, function() SelectTheBonus(dataTable[currentID].BonusType, confirminfo2, social.ID, nil, nil); end);
	
				if(bTickTock == false) then
					itemInstance.Box:SetColorVal(unpack(ltBlue));
				else
					itemInstance.Box:SetColorVal(unpack(dkBlue));
				end
				local buttonWidth, buttonHeight = itemInstance.Button:GetSizeVal();
				local descWidth, descHeight = itemInstance.Description:GetSizeVal();
				local newHeight = descHeight + 40;	
				itemInstance.Button:SetSizeVal(buttonWidth, newHeight);
				itemInstance.Box:SetSizeVal(buttonWidth + 20, newHeight);
				itemInstance.BounceAnim:SetSizeVal(buttonWidth + 20, newHeight + 5);
				itemInstance.BounceGrid:SetSizeVal(buttonWidth + 20, newHeight + 5);
				bTickTock = not bTickTock;
			end
		end
	else
		LuaEvents.ITGUpdateITGBonusOptionsQuicklyBeforeChoose(dataTable[currentID].BonusType, dataTable[currentID].BClass)
		if MapModData.ITGSG then
			dataTable[currentID] = MapModData.ITGSG;
			MapModData.ITGSG = nil;
		end
		if dataTable[currentID].LookAt then
			print("GC moving a camera", dataTable[currentID].LookAt);
			UI.LookAt(Map.GetPlotByIndex(dataTable[currentID].LookAt));
		end 
		if dataTable[currentID].IsTechButton then
			-- Rebuild some tables	
			if fTime then
				GatherInfoAboutUniqueStuff( GameInfo.Civilizations[pPlayer:GetCivilizationType()].Type, pPlayer:GetID());	
				fTime = false;
			end
		end
		Controls.PanelTitle:SetText(dataTable[currentID].TitleText)
		if Controls.PanelTitle:GetSizeY() > 40 then
			Controls.PanelTitle:SetOffsetY(18)
		else
			Controls.PanelTitle:SetOffsetY(30)
		end
		if dataTable[currentID].TitleIconTexture then
			Controls.ChooseImage:SetTexture(dataTable[currentID].TitleIconTexture)
			Controls.ChooseImage:SetTextureOffsetVal(0, 0)
		else
			IconHookup( dataTable[currentID].TitleIconIndex, 128, dataTable[currentID].TitleIconAtlas, Controls.ChooseImage )
		end
		for it, mdata in pairs(dataTable[currentID].UniqueOptions) do
			--BonusType
			--TitleText
			--TitleIconIndex
			--TitleIconAtlas
			
			--UniqueOptions:
			--Name
			--Help
			--ID
			--PortraitIndex (can be nil)
			--IconAtlas (can be nil)
			if dataTable[currentID].IsTechButton then
				local itemInstance = g_TechInstanceManager:GetInstance();
				AddITGTechButton( GameInfo.Technologies[mdata.ID], itemInstance)
				itemInstance.TechButton:RegisterCallback(Mouse.eLClick, function() SelectTheBonus(dataTable[currentID].BonusType, mdata.ID, mdata.ExtraInfo, nil, mdata.DoubleChooseType); end);
			else
				local itemInstance = g_ItemManagerUnit:GetInstance();
				itemInstance.Name:SetText(mdata.Name);
				itemInstance.Description:SetText(mdata.Help);
				itemInstance.Button:SetToolTipString(mdata.ToolTip or "");
				local minSizeY = 60;
				if mdata.IconAtlas then
					IconHookup( mdata.PortraitIndex, 64, mdata.IconAtlas, itemInstance.ProdImage )
					itemInstance.Frame:SetHide(false)
					itemInstance.ProdImage:SetHide(false)
					minSizeY = 80;
				else
					itemInstance.Frame:SetHide(true)
					itemInstance.ProdImage:SetHide(true)
				end				
				itemInstance.Button:RegisterCallback(Mouse.eLClick, function() SelectTheBonus(dataTable[currentID].BonusType, mdata.ID, mdata.ExtraInfo, nil, mdata.DoubleChooseType); end);
				itemInstance.Button:SetDisabled(mdata.Disabled);

		--
				if(bTickTock == false) then
					if mdata.Disabled then
						itemInstance.Box:SetColorVal(unpack(ltRed));
					else
						itemInstance.Box:SetColorVal(unpack(ltBlue));
					end
				else
					if mdata.Disabled then
						itemInstance.Box:SetColorVal(unpack(dkRed));
					else
						itemInstance.Box:SetColorVal(unpack(dkBlue));
					end
				end
				local buttonWidth, buttonHeight = itemInstance.Button:GetSizeVal();
				local descWidth, descHeight = itemInstance.Description:GetSizeVal();
				local newHeight = math.max(minSizeY, descHeight + 40);	
				itemInstance.Button:SetSizeVal(buttonWidth, newHeight);
				itemInstance.Box:SetSizeVal(buttonWidth + 20, newHeight);
				itemInstance.BounceAnim:SetSizeVal(buttonWidth + 20, newHeight + 5);
				itemInstance.BounceGrid:SetSizeVal(buttonWidth + 20, newHeight + 5);
				bTickTock = not bTickTock;
			end
		end
		if #dataTable[currentID].UniqueOptions == 0 then
			print("No options for bonus! ERROR!", dataTable[currentID].BonusType);
		end
	end

	UpdateLowButton()
	Controls.ItemStack:CalculateSize();
	Controls.ItemStack:ReprocessAnchoring();
	Controls.ItemScrollPanel:CalculateInternalSize();
end

function SelectPantheon(beliefID) 
	g_BeliefID = beliefID;
	local belief = GameInfo.Beliefs[beliefID];
	Controls.ConfirmText:LocalizeAndSetText("TXT_KEY_CONFIRM_PANTHEON", belief.ShortDescription);
	Controls.ChooseConfirm:SetHide(false);
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function ShowHideHandler( bIsHide, bInitState )

	bHidden = bIsHide;
    if( not bInitState ) then
        if( not bIsHide ) then
        	UI.incTurnTimerSemaphore();
        	Events.SerialEventGameMessagePopupShown(g_PopupInfo);
        	
        	RefreshList();
        
			local unitPanel = ContextPtr:LookUpControl( "/InGame/WorldView/UnitPanel/Base" );
			if( unitPanel ~= nil ) then
				unitPanel:SetHide( true );
			end
			
			local infoCorner = ContextPtr:LookUpControl( "/InGame/WorldView/InfoCorner" );
			if( infoCorner ~= nil ) then
				infoCorner:SetHide( true );
			end
               	
        else
      
			local unitPanel = ContextPtr:LookUpControl( "/InGame/WorldView/UnitPanel/Base" );
			if( unitPanel ~= nil ) then
				unitPanel:SetHide(false);
			end
			
			local infoCorner = ContextPtr:LookUpControl( "/InGame/WorldView/InfoCorner" );
			if( infoCorner ~= nil ) then
				infoCorner:SetHide(false);
			end
			
			if(g_PopupInfo ~= nil) then
				Events.SerialEventGameMessagePopupProcessed.CallImmediate(g_PopupInfo.Type, 0);
			end
            UI.decTurnTimerSemaphore();
        end
    end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );

----------------------------------------------------------------
-- 'Active' (local human) player has changed
----------------------------------------------------------------
function OnActivePlayerChanged()
	if (not Controls.ChooseConfirm:IsHidden()) then
		Controls.ChooseConfirm:SetHide(true);
    end
end
Events.GameplaySetActivePlayer.Add(OnActivePlayerChanged);

function DistributeInfoData(data)
	table.insert(dataTable, data)
	if not isVisible then
		isVisible = true;
		currentID = 1;
		OnPopupMessage()
	else
		UpdateLowButton()
	end
end
LuaEvents.ITGSG_CallQueueOfHumanDecisions.Add(DistributeInfoData)

function GetTextDescribingInnovationBasedOnEra(sct, bInnopair)
	if sct == 3 then
		return "Allows you to develop exclusive Innovation that other players will be unable to discover."
	elseif sct == 4 then
		return "Allows you to develop exclusive Innovation that other players will be able to acquire only through Globalization."
	elseif not bInnopair then
		return "Allows you to develop random Innovation.";
	end
	return "Allows you to choose between a pair of mutually exclusive innovations.";
end

function TriggeredWithPriority(priority)
	wasAlreadyUpdated = true;
	dataTable = {}
	currentID = 1;
	LuaEvents.ITGSG_RequestGT()
	local gPlayer = Game.GetActivePlayer();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	for type, extradata in pairs(gData.Players[gPlayer].HumanQueueActions) do
		table.insert(dataTable, extradata)
		if type == priority then
			currentID = #dataTable;
		end
	end
	for i, newextradata in pairs(gData.Players[gPlayer].NewHumanQueueActions) do
		table.insert(dataTable, newextradata)
		if newextradata.Text == priority then
			currentID = #dataTable;
		end
	end
	if priority == "CHOOSE TECHNOLOGY" and #gData.Players[gPlayer].TechChooseDecisions > 0 then
		currentID = #dataTable + 1;
	end
	for i, techoptions in pairs(gData.Players[gPlayer].TechChooseDecisions) do
		-- this will be needed to recode one day probably as well? Not truely working if two at one moment occurs? Just make sure it will not happen :)
		local thedata = {BonusType = "ChooseTechnology", TitleIconIndex = 2, TitleIconAtlas = "CITIZEN_ATLAS", TitleText = "Many technologies to choose from...", UniqueOptions = {}}
		for j, tech in pairs(techoptions) do
			if GameInfo.Technologies[tech].IsInnovation then
				table.insert(thedata.UniqueOptions, {
					Name = "Innovation",
					Help = "[NEWLINE]" .. GetTextDescribingInnovationBasedOnEra(gData.Globals.ScenarioType, MapModData.DBScenario.InnoPairs),
					PortraitIndex = 0, 
					IconAtlas = "TECH_ITG_ATLAS_1",
					ID = tech
				})
			else
				table.insert(thedata.UniqueOptions, {
					Name = Locale.Lookup(GameInfo.Technologies[tech].Description),
					Help = "[NEWLINE]" .. Locale.Lookup(GameInfo.Technologies[tech].Help),
					PortraitIndex = GameInfo.Technologies[tech].PortraitIndex,
					IconAtlas = GameInfo.Technologies[tech].IconAtlas,
					ID = tech
				})
			end
		end
		table.insert(dataTable, thedata)
		break;
	end
	if priority == "CHOOSE BELIEF" and #gData.Players[gPlayer].BeliefChooseDecisions > 0 then
		currentID = #dataTable + 1;
	end
	for i, beliefoptions in pairs(gData.Players[gPlayer].BeliefChooseDecisions) do
		-- this will be needed to recode one day probably as well? Not truely working if two at one moment occurs? Just make sure it will not happen :)
		local thedata = {BonusType = "ChooseBelief", TitleIconIndex =  16, TitleIconAtlas = "EXPANSION_UNIT_ATLAS_1", TitleText = "Choose new belief...", UniqueOptions = {}}
		for j, belief in pairs(beliefoptions) do
			table.insert(thedata.UniqueOptions, {
				Name = Locale.Lookup(GameInfo.Beliefs[belief].ShortDescription),
				Help = "[NEWLINE]" .. Locale.Lookup(GameInfo.Beliefs[belief].Description),
				ID = belief
			})
		end
		table.insert(dataTable, thedata)
		break;
	end
	isVisible = true;
	OnPopupMessage()
end
LuaEvents.ITG_GeneralChooseWithPriority.Add(TriggeredWithPriority)

function ForcedChoose()
	dataTable = {}
	currentID = 1;
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	for it, extradata in pairs(gData) do
		table.insert(dataTable, extradata)
	end
	isVisible = true;
	OnPopupMessage()
end
LuaEvents.ITG_CallForceGeneralChoose.Add(ForcedChoose)

function InputHandler( uiMsg, wParam, lParam )
    if uiMsg == KeyEvents.KeyDown then
        if wParam == Keys.VK_ESCAPE or wParam == Keys.VK_RETURN then
            OnRealClose();
            return true;
        end
    end
end
ContextPtr:SetInputHandler( InputHandler );

function AddITGTechButton( tech, thisTechButtonInstance)
	if thisTechButtonInstance then
	
		-- update the name of this instance
		local techName = Locale.ConvertTextKey( tech.Description );
		
		techName = Locale.TruncateString(techName, maxTechNameLength, true);
		thisTechButtonInstance.AlreadyResearchedTechName:SetText( techName );
		thisTechButtonInstance.CurrentlyResearchingTechName:SetText( techName );
		thisTechButtonInstance.AvailableTechName:SetText( techName );
		thisTechButtonInstance.UnavailableTechName:SetText( techName );
		thisTechButtonInstance.LockedTechName:SetText( techName );
		thisTechButtonInstance.FreeTechName:SetText( techName );
		
		thisTechButtonInstance.TechButton:SetToolTipString( GetHelpTextForTech(tech.ID) );
		
		-- update the picture
		if IconHookup( tech.PortraitIndex, 64, tech.IconAtlas, thisTechButtonInstance.TechPortrait ) then
			thisTechButtonInstance.TechPortrait:SetHide( false );
		else
			thisTechButtonInstance.TechPortrait:SetHide( true );
		end
		
		-- add the small pictures and their tooltips
		AddSmallButtonsToTechButton( thisTechButtonInstance, tech, maxSmallButtons, 45 );
		
	end
end