local govCausesT;
local govGrudgesT;
local CauseTypeStrings = {
[0] = {"Stackable", "A cause which importance is based on amount of certain actions/events that happened."},
[1] = {"Permanent", "Once active, this cause will be active for entire scenario."},
[2] = {"Situational", "A cause which is active if you meet criteria (tested every turn)."}, 
}

function ShatDreamGrudge(govID)
	if LastChance(50) then
		if gT.Globals.Governors[govID].Player ~= -1 then
			AddNewGovDream(govID, 7, gT.Globals.Governors[govID].Player, false)
		end
	end
end

govGrudgesT = {
{ID = 1, Name = "Family feud", Help = "Your ancestor have misdone the family of Governor.", Val = 10},
{ID = 2, Name = "Right to throne", Help = "Governor was bypassed when leadership was given to you.", Val = 20},
{ID = 3, Name = "Repressed", Help = "After reassignment less Cities ended under influence of Governor than before.", Val = 15},
{ID = 4, Name = "Unappreciated", Help = "You refused Governor's proposition which aimed in improving situation of state.", Val = 10},
{ID = 5, Name = "Publicly humiliated", Help = "You have refused to assign Governor to the City.", Val = 50},
{ID = 6, Name = "Shattered Dream", Help = "You have rejected an initiative that was very important to Governor.", Val = 50, OnAdded = ShatDreamGrudge},
{ID = 7, Name = "Undeserved Punishment", Help = "Governor believes that received punishment was unjust.", Val = 35},
{ID = 8, Name = "Rejection-Personal", Help = "You refused a proposition that was personal to Governor.", Val = 50},
{ID = 9, Name = "Omitted", Help = "You nominated/favoured less important/powerful Governor.", Val = 25},
}

function GenerateNiceGrudgeTooltip(id, bChanceTT)
	local str =  "[COLOR_YELLOW]" .. Locale.Lookup(govGrudgesT[id].Name) .. "[ENDCOLOR][NEWLINE]" .. Locale.Lookup(govGrudgesT[id].Help)
	if bChanceTT then
		str = str .. "[NEWLINE][NEWLINE]Chance to generate grudge is affected by Governor's character (pettiness factor)."
	end
	return str;
end

function GetGrudgeData(id, string)
	return govGrudgesT[id][string]
end

function GenerateChancedGrudge(govID, type, chance)
	if LastChance(chance * (96 + gT.Globals.Governors[govID].Character.Pettiness) / 100) then
		AddGrudge(govID, type)
		return true;
	end
	return false;
end

function AddGrudge(govID, gtype)
	gT.Globals.Governors[govID].Grudges[gtype] = (gT.Globals.Governors[govID].Grudges[gtype] or 0) + 1;
	if govGrudgesT[gtype].OnAdded then
		govGrudgesT[gtype].OnAdded(govID)
	end
end

function GetGrudgeBasedOnMorePowerfulGovs(iPlayer, govID)
	local hTT = ""
	for sGovID, bool in pairs(gT.Players[iPlayer].Governors) do
		if (sGovID ~= govID) and (gT.Globals.Governors[sGovID].Power > gT.Globals.Governors[govID].Power) then
			if hTT == "" then
				hTT = "[NEWLINE][ICON_ITG_GOVERNOR] " .. Locale.Lookup(gT.Globals.Governors[sGovID].Name)
			else
				hTT = ", [NEWLINE][ICON_ITG_GOVERNOR] " .. Locale.Lookup(gT.Globals.Governors[sGovID].Name)
			end
		end
	end
	if hTT == "" then
		return "Affects no [ICON_ITG_GOVERNOR] Governor."
	else
		return "This can affect:" .. hTT .. ".";
	end
end

function DoAssignInitialGrudges(govID, g, bInit)
	if LastRandom(1,12) == 3 then -- family grudge initial
		AddGrudge(govID, 1)
	end
	if bInit and (LastRandom(1,12) == 5) then
		AddGrudge(govID, 2)
	end
end

function GetGrudgesData(govID)
	local tab = {}
	for grg, ttg in pairs(gT.Globals.Governors[govID].Grudges) do
		table.insert(tab, {govGrudgesT[grg].Name, govGrudgesT[grg].Help})
	end
	return tab;
end

function GovCauseOntTurn1(iPlayer)
	if gT.Players[iPlayer].UniqueNumConquered > 3 then
		if GetITGGameTurn() > 20 + gT.Players[iPlayer].LastTurnCityCaptured then
			return true;
		end
	end
	return nil
end
function GovCauseOntTurn5(iPlayer)
	if gT.Players[iPlayer].MeritocracyLevel < 10 then
		gT.Players[iPlayer].MerZeroT = (gT.Players[iPlayer].MerZeroT or 0) + 1;
		if gT.Players[iPlayer].MerZeroT >= 10 then
			return true;
		end
	end
	return nil;
end
function GovCauseOntTurn6(iPlayer)
	if Players[iPlayer]:GetGold() < 50 then
		if gT.Players[iPlayer].PrevTurnGPT < 0 then
			return true;
		end
	end
	return nil
end
function GovCauseOntTurn7(iPlayer)
	if gT.Players[iPlayer].LegalPower < 0 then
		return true;
	end
	return nil;
end
function GovCauseOntTurn10(iPlayer)
	local iTurn = GetITGGameTurn() - 10;
	for a, b in pairs(gT.Heresies) do
		if (b.Active) and (b.Start <= iTurn) then
			if Map.GetPlotByIndex(b.City):GetOwner() == iPlayer then
				return true;
			end
		end
	end
	return nil;
end
function GovCauseOntTurn11(iPlayer)
	if gT.Players[iPlayer].Morale < 0 then
		if Players[iPlayer]:GetUnitProductionMaintenanceMod() ~= 0 then
			return true;
		end
	end
	return nil;
end
function GovCauseOntTurn18(iPlayer)
	local ibehind = GetPrinciplesBehind(iPlayer)
	if ibehind > 0 then
		return ibehind;
	end
	return nil;
end
function GovCauseOntTurn19(iPlayer)
	local pPow = GetPlayerGovernorPower(iPlayer);
	for govID, h in pairs(gT.Players[iPlayer].Governors) do
		if gT.Globals.Governors[govID].Power > pPow then
			return true;
		end
	end
	return nil;
end
--types
--0 stackable
--1 permanent
--2 situational
govCausesT = {
{ID = 1, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_1", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_1", Level = 2, Type = 2, OnTurnFunc = GovCauseOntTurn1, Val = {[-1] = 10, [1] = 60}, Anarchy = true},
{ID = 2, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_2", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_2", Level = 2, Type = 1, Val = {[-1] = 25}},
{ID = 3, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_3", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_3", Level = 2, Type = 1, Val = {[-1] = 50}},
{ID = 4, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_4", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_4", Level = 1, Type = 0, Val = {[-1] = 5}},
{ID = 5, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_5", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_5", Level = 2, Type = 2, OnTurnFunc = GovCauseOntTurn5, Val = {[-1] = 50, [2] = 60, [10] = 80}, Anarchy = true},
{ID = 6, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_6", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_6", Level = 1, Type = 2, OnTurnFunc = GovCauseOntTurn6, Val = {[-1] = 30, [6] = 60}, Anarchy = true},
{ID = 7, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_7", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_7", Level = 2, Type = 2, OnTurnFunc = GovCauseOntTurn7, Val = {[-1] = 50}, Anarchy = true},
{ID = 8, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_8", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_8", Level = 1, Type = 0, Val = {[-1] = 5, [3] = 10}},
{ID = 9, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_9", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_9", Level = 1, Type = 0, Val = {[-1] = 5, [10] = 10}},
{ID = 10, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_10", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_10", Level = 2, Type = 2, OnTurnFunc = GovCauseOntTurn10, Val = {[-1] = 20, [9] = 35}, Anarchy = true},
{ID = 11, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_11", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_11", Level = 1, Type = 2, OnTurnFunc = GovCauseOntTurn11, Val = {[-1] = 12, [1] = 30}},
{ID = 12, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_12", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_12", Level = 1, Type = 0, Val = {[-1] = 5, [1] = 10}},
{ID = 13, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_13", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_13", Level = 2, Type = 0, Val = {[-1] = 15}, Anarchy = true},
{ID = 14, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_14", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_14", Level = 1, Type = 0, Val = {[-1] = 8}},
{ID = 15, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_15", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_15", Level = 2, Type = 1, Val = {[-1] = 50}, Anarchy = true},
{ID = 16, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_16", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_16", Level = 1, Type = 1, Val = {[-1] = 3}},
{ID = 17, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_17", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_17", Level = 1, Type = 1, Val = {[-1] = 3, [10] = 20}},
{ID = 18, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_18", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_18", Level = 1, Type = 0, OnTurnFunc = GovCauseOntTurn18, Val = {[-1] = 4}},
{ID = 19, Name = "TXT_KEY_GOVERNOR_CAUSE_NAME_19", Help = "TXT_KEY_GOVERNOR_CAUSE_HELP_19", Level = 2, Type = 2, OnTurnFunc = GovCauseOntTurn19, Val = {[-1] = 15}},
}

function GetColorCause(iLevel)
	if iLevel == 1 then
		return "[COLOR_YELLOW]"
	elseif iLevel == 2 then
		return "[COLOR_NEGATIVE_TEXT]"
	end
end

function GenerateNiceCauseTooltip(id)
	return GetColorCause(govCausesT[id].Level) .. Locale.Lookup(govCausesT[id].Name) .. "[ENDCOLOR] (" .. CauseTypeStrings[govCausesT[id].Type][1] .. ")[NEWLINE]" .. Locale.Lookup(govCausesT[id].Help)
end

function GetGovCauseData(id, string)
	return govCausesT[id][string]
end

function DoUpdateCausesTurn(iPlayer)
	for i, gkh in pairs(govCausesT) do
		if gkh.OnTurnFunc then
			gT.Players[iPlayer].GovCauses[i] = gkh.OnTurnFunc(iPlayer);
		end
	end
end

function NoteGovCausesAnarchyTurns(iPlayer)
	gT.Players[iPlayer].GovCauses[2] = (gT.Players[iPlayer].EEFTurnsOfAnarchy > 9)
end

function NoteGovCausesStepForward(iPlayer)
	gT.Players[iPlayer].GovCauses[3] = (gT.Players[iPlayer].BackwardTechHelp > 1)
end

function NoteGovCausesBasicStack(iPlayer, id)
	if govCausesT[id].Type == 0 then	
		gT.Players[iPlayer].GovCauses[id] = (gT.Players[iPlayer].GovCauses[id] or 0) + 1;
	else
		gT.Players[iPlayer].GovCauses[id] = true;
	end
end

LuaEvents.ITG_RequestGovCauses.Add(function(iPlayer)
	local tab = {}
	for i, v in pairs(govCausesT) do
		table.insert(tab, {Name = v.Name, Help = v.Help, Level = v.Level, Type = v.Type, On = gT.Players[iPlayer].GovCauses[i]})
	end
	MapModData.ITGSG = tab;
end)

function GeneratePuppetRulerStackChance(iPlayer)
	local chance = 5;
	for govID, sth in pairs(gT.Players[iPlayer].ForeignGovernors) do
		chance = chance + 2;
	end
	return chance;
end

function GovEvalOfSituationInState(govID) -- percentage
	local iPlayer = gT.Globals.Governors[govID].Player;
	local pos, tot = 10, 10;
	local function Add(val)
		pos = pos + val;
		tot = tot + val;
	end
	for grudge, iAmount in pairs(gT.Globals.Governors[govID].Grudges) do
		tot = tot + govGrudgesT[grudge].Val * iAmount;
	end
	local gType = gT.Globals.Governors[govID].Type
	Add(gT.Globals.Governors[govID].Power)
	Add(GetNumCitiesUnderGovernor(iPlayer, govID) * 10)
	for ag,h in pairs(gT.Players[iPlayer].Governors) do
		if (ag ~= govID) and (gT.Globals.Governors[ag].Power > gT.Globals.Governors[govID].Power) then
			tot = tot + 5;
		end
	end
	for cause, h in pairs(gT.Players[iPlayer].GovCauses) do
		if govCausesT[cause].Type == 0 then
			tot = tot + h * (govCausesT[cause].Val[gType] or govCausesT[cause].Val[-1])  
		elseif h then
			tot = tot + (govCausesT[cause].Val[gType] or govCausesT[cause].Val[-1])  
		end
	end
	if gT.Players[iPlayer].HasIdeology[59] then
		if gT.Players[iPlayer].AuthorityRank == 1 then
			Add(25)
		end
	end
	return math.floor(pos * 100 / tot);
end

function GovEvalOfHisPositionInState(govID, iPlayer)
	local pp = GetPlayerGovernorPower(iPlayer);
	if pp >= gT.Globals.Governors[govID].Power then
		return 0;
	end
	local basePercent = math.floor(100 * (gT.Globals.Governors[govID].Power - pp) / pp);
	for othergov, h in pairs(gT.Players[iPlayer].Governors) do
		if gT.Globals.Governors[othergov].Power > gT.Globals.Governors[govID].Power then
			basePercent = basePercent - 25;
		end
	end
	return math.min(100, math.max(basePercent, 0))
end

function IsThereEnoughCausesForRevolution(iPlayer, tres)
	local c = 0;
	for a, b in pairs(govCausesT) do
		if b.Anarchy and gT.Players[iPlayer].GovCauses[a] then
			c = c + 1;
		end
	end
	return (c >= tres)
end