local l_govCityBonuses;
local l_govCareerTypes;

function InitGovRevDream(govID, iPlayer, bInit)
	local iCurrentGov = gT.Players[iPlayer].Goverment;
	if iCurrentGov == 1 then
		iCurrentGov = gT.Players[iPlayer].GovermentBeforeAnarchy;
	end
	if (gT.Globals.Governors[govID].Goverment) and (gT.Globals.Governors[govID].Goverment ~= iCurrentGov) then
		return;
	end
	local tab = {};
	for i = 2, 7 do
		if iCurrentGov ~= i then
			table.insert(tab, i);
		end
	end
	gT.Globals.Governors[govID].Goverment = RandomFromTab(tab);
end

local gDreamTypes = {
{ID = 1, Name = "Do nothing special at all", Chance = 10},
{ID = 2, Name = "Become king of city-state", Chance = 50, CoreAction = 27,
ActionChance = {[1] = 5, [4] = 10}},
{ID = 3, Name = "Make revolution", Chance = 30, Init = InitGovRevDream, CoreAction = 40},
{ID = 4, Name = "Get important position in country", Chance = 70, CoreAction = 38,
ActionChance = {[3] = 5}},
{ID = 5, Name = "Get more power than leader/other govs then rest", Chance = 50,
ActionChance = {[3] = 5}},
{ID = 6, Name = "Declare War on any opponent", Chance = 30, CoreAction = 28,
ActionChance = {[23] = 5, [24] = 5, [51] = 5}},
{ID = 7, Name = "Plot with foreigners", Chance = 30, CoreAction = 33,
ActionChance = {[1] = 20, [8] = 3}},
}

function GetGovActionChanceFromDream(govID, actionID)
	if gT.Globals.Governors[govID].Dream then
		if gT.Globals.Governors[govID].Dream.Active then
			if gDreamTypes[gT.Globals.Governors[govID].Dream.ID].ActionChance then
				return gDreamTypes[gT.Globals.Governors[govID].Dream.ID].ActionChance[actionID] or 0;
			end
		end
	end
	return 0;
end

local gLoyaltyTypes = {
{ID = 1, Name = "Loyal to the State", Help = "Less likely to look towards foreign power.",
DreamChance = {[7] = -20, [3] = -20}},
{ID = 2, Name = "Unquestionable", Help = "Loyal to leader he serves.",
DreamChance = {[7] = -50, [3] = -50}},
{ID = 3, Name = "To self", Help = "Doing what he thinks is the best for himself.",
DreamChance = {}},
{ID = 4, Name = "Lazy", Help = "Not truely interested in anything particular.",
DreamChance = {[1] = 50}},
{ID = 5, Name = "Stabby Stabby", Help = "Betrayal for the sake of betrayal.",
DreamChance = {[7] = 50, [3] = 25}},
{ID = 6, Name = "Easily offended", Help = "Causes and grudges more effective.",
DreamChance = {}},
{ID = 7, Name = "Based on benefits", Help = "More powerful, more loyal.", 
DreamChance = {[5] = 30, [1] = 10}},
{ID = 8, Name = "Random", Help = "Ultra random decisions.",
DreamChance = {[1] = 100, [2] = 100, [3] = 100, [4] = 100, [5] = 100, [6] = 100, [7] = 100}},
{ID = 9, Name = "Non-random", Help = "Non-random decisions.",
DreamChance = {[1] = -9, [2] = 50, [4] = 130, [5] = 50}},
{ID = 10, Name = "Follower", Help = "Based on behaviour of other governors (if any).",
DreamChance = {}},
}

function GetDreamChanceFromLoyaltyType(lType, iDream)
	if gLoyaltyTypes[lType].DreamChance and gLoyaltyTypes[lType].DreamChance[iDream] then
		return gLoyaltyTypes[lType].DreamChance[iDream];
	end
	return 0;
end

function GovCareerTypeBE1(iPlayer, govID, govPower, bString)
	local moraleDrop = math.min(15, math.max(1, math.floor(govPower / 25)))
	if bString then
		return {"[ICON_ITG_MORALE] Morale drops by [COLOR_NEGATIVE_TEXT]" .. moraleDrop .. "[ENDCOLOR].", ""}
	else
		TakeCareOfEventMorale(iPlayer, 25, {-moraleDrop, "Diplo Action"})
		return true;
	end
end
function GovCareerTypeBE2(iPlayer, govID, govPower, bString)
	local xploss = math.min(math.max(5, math.floor(govPower / 10)), 50)
	if bString then
		return {"Agents stop their missions and loss " .. xploss .. "% of XP.", ""}
	else
		local bAny = false;
		for iAgent, data in pairs(gT.Players[iPlayer].Agents) do
			if (not data.IgnoreItForever) then
				if not data.Rekonwalescencja then
					MakeSpyUnassigned(iPlayer, iAgent, true)
				end
				ChangeSpyExperience(iPlayer, iAgent, -1 * math.floor(gT.Players[iPlayer].Agents[iAgent].XP * xploss / 100))
				bAny = true;
			end
		end
		return bAny;
	end
end
function GovCareerTypeBE3(iPlayer, govID, govPower, bString)
	local chance = math.min(20, math.max(1, math.floor(govPower / 10)))
	if bString then
		return {"Other [ICON_ITG_GOVERNOR] Governors have " .. chance .. "% to abandon you.", ""}
	else
		local bAny = false;
		for gID, h in pairs(gT.Players[iPlayer].Governors) do
			if gID ~= govID then
				if LastChance(chance) then
					MakeGovernorChangePlayer(gID, -1)
					bAny = true;
				end
			end
		end
		return bAny;
	end
end
function GovCareerTypeBE4(iPlayer, govID, govPower, bString)
	local chance = math.max(5, math.min(50, math.floor(govID / 6)));
	if bString then
		return {"Cities loose " .. chance .. "% of [ICON_WORKER] Work Rate and have " .. chance .. "% chance for adjacent improvements to be pillaged.", ""}
	else
		for iCity in Players[iPlayer]:Cities() do
			local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
			gT.Cities[cID].WorkRate = math.floor(gT.Cities[cID].WorkRate * (100 - chance) / 100)
			for i = 0, 5 do
				local nPlot = Map.PlotDirection(iCity:GetX(), iCity:GetY(), i);
				if nPlot and (nPlot:GetOwner() == iPlayer) and (nPlot:GetImprovementType() ~= -1) and (not nPlot:IsImprovementPillaged()) then
					if LastChance(chance) then
						nPlot:SetImprovementPillaged(true)
					end
				end
			end
		end
		return (Players[iPlayer]:GetNumCities() > 0)
	end
end
function GovCareerTypeBE5(iPlayer, govID, govPower, bString)
	if bString then
		return {"[ICON_ITG_FESTIVAL] Festivals end in all Cities. Temporary chances for new [ICON_ITG_FESTIVAL] Festival are reset.",""}
	else
		ResetFestivalTemporaryChance(iPlayer)
		local bAny = false;
		for iCity in Players[iPlayer]:Cities() do
			local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
			if gT.Cities[cID].FestivalET then
				EndImmediatelyFestival(iCity, cID, iPlayer)
				bAny = true;
			end
		end
		return bAny;
	end
end
function GovCareerTypeBE6(iPlayer, govID, govPower, bString)
	local percent = math.max(10, math.min(50, math.floor(govPower / 5)))
	if bString then
		return {"All current [ICON_GOLD] Gold from Trade-Routes is lost. Cities loss [COLOR_NEGATIVE_TEXT]" .. percent .. "%[ENDCOLOR] of stored [ICON_CARAVAN] Land and [ICON_CARGO] Naval Trade Routes' points.", ""}
	else
		TradeRouteGoldDead(iPlayer)
		for iCity in Players[iPlayer]:Cities() do
			local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
			gT.Cities[cID].CaravanProgress = math.floor((gT.Cities[cID].CaravanProgress or 0) * (100 - percent) / 100)
			gT.Cities[cID].CargoProgress = math.floor((gT.Cities[cID].CargoProgress or 0) * (100 - percent) / 100)
		end
		return true;
	end
end
function GovCareerTypeBE7(iPlayer, govID, govPower, bString)
	local percent = math.max(5, math.min(25, math.floor(govPower / 10)))
	if bString then
		return {"[ICON_INFLUENCE] Influence drops with all City-States by [COLOR_NEGATIVE_TEXT]" .. percent .. "%[ENDCOLOR].", ""}
	else
		local bAny = false;
		for g, h in pairs(gT.Players) do
			if Players[g]:IsMinorCiv() then
				local inf = math.floor(Players[g]:GetMinorCivFriendshipWithMajor(iPlayer) * percent / 100)
				if inf > 0 then
					Players[g]:ChangeMinorCivFriendshipWithMajor(iPlayer, -inf);
					bAny = true;
				end
			end
		end
		return bAny;
	end
end
function GovCareerTypeBE8(iPlayer, govID, govPower, bString)
	local chance = math.max(1, math.min(10, math.floor(govPower / 10)));
	if bString then
		return {"Trigger a [ICON_RESISTANCE] demolition chance for buildings in all Cities (+" .. chance .. "%).", ""}
	else
		local bDestroyed = 0;
		for iCity in Players[iPlayer]:Cities() do
			bDestroyed = bDestroyed + #DoTriggerDemolitionOfBuildings(iPlayer, iCity, chance)
		end
		return (bDestroyed > 0)
	end
end
function GovCareerTypeBE9(iPlayer, govID, govPower, bString)
	local percent = math.max(5, math.min(20, math.floor(govPower / 12)))
	if bString then
		return {"[ICON_ITG_PRESENCE] Cultural Presence drops in all your Cities by [COLOR_NEGATIVE_TEXT]" .. percent .. "%[ENDCOLOR].", ""}
	else
		for iCity in Players[iPlayer]:Cities() do
			local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
			if gT.Cities[cID].CultureStored[iPlayer] then
				local drop = math.floor(gT.Cities[cID].CultureStored[iPlayer] * percent / 100)
				if drop > 0 then
					DoChangeCityCultureOwnership(cID, iCity, iPlayer, -drop, "GovCareer")
				end
			end
		end
		return (Players[iPlayer]:GetNumCities() > 0)
	end
end
function GovCareerTypeBE10(iPlayer, govID, govPower, bString)
	local gpTreshold = math.min(150, math.max(25, math.floor(govPower / 2)))
	if bString then
		return {"Increases treshold to acquire [ICON_GREAT_PEOPLE] Great People by [COLOR_NEGATIVE_TEXT]" .. gpTreshold .. "[ENDCOLOR].", ""}
	else
		gT.Players[iPlayer].GPTresholdExtra = (gT.Players[iPlayer].GPTresholdExtra or 0) + gpTreshold;
		return true;
	end
end
function GovCareerTypeBE11(iPlayer, govID, govPower, bString)
	local bChange = math.min(3, math.max(1, math.floor(govPower / 42)))
	if bString then
		return {"Reduces your range for contesting governship by [COLOR_NEGATIVE_TEXT]" .. bChange .. "[ENDCOLOR].", ""}
	else
		gT.Players[iPlayer].GovernorshipRange = gT.Players[iPlayer].GovernorshipRange - bChange;
		return true;
	end
end

l_govCareerTypes = {
{ID = 1, Icon = "[ICON_WAR]", Name = "Soldier", Help = "TXT_KEY_GOVERNOR_CAREER_HELP_1",
ExtraActionChance = {[23] = 10, [24] = 10, [28] = 10, [51] = 10},
DreamChance = {[6] = 25},
BuildingsC = {GameInfoTypes.BUILDINGCLASS_WALLS,GameInfoTypes.BUILDINGCLASS_BARRACKS,GameInfoTypes.BUILDINGCLASS_FORGE,GameInfoTypes.BUILDINGCLASS_ARMORY,GameInfoTypes.BUILDINGCLASS_CASTLE},
BadEffectFunc = GovCareerTypeBE1, BadEffectLogString = "TXT_KEY_GOV_CAREER_LOG_1"},
{ID = 2, Icon = "[ICON_SPY]", Name = "Schemer", Help = "TXT_KEY_GOVERNOR_CAREER_HELP_2",
ExtraActionChance = {[5] = 30, [49] = 10},
DreamChance = {[7] = 10},
BuildingsC = {GameInfoTypes.BUILDINGCLASS_COURTHOUSE,GameInfoTypes.BUILDINGCLASS_MARKET,GameInfoTypes.BUILDINGCLASS_CONSTABLE,GameInfoTypes.BUILDINGCLASS_SHRINE,GameInfoTypes.BUILDINGCLASS_THEATRE,GameInfoTypes.BUILDINGCLASS_LIBRARY,GameInfoTypes.BUILDINGCLASS_GARDEN},
BadEffectFunc = GovCareerTypeBE2, BadEffectLogString = "TXT_KEY_GOV_CAREER_LOG_2"},
{ID = 3, Icon = "[ICON_ITG_POWER]", Name = "Politician", Help = "TXT_KEY_GOVERNOR_CAREER_HELP_3",
ExtraActionChance = {[44] = 5, [27] = 10, [49] = 10},
DreamChance = {[2] = 10, [3] = 10, [4] = 10, [5] = 10, [6] = 5, [7] = 5},
BuildingsC = {GameInfoTypes.BUILDINGCLASS_COURTHOUSE,GameInfoTypes.BUILDINGCLASS_STOCK_EXCHANGE,GameInfoTypes.BUILDINGCLASS_CONSTABLE,GameInfoTypes.BUILDINGCLASS_SHRINE,GameInfoTypes.BUILDINGCLASS_TEMPLE,GameInfoTypes.BUILDINGCLASS_COLOSSEUM,GameInfoTypes.BUILDINGCLASS_GARDEN,GameInfoTypes.BUILDINGCLASS_CIRCUS},
BadEffectFunc = GovCareerTypeBE3, BadEffectLogString = "TXT_KEY_GOV_CAREER_LOG_3"},
{ID = 4, Icon = "[ICON_CITIZEN]", Name = "Landlord", Help = "TXT_KEY_GOVERNOR_CAREER_HELP_4",
ExtraActionChance = {[7] = 5, [22] = 5, [31] = 5},
DreamChance = {[4] = 10},
BuildingsC = {GameInfoTypes.BUILDINGCLASS_AQUEDUCT,GameInfoTypes.BUILDINGCLASS_GRANARY,GameInfoTypes.BUILDINGCLASS_CARAVANSARY,GameInfoTypes.BUILDINGCLASS_WALLS,GameInfoTypes.BUILDINGCLASS_LIBRARY,GameInfoTypes.BUILDINGCLASS_WATERMILL,GameInfoTypes.BUILDINGCLASS_WINDMILL,GameInfoTypes.BUILDINGCLASS_CASTLE},
BadEffectFunc = GovCareerTypeBE4, BadEffectLogString = "TXT_KEY_GOV_CAREER_LOG_4"},
{ID = 5, Icon = "[ICON_OMEGA]", Name = "Opportunist", Help = "TXT_KEY_GOVERNOR_CAREER_HELP_5",
ExtraActionChance = {[27] = 10},
DreamChance = {[3] = 5, [7] = 5},
BuildingsC = {GameInfoTypes.BUILDINGCLASS_UNIVERSITY,GameInfoTypes.BUILDINGCLASS_MONUMENT,GameInfoTypes.BUILDINGCLASS_COLOSSEUM,GameInfoTypes.BUILDINGCLASS_GARDEN},
BadEffectFunc = GovCareerTypeBE5, BadEffectLogString = "TXT_KEY_GOV_CAREER_LOG_5"},
{ID = 6, Icon = "[ICON_INVEST]", Name = "Financist", Help = "TXT_KEY_GOVERNOR_CAREER_HELP_6",
ExtraActionChance = {[2] = 25, [35] = 5},
DreamChance = {[5] = 15, [4] = 10},
BuildingsC = {GameInfoTypes.BUILDINGCLASS_STOCK_EXCHANGE,GameInfoTypes.BUILDINGCLASS_BANK,GameInfoTypes.BUILDINGCLASS_MARKET,GameInfoTypes.BUILDINGCLASS_CARAVANSARY,GameInfoTypes.BUILDINGCLASS_THEATRE,GameInfoTypes.BUILDINGCLASS_LIGHTHOUSE},
BadEffectFunc = GovCareerTypeBE6, BadEffectLogString = "TXT_KEY_GOV_CAREER_LOG_6"},
{ID = 7, Icon = "[ICON_ITG_VOTES]", Name = "Diplomat", Help = "TXT_KEY_GOVERNOR_CAREER_HELP_7",
ExtraActionChance = {[4] = 8, [27] = 10, [8] = 10, [49] = 10},
DreamChance = {[2] = 25, [6] = 5, [7] = 10},
BuildingsC = {GameInfoTypes.BUILDINGCLASS_STOCK_EXCHANGE,GameInfoTypes.BUILDINGCLASS_BANK,GameInfoTypes.BUILDINGCLASS_CONSTABLE,GameInfoTypes.BUILDINGCLASS_OBSERVATORY,GameInfoTypes.BUILDINGCLASS_LIGHTHOUSE},
BadEffectFunc = GovCareerTypeBE7, BadEffectLogString = "TXT_KEY_GOV_CAREER_LOG_7"},
{ID = 8, Icon = "[ICON_PRODUCTION]", Name = "Industrialist", Help = "TXT_KEY_GOVERNOR_CAREER_HELP_8",
ExtraActionChance = {[7] = 5, [31] = 5, [16] = 5, [18] = 4},
BuildingsC = {GameInfoTypes.BUILDINGCLASS_FACTORY,GameInfoTypes.BUILDINGCLASS_WORKSHOP,GameInfoTypes.BUILDINGCLASS_CARAVANSARY,GameInfoTypes.BUILDINGCLASS_STONE_WORKS,GameInfoTypes.BUILDINGCLASS_FORGE,GameInfoTypes.BUILDINGCLASS_LIGHTHOUSE,GameInfoTypes.BUILDINGCLASS_WATERMILL,GameInfoTypes.BUILDINGCLASS_WINDMILL},
BadEffectFunc = GovCareerTypeBE8, BadEffectLogString = "TXT_KEY_GOV_CAREER_LOG_8"},
{ID = 9, Icon = "[ICON_PROPHET]", Name = "Fanatic", Help = "TXT_KEY_GOVERNOR_CAREER_HELP_9",
ExtraActionChance = {[6] = 35, [15] = 30, [26] = 10, [19] = 10},
DreamChance = {[3] = 10, [5] = 10},
BuildingsC = {GameInfoTypes.BUILDINGCLASS_MONUMENT,GameInfoTypes.BUILDINGCLASS_SHRINE,GameInfoTypes.BUILDINGCLASS_TEMPLE},
BadEffectFunc = GovCareerTypeBE9, BadEffectLogString = "TXT_KEY_GOV_CAREER_LOG_9"},
{ID = 10, Icon = "[ICON_ITG_MERITOCRACY]", Name = "Saviour", Help = "TXT_KEY_GOVERNOR_CAREER_HELP_10",
ExtraActionChance = {[3] = 50, [51] = 8},
DreamChance = {[3] = 20, [4] = 20},
BuildingsC = {GameInfoTypes.BUILDINGCLASS_GRANARY,GameInfoTypes.BUILDINGCLASS_MONUMENT,GameInfoTypes.BUILDINGCLASS_WALLS,GameInfoTypes.BUILDINGCLASS_GARDEN},
BadEffectFunc = GovCareerTypeBE10, BadEffectLogString = "TXT_KEY_GOV_CAREER_LOG_10"},
{ID = 11, Icon = "[ICON_MOVES]", Name = "Expansionist", Help = "TXT_KEY_GOVERNOR_CAREER_HELP_11",
ExtraActionChance = {[2] = 7, [7] = 3, [4] = 5, [6] = 5, [8] = 5, [31] = 5, [48] = 5, [49] = 5},
DreamChance = {[1] = -5, [2] = 5, [4] = 10, [7] = 5},
BuildingsC = {GameInfoTypes.BUILDINGCLASS_CARAVANSARY,GameInfoTypes.BUILDINGCLASS_LIGHTHOUSE,GameInfoTypes.BUILDINGCLASS_HARBOR,GameInfoTypes.BUILDINGCLASS_STABLE},
BadEffectFunc = GovCareerTypeBE11, BadEffectLogString = "TXT_KEY_GOV_CAREER_LOG_11"},
}

function GetDreamChanceFromCareerType(gType, dreamID)
	if l_govCareerTypes[gType].DreamChance and l_govCareerTypes[gType].DreamChance[dreamID] then
		return l_govCareerTypes[gType].DreamChance[dreamID];
	end
	return 0;
end

function IsAplifiedGovAction(actionID, cType)
	if l_govCareerTypes[cType].ExtraActionChance[actionID] then
		return true;
	end
	return false;
end

function GetGovActionChanceBasedOnCareerType(actionID, cType)
	return l_govCareerTypes[cType].ExtraActionChance[actionID] or 0;
end

function GetGovernorCareerBadEffectLogString(govID)
	local govType = gT.Globals.Governors[govID].Type;
	return Locale.Lookup(l_govCareerTypes[govType].BadEffectLogString);
end

function DoTriggerGovernorTypeBadEffect(iPlayer, govID, govPower, bString)
	local govType = gT.Globals.Governors[govID].Type;
	return l_govCareerTypes[govType].BadEffectFunc(iPlayer, govID, govPower, bString)
end

LuaEvents.ITG_RequestGovernorTypesStrings.Add(function()
	local tab = {};
	for a, b in pairs(l_govCareerTypes) do
		table.insert(tab, b.Icon .. " " .. b.Name);
	end
	MapModData.ITGSG = tab;
end)

function GetStringOfGovType(govID)
	local type = gT.Globals.Governors[govID].Type;
	return {l_govCareerTypes[type].Icon .. " " .. l_govCareerTypes[type].Name, Locale.Lookup(l_govCareerTypes[type].Help)}
end

function GovCityTraitOnAssign9(iPlayer, govID)
	if govID == -1 then
		gT.Players[iPlayer].GovCityReligion = Players[iPlayer]:GetReligionCreatedByPlayer();
	else
		gT.Globals.Governors[govID].CityReligion = Players[iPlayer]:GetReligionCreatedByPlayer();
	end
end

function GovCityTraitHelpFunc9(iPlayer, govID, strHelp)
	local iReligion = 0;
	if govID == -1 then
		iReligion = gT.Players[iPlayer].GovCityReligion
	else
		iReligion = gT.Globals.Governors[govID].CityReligion
	end
	return "+" .. strHelp .. " pressure of " .. GameInfo.Religions[iReligion].IconString .. " " .. Locale.Lookup(GameInfo.Religions[iReligion].Description) .. ".";
end

function GovCityTraitAddRemove9(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	local change = (l_govCityBonuses[selfID].Values[tLevel] or 0) - (l_govCityBonuses[selfID].Values[fLevel] or 0);
	local iReligion = 0;
	if govID == -1 then
		iReligion = gT.Players[iPlayer].GovCityReligion
	else
		iReligion = gT.Globals.Governors[govID].CityReligion
	end
	iCity:ChangeBaseReligionPressure(iReligion, change)
end

function GovCityTraitAddRemove12(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	local iTeam = Players[iPlayer]:GetTeam()
	if fLevel ~= 0 then
		local fValue = l_govCityBonuses[selfID].Values[fLevel];	
		local sTab = GetPlotsInRadiusOfPlot(iCity:Plot(), fValue);
		for h, hPlot in pairs(sTab) do
			hPlot:ChangeVisibilityCount(iTeam, -1)
		end
	end
	if tLevel ~= 0 then
		local tValue = l_govCityBonuses[selfID].Values[tLevel];	
		local sTab = GetPlotsInRadiusOfPlot(iCity:Plot(), tValue);
		for h, hPlot in pairs(sTab) do
			hPlot:ChangeVisibilityCount(iTeam, 1)
		end
	end
end

function GovCityTraitAddRemove15(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	if fLevel ~= 0 then
		if not bConquest then -- conquests clears BaseYieldRateFromBuildings;
			local fValue = l_govCityBonuses[selfID].Values[fLevel];
			iCity:ChangeBaseYieldRateFromBuildings(yFood, -fValue)
		end
	end
	SetITGCityInteger(iCity, "CI_FOOD_GOVERNANCE", 0)
	if tLevel ~= 0 then
		local tValue = l_govCityBonuses[selfID].Values[tLevel];
		iCity:ChangeBaseYieldRateFromBuildings(yFood, tValue)
		SetITGCityInteger(iCity, "CI_FOOD_GOVERNANCE", l_govCityBonuses[selfID].Values[tLevel])
	end
end

function GovCityTraitAddRemove16(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	if fLevel ~= 0 then
		if not bConquest then
			local fValue = l_govCityBonuses[selfID].Values[fLevel];
			iCity:ChangeBaseYieldRateFromBuildings(yGold, -fValue)
		end
	end
	SetITGCityInteger(iCity, "CI_GOVERNANCE_GOLD", 0)
	if tLevel ~= 0 then
		local tValue = l_govCityBonuses[selfID].Values[tLevel];
		iCity:ChangeBaseYieldRateFromBuildings(yGold, tValue)
		SetITGCityInteger(iCity, "CI_GOVERNANCE_GOLD", tValue)
	end
end

function GovCityTraitAddRemove5(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	if fLevel ~= 0 then
		if not bConquest then
			local fValue = l_govCityBonuses[selfID].Values[fLevel];
			iCity:ChangeYieldRateModifier(yProduction, -fValue)
		end
	end
	SetITGCityInteger(iCity, "CI_GOVERNANCE_PROD", 0)
	if tLevel ~= 0 then
		local tValue = l_govCityBonuses[selfID].Values[tLevel];
		iCity:ChangeYieldRateModifier(yProduction, tValue)
		SetITGCityInteger(iCity, "CI_GOVERNANCE_PROD", tValue)
	end
end

function GovCityTraitAddRemove10(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	SetITGCityInteger(iCity, "CI_GOV_CHARM", l_govCityBonuses[selfID].Values[tLevel] or 0)
end
function GovCityTraitAddRemove14(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	if not bConquest then
		local change = (l_govCityBonuses[selfID].Values[tLevel] or 0) - (l_govCityBonuses[selfID].Values[fLevel] or 0);
		iCity:ChangeAttackMod(change);
	end
end
function GovCityTraitAddRemove20(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	SetITGCityInteger(iCity, "CI_GOV_TR_APPEAL", l_govCityBonuses[selfID].Values[tLevel] or 0)
end
function GovCityTraitAddRemove1(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	SetITGCityInteger(iCity, "CI_GOV_PRESENCE", l_govCityBonuses[selfID].Values[tLevel] or 0)
end
function GovCityTraitAddRemove2(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	SetITGCityInteger(iCity, "CI_GOV_HEALTH", l_govCityBonuses[selfID].Values[tLevel] or 0)
end
function GovCityTraitAddRemove3(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	SetITGCityInteger(iCity, "CI_XP_FROM_GOV", l_govCityBonuses[selfID].Values[tLevel] or 0)
end
function GovCityTraitAddRemove4(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	SetITGCityInteger(iCity, "CI_GOV_UNHAP", l_govCityBonuses[selfID].Values[tLevel] or 0)
end
function GovCityTraitAddRemove6(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	SetITGCityInteger(iCity, "CI_GOV_WORK_RATE", l_govCityBonuses[selfID].Values[tLevel] or 0)
end
function GovCityTraitAddRemove7(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	SetITGCityInteger(iCity, "CI_GOV_LAND_TR", l_govCityBonuses[selfID].Values[tLevel] or 0)
end
function GovCityTraitAddRemove5(iCity, cID, iPlayer, selfID, fLevel, tLevel, bConquest, govID)
	if not bConquest then
		local change = (l_govCityBonuses[selfID].Values[tLevel] or 0) - (l_govCityBonuses[selfID].Values[fLevel] or 0);
		AdjustStrengthOfOneCity(iCity, change * 100, true)
	end
end

l_govCityBonuses = {
{ID = 1, Name = "Influencer", OnAddRemove = GovCityTraitAddRemove1, Values = {1,2,3}, },
{ID = 2, Name = "Medician", OnAddRemove = GovCityTraitAddRemove2, Values = {1,2,3}, },
{ID = 3, Name = "Combat Experience", OnAddRemove = GovCityTraitAddRemove3, Values = {5,10,15}, Bias = {[1] = true}},
{ID = 4, Name = "Easy-to-be-around", OnAddRemove = GovCityTraitAddRemove4, Values = {2,4,6}, Bias = {[3] = true}},
{ID = 5, Name = "Industrialist", OnAddRemove = GovCityTraitAddRemove5, Values = {3,5,7}},
{ID = 6, Name = "Landscaper", OnAddRemove = GovCityTraitAddRemove6, Values = {5,10,15}, },
{ID = 7, Name = "Trader", OnAddRemove = GovCityTraitAddRemove7, Values = {1,2,3}, },
{ID = 8, Name = "Defender", Values = {1,2,3}, Bias = {[1] = true}},
{ID = 9, Name = "Priest", Values = {5,10,15}, HelpFunc = GovCityTraitHelpFunc9, Religion = true, OnAddRemove = GovCityTraitAddRemove9, OnAssign = GovCityTraitOnAssign9, Sc = {[1] = true, [2] = true, [3] = true}},
{ID = 10, Name = "Thursday Dinners", OnAddRemove = GovCityTraitAddRemove10, Values = {10,15,20} , Sc = {}},
{ID = 11, Name = "Publican", Values = {20,40,60}, },
{ID = 12, Name = "Eagle Eye", OnAddRemove = GovCityTraitAddRemove12, Values = {4,5,6}, Sc = {[1] = true, [2] = true, [3] = true}, Bias = {[4] = true}},
{ID = 13, Name = "Crowd Control", Values = {2,3,4}, Bias = {[3] = true}},
{ID = 14, Name = "Guardian", OnAddRemove = GovCityTraitAddRemove14, Values = {15,30,45}, Bias = {[1] = true}},
{ID = 15, Name = "Food Distribution", OnAddRemove = GovCityTraitAddRemove15, Values = {1,2,3}, },
{ID = 16, Name = "Entrepreneur", OnAddRemove = GovCityTraitAddRemove16, Values = {1,2,3}, },
{ID = 17, Name = "Archaeologist", Values = {1,2,3}, Sc = {[4] = true}},
{ID = 18, Name = "Patron of the arts", Values = {1,2,3}, },
{ID = 19, Name = "Preacher", Values = {1,2,3}, Sc = {[1] = true, [2] = true, [3] = true}},
{ID = 20, Name = "Trade Attractiveness", OnAddRemove = GovCityTraitAddRemove20, Values = {6,9,12}, },
}

function GetGovernorCityTraitTXT(iPlayer, govID)
	local strhelp = ""
	local gtrait, glevel = GetGovernorCityTraitAndLevel(govID, iPlayer)
	for j, b in pairs(l_govCityBonuses[gtrait].Values) do
		if j ~= 1 then
			strhelp = strhelp .. "/"
		end
		if j == glevel then
			strhelp = strhelp .. "[COLOR_POSITIVE_TEXT]" .. b .. "[ENDCOLOR]"
		else
			strhelp = strhelp .. b
		end
	end
	if l_govCityBonuses[gtrait].HelpFunc then
		return {l_govCityBonuses[gtrait].Name, l_govCityBonuses[gtrait].HelpFunc(iPlayer, govID, strhelp)}
	else
		return {l_govCityBonuses[gtrait].Name, Locale.ConvertTextKey("TXT_KEY_GOVERNOR_CITY_TRAIT_HELP_" .. gtrait, strhelp)}
	end
end

function GetCityGovTraitBonus(iCity, traitID)
	local gID, gLevel = GetCityGovernorTraitAndLevel(iCity)
	if gID == traitID then
		return l_govCityBonuses[gID].Values[gLevel];
	end
	return 0;
end

function ChangeGovCityBonuses(iCity, cID, iPlayer, traitID, fromLevel, toLevel, bConquest, govID)
	if l_govCityBonuses[traitID].OnAddRemove then
		l_govCityBonuses[traitID].OnAddRemove(iCity, cID, iPlayer, traitID, fromLevel, toLevel, bConquest, govID)
	end
end

function PlayerAssignGovernorTrait(iPlayer)
-- player has no longer a city trait
--[[	local tab = {}
	for g, h in pairs(l_govCityBonuses) do
		if (not h.Sc) or h.Sc[gT.Globals.ScenarioType] then
			if (not h.Religion) or (Players[iPlayer]:GetReligionCreatedByPlayer() > 0) then
				table.insert(tab, g)
			end
		end
	end
	local lTrait = tab[LastRandom(1,#tab)];
	if l_govCityBonuses[lTrait].OnAssign then
		l_govCityBonuses[lTrait].OnAssign(iPlayer, -1)
	end
	gT.Players[iPlayer].GovCityTrait = lTrait ]]
end

function GenerateStartingGovernorTraits(iPlayer, govID)
	local tab = {}
	local lCareerType = LastRandom(1,#l_govCareerTypes)
	gT.Globals.Governors[govID].Type = lCareerType;
	gT.Globals.Governors[govID].Character.Decisiveness = LastRandom(1,10);
	gT.Globals.Governors[govID].Character.Pettiness = LastRandom(1,10);
	gT.Globals.Governors[govID].Character.Conflict = LastRandom(1,10);
	gT.Globals.Governors[govID].Character.LoyaltyType = LastRandom(1,#gLoyaltyTypes);
	for g, h in pairs(l_govCityBonuses) do
		if (not h.Sc) or h.Sc[gT.Globals.ScenarioType] then
			if (not h.Religion) or (Players[iPlayer]:GetReligionCreatedByPlayer() > 0) then
				table.insert(tab, g)
				if h.Bias and h.Bias[lCareerType] then
					table.insert(tab, g)
				end
			end
		end
	end
	local lTrait = tab[LastRandom(1,#tab)];
	if l_govCityBonuses[lTrait].OnAssign then
		l_govCityBonuses[lTrait].OnAssign(iPlayer, govID)
	end
	gT.Globals.Governors[govID].CityTrait = lTrait;
end

function CollectCityBonusesEmpireWide(iPlayer, traitID)
	local vals = l_govCityBonuses[traitID].Values;
	local c = 0;
	for iCity in Players[iPlayer]:Cities() do
		local gTrait, gLevel = GetCityGovernorTraitAndLevel(iCity)
		if gTrait == traitID then
			c = c + vals[gLevel]
		end
	end
	return c;
end

function GetCityGovernorTraitAndLevel(iCity)
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
	local govID = gT.Cities[cID].Governor
	if govID and (govID ~= -1) then
		return GetGovernorCityTraitAndLevel(govID, iCity:GetOwner())
	end
	return;
end

function GetGovernorCityTraitAndLevel(govID, iPlayer)
	return gT.Globals.Governors[govID].CityTrait, gT.Globals.Governors[govID].CityLevel
end

function EraConversionRemoveGovernorAbilities()
	for cID, info in pairs(gT.Cities) do
		if info.Governor then
			local iCity = Map.GetPlotByIndex(info.Plot):GetPlotCity();
			RemoveGovernorEffectsFromCity(iCity, cID, iCity:GetOwner())
		end
	end
end

local govPositionTab = {
{ID = 1, Type = "HighPriest", Name = "High Priest", ToolTip = "High Priest chooses randomly your reward from religious bonus. [ICON_ITG_GOVERNOR] Governors with such position are less likely to be removed from office (execution).", Flavor1 = "FLAVOR_RELIGION", Flavor2 = "ReligionRewardTreshold"},
{ID = 2, Type = "CommanderInChief", Name = "Commander-in-Chief", ToolTip = "Commander-in-Chief chooses randomly your reward from general and admiral bonuses. [ICON_ITG_GOVERNOR] Governors with such position are less likely to be removed from office (execution).", Flavor1 = "FLAVOR_OFFENSE", Flavor2 = "NextGeneralTreshold", Scale2 = 0.01},
{ID = 3, Type = "Chancellor", Name = "Chancellor", ToolTip = "Chancellor chooses randomly your reward from scientific bonus. [ICON_ITG_GOVERNOR] Governors with such position are less likely to be removed from office (execution).", Flavor1 = "FLAVOR_SCIENCE", Flavor2 = "ScienceBonusAdopted"},
{ID = 4, Type = "Curator", Name = "Protector", ToolTip = "Protector chooses randomly your reward from cultural bonus. [ICON_ITG_GOVERNOR] Governors with such position are less likely to be removed from office (execution).", Flavor1 = "FLAVOR_CULTURE", Flavor2 = "CultureBonusAdopted"},
}

function GetAIEvalOfGovPosition(iPlayer, sType)
	for id, h in pairs(govPositionTab) do
		if h.Type == sType then
			return {id, GetAIFlavour(iPlayer, h.Flavor1) + math.floor(gT.Players[iPlayer][h.Flavor2] * (h.Scale2 or 1))};
		end
	end
	return {1, 0};
end

function GetDataForGovPositionID(id)
	return {govPositionTab[id].Name, govPositionTab[id].ToolTip, govPositionTab[id].Type}
end

function AIOnNominatingGovernors(iPlayer)
	local gTurn = GetITGGameTurn();
	if (gTurn > 40) and (gTurn < 60) then
		if (not GetGovActionBlockType(iPlayer)) and (not gT.Players[iPlayer].GovCauses[16]) then
			if LastChance(25) then
				if (Players[iPlayer]:IsGoldenAge()) or gT.Players[iPlayer].HadGoldenAge then
					local avGovs = {}
					for govID, h in pairs(gT.Players[iPlayer].Governors) do
						if (not GovHasSignificantPosition(govID)) then
							table.insert(avGovs, govID);
						end
					end
					if #avGovs > 0 then
						local posTab = GetAvailableGovPositions(iPlayer);
						local iPosTab = {}
						for i, b in pairs(posTab) do
							table.insert(iPosTab, GetAIEvalOfGovPosition(iPlayer, i))
						end
						if #iPosTab > 0 then
							table.sort(iPosTab, SortFunctionForGreaterInteger2)
							if iPosTab[1][2] > LastRandom(1,10) then
								local govID = RandomFromTab(avGovs);
								AIResolveVoidedDiploNote(iPlayer, 50, iPosTab[1][1], govID)
							end
						end
					end
				end
			end
		end
	end
end

function HasAnyValidPositionForNomination(iPlayer)
	for a, b in pairs(GetAvailableGovPositions(iPlayer)) do
		return true;
	end
	return false;
end

function GetStringFromGovPosition(type, bTT)
	for a, b in pairs(govPositionTab) do
		if b.Type == type then
			return b.Name, b.ToolTip;
		end
	end
	return "ERRoR 43.21.44: " .. (type or "X?");
end

function ClearGovernorNotablePositions(govID)
	for a, b in pairs(govPositionTab) do
		gT.Globals.Governors[govID][b.Type] = nil;
	end
end

function GovHasSignificantPosition(govID)
	for a, credential in pairs(govPositionTab) do
		if gT.Globals.Governors[govID][credential.Type] then
			return true;
		end
	end
	return false;
end

function GetAvailableGovPositions(iPlayer)
	local taken = GetTakenGovPositions(iPlayer)
	local tab = {};
	for a, b in pairs(govPositionTab) do
		if not taken[b.Type] then
			tab[b.Type] = true;
		end
	end
	return tab;
end

function GetTakenGovPositions(iPlayer)
	local taken = {}
	for govID, h in pairs(gT.Players[iPlayer].Governors) do
		for a, b in pairs(govPositionTab) do
			if gT.Globals.Governors[govID][b.Type] then
				taken[b.Type] = true;
			end
		end
	end
	if gT.Players[iPlayer].SecularState then
		taken["HighPriest"] = true;
	end
	return taken;
end

function GetBestPositionToAspireTo(govID, iPlayer)
	local taken = GetTakenGovPositions(iPlayer)
	local tab = {};
	local function AddPositionToOptions(pos)
		if not taken[pos] then
			table.insert(tab, pos)
		end
	end
	for a, b in pairs(govPositionTab) do
		AddPositionToOptions(b.Type)
	end
	local gType = gT.Globals.Governors[govID].Type;
	if gType == 1 then
		AddPositionToOptions("CommanderInChief")
	elseif gType == 6 then
		AddPositionToOptions("HighPriest")
	end
	if #tab > 0 then
		return RandomFromTab(tab);
	end
	return nil;
end

function GetUnitSupplyFromGovernors(iPlayer)
	local t = 0;
	for govID, bool in pairs(gT.Players[iPlayer].Governors) do
		t = t + (gT.Globals.Governors[govID].Supply or 0);
	end
	return t;
end

function GetGovernorExtraInfoTT(govID)
	local tt = ""
	if (gT.Globals.Governors[govID].MeritPT or 0) > 0 then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]+" .. gT.Globals.Governors[govID].MeritPT .. "[ICON_ITG_MERITOCRACY] Meritocracy and [ICON_ITG_POWER] Power per turn."
	end
	if (gT.Globals.Governors[govID].LegalCity) then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]Legalizing ownership of " .. Map.GetPlotByIndex(gT.Globals.Governors[govID].LegalCity[1]):GetPlotCity():GetName() .. "."; 
	end
	if gT.Globals.Governors[govID].Supply then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]+" .. gT.Globals.Governors[govID].Supply .. " [ICON_ITG_SUPPLY] Unit Supply and +1 [ICON_ITG_POWER] Power per turn."
	end
	if gT.Globals.Governors[govID].Ideo59 then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]Reduces cost of training " .. Locale.Lookup(GameInfo.UnitClasses[gT.Globals.Governors[govID].Ideo59].Description) .. " by 30 [ICON_PRODUCTION] Production."
	end
	if gT.Globals.Governors[govID].IntelInsight then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]In case of conflict your [ICON_ITG_POWER] Power is 25% less effective against this [ICON_ITG_GOVERNOR] Governor."
	end
	if gT.Globals.Governors[govID].Art > 0 then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]+" .. gT.Globals.Governors[govID].Art .. "[ICON_GREAT_ARTIST] Art points per turn."
	end
	if gT.Globals.Governors[govID].HighPriest then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]High Priest (choosing reward from religious bonus)."
	end
	if gT.Globals.Governors[govID].CommanderInChief then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]Commander-In-Chief (choosing reward from general and admiral bonuses)."
	end
	if gT.Globals.Governors[govID].Chancellor then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]Commander-In-Chief (choosing reward from scientific bonus)."
	end
	if gT.Globals.Governors[govID].Curator then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]Protector (choosing reward from cultural bonus)."
	end
	if gT.Globals.Governors[govID].RelBonusDiscount then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]Discounts religious bonus cost by " .. 100 * gT.Globals.Governors[govID].RelBonusDiscount .. "%."
	end
	if gT.Globals.Governors[govID].MilBonusDiscount then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]Discounts general and admiral bonuses cost by " .. 100 * gT.Globals.Governors[govID].MilBonusDiscount .. "%."
	end
	if gT.Globals.Governors[govID].ScienceBonusDiscount then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]Discounts scientific bonus cost by " .. 100 * gT.Globals.Governors[govID].ScienceBonusDiscount .. "%."
	end
	if gT.Globals.Governors[govID].CodexInfluence then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]Grants +" .. gT.Globals.Governors[govID].CodexInfluence .. "% [ICON_ITG_LAWS_INFLUENCE] Codex Influence."
	end
	if gT.Globals.Governors[govID].CultureBonusDiscount then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]Discounts cultural bonus cost by " .. 100 * gT.Globals.Governors[govID].CultureBonusDiscount .. "%."
	end
	if gT.Globals.Governors[govID].GrowWary then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]Grew wary - less likely to be executed."
	end
	if gT.Globals.Governors[govID].Betrayed then
		tt = GetNewlineIFNot(tt, "")
		tt = tt .. "[ICON_BULLET]If not already assigned, this [ICON_ITG_GOVERNOR] Governor will be automatically assigned to any " .. Players[gT.Globals.Governors[govID].Betrayed]:GetCivilizationAdjective() .. " city you conquer."
	end
	return tt;
end

function GetValidGovernorBuilding(govID, iCity)
	local iOwner = iCity:GetOwner();
	local tab = {};
	for i, buildingclass in pairs(l_govCareerTypes[gT.Globals.Governors[govID].Type].BuildingsC or {}) do
		local bType = Players[iOwner]:GetPlayerUniqueBuilding(buildingclass)
		if bType ~= -1 then
			if IsValidBuildingToBeGivenToCity(iCity, bType) then
				table.insert(tab, bType)
			end
		end
	end
	if #tab > 0 then
		return tab[LastRandom(1,#tab)];
	end
end

function ApplyGovernorBonusOnCityFound(iPlayer, gBonus, gLevel, iCity, cID, bAll)
	local blog1 = true;
	if gBonus == -1 then
		ChangeMeritocracy(iPlayer, 7)
	else
		if gBonus == 1 then
			blog1 = DoITGCityAcquirePlot(iCity, bActive)
			for i = 1, gLevel * 2 - 1 do
				DoITGCityAcquirePlot(iCity, bActive)
			end
		elseif gBonus == 2 then
			iCity:ChangePopulation(gLevel, true)
		elseif gBonus == 3 then
			DoChangeCityCultureOwnership(cID, iCity, iPlayer, 20 + 30 * gLevel, "OnFoundedGov")
		elseif gBonus == 4 then
			local bTab = GetRandomBuildingsAvailable(iCity);
			local given = math.min(gLevel, #bTab);
			if given > 0 then
				for a, building in pairs(bTab) do
					if IsValidBuildingToBeGivenToCity(iCity, building) then
						iCity:SetNumRealBuilding(building, 1);
						given = given - 1;
						if given == 0 then
							break;
						end
					end
				end
			else
				blog1 = false;
			end
		end
		if not bAll then
			ChangeMeritocracy(iPlayer, 5)
		end
	end
	return blog1;
end

function GovernorDreamActionToMake(govID, iPlayer)
	local gTurn = GetITGGameTurn();
	if gT.Globals.Governors[govID].Dream then
		if not gT.Globals.Governors[govID].Dream.Active then
			return false;
		end
		local dreamID = gT.Globals.Governors[govID].Dream.ID;
		local bStrongetThanPlayer = gT.Globals.Governors[govID].Power > GetPlayerGovernorPower(iPlayer);
		local bStrongestGov = IsTheStrongestGovernor(govID, iPlayer);
		if dreamID == 1 then
			gT.Globals.Governors[govID].Dream.Active = false;
			return false;
		elseif dreamID == 5 then
			if bStrongetThanPlayer and bStrongestGov then
				return true;
			end
		else
			if gTurn < LastRandom(75,85) then
				--still have time to act!
				if dreamID == 2 then
					local chancetoact = 25;
					if bStrongetThanPlayer then
						chancetoact = chancetoact + 25;
					end
					if gTurn > 70 then
						chancetoact = chancetoact + 50;
					end
					if LastChance(chancetoact) then
						if DoGovernorConsiderActionIfPossible(govID, 27) then 
							return true;
						end
					end
				elseif dreamID == 3 then
					if gT.Players[iPlayer].RevolutionGoverment then
						gT.Globals.Governors[govID].Dream.Active = false;
						return false;
					elseif bStrongestGov then
						local chancetoact = 30;
						if bStrongetThanPlayer then
							chancetoact = chancetoact + 30;
						end
						if gTurn > 70 then
							chancetoact = chancetoact + 40;
						end
						if LastChance(chancetoact) then
							if DoGovernorConsiderActionIfPossible(govID, 40, true) then
								gT.Globals.Governors[govID].Dream.Active = false;
								return true;
							end
						end
					end
				elseif dreamID == 4 then
					if GovHasSignificantPosition(govID) then
						gT.Globals.Governors[govID].Dream.Active = false;
					else
						local chancetoact = 10;
						if bStrongetThanPlayer then
							chancetoact = chancetoact + 40;
						end
						if bStrongestGov then
							chancetoact = chancetoact + 20;
						end
						if LastChance(chancetoact) then
							if gTurn > 70 then
								if DoGovernorConsiderActionIfPossible(govID, 38, true) then
									gT.Globals.Governors[govID].Dream.Active = false;
									return true;
								end
							elseif DoGovernorConsiderActionIfPossible(govID, 38) then 
								gT.Globals.Governors[govID].Dream.Active = false;
								return true;
							end
						end
					end
				elseif dreamID == 6 then
					local chanceToAct = 10;
					if bStrongetThanPlayer then
						chanceToAct = chanceToAct + 10;
					end
					if bStrongestGov then
						chanceToAct = chanceToAct + 10;
					end
					if Teams[Players[iPlayer]:GetTeam()]:GetAtWarCount(true) == 0 then
						chanceToAct = chanceToAct + 50;
					end
					if LastChance(chanceToAct) then
						if DoGovernorConsiderActionIfPossible(govID, 28) then 
							gT.Globals.Governors[govID].Dream.Active = false;
							return true;
						end
					end
				elseif dreamID == 7 then
					local chanceToAct = 70;
					for a, b in pairs(gT.Globals.Governors[govID].ForeignContacts) do
						if b.SecretPact then
							chanceToAct = chanceToAct - 20;
						end
					end
					if chanceToAct >= LastRandom(1, 100) then
						if DoGovernorConsiderActionIfPossible(govID, 33) then 
							return true; 
						end
					end
				end
			end
		end
		return false;
	end
--shall we develop new dream
	if gTurn < 45 and gTurn >= 20 then
		if LastChance(30) then
			DevelopGovDream(govID, iPlayer, false)
		end
	end
	return false;
end

function GovernorDreamShatteredVengeanceNow(govID, iPlayer, actionID)
	if LastChance(35) then
		if gT.Globals.Governors[govID].Dream then
			if not gT.Globals.Governors[govID].Dream.Active then
				if gDreamTypes[gT.Globals.Governors[govID].Dream.ID].CoreAction == actionID then
					gT.Globals.Governors[govID].Dream = nil;
					AddNewGovDream(govID, 7, iPlayer, false)
				end
			end
		end
	end
end

function DevelopGovDream(govID, iPlayer, bInit)
	local tab = {}
	for a, b in pairs(gDreamTypes) do
		local baseChance = b.Chance + GetDreamChanceFromLoyaltyType(gT.Globals.Governors[govID].Character.LoyaltyType, a) + GetDreamChanceFromCareerType(gT.Globals.Governors[govID].Type, a);
		if baseChance > 0 then
			table.insert(tab, {a, baseChance})
		end
	end
	if #tab == 0 then
		return;
	end
	AddNewGovDream(govID, GetRandomFromBiasedChanceTable(tab), iPlayer, bInit)
end

function AddNewGovDream(govID, dreamID, iPlayer, bInit)
	gT.Globals.Governors[govID].Dream = {ID = dreamID, Active = true}
	if gDreamTypes[dreamID].Init then
		gDreamTypes[dreamID].Init(govID, iPlayer, bInit);
	end
end

function IsGovDuringDreamRelatedSituation(govID, iPlayer)
	if gT.Globals.Governors[govID].Dream then
		local cActionID = gDreamTypes[gT.Globals.Governors[govID].Dream.ID].CoreAction;
		if cActionID then
			for a, b in pairs(gT.Players[iPlayer].DiploNotes) do
				if (b.Type == cActionID) and (b.Governor == govID) then
					return true;
				end
			end
		end
	end
	return false;
end