local synTab = {
["astonishing"] = {"astonishing", "magnificent", "wonderful", "remarkable", "marvelous", "impressive", "stunning", "astounding", "breathtaking", "extraordinary", "spectacular"},
["coincidently"] = {"coincidently", "accidentally", "coincidentally", "by chance"},
["enormous"] = {"enormous", "huge", "vast", "extensive", "expansive", "broad", "wide"},
["noticeable"] = {"noticeable", "distinguishable", "observable", "visible", "recognizable", "notable", "measurable"},
["inexplicable"] = {"inexplicable", "unaccountable", "unexplainable", "incomprehensible", "baffling", "puzzling"},
["important"] = {"important", "principal", "key", "major", "prime", "cardinal", "crucial", "vital", "critical", "essential", "significant", "urgent"},
["extraordinarily"] = {"extraordinarily", "remarkably", "exceptionally", "incredibly", "spectacularly"},
["obscure"] = {"obscure", "unknown"},
["undeniable"] = {"undeniable", "indisputable", "unquestionable", "beyond doubt", "beyond question", "inarguable", "undebatable"},
["prominent"] = {"prominent", "promising", "hopeful", "auspicious", "bright"},
["unpleasant"] = {"unpleasant", "troublesome", "displeasing", "horrible", "terrible", "awful", "dreadful", "repulsive", "repellent"},
["blatant"] = {"blatant", "flagrant", "glaring", "open", "evident"},
["substantial"] = {"substantial", "considerable", "solid", "significant", "notable", "worthwhile"},
}
function HnGSynonym(type, exclude)
	if not synTab[type] then
		print("Missing synonym!", type)
		return type;
	end
	if exclude then
		local tab = {}
		for a, b in pairs(synTab[type]) do
			if b ~= exclude then
				table.insert(tab, b)
			end
		end
		return tab[LastRandom(1,#tab)];
	else
		return synTab[type][LastRandom(1,#synTab[type])]
	end
end

local numberthTab = {"first", "second", "third", "fourth", "fifth", "sixth", "seventh", "eighth", "ninth", "tenth", "eleventh", "twelfth", "thirteenth", "fourteenth", "fifteenth", "sixteenth", "seventeenth", "eighteenth", "nineteenth", "twentieth", "twenty first", "twenty second", "twenty third", "twenty fourth", "twenty fifth" }
local numbT = {"one", "two", "three", "four", "five", "six", "seven", "eight", "nine", "ten", "eleven", "twelve", "thirteen", "fourteen", "fifteen", "sixteen", "seventeen", "eighteen", "nineteen"}
local numbTtens = {"ten", "twenty", "thirty", "fourty", "fifty", "sixty", "seventy", "eighty", "ninety"}

function GetNumberTH(val)
	return numberthTab[val] or "ERROR 934/3423"
end

function GetYearthOfReign(turn)
	return GetNumberTH(math.ceil(turn / 4))
end

function GetNumberString(hV)
	if hV < 20 then
		return numbT[hV];
	else
		local tens = math.floor(hV / 10)
		local left = hV - tens * 10;
		if left == 0 then
			return numbTtens[tens]
		end	
		return numbTtens[tens] .. numbT[left];
	end
end

function HnGPeriodOfTime(time)
	local hV = math.ceil(time/4);
	if hV <= 1 then
		return "a year"
	else
		return GetNumberString(hV) .. " years";
	end
end

function GetCenturyThFromYear(turn)
	local cth = GetCenturyFromTurn(turn, gT.Globals.StartDate)
	local adbc = "" --" AD"
	if cth < 0 then
		cth = -cth;
		adbc = " BC"
	end
	return cth .. GetThStNdRd(cth) .. " century" .. adbc
end

function HnGGetScenarioAdjective()
	if gT.Globals.ScenarioType == 1 then
		return "renaissance" 
	elseif gT.Globals.ScenarioType == 2 then
		return "medieval"
	elseif gT.Globals.ScenarioType == 3 then
		return "ancient"
	elseif gT.Globals.ScenarioType == 4 then
		return "industrial"
	elseif gT.Globals.ScenarioType == 5 then
		return "modern"
	end
	return "UNKNOWN ADJECTIVE SCENARIO"
end