local ideLetters = {"R", "F", "A", "S", "E", "O", "L"}
local getPrevErasMinor = {[1] = {2, 3}, [2] = {3}, [3] = {}, [4] = {1, 2, 3}, [5] = {1, 2, 3}}
local revIdeoPos;

function GetIdeoLetters()
	return ideLetters;
end

function GetPositionOfIdeology(id)
	if not revIdeoPos then
		revIdeoPos = ReverseTable(gT.IdeologiesPosition)
	end
	return revIdeoPos[id];
end

function InitIdeologiesPosition()
	gT.IdeologiesPosition = {};
	for i = 1, 93 do
		table.insert(gT.IdeologiesPosition, i)
	end
	if (PreGame.GetGameOption("GAMEOPTION_ITG_SHUFFLE_IDEOLOGIES") == 1) then
		gT.IdeologiesPosition = Shuffle(gT.IdeologiesPosition);
	end
end

function InitIdelogies(hMajorTab)
	for i, player in pairs(hMajorTab) do
		gT.Players[player].Ideologies = {}
		for h, l in pairs(ideLetters) do
			gT.Players[player].Ideologies[l] = 0;
		end
	end
	gT.IdeologyTurns = {}
	gT.AvailableIdeologies = {}
	gT.IdeologyPlayersLeft = {}
	gT.IdeologyTaken = {}
	gT.NumIdeologyPlayersLeft = 0;
	ReInstallIdeologies(hMajorTab)
	if DBScenario.IdeologyOn then
		if gT.Globals.ScenarioType == 4 then
			local nummajorcivs = #hMajorTab;
			local rounds = 5;
			local thdd = 18;
			if nummajorcivs > 18 then
				rounds = 4;
				thdd = 22;
			end
			for i = 1, rounds do
				gT.IdeologyTurns[6 + (i - 1) * thdd] = 1;	
			end
			gT.AvailableIdeologies[gT.IdeologiesPosition[47]] = 1;
			DoMakeOtherTenetsAvailable(gT.IdeologiesPosition[47])
		end
	else
		local avTab =  GetTabOfEraMinorIdeologies(gT.Globals.ScenarioType)
		if #avTab > 0 then
			for j, h in pairs(avTab) do
				gT.AvailableIdeologies[h] = 1;
			end
			gT.IdeologyTurns[84] = 1;
		end
	end
end

function DoTurnIdeologies()
	if gT.IdeologyPlayerPicking then
		ChooseRandomIdeologyForPlayer(gT.IdeologyPlayerPicking, true)
	end
	if gT.IdeologyTurns[GetITGGameTurn()] then
		gT.IdeologyPlayersLeft = {}
		local t = 0;
		for g, gh in pairs(gT.Players) do
			if not Players[g]:IsMinorCiv() then
				gT.IdeologyPlayersLeft[g] = 1;
				t = t + 1;
			end
		end
		gT.NumIdeologyPlayersLeft = t;
	end
	if gT.NumIdeologyPlayersLeft > 0 then
		local tH, hTab = -1, {};
		for g, tt in pairs(gT.IdeologyPlayersLeft) do
			local cindi = gT.Players[g].NatUnity;
			if cindi > tH then
				tH = cindi;
				hTab = {g}
			elseif cindi == tH then
				table.insert(hTab, g)
			end
		end
		if #hTab > 0 then
			local lP = hTab[LastRandom(1,#hTab)];
			gT.IdeologyPlayerPicking = lP;
			gT.Players[lP].IsPickingIdeology = true
		else
			print("Very heavy error 352235/23432")
		end
	end
end

function FinishAssigningMinorIdeology()
	local lTT;
	for t, s in pairs(gT.AvailableIdeologies) do
		lTT = t;
	end
	gT.AvailableIdeologies = {}
	for player, h in pairs(gT.IdeologyPlayersLeft) do
		PlayerChooseIdeologyTenet(player, lTT, false)
	end
	gT.IdeologyPlayersLeft = {}
	gT.NumIdeologyPlayersLeft = 0;
end

LuaEvents.ITG_RedirectPlayerIdeologyTenet.Add(function()
	if DBScenario.IdeologyOn then
		Events.SerialEventGameMessagePopup( { Type = ButtonPopupTypes.BUTTONPOPUP_CHOOSEPOLICY, Data2 = 2} );
	else
		Events.SerialEventGameMessagePopup( { Type = ButtonPopupTypes.BUTTONPOPUP_CHOOSEPOLICY, Data2 = -1} );
	end
end)

function DefineAIIdeoPickLogic(iPlayer)
	if gT.Globals.ScenarioType == 4 then
		if LastRandom(1,3) == 2 then
			gT.Players[iPlayer].AI_IdeoPickLogic = 1;
			return;
		end
	end
	gT.Players[iPlayer].AI_IdeoPickLogic = 0;
end

function GetAIFavouredIdeasIdeology(iPlayer)
	local tab = {};
	local bestScore = 0;
	for h, l in pairs(ideLetters) do
		if gT.Players[iPlayer].Ideologies[l] > bestScore then
			tab = {l}
			bestScore = gT.Players[iPlayer].Ideologies[l];
		elseif gT.Players[iPlayer].Ideologies[l] == bestScore then
			table.insert(tab, l)
		end
	end
	return tab;
end

function DoAIPickNewIdeology(iPlayer)
	if DBScenario.IdeologyOn then
		if not gT.Players[iPlayer].AI_IdeoPickLogic then
			DefineAIIdeoPickLogic(iPlayer)
		end
		GenerateAIIdeologyFavours(iPlayer)
		local rTab = {}
		if gT.Players[iPlayer].AI_IdeoPickLogic == 1 then
			local bestScore = 0;
			local tempTab = {}
			local listOfFavouredVals = GetAIFavouredIdeasIdeology(iPlayer)
			for h, i in pairs(gT.AvailableIdeologies) do
				local hRow = GetIdeologyDBRow(GetPositionOfIdeology(h));
				local tempVal = 0;
				for j, letter in pairs(listOfFavouredVals) do
					tempVal = tempVal + hRow[letter];
				end
				if tempVal > bestScore then
					tempTab = {h};
					bestScore = tempVal;
				elseif tempVal == bestScore then
					table.insert(tempTab, h)
				end
			end
			for i, h in pairs(tempTab) do
				table.insert(rTab, {h, GetAIEvalOfIdeologyTenet(iPlayer, h)})
			end
		else
			for h, i in pairs(gT.AvailableIdeologies) do
				table.insert(rTab, {h, GetAIEvalOfIdeologyTenet(iPlayer, h)})
			end
		end
		rTab = Shuffle(rTab)
		table.sort(rTab, SortFunctionForGreaterInteger2)
		PlayerChooseIdeologyTenet(iPlayer, rTab[1][1], true)
	else
		local rTab = {}
		for h, i in pairs(gT.AvailableIdeologies) do
			table.insert(rTab, {h, GetMinorIdeologyDBRow(h).AIValueFunc(iPlayer)})
		end
		PlayerChooseIdeologyTenet(iPlayer, GetRandomBiasedAmplifiedTable(rTab, 2), true)
	end
end

function DoAddNumbersIdeology(rPlayer, tenet, c)
	local rowData = GetIdeologyDBRow(GetPositionOfIdeology(tenet))
	for i, letter in pairs(ideLetters) do
		if rowData[letter] > 0 then
			gT.Players[rPlayer].Ideologies[letter] = gT.Players[rPlayer].Ideologies[letter] + c * rowData[letter]
		end
	end
end

function DoAddNumberIdeologyMinor(rPlayer, tenet, c)
	local letter = GetMinorIdeologyDBRow(tenet).Ideology
	gT.Players[rPlayer].Ideologies[letter] = gT.Players[rPlayer].Ideologies[letter] + c
end

function DoMakeOtherTenetsAvailable(tenet)
	local rowData = GetIdeologyDBRow(GetPositionOfIdeology(tenet))
	for i, tt in pairs(rowData.Connected) do	
		local ideoOnSlot = gT.IdeologiesPosition[tt];
		if not gT.IdeologyTaken[ideoOnSlot] then
			gT.AvailableIdeologies[ideoOnSlot] = 1;
		end
	end
end

function ChooseRandomIdeologyForPlayer(iPlayer, bNow)
	local rTab = {}
	for h, i in pairs(gT.AvailableIdeologies) do
		table.insert(rTab, h)
	end
	PlayerChooseIdeologyTenet(iPlayer, rTab[LastRandom(1, #rTab)], bNow)
end

function WasLastIdeology()
	local gTurn = GetITGGameTurn();
	for turn, sth in pairs(gT.IdeologyTurns) do
		if turn >= gTurn then
			return false;
		end
	end
	return true;
end

function GetLeftIdeologiesAv()
	return GetTableSizeInteger(gT.AvailableIdeologies, "opdede")
end

function AddEffectsOfIdeology(rPlayer, tenet)
	DoAddNumbersIdeology(rPlayer, tenet, 1)
	gT.Players[rPlayer].HasIdeology[tenet] = 1;
	if gT.Globals.GameGenerated then
		GovTypePowerBoost(rPlayer, 9, "Fanatism", 1, false)
	end
	DoAddEffectsOfIdeology(tenet, rPlayer, bNowPicked)
end

function PlayerChooseIdeologyTenet(rPlayer, tenet, bNowPicked, iScenarioType)
	gT.IdeologyPlayerPicking = nil;
	gT.Players[rPlayer].IsPickingIdeology = nil;
	if bNowPicked then
		gT.IdeologyPlayersLeft[rPlayer] = nil;
		gT.NumIdeologyPlayersLeft = gT.NumIdeologyPlayersLeft - 1;
	end
	if DBScenario.IdeologyOn and (not iScenarioType) then
		gT.IdeologyTaken[tenet] = rPlayer 
		gT.AvailableIdeologies[tenet] = nil
		DoMakeOtherTenetsAvailable(tenet)
		--)F) effects of ideologies
		AddEffectsOfIdeology(rPlayer, tenet)
		if bNowPicked then
			MakeSomeCultureDevsFadeAway(rPlayer, 2)
			if WasLastIdeology() then
				local cloneTab = {}
				for h, v in pairs(gT.Players[rPlayer].Ideologies) do
					cloneTab[h] = v;
				end
				gT.Players[rPlayer].IdeologyName = GetIdeologyName(rPlayer, cloneTab)
				--notification!
				for i, iHuman in pairs(gT.Globals.HumanPlayers) do
					if Players[iHuman]:IsAlive() then
						if iHuman == rPlayer then
							Players[iHuman]:AddNotification(NotificationTypes.NOTIFICATION_IDEOLOGY_CHOSEN, "As long as we believe in " .. gT.Players[rPlayer].IdeologyName .. " we will make world a better place!", "Ideology")
						else
							Players[iHuman]:AddNotification(NotificationTypes.NOTIFICATION_IDEOLOGY_CHOSEN, Players[rPlayer]:GetCivilizationShortDescription() .. " claims that " .. gT.Players[rPlayer].IdeologyName .. " will make world a better place!", "Ideology")
						end
					end
				end
				if Players[rPlayer]:IsHuman() then
					local bAchievementBelow6 = true;
					for h, v in pairs(gT.Players[rPlayer].Ideologies) do
						if v > 5 then
							bAchievementBelow6 = false;
							if v >= 12 then
								LuaEvents.ITG_AchievementCompleted(16)
							end
						end
						if v == 0 then
							LuaEvents.ITG_AchievementCompleted(75)
						end
					end	
					if gT.Players[rPlayer].Ideologies.F == gT.Players[rPlayer].Ideologies.A then
						if gT.Players[rPlayer].Ideologies.S == gT.Players[rPlayer].Ideologies.E then
							if gT.Players[rPlayer].Ideologies.O == gT.Players[rPlayer].Ideologies.L then
								LuaEvents.ITG_AchievementCompleted(76)
							end
						end
					end
					if bAchievementBelow6 then
						LuaEvents.ITG_AchievementCompleted(17)
					end
				end
			end
		end
	else
		gT.Players[rPlayer].HasMinorIdeology[tenet] = iScenarioType or gT.Globals.ScenarioType;
		DoAddNumberIdeologyMinor(rPlayer, tenet, 1)
		if GetMinorIdeologyDBRow(tenet).OnGoldenAge then
			if Players[rPlayer]:IsGoldenAge() then	
				DoTriggerIdeologyTenetsOnGoldenAgeOutbreak(rPlayer, tenet)
			end
		end
		if GetMinorIdeologyDBRow(tenet).WonderDirty then
			for iCity in Players[rPlayer]:Cities() do
				WonderRuinsCityDirty(iCity)
			end
		end
		if gT.Globals.GameGenerated then
			GovTypePowerBoost(rPlayer, 9, "Fanatism", 1, false)
		end
		if bNowPicked then
			gT.AvailableIdeologies[tenet] = nil
			if GetLeftIdeologiesAv() == 1 then
				FinishAssigningMinorIdeology()
			end
		end
	end
end

function GetNumIdeologiesFromPlayer(numPlayers)
	return math.max(2, math.ceil(numPlayers / 6))
end

function ReinstallIdeologyForModernEra(hMajorTab)
	local numIdeologies = GetNumIdeologiesFromPlayer(#hMajorTab)
	hMajorTab = Shuffle(hMajorTab);
	local ideoTab = {}
	for i = 1, 93 do
		ideoTab[i] = true;
	end
	-- here era data
	local aHtop = 2;
	if gT.EraData and gT.PreviousEra and gT.EraData[gT.PreviousEra] then
		for sth, player in pairs(hMajorTab) do
			local numideologies = 0;
			for ideology, sctype in pairs(tempconvinit[player].HasIdeology) do
				DoAddNumbersIdeology(player, ideology, 1)
				gT.Players[player].HasIdeology[ideology] = 1;
				numideologies = numideologies + 1;
				ideoTab[ideology] = false;
			end
			aHtop = math.max(numideologies, aHtop);
			gT.Players[player].IdeologyName = tempconvinit[player].IdeologyName;
		end
	end
	-- here era data
	aHtop = aHtop - 1;
	local lastIdeology = -1;
	for i, player in pairs(hMajorTab) do
		local nIdeo = 0;
		for a, b in pairs(gT.Players[player].HasIdeology) do
			nIdeo = nIdeo + 1;
		end
		for dd = nIdeo, aHtop do -- in case some player was eleminanted before choosing any ideology?
			local addIdeo = lastIdeology;
			local avI = {}
			for c, b in pairs(ideoTab) do
				if b then
					table.insert(avI, c)
				end
			end
			if #avI > 0 then
				addIdeo = avI[LastRandom(1, #avI)];
			end
			DoAddNumbersIdeology(player, addIdeo, 1)
			gT.Players[player].HasIdeology[addIdeo] = 1;
			ideoTab[addIdeo] = false;
		end
	end
	DoFinishCreatingInitialIdeologies(numIdeologies, hMajorTab)
end

function DoInitialIdeologiesModernEra(hMajorTab)
	local numIdeologies = math.max(2, math.ceil(#hMajorTab / 6))
	local nIdeo = -1;
	-- 5 ideologies
	local ideoTab = {}
	for i = 1, 93 do
		table.insert(ideoTab, i)
	end
	hMajorTab = Shuffle(hMajorTab);
	for j = 1, 5 do
		for i, player in pairs(hMajorTab) do
			if #ideoTab > 0 then
				local lKey = LastRandom(1, #ideoTab);
				nIdeo = ideoTab[lKey];
				table.remove(ideoTab, lKey)
			end
			DoAddNumbersIdeology(player, nIdeo, 1)
			gT.Players[player].HasIdeology[nIdeo] = 1;
		end	
	end
	DoFinishCreatingInitialIdeologies(numIdeologies, hMajorTab)
end
	--the copy paste same    numideologies hMajorTab
function DoFinishCreatingInitialIdeologies(numIdeologies, hMajorTab)
	local tab = Shuffle(GetPlayersWithMostDiverseIdeologies(numIdeologies, hMajorTab))
	local ideoToDistribute = {};
	local baseNumPlayers = math.floor((#hMajorTab - #tab) / #tab);
	local leftovers = #hMajorTab - math.floor(1 + (#hMajorTab - #tab) / #tab) * #tab;
	local extraIdeologies = {};
	for i, player in pairs(tab) do
		ideoToDistribute[player] = baseNumPlayers;
		if i <= leftovers then
			ideoToDistribute[player] = ideoToDistribute[player] + 1;
		end
		--add properly ideology
		local numideologies = 0;
		for h, v in pairs(gT.Players[player].HasIdeology) do
			DoAddNumbersIdeology(player, h, -1)
			AddEffectsOfIdeology(player, h)
			gT.IdeologyTaken[h] = player;
			numideologies = numideologies + 1;
		end
		for j = numideologies, 4 do -- this ensures start with 5 ideologies
			table.insert(extraIdeologies, player);
		end
		if not gT.Players[player].IdeologyName then
			gT.Players[player].IdeologyName = GetIdeologyName(player, gT.Players[player].Ideologies)
		end
		gT.Players[player].IdeologyFromPlayer = player;
	end
	if #extraIdeologies > 0 then
		extraIdeologies = Shuffle(extraIdeologies);
		for a, iPlayer in pairs(extraIdeologies) do
			local tenetoptions = GetTenetsToChooseFromLeague(iPlayer)
			if #tenetoptions > 0 then
				local tenetID = RandomFromTab(tenetoptions);
				AddEffectsOfIdeology(iPlayer, tenetID)
				gT.IdeologyTaken[tenetID] = iPlayer;
			end
		end
	end
	local tgt = {}
	for g, h in pairs(gT.Players) do
		if not Players[g]:IsMinorCiv() then
			if not ideoToDistribute[g] then
				for a, b in pairs(ideoToDistribute) do
					table.insert(tgt, {g, GetDifIdeologyOfTwoPlayer(g, a), a})
				end
				for c, v in pairs(h.HasIdeology) do
					DoAddNumbersIdeology(g, c, -1)
				end
				gT.Players[g].HasIdeology = {};
			end
		end
	end
	tgt = Shuffle(tgt)
	table.sort(tgt, SortFunctionForLowestMapDistance);
	for i, data in pairs(tgt) do
		local g = data[1];
		if not gT.Players[g].IdeologyFromPlayer then
			local lIdeo = data[3];
			if ideoToDistribute[lIdeo] > 0 then
				for c, v in pairs(gT.Players[lIdeo].HasIdeology) do
					AddEffectsOfIdeology(g, c)
				end
				gT.Players[g].IdeologyFromPlayer = lIdeo;
				gT.Players[g].IdeologyName = gT.Players[lIdeo].IdeologyName;
				ideoToDistribute[lIdeo] = ideoToDistribute[lIdeo] - 1;
			end
		end
	end
end

function GetPlayersWithMostDiverseIdeologies(iNumIdeologies, hMajorTab)
	local aJways = {}
	local aJc = 0;
	local bLoopy = true;
	for jjj = 1, iNumIdeologies do
		aJc = #aJways;
		for i, player in pairs(hMajorTab) do
			if jjj == 1 then
				table.insert(aJways, {Players = {player}, Largest = player})		
			else
				for ath = 1, aJc do
					if #aJways[ath].Players == jjj - 1 then
						if aJways[ath].Largest < player then
							table.insert(aJways, {Players = {player}, Largest = player})
							for tht, bub in pairs(aJways[ath].Players) do
								table.insert(aJways[#aJways].Players, bub)
							end
						end
					end
				end
			end
		end
	end
	local tSort = {}
	for a, b in pairs(aJways) do
		if #b.Players == iNumIdeologies then
			local eValuate;
			local eTotal = 0;
			for i, player in pairs(aJways[a].Players) do
				for j, jplayer in pairs(aJways[a].Players) do
					if j > i then
						local eDif = GetDifIdeologyOfTwoPlayer(player, jplayer);
						eTotal = eTotal + eDif;
						if not eValuate then
							eValuate = eDif;
						else
							eValuate = math.min(eValuate, eDif);
						end
					end
				end
			end
			table.insert(tSort, {a, eValuate, eTotal})
		end
	end
	print("Ideo generating, num players, num options", #hMajorTab, #tSort);
	table.sort(tSort, SortFunctionGreatest2Greatest3)
	return aJways[tSort[1][1]].Players;
end

function GetDifIdeologyOfTwoPlayer(a, b)
	local dis = 0
	for i, letter in pairs(ideLetters) do
		dis = dis + math.pow(gT.Players[a].Ideologies[letter] - gT.Players[b].Ideologies[letter],2)
	end
	dis = math.floor(dis * 100) / 100;
	return dis;
end

function ReInstallIdeologies(hMajorTab)
	if gT.EraData and gT.PreviousEra and gT.EraData[gT.PreviousEra] then
		for sth, hMajor in pairs(hMajorTab) do
			for minorT, sctype in pairs(tempconvinit[hMajor].HasMinorIdeology) do
				PlayerChooseIdeologyTenet(hMajor, minorT, false, sctype)
			end
		end
		if gT.Globals.ScenarioType == 5 then
			ReinstallIdeologyForModernEra(hMajorTab)
		end
	else
		if not getPrevErasMinor[gT.Globals.ScenarioType] then
			print("No logic for reinstalling scenario ideology!!")
		else
			for i, prevEra in pairs(getPrevErasMinor[gT.Globals.ScenarioType]) do
				local hTab = Shuffle(GetTabOfEraMinorIdeologies(prevEra))
				local lTene = hTab[LastRandom(1,#hTab)];
				local cptab = Shuffle(hMajorTab);
				for j, player in pairs(cptab) do
					if hTab[j] then
						PlayerChooseIdeologyTenet(player, hTab[j], false, prevEra)
					else
						PlayerChooseIdeologyTenet(player, lTene, false, prevEra)
					end
				end
			end
		end
		if gT.Globals.ScenarioType == 5 then
			DoInitialIdeologiesModernEra(hMajorTab);
		end
	end
end

LuaEvents.ITGSG_DoLeaderHeadStats.Add(function()
	if DBScenario.IdeologyOn then
		if gT.Globals.IdeologiesLH then
			return;
		end
		local tab = {}
		local avgvalues = {}
		for i, letter in pairs(ideLetters) do
			avgvalues[letter] = 0;
			local ipp = 0;
			for g, h in pairs(gT.Players) do
				if not Players[g]:IsMinorCiv() then
					ipp = ipp + 1;
					avgvalues[letter] = avgvalues[letter] + h.Ideologies[letter];
				end
			end
			avgvalues[letter] = avgvalues[letter] / ipp;
		end
		for g, h in pairs(gT.Players) do
			if not Players[g]:IsMinorCiv() then
				local dis = 0
				for i, letter in pairs(ideLetters) do
					dis = dis + math.pow(avgvalues[letter] - h.Ideologies[letter],2)
				end
				dis = math.floor(dis * 100) / 100;
				table.insert(tab, {g, dis})
			end
		end
		table.sort(tab, SortFunctionForGreaterInteger2)
		gT.Globals.IdeologiesLH = {}
		for i, t in pairs(tab) do
			table.insert(gT.Globals.IdeologiesLH, {tab[i][1], tab[i][2]})
		end
	end
end)

function GenerateAIIdeologyFavours(iPlayer)
	if gT.Players[iPlayer].AIIdeologyFavours then
		return;
	end
	gT.Players[iPlayer].AIIdeologyFavours = {}
	gT.Players[iPlayer].AIIdeologyFavours.MassUnits = 2 * LastRandom(0, 3)
	gT.Players[iPlayer].AIIdeologyFavours.QualityUnits = 2 * LastRandom(0, 3)
	local cspyof = LastRandom(0, 2) * 2
	gT.Players[iPlayer].AIIdeologyFavours.SpyOffense = 2 + cspyof;
	gT.Players[iPlayer].AIIdeologyFavours.SpyDefense = 2 + 4 - cspyof;
	gT.Players[iPlayer].AIIdeologyFavours.FeelLucky =  2 * LastRandom(0, 3)
	gT.Players[iPlayer].AIIdeologyFavours.MishMash =  2 * LastRandom(0, 3)
end

function GetTenetsToChooseFromLeague(iPlayer)
	local tab = {}
	for i = 1, 93 do
		if not gT.IdeologyTaken[i] then
			local hVal = 0;
			local rowData = GetIdeologyDBRow(GetPositionOfIdeology(i))
			for i, letter in pairs(ideLetters) do
				if rowData[letter] > 0 then
					hVal = hVal + gT.Players[iPlayer].Ideologies[letter] * rowData[letter]
				end
			end
			table.insert(tab, {i, math.max(1, hVal)})
		end
	end
	if #tab >= 5 then
		local fTab = {};
		for i = 1, 5 do
			local tKey = GetRandomFromBiasedChanceTable(tab, true)
			table.insert(fTab, tab[tKey][1]);
			table.remove(tab, tKey);
		end
		return fTab;
	end
	return {}
end