local improChateau = GameInfoTypes.IMPROVEMENT_CHATEAU;

function HighlightChateauLocations(iPlayer)
	MapModData.ITGSG = GetPossibleChateauLocations(iPlayer);
end

function GetPossibleChateauLocations(iPlayer)
	local hTab = {}
	for i = 0, Map.GetNumPlots() - 1 do
		local fPlot = Map.GetPlotByIndex(i);
		if fPlot:GetOwner() == iPlayer then
			if CanPlotHaveImprovement(fPlot, iPlayer, improChateau) then
				table.insert(hTab, fPlot)
			end
		end
	end
	return hTab;
end

function CanPlotHaveImprovement(fPlot, iPlayer, improID)
	if fPlot:CanHaveImprovement(improID, -1) then
		if CanHaveTruelyImprovement(improID, fPlot, iPlayer) then
			if not fPlot:IsCity() then
				if (not IsSpecialControlPlot(fPlot:GetPlotIndex())) then
					if fPlot:GetImprovementType() ~= improID then
						return true;
					end
				end
			end
		end
	end
	return false;
end

function HumanWantsChateau(iPlayer, pPlot)
	if CanPlotHaveImprovement(pPlot, iPlayer, improChateau) then
		DoPlayerAddChateauOnPlot(iPlayer, pPlot)
		local hex = ToHexFromGrid(Vector2(pPlot:GetX(),pPlot:GetY()));
		Events.GameplayFX(hex.x, hex.y, -1)
	end
end

function DoPlayerAddChateauOnPlot(iPlayer, pPlot)
	gT.Players[iPlayer].ChateauCharges = gT.Players[iPlayer].ChateauCharges - 1;
	DoCreateChateauOnPlot(pPlot)
end

function DoCreateChateauOnPlot(pPlot)
	pPlot:SetImprovementType(improChateau, -1)
	pPlot:SetImprovementPillaged(false)
	local oPlayer = pPlot:GetOwner();
	gT.Globals.Chateaus[pPlot:GetPlotIndex()] = true;
	gT.Players[oPlayer].ChateauAuthority = gT.Players[oPlayer].ChateauAuthority + CHATEAU_AUTHORITY;
end

function CalculateChateauAuthority()
	for g, h in pairs(gT.Players) do
		gT.Players[g].ChateauAuthority = 0;
	end
	for index, sth in pairs(gT.Globals.Chateaus) do
		local iPlot = Map.GetPlotByIndex(index);
		if iPlot:GetImprovementType() == improChateau then
			if not iPlot:IsImprovementPillaged() then
				local iOwner = iPlot:GetOwner();
				if gT.Players[iOwner] then
					gT.Players[iOwner].ChateauAuthority = gT.Players[iOwner].ChateauAuthority + CHATEAU_AUTHORITY;
				end
			end
		else
			gT.Globals.Chateaus[index] = nil;
		end
	end
end

function AIHandleUniqueImprovementsCharges(iPlayer)
	if gT.Players[iPlayer].ChateauCharges > 0 then
		local lTab = GetPossibleChateauLocations(iPlayer);
		local evalT = {}
		for i, iPlot in pairs(lTab) do
			local val = 1;
			if not iPlot:IsPlayerCityRadius(iPlayer) then
				val = val + 70;
			end
			if iPlot:GetImprovementType() == -1 then
				val = val + 30;
			end
			table.insert(evalT, {iPlot, val})
		end
		local th = gT.Players[iPlayer].ChateauCharges;
		for j = 1, th do
			if #evalT > 0 then
				local lKey = GetRandomFromBiasedChanceTable(evalT, true);
				DoPlayerAddChateauOnPlot(iPlayer, evalT[lKey][1])
				table.remove(evalT, lKey)
			end
		end
	end
	if gT.Players[iPlayer].FeitoriaCharges > 0 then
		local hTab = GetValidFeitoriaLocations(iPlayer)
		if hTab[1] then
			hTab = Shuffle(hTab);
			local nTab = {};
			for i, plot in pairs(hTab) do
				local val = GetPlotValueToBeAFort(plot) + GetPlotValueToBeFeitoria(plot)
				table.insert(nTab, {plot, val})
			end
			table.sort(hTab, SortFunctionForGreaterInteger2)
			for i, data in pairs(nTab) do
				if gT.Players[iPlayer].FeitoriaCharges > 0 then
					gT.Players[iPlayer].FeitoriaCharges = gT.Players[iPlayer].FeitoriaCharges - 1;
					CreateNewFortForPlayer(iPlayer, data[1], 1, false, true)
				else
					break;
				end
			end
		end
	end
end

function DoReplacePlayerImproWithUnique(iPlayer, newImprovement, oldImprovement)
	for i = 0, Map.GetNumPlots() - 1 do
		local fPlot = Map.GetPlotByIndex(i);
		if fPlot:GetOwner() == iPlayer then
			if fPlot:GetImprovementType() == oldImprovement then
				if CanPlotHaveImprovement(fPlot, iPlayer, newImprovement) then
					fPlot:SetImprovementType(newImprovement, -1);
				end
			end
		end
	end
end

function DoTurnUniqueImprovements()
	RecalculateKasbahsBonus()
end

local uKasbah = GameInfoTypes.IMPROVEMENT_KASBAH;
function RecalculateKasbahsBonus()
	if not gT.Globals.Kasbahs then
		return;
	end
	for i, v in pairs(gT.Cities) do
		gT.Cities[i].KasbahTR = 0;
	end
	for i, v in pairs(gT.Globals.Kasbahs) do
		local iPlot = Map.GetPlotByIndex(i);
		if iPlot:GetImprovementType() == uKasbah then
			if not iPlot:IsImprovementPillaged() then
				local iOwner = iPlot:GetOwner();
				if gT.Players[iOwner] then
					for iCity in Players[iOwner]:Cities() do
						if Map.PlotDistance(iCity:GetX(), iCity:GetY(), iPlot:GetX(), iPlot:GetY()) < 4 then
							local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
							gT.Cities[cID].KasbahTR = gT.Cities[cID].KasbahTR + 1;
						end
					end
				end
			end
		else
			gT.Globals.Kasbahs[i] = nil;
		end
	end
	for i, v in pairs(gT.Globals.Kasbahs) do
		return;
	end
	gT.Globals.Kasbahs = nil;
end

function CanBeFeitoria(fPlot)
	if fPlot:IsCoastalLand() then
		if fPlot:IsFlatlands() then
			return true;
		end
	end
	return false;
end

function HumanWantsFeitoria(iPlayer, pPlot)
	if CanBeAFortUpgradeOrNewForPlayer(pPlot, iPlayer, true) then
		if CanBeFeitoria(pPlot) then
			gT.Players[iPlayer].FeitoriaCharges = gT.Players[iPlayer].FeitoriaCharges - 1;
			CreateNewFortForPlayer(iPlayer, pPlot, 1, false, true)
		end
	end
end

function GetValidFeitoriaLocations(iPlayer)
	local hTab = {}
	for i = 0, Map:GetNumPlots() - 1, 1 do
		local fPlot = Map.GetPlotByIndex( i );
		if CanBeAFortUpgradeOrNewForPlayer(fPlot, iPlayer, true) then
			if CanBeFeitoria(fPlot) then
				table.insert(hTab, fPlot)
			end
		end
	end
	return hTab;
end

function HighlightFeitoriaLocations(iPlayer)
	MapModData.ITGSG = GetValidFeitoriaLocations(iPlayer);
end

function GetPlotValueToBeFeitoria(iPlot)
	if #GetCoastOfOceanTab(iPlot) > 1 then
		return 20;
	end
	return 0;
end

function DoPointPossibleCanalConnection(iPlot, change, iFeitoriaPlayer)
	local hOceans =  GetCoastOfOceanTab(iPlot);
	local hCoasts = GetCoastOfCoastsTab(iPlot);
	for i, vOcean in pairs(hOceans) do
		for c, bOcean in pairs(hOceans) do
			if bOcean > vOcean then
				OceanCoastCanalConnect(vOcean, bOcean, change)
				if iFeitoriaPlayer then
					if Players[iFeitoriaPlayer]:IsHuman() then
						if GetSizeOfOcean(vOcean) >= 50 then
							if GetSizeOfOcean(bOcean) >= 50 then
								LuaEvents.ITG_AchievementCompleted(135)
							end
						end
					end
				end
			end
		end
	end
	for i, vOcean in pairs(hCoasts) do
		for c, bOcean in pairs(hCoasts) do
			if bOcean > vOcean then
				BasicCoastCanalConnect(vOcean, bOcean, change)
			end
		end
	end
end

function InitializePossibleInitCanals()
	for i = 0, Map.GetNumPlots() - 1 do
		local fPlot = Map.GetPlotByIndex(i)
		if fPlot:GetAllowsSwimLand() > 0 then
			DoPointPossibleCanalConnection(fPlot, 1)
		end
	end
end