local LawsDataT = {}
local LawsTypes = {}
for row in DB.Query("SELECT * FROM ITGLaws") do
	local index = row.ID;
	LawsTypes[row.Type] = index;
	LawsDataT[index] = {}
	for a, b in pairs(row) do
		LawsDataT[index][a] = b
	end
end
for row in DB.Query("SELECT * FROM ITG_CultureRestrictedLaws") do
	local lawID = LawsTypes[row.LawType];
	local cultID = ITG_CultureTypes[row.CultureType]
	if row.Inverse then
		if not LawsDataT[lawID].CultureOnly then
			LawsDataT[lawID].CultureOnly = {};
		end
		LawsDataT[lawID].CultureOnly[cultID] = true;
	else
		if not LawsDataT[lawID].CultureRestricted then
			LawsDataT[lawID].CultureRestricted = {};
		end
		LawsDataT[lawID].CultureRestricted[cultID] = true;
	end
end
 
function GetLawsDataTable()
	return LawsDataT;
end

function GetInfoFromLawT(lawID, type)
	return LawsDataT[lawID][type]
end

LuaEvents.ITGSG_LawData.Add(function(lawID)
	MapModData.ITGSG = LawsDataT[lawID];
end)

function CanPlayerHaveLaw(g, lawID)
	if (not LawsDataT[lawID].Disabled) then
		if ((not LawsDataT[lawID].OfficialReligion) or (gT.Players[g].OfficialReligion)) and ((not LawsDataT[lawID].CultureRestricted) or (not LawsDataT[lawID].CultureRestricted[gT.Players[g].CultureType])) then
			if (not LawsDataT[lawID].CultureOnly) or (LawsDataT[lawID].CultureOnly[gT.Players[g].CultureType]) then
				return true;
			end
		end
	end
	return false;
end

LuaEvents.ITGSG_RequestAvailableLaws.Add(function(iPlayer)
	MapModData.ITGSG = GetAdjustableLaws(iPlayer);
end)

function GetAdjustableLaws(iPlayer)
	local pAvailableLaws = {}
	for i, b in pairs(LawsDataT) do
		if CanPlayerHaveLaw(iPlayer, i) then
			table.insert(pAvailableLaws, i);
		end
	end
	return pAvailableLaws;
end

function GetCivInfoLawsData(iPlayer)
	local tab = {}
	for i, b in pairs(LawsDataT) do
		if CanPlayerHaveLaw(iPlayer, i) then
			tab[i] = gT.Players[iPlayer].Laws[i][1]
		end
	end
	return tab
end

function PrepareLawsForEraConversion(currentEra, g)
	for i, data in pairs(LawsDataT) do
		if CanPlayerHaveLaw(g, i) then
			tempconvinit[g].Laws[i] = gT.Players[g].Laws[i][1]
		end
	end
end

function CanAffordLawPlayer(iPlayer, lawID)
	if gT.Players[iPlayer].FreeLaws > 0 then
		return true;
	end
	return gT.Players[iPlayer].LegalPower >= 1;
end

function DoRemoveLawsBasedOnOfficialReligion(iPlayer)
	if not gT.Players[iPlayer].Laws then
		return;
	end
	for lawID, data in pairs(LawsDataT) do
		if LawsDataT[lawID].OfficialReligion then
			if gT.Players[iPlayer].Laws[lawID][1] ~= 0 then
				OnLawChanged(iPlayer, lawID, 0, true)	
			end
		end
	end
end