include( "InstanceManager" );
include( "IconSupport" ); 
include( "ITGSG_HelpFunctions.lua" );
include( "FLuaVector" );

local g_GraphLegendInstanceManager = InstanceManager:new("GraphLegendInstance", "GraphLegend", Controls.GraphLegendStack);
local g_LineSegmentInstanceManager = InstanceManager:new("GraphLineInstance","LineSegment", Controls.GraphCanvas);
local g_TopPanelSelection = InstanceManager:new( "TopPanelGraphInstance", "Box", Controls.GraphsInfoStack );

local graphIndexes = {};
local scIndexes = {};
local gTabOfGraphs = {};
local cSelected = -1;
local gTopGraphsIcons = {};
local currentID = 1
local grphData = {}
local gCaNodes = {}
local gGTdata;
local graphXsize = Controls.GraphCanvas:GetSizeX();
local graphYsize = Controls.GraphCanvas:GetSizeY();
local jmpX = graphXsize / 100;
local PlayerGraphColors = {}
local SegmentsByPlayer = {}
local GraphLegendsByPlayer = {}
local tADBC = {"0AD", "0AD", "OAD", "OAD", "0AD", "0AD"}

LuaEvents.ITG_ClearInstancesEndEra.Add(function()
	gTopGraphsIcons = nil;
	g_TopPanelSelection:ResetInstances();
	g_GraphLegendInstanceManager:ResetInstances();
	g_LineSegmentInstanceManager:ResetInstances();
end)

function GetBasicGraphTab(gh, gType, gPlayer)
	local cVal = 0;
	local hTab = {}
	local prepTab = {}
	for iTurn, val in pairs(Players[gPlayer]:GetGraphData(gType)) do
		table.insert(prepTab, {iTurn, val})
	end
	table.sort(prepTab, SortFunctionForLowestInteger1)
	for a, b in pairs(prepTab) do
		cVal = cVal + 1;
		hTab[b[1]] = cVal;
	end
	return cVal, hTab;
end

function GetJumpyGraphTab(gh, gType, gPlayer)
	local cVal = 0;
	local hTab = {}
	local prepTab = {}
	for iTurn, val in pairs(Players[gPlayer]:GetGraphData(gType)) do
		table.insert(prepTab, {iTurn, val})
	end
	table.sort(prepTab, SortFunctionForLowestInteger1)
	for a, b in pairs(prepTab) do
		cVal = math.max(cVal, b[2]);
		hTab[b[1]] = cVal;
	end
	return cVal, hTab;
end

function GenerateStairGraph(gfunc, iVal1)
	grphData = {}
	local minV, maxV = 0, 0;
	grphData.LabelHorizontal = tADBC;
	grphData.Graphs = {}
	for g, gh in pairs(gGTdata.Players) do
		if not Players[g]:IsMinorCiv() then
			grphData.Graphs[g] = {Data = {}, Color = PlayerGraphColors[g], Player = true}
			local cVal, hTab = gfunc(gh, iVal1, g);
			for t, cc in pairs(hTab) do
				table.insert(grphData.Graphs[g].Data, {t, cc})
			end
			table.sort(grphData.Graphs[g].Data, SortFunctionForLowestInteger1)
			maxV = math.max(maxV, cVal);
		end
	end
	maxV = minV + (math.ceil((maxV - minV) / 4)) * 4;
	grphData.Min = minV;
	grphData.Max = math.max(maxV, minV + 4);
	grphData.BaseV = 0;
	grphData.LabelVertical = {minV, (maxV - minV) / 4, (maxV - minV) / 2,  3 * (maxV - minV) / 4, maxV}
end

function GenerateEachTurnGraph(rID)
	grphData = {}
	local minV, maxV = gTabOfGraphs[rID].MinVal, gTabOfGraphs[rID].MaxVal;
	grphData.LabelHorizontal = tADBC;
	grphData.Graphs = {}
	local bAcceptNils = gTabOfGraphs[rID].CanBeNil
	for g, gh in pairs(gGTdata.Players) do
		if not Players[g]:IsMinorCiv() then
			grphData.Graphs[g] = {Data = {}, Color = PlayerGraphColors[g], Player = true}
			local prepTab = {}
			for a, b in pairs(Players[g]:GetGraphData(rID)) do
				prepTab[a] = b;
			end
			for it = 0, 100 do
				local hVal = prepTab[it];
				if (not hVal) and (not bAcceptNils) then
					hVal = 0;
				end
				if hVal then
					table.insert(grphData.Graphs[g].Data, {it, hVal})
					if not minV then
						minV = hVal;
					else
						minV = math.min(minV, hVal)
					end
					if not maxV then
						maxV = hVal;
					else							
						maxV = math.max(maxV, hVal)
					end
				end
			end
		end
	end
	local scl = gTabOfGraphs[rID].Scale / 100;
	grphData.BaseV = 0;
	if not maxV then
		grphData.Min = 0;
		grphData.Max = 0;
		grphData.LabelVertical = {scl * -1, scl * -0.5, scl * 0,  scl * 0.5, scl * 1}
	else
		maxV = minV + (math.ceil((maxV - minV) / 4)) * 4;
		grphData.Min = minV;
		grphData.Max = math.max(maxV, minV + 4);
		grphData.LabelVertical = {scl * minV, scl * ((maxV - minV) / 4 + minV), scl * ((maxV - minV) / 2 + minV),  scl * (3 * (maxV - minV) / 4 + minV), scl * maxV}
	end
	if gTabOfGraphs[rID].ShowLine and (grphData.LabelVertical[5] > gTabOfGraphs[rID].ShowLine) and (grphData.LabelVertical[1] < gTabOfGraphs[rID].ShowLine) then
		grphData.ZeroY = (grphData.LabelVertical[1] - gTabOfGraphs[rID].ShowLine) / (grphData.LabelVertical[5] - grphData.LabelVertical[1])
	end
end

function GetDiploNotesGraphTab(gh) -- for ID 0
	local cVal = 0;
	local hTab = {}
	local numLog = #gh.GovDiploLog;
	for i = 1, numLog do
		local ji = numLog -i + 1;
		if gh.GovDiploLog[ji].Action then
			cVal = cVal + 1;
			hTab[gh.GovDiploLog[ji].Turn] = cVal;
		end
	end
	return cVal, hTab;
end

for row in DB.Query("SELECT * FROM ITG_Graphs") do
	local index = row.ID;
	gTabOfGraphs[index] = {}
	graphIndexes[row.Type] = index;
	for a, b in pairs(row) do
		gTabOfGraphs[index][a] = b
	end
end

for row in DB.Query("SELECT ID, Type FROM ITG_Scenarios") do
	scIndexes[row.Type] = row.ID;
end

for row in DB.Query("SELECT * FROM ITG_Graphs_Scenarios") do
	local index = graphIndexes[row.GraphType]
	if row.Exclude then
		if not gTabOfGraphs[index].NoScenario then
			gTabOfGraphs[index].NoScenario = {};
		end
		gTabOfGraphs[index].NoScenario[scIndexes[row.ScenarioType]] = true;
	else
		if not gTabOfGraphs[index].Scenario then
			gTabOfGraphs[index].Scenario = {};
		end
		gTabOfGraphs[index].Scenario[scIndexes[row.ScenarioType]] = true;
	end
end

function GenerateGraphDrawData(selectedID)
	if gTabOfGraphs[selectedID].DrawType == 1 then
		GenerateStairGraph(GetBasicGraphTab, selectedID)
	elseif gTabOfGraphs[selectedID].DrawType == 2 then
		GenerateEachTurnGraph(selectedID)
	elseif gTabOfGraphs[selectedID].DrawType == 3 then
		GenerateStairGraph(GetJumpyGraphTab, selectedID)
	elseif gTabOfGraphs[selectedID].DrawType == 4 then
		if selectedID == 0 then
			GenerateStairGraph(GetDiploNotesGraphTab) -- based on govdiploaction log instead dll
		end
	end
end

function DrawALine(x1, y1, x2, y2, bPlayer, igraph, Color, bVisible)
	local lineSegment = g_LineSegmentInstanceManager:GetInstance();
	lineSegment.LineSegment:SetStartVal(x1, graphYsize - y1);
	lineSegment.LineSegment:SetEndVal(x2, graphYsize - y2);
	lineSegment.LineSegment:SetColorVal(Color.Red, Color.Green, Color.Blue, Color.Alpha);
	lineSegment.LineSegment:SetHide(not bVisible);	
	if bPlayer then
		table.insert(SegmentsByPlayer[igraph], lineSegment)
	end
end

function DoUpdateGraphData()
	if not gGTdata then	
		LuaEvents.ITGSG_RequestGT();
		gGTdata = MapModData.ITGSG;
		MapModData.ITGSG = nil;
		if not gGTdata then
			print("Unthinkable Error 63424123")
			return
		end
		PrepareGraphsForCurrentEra(gGTdata.Globals.ScenarioType)
	end
	if cSelected ~= -1 then
		gTopGraphsIcons[cSelected].TopClick:SetAlpha(0.6);
		gTopGraphsIcons[cSelected].ImageFrame:SetHide(true)
	end
	cSelected = gCaNodes[currentID];
	GenerateGraphDrawData(cSelected)
	g_LineSegmentInstanceManager:ResetInstances();
	SegmentsByPlayer = {}
	for i, v in pairs(grphData.LabelHorizontal) do
		Controls["HorizontalLabel" .. i]:SetText(v)
	end
	for i, v in pairs(grphData.LabelVertical) do
		Controls["VerticalLabel" .. i]:SetText(v)
	end
	local bStairs = gTabOfGraphs[gCaNodes[currentID]].Stairs
	local bIgnoreFirstStep = gTabOfGraphs[gCaNodes[currentID]].IgnoreFirstStep
	if grphData.ZeroY then
		Controls.ZeroLine:SetOffsetY(360 * grphData.ZeroY)
		Controls.ZeroLine:SetHide(false)
	else
		Controls.ZeroLine:SetHide(true)
	end

	for igraph, grd in pairs(grphData.Graphs) do
		-- let's draw!
		local bVisible = true;
		if grd.Player then
			bVisible = GraphLegendsByPlayer[igraph].ShowHide:IsChecked();
			SegmentsByPlayer[igraph] = {}
		end
		local startX, newX, startY, newY = 0, 0, math.floor((grphData.BaseV - grphData.Min) / (grphData.Max - grphData.Min) * graphYsize), 0;
		local lTurn = 0;
		local bFirst = true;
		for a, d in pairs(grd.Data) do
			if bFirst then
				startX = jmpX * d[1];
				if bStairs and (d[2] ~= grphData.BaseV) and (not bIgnoreFirstStep) then
					newY = math.floor((d[2] - grphData.Min) / (grphData.Max - grphData.Min) * graphYsize)
					DrawALine(startX, startY, startX, newY, grd.Player, igraph, grd.Color, bVisible)
					startY = newY;
				else
					startY = math.floor((d[2] - grphData.Min) / (grphData.Max - grphData.Min) * graphYsize)
				end
				lTurn = d[1];
				bFirst = false;
			else
				local bDraw = true;
				if bStairs then
					newX = jmpX * d[1];
					DrawALine(startX, startY, newX, startY, grd.Player, igraph, grd.Color, bVisible)
					startX = newX;
				elseif d[1] ~= lTurn + 1 then
					bFirst = true; -- let's break drawing;
					bDraw = false;
				end
				if bDraw then
					newX = jmpX * d[1];
					newY = math.floor((d[2] - grphData.Min) / (grphData.Max - grphData.Min) * graphYsize)
					DrawALine(startX, startY, newX, newY, grd.Player, igraph, grd.Color, bVisible)
					startX = newX;
					startY = newY;
					lTurn = d[1];
				end
			end
		end
		if lTurn ~= 100 then
			if (not bFirst) and bStairs then
				newX = jmpX * 100;
				DrawALine(startX, startY, newX, startY, grd.Player, igraph, grd.Color, bVisible)
				startX = newX;
			end
		end
	end
	Controls.GraphTitle:SetText(Locale.Lookup(gTabOfGraphs[gCaNodes[currentID]].Description));
	if cSelected ~= -1 then
		gTopGraphsIcons[cSelected].TopClick:SetAlpha(1.0);
		gTopGraphsIcons[cSelected].ImageFrame:SetHide(false)
	end
end

function OnNextGraphData()
	if gCaNodes[currentID + 1] then
		currentID = currentID + 1;
		DoUpdateGraphData()
	end	
end
Controls.NextGraphButton:RegisterCallback( Mouse.eLClick, OnNextGraphData );

function OnPrevGraphData()
	if gCaNodes[currentID - 1] then
		currentID = currentID - 1;
		DoUpdateGraphData()
	end	
end
Controls.PrevGraphButton:RegisterCallback( Mouse.eLClick, OnPrevGraphData );
-------------------------------------------------------------------------------

--[[ this is about graph now drawing
function GetWhosWinningDataOfType(id)

	rData = {}
	gTabOfLists[id].Dfunc();
	aMadeT[id] = rData;
	return rData;
end ]]

function GetGraphIDFromId(id)
	for i, b in pairs(gCaNodes) do
		if id == b then
			return i;
		end
	end
	print("Unthinkable 1256324c")
end

function OnTopGraphClicked(id)
	if gCaNodes[currentID] ~= id then
		currentID = GetGraphIDFromId(id);
		DoUpdateGraphData()
	end
end

function ShouldGraphNodeBeVisible(i, gType)
	if gTabOfGraphs[i].Scenario and (not gTabOfGraphs[i].Scenario[gType]) then
		return false;
	end
	if gTabOfGraphs[i].NoScenario and (gTabOfGraphs[i].NoScenario[gType]) then
		return false;
	end
	return true;
end

function DoChangeAllGraphs(bCheck)
	for g, segm in pairs(SegmentsByPlayer) do
		for i,v in ipairs(segm) do
			v.LineSegment:SetHide(not bCheck);
		end
	end
	for g, inst in pairs(GraphLegendsByPlayer) do
		inst.ShowHide:SetCheck(bCheck);
	end
end

function PrepareGraphsForCurrentEra(gType)
	gCaNodes = {}
	g_TopPanelSelection:ResetInstances()
	cSelected = -1;
	gTopGraphsIcons = {}
	for i, v in pairs(gTabOfGraphs) do
		if ShouldGraphNodeBeVisible(i, gType) then
			local controlTable = g_TopPanelSelection:GetInstance();
			controlTable.TopClick:SetText(v.IconString)
			controlTable.TopClick:SetVoid1(i)
			controlTable.TopClick:RegisterCallback( Mouse.eLClick, OnTopGraphClicked );
			controlTable.TopClick:SetToolTipString(Locale.Lookup(v.Help))
			controlTable.TopClick:SetAlpha(0.6);
			controlTable.ImageFrame:SetHide(true)
			IconHookup( 62, 32, "ABILITY_ATLAS", controlTable.ImageFrame );
			gTopGraphsIcons[i] = controlTable;

			table.insert(gCaNodes, i)
		end
	end
	--legend colours
	tADBC[1] = GetTextDate(0, gGTdata.Globals.StartDate, true);
	tADBC[2] = GetTextDate(20, gGTdata.Globals.StartDate, true);
	tADBC[3] = GetTextDate(40, gGTdata.Globals.StartDate, true);
	tADBC[4] = GetTextDate(60, gGTdata.Globals.StartDate, true);
	tADBC[5] = GetTextDate(80, gGTdata.Globals.StartDate, true);
	tADBC[6] = GetTextDate(100, gGTdata.Globals.StartDate, true);
	g_GraphLegendInstanceManager:ResetInstances();
	for g, gd in pairs(gGTdata.Players) do
		if not Players[g]:IsMinorCiv() then
					
			local civ = GameInfo.Civilizations[Players[g]:GetCivilizationType()];
			local graphLegendInstance = g_GraphLegendInstanceManager:GetInstance();
					
			IconHookup( civ.PortraitIndex, 32, civ.IconAtlas, graphLegendInstance.LegendIcon );
					
			local color = DetermineGraphColor(GameInfo.PlayerColors[Players[g]:GetPlayerColor()]);
			PlayerGraphColors[g] = color;
									
			graphLegendInstance.LegendLine:SetColorVal(color.Red, color.Green, color.Blue, color.Alpha);			
			graphLegendInstance.LegendName:LocalizeAndSetText(Players[g]:GetName());

					
			local checked = true;
			graphLegendInstance.ShowHide:SetCheck(checked);
					
			graphLegendInstance.ShowHide:RegisterCheckHandler( function(bCheck)
				local segments = SegmentsByPlayer[g];
				if(segments) then
					for i,v in ipairs(segments) do
						v.LineSegment:SetHide(not bCheck);
					end
				end
			end);

					
			GraphLegendsByPlayer[g] = graphLegendInstance;
		end
	end
--All
	local graphLegendInstance = g_GraphLegendInstanceManager:GetInstance();					
	graphLegendInstance.LegendLine:SetHide(true);			
	graphLegendInstance.LegendIcon:SetHide(true);			
	graphLegendInstance.LegendName:LocalizeAndSetText("Change All");
	graphLegendInstance.ShowHide:SetCheck(true);
	graphLegendInstance.ShowHide:RegisterCheckHandler( function(bCheck)
		DoChangeAllGraphs(bCheck)
	end);


	Controls.GraphLegendStack:CalculateSize();
	Controls.GraphLegendStack:ReprocessAnchoring();
	Controls.GraphLegendScrollPanel:CalculateInternalSize();
end


LuaEvents.ITGLHInitAll.Add(function()
	gGTdata = nil;
end)

Controls.GraphsInfoStack:CalculateSize();
Controls.GraphsInfoStack:ReprocessAnchoring();
Controls.GraphsScrollPanel:CalculateInternalSize()

function ShowHideHandler( bIsHide, bInitState )
    if( not bIsHide ) then
    	DoUpdateGraphData();
	end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );

local colorDistances = {};
local pow = math.pow;
function ColorDistance(color1, color2)
	local distance = nil;
	local colorDistanceTable = colorDistances[color1.Type];
	if(colorDistanceTable == nil) then
		colorDistanceTable = {
			[color1.Type] = 0;	--We can assume the distance of a color to itself is 0.
		};
						
		colorDistances[color1.Type] = colorDistanceTable
	end
					
	local distance = colorDistanceTable[color2.Type];
	if(distance == nil) then
		local r2 = pow((color1.Red - color2.Red)*255, 2);
		local g2 = pow((color1.Green - color2.Green)*255, 2);
		local b2 = pow((color1.Blue - color2.Blue)*255, 2);	
						
		distance = r2 + g2 + b2;
		colorDistanceTable[color2.Type] = distance;
	end
					
	return distance;
end
				
local colorNotUnique = {};
local colorBlack = {Type = "COLOR_BLACK", Red = 0, Green = 0, Blue = 0, Alpha = 1,};
function IsUniqueColor(color)
	if(colorNotUnique[color.Type]) then
		return false;
	end
					
	local blackThreshold = 5000;
	local differenceThreshold = 3000;
					
	local distanceAgainstBlack = ColorDistance(color, colorBlack);
	if(distanceAgainstBlack > blackThreshold) then
		for i, v in pairs(PlayerGraphColors) do
			local distanceAgainstOtherPlayer = ColorDistance(color, v);
			if(distanceAgainstOtherPlayer < differenceThreshold) then
				colorNotUnique[color.Type] = true;
				return false;
			end
		end
						
		return true;
	end
					
	colorNotUnique[color.Type] = true;
	return false;
end

function DetermineGraphColor(playerColor)
	if(IsUniqueColor(GameInfo.Colors[playerColor.PrimaryColor])) then
		return GameInfo.Colors[playerColor.PrimaryColor];
	elseif(IsUniqueColor(GameInfo.Colors[playerColor.SecondaryColor])) then
		return GameInfo.Colors[playerColor.SecondaryColor];
	else
		for color in GameInfo.Colors() do
			if color.Alpha == 1 then
				if(IsUniqueColor(color)) then
					return color;
				end	
			end
		end
	end
										
	--error("Could not find a unique color");
	if (ColorDistance(GameInfo.Colors[playerColor.PrimaryColor], colorBlack) > 5000) then
		return GameInfo.Colors[playerColor.PrimaryColor];
	end
	return GameInfo.Colors[playerColor.SecondaryColor];
end