include( "InstanceManager" );
include( "IconSupport" ); 
include( "ITGSG_HelpFunctions.lua" );

local isCall = false;
local cSelectedItem = 6; -- opening at score right now!
local bGeneratedPS = false;
local g_SlotInstances = {};
local CultureTypesOfEraHC = {}
local TableOfIDCultureNames = {[-1] = "TXT_KEY_ITG_RANDOM_CULTURE"}

function GenerateDBDataNeededForNextScenario()
	CultureTypesOfEraHC = {}
	table.insert(CultureTypesOfEraHC, -1)
	local nextScType = MapModData.DBScenario.NextScenario;
	--sorted options
	local itgCultureTypes = {};
	local itgSubcultureTypes = {};
	for row in DB.Query("SELECT ID, Type, Description FROM ITG_Cultures") do
		TableOfIDCultureNames[row.ID] = row.Description;
		itgCultureTypes[row.Type] = row.ID;
	end
	for row in DB.Query("SELECT ID, Type, Description FROM ITG_Subcultures") do
		TableOfIDCultureNames[1000 + row.ID] = row.Description;
		itgSubcultureTypes[row.Type] = row.ID;
	end
	local tBsorted = {}
	for row in DB.Query("SELECT * FROM ITG_ScenarioCultures WHERE ScenarioType = '" .. nextScType .. "'") do
		table.insert(tBsorted, {itgCultureTypes[row.CultureType], Locale.Lookup(TableOfIDCultureNames[itgCultureTypes[row.CultureType]])})
		for sRow in DB.Query("SELECT * FROM ITG_CultureSubcultures WHERE CultureType = '" .. row.CultureType .. "'") do
			table.insert(tBsorted, {1000 + itgSubcultureTypes[sRow.SubcultureType], Locale.Lookup(TableOfIDCultureNames[1000 + itgSubcultureTypes[sRow.SubcultureType]])})
		end
	end
	table.sort(tBsorted, function(a, b) 
		return Locale.Compare(a[2], b[2]) == -1;
	end);
	for a, b in pairs(tBsorted) do
		table.insert(CultureTypesOfEraHC, b[1])
	end
end

function SetCultureTypePlayer(playerID, playerChoiceID)
	Modding.OpenSaveData().SetValue("ITG_pgd_PlayerCultureType" .. playerID, playerChoiceID)
	local slotInstance = g_SlotInstances[playerID];
	if( slotInstance ~= nil ) then
		slotInstance.TeamLabel:LocalizeAndSetText( TableOfIDCultureNames[playerChoiceID] );
		slotInstance.TeamLabel:SetToolTipString( Locale.Lookup(TableOfIDCultureNames[playerChoiceID] .. "_DOM") );
	else
		Controls.TeamLabel:LocalizeAndSetText( TableOfIDCultureNames[playerChoiceID] );
		Controls.TeamLabel:SetToolTipString( Locale.Lookup(TableOfIDCultureNames[playerChoiceID] .. "_DOM") );
	end
end

function CreateSlotInstances()
	if not bGeneratedPS then
		bGeneratedPS = true;
		LuaEvents.ITGSG_RequestGT();
		local gData = MapModData.ITGSG;
		MapModData.ITGSG = nil;
		GenerateDBDataNeededForNextScenario()

		for g, h in pairs(gData.Players) do
			if (not Players[g]:IsMinorCiv()) then
				local civ = GameInfo.Civilizations[Players[g]:GetCivilizationType()];
				local leader = GameInfo.Leaders[Players[g]:GetLeaderType()];
				if g == 0 then
					Controls.CivPulldown:SetHide(false);
					Controls.CivPulldown:GetButton():SetText( Locale.ConvertTextKey( "TXT_KEY_RANDOM_LEADER_CIV", Locale.ConvertTextKey( leader.Description ), Locale.ConvertTextKey( civ.ShortDescription ) ) );
					IconHookup( leader.PortraitIndex, 64, leader.IconAtlas, Controls.Portrait );
					IconHookup( civ.PortraitIndex, 64, civ.IconAtlas, Controls.Icon );
					Controls.CivPulldown:SetDisabled(true);
					--populate options
					--register callback
					local pullDown = Controls.TeamPullDown;
					pullDown:ClearEntries();
					for i, name in pairs(CultureTypesOfEraHC) do
						local nestedTable = {};
						pullDown:BuildEntry( "InstanceOne", nestedTable );
						nestedTable.Button:LocalizeAndSetText(TableOfIDCultureNames[name]);
						nestedTable.Button:SetToolTipString( Locale.Lookup(TableOfIDCultureNames[name] .. "_DOM") );
						nestedTable.Button:SetVoids( g, name );
					end    
					pullDown:CalculateInternals();
					pullDown:RegisterSelectionCallback(SetCultureTypePlayer);
					Controls.TeamLabel:LocalizeAndSetText( TableOfIDCultureNames[-1] );
					Controls.TeamLabel:SetToolTipString( Locale.Lookup(TableOfIDCultureNames[-1] .. "_DOM") );
				else
					local controlTable = {};
					ContextPtr:BuildInstanceForControl( "PlayerSlot", controlTable, Controls.SlotStack );
					controlTable.Root:SetHide( false );	
					controlTable.CivPulldown:GetButton():LocalizeAndSetText("TXT_KEY_RANDOM_LEADER_CIV", Locale.ConvertTextKey(leader.Description), Locale.ConvertTextKey(civ.ShortDescription));
					IconHookup( leader.PortraitIndex, 64, leader.IconAtlas, controlTable.Portrait );
					IconHookup( civ.PortraitIndex, 64, civ.IconAtlas, controlTable.Icon );
					controlTable.CivPulldown:SetDisabled(true);
					--populate options
					--register callback
					local pullDown = controlTable.TeamPullDown;
					pullDown:ClearEntries();
					for i, name in pairs(CultureTypesOfEraHC) do
						local nestedTable = {};
						pullDown:BuildEntry( "InstanceOne", nestedTable );
						nestedTable.Button:LocalizeAndSetText(TableOfIDCultureNames[name]);
						nestedTable.Button:SetToolTipString( Locale.Lookup(TableOfIDCultureNames[name] .. "_DOM") );
						nestedTable.Button:SetVoids( g, name );
					end    
					pullDown:CalculateInternals();
					pullDown:RegisterSelectionCallback(SetCultureTypePlayer);
					controlTable.TeamLabel:LocalizeAndSetText( TableOfIDCultureNames[-1] );
					controlTable.TeamLabel:SetToolTipString( Locale.Lookup(TableOfIDCultureNames[-1] .. "_DOM") );
					g_SlotInstances[g] = controlTable;
				end
			end
		end
		
		Controls.SlotStack:CalculateSize();
		Controls.SlotStack:ReprocessAnchoring();
		Controls.ListingScrollPanel:CalculateInternalSize();
	end
end

LuaEvents.ITG_IsEraPanelOn.Add(function()
	MapModData.ITGSG = isCall;
end)

LuaEvents.ITG_EndOfAllTheStuff.Add(function()
	isCall = true;
	UIManager:QueuePopup( ContextPtr, PopupPriority.WhosWinning );
	LuaEvents.ITG_CallSafeUIEndEra() -- new staff!
end)

function EditedTimeString(time)
	local intTime = math.floor(time)
	local hours = math.floor(intTime / 3600);
	intTime = intTime - 3600 * hours;
	local minutes = math.floor(intTime / 60);
	local seconds = intTime - minutes * 60;
	if seconds < 10 then
		seconds = "0" .. seconds;
	end
	if minutes < 10 then
		minutes = "0" .. minutes;
	end
	return hours .. "h " .. minutes .."m " .. seconds .. "s";
end

LuaEvents.ITG_CallSafeUIEndEra.Add(function()
	LuaEvents.ITGSG_RequestGT();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	LuaEvents.ITGLHInitAll(gData.Globals.ScenarioType)
	Controls.TimerLabel:SetText(EditedTimeString(gData.Globals.TotalTimer))
	local nextScenarioType = DB.Query("SELECT NextScenario FROM ITG_Scenarios WHERE ID = " .. gData.Globals.ScenarioType)().NextScenario
	if nextScenarioType then
		Controls.IntoNextOne:SetText(string.upper(Locale.Lookup(DB.Query("SELECT Intro FROM ITG_Scenarios WHERE Type = '" .. nextScenarioType .. "'")().Intro)))
		Controls.IntoNextOne:SetHide(false)
	else
		Controls.IntoNextOne:SetHide(true)
	end

	ContextPtr:SetHide(false)
	UIManager:SetUICursor(0);
--	UI.SetInterfaceMode(2) -- this stuff is changing mouse from "waiting" to normal
--	UI.SetInterfaceMode(1)
	Controls.GraphsInfoStack:SetHide(false)
	TabSelect(cSelectedItem)
end)

function OnRealClose()
	UIManager:DequeuePopup( ContextPtr );
	ContextPtr:SetHide(true)
end
LuaEvents.ITG_HideLeaderHead(OnRealClose)

function OnClose ()
	if isCall then
		isCall = false;
		LuaEvents.ITG_EndEraPanelClosing()
		Game.SetITGPaused(false)
		OnRealClose()
	end
end
Controls.CloseButton:RegisterCallback( Mouse.eLClick, OnClose );

function EraConversionCall()
	LuaEvents.ITG_ClearInstancesEndEra()
	Controls.GraphsInfoStack:SetHide(true)
	Controls.PanelNewEra:SetHide(true);
	LuaEvents.ITGSG_DoEraConversion()
	LuaEvents.ITG_QuickReloadDelay()
end

function OnConversionError(string)
	print(string);
	LuaEvents.ITG_RequestToConvertErrConv(string)
	return false;
end

function LetsGoButton()
	if isCall then
		isCall = false;
		xpcall(EraConversionCall, OnConversionError)
	end
end
Controls.LetsGoButton:RegisterCallback( Mouse.eLClick, LetsGoButton );

function IntoNextOne ()
	CreateSlotInstances()
	Controls.PanelMain:SetHide(true)
	Controls.PanelNewEra:SetHide(false);
end
Controls.IntoNextOne:RegisterCallback( Mouse.eLClick, IntoNextOne );

function ReturnToMenuButton ()
	Controls.PanelMain:SetHide(false)
	Controls.PanelNewEra:SetHide(true);
end
Controls.ReturnToMenuButton:RegisterCallback( Mouse.eLClick, ReturnToMenuButton );

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function InputHandler( uiMsg, wParam, lParam )
    if uiMsg == KeyEvents.KeyDown then
        if wParam == Keys.VK_ESCAPE or wParam == Keys.VK_RETURN then
   --         OnClose();
            return true;
        end
    end
end
ContextPtr:SetInputHandler( InputHandler );

function OnSave()
    UIManager:QueuePopup( ContextPtr:LookUpControl("/InGame/GameMenu/SaveMenu"), PopupPriority.SaveMenu );
end
Controls.SaveGameButton:RegisterCallback( Mouse.eLClick, OnSave );

function OnExitGame()
 --   Events.UserRequestClose();
	UI.ExitGame();
end
Controls.ExitGameButton:RegisterCallback( Mouse.eLClick, OnExitGame );

function OnMainMenu()
	Events.ExitToMainMenu();
end
Controls.MainMenuButton:RegisterCallback( Mouse.eLClick, OnMainMenu );

function TabSelect(id)
	Controls.ListsSelectHighlight:SetHide(id ~= 1)
	Controls.GraphsSelectHighlight:SetHide(id ~= 2)
	Controls.ContinueSelectHighlight:SetHide(id ~= 3)
	Controls.NoteSelectHighlight:SetHide(id ~= 4)
	Controls.GlobalsSelectHighlight:SetHide(id ~= 7)
	Controls.CivInfoSelectHighlight:SetHide(id ~= 8)
	Controls.CreditsSelectHighlight:SetHide(id ~= 5)
	Controls.ScoresSelectHighlight:SetHide(id ~= 6)
	Controls.LeaderHeadListsInfo:SetHide(id ~= 1)
	Controls.LeaderHeadGraphsInfo:SetHide(id ~= 2)
	Controls.PanelMain:SetHide(id ~= 3)
	Controls.PanelNewEra:SetHide(true);
	Controls.LeaderHeadNoteInfo:SetHide(id ~= 4)
	Controls.LeaderHeadCredutsInfo:SetHide(id ~= 5)
	Controls.LeaderHeadScoreInfo:SetHide(id ~= 6)
	Controls.LeaderHeadGlobalInfo:SetHide(id ~= 7)
	Controls.LeaderHeadCivInfo:SetHide(id ~= 8)
	cSelectedItem = tab;
end
Controls.TabButtonLists:RegisterCallback( Mouse.eLClick, function() TabSelect(1); end );
Controls.TabButtonGraphs:RegisterCallback( Mouse.eLClick, function() TabSelect(2); end );
Controls.TabButtonContinue:RegisterCallback( Mouse.eLClick, function() TabSelect(3); end );
Controls.TabButtonNote:RegisterCallback( Mouse.eLClick, function() TabSelect(4); end );
Controls.TabButtonCredits:RegisterCallback( Mouse.eLClick, function() TabSelect(5); end );
Controls.TabButtonScore:RegisterCallback( Mouse.eLClick, function() TabSelect(6); end );
Controls.TabButtonGlobals:RegisterCallback( Mouse.eLClick, function() TabSelect(7); end );
Controls.TabButtonCivInfo:RegisterCallback( Mouse.eLClick, function() TabSelect(8); end );
Events.GameplaySetActivePlayer.Add(OnRealClose);

LuaEvents.ITG_EraConversionError.Add(function(string)
	Controls.DescErrLabel:SetText("Please report following error:[NEWLINE]" .. string)
	Controls.ErrorGrid:SetSizeY(math.max(270, 160 + Controls.DescErrLabel:GetSizeY()));
	Controls.ErrorBox:SetHide(false)
end)

function OnErrMainMenuButtonClicked()
	Events.ExitToMainMenu();
end
Controls.ErrMainMenuButton:RegisterCallback( Mouse.eLClick, OnErrMainMenuButtonClicked );

function OnExitGameButtonClicked()
	UI.ExitGame();
end
Controls.ErrExitButton:RegisterCallback( Mouse.eLClick, OnExitGameButtonClicked );