local fAtoll = GameInfoTypes.FEATURE_ATOLL;
local fIce = GameInfoTypes.FEATURE_ICE;
local tTundra = GameInfoTypes.TERRAIN_TUNDRA;
local HelpFeatureTab = {fForest, fJungle}
local ResLand;
local ResWater;
local ResLuxCount;
local atLLtab = {}

function GetResAmountDB(res)
	local tab = {}
	local hSt = GameInfo.Resources[res].Type;
	for row in GameInfo.Resource_QuantityTypes() do
		if row.ResourceType == hSt then
			table.insert(tab, row.Quantity);
		end 
	end
	if #tab > 0 then
		return tab[LastRandom(1,#tab)];
	end
	return nil
end

function DoResetCachedNumResources()
	ResLand = nil;
	ResWater = nil;
	ResLuxCount = nil;
end

function DoQuickResTabsForMapScripts()
	if not ResLand then
		local allresources = GetResourcesPresentOnMap();
		ResLand = {}
		ResWater = {}
		ResLuxCount = {};
		for res, plots in pairs(allresources) do
			if not GameInfo.Resources[res].Migration then
				for dindex, sth in pairs(plots) do
					local dplot = Map.GetPlotByIndex(dindex)
					if dplot:IsWater() then
						table.insert(ResWater, res)
					else
						table.insert(ResLand, res)
					end
					break;
				end
				if GameInfo.Resources[res].Happiness > 0 then
					ResLuxCount[res] = 0;
					for dindex, sth in pairs(plots) do
						ResLuxCount[res] = ResLuxCount[res] + 1;
					end
				end
			end
		end
	end
end

function GenerateBestResourceMaintainingBalance(iPlot, bGoody, bTemp, bAtoll)
	DoQuickResTabsForMapScripts();
	local rTab = ResLand;
	if iPlot:IsWater() then
		rTab = ResWater;
	end
	local finalRes = -1;
	local iValidLuxuries = 0;
	local bestLuxury = -1;
	local otherGood = {}
	for i, vRes in pairs(rTab) do
		if iPlot:CanHaveResource(vRes, true) then
			if (not bGoody) or (IsGoodyResource(vRes)) then
				if ResLuxCount[vRes] then
					iValidLuxuries = iValidLuxuries + 1;
					if bestLuxury == -1 or (ResLuxCount[vRes] < ResLuxCount[bestLuxury]) or (ResLuxCount[vRes] == ResLuxCount[bestLuxury] and LastRandom(1,2)) then
						bestLuxury = vRes;
					end
				else
					table.insert(otherGood, vRes)
				end
			end
		end
	end
	if iValidLuxuries + #otherGood > 0 then
		if LastRandom(1, iValidLuxuries + #otherGood) <= iValidLuxuries then
			finalRes = bestLuxury;
			ResLuxCount[bestLuxury] = ResLuxCount[bestLuxury] + 1;
		else
			finalRes = otherGood[LastRandom(1,#otherGood)];
		end
	elseif bAtoll then
		table.insert(atLLtab, iPlot:GetPlotIndex())
	else
		print("Literally no possible option for extra res?", iPlot:GetPlotIndex());
	end
	if finalRes ~= -1 then
		if bTemp then
			AddProperlyResourceOnPlot(iPlot, finalRes, true)
		else
			AddProperlyResourceOnPlot(iPlot, finalRes, false)
		end
	end
end

function GetNaturalWonderPlot(fType)
	for i = 0, Map:GetNumPlots() - 1, 1 do
		local fPlot = Map.GetPlotByIndex( i );
		if fPlot:GetFeatureType() == fType then
			return fPlot;
		end
	end
	return nil;
end

function BasicIslandMaker(iPlot, tChances, bAddBadPlot)
	local lPlots = {};
	for a, plot in pairs(GetPlotsInRadiusOfPlot(iPlot, 3)) do
		local dis = Map.PlotDistance(plot:GetX(), plot:GetY(), iPlot:GetX(), iPlot:GetY())
		if LastChance(tChances[dis] or 0) then
			if plot:IsWater() and (not plot:IsLake()) and (plot:GetFeatureType() == -1) then
				plot:SetResourceType(-1)
				plot:SetNumResource(0)

				table.insert(lPlots, plot)
				GenerateRandomIslandPlot(plot, bAddBadPlot)
			end
		end
	end
	for i, plot in pairs(lPlots) do
		for j = 0, 5 do
			local jPlot = Map.PlotDirection(plot:GetX(), plot:GetY(), j);
			if jPlot:GetTerrainType() == tOcean then
				GenerateRandomCoastPlot(jPlot);
			end
		end
	end
end

function DoMapModificationKrakatoa()
	local iPlot = GetNaturalWonderPlot(GameInfoTypes.FEATURE_VOLCANO)
	if iPlot then
		print("Krakatoa found - Map script")
		BasicIslandMaker(iPlot, {[2] = 80, [3] = 35}, false)
	end
	for i, v in pairs(Players) do
		if v:IsAlive() then
			if not v:IsBarbarian() then
				local sPlot = v:GetStartingPlot();
				if sPlot then
					if NeedsSomeInitLand(sPlot) then
						BasicIslandMaker(sPlot, {60,40,20}, false)
					end
				end
			end
		end
	end
end

function NeedsSomeInitLand(sPlot)
	local iH = 0;
	for i = 0, 5 do
		local nPlot = Map.PlotDirection(sPlot:GetX(), sPlot:GetY(), i);
		if nPlot and (nPlot:IsWater()) and (not nPlot:IsLake()) then
			iH = iH + 1;
		end
	end
	return LastRandom(5,6) <= iH;
end

function DoMapModificationOnStartNewWorld()
	GenerateTropicalIslands(true)
end

function GenerateTropicalIslands(bAddBadPlot)
	local grandeWaterDisToLand = {}
	local bEdge;
	local mx, my = Map.GetGridSize();
	if my < 28 then
		return;
	end
	DoQuickResTabsForMapScripts()
	for i = 0, Map:GetNumPlots() - 1, 1 do
		local fPlot = Map.GetPlotByIndex( i );
		if fPlot:IsWater() then
			bEdge = false;
			for j = 0, 5 do
				local jPlot = Map.PlotDirection(fPlot:GetX(), fPlot:GetY(), j)
				if not jPlot or not jPlot:IsWater() then
					bEdge = true;
					break;
				end
			end
			if bEdge then
				grandeWaterDisToLand[i] = 1;
			else
				grandeWaterDisToLand[i] = -1;
			end
		end
	end
	local bLoop = true;
	local cVal = 1;
	while bLoop do
		bLoop = false;
		for i, sth in pairs(grandeWaterDisToLand) do
			if sth == -1 then
				local fPlot = Map.GetPlotByIndex( i );
				local bNull = true;
				for j = 0, 5 do
					local jPlot = Map.PlotDirection(fPlot:GetX(), fPlot:GetY(), j)
					if grandeWaterDisToLand[jPlot:GetPlotIndex()] == cVal then
						grandeWaterDisToLand[i] = cVal + 1;
						bNull = false;
						break;
					end
				end
				if bNull then
					bLoop = true;
				end
			end
		end
		cVal = cVal + 1;
	end	
	-- we have nice dis :/ minimum is now 7
	local hTab = {}
	for i, sth in pairs(grandeWaterDisToLand) do
		if sth > 6 then
			table.insert(hTab, {i, sth})
		end
	end
	local numPlots7Away = #hTab;
	if numPlots7Away > 0 then
		table.sort(hTab, SortFunctionForGreaterInteger2)
		local alrDP = {}
		local numislands = 1 + math.floor(numPlots7Away / LastRandom(85, 115));  -- LastRandom(115, 145)
		--for i = 1, numislands do
		for i, vt in pairs(hTab) do
			local bCont = true;
			local fPlot = Map.GetPlotByIndex(vt[1])
			for i, plot in pairs(alrDP) do
				if Map.PlotDistance(fPlot:GetX(), fPlot:GetY(), plot:GetX(), plot:GetY()) < 7 then
					bCont = false;
					break;
				end
			end
			if bCont then
				GenerateTropicoIsland(fPlot, bAddBadPlot)
				table.insert(alrDP, fPlot)
				if #alrDP >= numislands then
					break;
				end
			end
		end
		if #alrDP > 0 then
			gT.Globals.RequireReload = true;
		end
	end
	if (PreGame.GetGameOption("GAMEOPTION_ITG_RESOURCE_BALANCE") == 1) and gT.EraData and gT.PreviousEra then
		DoQuickResTabsForMapScripts();
		local resMin = 100;
		for res, val in pairs(ResLuxCount) do
			resMin = math.min(resMin, val);
		end
		if #ResWater > 0 then
			for i, waterRes in pairs(ResWater) do
				if ResLuxCount[waterRes] then
					local tH = math.floor(ResLuxCount[waterRes] / 2);
					if tH >= resMin then
						print("Trying to split water resource into two", waterRes, tH)
						local allresources = GetResourcesPresentOnMap();
						if allresources[waterRes] then
							local plotTab = {}
							for b, c in pairs(allresources[waterRes]) do
								table.insert(plotTab, b)
							end
							if #plotTab ~= ResLuxCount[waterRes] then
								print("Inconsequential amount of res", waterRes, #plotTab, ResLuxCount[waterRes])
							end
							if #plotTab >= ResLuxCount[waterRes] then
								local tPlot = Map.GetPlotByIndex(plotTab[1])
								local cResTab = {}
								tPlot:SetResourceType(-1)
								for row in GameInfo.Resources() do
									if (not row.Migration) and (row.Happiness ~= 0) then
										if not allresources[row.ID] then
											if tPlot:CanHaveResource(row.ID) then
												table.insert(cResTab, row.ID);
											end
										end
									end
								end
								tPlot:SetResourceType(waterRes)
								if #cResTab > 0 then
									local lRes = cResTab[LastRandom(1,#cResTab)];
									print("Splitting into:", lRes);
									plotTab = Shuffle(plotTab);
									for kjk = 1, tH do
										AddProperlyResourceOnPlot(Map.GetPlotByIndex(plotTab[kjk]), lRes, false, false, 1)
									end
								end
							end
						else
							print("Weird 687/12395/34", waterRes);
						end
						break;
					end
				end
			end
		elseif #atLLtab >= resMin then
			print("Trying to find new water resource from atoll tab")
			local tPlot = Map.GetPlotByIndex(atLLtab[1])
			local cResTab = {}
			local allresources = GetResourcesPresentOnMap();
			for row in GameInfo.Resources() do
				if (not row.Migration) and (row.Happiness ~= 0) then
					if not allresources[row.ID] then
						if tPlot:CanHaveResource(row.ID) then
							table.insert(cResTab, row.ID);
						end
					end
				end
			end
			if #cResTab > 0 then
				local lRes = cResTab[LastRandom(1,#cResTab)];
				print("Found resource:", lRes)
				atLLtab = Shuffle(atLLtab);
				for i, v in pairs(atLLtab) do
					if i <= resMin then
						AddProperlyResourceOnPlot(Map.GetPlotByIndex(v), lRes, false, false, 1)
					else
						Map.GetPlotByIndex(v):SetFeatureType(fAtoll)
					end
				end
				atLLtab = {};
			end
		end
		DoResetResourcesPresentOnMap()
		DoResetCachedNumResources()
	end
	--add those juicy atolls
	for i, v in pairs(atLLtab) do
		Map.GetPlotByIndex(v):SetFeatureType(fAtoll);
	end
	atLLtab = {};
end

function GenerateTropicoIsland(tPlot, bAddBadPlot)
	local lPlots = {}
	if LastRandom(1, 100) <= 72 then
		table.insert(lPlots, tPlot)
		GenerateRandomIslandPlot(tPlot, bAddBadPlot)	
	end
	for h, j in pairs({50, 31, 15}) do
		for i, nPlot in pairs(GetPlotsInRadiusExactOfPlot(tPlot, h)) do
			if LastRandom(1, 100) <= j then
				table.insert(lPlots, nPlot)
				GenerateRandomIslandPlot(nPlot, bAddBadPlot)	
			end
		end
	end
	for i, plot in pairs(lPlots) do
		for j = 0, 5 do
			local jPlot = Map.PlotDirection(plot:GetX(), plot:GetY(), j);
			if jPlot:GetTerrainType() == tOcean then
				GenerateRandomCoastPlot(jPlot);
			end
		end
	end
end

function GenerateRandomCoastPlot(iPlot)
	iPlot:SetTerrainType(tCoast);
	if LastRandom(1, 100) < 7 then
		iPlot:SetFeatureType(fAtoll);
	elseif LastRandom(1, 100) < 7 then
		--greater chance because of filthy fishes and whales
		GenerateBestResourceMaintainingBalance(iPlot, nil, nil, true);
	end
end

function GenerateRandomIslandPlot(iPlot, bAddBadPlot)
	if bAddBadPlot then
		gT.Globals.BadPoints[iPlot:GetPlotIndex()] = {7, 0};
		--7 is special ID for tropical nations for their initial city
		--it blocks cities/holysites/forts/colonies on this plot
	end
	if LastRandom(1,2) == 2 then
		iPlot:SetTerrainType(tPlains);
	else
		iPlot:SetTerrainType(tGrass);
	end
	if LastRandom(1, 100) < 17 then
		iPlot:SetPlotType(PlotTypes.PLOT_MOUNTAIN);
	else
		gT.Globals.PlotAreas[iPlot:GetPlotIndex()] = "TI"; -- this makes it more attractive for settling
		if LastRandom(1,100) < 35 then
			iPlot:SetPlotType(PlotTypes.PLOT_HILLS);
		end
		if LastRandom(1, 100) < 40 then
			local hTab = {}
			for i, feature in pairs(HelpFeatureTab) do
				if iPlot:CanHaveFeature(feature) then
					table.insert(hTab, feature);
				end
			end
			if #hTab > 0 then
				iPlot:SetFeatureType(hTab[LastRandom(1,#hTab)])
			end
		end
		if LastRandom(1, 100) < 55 then
			GenerateBestResourceMaintainingBalance(iPlot);
		end
	end
end

local MCAREAS = {}
local MCtheT = {}
local MCARC = 0;
function DoStrokeOfCoast()
	gT.Globals.MedievalOceansIntoCoast = {}
	for i = 0, Map:GetNumPlots() - 1, 1 do
		local fPlot = Map.GetPlotByIndex( i );
		if fPlot:GetTerrainType() == tOcean then
			if IsAdjacentTerrain(fPlot:GetX(), fPlot:GetY(), tCoast) then
				gT.Globals.MedievalOceansIntoCoast[i] = 1;
			end
		end
	end
	for index, sth in pairs(gT.Globals.MedievalOceansIntoCoast) do
		Map.GetPlotByIndex(index):SetTerrainType(tCoast)
	end
--connect continents with major players :!
	for i = 0, Map:GetNumPlots() - 1, 1 do
		if not MCAREAS[i] then
			local fPlot = Map.GetPlotByIndex( i );
			if (not fPlot:IsWater()) or (fPlot:GetTerrainType() == tCoast) then
				MCARC = MCARC + 1;
				MCtheT[MCARC] = {}
				MedievalITPCoast(fPlot, MCARC)
			end
		end
	end
	MCAREAS = {};
	local popAr = {}
	local arC = {}
	for i, v in pairs(Players) do
		if (not v:IsMinorCiv()) and v:IsAlive() and (not v:IsBarbarian()) then
			local medCt = GetMedCT(v:GetStartingPlot())
			if not arC[medCt] then
				table.insert(popAr, medCt)
				arC[medCt] = true;
			end
		end
	end
	print("Majors distributed", #popAr)
	if #popAr > 1 then
		local pairsAndDis = {}
		for j = 1, #popAr - 1 do
			for i = j + 1, #popAr do
				table.insert(pairsAndDis, {popAr[j], popAr[i], GetMajorPlayerDisLivingOnConts(popAr[j], popAr[i])})
			end
		end
		local greatConnect = {}
		table.sort(pairsAndDis, SortFunctionForLowestInteger3)
--		table.sort(pairsAndDis, SortFunctionForGreaterInteger3)
		greatConnect[pairsAndDis[1][1]] = true;
		for i, sth in pairs(pairsAndDis) do
			if ((not greatConnect[sth[1]]) and greatConnect[sth[2]]) or ((not greatConnect[sth[2]]) and greatConnect[sth[1]])  then
				print("We shall connect those lands by the mere will.", sth[1], sth[2])
				local btt = {}
				for plotindex, dd in pairs(MCtheT[sth[1]]) do
					table.insert(btt, Map.GetPlotByIndex(plotindex))
				end
				local hTSP = {}
				for plotindex, dd in pairs(MCtheT[sth[2]]) do
					hTSP[plotindex] = true;
				end
				local rTab = GenerateShortestPath(btt, hTSP, nil, 35, nil, true, nil, {[tOcean] = true}, nil, nil)
				if rTab then
					greatConnect[sth[1]] = true;
					greatConnect[sth[2]] = true;
					print("We are creating coastal bridge", sth[1], sth[2])
					local randomT = {}
					for j, jplot in pairs(rTab) do
						if jplot:GetTerrainType() == tOcean then
							jplot:SetTerrainType(tCoast)
							gT.Globals.MedievalOceansIntoCoast[jplot:GetPlotIndex()] = 2;
							for h = 0, 5 do
								local hPlot = Map.PlotDirection(jplot:GetX(), jplot:GetY(), h)
								if hPlot and (hPlot:GetTerrainType() == tOcean) then
									table.insert(randomT, hPlot)
								end
							end
						end
					end
					for j, jplot in pairs(randomT) do
						if LastRandom(1, 100) < 65 then
							jplot:SetTerrainType(tCoast)
							gT.Globals.MedievalOceansIntoCoast[jplot:GetPlotIndex()] = 3;
						end
					end
				end
			end
		end
	end
	MCtheT = {};
	MCARC = 0;
end
function DoStrokeOfFeature(fType)
--gT.Globals.OriginalFeature
	local tbdf = {}
	for i = 0, Map:GetNumPlots() - 1, 1 do
		local fPlot = Map.GetPlotByIndex( i );
		if fPlot:GetFeatureType() == fType then
			for j = 0, 5 do
				local jPlot = Map.PlotDirection(fPlot:GetX(), fPlot:GetY(), j);
				if jPlot then
					if tbdf[jPlot] then
						tbdf[jPlot] = 1 + tbdf[jPlot]
					elseif jPlot:CanHaveFeature(fType) then
						if jPlot:GetFeatureType() == -1 then
							tbdf[jPlot] = 1;
						end
					end
				end
			end
		end
	end
	for plot, v in pairs(tbdf) do
		if LastRandom(1,100) <= 18 + 4 * v then
			plot:SetFeatureType(fType);
			gT.Globals.OriginalFeature[plot:GetPlotIndex()] = -1;
		end
	end
end

function ReduceAmountOfResource(rType, max)
	for i = 0, Map:GetNumPlots() - 1, 1 do
		local fPlot = Map.GetPlotByIndex( i );
		if fPlot:GetResourceType() == rType then
			if not gT.Globals.ResourceQuantity[i] then
				gT.Globals.ResourceQuantity[i] = fPlot:GetNumResource();
			end
			fPlot:SetNumResource(math.min(max, fPlot:GetNumResource()))
		end
	end
end

function DoMapModificationOnStartTheTitans()
	DoStrokeOfCoast()
	DoStrokeOfFeature(fJungle)
	DoStrokeOfFeature(fMarsh)
	DoStrokeOfFeature(fForest)
	ReduceAmountOfResource(rHorse, 1)
	ReduceAmountOfResource(rIron, 1)
	gT.Globals.RequireReload = true;
end

function DoMapModificationOnStartHope(bFirstInit)
	if bFirstInit then
		GenerateTropicalIslands(false)
	end
end

function DoMapModificationOnStartSoR(bFirstInit)
	if bFirstInit then
		GenerateTropicalIslands(false)
	end
end

function DoMapModificationOnStartTheTrial()
	DoStrokeOfCoast()
	gT.Globals.RequireReload = true;
end

function GetMajorPlayerDisLivingOnConts(j, i)
	local dis = 9999;
	local tab1 = {}
	local tab2 = {}
	for h, v in pairs(Players) do
		if v:IsAlive() and (not v:IsBarbarian()) then
			local sPlot = v:GetStartingPlot()
			local iCs = GetMedCT(sPlot);
			if j == iCs then
				table.insert(tab1, sPlot)
			elseif i == iCs then
				table.insert(tab2, sPlot)
			end
		end
	end
	for ki, sp in pairs(tab1) do
		for kj, hp in pairs(tab2) do
			dis = math.min(dis, Map.PlotDistance(hp:GetX(), hp:GetY(), sp:GetX(), sp:GetY()))
		end
	end
	return dis;
end

function GetMedCT(iPlot)
	local index = iPlot:GetPlotIndex();
	for i = 1, MCARC do
		if MCtheT[i][index] then
			return i;
		end
	end
	print("Grats error 1245/1313")
end

function MedievalITPCoast(iPlot, cID)
	MCAREAS[iPlot:GetPlotIndex()] = 1;
	MCtheT[cID][iPlot:GetPlotIndex()] = 1;	
	for j = 0, 5 do
		local jPlot = Map.PlotDirection(iPlot:GetX(), iPlot:GetY(), j);
		if jPlot then
			if not MCAREAS[jPlot:GetPlotIndex()] then
				if (not jPlot:IsWater()) or (jPlot:GetTerrainType() == tCoast) then
					MedievalITPCoast(jPlot, cID)
				end
			end
		end
	end
end

function IsAdjacentTerrain(x, y, tType)
	for i = 0, 5 do
		local nPlot = Map.PlotDirection(x, y, i);
		if nPlot then
			if nPlot:GetTerrainType() == tType then
				return true;
			end
		end
	end
	return false;
end

function DoColdWarmClimate(type)
	gT.Globals.ClimateType = type;
	local totX, totY = Map.GetGridSize();
	totY = totY - 1;
	if type == 1 then
		--cold
		local tabOfAllTundras = {}
		local tabOfAllIceSurrond = {}
		for i = 0, Map:GetNumPlots() - 1, 1 do
			local fPlot = Map.GetPlotByIndex( i );
			if fPlot:GetTerrainType() == tTundra then
				table.insert(tabOfAllTundras, fPlot)
			elseif fPlot:GetFeatureType() == fIce then
				for j = 0, 5 do
					local nPlot = Map.PlotDirection(fPlot:GetX(), fPlot:GetY(), j);
					if nPlot and nPlot:IsWater() and (nPlot:GetFeatureType() == -1) and (nPlot:GetResourceType() == -1) then
						tabOfAllIceSurrond[nPlot] = (tabOfAllIceSurrond[nPlot] or 0) + 1;
					end
				end
			end
		end
		for i, h in pairs(tabOfAllTundras) do
			if LastRandom(1,4) == 3 then
				gT.Globals.OriginalTerrain[h:GetPlotIndex()] = tTundra;
				h:SetTerrainType(tSnow)
			end
		end
		for h, nk in pairs(tabOfAllIceSurrond) do
			if LastRandom(1, 12) <= 2 + nk then
				gT.Globals.OriginalFeature[h:GetPlotIndex()] = -1;
				h:SetFeatureType(fIce)
			end
		end 
	elseif type == 2 then
		--warm
		local tabOfAllTundras = {}
		local tabOfAllIce = {}
		for i = 0, Map:GetNumPlots() - 1, 1 do
			local fPlot = Map.GetPlotByIndex( i );
			if fPlot:GetTerrainType() == tTundra then
				table.insert(tabOfAllTundras, fPlot)
			elseif fPlot:GetFeatureType() == fIce then
				if fPlot:GetY() ~= 0 and fPlot:GetY() ~= totY then
					table.insert(tabOfAllIce, fPlot)
				end
			end
		end
		for i, h in pairs(tabOfAllTundras) do
			if LastRandom(1,4) == 3 then
				gT.Globals.OriginalTerrain[h:GetPlotIndex()] = tTundra;
				h:SetTerrainType(tPlains)
			end
		end
		for i, h in pairs(tabOfAllIce) do
			if LastRandom(1, 4) <= 3 then
				gT.Globals.OriginalFeature[h:GetPlotIndex()] = fIce;
				h:SetFeatureType(-1)
			end
		end 
	end
end

function DoSleepOfReasonVisionStuff()
	for g, h in pairs(gT.Players) do
		local iTeam = Players[g]:GetTeam();
		for i = 0, Map:GetNumPlots() - 1, 1 do
			local fPlot = Map.GetPlotByIndex( i );
			if not fPlot:IsRevealed(iTeam) then
				fPlot:SetRevealed(iTeam, true)
			end
		end
	end
end

function DoLightOfHopeVisionStuff()
	CallPreviousEraIfConversion()
	local sTtra = {}
	local ttTeamT = {}
	for g, gh in pairs(gT.Players) do
		ttTeamT[Players[g]:GetTeam()] = 1;
	end
	for i = 0, Map:GetNumPlots() - 1, 1 do
		local fPlot = Map.GetPlotByIndex( i );
		local v = 0;
		for team, vh in pairs(ttTeamT) do
			if fPlot:IsRevealed(team) then
				v = v + 1;
			end
		end
		table.insert(sTtra, {fPlot, v})
	end
	sTtra = Shuffle(sTtra)
	table.sort(sTtra, SortFunctionForGreaterInteger2)
	local todiscover = math.floor(0.9 * Map.GetNumPlots());
	for i, data in pairs(sTtra) do
		if i <= todiscover then
			for team, vh in pairs(ttTeamT) do
				data[1]:SetRevealed(team, true)
			end
		else
			break;
		end
	end
end

function DoNewWorldVisionStuff()
	local tPlayersV = {}
	for g, gd in pairs(gT.Players) do
		if not Players[g]:IsMinorCiv() then
			tPlayersV[Players[g]:GetTeam()] = {};
		end
	end
	for i = 0, Map:GetNumPlots() - 1, 1 do
		local fPlot = Map.GetPlotByIndex( i );
		for a, h in pairs(tPlayersV) do
			if fPlot:IsRevealed(a) then
				table.insert(tPlayersV[a], i)
			end
		end
	end
	for g, gd in pairs(gT.Players) do
		if not Players[g]:IsMinorCiv() then
			local aTeam = Players[g]:GetTeam();
			local vCm, sTm = {}, {};
			for j, b in pairs(gT.Players) do
				if not Players[j]:IsMinorCiv() then
					if b.CultureType == gd.CultureType and (j ~= g) then
						if b.SubcultureType ~= 14 then
							if b.SubcultureType == gd.SubcultureType then
								table.insert(vCm, Players[j]:GetTeam())
							else
								table.insert(sTm, Players[j]:GetTeam())
							end
						end
					end					
				end
			end
			local mVisMax = 3;
			if gd.CultureType == 2 then
				mVisMax = LastRandom(1, 3);
			end
			if #vCm > mVisMax then
				vCm = Shuffle(vCm)
			end
			local cTab = {}
			for d, bTeam in pairs(vCm) do
				if mVisMax > 0 then
					table.insert(cTab, bTeam);
					mVisMax = mVisMax - 1;
				else
					break;
				end
			end
			if mVisMax > 0 then
				if #sTm > mVisMax then
					sTm = Shuffle(sTm)
				end
				for d, bTeam in pairs(sTm) do
					if mVisMax > 0 then
						table.insert(cTab, bTeam);
						mVisMax = mVisMax - 1;
					else
						break;
					end
				end
			end
			for jj, bTeam in pairs(cTab) do
				if not Teams[bTeam]:IsHasMet(aTeam) then
					Teams[bTeam]:Meet(aTeam);
				end
				for dd, indexP in pairs(tPlayersV[bTeam]) do
					local bPlot = Map.GetPlotByIndex(indexP);
					if not bPlot:IsRevealed(aTeam) then
						bPlot:SetRevealed(aTeam, true)
					end
				end
			end
		end
	end
end

function SimpleMapScriptsInitGameTruely(scType)
	if scType == 5 then
		DoReduceRemovableFeatures(55)
	elseif scType == 4 then
		DoReduceRemovableFeatures(40)
	elseif scType == 1 then
		DoReduceRemovableFeatures(25)
	end
end

function DoReduceRemovableFeatures(tChance)
	local fTr = {[fForest] = true, [fJungle] = true, [fMarsh] = true};
--gT.Globals.OriginalFeature do not return; it simulates starting in later eras
	for i = 0, Map:GetNumPlots() - 1, 1 do
		local fPlot = Map.GetPlotByIndex( i );
		if fTr[fPlot:GetFeatureType()] then
			if LastRandom(1, 100) <= tChance then
				fPlot:SetFeatureType(-1);
			end
		end
	end
end