local palaceTab = {}
local upgradeTab = {}

for a, b in pairs(ITG_PalaceTypes) do
	palaceTab[b] = {Upgrades = {}, Offsets = {}}
end
for row in DB.Query("SELECT * FROM ITG_PalaceUpgrades") do
	upgradeTab[row.ID] = {Name = Locale.Lookup(row.Description)}
	for a, b in pairs(row) do
		upgradeTab[row.ID][a] = b;
	end
end
for row in DB.Query("SELECT * FROM ITG_PalaceUpgradeReqs") do
	local rowid1 = ITG_PalaceUpgradeTypes[row.PalaceUpgradeType]
	local rowid2 = ITG_PalaceUpgradeTypes[row.ReqUpgrade]
	if not upgradeTab[rowid1].Requirements then
		upgradeTab[rowid1].Requirements = {};
	end
	table.insert(upgradeTab[rowid1].Requirements,rowid2)
end
for row in DB.Query("SELECT * FROM ITG_Palace_Upgrades") do
	local rowid1 = ITG_PalaceTypes[row.PalaceType]
	local rowid2 = ITG_PalaceUpgradeTypes[row.PalaceUpgradeType]
	table.insert(palaceTab[rowid1].Upgrades, rowid2);
	palaceTab[rowid1].Offsets[rowid2] = {row.OffX, row.OffY};
end

local palaceclass = GameInfoTypes.BUILDINGCLASS_ITG_LS_PALACE_UPGRADE
local eraPalaceB = {GameInfoTypes.BUILDING_ITG_LS_PALACE_UPGRADE_REN, GameInfoTypes.BUILDING_ITG_LS_PALACE_UPGRADE_MED, GameInfoTypes.BUILDING_ITG_LS_PALACE_UPGRADE, GameInfoTypes.BUILDING_ITG_LS_PALACE_UPGRADE_IND, -1}
local yieldT = {{"Authority", "[ICON_ITG_AUTHORITY] Authority: "},
{"Science", "[ICON_RESEARCH] Science: "},
{"Gold", "[ICON_GOLD] Gold: "},
{"Culture", "[ICON_CULTURE] Culture: "},
{"Production", "[ICON_PRODUCTION] Production: "},
{"WorkRate", "[ICON_WORKER] Work Rate: "},
{"GreatPeople", "[ICON_GREAT_PEOPLE] Great People: "},
{"Art", "[ICON_GREAT_ARTIST] Art: "},
{"Presence", "[ICON_ITG_PRESENCE] Presence: "},
{"Legal", "[ICON_ITG_LEGAL_POWER] Legal: ", 0.01},
{"Defense", "[ICON_STRENGTH] Defense: ", 0.01},
{"Faith", "[ICON_PEACE] Faith: "},
{"Intrigue","[ICON_ITG_POWER] Power: "},
{"TradeAppeal","[ICON_TRADE_WHITE] Trade Appeal: "},
}
local basicPalNums = {"Authority", "WorkRate", "Presence", "Art", "GreatPeople", "Happiness", "Legal", "Defense", "Intrigue", "TradeAppeal"}

function SetOffsetValHelp(t, v, x, y)
	palaceTab[t].Offsets[v][1] = x;
	palaceTab[t].Offsets[v][2] = y;
end

function GetHelpStringPalace(v)
	local tt = upgradeTab[v].Help or ""
	if upgradeTab[v].Requirements then
		if tt ~= "" then	
			tt = tt .. "[NEWLINE][NEWLINE]"
		end
		local bFirst = true;
		tt = tt .. "Requirements: "
		for i, h in pairs(upgradeTab[v].Requirements) do
			if bFirst then
				bFirst = false;
				tt = tt .. upgradeTab[h].Name;
			else
				tt = tt .. ", " .. upgradeTab[h].Name;
			end
		end
		tt = tt .. "."
	end
	return tt;
end

function GetYieldStringPalace(v)
	local ss = ""
	for i, b in pairs(yieldT) do
		if upgradeTab[v][b[1]] then
			ss = ss .. b[2]  .. (upgradeTab[v][b[1]] * (b[3] or 1)) .. (b[4] or "") .. "[NEWLINE]"
		end
	end
	if upgradeTab[v].Happiness then
		ss = ss .. "[ICON_HAPPINESS_1] Happiness: " .. upgradeTab[v].Happiness .. "%[NEWLINE]"
	end
	return ss;
end

function CanUpgradeAnyPalace(iPlayer)
	if gT.Players[iPlayer].FreePalacePoints > 0 then
		for iCity in Players[iPlayer]:Cities() do
			local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
			if gT.Cities[cID].Palace then
				if gT.Cities[cID].Palace.NumUpgrades < 10 then
					return true;
				end
			end
		end
		return false;
	end
	for iCity in Players[iPlayer]:Cities() do
		local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
		if gT.Cities[cID].PalaceUpgrades > 0 then
			if gT.Cities[cID].Palace.NumUpgrades < 10 then
				return true;
			end
		end
	end
	return false;
end

function DoSetPalaceReplacementInit()
	for g, h in pairs(gT.Players) do
		if not Players[g]:IsMinorCiv() then
			Players[g]:SetPlayerUniqueBuilding(palaceclass, eraPalaceB[gT.Globals.ScenarioType] or -1)
		else
			Players[g]:SetPlayerUniqueBuilding(palaceclass, -1)
		end
	end
end

function DoInitPalaces()
	for g, h in pairs(gT.Players) do
		if not Players[g]:IsMinorCiv() then
			local dat2 = GetDBCulturePlayerInfo(g, "MaxPalaceUpgrades")
			local numpalacesToAdd = LastRandom(GetDBCulturePlayerInfo(g, "MinStartPalace"), GetDBCulturePlayerInfo(g, "MaxStartPalace"));
			if numpalacesToAdd > 0 then
				local cCity = Players[g]:GetCapitalCity();
				if cCity then
					CreateNewPalace(cCity, g, LastRandom(0, dat2), LastRandom(-300, -50))
					numpalacesToAdd = numpalacesToAdd - 1;
					if numpalacesToAdd > 0 then
						local tab = {}
						for iCity in Players[g]:Cities() do
							if iCity ~= cCity then
								table.insert(tab, iCity)
							end
						end
						tab = Shuffle(tab)
						for ip, pp in pairs(tab) do
							if ip <= numpalacesToAdd then
								CreateNewPalace(pp, g, math.min(LastRandom(0, dat2), LastRandom(0, dat2)), LastRandom(-300, -50))
							end
						end
					end
				end
			end
		end
	end
end

local scPalaceFavours = {}
local ctPalaceFavours = {}
local subctPalaceFavours = {}
local typePalaceTab = {}
for row in DB.Query("SELECT * FROM ITG_Palace_Scenarios") do
	local rowid1 = ITG_ScenariosTypes[row.ScenarioType];
	if not scPalaceFavours[rowid1] then
		scPalaceFavours[rowid1] = {};
	end
	table.insert(scPalaceFavours[rowid1], {ITG_PalaceTypes[row.PalaceType], row.Favour})
end
for row in DB.Query("SELECT * FROM ITG_Palace_Cultures") do
	if ITG_SubcultureTypes[row.CultureType] then
		local rowid1 = ITG_SubcultureTypes[row.CultureType];
		if not subctPalaceFavours[rowid1] then
			subctPalaceFavours[rowid1] = {};
		end
		table.insert(subctPalaceFavours[rowid1], {ITG_PalaceTypes[row.PalaceType], row.Favour})
	else
		local rowid1 = ITG_CultureTypes[row.CultureType];
		if not ctPalaceFavours[rowid1] then
			ctPalaceFavours[rowid1] = {};
		end
		table.insert(ctPalaceFavours[rowid1], {ITG_PalaceTypes[row.PalaceType], row.Favour, row.NoSubculture})
	end
end

function PopulateTypeOfPalace(iPlayer, val)
	if gT.Players[iPlayer].SubcultureType and subctPalaceFavours[gT.Players[iPlayer].SubcultureType] then
		for a, b in pairs(subctPalaceFavours[gT.Players[iPlayer].SubcultureType]) do
			table.insert(typePalaceTab, {b[1], val * b[2]})
		end
		return;
	end
	if ctPalaceFavours[gT.Players[iPlayer].CultureType] then
		for a, b in pairs(ctPalaceFavours[gT.Players[iPlayer].CultureType]) do
			if (not b[3]) or (not gT.Players[iPlayer].SubcultureType) then
				table.insert(typePalaceTab, {b[1], val * b[2]})
			end
		end
		if #typePalaceTab > 0 then
			return;
		end
	end
	if scPalaceFavours[gT.Globals.ScenarioType] then
		for a, b in pairs(scPalaceFavours[gT.Globals.ScenarioType]) do
			table.insert(typePalaceTab, {b[1], val * b[2]})
		end
		return;
	end
	--if still undefined, throw random
	for a, b in pairs(palaceTab) do
		table.insert(typePalaceTab, {a, val})
	end
end

function GetTypeOfPalace(iCity, iPlayer)
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
	local oPlayer = iCity:GetOriginalOwner();
	typePalaceTab = {}
	PopulateTypeOfPalace(iPlayer, 5)
	if oPlayer ~= iPlayer then
		PopulateTypeOfPalace(oPlayer, 1)
	end
	if gT.Cities[cID].CultureOwner ~= iPlayer then
		if gT.Players[gT.Cities[cID].CultureOwner] then
			PopulateTypeOfPalace(gT.Cities[cID].CultureOwner, 1)
		end
	end
	if #typePalaceTab == 0 then
		print("ERROR no palace types", iPlayer, iCity:GetID())
	end
	return GetRandomFromBiasedChanceTable(typePalaceTab);
end

function CreateNewPalace(iCity, iPlayer, upgrades, fturn)
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
	gT.Cities[cID].Palace = {Upgrades = {}, NumUpgrades = 0, Created = fturn or LastRandom(-300, -50), Type = GetTypeOfPalace(iCity, iPlayer), Authority = 1, Production = 1, Gold = 0, Happiness = 0, WorkRate = 0, Science = 0, Culture = 1, Presence = 0, Art = 0, GreatPeople = 0, Legal = 0, Intrigue = 0, TradeAppeal = 0, Defense = 0, Faith = 0}
	ReassignYieldsAndBonusesFromPalace(iCity, cID, iPlayer)
	if upgrades then
		for i = 1, upgrades do
			local tab = GetTabOfAvailableUpgradesPalace(cID, iPlayer, iCity)
			if #tab > 0 then
				AddUpgradeToPalace(iCity, cID, iPlayer, tab[LastRandom(1,#tab)], true)
			end
		end
	end
	if not gT.Cities[cID].PalaceLog then
		gT.Cities[cID].PalaceLog = {}
	end
	if gT.Globals.GameGenerated then
		table.insert(gT.Cities[cID].PalaceLog, {GetITGGameTurn(), 1, iPlayer})
	else
		table.insert(gT.Cities[cID].PalaceLog, {0, 2, iPlayer})
	end
end

function HasAllRequirementUpgradesPalace(cID, upg)
	for i, v in pairs(upgradeTab[upg].Requirements) do
		if not gT.Cities[cID].Palace.Upgrades[v] then
			return false;
		end
	end
	return true;
end

function GetTabOfAvailableUpgradesPalace(cID, iPlayer, iCity)
	local tab = {}
	for i, upg in pairs(palaceTab[gT.Cities[cID].Palace.Type].Upgrades) do
		if not gT.Cities[cID].Palace.Upgrades[upg] then
			if (not upgradeTab[upg].Requirements) or HasAllRequirementUpgradesPalace(cID, upg) then
				table.insert(tab, upg)
			end
		end
	end
	return tab;
end

function GetRandomValidUpgradePalaceToRemove(cID)
	local bTab = {}
	local rTab = {}
	for upgrade, sth in pairs(gT.Cities[cID].Palace.Upgrades) do
		for i, dep in pairs(upgradeTab[upgrade].Requirements or {}) do
			bTab[dep] = 1;
		end
	end
	for upgrade, sth in pairs(gT.Cities[cID].Palace.Upgrades) do
		if not bTab[upgrade] then
			table.insert(rTab, upgrade)
		end
	end
	return rTab[LastRandom(1,#rTab)];	
end

function CanPalaceBeFullyDestroyed()
	if gT.Globals.ScenarioType == 5 then
		return false;
	end
	return true;
end

function DestroyPalaceInCity(iCity, cID, iPlayer, logType, loginfo)
	ReassignYieldsAndBonusesFromPalace(iCity, cID, iPlayer, -GetCityPalaceYieldMod(iCity, cID, iPlayer))
	gT.Cities[cID].Palace = nil;
	table.insert(gT.Cities[cID].PalaceLog, {GetITGGameTurn(), logType, loginfo})		
end

function CityConquerPalaceMatters(iCity, cID, nPlayer, oPlayer)
	if not gT.Cities[cID].Palace then
		return;
	end
	ReassignYieldsAndBonusesFromPalace(iCity, cID, nPlayer, nil, oPlayer) -- ALWAYS FIRE for proper yields and resources;
	if LastRandom(1, 100) <= 15 then
		local destroy = LastRandom(2, 4);
		if not CanPalaceBeFullyDestroyed() then
			destroy = math.min(destroy, gT.Cities[cID].Palace.NumUpgrades);
		end
		if destroy > 0 then
			if gT.Cities[cID].Palace.NumUpgrades < destroy then
				destroy = gT.Cities[cID].Palace.NumUpgrades;
				for i = 1, destroy do
					RemoveUpgradeFromPalace(iCity, cID, nPlayer, GetRandomValidUpgradePalaceToRemove(cID)) -- needs to fire!
				end
				DestroyPalaceInCity(iCity, cID, nPlayer, 4, nPlayer)
				if Players[oPlayer]:IsHuman() then
					Players[oPlayer]:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, "Barbarian troops of " .. Players[nPlayer]:GetName() .. " had completely destroyed Palace of " .. iCity:GetName() .. ".", "Palace destroyed!")
				end
			else
				table.insert(gT.Cities[cID].PalaceLog, {GetITGGameTurn(), 5, nPlayer, destroy})
				for i = 1, destroy do
					RemoveUpgradeFromPalace(iCity, cID, nPlayer, GetRandomValidUpgradePalaceToRemove(cID))
				end
			end
		end
	end
end

function RemoveUpgradeFromPalace(iCity, cID, iPlayer, upgrade)
	if not upgrade then
		print("No upgrade to destroy", iCity, cID, iPlayer, upgrade)
		return;
	end
	gT.Cities[cID].Palace.Upgrades[upgrade] = nil;
	gT.Cities[cID].Palace.NumUpgrades = gT.Cities[cID].Palace.NumUpgrades - 1;
	HandleYieldsOfPalaceOnDeUpgrade(iCity, cID, upgrade, iPlayer, -1)
end

function AddUpgradeToPalace(iCity, cID, iPlayer, upgrade, bFree)
	gT.Cities[cID].Palace.Upgrades[upgrade] = 1;
	gT.Cities[cID].Palace.NumUpgrades = gT.Cities[cID].Palace.NumUpgrades + 1;
	HandleYieldsOfPalaceOnDeUpgrade(iCity, cID, upgrade, iPlayer, 1)
	if not bFree then
		if gT.Cities[cID].PalaceUpgrades > 0 then
			gT.Cities[cID].PalaceUpgrades = gT.Cities[cID].PalaceUpgrades - 1;
		elseif gT.Players[iPlayer].FreePalacePoints > 0 then
			gT.Players[iPlayer].FreePalacePoints = gT.Players[iPlayer].FreePalacePoints - 1;
		else
			print("Somebody used palace upgrade, but cant pay", iCity, cID, iPlayer, upgrade)
		end
	end
	if gT.Globals.GameGenerated then
		DiploNoteDirty(iPlayer, 16)
		BasicCSMissionCompleteFunc(iPlayer, 21, 1)
		VictoryTaskCompletion(iPlayer, 60)
		if (gT.Cities[cID].LHPalaceTop or 0) < gT.Cities[cID].Palace.NumUpgrades then
			gT.Cities[cID].LHPalaceTop = gT.Cities[cID].Palace.NumUpgrades
			gT.Cities[cID].LHPalaceTurn = GetITGGameTurn();
			gT.Cities[cID].LHPalacePlayer = iPlayer;
			if gT.Cities[cID].LHPalaceTop == 10 then
				if Players[iPlayer]:IsHuman() then
					LuaEvents.ITG_AchievementCompleted(49)
				end
			end
		end
		if gT.Players[iPlayer].HasPlayerWonder[GameInfoTypes.BUILDING_ITG_NEW_WORLD_WONDER_20] then
			BoostArtisticPoints(iPlayer, 100, false)
		end
		if gT.Players[iPlayer].HasPlayerWonder[GameInfoTypes.BUILDING_ITG_NEW_WORLD_WONDER_18] then
			ResetGPMigrationInTheCity(iPlayer, iCity, cID)
		end
		if HasPlayerTech(iPlayer, GameInfoTypes.TECH_UNIQUE_ITG_MEDIEVAL_AFR_3, "innoTafr3") then
			ChangeMeritocracy(iPlayer, 3)
		end
		if iCity:IsHasBuilding(GameInfoTypes.BUILDING_ITG_NEW_INDUSTRIAL_WONDER_29) then
			BoostInfluenceMetNotInWarCS(iPlayer, 10)
		end
		if Players[iPlayer]:IsHuman() then
			LuaEvents.ITG_AchievementCountChange(80, 1)
		end
		table.insert(gT.Cities[cID].PalaceLog, {GetITGGameTurn(), 3, iPlayer, upgrade})
	end
end

function GetCityPalaceYieldMod(iCity, cID, iPlayer)
	local t = gT.Players[iPlayer].NoAuthYieldsFromPalaceMod;
	if gT.Cities[cID].FestivalET then
		if gT.Players[iPlayer].FestivalBonusesActive[16] then
			t = t + 1;
		end
	end
	return t;
end

function GetCityYieldBasicFromPalaceTT(iCity, cID, iPlayer, type)
	if gT.Cities[cID].Palace then
		return gT.Cities[cID].Palace[type] * GetCityPalaceYieldMod(iCity, cID, iPlayer)
	end
	return 0;
end

function ReassignYieldsAndBonusesFromPalace(iCity, cID, iPlayer, val, pPlayer)
	local igold = gT.Cities[cID].Palace.Gold;
	local palaceYmod = val or GetCityPalaceYieldMod(iCity, cID, iPlayer)
	local iproduction = gT.Cities[cID].Palace.Production;
	local iculture = gT.Cities[cID].Palace.Culture;
	local iscience = gT.Cities[cID].Palace.Science;
	local ifaith = gT.Cities[cID].Palace.Faith;
	if iscience ~= 0 then
		iCity:ChangeBaseYieldRateFromBuildings(yScience, iscience * palaceYmod)
	end
	if iculture ~= 0 then
		iCity:ChangeJONSCulturePerTurnFromBuildings(iculture * palaceYmod)
	end
	if iproduction ~= 0 then
		iCity:ChangeBaseYieldRateFromBuildings(yProduction, iproduction * palaceYmod)
	end
	if igold ~= 0 then
		iCity:ChangeBaseYieldRateFromBuildings(yGold, igold * palaceYmod)
	end
	if ifaith ~= 0 then
		iCity:ChangeFaithPerTurnFromBuildings(ifaith * palaceYmod)
	end
	AdjustStrengthOfAllCities(iPlayer)
end

function HandleYieldsOfPalaceOnDeUpgrade(iCity, cID, upgrade, iPlayer, val)
	local palaceYmod = GetCityPalaceYieldMod(iCity, cID, iPlayer);
	for i, bb in pairs(basicPalNums) do
		if upgradeTab[upgrade][bb] then
			gT.Cities[cID].Palace[bb] = gT.Cities[cID].Palace[bb] + val * upgradeTab[upgrade][bb];
		end
	end
	if upgradeTab[upgrade].Production then
		gT.Cities[cID].Palace.Production = gT.Cities[cID].Palace.Production + val * upgradeTab[upgrade].Production * palaceYmod;
		iCity:ChangeBaseYieldRateFromBuildings(yProduction, val * upgradeTab[upgrade].Production * palaceYmod)
	end
	if upgradeTab[upgrade].Culture then
		gT.Cities[cID].Palace.Culture = gT.Cities[cID].Palace.Culture + val * upgradeTab[upgrade].Culture  * palaceYmod;
		iCity:ChangeJONSCulturePerTurnFromBuildings(val * upgradeTab[upgrade].Culture  * palaceYmod)
	end
	if upgradeTab[upgrade].Science then
		gT.Cities[cID].Palace.Science = gT.Cities[cID].Palace.Science + val * upgradeTab[upgrade].Science * palaceYmod;
		iCity:ChangeBaseYieldRateFromBuildings(yScience, val * upgradeTab[upgrade].Science * palaceYmod)
	end
	if upgradeTab[upgrade].Gold then
		gT.Cities[cID].Palace.Gold = gT.Cities[cID].Palace.Gold + val * upgradeTab[upgrade].Gold * palaceYmod;
		iCity:ChangeBaseYieldRateFromBuildings(yGold, val * upgradeTab[upgrade].Gold * palaceYmod)
	end
	if upgradeTab[upgrade].Faith then
		gT.Cities[cID].Palace.Faith = gT.Cities[cID].Palace.Faith + val * upgradeTab[upgrade].Faith * palaceYmod;
		iCity:ChangeFaithPerTurnFromBuildings(val * upgradeTab[upgrade].Faith * palaceYmod)
	end
	if upgradeTab[upgrade].Defense then
		AdjustStrengthOfAllCities(iPlayer)
	end	
end

function DoAutomaticPalaceUpgrades(iPlayer)
	if gT.Players[iPlayer].GrandArchitect then
		AIHandlePalacePoints(iPlayer)
	end
end

function AIHandlePalacePoints(iPlayer)
	for iCity in Players[iPlayer]:Cities() do
		local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
		if gT.Cities[cID].Palace then
			for d = 1, gT.Cities[cID].PalaceUpgrades do
				if gT.Cities[cID].Palace.NumUpgrades < 10 then
					local tab = GetTabOfAvailableUpgradesPalace(cID, iPlayer, iCity)
					if #tab == 0 then
						print("OH not good palace stuff X1 ", cID, iPlayer)
					end
					AddUpgradeToPalace(iCity, cID, iPlayer, RandomFromTab(tab))
				end
			end
		end
	end
	local freeupgrades = gT.Players[iPlayer].FreePalacePoints
	for i = 1, freeupgrades do
		local htab = {}
		for iCity in Players[iPlayer]:Cities() do
			local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
			if gT.Cities[cID].Palace then
				if gT.Cities[cID].Palace.NumUpgrades < 10 then
					table.insert(htab, {iCity, GetAIValueForCityToHavePalaceUpgrade(iCity,cID, iPlayer), cID})
				end
			end
		end
		if #htab > 0 then
			local ikey = GetRandomFromBiasedChanceTable(htab, true)
			local utab = GetTabOfAvailableUpgradesPalace(htab[ikey][3], iPlayer, htab[ikey][1])
			if #utab == 0 then
				print("OH not good palace stuff", htab[ikey][3], iPlayer)
			end
			AddUpgradeToPalace(htab[ikey][1], htab[ikey][3], iPlayer, utab[LastRandom(1,#utab)])
		end
	end
end

function GetAIValueForCityToHavePalaceUpgrade(iCity,cID, iPlayer)
	local basev = 1 + gT.Cities[cID].Palace.NumUpgrades;
	if iCity:IsCapital() then
		basev = basev + 5;
	end
	basev = basev math.floor(10 * iCity:GetDamage() / iCity:GetMaxHitPoints()) 
	return basev;
end

function RequestEditedPalaceUpgrades(cityID)
	local dd = {}
	local iPlayer= Game.GetActivePlayer();
	local iCity = Players[iPlayer]:GetCityByID(cityID);
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
	local utab = GetTabOfAvailableUpgradesPalace(cID, iPlayer, iCity)
	local pType = gT.Cities[cID].Palace.Type;
	for i, v in pairs(utab) do
		table.insert(dd, {ID = v, Name = upgradeTab[v].Name, Yields = GetYieldStringPalace(v), Help = GetHelpStringPalace(v), OffsetX =  palaceTab[pType].Offsets[v][1], OffsetY = palaceTab[pType].Offsets[v][2]})
	end
	MapModData.ITGSG = dd;
end
LuaEvents.ITGSG_RequestEditedPalaceUpgrades.Add(RequestEditedPalaceUpgrades)

LuaEvents.ITG_CallPalaceUpgrade.Add(function(cityID, upgrade)
	local iPlayer= Game.GetActivePlayer();
	local iCity = Players[iPlayer]:GetCityByID(cityID);
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
	AddUpgradeToPalace(iCity, cID, iPlayer, upgrade)
	RefreshAllDirtyPlayer()
end)

function CallCityCreatedPalaceExpand(iCity, cID, iPlayer, bOnConstructed)
	if not gT.Cities[cID].Palace then
		CreateNewPalace(iCity, iPlayer)
	else
		if gT.Cities[cID].PalaceUpgrades + gT.Cities[cID].Palace.NumUpgrades >= 10 then
--			if GetDBCulturePlayerInfo(iPlayer, "CanAcquireChateau") then
--				gT.Players[iPlayer].ChateauCharges = gT.Players[iPlayer].ChateauCharges + 1;
--			end
		else
			gT.Cities[cID].PalaceUpgrades =  gT.Cities[cID].PalaceUpgrades  + 1;
		end
		if iPlayer == Game.GetActivePlayer() then
			if Players[iPlayer]:IsHuman() and (not bOnConstructed) then
				if Players[iPlayer]:IsTurnActive() then
					DoAutomaticPalaceUpgrades(iPlayer)
				end
			end
			Events.SerialEventEndTurnDirty()
		end
	end
	if bOnConstructed then
		local iPalaceUnhap = GetPalaceUnhappinessFromLowMeritocracy(iPlayer);
		if iPalaceUnhap > 0 then
			ChangeITGHappinessInTheCity(iCity, -iPalaceUnhap, nil, "ygrnefeavf")
		end
		gT.Players[iPlayer].LHNumPalaceUpgrades = gT.Players[iPlayer].LHNumPalaceUpgrades + 1;
		BoostGovernorPowerInCity(iCity, "Palaces", 1, true, false)
	end
end

function AddFreePalaceUpgrade(iPlayer, val)
	gT.Players[iPlayer].FreePalacePoints = gT.Players[iPlayer].FreePalacePoints + val;
	if iPlayer == Game.GetActivePlayer() then
		RefreshAllDirtyPlayer()
		if Players[iPlayer]:IsHuman() then
			if Players[iPlayer]:IsTurnActive() then
				DoAutomaticPalaceUpgrades(iPlayer)
			end
		end
	end
end

function ChangeYieldModFromPalace(iPlayer, am)
	gT.Players[iPlayer].NoAuthYieldsFromPalaceMod = gT.Players[iPlayer].NoAuthYieldsFromPalaceMod + am;
	for iCity in Players[iPlayer]:Cities() do
		local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
		if gT.Cities[cID].Palace then
			ReassignYieldsAndBonusesFromPalace(iCity, cID, iPlayer, am)
		end
	end
end

function ChangeCityPalaceYieldMod(iCity, iPlayer, am)
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
	if gT.Cities[cID].Palace then
		ReassignYieldsAndBonusesFromPalace(iCity, cID, iPlayer, am)
	end
end