local maxStoreResourceBase = {
[rHorse] = 50,
[rIron] = 50,
[rCoal] = 1000,
[rOil] = 1000,
}

function GetMaxResourceStored(iPlayer,res)
	local base = maxStoreResourceBase[res];
	if res == rHorse then
		base = base + 5 * Players[iPlayer]:CountNumBuildings(GameInfoTypes.BUILDING_ITG_IND_STABLE);
	elseif res == rIron then
		base = base + 5 * Players[iPlayer]:CountNumBuildings(GameInfoTypes.BUILDING_ITG_IND_FORGE);
	end
	return base;
end

function ChangeTradeabilityOfStrategicResources(eVal)
	for row in DB.Query("SELECT ResourceType from ITG_ScenarioUntradeable WHERE ScenarioType = '" .. DBScenario.Type .. "'") do
		local resID = GameInfo.Resources[row.ResourceType].ID;
		for g, gdata in pairs(gT.Players) do
			Players[g]:ChangeCanTradeResourceAway(resID, eVal)
		end
	end
end

function CheckWLTKTurn(iPlayer)
	if gT.Players[iPlayer].WLTKResource then
		if Players[iPlayer]:GetNumResourceTotal(gT.Players[iPlayer].WLTKResource, true) > 0 then
			gT.Players[iPlayer].WLTKChance = gT.Players[iPlayer].WLTKChance + 10;
			gT.Players[iPlayer].WLTKTurns = LastRandom(8, 15) + GetITGGameTurn();
			if Players[iPlayer]:IsHuman() then
				Players[iPlayer]:AddNotification(NotificationTypes.NOTIFICATION_DISCOVERED_BONUS_RESOURCE, "Providing our Citizens a source of " .. GameInfo.Resources[gT.Players[iPlayer].WLTKResource].IconString .. " " .. Locale.Lookup(GameInfo.Resources[gT.Players[iPlayer].WLTKResource].Description) .. " increased chance to acquire new [ICON_ITG_FESTIVAL] Festival by 1%.", "Resource Provided!", -1, -1, gT.Players[iPlayer].WLTKResource)
			end
			 gT.Players[iPlayer].WLTKResource = nil;
		end
	elseif gT.Players[iPlayer].WLTKTurns <= GetITGGameTurn() then
		local hRT = {}
		local resTab = GetResourcesPresentOnMap()
		for row in GameInfo.Resources() do
			if not row.Migration then
				if row.Happiness > 0 and (not gT.CurrencyResources[row.ID]) then
					if resTab[row.ID] then
						if Players[iPlayer]:GetNumResourceTotal(row.ID, true) == 0 then
							table.insert(hRT, row.ID)
						end
					end
				end
			end
		end
		if #hRT == 0 then
			gT.Players[iPlayer].WLTKChance = gT.Players[iPlayer].WLTKChance + 10;
			gT.Players[iPlayer].WLTKTurns = LastRandom(8, 15) + GetITGGameTurn();
		else
			gT.Players[iPlayer].WLTKResource = hRT[LastRandom(1,#hRT)];
			if Players[iPlayer]:IsHuman() then
				Players[iPlayer]:AddNotification(NotificationTypes.NOTIFICATION_DISCOVERED_BONUS_RESOURCE, "Providing our Citizens a source of " .. GameInfo.Resources[gT.Players[iPlayer].WLTKResource].IconString .. " " .. Locale.Lookup(GameInfo.Resources[gT.Players[iPlayer].WLTKResource].Description) .. " would increase chance to acquire next [ICON_ITG_FESTIVAL] Festival by 1%.", "Resource Demanded!", -1, -1, gT.Players[iPlayer].WLTKResource)
			end
		end
	end
end

LuaEvents.ITGAuctionHouseHumanChangeOffer.Add(function(iPlayer, res, change)
	local nNumOffered = (gT.Players[iPlayer].AuctionOffers[res] or 0) + change;
	if Players[iPlayer]:GetNumResourceTotal(res) >= nNumOffered then
		if nNumOffered >= 0 then
			SetAuctionHouseOffer(iPlayer, res, nNumOffered)
			MapModData.ITGSG = {nNumOffered, Players[iPlayer]:GetNumResourceTotal(res)};
		end
	end
end)

function SetAuctionHouseOffer(iPlayer, res, nNumOffered)
	if gT.Players[iPlayer].AuctionOffers[res] == nNumOffered then
		return;
	end
	gT.Players[iPlayer].AuctionOffers[res] = nNumOffered;
	if not gT.AuctionHouse[res] then
		gT.AuctionHouse[res] = {}
	end
	local bFound = false;
	local tRemove;
	for i, v in pairs(gT.AuctionHouse[res]) do
		if v.Player == iPlayer then
			bFound = true;
			if nNumOffered == 0 then
				tRemove = i;
			else
				gT.AuctionHouse[res][i].Amount = nNumOffered;
			end
		end
	end
	if tRemove then
		table.remove(gT.AuctionHouse[res], tRemove);
	end
	if not bFound then
		if nNumOffered ~= 0 then
			table.insert(gT.AuctionHouse[res], {Player = iPlayer, Amount = nNumOffered})
		end
	end
end

function DefineInitPriceAuctionHouse()
	for row in GameInfo.Resources() do
		if not row.Migration then
			if row.Happiness > 0 then
				if not gT.CurrencyResources[row.ID] then
					gT.AuctionPrices[row.ID] = 300;
					gT.Globals.AuctionPriceData[row.ID] = {}
				end
			end
		end
	end
end

function GetPriceAuctionHouse(res, iPlayer, bImportRed)
	local importRed = 0;
	if bImportRed then
		importRed = GoldChangesToImportingAH(iPlayer) * 30;
	end
	if gT.AuctionHouse[res] then
		local numAva, numSources = 0, 0;
		local extraCostFromEmbargo = 5;
		for a, b in pairs(gT.AuctionHouse[res]) do
			if Teams[Players[b.Player]:GetTeam()]:IsHasMet(Players[iPlayer]:GetTeam()) then
				local iRnum = math.min(Players[b.Player]:GetNumResourceTotal(res), b.Amount);
				if iRnum > 0 then
					numSources = numSources + 1;
					numAva = numAva + iRnum
				end
				if (gT.Players[b.Player].OpenTrade[iPlayer] or 0) > 0 then
					extraCostFromEmbargo = 0;
				else
					extraCostFromEmbargo = math.min(extraCostFromEmbargo, gT.Players[b.Player].Embargos[iPlayer] or 0)
				end
			end
		end
		if numAva > 0 then
			local b30 = gT.AuctionPrices[res] + importRed + extraCostFromEmbargo * 25;
			if numSources < 2 then
				b30 = b30 + 30;
			end
			if DBScenario.GoldBudgetOnly then
				return math.ceil(b30 / 30) * 30, math.ceil(math.ceil(b30 * 3 / 5) / 15) * 15, numAva;
			end
			return b30, math.ceil(b30 * 3 / 5), numAva;
		end
	end
	return nil, nil, nil;
end

function GetFromPlayerAuctionHouse(res, iPlayer)
	local pTabB = {}
	if gT.AuctionHouse[res] then
		local numAva, numSources = 0, 0;
		for a, b in pairs(gT.AuctionHouse[res]) do
			if Teams[Players[b.Player]:GetTeam()]:IsHasMet(Players[iPlayer]:GetTeam()) then
				local iRnum = math.min(Players[b.Player]:GetNumResourceTotal(res), b.Amount);
				if iRnum > 0 then
					local eval = gT.Players[b.Player].BrokenAHDeals;
					if (gT.Players[b.Player].OpenTrade[iPlayer] or 0) == 0 then
						eval = eval + (gT.Players[iPlayer].Embargos[b.Player] or 0);
					end
					if gT.Players[b.Player].VotesFromBreakingLeague then
						eval = eval + gT.Players[b.Player].VotesFromBreakingLeague;
					end
					table.insert(pTabB, {b.Player, eval, LastRandom(1,iRnum)}) -- players with greater amount are favoured yet not always;
				end
			end
		end
	end
	if #pTabB > 0 then
		pTabB = Shuffle(pTabB);
		table.sort(pTabB, SortFunctionLowest2Greatest3)
		return pTabB[1][1];
	else
		print("Error 3255324")
	end
end

function DecideIfAIDontWantToShareTheirLuxuries(iPlayer)
	local infavour = 8 + GetAIFlavour(iPlayer, "FLAVOR_GOLD")
	local against = 2 + GetAIFlavour(iPlayer, "FLAVOR_HAPPINESS")
	if DBScenario.GoldBudgetOnly then
		infavour = infavour + 5;
	end
	if GetITGGameTurn() < 20 then
		infavour = infavour + 5
	end
	return LastRandom(1, against + infavour) <= against;
end

function SetAIAuctionManager(iPlayer)
	local dValue = 1;
	local gPT = Players[iPlayer]:GetCachedGoldRate();
	if gPT > 30 and DecideIfAIDontWantToShareTheirLuxuries(iPlayer) then
		dValue = 2;
	else
		local bKeepOne = (gPT > LastRandom(40, 100)) or (GetAIFlavour(iPlayer, "FLAVOR_GOLD") < GetAIFlavour(iPlayer, "FLAVOR_HAPPINESS"))
		local bIgnore200 = (gPT > LastRandom(10, 40));
		local b1OfAll = 3 == LastRandom(1, 4);
		if bKeepOne then
			if b1OfAll then
				if bIgnore200 then
					dValue = 6;
				else
					dValue = 5
				end
			elseif bIgnore200 then
				dValue = 3
			else
				dValue = 4;
			end
		elseif b1OfAll then
			if bIgnore200 then
				dValue = 7;
			else
				dValue = 8;
			end
		elseif bIgnore200 then
			dValue = 9;
		else
			dValue = 1;
		end
	end
	gT.Players[iPlayer].AIAuctionManager = dValue;
	gT.Players[iPlayer].AIAuctionReset = GetITGGameTurn() + LastRandom(20, 35);
end

function DoAIAuctionHouseManagemenet(iPlayer)
	if not gT.Players[iPlayer].AIAuctionManager then
		if Players[iPlayer]:IsHuman() then
			return;
		else
			SetAIAuctionManager(iPlayer)
		end
	end
	if not Players[iPlayer]:IsHuman() then
		if (gT.Players[iPlayer].AIAuctionReset or -1) <= GetITGGameTurn() then
			SetAIAuctionManager(iPlayer)
		end
	end
-- 1. Everything
-- 2. nothing! (AI only)
-- 3. Keep 1, ignore less than 200
-- 4. Keep 1
-- 5. 1 of all, Keep1 
-- 6. 1 of all, Keep1 , ignore less than 200
-- 7. 1 of all, ignore less than 200
-- 8. 1 of all
-- 9. Ignore less than 200,
	local bNoneAtAll, bAll, bKeepOne, b1OfAll, bIgnore200 = false, false, false, false, false;
	if gT.Players[iPlayer].AIAuctionManager == 1 then
		bAll = true;
	elseif gT.Players[iPlayer].AIAuctionManager == 2 then
		bNoneAtAll = true;
	elseif gT.Players[iPlayer].AIAuctionManager == 3 then
		bIgnore200 = true;
		bKeepOne = true;
	elseif gT.Players[iPlayer].AIAuctionManager == 4 then
		bKeepOne = true;
	elseif gT.Players[iPlayer].AIAuctionManager == 5 then
		bKeepOne = true;
		b1OfAll = true;
	elseif gT.Players[iPlayer].AIAuctionManager == 6 then
		bKeepOne = true;
		b1OfAll = true;
		bIgnore200 = true;
	elseif gT.Players[iPlayer].AIAuctionManager == 7 then
		b1OfAll = true;
		bIgnore200 = true;
	elseif gT.Players[iPlayer].AIAuctionManager == 8 then
		b1OfAll = true;
	elseif gT.Players[iPlayer].AIAuctionManager == 9 then
		bAll = true;
		bIgnore200 = true;
	end
	for row in GameInfo.Resources() do
		if not row.Migration then
			if row.Happiness > 0 then
				if not gT.CurrencyResources[row.ID] then
					local inumRes = Players[iPlayer]:GetNumResourceTotal(row.ID);
					if inumRes == 0 then
						SetAuctionHouseOffer(iPlayer, row.ID, 0)
					else
						if bNoneAtAll then
							SetAuctionHouseOffer(iPlayer, row.ID, 0)
						else
							if bIgnore200 and (gT.AuctionPrices[row.ID] < 200) then
								SetAuctionHouseOffer(iPlayer, row.ID, 0)
							elseif bAll then
								SetAuctionHouseOffer(iPlayer, row.ID, inumRes)
							else
								if bKeepOne then
									inumRes = inumRes - 1;
								end
								if b1OfAll then
									inumRes = math.min(1, inumRes)
								end
								SetAuctionHouseOffer(iPlayer, row.ID, inumRes)
							end
						end
					end
				end
			end
		end
	end
end

function HumanSetAIAutoManagementAuctionH(iPlayer, type)
	if type == -1 then
		gT.Players[iPlayer].AIAuctionManager = nil;
	else
		gT.Players[iPlayer].AIAuctionManager = type;
		DoAIAuctionHouseManagemenet(iPlayer)
	end
end
LuaEvents.ITGHumanPlayerChooseAIAuctionManagement.Add(HumanSetAIAutoManagementAuctionH)

function HumanSetAIAutoManagementAcademySupport(iPlayer, type)
	if type == -1 then
		gT.Players[iPlayer].AIAcademyManager = nil;
	else
		gT.Players[iPlayer].AIAcademyManager = type;
		DoAutomaticAcademySupport(iPlayer)
	end
end
LuaEvents.ITGHumanPlayerChooseAIAcademyManagement.Add(HumanSetAIAutoManagementAcademySupport)

function HumanSetAIAutoManagementRailroad()
	local iPlayer = Game.GetActivePlayer();
	gT.Players[iPlayer].AIRailroad = (not gT.Players[iPlayer].AIRailroad);
	DoAutomaticRailroadMaking(iPlayer)
end
LuaEvents.ITGHumanPlayerChooseAIRailroad.Add(HumanSetAIAutoManagementRailroad)

function HumanPlayerChooseAIBombardment()
	local iPlayer = Game.GetActivePlayer();
	gT.Players[iPlayer].AutomaticBombardment = (not gT.Players[iPlayer].AutomaticBombardment);
	DoAutomaticBombardmentCities(iPlayer)
end
LuaEvents.ITGHumanPlayerChooseAIBombardment.Add(HumanPlayerChooseAIBombardment)

function HumanSetAIAutoManagementBullying(iPlayer, type)
	if type == -1 then
		gT.Players[iPlayer].AIBullying = nil;
	else
		gT.Players[iPlayer].AIBullying = type;
		DoAutomaticBullying(iPlayer)
	end
end
LuaEvents.ITGHumanPlayerChooseAIBullying.Add(HumanSetAIAutoManagementBullying)

function HumanSetAIConstructionManagement(iPlayer, tab)
	gT.Players[iPlayer].AIConstructionManager = tab;
	DoAutomaticConstructionOrder(iPlayer)
end
LuaEvents.ITGHumanChooseAIConstructionManagement.Add(HumanSetAIConstructionManagement)

function MakeAuctionHousePurchase(toPlayer, iRes, turns)
--	if turns == 30 and (GetITGGameTurn() > 70) then
--		print("Player tried to purchase 30T deal after 70T.", toPlayer)
--	end
	local h30, h15, amount = GetPriceAuctionHouse(iRes, toPlayer)
	if not h30 then
		print("Hello buyer bug", toPlayer, iRes, turns)
	end
--	if turns == 15 then
--		h30 = h15;
--	end
	local fromPlayer = GetFromPlayerAuctionHouse(iRes, toPlayer)
	local hPT = math.floor(h30 / turns);
	local lumpSum = h30 - turns * hPT;
	SpendITGGold(toPlayer, lumpSum)
	ChangeGoldFromAuctionHouse(toPlayer, -hPT + GoldChangesToImportingAH(toPlayer))
	ChangeGoldFromAuctionHouse(fromPlayer, hPT + GoldChangesToExportingAH(fromPlayer))
	if HasPlayerTech(fromPlayer, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ARAB_2,"563452342f") then
		ChangeITGGold(fromPlayer, 50);
		for iCity in Players[fromPlayer]:Cities() do
			local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
			gT.Cities[cID].CaravanProgress = gT.Cities[cID].CaravanProgress + 5;
			gT.Cities[cID].CargoProgress = gT.Cities[cID].CargoProgress + 5;
		end
	end
	if HasPlayerTech(toPlayer, GameInfoTypes.TECH_UNIQUE_ITG_INDUSTRIAL_18, "gjfkdvr") then
		DoTriggerHappinessChanceInAllCities(toPlayer)
	end
	if gT.Players[toPlayer].CultBonusOn[74] then
		DistributeRandomlyHappiness(toPlayer, 1)
	end
	table.insert(gT.Globals.AuctionDeals, {TurnStart = GetITGGameTurn(), Res = iRes, ToPlayer = toPlayer, FromPlayer = fromPlayer, FullGold = gT.AuctionPrices[iRes], GPT = hPT, GoldLumpSum = lumpSum, Duration = turns })
	DoExportResource(fromPlayer, toPlayer, iRes, 1)
	SetAuctionHouseOffer(fromPlayer, iRes, gT.Players[fromPlayer].AuctionOffers[iRes] - 1 )
	gT.Globals.AuctionPriceData[iRes].IgnoreThisTurn = true;
	gT.AuctionPrices[iRes] = gT.AuctionPrices[iRes] + LastRandom(3, 5) * 10;
	gT.Players[toPlayer].LHAuctionDeals = gT.Players[toPlayer].LHAuctionDeals + 1;
	gT.Players[toPlayer].LHAuctionDealsBuy = gT.Players[toPlayer].LHAuctionDealsBuy + 1;
	gT.Players[fromPlayer].LHAuctionDeals = gT.Players[fromPlayer].LHAuctionDeals + 1;
	DoNoteCustomsHouseAuctionDeal(toPlayer)
	DoNoteCustomsHouseAuctionDeal(fromPlayer)
	if Players[fromPlayer]:IsHuman() then
		Players[fromPlayer]:AddNotification(94343293, "We are now exporting " .. GameInfo.Resources[iRes].IconString .. " " .. Locale.Lookup(GameInfo.Resources[iRes].Description) .. " (+" .. hPT .. "[ICON_GOLD] Gold per turn) for next " .. turns .. " turns.", "New Deal", -1, -1, 26)
		if h30 >= 300 then
			LuaEvents.ITG_AchievementCompleted(20)
		end
	end
end
LuaEvents.ITG_HumanPlayerBuysResource.Add(MakeAuctionHousePurchase)

function GetPreparedPurchaseItemsAuctionHouse(iPlayer)
	local tab = {}
	for row in GameInfo.Resources() do
		if not row.Migration then
			if row.Happiness > 0 then
				if not gT.CurrencyResources[row.ID] then
					if Players[iPlayer]:GetNumResourceTotal(row.ID, true) < 1 then
						if CanSeeUseResource(iPlayer, row.ID) then
							local g30, g15, amount = GetPriceAuctionHouse(row.ID, iPlayer, true)
							if g30 then
								table.insert(tab, {row.ID, g30, amount})
							end
						end
					end
				end
			end
		end
	end
	return tab;
end

function EndTurnAuctionManagement()
	for res, ddd in pairs(gT.AuctionHouse) do
		for a, b in pairs(ddd) do
			local iRnum = Players[b.Player]:GetNumResourceTotal(res);
			if iRnum < b.Amount then
				SetAuctionHouseOffer(b.Player, res, math.max(iRnum, 0))
			end
		end
	end
	local vbTab = {}
	local gTurn = GetITGGameTurn();
	for a, b in pairs(gT.Globals.AuctionDeals) do
		if b.TurnStart + b.Duration <= gTurn then
			table.insert(vbTab, {a, nil})
		elseif Players[b.FromPlayer]:GetNumResourceTotal(b.Res) < 0 then
			table.insert(vbTab, {a, b.FromPlayer, true})
		elseif not Players[b.ToPlayer]:IsAlive() then
			table.insert(vbTab, {a, b.ToPlayer})
		end
	end
	local nttt = #vbTab;
	for i = 1, nttt do
		local indexvb = nttt + 1 - i;
		if vbTab[indexvb][3] then
			local auctionID = vbTab[indexvb][1];
			local fromPlayer = vbTab[indexvb][2];
			if Players[fromPlayer]:GetNumResourceTotal(gT.Globals.AuctionDeals[auctionID].Res) < 0 then
				EndDealAuctionHouse(auctionID, fromPlayer)
			end
		else
			EndDealAuctionHouse(vbTab[indexvb][1], vbTab[indexvb][2])
		end
	end
	vbTab = {}
	for a, b in pairs(gT.Globals.PirateDeals) do
		if b.TurnStart + b.Duration <= gTurn then
			table.insert(vbTab, a)
		end
	end
	nttt = #vbTab;
	for i = 1, nttt do
		EndPirateDeal(vbTab[nttt + 1 - i])
	end
--do adjust gold cost gT.AuctionPrices
	for res, data in pairs(gT.AuctionHouse) do
		if not gT.Globals.AuctionPriceData[res].IgnoreThisTurn then
			local nOff = 0;
			local nSources = 0;
			for a, b in pairs(data) do
				if b.Amount > 0 then
					nOff = nOff + b.Amount;
					nSources = nSources + 1;
				end
			end
			if nOff == 0 then
				gT.Globals.AuctionPriceData[res].Rise = math.min(20, (gT.Globals.AuctionPriceData[res].Rise or 0) + 5)
				gT.AuctionPrices[res] = math.min(300, gT.AuctionPrices[res] + gT.Globals.AuctionPriceData[res].Rise)
			else
				gT.Globals.AuctionPriceData[res].Drop = math.min(20, (gT.Globals.AuctionPriceData[res].Drop or 0) + 5);
				gT.Globals.AuctionPriceData[res].Rise = nil;
				if gT.AuctionPrices[res] > 200 then
					gT.AuctionPrices[res] = math.max(200, gT.AuctionPrices[res] - math.min(gT.Globals.AuctionPriceData[res].Drop, 5 + math.min(nSources * 100 - 99, nOff)))
				elseif nSources > 1 then
					if gT.AuctionPrices[res] > 100 then
						gT.AuctionPrices[res] = math.max(100, gT.AuctionPrices[res] - math.min(gT.Globals.AuctionPriceData[res].Drop, 3 + math.ceil(nOff / 2 + LastRandom(0,1) * 0.5)))
					elseif gT.AuctionPrices[res] > 50 then
						gT.AuctionPrices[res] = math.max(50, gT.AuctionPrices[res] - math.min(gT.Globals.AuctionPriceData[res].Drop, math.ceil(nOff / 4)))
					end
				end
			end
		else
			gT.Globals.AuctionPriceData[res].Drop = nil;
			gT.Globals.AuctionPriceData[res].IgnoreThisTurn = nil;
		end
	end
end

function PlayerCheckAuctionDeals(iPlayer)
	local vbTab = {}
	local gTurn = GetITGGameTurn();
	for a, b in pairs(gT.Globals.AuctionDeals) do
		if b.TurnStart + b.Duration <= gTurn then
			if b.ToPlayer == iPlayer then
				table.insert(vbTab, a)
			end
		end
	end
	local nttt = #vbTab;
	if nttt > 0 then
		for i = 1, nttt do
			EndDealAuctionHouse(vbTab[nttt + 1 - i])
		end
		gT.Players[iPlayer].AIAuctionNow = true;
	end
end

function EndPirateDeal(dealID)
	local iRes = gT.Globals.PirateDeals[dealID].Res
	local toPlayer = gT.Globals.PirateDeals[dealID].ToPlayer
	local fromPlayer = gT.Globals.PirateDeals[dealID].FromPlayer
	DoExportResource(fromPlayer, toPlayer, iRes, -1)
	table.remove(gT.Globals.PirateDeals, dealID)
end

function DoExportResource(fromPlayer, toPlayer, iRes, amount)
	Players[toPlayer]:ChangeNumResourceImported(iRes, amount)
	Players[fromPlayer]:ChangeNumResourceExported(iRes, amount)
	table.insert(gT.Globals.ExportLog, {fromPlayer, toPlayer, iRes, amount})
end

function EndDealAuctionHouse(dealID, bPrematurely, bEndEra)
	local iRes = gT.Globals.AuctionDeals[dealID].Res
	local toPlayer = gT.Globals.AuctionDeals[dealID].ToPlayer
	local fromPlayer = gT.Globals.AuctionDeals[dealID].FromPlayer
	local fullGold = gT.Globals.AuctionDeals[dealID].FullGold
	local iGoldLump = gT.Globals.AuctionDeals[dealID].GoldLumpSum;
	local gpt = gT.Globals.AuctionDeals[dealID].GPT;
	local dur = gT.Globals.AuctionDeals[dealID].Duration;
	DoExportResource(fromPlayer, toPlayer, iRes, -1)
	if bPrematurely then
		if bPrematurely == fromPlayer then
			gT.Players[fromPlayer].BrokenAHDeals = gT.Players[fromPlayer].BrokenAHDeals + 1;
-- dunno why it was returning gold from void?
--			if iGoldLump > 0 then
--				ChangeITGGold(toPlayer, iGoldLump);
--			end
		end
	end
	ChangeGoldFromAuctionHouse(toPlayer, gpt - GoldChangesToImportingAH(toPlayer))
	ChangeGoldFromAuctionHouse(fromPlayer, -gpt - GoldChangesToExportingAH(fromPlayer))
	if not bEndEra then
		table.insert(gT.Globals.AuctionDealHistory, {Res = iRes, ToPlayer = toPlayer, FromPlayer = fromPlayer, FullGold = fullGold, GoldLumpSum = iGoldLump, GPT = gpt, TurnEnd = GetITGGameTurn(), TurnStart = gT.Globals.AuctionDeals[dealID].TurnStart, Duration = dur})
		gT.Globals.AuctionPriceData[iRes].IgnoreThisTurn = true;
		gT.AuctionPrices[iRes] = math.max(gT.AuctionPrices[iRes], fullGold)
		if Players[toPlayer]:IsHuman() then
			--notification icons
			if bPrematurely == fromPlayer then
				Players[toPlayer]:AddNotification(94343293, "Apparently civilization exporting " .. GameInfo.Resources[iRes].IconString .. " " .. Locale.Lookup(GameInfo.Resources[iRes].Description) .. " to us is no longer able to provide it (+" .. gpt .. "[ICON_GOLD] Gold per turn).", "Deal broken", -1, -1, 25)
			else
				Players[toPlayer]:AddNotification(94343293, "After " .. dur .. " turns we are no longer importing " .. GameInfo.Resources[iRes].IconString .. " " .. Locale.Lookup(GameInfo.Resources[iRes].Description) .. " (+" .. gpt .. "[ICON_GOLD] Gold per turn).", "Deal ended", -1, -1, 24)
			end
		end
		if Players[fromPlayer]:IsHuman() then
			--notification icons
			if bPrematurely == fromPlayer then
				Players[fromPlayer]:AddNotification(94343293, "We were no longer able to provide " .. GameInfo.Resources[iRes].IconString .. " " .. Locale.Lookup(GameInfo.Resources[iRes].Description) .. " (-" .. gpt .. "[ICON_GOLD] Gold per turn).", "Deal broken", -1, -1, 25)
			elseif bPrematurely == toPlayer then
				Players[fromPlayer]:AddNotification(94343293, "Apparently civilization to which we were exporting " .. GameInfo.Resources[iRes].IconString .. " " .. Locale.Lookup(GameInfo.Resources[iRes].Description) .. " no longer exists (-" .. gpt .. "[ICON_GOLD] Gold per turn).", "Deal broken", -1, -1, 25)
			else
				Players[fromPlayer]:AddNotification(94343293, "After " .. dur .. " turns we are no longer obligated to export a copy of " .. GameInfo.Resources[iRes].IconString .. " " .. Locale.Lookup(GameInfo.Resources[iRes].Description) .. " (-" .. gpt .. "[ICON_GOLD] Gold per turn).", "Deal ended", -1, -1, 24)
			end
		end
		SetAuctionHouseOffer(fromPlayer, iRes, (gT.Players[fromPlayer].AuctionOffers[iRes] or 0) + 1 )
	end
	table.remove(gT.Globals.AuctionDeals, dealID)
end

function ChangeGoldFromAuctionHouse(iPlayer, change)
	gT.Players[iPlayer].GoldFromAuctionHouse = gT.Players[iPlayer].GoldFromAuctionHouse + change;
end

LuaEvents.ITGSG_PrepareItemsToBuyAuctionHouse.Add(function(iPlayer)
	local tab = {GPT = GetRealCalculateGoldRate(iPlayer, true), Data = GetPreparedPurchaseItemsAuctionHouse(iPlayer), Basic = GetPreparedPurchaseBasicStuff(iPlayer), Special = GetPreparedSpecialAHStuff(iPlayer)}
	for a, b in pairs(tab.Data) do
		tab.Data[a][4] = math.floor(b[2] / 30)
		tab.Data[a][5] = b[2] - tab.Data[a][4] * (30)
		tab.Data[a][6] = 10 -- math.floor(b[3] / 15)
		tab.Data[a][7] = 150; -- b[3] - tab.Data[a][6] * (15)
		tab.Data[a][8] = "Acquiring this Luxury for 30 turns will cost " ..  tab.Data[a][5] .. " [ICON_GOLD] Gold and an additional " .. tab.Data[a][4] .. " [ICON_GOLD] Gold per turn.";
		tab.Data[a][9] = "";
--		tab.Data[a][9] = "Acquiring this Luxury for 15 turns will cost " ..  tab.Data[a][7] .. " [ICON_GOLD] Gold and an additional " .. tab.Data[a][6] .. " [ICON_GOLD] Gold per turn.";
	end
	MapModData.ITGSG = tab
end)

function GetPreparedPurchaseBasicStuff(iPlayer)
	local tab = {};
	if CanPurchaseAcademyBasic(iPlayer) then
		local brb = {ID = 1, Name = "Academy", Cost = GetITGAcademyCost(iPlayer), ToolTip = Locale.Lookup("TXT_KEY_ITG_TT_PURCHASE_ACADEMY")}
		brb.Disabled = Players[iPlayer]:GetGold() < brb.Cost;
		table.insert(tab, brb)
	end
	if gT.Players[iPlayer].LegalAuthorityCost  ~= -1 then
		local brb = {ID = 2, Name = "1 [ICON_ITG_LEGAL_POWER] Legal Power", Cost = gT.Players[iPlayer].LegalAuthorityCost, ToolTip = ""}
		brb.Disabled = Players[iPlayer]:GetGold() < brb.Cost;
		table.insert(tab, brb)
	end
	if GetDBCulturePlayerInfo(iPlayer, "CanBuyAgents") and (not DBScenario.GoldBudgetOnly) and (gT.Players[iPlayer].CondemnEspionage <= 0) then
		local brb = {ID = 3, Name = "Agent", Cost = GetCostOfNewAgent(iPlayer), ToolTip = Locale.Lookup("TXT_KEY_ITG_TT_PURCHASE_AGENT")}
		brb.Disabled = Players[iPlayer]:GetGold() < brb.Cost;
		table.insert(tab, brb)
	end
	if MapModData.DBScenario.MF_ImproCost100 then
		local mfInfo = GetInfoManufactoryPurchase(iPlayer);
		local brb = {ID = 4, Name = "Manufactory", StrCost = mfInfo.Str, ToolTip = mfInfo.ToolTip}
		brb.Disabled = (not mfInfo.CanPurchase);
		table.insert(tab, brb)
	end
	return tab;
end

LuaEvents.ITG_HumanPlayerBuysBasicDeal.Add(function(iPlayer, id)
	if id == 1 then
		DoPlayerPurchaseAcademyPoint(iPlayer)
	elseif id == 2 then
		PlayerWantToPurchaseLegalAuthority(iPlayer, 1)
	elseif id == 3 then
		PlayerBoughtNewAgentWithGold(iPlayer)
	elseif id == 4 then
		DoPurchaseManufactory(iPlayer)
	end
end)

function GoldChangesToExportingAH(iPlayer)
	local b = 0;
	if HasPlayerTech(iPlayer, GameInfoTypes.TECH_UNIQUE_ITG_NW_TROPICAL_5, "Fkoerkfe") then
		b = b + 2;
	end
	return b;
end

function GoldChangesToImportingAH(iPlayer)
	local b = 0;
	if HasPlayerTech(iPlayer, GameInfoTypes.TECH_UNIQUE_ITG_NW_TROPICAL_5, "Fkoerkfe") then
		b = b - 1;
	end
	return b;
end

function ChangeGoldFromExportingAllDeals(iPlayer, amount)
	for dealID, data in pairs(gT.Globals.AuctionDeals) do
		if data.ToPlayer == iPlayer then
			ChangeGoldFromAuctionHouse(iPlayer, amount)
		end
	end
end

function ChangeGoldFromImportingAllDeals(iPlayer, amount)
	for dealID, data in pairs(gT.Globals.AuctionDeals) do
		if data.FromPlayer == iPlayer then
			ChangeGoldFromAuctionHouse(iPlayer, amount)
		end
	end
end

function GetNumDealsAuctionHouse(iPlayer, bImport, bExport)
	local t = 0;
	for dealID, data in pairs(gT.Globals.AuctionDeals) do
		if (not iPlayer) or (data.FromPlayer == iPlayer) or (data.ToPlayer == iPlayer) then
			if (not bExport) or (data.FromPlayer == iPlayer) then
				if (not bImport) or (data.ToPlayer == iPlayer) then
					t = t + 1;
				end
			end
		end
	end
	return t;
end

function ChangeNumResourcesUsed(iPlayer, res, t)
	gT.Players[iPlayer].ResourcesUsed[res] = (gT.Players[iPlayer].ResourcesUsed[res] or 0) + t;
	ChangeITGExtraResources(iPlayer, res, -t)
end

function GetResourceTooltipITG(gPlayer)
	local pPlayer = Players[gPlayer];
	local pTeam = Teams[pPlayer:GetTeam()];
	
	local strText = "";
	
	local pResource;
	local bShowResource;
	local bThisIsFirstResourceShown = true;
	local iNumAvailable;
	local iNumUsed;
	local iNumTotal;

	local pCtR = GetTPResourcesConsumedPerT(gPlayer);
	
	for iRes, i in pairs(gT.Globals.ResourcesInScenario) do
		pResource = GameInfo.Resources[iRes];
		local iResourceLoop = pResource.ID;
			
		bShowResource = CanSeeUseResource(gPlayer, iRes);
			
		if (bShowResource) or (pPlayer:GetNumResourceUsed(iResourceLoop) > 0) then
			if (bThisIsFirstResourceShown) then
				strText = "";
				bThisIsFirstResourceShown = false;
			else
				strText = strText .. "[NEWLINE][NEWLINE]";
			end
			iNumUsed = pPlayer:GetNumResourceUsed(iResourceLoop) + (gT.Players[gPlayer].ResourcesUsed[iResourceLoop] or 0);
			iNumTotal = pPlayer:GetNumResourceTotal(iResourceLoop, true) + (gT.Players[gPlayer].ResourcesUsed[iResourceLoop] or 0)
			if DBScenario.StrategicResStorage then
				iNumAvailable = pPlayer:GetResourceStored(iResourceLoop) - iNumUsed;
				iNumTotal = iNumTotal - (pCtR[iResourceLoop] or 0) 
				strText = strText .. iNumAvailable .. "/" .. GetMaxResourceStored(gPlayer, iResourceLoop) .. " " .. pResource.IconString .. " " .. Locale.ConvertTextKey(pResource.Description);
				if iNumTotal > 0 then
					strText = strText .. " [COLOR_POSITIVE_TEXT](" .. iNumTotal .. " per turn)[ENDCOLOR],"
				elseif iNumTotal < 0 then
					strText = strText .. " [COLOR_NEGATIVE_TEXT](" .. iNumTotal .. " per turn)[ENDCOLOR],"
				end
				if iNumUsed > 0 then
					strText = strText .. " [COLOR_NEGATIVE_TEXT]" .. iNumUsed .. " Used[ENDCOLOR]"
				end
			else
				iNumAvailable = pPlayer:GetNumStrategicResourceAvailable(iResourceLoop, true);
		--		if DBScenario.ResLockedTurns then
		--			iNumTotal = iNumTotal + (gT.Players[gPlayer].ResourcesUsed[iResourceLoop] or 0)
		--		end		
				
				strText = strText .. iNumAvailable .. " " .. pResource.IconString .. " " .. Locale.ConvertTextKey(pResource.Description);
				-- Details
				if (iNumUsed ~= 0 or iNumTotal ~= 0) then
					strText = strText .. ": ";
					if DBScenario.StrategicResourceLimit then
						if iRes == rCoal then
							--hardcoded visual for 5th era
							strText = strText .. ": [COLOR_POSITIVE_TEXT]" .. iNumTotal .. "/" .. DBScenario.StrategicResourceLimit .. " Total[ENDCOLOR], (" .. GetRawManufactoryCost(gPlayer) .. "[ICON_ITG_MANUFACTORY])";
						else
							strText = strText .. Locale.ConvertTextKey("TXT_KEY_TP_RESOURCE_INFO", iNumTotal .. "/" .. DBScenario.StrategicResourceLimit, iNumUsed);
						end
					else
						strText = strText .. Locale.ConvertTextKey("TXT_KEY_TP_RESOURCE_INFO", iNumTotal, iNumUsed);
					end
				end
			end
		end
	end
	if DBScenario.ResLockedTurns then
		strText = strText .. "[NEWLINE][NEWLINE]After training unit or constructing building requiring a strategic resource, that resource will be locked from further use for " .. DBScenario.ResLockedTurns .. " turns.";
		local gTurn = GetITGGameTurn();
		local hTab = {};
		for i, data in pairs(gT.Players[gPlayer].ResLockedT) do
			local eDur = data[3] - gTurn;
			if not hTab[eDur] then 
				hTab[eDur] = {}
			end
			hTab[eDur][data[1]] = (hTab[eDur][data[1]] or 0) + data[2];
		end
		for i = 1, 20 do
			if hTab[i] then
				for res, val in pairs(hTab[i]) do
					local pResource = GameInfo.Resources[res];
					strText = strText .. "[NEWLINE][ICON_BULLET]" ..  val .. " " .. pResource.IconString .. " " .. Locale.ConvertTextKey(pResource.Description) .. " (" .. i .. "T)"
				end
			end
		end
	end
	return strText;
end

function DoConsumeResource(iPlayer, res, amount, bForce)
	local consumed = amount;
	if not bForce then
		consumed = math.min(math.max(0, Players[iPlayer]:GetResourceStored(res)), amount);
	end
	if consumed > 0 then
		Players[iPlayer]:ChangeResourceStored(res, -consumed)
		gT.Players[iPlayer].ConsumedRes[res] = (gT.Players[iPlayer].ConsumedRes[res] or 0) + consumed;
	end
	return consumed; 
end

function DoStoreResource(iPlayer, res, amount)
	if amount > 0 then
		Players[iPlayer]:SetResourceStored(res, math.min(Players[iPlayer]:GetResourceStored(res) + amount, GetMaxResourceStored(iPlayer,res)))
	end
end

function GetMaxResourceAddedBasedOnCap(iPlayer, iRes)
	return (GetMaxResourceStored(iPlayer,iRes) - Players[iPlayer]:GetResourceStored(iRes))
end

function GetResourceStoredFromPillage(iPlayer, iPlot)
	if DBScenario.StrategicResStorage then
		local iRes = iPlot:GetResourceType(Players[iPlayer]:GetTeam());
		if gT.Globals.ResourcesInScenario[iRes] then
			local iChange = math.min(LastRandom(-15,15), GetMaxResourceAddedBasedOnCap(iPlayer, iRes));
			if iChange > 0 then
				DoStoreResource(iPlayer, iRes, iChange)
				return "+" .. iChange .. GameInfo.Resources[iRes].IconString;
			end
		end
	end
	return nil;
end

function GetTPResourcesConsumedPerT(iPlayer)
	local tab = {}
	if gT.Globals.ScenarioType == 4 then
		tab[rCoal] = Players[iPlayer]:CountNumBuildings(GameInfoTypes.BUILDING_ITG_FACTORY)
		local istable = Players[iPlayer]:CountNumBuildings(GameInfoTypes.BUILDING_ITG_IND_STABLE) + (gT.Players[iPlayer].ResGenPT[rHorse] or 0);
		local iforge = Players[iPlayer]:CountNumBuildings(GameInfoTypes.BUILDING_ITG_IND_FORGE) + (gT.Players[iPlayer].ResGenPT[rIron] or 0);
		if (istable > 0) and (Players[iPlayer]:GetResourceStored(rHorse) < RES_STORED_MAX_TO_GEN_NEW) then
			tab[rHorse] = -1 * istable / RES_IRON_HORSE_GEN_RATIO;
		end
		if (iforge > 0) and (Players[iPlayer]:GetResourceStored(rIron) < RES_STORED_MAX_TO_GEN_NEW) then
			tab[rIron] = -1 * iforge / RES_IRON_HORSE_GEN_RATIO;
		end
	end
	return tab;
end

function GetNumAvailableLuxuriesAH(gPlayer)
	return #GetPreparedPurchaseItemsAuctionHouse(gPlayer);
end