local SeaLevelTable = {
{25, {{0,2},{0,1},{0,1},{0,1},{0,2},{0,0},{0,1},{0,1},{0,0}, {0,1}, {0, 1}}, {{0,0},{0,0}, {0,0}, {0,0},{0,1}, {0,0}, {0,0},{0,0},{0,0}, {0,0}, {0, 1}}},
{75, {{0,3},{0,2},{0,2},{0,2},{0,3},{0,1},{0,2},{0,1},{0,0}, {0,2}, {0, 2}}, {{0,2}, {0,1}, {0,1}, {0,1},{0,2}, {0,0},{0,1},{0,1},{0,0}, {0,1}, {1, 2}}},
{200, {{1,5},{1,3},{1,3},{1,3},{0,4},{0,1},{0,2},{0,1},{0,0}, {0,3}, {0, 4}}, {{0,3}, {0,2}, {0,2}, {0,2}, {0,3}, {0,1},{0,1},{0,1},{0,0}, {0, 3}, {1, 3}}},
{500, {{3,6},{2,4},{2,4},{2,4},{1,5},{1,2},{1,3},{1,2},{0,1}, {0,5}, {0, 6}}, {{0,5}, {0,3}, {0,3}, {0,3}, {0,4}, {0,1},{1,1},{0,1},{0,1}, {0, 4}, {1, 4}}},
{100000, {{4,8},{3,5},{3,5},{3,5},{2,6},{1,3},{1,4},{1,3},{0,1}, {0,7}, {3, 10}}, {{1,5}, {1,4}, {1,4}, {1,4}, {2,4}, {0,3},{1,1},{0,1},{0,1}, {1, 4}, {3, 5}}},
}

local CombatUnitsNoXP = { --boolean if counts for supply
[5] = {
[GameInfoTypes.UNITCLASS_ATOMIC_BOMB] = {GameInfoTypes.UNIT_ITG_ATOMIC_BOMB, true},
[GameInfoTypes.UNITCLASS_GUIDED_MISSILE] = {GameInfoTypes.UNIT_ITG_GUIDED_MISSILE, false},
}
}

function IsCombatUnitWithNoXP(unitType)
	if CombatUnitsNoXP[gT.Globals.ScenarioType] then
		if CombatUnitsNoXP[gT.Globals.ScenarioType][GameInfoTypes[GameInfo.Units[unitType].Class]] then
			return true;
		end
	end
	return false;
end

local CultureTypesUnitClasses;
local SubCultureClassesOneFrom;
local SubCultureClasses;
local UniqueUnitsDefCultureTypes;
local UniqueUnitsDefSubcultureTypes;
local FreePromotionsOfUnitType;
local BarbariansUnitClasses;

function DefineUnitGeneratingDB()
	if BarbariansUnitClasses then
		return;
	end
	BarbariansUnitClasses = {};
	FreePromotionsOfUnitType = {};
	SubCultureClasses = {};
	CultureTypesUnitClasses = {};
	SubCultureClassesOneFrom = {};
	UniqueUnitsDefCultureTypes = {};
	UniqueUnitsDefSubcultureTypes = {};
	for row in DB.Query("SELECT * FROM ITG_ScBarbarianUnitClasses") do
		local scIndex = ITG_ScenariosTypes[row.ScenarioType];
		if not BarbariansUnitClasses[scIndex] then
			BarbariansUnitClasses[scIndex] = {};
		end
		if row.UnitType == "NO_UNIT" then
			BarbariansUnitClasses[scIndex][GameInfo.UnitClasses[row.UnitClassType].ID] = -1;
		else
			BarbariansUnitClasses[scIndex][GameInfo.UnitClasses[row.UnitClassType].ID] = GameInfo.Units[row.UnitType].ID;
		end
	end
	for row in DB.Query("SELECT * FROM ITG_UniqueUnitPromotions") do
		local unitType = GameInfo.Units[row.UnitType].ID;
		if not FreePromotionsOfUnitType[unitType] then
			FreePromotionsOfUnitType[unitType] = {};
		end
		table.insert(FreePromotionsOfUnitType[unitType], GameInfo.UnitPromotions[row.PromotionType].ID)
	end
	for row in DB.Query("SELECT * FROM ITG_SubculturesUnits") do
		local scIndex = ITG_SubcultureTypes[row.SubcultureType];
		if not SubCultureClasses[scIndex] then
			SubCultureClasses[scIndex] = {};
		end
		if row.UnitType == "NO_UNIT" then
			SubCultureClasses[scIndex][GameInfo.UnitClasses[row.UnitClassType].ID] = -1;
		else
			SubCultureClasses[scIndex][GameInfo.UnitClasses[row.UnitClassType].ID] = GameInfo.Units[row.UnitType].ID;
		end
	end
	for row in DB.Query("SELECT * FROM ITG_CulturesUnits") do
		local scIndex = ITG_CultureTypes[row.CultureType];
		if not CultureTypesUnitClasses[scIndex] then
			CultureTypesUnitClasses[scIndex] = {};
		end
		if row.UnitType == "NO_UNIT" then
			CultureTypesUnitClasses[scIndex][GameInfo.UnitClasses[row.UnitClassType].ID] = -1;
		else
			CultureTypesUnitClasses[scIndex][GameInfo.UnitClasses[row.UnitClassType].ID] = GameInfo.Units[row.UnitType].ID;
		end
	end
	for row in DB.Query("SELECT * FROM ITG_SubcultureUUExclusive") do
		local scIndex = ITG_SubcultureTypes[row.SubcultureType];
		if not SubCultureClassesOneFrom[scIndex] then
			SubCultureClassesOneFrom[scIndex] = {};
		end
		if not SubCultureClassesOneFrom[scIndex][row.Type] then
			SubCultureClassesOneFrom[scIndex][row.Type] = {};
		end
		table.insert(SubCultureClassesOneFrom[scIndex][row.Type], {GameInfo.UnitClasses[row.UnitClassType].ID, GameInfo.Units[row.UnitType].ID})
	end
	for row in DB.Query("SELECT * FROM ITG_CultureUU") do
		local scIndex = ITG_CultureTypes[row.CultureType];
		if not UniqueUnitsDefCultureTypes[scIndex] then
			UniqueUnitsDefCultureTypes[scIndex] = {};
		end
		local unitclassID = GameInfo.UnitClasses[row.UnitClassType].ID;
		if not UniqueUnitsDefCultureTypes[scIndex][unitclassID] then
			UniqueUnitsDefCultureTypes[scIndex][unitclassID] = {};
		end
		table.insert(UniqueUnitsDefCultureTypes[scIndex][unitclassID], {GameInfo.Units[row.UnitType].ID, row.Multiple, row.Chance})
	end
	for row in DB.Query("SELECT * FROM ITG_SubcultureUU") do
		local scIndex = ITG_SubcultureTypes[row.SubcultureType];
		if not UniqueUnitsDefSubcultureTypes[scIndex] then
			UniqueUnitsDefSubcultureTypes[scIndex] = {};
		end
		local unitclassID = GameInfo.UnitClasses[row.UnitClassType].ID;
		if not UniqueUnitsDefSubcultureTypes[scIndex][unitclassID] then
			UniqueUnitsDefSubcultureTypes[scIndex][unitclassID] = {};
		end
		table.insert(UniqueUnitsDefSubcultureTypes[scIndex][unitclassID], {GameInfo.Units[row.UnitType].ID, row.Multiple, row.Chance})
	end
end
DefineUnitGeneratingDB()

local FakeTrainCultureTypes = {
[1] = {},
[2] = {},
[3] = {
[GameInfoTypes.UNITCLASS_ITG_SIEGE_TOWER] = GameInfoTypes.UNIT_ITG_SIEGE_TOWER_SUPPORT,
[GameInfoTypes.UNITCLASS_ITG_BATTERING_RAM] = GameInfoTypes.UNIT_ITG_BATTERING_RAM_SUPPORT,
},
[4] = {
[GameInfoTypes.UNITCLASS_ITG_SIEGE_TOWER] = GameInfoTypes.UNIT_ITG_SIEGE_TOWER_SUPPORT,
[GameInfoTypes.UNITCLASS_ITG_BATTERING_RAM] = GameInfoTypes.UNIT_ITG_BATTERING_RAM_SUPPORT,
},
[5] = {
[GameInfoTypes.UNITCLASS_ITG_SIEGE_TOWER] = GameInfoTypes.UNIT_ITG_SIEGE_TOWER_SUPPORT,
[GameInfoTypes.UNITCLASS_ITG_BATTERING_RAM] = GameInfoTypes.UNIT_ITG_BATTERING_RAM_SUPPORT,
},
[6] = {
[GameInfoTypes.UNITCLASS_ITG_SIEGE_TOWER] = -1,
[GameInfoTypes.UNITCLASS_ITG_BATTERING_RAM] = GameInfoTypes.UNIT_ITG_BATTERING_RAM_SUPPORT,
},
[7] = {
[GameInfoTypes.UNITCLASS_ITG_SIEGE_TOWER] = -1,
[GameInfoTypes.UNITCLASS_ITG_BATTERING_RAM] = -1,
},
[8] = {
[GameInfoTypes.UNITCLASS_ITG_SIEGE_TOWER] = -1,
[GameInfoTypes.UNITCLASS_ITG_BATTERING_RAM] = -1,
},
[9] = {
[GameInfoTypes.UNITCLASS_ITG_SIEGE_TOWER] = -1,
[GameInfoTypes.UNITCLASS_ITG_BATTERING_RAM] = -1,
},
[10] = {},
[11] = {},
}
local FakeTrainSubcultureTypes = {
[1] = {},
[2] = {},
[3] = {},
[4] = {
[GameInfoTypes.UNITCLASS_ITG_SIEGE_TOWER] = -1,
[GameInfoTypes.UNITCLASS_ITG_BATTERING_RAM] = -1,
},
[5] = {},
[6] = {},
[7] = {},
[8] = {},
[9] = {},
[10] = {},
[11] = {},
[12] = {},
[13] = {},
[14] = {}
}

local BadBarbarianCombatClasses = {
["UNITCOMBAT_SUBMARINE"] = true,
["UNITCOMBAT_CARRIER"] = true,
["UNITCOMBAT_ARMOR"] = true,
}

function InitializeUnitClass(it, iclass)
	gT.Players[it].UnitClassXP[iclass] = 0;
	gT.Players[it].UnitClassTreshold[iclass] = 1;
	gT.Players[it].UnitClassPromotions[iclass] = {};
	gT.Players[it].UnitClassAIPromPath[iclass] = GetRandomValidAIPromPathForClass(iclass, it);
	gT.Players[it].UnitClassDelayedXPGain[iclass] = 0;
	gT.Players[it].UnitClassXPNeededNext[iclass] = 50;
	gT.Players[it].UnitClassNumPromotions[iclass] = 0;
	gT.Players[it].UnitClassMaxNumPromotions[iclass] = 5;
	Players[it]:SetUnitClassSupplyRequired(iclass, true)
end

function AssignInitialUnits()
	gT.Globals.UniqueUnitsUsed = {}
	for i, v in pairs(BarbariansUnitClasses) do
		if i ~= gT.Globals.ScenarioType then
			for iClass, iType in pairs(v) do
				Players[63]:SetPlayerUniqueUnit(iClass, -1)
			end
		end
	end
	for iClass, iType in pairs(BarbariansUnitClasses[ gT.Globals.ScenarioType]) do
		Players[63]:SetPlayerUniqueUnit(iClass, iType)
	end
	for it, gdata in pairs(gT.Players) do
		local gTab = {}
--		gT.Players[it].UniqueUnits = {}
		DoResourceStoredPreInitUnit(it)
		DoUpdateScoutReplacement(it) --also archaeologists
		SupplyBaseValDirty(it);
		DoDirtyUnitGoldCost(it);
		for iclass, itype in pairs(CultureTypesUnitClasses[gdata.CultureType]) do
			InitializeUnitClass(it, iclass)
			gTab[iclass] = {itype, true};
		end
--		for iclass, itype in pairs(hTab) do
--			SetUniqueUnitProperly(it, iclass, itype, true)
--		end
		for iclass, data in pairs(CombatUnitsNoXP[gT.Globals.ScenarioType] or {}) do
			SetUniqueUnitProperly(it, iclass, data[1], true)
			Players[it]:SetUnitClassSupplyRequired(iclass, data[2])
		end
		if gdata.SubcultureType then
			for iclass, itype in pairs(SubCultureClasses[gdata.SubcultureType] or {}) do
				gTab[iclass] = {itype, true};
				if not gT.Players[it].UnitClassXP[iclass] then
					InitializeUnitClass(it, iclass)
				end
			end
			for i, b in pairs(SubCultureClassesOneFrom[gdata.SubcultureType] or {}) do
				local tk = LastRandom(1,#b);
				gTab[b[tk][1]] = {b[tk][2], false}; 
			end
			for iclass, pData in pairs(UniqueUnitsDefSubcultureTypes[gdata.SubcultureType] or {}) do
				local optiondata = Shuffle(pData)
				for j, data in pairs(optiondata) do
					if LastChance(data[3]) then
						if data[2] or (not gT.Globals.UniqueUnitsUsed[data[1]]) then
							gTab[iclass] =  {data[1], false};
			--				table.insert(gT.Players[it].UniqueUnits, data[1])
							gT.Globals.UniqueUnitsUsed[data[1]] = (gT.Globals.UniqueUnitsUsed[data[1]] or 0) + 1;
							break;
						end
					end
				end
			end
		else
			for iclass, pData in pairs(UniqueUnitsDefCultureTypes[gdata.CultureType] or {}) do
				local optiondata = Shuffle(pData)
				for j, data in pairs(optiondata) do
					if LastRandom(1,100) <= data[3] then
						if data[2] or (not gT.Globals.UniqueUnitsUsed[data[1]]) then
							gTab[iclass] =  {data[1], false};
			--				table.insert(gT.Players[it].UniqueUnits, data[1])
							gT.Globals.UniqueUnitsUsed[data[1]] = (gT.Globals.UniqueUnitsUsed[data[1]] or 0) + 1;
							break;
						end
					end
				end
			end
		end
		for iclass, itype in pairs(gTab) do
			if (not gdata.SpecialUULog) or (not gdata.SpecialUULog[iclass]) then -- this means it was already done
				SetUniqueUnitProperly(it, iclass, itype[1], itype[2])
			end
			DoInitialClassXPLevel(it, iclass)
			Players[it]:SetUnitClassSupplyRequired(iclass, true) --safe repeat
		end
		if Players[it]:IsMinorCiv() then
			for iclass, itype in pairs(gTab) do
				DoInitialCSUpgradePath(it, iclass)
			end
			DoMinorCivInitialUnitStuff(it)
		else
			DoMajorCivInitialUnitStuff(it)
		end
		DoResourceStoredPostInitUnit(it)
	end
	NowThatUnitsAreGeneratedCalculateSpecial()
end

function DoResourceStoredPreInitUnit(it)
	if gT.Globals.ScenarioType == 4 then
		Players[it]:SetResourceStored(rHorse, 120);
		Players[it]:SetResourceStored(rIron, 120);
	end
end

function DoResourceStoredPostInitUnit(it)
	if gT.Globals.ScenarioType == 4 then
		Players[it]:SetResourceStored(rHorse, math.min(Players[it]:GetNumResourceTotal(rHorse, true) * LastRandom(1,3) + LastRandom(0, 12), GetMaxResourceStored(it,rHorse)));
		Players[it]:SetResourceStored(rIron, math.min(Players[it]:GetNumResourceTotal(rIron, true) * LastRandom(1,3) + LastRandom(0, 12), GetMaxResourceStored(it,rIron)));
	end
end

function SetUniqueUnitProperly(iPlayer, iclass, itype, bDoNotLog)
	local currentu = Players[iPlayer]:GetPlayerUniqueUnit(iclass);
	local tab = {}
	local commandT = {}
	if not bDoNotLog then
		DoLogSpecialUnitForPlayer(iPlayer, iclass, itype)
	end
	if (currentu ~= -1) and (itype ~= -1) then
		for iCity in Players[iPlayer]:Cities() do
			if iCity:GetUnitProduction(currentu) > 0 then
				table.insert(tab, {iCity, iCity:GetUnitProduction(currentu)})
				iCity:SetUnitProduction(currentu, 0)
			end
			if iCity:GetProductionUnit() == currentu then
				table.insert(commandT, iCity)
			end
		end	
	end
	if gT.Players[iPlayer].AltUnlockUnits[iclass] and (itype ~= -1) then
		Players[iPlayer]:DoOverideUnitTechRequirement(itype, true)
	end
	Players[iPlayer]:SetPlayerUniqueUnit(iclass, itype)
	if gT.Players[iPlayer].AltUnlockUnits[iclass] and (currentu ~= -1) then
		if Players[iPlayer]:IsUnitTechRequirementOverridden(currentu) then
			Players[iPlayer]:DoOverideUnitTechRequirement(currentu, false)
		end
	end
	for i, promID in pairs(FreePromotionsOfUnitType[itype] or {}) do
		gT.Players[iPlayer].UnitClassPromotions[iclass][promID] = -1;
	end
	for i, v in pairs(tab) do
		v[1]:SetUnitProduction(itype, v[2])
	end
	for i, iCity in pairs(commandT) do
		iCity:PushOrder(OrderTypes.ORDER_TRAIN, itype)
	end
end

function DoMinorCivInitialUnitStuff(it)
	if Players[it]:GetNumCities() > 0 then -- is alive can be false?
		local gType = gT.Players[it].CultureType;
		local minPerCity = GetDBCulturePlayerInfo(it,"CSMinStartUnitsPerCity");
		local maxPerCity = GetDBCulturePlayerInfo(it,"CSMaxStartUnitsPerCity");
		local maxExtraBias = GetDBCulturePlayerInfo(it,"CSBiasMaxUnitsPerCity");
		for iCity in Players[it]:Cities() do
			local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
			local numunits = LastRandom(minPerCity, maxPerCity);
			if LastRandom(1,100) <= maxExtraBias then
				 numunits = math.max(numunits, LastRandom(minPerCity, maxPerCity))
			end
			local getnaval, fishboats, lPlot, cMakeWorkboats = GetCityNavalUnitsBasedOnBasinSize(iCity:Plot(), gType)
			gT.Cities[cID].CanDoWorkboats = cMakeWorkboats;
			
			fishboats = math.min(2, fishboats);
			getnaval = math.min(getnaval, math.ceil(iCity:GetPopulation() / 2))
			fishboats = math.min(getnaval, math.ceil(iCity:GetPopulation() / 3))
			if LastRandom(1, 2) == 2 then
				numunits = numunits + 1;
			end
			for i = 1, numunits do
				CreateRandomStartingUnit(it, iCity:Plot(), "DOMAIN_LAND")
			end
			for i = 1, getnaval do
				CreateRandomStartingUnit(it, lPlot, "DOMAIN_SEA")
			end
			for i = 1, fishboats do
				CreateFishingBoats(it, lPlot)
			end
		end
		for indexplot, data in pairs(gT.Forts) do
			if data.Player == it then
				local numunits = LastRandom(0, 1);
				if LastRandom(1,100) <= 25 then
					 numunits = math.max(numunits, LastRandom(0, 1))
				end
				for i = 1, numunits do
					CreateRandomStartingUnit(it, Map.GetPlotByIndex(indexplot), "DOMAIN_LAND")
				end
			end
		end
		for indexplot, data in pairs(gT.Colonies) do
			if data.Player == it then
				if LastRandom(1, 5) < 4 then
					CreateRandomStartingUnit(it, Map.GetPlotByIndex(indexplot), "DOMAIN_LAND")
				end
			end
		end
		local extras = LastRandom(0,1);
		local sPlot = Map.GetPlotByIndex(gT.Players[it].StartingPlot);
		if not sPlot then
			print("No starting plot 7543")
		end
		for i = 1, extras do
			CreateRandomStartingUnit(it, sPlot, "DOMAIN_LAND")
		end
	end
end

function StartingUnitsForCity(iCity, gType, ncit, it)
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
	local numunits = LastRandom(0, GetDBCulturePlayerInfo(it,"MaxStartUnitsPerCity")) + LastRandom(0, math.max(0, 4 - ncit));
	if LastRandom(1,100) <= GetDBCulturePlayerInfo(it,"BiasMaxStartUnits") then
		numunits = math.max(numunits, LastRandom(0, GetDBCulturePlayerInfo(it,"MaxStartUnitsPerCity")) + LastRandom(0, math.max(0, 4 - ncit)))
	end
	if Players[it]:GetNumUnits() > 25 then
		if numunits > 0 then
			numunits = LastRandom(0, numunits)
		end
	end
	local getnaval, fishboats, lPlot, cMakeWorkboats = GetCityNavalUnitsBasedOnBasinSize(iCity:Plot(), gType)
	gT.Cities[cID].CanDoWorkboats = cMakeWorkboats;
	getnaval = math.min(getnaval, math.ceil(iCity:GetPopulation() / 2))
	fishboats = math.min(fishboats, math.ceil(iCity:GetPopulation() / 3))
	local bDamaged = false;
	if iCity:GetOriginalOwner() ~= it then
		fishboats = 0;
		if iCity:GetDamage() > 0 then
			bDamaged = true;
		end
	end
	if gT.Players[it].SubcultureType == 14 then
		if getnaval > 0 then
			getnaval = getnaval + numunits;
			numunits = 0;
		end
	end
	for i = 1, numunits do
		CreateRandomStartingUnit(it, iCity:Plot(), "DOMAIN_LAND", true, bDamaged)
	end
	for i = 1, getnaval do
		CreateRandomStartingUnit(it, lPlot, "DOMAIN_SEA", true, bDamaged)
	end
	if gT.Globals.ScenarioType == 5 then
		local numair = LastRandom(-3, 2)
		for i = 1, numair do
			CreateRandomStartingUnit(it, iCity:Plot(), "DOMAIN_AIR", false, bDamaged)
		end
	end
	for i = 1, fishboats do
		CreateFishingBoats(it, lPlot)
	end
end
function DoMajorCivInitialUnitStuff(it)
	--cities
	--colonies
	--forts
	--navalforces
	local ncit = math.min(5, Players[it]:GetNumCities());
	local tCities = {}
	local gType = gT.Players[it].CultureType;
	for iCity in Players[it]:Cities() do
		if iCity:GetOriginalOwner() ~= it then
			StartingUnitsForCity(iCity, gType, ncit, it)
		else
			table.insert(tCities, iCity)
		end
	end
	tCities = Shuffle(tCities);
	for i, iCity in pairs(tCities) do
		if i < 6 then
			StartingUnitsForCity(iCity, gType, ncit, it)
		else
			--stuff that has to be done anyway
			local getnaval, fishboats, lPlot, cMakeWorkboats = GetCityNavalUnitsBasedOnBasinSize(iCity:Plot(), gType)
			local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
			gT.Cities[cID].CanDoWorkboats = cMakeWorkboats;
		end
	end
	local tFortsUnit = 3;
	local nForts = 0;
	for indexplot, data in pairs(gT.Forts) do
		if data.Player == it then
			nForts = nForts + 1;
		end
	end
	for indexplot, data in pairs(gT.Forts) do
		if data.Player == it then
			if IsFortValidForInitUnits(indexplot, it) then
				local numunits = LastRandom(0, tFortsUnit);
				if Players[it]:GetNumUnits() > 15 then
					if numunits > 0 then
					numunits = LastRandom(0, numunits)
				end
				elseif LastRandom(1,100) <= 25 then
					 numunits = math.max(numunits, LastRandom(0, tFortsUnit))
				end
				for i = 1, numunits do
					CreateRandomStartingUnit(it, Map.GetPlotByIndex(indexplot), "DOMAIN_LAND")
				end
				tFortsUnit = math.max(tFortsUnit - 1, 1)
			end
		end
	end
	for indexplot, data in pairs(gT.Colonies) do
		if data.Player == it then
			if LastRandom(1,2) == 2 then
				CreateRandomStartingUnit(it, Map.GetPlotByIndex(indexplot), "DOMAIN_LAND")
			end
		end
	end
	if Players[it]:GetNumUnits() < 15 then
		local extras = LastRandom(0,3);
		local sPlot = Map.GetPlotByIndex(gT.Players[it].StartingPlot);
		for i = 1, extras do
			CreateRandomStartingUnit(it, sPlot, "DOMAIN_LAND")
		end
	end
	CreateAScoutStarting(it, GetNiceNumberOfScouts(it, gType))
	CreateInitialSupportPromotions(it)
end

function GetNiceNumberOfScouts(it, gType)
	local imin, imax = 0, 10;
	imax = math.min(imax, math.floor(Players[it]:GetNumCities() * 2.5))
	imin = math.max(imin, math.floor(Players[it]:GetNumCities() * 0.3))
	if gType == 1 then
		imin = imin + 1;
	end
	return math.min(3, LastRandom(imin, imax))
end

local HasCustomRandomValueForInitUnit = {
[GameInfoTypes.UNITCLASS_TREBUCHET] = 70,
[GameInfoTypes.UNITCLASS_CATAPULT] = 70,
[GameInfoTypes.UNITCLASS_ITG_SIEGE_TOWER] = 60,
[GameInfoTypes.UNITCLASS_ITG_BATTERING_RAM] = 50,
}
function GetRandomUnitTypeOfDomain(iPlayer, domain, bIgnoreRes)
	local hTab = {}
	for iclass, btype in pairs(CultureTypesUnitClasses[gT.Players[iPlayer].CultureType]) do
		local itype = Players[iPlayer]:GetPlayerUniqueUnit(iclass);
		if itype ~= -1 then
			if GameInfo.Units[itype].Domain == domain then
				local iTech = GameInfo.Units[itype].PrereqTech;
				if (not iTech) or Teams[Players[iPlayer]:GetTeam()]:IsHasTech(GameInfoTypes[iTech]) then
					if bIgnoreRes or CheckResourceOfUnitAvailable(iPlayer, itype, true) then
						if HasCustomRandomValueForInitUnit[iclass] then
							table.insert(hTab, {itype, HasCustomRandomValueForInitUnit[iclass]}) 
						else
							table.insert(hTab, {itype, 100}) 
						end
					end
				end
			end
		end
	end
	if #hTab > 0 then
		return GetRandomFromBiasedChanceTable(hTab);
	end
end

function CreateRandomStartingUnit(iPlayer, dPlot, domain, bMove, bDamaged)
	local jType = GetRandomUnitTypeOfDomain(iPlayer, domain, DBScenario.StrategicResStorage)
	if jType then
		local jUnit = Players[iPlayer]:InitUnit(jType, dPlot:GetX(), dPlot:GetY())
		jUnit:JumpToNearestValidPlot()
		if bDamaged then
			jUnit:ChangeDamage(LastRandom(0, 33))
		end
		ITGSCNewUnitCreated(iPlayer, jUnit, nil, true, nil, nil, bMove or true)
	end
end

function GenerateFreeUnits(iPlayer, amount, domainString)
	local sPlot;
	if Players[iPlayer]:GetCapitalCity() then
		sPlot = Players[iPlayer]:GetCapitalCity():Plot();
	else
		for iUnit in Players[iPlayer]:Units() do
			sPlot = iUnit:GetPlot();
			break;
		end
	end
	if sPlot then
		for i = 1, amount do
			CreateRandomStartingUnit(iPlayer, sPlot, domainString)
		end
	end
end

function CreateFishingBoats(it, dPlot)
	local fishB = GameInfoTypes.UNIT_WORKBOAT;
	local jUnit = Players[it]:InitUnit(fishB, dPlot:GetX(), dPlot:GetY())
	local jID = jUnit:GetID() .. "," .. it .. "," .. jUnit:GetGameTurnCreated();
	gT.Players[it].UnitsByID[jID] = {XP = 0}
	jUnit:JumpToNearestValidPlot()
	NewlyCreatedInitialUnitMovedAround(jUnit)
end

function GetCityNavalUnitsBasedOnBasinSize(iPlot, gType)
	local iSize = 0;
	local gPlot;
	local cWorkboats = false;
	for j = 0, 5 do
		local jPlot = Map.PlotDirection(iPlot:GetX(), iPlot:GetY(), j);
		if jPlot then
			if jPlot:IsWater() then
				if not jPlot:IsImpassable() then
					local oceanID = GetOceanID(jPlot)
					local aSize = GetSizeOfOcean(oceanID)
					if gT.ValidOceanFish[oceanID] then
						cWorkboats = true;
					end
					if aSize > iSize then
						gPlot = jPlot;
						iSize = aSize;
					end
				end
			end
		end
	end
	if iSize > 20 then
		local numval = 0;
		for i = 1, #SeaLevelTable do
			if iSize < SeaLevelTable[i][1] then
				numval = i;
				break
			end
		end
		local numnaval = LastRandom(SeaLevelTable[numval][2][gType][1],SeaLevelTable[numval][2][gType][2])
		local numfishb = 0
		if cWorkboats then
			numfishb = LastRandom(SeaLevelTable[numval][3][gType][1],SeaLevelTable[numval][3][gType][2])
		end
		return numnaval, numfishb, gPlot, cWorkboats;
	else
		return 0, 0, nil, false;
	end
end

function CreateAScoutStarting(iPlayer, num)
	local theScout = Players[iPlayer]:GetPlayerUniqueUnit(GameInfoTypes.UNITCLASS_SCOUT)
	if theScout == -1 then
		return;
	end
	--GameInfoTypes.UNIT_SCOUT
	if Players[iPlayer]:GetNumUnitsOutOfSupply() > 0 then
		return
	end
	local iTeam = Players[iPlayer]:GetTeam();
	local bTab = {}
	local bCanEmbark = (-1 ~= GetTresholdOfEmbarkDamage(iPlayer))
	local iStartCont = GetContID(Players[iPlayer]:GetStartingPlot(), "eokcerf")
	for i = 0, Map:GetNumPlots() - 1, 1 do
		local fPlot = Map.GetPlotByIndex( i );
		if fPlot:IsRevealed(iTeam) then
			if not fPlot:IsImpassable() then
				if not fPlot:IsCity() then
					if fPlot:GetNumUnits() == 0 then
						if not fPlot:IsWater() then
							if bCanEmbark or (iStartCont == GetContID(fPlot, "Verf")) then
								table.insert(bTab, fPlot)
							end
						end
					end
				end
			end
		end
	end
	if #bTab > num then
		bTab = Shuffle(bTab)
		for i = 1, num do
			local dPlot = bTab[i];
			local jUnit = Players[iPlayer]:InitUnit(theScout, dPlot:GetX(), dPlot:GetY())
			if Players[iPlayer]:GetNumUnitsOutOfSupply() > 0 then
				jUnit:Kill(false, -1)
				return
			end
			jUnit:JumpToNearestValidPlot()
			jUnit:SetUnitAIType(GameInfoTypes.UNITAI_EXPLORE)
			ITGSCNewUnitCreated(iPlayer, jUnit, nil, true, nil, nil, true)
		end
	else
		print("Too many scouts initial", #bTab, num, iPlayer)
	end
end

function CheckResourceOfUnitAvailable(iPlayer, itype, bRandom)
	local type = GameInfo.Units[itype].Type;
	for row in GameInfo.Unit_ResourceQuantityRequirements() do
		if type == row.UnitType then
			local afterRes = Players[iPlayer]:GetNumStrategicResourceAvailable(GameInfoTypes[row.ResourceType], true) - row.Cost;
			if afterRes < 0 then
				if LastRandom(1, 1 + -1 *afterRes) > 1 then
					return false;
				end
			end
		end
	end
	return true;
end

function GetAnyValidMilitaryUnitType(iPlayer, domain, bBarb, bPlayer)
	if bBarb and (gT.Globals.ScenarioType == 5) then
		if GameInfo.Domains[domain].ID == dLand then
			return GameInfoTypes.UNIT_ITG_INFANTRY_BARB
		elseif GameInfo.Domains[domain].ID == dSea then
			return GameInfoTypes.UNIT_ITG_DESTROYER_BARB 
		else
			return -1;
		end
	end
	local hTab = {}
	local iTeam = Players[iPlayer]:GetTeam();
	for iclass, btype in pairs(CultureTypesUnitClasses[gT.Players[iPlayer].CultureType]) do
		local itype = Players[iPlayer]:GetPlayerUniqueUnit(iclass);
		if itype ~= -1 then
			if (GameInfo.Units[itype].Domain == domain) or (GameInfoTypes[GameInfo.Units[itype].Domain] == domain) then
				if (not bBarb) or (not BadBarbarianCombatClasses[GameInfo.Units[itype].CombatClass]) then
					local iTech = GameInfo.Units[itype].PrereqTech;
					if (not iTech) or Teams[iTeam]:IsHasTech(GameInfoTypes[iTech]) or Players[iPlayer]:IsUnitTechRequirementOverridden(itype) then
						if bPlayer and (bPlayer ~= 63) and (Players[bPlayer]:GetPlayerUniqueUnit(iclass) ~= -1) then
							table.insert(hTab, Players[bPlayer]:GetPlayerUniqueUnit(iclass)) 
						else
							table.insert(hTab, itype) 
						end				
					end
				end
			end
		end
	end
	if #hTab > 0 then
		local jType = hTab[LastRandom(1,#hTab)]
		return jType;
	else
		print("Found no unit for: ", iPlayer, domain, bBarb, bPlayer)
		return -1;
	end
end

function GetPlayerBonusPerVeterancy(iPlayer, jlevel)
	local bPerLevel = 3 + (gT.Players[iPlayer].WonderRuinsEffects[18] or 0);
	if gT.Players[iPlayer].CultBonusOn[116] then
		bPerLevel = bPerLevel + 2;
	end
	if gT.Players[iPlayer].HasIdeology[90] then
		bPerLevel = bPerLevel + jlevel;
	end
	return bPerLevel * jlevel;
end

function GetUnitBonusFromVeterancy(iUnit, iPlayer)
	return GetPlayerBonusPerVeterancy(iPlayer, iUnit:GetLevel() - 1)
end

function AdjustCombatBonusFromLevel(iUnit, uID, iPlayer)
	if not IsCombatClassITGUnit(iUnit:GetUnitClassType(), iPlayer)  then
		return;
	end
	DoTestITGVeteranPromotion(iPlayer, iUnit:GetID())
	local bonus = GetUnitBonusFromVeterancy(iUnit, iPlayer);
	iUnit:ChangeCombatPercentModifier(bonus - (gT.Players[iPlayer].UnitsByID[uID].VeteranModifier or 0))
	gT.Players[iPlayer].UnitsByID[uID].VeteranModifier = bonus;
end

function RemoveTooManyWorkBoats()
	for g, gdata in pairs(gT.Players) do
		local up = Players[g]:GetUnitClassCountPlusMaking(GameInfoTypes.UNITCLASS_WORKBOAT) - GetSupportedFishingBoats(g);
		if up > 0 then
			local hTab = {}
			for iUnit in Players[g]:Units() do
				if iUnit:GetUnitClassType() == GameInfoTypes.UNITCLASS_WORKBOAT then
					table.insert(hTab, iUnit)
				end
			end
			hTab = Shuffle(hTab);
			for i = 1, up do
				if hTab[i] then
					hTab[i]:Kill(false, -1);
				end
			end
		end
	end
end

function DoUpdateScoutReplacement(iPlayer)
	if (not Players[iPlayer]:IsMinorCiv()) and (GetDBCulturePlayerInfo(iPlayer, "CanHaveScouts") or gT.Players[iPlayer].CultBonusOn[70]) then
		Players[iPlayer]:SetPlayerUniqueUnit(GameInfoTypes.UNITCLASS_SCOUT, GameInfoTypes.UNIT_SCOUT)
	else
		Players[iPlayer]:SetPlayerUniqueUnit(GameInfoTypes.UNITCLASS_SCOUT, -1)
	end
	if (not Players[iPlayer]:IsMinorCiv()) and DBScenario.Archaeology then
		Players[iPlayer]:SetPlayerUniqueUnit(GameInfoTypes.UNITCLASS_ARCHAEOLOGIST, GameInfoTypes.UNIT_ITG_ARCHAEOLOGIST)
	else
		Players[iPlayer]:SetPlayerUniqueUnit(GameInfoTypes.UNITCLASS_ARCHAEOLOGIST, -1)
	end
	Players[iPlayer]:SetUnitClassSupplyRequired(GameInfoTypes.UNITCLASS_SCOUT, true)
	Players[iPlayer]:SetUnitClassSupplyRequired(GameInfoTypes.UNITCLASS_ARCHAEOLOGIST, true)
	local cType = gT.Players[iPlayer].CultureType;
	local sbctype = gT.Players[iPlayer].SubcultureType
	if FakeTrainCultureTypes[cType] then
		for fakeclass, type in pairs(FakeTrainCultureTypes[cType]) do
			Players[iPlayer]:SetPlayerUniqueUnit(fakeclass, type)
		end
	else
		print("Missing culture data 2312", cType)
	end
	if sbctype then
		if FakeTrainSubcultureTypes[sbctype] then
			for fakeclass, type in pairs(FakeTrainSubcultureTypes[sbctype]) do
				Players[iPlayer]:SetPlayerUniqueUnit(fakeclass, type)
			end
		else
			print("Missing subculture data 2312", sbctype)
		end
	end
end

function CreateInitialSupportPromotions(iPlayer)
	if gT.Globals.ScenarioType == 2 then
		local itype = Players[iPlayer]:GetPlayerUniqueUnit(GameInfoTypes.UNITCLASS_ITG_BATTERING_RAM);
		if itype ~= -1 then
			local iTech = GameInfo.Units[itype].PrereqTech;
			if (not iTech) or Teams[Players[iPlayer]:GetTeam()]:IsHasTech(GameInfoTypes[iTech]) then
				DoAddPercentageOfSupportProm(iPlayer, GameInfoTypes.PROMOTION_ITG_BATTERING_RAM_SUPPORT, 25)
			end
		end
		itype = Players[iPlayer]:GetPlayerUniqueUnit(GameInfoTypes.UNITCLASS_ITG_SIEGE_TOWER);
		if itype ~= -1 then
			local iTech = GameInfo.Units[itype].PrereqTech;
			if (not iTech) or Teams[Players[iPlayer]:GetTeam()]:IsHasTech(GameInfoTypes[iTech]) then
				DoAddPercentageOfSupportProm(iPlayer, GameInfoTypes.PROMOTION_ITG_SIEGE_TOWER_SUPPORT, 15)
			end
		end
	end
end

function DoAddPercentageOfSupportProm(iPlayer, prom, perc)
	local tab = {}
	for iUnit in Players[iPlayer]:Units() do
		if CanHaveBatteringRamSupport(iUnit, iPlayer, prom, true) then
			table.insert(tab, iUnit)
		end
	end
	local toAdd = math.min(1, math.floor(#tab * perc / 100))
	for i, h in pairs(tab) do
		if i <= toAdd then
			h:SetHasPromotion(prom, true)
		else
			return;
		end
	end
end

function GetAllPossibleUnitsFromClasses(sc, cTypes, sTypes)
	local alRtab = {[-1] = true}
	local fTab = {}
	local function AddIntoTypeTab(unitType)
		if not alRtab[unitType] then
			table.insert(fTab, unitType)
			alRtab[unitType] = true;
		end
	end
	for iClass, data in pairs(CombatUnitsNoXP[sc] or {}) do 
		AddIntoTypeTab(data[1])
	end
	for c, cType in pairs(cTypes) do
		for tHClass, tHunit in pairs(CultureTypesUnitClasses[cType]) do
			AddIntoTypeTab(tHunit)
		end
		for tHClass, sb in pairs(UniqueUnitsDefCultureTypes[cType] or {}) do
			for j, tDD in pairs(sb) do
				AddIntoTypeTab(tDD[1])
			end
		end
	end
	for c, cType in pairs(sTypes) do
		for tHClass, sb in pairs(SubCultureClassesOneFrom[cType] or {}) do
			for j, tDD in pairs(sb) do
				AddIntoTypeTab(tDD[2])
			end
		end
		for tHClass, tHunit in pairs(SubCultureClasses[cType] or {}) do
			AddIntoTypeTab(tHunit)
		end
		for tHClass, sb in pairs(UniqueUnitsDefSubcultureTypes[cType] or {}) do
			for j, tDD in pairs(sb) do
					AddIntoTypeTab(tDD[1])
			end
		end
	end
	return fTab;
end

function IsFortValidForInitUnits(indexplot, it)
	local iPlot = Map.GetPlotByIndex(indexplot)
	for iCity in Players[it]:Cities() do
		if Map.PlotDistance(iCity:GetX(), iCity:GetY(), iPlot:GetX(), iPlot:GetY()) < 5 then
			return false
		end
	end
	return true;
end

local sortHInfo1
function SortFuncUnitsCivilopedia(a, b)
	if (a[2] == -1) and (b[2] ~= -1) then
		return false;
	elseif (a[2] ~= -1) and (b[2] == -1) then
		return true;
	elseif a[2] ~= b[2] then
		return Locale.Compare(a[3], b[3]) == -1
	else
		return Locale.Compare(Locale.Lookup(GameInfo[sortHInfo1][a[1]].Description), Locale.Lookup(GameInfo[sortHInfo1][b[1]].Description)) == -1
	end
end

function FinalizeCivilopediaUnits(tab, type, tkey, tClass)
	local byClassN = {}
	sortHInfo1 = type;
	local byClassString = {}
	for i, unit in pairs(tab) do 
		local iClass = GameInfoTypes[GameInfo[type][unit][tkey]]
		byClassN[iClass] = (byClassN[iClass] or 0) + 1;
		if not byClassString[iClass] then
			byClassString[iClass] = Locale.Lookup(GameInfo[tClass][iClass].Description)
		end
	end
	local preSort = {}
	for i, unit in pairs(tab) do 
		local iClass = GameInfoTypes[GameInfo[type][unit][tkey]]
		if byClassN[iClass] == 1 then
			table.insert(preSort, {unit, -1, byClassString[iClass]})
		else
			table.insert(preSort, {unit, iClass, byClassString[iClass]})
		end
	end
	table.sort(preSort, SortFuncUnitsCivilopedia)
	return preSort;
end