local promEngRoads = GameInfoTypes.PROMOTION_ROAD_ENGINEER

-- pikeman - > tercio
-- swordsman -> longswordsman - > tercio
-- musketman -> tercio
-- horseman -> knight -> lancer
-- catapult -> trebuchet -> cannon
-- composite -> crossbowman
-- trireme -> caravel

local TableOfPossibleUpgrades = {
[GameInfoTypes.UNITCLASS_PIKEMAN] = GameInfoTypes.UNITCLASS_ITG_NW_TERCIO,
[GameInfoTypes.UNITCLASS_SPEARMAN] = GameInfoTypes.UNITCLASS_PIKEMAN,
[GameInfoTypes.UNITCLASS_LONGSWORDSMAN] = GameInfoTypes.UNITCLASS_ITG_NW_TERCIO,
[GameInfoTypes.UNITCLASS_ITG_NW_TERCIO] = GameInfoTypes.UNITCLASS_RIFLEMAN,
[GameInfoTypes.UNITCLASS_MUSKETMAN] = GameInfoTypes.UNITCLASS_ITG_NW_TERCIO,
[GameInfoTypes.UNITCLASS_KNIGHT] = GameInfoTypes.UNITCLASS_LANCER,
[GameInfoTypes.UNITCLASS_TREBUCHET] = GameInfoTypes.UNITCLASS_CANNON,
[GameInfoTypes.UNITCLASS_COMPOSITE_BOWMAN] = GameInfoTypes.UNITCLASS_CROSSBOWMAN,
[GameInfoTypes.UNITCLASS_CATAPULT] = GameInfoTypes.UNITCLASS_TREBUCHET,
[GameInfoTypes.UNITCLASS_TRIREME] = GameInfoTypes.UNITCLASS_CARAVEL,
[GameInfoTypes.UNITCLASS_HORSEMAN] = GameInfoTypes.UNITCLASS_KNIGHT,
[GameInfoTypes.UNITCLASS_CHARIOT_ARCHER] = GameInfoTypes.UNITCLASS_KNIGHT,
[GameInfoTypes.UNITCLASS_SWORDSMAN] = GameInfoTypes.UNITCLASS_LONGSWORDSMAN,
[GameInfoTypes.UNITCLASS_FRIGATE] = GameInfoTypes.UNITCLASS_ITG_IND_SOTL,
[GameInfoTypes.UNITCLASS_ITG_IND_SOTL] = GameInfoTypes.UNITCLASS_IRONCLAD,
[GameInfoTypes.UNITCLASS_ITG_IND_CANNON_1] = GameInfoTypes.UNITCLASS_ITG_IND_CANNON_2,
[GameInfoTypes.UNITCLASS_CANNON] = GameInfoTypes.UNITCLASS_ITG_IND_CANNON_1,
[GameInfoTypes.UNITCLASS_LANCER] = GameInfoTypes.UNITCLASS_ITG_IND_CUIRASS,
[GameInfoTypes.UNITCLASS_ITG_IND_CUIRASS] = GameInfoTypes.UNITCLASS_CAVALRY,
[GameInfoTypes.UNITCLASS_CROSSBOWMAN] = GameInfoTypes.UNITCLASS_GATLINGGUN,
}

function HasValidTechForUnit(unitT, iPlayer)
	local tech = GameInfo.Units[unitT].PrereqTech;
	if (not tech) or Teams[Players[iPlayer]:GetTeam()]:IsHasTech(GameInfoTypes[tech]) then
		return true;
	end
	return Players[iPlayer]:IsUnitTechRequirementOverridden(unitT);
end

function GetValidUpgradeClassTo(iPlayer, iclass)
	local tab = {}
	local toClass = TableOfPossibleUpgrades[iclass];
	if gT.Players[iPlayer].ObsoleteUnitClasses[toClass] then
		local htab = GetValidUpgradeClassTo(iPlayer, toClass)
		for i, class in pairs(htab) do
			table.insert(tab, class)
		end
	else
		local unitNew = Players[iPlayer]:GetPlayerUniqueUnit(toClass);
		if unitNew ~= -1 and HasValidTechForUnit(unitNew, iPlayer) then
			table.insert(tab, toClass);
		end
	end
	return tab;
end

function PrepareTabOfPossibleUpgradeClasses(iPlayer)
	local sTab = {}
	for iclass, uclass in pairs(TableOfPossibleUpgrades) do
		if not gT.Players[iPlayer].ObsoleteUnitClasses[iclass] then
			if gT.Players[iPlayer].UnitClassXP[iclass] then
				local unitT = Players[iPlayer]:GetPlayerUniqueUnit(iclass);
				if unitT ~= -1 and HasValidTechForUnit(unitT, iPlayer) then
					local classTo = GetValidUpgradeClassTo(iPlayer, iclass)
					for jtt, newclass in pairs(classTo) do
						table.insert(sTab, {iclass, newclass})
					end
				end
			end
		end
	end
	return sTab;
end

function ItIsOkayForScenarioToClaimReligionByPlayer(iPlayer, rel)
	if gT.Globals.ScenarioType == 2 then
		if gT.Players[iPlayer].OfficialReligion ~= rel then
			return false;
		end
	end
	return true;
end

function PreparedDataForClaimReligion(iPlayer)
	local civ = GameInfo.Civilizations[Players[iPlayer]:GetCivilizationType()];
	local data = {BonusType = 14, SpecialType = "Action", TitleText = "Claim Religion", TitleIconIndex = civ.PortraitIndex, TitleIconAtlas = civ.IconAtlas}
	local hTab = {}
	if Players[iPlayer]:GetReligionCreatedByPlayer() < 1 then
		if not gT.Players[iPlayer].SecularState then
			for g, gdata in pairs(gT.Players) do
				local rel = Players[g]:GetReligionCreatedByPlayer();
				if rel > 0 then
					local holyCity = Game.GetHolyCityForReligion(rel, -1);
					if holyCity:GetOwner() == iPlayer then
						if IsFullyControlled(holyCity, iPlayer) then
							if ItIsOkayForScenarioToClaimReligionByPlayer(iPlayer, rel) then
								local legalcost = 10;
								if gT.Players[iPlayer].EverControlReligion[rel] then
									legalcost = 5;
								end
								table.insert(hTab, {Name = "Claim " .. GameInfo.Religions[rel].IconString .. " " .. Locale.Lookup(Game.GetReligionName(rel)) .. " - " .. legalcost .. "[ICON_ITG_LEGAL_POWER]",
									ID = rel,
									PortraitIndex = GameInfo.Religions[rel].PortraitIndex,
									IconAtlas = GameInfo.Religions[rel].IconAtlas,
									Help = "[NEWLINE]You will become rightful owner of " .. GameInfo.Religions[rel].IconString .. " " .. Locale.Lookup(Game.GetReligionName(rel)) .. ".",
									Disabled = (legalcost > gT.Players[iPlayer].LegalPower),
									LegalCost = legalcost
									})	
							end
						end
					end
				end
			end
		end
	end
	data.UniqueOptions = hTab;
	return data;
end

function GetNumTurnsNeededForCSMarriage(iPlayer)
	if HasPlayerTech(iPlayer, GameInfoTypes.TECH_UNIQUE_ITG_MEDIEVAL_AFR_6, "kekoekforf") then
		return 5;
	end
	return 20;
end

function PreparedDataLegalDiplomaticMarriage(iPlayer)
	local civ = GameInfo.Civilizations[Players[iPlayer]:GetCivilizationType()];
	local data = {BonusType = 16, SpecialType = "Action", TitleText = "Incorporate City-State", TitleIconIndex = civ.PortraitIndex, TitleIconAtlas = civ.IconAtlas}
	local hTab = {}
	local minTurnsNeeded = GetNumTurnsNeededForCSMarriage(iPlayer)
	for g, gdata in pairs(gT.Players) do
		if Players[g]:IsMinorCiv() and (Players[g]:GetAlly() == iPlayer) and Players[g]:IsAlive() then
			if not gT.Players[g].GovAsLeader then
				if Players[g]:GetNumCities() > 0 then
					if gdata.LastAllyChange <= GetITGGameTurn() - minTurnsNeeded then
						table.insert(hTab, {Name = "Claim " .. Players[g]:GetName(),
									ID = g,
									Help = "[NEWLINE]You will incorporate all cities and units of " .. Players[g]:GetName() .. " (+" ..  math.floor(Players[g]:GetMinorCivFriendshipWithMajor(iPlayer) / 10) .. "[ICON_ITG_MERITOCRACY]).",
									})	
					end
				end
			end
		end
	end
	data.UniqueOptions = hTab;
	return data;
end

function GetNumLegalCities(iPlayer)
	local t = 0;
	for i, v in pairs(Players) do
		for iCity in v:Cities() do
			if iCity:GetOriginalOwner() == iPlayer then
				t = t + 1;
			end
		end
	end
	return t;
end

function PreparedDataForNewCapital(iPlayer)
	local civ = GameInfo.Civilizations[Players[iPlayer]:GetCivilizationType()];
	local data = {BonusType = 27, SpecialType = "Action", TitleText = "Establish Capital", TitleIconIndex = civ.PortraitIndex, TitleIconAtlas = civ.IconAtlas}
	local hTab = {}
	for iCity in Players[iPlayer]:Cities() do
		if iCity:GetOriginalOwner() == iPlayer then
			if not iCity:IsOriginalMajorCapital() then
				table.insert(hTab, {Name = iCity:GetName(),
							ID = iCity:GetID(),
							Help = "[NEWLINE]This City will become your legal Capital.",
							})	
			end
		end
	end
	data.UniqueOptions = hTab;
	return data;
end

function CanCityBeLegalizedNow(iPlayer, iCity)
	local iOriginalPlayer = iCity:GetOriginalOwner();
	if iOriginalPlayer ~= iPlayer then
		if Players[iOriginalPlayer]:IsMinorCiv() or (GetNumLegalCities(iOriginalPlayer) > 1) then
			if not iCity:IsResistance() then
				if iCity:GetDamage() == 0 then
					local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
					if gT.Cities[cID].LastTimeDamaged <= GetITGGameTurn() - 5 then
						return true;
					end
				end
			end
		end
	end
	return false;
end

function PreparedDataForClaimCities(iPlayer)
	local civ = GameInfo.Civilizations[Players[iPlayer]:GetCivilizationType()];
	local data = {BonusType = 15, SpecialType = "Action", TitleText = "Claim City", TitleIconIndex = civ.PortraitIndex, TitleIconAtlas = civ.IconAtlas}
	local hTab = {}
	for iCity in Players[iPlayer]:Cities() do
		if CanCityBeLegalizedNow(iPlayer, iCity) then
			table.insert(hTab, {Name = "Claim " .. iCity:GetName(),
				ID = iCity:GetID(),
				Help = "[NEWLINE]You will become the original owner of " .. iCity:GetName() .. ".",
				})	
		end
	end
	data.UniqueOptions = hTab;
	return data;
end

function PreparedDataForUpgradeUnitAction(iPlayer)
	local civ = GameInfo.Civilizations[Players[iPlayer]:GetCivilizationType()];
	local data = {BonusType = 12, SpecialType = "Action", TitleText = "Modernize your troops.", TitleIconIndex = civ.PortraitIndex, TitleIconAtlas = civ.IconAtlas}
	local hTab = {}
	local iTeam = Players[iPlayer]:GetTeam();
	local sTab = PrepareTabOfPossibleUpgradeClasses(iPlayer);
	for i, fdata in pairs(sTab) do
		local unitFrom = Players[iPlayer]:GetPlayerUniqueUnit(fdata[1]);
		local unitTo = Players[iPlayer]:GetPlayerUniqueUnit(fdata[2]);
		table.insert(hTab, {Name = Locale.Lookup(GameInfo.Units[unitFrom].Description) .. " - " .. gT.Players[iPlayer].UnitClassXP[fdata[1]] .. "XP / " .. Players[iPlayer]:GetUnitClassCount(fdata[1]),
			ID = fdata[1],
			PortraitIndex = GameInfo.Units[unitFrom].PortraitIndex,
			IconAtlas = GameInfo.Units[unitFrom].IconAtlas,
			Help = "[NEWLINE]Makes " .. Locale.Lookup(GameInfo.Units[unitFrom].Description) .. " obsolete, allowing any existing unit to upgrade into " .. Locale.Lookup(GameInfo.Units[unitTo].Description) .. ".",
			ExtraInfo = fdata[2]
			})	
	end
	data.UniqueOptions = hTab;
	return data;
end

function DoInitialCSUpgradePath(iPlayer, iClass)
	if TableOfPossibleUpgrades[iClass] then
		Players[iPlayer]:SetPlayerForcedUpgradeClass(iClass, TableOfPossibleUpgrades[iClass]);
	end
end

function ByImperialOrderUpgradeRightThere(iPlayer, iClass, ToClass)
	TakeCareOfEventMorale(iPlayer, 18)
	local unitType = Players[iPlayer]:GetPlayerUniqueUnit(iClass);
	local newunitType = Players[iPlayer]:GetPlayerUniqueUnit(ToClass);
	for iCity in Players[iPlayer]:Cities() do
		if iCity:GetProductionUnit() == unitType then
			iCity:PushOrder(OrderTypes.ORDER_TRAIN, newunitType);
		end
		iCity:SetUnitProduction(newunitType, iCity:GetUnitProduction(newunitType) + iCity:GetUnitProduction(unitType));
		iCity:SetUnitProduction(unitType, 0);
	end
	local xpmod = 0.33;
	if gT.Players[iPlayer].CultBonusOn[158] then
		xpmod = xpmod + 0.33;
	end
	gT.Players[iPlayer].UnitClassXP[ToClass] = gT.Players[iPlayer].UnitClassXP[ToClass] + math.ceil(gT.Players[iPlayer].UnitClassXP[iClass] * xpmod);
	gT.Players[iPlayer].ObsoleteUnitClasses[iClass] = ToClass;
	Players[iPlayer]:SetPlayerForcedUpgradeClass(iClass, ToClass);
	HnGLogUnitUpgradeClass(iPlayer, iClass, ToClass)
end

GameEvents.UnitUpgraded.Add(function(iPlayer, unitId, newUnitId, bGoodyHut)
	local iUnit = Players[iPlayer]:GetUnitByID(unitId);
	local nUnit = Players[iPlayer]:GetUnitByID(newUnitId);
	local uID = iUnit:GetID() .. "," .. iPlayer .. "," .. iUnit:GetGameTurnCreated();
	local nID = newUnitId .. "," .. iPlayer .. "," .. iUnit:GetGameTurnCreated();
	gT.Players[iPlayer].UnitsByID[nID] = {}
	gT.Players[iPlayer].NumUnitsUpdated = gT.Players[iPlayer].NumUnitsUpdated + 1;
	for j, k in pairs(gT.Players[iPlayer].UnitsByID[uID]) do
		gT.Players[iPlayer].UnitsByID[nID][j] = k;
	end
	local delay, px, py;
	if Game.GetActivePlayer() == iPlayer then
		delay = 0;
		px, py = nUnit:GetX(), nUnit:GetY();
	end
	local strDifferForBonuses = math.max(5, math.max(nUnit:GetBaseCombatStrength(true), nUnit:GetBaseRangedCombatStrength()) - math.max(iUnit:GetBaseCombatStrength(true), iUnit:GetBaseRangedCombatStrength()))
	local iScience = strDifferForBonuses * GetNumPlayerManufactoryTrait(iPlayer, 17) * 5;
	local iGreatAdmiral, iGreatGeneral, iCulture = 0, 0, 0;
	if nUnit:GetUnitClassType() ~= iUnit:GetUnitClassType() then 
		if IsHasLaw(iPlayer, 14, 3) then
			iScience = iScience + 20;
		elseif IsHasLaw(iPlayer, 14, 4) then
			iCulture = iCulture + 25;
		elseif IsHasLaw(iPlayer, 14, 2) then
			if nUnit:GetDomainType() == dSea then
				iGreatAdmiral = iGreatAdmiral + 17;
			else
				iGreatGeneral = iGreatGeneral + 17;
			end
		end
	end
	if iScience > 0 then
		ChangeITGScience(iPlayer, iScience)
		if delay then
			delay = delay + 0.5;
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(px,py))), "+" .. iScience .. " [ICON_RESEARCH]", delay)
		end
	end
	if iCulture > 0 then
		ChangeITGCulture(iPlayer, iCulture)
		if delay then
			delay = delay + 0.5;
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(px,py))), "+" .. iCulture .. " [ICON_CULTURE]", delay)
		end
	end
	if iGreatGeneral > 0 then
		IncreaseGeneralPointsByNonCombat(iPlayer, iGreatGeneral)
		if delay then
			delay = delay + 0.5;
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(px,py))), "+" .. iGreatGeneral .. " [ICON_GREAT_GENERAL]", delay)
		end
	end
	if iGreatAdmiral > 0 then
		IncreaseAdmiralPointsByNonCombat(iPlayer, iGreatAdmiral)
		if delay then
			delay = delay + 0.5;
			Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(px,py))), "+" .. iGreatAdmiral .. " [ICON_GREAT_ADMIRAL]", delay)
		end
	end
	--unitclass promotions shall be hunted down
	for type, prom in pairs(gT.Players[iPlayer].UnitsByID[uID].UniqueFreeClassProm or {}) do
		nUnit:SetHasPromotion(prom, false)
		iUnit:SetHasPromotion(prom, false)	
	end
	if not gT.Players[iPlayer].UnitClassPromotions[iUnit:GetUnitClassType()] then
		print("Error upgrading", iPlayer, iUnit:GetUnitType(), nUnit:GetUnitType(), unitId, newUnitId, bGoodyHut)
	end
	for prom, sth in pairs(gT.Players[iPlayer].UnitClassPromotions[iUnit:GetUnitClassType()]) do
		nUnit:SetHasPromotion(prom, false)
		iUnit:SetHasPromotion(prom, false)
	end
	for prom, sth in pairs(gT.Players[iPlayer].UnitClassPromotions[nUnit:GetUnitClassType()]) do
		nUnit:SetHasPromotion(prom, true)
		iUnit:SetHasPromotion(prom, true)
	end
	--special prom military enginner
	if iUnit:IsHasPromotion(promEngRoads) then
		iUnit:SetHasPromotion(promEngRoads, false)
		nUnit:SetHasPromotion(promEngRoads, true)
	end
	-- some abilities need reinstalling
	BasicCSMissionCompleteFunc(iPlayer, 40, 1);
	VictoryTaskCompletion(iPlayer, 23)
end)

GameEvents.AfterUnitUpgraded.Add(function(iPlayer, unitId, newUnitId, bGoodyHut)
	local iUnit = Players[iPlayer]:GetUnitByID(unitId);
	local nUnit = Players[iPlayer]:GetUnitByID(newUnitId);
	local uID = iUnit:GetID() .. "," .. iPlayer .. "," .. iUnit:GetGameTurnCreated();
	local nID = newUnitId .. "," .. iPlayer .. "," .. iUnit:GetGameTurnCreated();

	--general :)
	if gT.Players[iPlayer].UnitsByID[uID].GeneralName then
		local gpID = gT.Players[iPlayer].UnitsByID[uID].GreatPersonID;
		RemoveGreatGeneralFromUnit(uID, iPlayer, gpID)
		AssignGeneralToTheUnit(nUnit, iPlayer, nil, gpID, nID)
	end
	--admiral :)
	if gT.Players[iPlayer].UnitsByID[uID].AdmiralName then
		local gpID = gT.Players[iPlayer].UnitsByID[uID].GreatPersonID;
		RemoveGreatAdmiralFromUnit(uID, iPlayer, gpID)
		AssignAdmiralToTheUnit(nUnit, iPlayer, nil, gpID, nID)
	end
	--i think it is needed for greatpeopleoverview;
	gT.Players[iPlayer].UnitsByID[uID].AdmiralName = nil;
	gT.Players[iPlayer].UnitsByID[uID].GeneralName = nil;
	gT.Players[iPlayer].UnitsByID[uID].GreatPersonID = nil;

	--bonus combat strenght kept over upgrade:
	local cbFromColdWar = gT.Players[iPlayer].UnitsByID[uID].ColdWar;
	if cbFromColdWar then
		ChangeITGBaseCombatStrength(nUnit, cbFromColdWar, cbFromColdWar)
		gT.Players[iPlayer].UnitsByID[nID].ColdWar = cbFromColdWar;
	end

	gT.Players[iPlayer].UnitsByID[nID].TempSipahi = nil; --this bonus poofs on upgrade
	gT.Players[iPlayer].UnitsByID[nID].XP = math.max(gT.Players[iPlayer].UnitsByID[nID].XP + GetXPChangeOnUpgrade(iPlayer), 0);
	
	gT.Players[iPlayer].UnitsByID[nID].ForceITGStackType = nil;
	gT.Players[iPlayer].UnitsByID[nID].ForceITGStackType = nil;
	
	gT.Players[iPlayer].UnitsByID[nID].MoraleModifier = 0; --morale bonus needs readding
	UpdateUnitMoraleBonus(nUnit, iPlayer, true)
	gT.Players[iPlayer].UnitsByID[nID].VeteranModifier = 0; --veteran bonus needs readding
	AdjustCombatBonusFromLevel(nUnit, nID, iPlayer)

	NewUnitCreatedForBasicAbilitiesThatShouldBeDLL(nUnit)
	NewUnitCreatedForInnovations(iPlayer, nUnit, nil, nID)
	NewUnitCreatedForSocialPolicyBonuses(iPlayer, nUnit, nil, nID)
end)

function NewUnitCreatedForBasicAbilitiesThatShouldBeDLL(iUnit)
	local unitType = iUnit:GetUnitType();
	-- max hp done in dll
	-- changeadjacentsameclassmodifier should be via prom // dll
	-- awardwithrandomveteranproms should be here
	if unitType == GameInfoTypes.UNIT_ITG_AOT_HORSEMAN_UU_3 then
		iUnit:ChangeAdjacentSameClassModifier(10);
	elseif unitType == GameInfoTypes.UNIT_ITG_IND_CUIRASS_UU_1 then
		iUnit:ChangeAdjacentSameClassModifier(12);
	elseif unitType == GameInfoTypes.UNIT_ITG_BATTLESHIP_UU_3 then
		iUnit:ChangeAdjacentSameClassModifier(12)
	elseif unitType == GameInfoTypes.UNIT_ITG_TRIAL_SWORDSMAN_AFRICAN then
		iUnit:ChangeAdjacentSameClassModifier(5)
	elseif unitType == GameInfoTypes.UNIT_ITG_AOT_SPEARMAN_UU_3 then
		AwardWithRandomVeteranProms(iUnit, 4)
	elseif unitType == GameInfoTypes.UNIT_ITG_IND_LINE_INFANTRY_UU_5 then
		AwardWithRandomVeteranProms(iUnit, 2)
	elseif unitType == GameInfoTypes.UNIT_ITG_KRIS_SWORDSMAN then
		AwardWithRandomVeteranProms(iUnit, 1)
	end
	for a, gfunc in pairs(gf_NewUnitCreatedUpgradedInto) do
		gfunc(iUnit, unitType)
	end
end

function DefinePlayerForcedUpgradeCost(iPlayer)
	local cost = GetDBCulturePlayerInfo(iPlayer, "ForcedUpgradeCost");
	--laws and other magic :)
	if not IsHasLaw(iPlayer, 14, 0) then
		cost = cost + 15;
	end
	if gT.Players[iPlayer].CultBonusOn[158] then
		cost = cost - 30;
	end
	cost = cost - 20 * GetNumPlayerManufactoryTrait(iPlayer, 17)
	cost = math.max(25, cost)
	Players[iPlayer]:SetForcedUpgradeCost(cost)
end

function DefineXPChangeOnUpgrade(iPlayer)
	Players[iPlayer]:SetXPUpgradeChange(GetXPChangeOnUpgrade(iPlayer))
end

function GetXPChangeOnUpgrade(iPlayer)
	local xpchange = 0;
	if IsHasLaw(iPlayer, 14, 1) then
		xpchange = 25;
	end
	return xpchange;
end