-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- City State Diplo Popup
--
-- Authors: Anton Strenger
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

include( "IconSupport" );
include( "InfoTooltipInclude" );
include( "CityStateStatusHelper" );

local bIsProcess = false;
local g_iMinorCivID = -1;
local g_iMinorCivTeamID = -1;
local m_PopupInfo = nil;
local m_bNewQuestAvailable = false;
local lastBackgroundImage = "citystatebackgroundculture.dds"
local WordWrapOffset = 19;
local WordWrapAnimOffset = 3;

local kiNoAction = 0;
local kiMadePeace = 1;
local kiBulliedGold = 2;
local kiBulliedUnit = 3;
local kiGiftedGold = 4;
local kiPledgedToProtect = 5;
local kiDeclaredWar = 6;
local kiRevokedProtection = 7;
local m_iLastAction = kiNoAction;
local m_iPendingAction = kiNoAction; -- For bullying dialog popups

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- HANDLERS AND HELPERS
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function IntegerTableHasField(tab, field)
	for i, b in pairs(tab) do
		if b == field then
			return true;
		end
	end
end

function SetButtonSize(textControl, buttonControl, animControl, buttonHL)

	--print(textControl:GetText());
	local sizeY = textControl:GetSizeY() + WordWrapOffset;
	buttonControl:SetSizeY(sizeY);
	animControl:SetSizeY(sizeY+WordWrapAnimOffset);
	buttonHL:SetSizeY(sizeY+WordWrapAnimOffset);
end

local prefBSPYoffset = 125;
function SetPBSPYO(i)
	prefBSPYoffset = i;
	UpdateButtonStack()
end

function UpdateButtonStack()
	Controls.GiveStack:CalculateSize();
    Controls.GiveStack:ReprocessAnchoring();
    
    Controls.TakeStack:CalculateSize();
    Controls.TakeStack:ReprocessAnchoring();
    
    Controls.ButtonStack:CalculateSize();
    Controls.ButtonStack:ReprocessAnchoring();

	Controls.CSInfoStack:CalculateSize();
	Controls.CSInfoStack:ReprocessAnchoring();

	Controls.CSSecInfoStack:CalculateSize();
	Controls.CSSecInfoStack:ReprocessAnchoring();

	Controls.ITGProcessStack:CalculateSize();
	Controls.ITGProcessStack:ReprocessAnchoring();

	Controls.ButtonScrollPanel:CalculateInternalSize();
	Controls.ButtonScrollPanel:SetOffsetY(math.max(0, prefBSPYoffset - Controls.SplitBox:GetSizeY()))

	Controls.MainStackCSDP:CalculateSize();
	Controls.MainStackCSDP:ReprocessAnchoring();
end

function InputHandler( uiMsg, wParam, lParam )
    if uiMsg == KeyEvents.KeyDown then
        if wParam == Keys.VK_ESCAPE or wParam == Keys.VK_RETURN then
			if (Controls.BullyConfirm:IsHidden()) then
	            OnCloseButtonClicked();
			else
				m_iPendingAction = kiNoAction;
				Controls.BullyConfirm:SetHide(true);
            	Controls.BGBlock:SetHide(false);
			end
			return true;
        end
    end
end
ContextPtr:SetInputHandler( InputHandler );

function ShowHideHandler( bIsHide, bInitState )

    if( not bInitState ) then
 	   Controls.BackgroundImage:UnloadTexture();
       if( not bIsHide ) then
			Controls.BackgroundImage:SetTexture(lastBackgroundImage);
        	UI.incTurnTimerSemaphore();
        	Events.SerialEventGameMessagePopupShown(m_PopupInfo);
        else
            UI.decTurnTimerSemaphore();
            
            if(m_PopupInfo ~= nil) then
				Events.SerialEventGameMessagePopupProcessed.CallImmediate(m_PopupInfo.Type, 0);
			end
        end
    end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );



-------------------------------------------------
-- On Event Received
-------------------------------------------------
function OnEventReceived( popupInfo )


	local bGreeting = popupInfo.Type == ButtonPopupTypes.BUTTONPOPUP_CITY_STATE_GREETING;
	local bMessage = popupInfo.Type == ButtonPopupTypes.BUTTONPOPUP_CITY_STATE_MESSAGE;
	local bDiplo = popupInfo.Type == ButtonPopupTypes.BUTTONPOPUP_CITY_STATE_DIPLO;
	
	if(not bMessage and not bDiplo) then
		return;
	end

	if (not bMessage) then
		if not Players[Game.GetActivePlayer()]:IsTurnActive() then
			return;
		end
	end
	
	m_PopupInfo = popupInfo;	
	
    local iPlayer = popupInfo.Data1;
    local pPlayer = Players[iPlayer];
	local iTeam = pPlayer:GetTeam();
	local pTeam = Teams[iTeam];
	
	local iQuestFlags = popupInfo.Data2;
    
    g_iMinorCivID = iPlayer;
    g_iMinorCivTeamID = iTeam;
	
	m_iLastAction = kiNoAction;
	m_iPendingAction = kiNoAction;
	
	if (iQuestFlags == 1) then
		m_bNewQuestAvailable = true;
	else
		m_bNewQuestAvailable = false;
	end
	
	OnDisplay();
	
	UIManager:QueuePopup( ContextPtr, PopupPriority.CityStateDiplo );

	LuaEvents.ITGCheckIfCanDoDiploPopup()
	if MapModData.ITGSG then
		MapModData.ITGSG = nil
		OnCloseButtonClicked ()
	end
end
Events.SerialEventGameMessagePopup.Add( OnEventReceived );

-------------------------------------------------
-- On Game Info Dirty
-------------------------------------------------
function OnGameDataDirty()

	if (ContextPtr:IsHidden()) then
		return;
	end
	if bIsProcess then
		-- it will prevent from updating on change faith from process
		bIsProcess = false;
		return;
	end
	OnDisplay();
	
end
Events.SerialEventGameDataDirty.Add(OnGameDataDirty);

-------------------------------------------------
-- On Display
-------------------------------------------------
function OnDisplay()
    
	-- print("Displaying City State Diplo Popup");
	bIsProcess = false;
	
	Controls.ProtectConfirm:SetHide(true)
    
    local iActivePlayer = Game.GetActivePlayer();
    local pActivePlayer = Players[iActivePlayer];
    local iActiveTeam = Game.GetActiveTeam();
    local pActiveTeam = Teams[iActiveTeam];
    
    local iPlayer = g_iMinorCivID;
    local pPlayer = Players[iPlayer];
	local iTeam = g_iMinorCivTeamID;
	local pTeam = Teams[iTeam];
	local sMinorCivType = pPlayer:GetMinorCivType();
	
	local strShortDescKey = pPlayer:GetCivilizationShortDescriptionKey();
	
	local bAllies = pPlayer:IsAllies(iActivePlayer);
	local bFriends = pPlayer:IsFriends(iActivePlayer);
	
	-- At war?
	local bWar = pActiveTeam:IsAtWar(iTeam);
	

	LuaEvents.ITGSG_RequestGT();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;

	-- Update colors
	local primaryColor, secondaryColor = pPlayer:GetPlayerColors();
	primaryColor, secondaryColor = secondaryColor, primaryColor;
	local textColor = {x = primaryColor.x, y = primaryColor.y, z = primaryColor.z, w = 1};

	-- Title
	strTitle = Locale.ConvertTextKey("{"..pPlayer:GetCivilizationShortDescriptionKey()..":upper}");
	Controls.TitleLabel:SetText(strTitle);
	Controls.TitleLabel:SetColor(textColor, 0);
	
	civType = pPlayer:GetCivilizationType();
	civInfo = GameInfo.Civilizations[civType];

	local trait = pPlayer:GetMinorCivTrait()  --GameInfo.MinorCivilizations[sMinorCivType].MinorCivTrait;
	Controls.TitleIcon:SetTexture(GameInfo.MinorCivTraits[trait].TraitTitleIcon);
	
	-- Set Background Image
	lastBackgroundImage = GameInfo.MinorCivTraits[trait].BackgroundImage;
	Controls.BackgroundImage:SetTexture(lastBackgroundImage);
	
	local iconColor = textColor;
	IconHookup( civInfo.PortraitIndex, 32, civInfo.AlphaIconAtlas, Controls.CivIcon );
	Controls.CivIcon:SetColor(iconColor);
	
	local strStatusText = GetCityStateStatusText(iActivePlayer, iPlayer);
	local strStatusTT = GetCityStateStatusToolTip(iActivePlayer, iPlayer, true);
	Controls.StatusIcon:SetTexture(GameInfo.MinorCivTraits[trait].TraitIcon);
	UpdateCityStateStatusUI(iActivePlayer, iPlayer, Controls.PositiveStatusMeter, Controls.NegativeStatusMeter, Controls.StatusMeterMarker, Controls.StatusIconBG);
	Controls.StatusInfo:SetText(strStatusText);
	Controls.StatusInfo:SetToolTipString(strStatusTT);
	Controls.StatusLabel:SetToolTipString(strStatusTT);
	Controls.StatusIconBG:SetToolTipString(strStatusTT);
	Controls.PositiveStatusMeter:SetToolTipString(strStatusTT);
	Controls.NegativeStatusMeter:SetToolTipString(strStatusTT);
	
	-- Trait
	local strTraitText = GetCityStateTraitText(iPlayer);
	local strTraitTT = GetCityStateTraitToolTip(iPlayer);
	
	strTraitText = "[COLOR:200:255:200:255]" .. strTraitText .. "[ENDCOLOR]";
	
	Controls.TraitInfo:SetText(strTraitText);
	Controls.TraitInfo:SetToolTipString(strTraitTT);
	Controls.TraitLabel:SetToolTipString(strTraitTT);
	
	-- Personality
	local strPersonalityText = "";
	local strPersonalityTT = "";
	local iPersonality = pPlayer:GetPersonality();
	if (iPersonality == MinorCivPersonalityTypes.MINOR_CIV_PERSONALITY_FRIENDLY) then
		strPersonalityText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_PERSONALITY_FRIENDLY");
		strPersonalityTT = "Friendly city-states require less percentage of total [ICON_INFLUENCE] Influence to achieve Friendly or Allied level.";
--		strPersonalityTT = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_PERSONALITY_FRIENDLY_TT");
	elseif (iPersonality == MinorCivPersonalityTypes.MINOR_CIV_PERSONALITY_NEUTRAL) then
		strPersonalityText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_PERSONALITY_NEUTRAL");
		strPersonalityTT = "Neutral city-states require base percentage (based on number of civilizations) of total [ICON_INFLUENCE] Influence to achieve Friendly or Allied level.";
--		strPersonalityTT = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_PERSONALITY_NEUTRAL_TT");
	elseif (iPersonality == MinorCivPersonalityTypes.MINOR_CIV_PERSONALITY_HOSTILE) then
		strPersonalityText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_PERSONALITY_HOSTILE");
		strPersonalityTT = "Hostile city-states require more percentage of total [ICON_INFLUENCE] Influence to achieve Friendly or Allied level.";
--		strPersonalityTT = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_PERSONALITY_HOSTILE_TT");
	elseif (iPersonality == MinorCivPersonalityTypes.MINOR_CIV_PERSONALITY_IRRATIONAL) then
		strPersonalityText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_PERSONALITY_IRRATIONAL");
		strPersonalityTT = "Irrational city-states require randomly more, less, or base percentage of total [ICON_INFLUENCE] Influence to achieve Friendly or Allied level.";
--		strPersonalityTT = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_PERSONALITY_IRRATIONAL_TT");
	end
	
	strPersonalityText = "[COLOR:197:255:104:255]" .. strPersonalityText .. "[ENDCOLOR]";
	Controls.PersonalityInfo:SetText(strPersonalityText);
	Controls.PersonalityInfo:SetToolTipString(strPersonalityTT);
	Controls.PersonalityLabel:SetToolTipString(strPersonalityTT);
	
	-- Ally Status
	local iAlly = pPlayer:GetAlly();
	local bHideIcon = true;
	local bHideText = true;
	if (iAlly ~= nil and iAlly ~= -1) then
		if (iAlly ~= iActivePlayer) then
			if (Teams[Players[iAlly]:GetTeam()]:IsHasMet(Game.GetActiveTeam())) then
				bHideIcon = false;
				CivIconHookup(iAlly, 32, Controls.AllyIcon, Controls.AllyIconBG, Controls.AllyIconShadow, false, true);
			else
				bHideIcon = false;
				CivIconHookup(-1, 32, Controls.AllyIcon, Controls.AllyIconBG, Controls.AllyIconShadow, false, true);
			end
		else
			bHideText = false;
		end
	else
		bHideText = false;
	end
	local strAlly = GetAllyText(iActivePlayer, iPlayer);
	Controls.AllyText:SetText(strAlly);
	local strAllyTT = GetAllyToolTip(iActivePlayer, iPlayer);
	Controls.AllyIcon:SetToolTipString(strAllyTT);
	Controls.AllyIconBG:SetToolTipString(strAllyTT);
	Controls.AllyIconShadow:SetToolTipString(strAllyTT);
	Controls.AllyText:SetToolTipString(strAllyTT);
	Controls.AllyLabel:SetToolTipString(strAllyTT);
	Controls.AllyIconContainer:SetHide(bHideIcon);
	Controls.AllyText:SetHide(bHideText);
	
	-- Nearby Resources
	local pCapital = pPlayer:GetCapitalCity();
	if (pCapital ~= nil) then
		
		local strResourceText = "";
		local shortStrResourceText = "";
		local iNumResourcesFound = 0;
		
		local thisX = pCapital:GetX();
		local thisY = pCapital:GetY();
		
		local iRange = GameDefines["MINOR_CIV_RESOURCE_SEARCH_RADIUS"]; --5
		local iCloseRange = math.floor(iRange/2); --2
		local tResourceList = {};
		
		for iDX = -iRange, iRange, 1 do
			for iDY = -iRange, iRange, 1 do
				local pTargetPlot = Map.GetPlotXY(thisX, thisY, iDX, iDY);
				
				if pTargetPlot ~= nil then
					
					local iOwner = pTargetPlot:GetOwner();
					
					if (iOwner == iPlayer or iOwner == -1) then
						local plotX = pTargetPlot:GetX();
						local plotY = pTargetPlot:GetY();
						local plotDistance = Map.PlotDistance(thisX, thisY, plotX, plotY);
						
						if (plotDistance <= iRange and (plotDistance <= iCloseRange or iOwner == iPlayer)) then
							
							local iResourceType = pTargetPlot:GetResourceType(Game.GetActiveTeam());
							
							if (iResourceType ~= -1) then
								
								if (Game.GetResourceUsageType(iResourceType) == ResourceUsageTypes.RESOURCEUSAGE_LUXURY) or gData.Globals.ResourcesInScenario[iResourceType] or gData.CurrencyResources[iResourceType] then
									
									if (tResourceList[iResourceType] == nil) then
										tResourceList[iResourceType] = 0;
									end
									
									tResourceList[iResourceType] = tResourceList[iResourceType] + pTargetPlot:GetNumResource();
									
								end
							end
						end
					end
					
				end
			end
		end
		
		for iResourceType, iAmount in pairs(tResourceList) do
			if (iNumResourcesFound > 0) then
				strResourceText = strResourceText .. ", ";
				shortStrResourceText = shortStrResourceText .. " ";
			end
			local pResource = GameInfo.Resources[iResourceType];
			strResourceText = strResourceText .. pResource.IconString .. " [COLOR_POSITIVE_TEXT]" .. Locale.ConvertTextKey(pResource.Description) .. " (" .. iAmount .. ") [ENDCOLOR]";
			shortStrResourceText = shortStrResourceText .. pResource.IconString;
			iNumResourcesFound = iNumResourcesFound + 1;
		end
		local bAddedColoniesStr = false;
		for plotIndex, sth in pairs(gData.Colonies) do
			if sth.Player == iPlayer then
				if not bAddedColoniesStr then
					strResourceText = strResourceText .. "[NEWLINE][NEWLINE]Colonies owned:";
					bAddedColoniesStr = true;
				end
				local iResourceType = Map.GetPlotByIndex(plotIndex):GetResourceType();
				local pResource = GameInfo.Resources[iResourceType];
				if pResource then
					strResourceText = strResourceText .. "[NEWLINE][COLOR_YIELD_FOOD]" .. pResource.IconString .. " " .. Locale.ConvertTextKey(pResource.Description) .. "[ENDCOLOR]";
					if not tResourceList[iResourceType] then
						shortStrResourceText = shortStrResourceText ..  GameInfo.Resources[iResourceType].IconString
						tResourceList[iResourceType] = true;
						iNumResourcesFound = iNumResourcesFound + 1;
					end
				end
			end
		end
		if iNumResourcesFound > 0 then
			Controls.ResourcesInfo:SetText(shortStrResourceText);
		
			Controls.ResourcesLabel:SetHide(false);
			Controls.ResourcesInfo:SetHide(false);
		
			local strResourceTextTT = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_RESOURCES_TT") .. "[NEWLINE][NEWLINE]" .. strResourceText;
			Controls.ResourcesInfo:SetToolTipString(strResourceTextTT);
			Controls.ResourcesLabel:SetToolTipString(strResourceTextTT);
		else
			Controls.ResourcesLabel:SetHide(true);
			Controls.ResourcesInfo:SetHide(true);
		end		
	else
		Controls.ResourcesLabel:SetHide(true);
		Controls.ResourcesInfo:SetHide(true);
	end
	
	local bCanImproveRelation = (not bWar) and (Players[iPlayer]:GetNumCities() > 0)
	Controls.VisitCSAnim:SetHide(not bCanImproveRelation)
	Controls.VisitCS:SetHide(not bCanImproveRelation)
	local strTVisitCS = "You may spend 1 [ICON_ITG_LEGAL_POWER] Legal Power to improve your [ICON_INFLUENCE] Influence with City-State by 20."
	if bCanImproveRelation then
		if (not gData.Players[iActivePlayer].LegalCSVisitCD[iPlayer]) or (gData.Players[iActivePlayer].LegalCSVisitCD[iPlayer] <= gData.ITGTurn) then
			if gData.Players[iActivePlayer].LegalPower >= 1 then
				if gData.Players[iActivePlayer].Goverment == 1 then
					Controls.VisitCS:SetDisabled(true)
					Controls.VisitCSLabel:SetText("[COLOR_GREY]Improve [ICON_ITG_LEGAL_POWER] Relations")
					strTVisitCS = strTVisitCS .. "[NEWLINE][NEWLINE][COLOR_NEGATIVE_TEXT]Cannot be used while in Anarchy![ENDCOLOR]"
				else
					Controls.VisitCS:SetDisabled(false)
					Controls.VisitCSLabel:SetText("Improve [ICON_ITG_LEGAL_POWER] Relations")
				end
			else
				Controls.VisitCS:SetDisabled(true)
				Controls.VisitCSLabel:SetText("[COLOR_GREY]Improve [ICON_ITG_LEGAL_POWER] Relations")
				strTVisitCS = strTVisitCS .. "[NEWLINE][NEWLINE][COLOR_NEGATIVE_TEXT]You don't have enough [ICON_ITG_LEGAL_POWER] Legal Power.[ENDCOLOR]"
			end
		else
			Controls.VisitCS:SetDisabled(true)
			Controls.VisitCSLabel:SetText("[COLOR_GREY]Improve [ICON_ITG_LEGAL_POWER] Relations")
			strTVisitCS = strTVisitCS .. "[NEWLINE][NEWLINE]The action is disabled for next [COLOR_NEGATIVE_TEXT]" .. (gData.Players[iActivePlayer].LegalCSVisitCD[iPlayer] - gData.ITGTurn) .. "[ENDCOLOR] turns due to prior usage."; 
		end
	end
	Controls.VisitCS:SetToolTipString(strTVisitCS)

--NEW pledge
	local bCanProtect = (bCanImproveRelation and (Players[iPlayer]:GetMinorCivFriendshipWithMajor(iActivePlayer) >= 0))
	local bShowPledgeButton = false;
	local bEnablePledgeButton = false;
	local strProtectButton = "Pledge to Protect (" .. MapModData.DBScenario.PoPLegalCost .. "[ICON_ITG_LEGAL_POWER])";
--	local strProtectTT = Locale.Lookup("TXT_KEY_POP_CSTATE_PLEDGE_TT", GameDefines.MINOR_FRIENDSHIP_ANCHOR_MOD_PROTECTED, 10); --antonjs: todo: xml
--	local strRevokeProtectButton = Locale.Lookup("TXT_KEY_POP_CSTATE_REVOKE_PROTECTION");
--	local strRevokeProtectTT = Locale.Lookup("TXT_KEY_POP_CSTATE_REVOKE_PROTECTION_TT");
	local strProtectTT = Locale.Lookup("TXT_KEY_GOV_ACTION_HELP_26")
	if gData.Players[iActivePlayer].NumMaxPledges <= #gData.Players[iActivePlayer].PledgesOfProtection then
		strProtectTT = strProtectTT .. "[NEWLINE][NEWLINE]This action will remove Protection over " .. Players[gData.Players[iActivePlayer].PledgesOfProtection[1][1]]:GetName() .. ".";
	end
	
	if bCanProtect then
		-- PtP in effect
		if (pPlayer:IsProtectedByMajor(iActivePlayer)) then
		else
			bShowPledgeButton = true;
			if gData.Players[iActivePlayer].LegalPower >= MapModData.DBScenario.PoPLegalCost then
				bEnablePledgeButton = true;
			else
				bEnablePledgeButton = false;
				strProtectButton = "[COLOR_GREY]" .. strProtectButton .. "[ENDCOLOR]";
				strProtectTT = strProtectTT .. "[NEWLINE][NEWLINE][COLOR_NEGATIVE_TEXT]You don't have enough [ICON_ITG_LEGAL_POWER] Legal Power.[ENDCOLOR]";
			end
		end
	end
	Controls.PledgeAnim:SetHide(not bEnablePledgeButton);
	Controls.PledgeButton:SetHide(not bShowPledgeButton);
	Controls.PledgeButton:SetDisabled(not bEnablePledgeButton);
	if (bShowPledgeButton) then
		Controls.PledgeLabel:SetText(strProtectButton);
		Controls.PledgeButton:SetToolTipString(strProtectTT);
	end

	Controls.RevokePledgeAnim:SetHide(true);
	Controls.RevokePledgeButton:SetHide(true);

--new pledge over


	if gData then
		Controls.CulturesInfo:SetText("[COLOR:230:160:255:255]" .. gData.Players[iPlayer].CultureString);
		Controls.GovInfo:SetText("[COLOR:150:200:255:255]" .. Locale.Lookup(gData.Players[iPlayer].GovString));
		if gData.Players[iPlayer].GovAsLeader then
			Controls.LeaderInfo:SetText("[ICON_ITG_GOVERNOR] [COLOR:220:180:255:255]" .. Locale.Lookup(gData.Globals.Governors[gData.Players[iPlayer].GovAsLeader].Name))
			Controls.LeaderLabel:SetHide(false)
		else
			Controls.LeaderLabel:SetHide(true)
		end
		if gData.Players[iPlayer].CodexFrom and (pActiveTeam:IsHasMet(Players[gData.Players[iPlayer].CodexFrom]:GetTeam())) then
			Controls.CodexInfo:SetText("[ICON_ITG_CODEX] [COLOR:255:230:165:255]" .. Players[gData.Players[iPlayer].CodexFrom]:GetCivilizationAdjective())
			Controls.CodexLabel:SetHide(false)
		else
			Controls.CodexLabel:SetHide(true)
		end
		if gData.Players[iPlayer].CoupDone then
			Controls.CoupInfo:SetText("[ICON_SPY] [COLOR:255:70:45:255]" .. Players[gData.Players[iPlayer].CoupDone[1]]:GetCivilizationAdjective() .. " (" .. gData.Players[iPlayer].CoupDone[2] .. "T)" )
			Controls.CoupLabel:SetHide(false)
		else
			Controls.CoupLabel:SetHide(true)
		end
		if gData.Players[iPlayer].OfficialReligion then
			local iRel = gData.Players[iPlayer].OfficialReligion
			Controls.ReligionInfo:SetText(GameInfo.Religions[iRel].IconString .. " " .. Locale.Lookup(Game.GetReligionName(iRel)))
			Controls.ReligionLabel:SetHide(false)
		else
			Controls.ReligionLabel:SetHide(true)
		end
		local numProtected = 0;
		local sTTProtected = ""
		local sName;
		for g, h in pairs(gData.Players) do
			if not Players[g]:IsMinorCiv() then
				if Players[iPlayer]:IsProtectedByMajor(g) then
					if g == iActivePlayer then
						sName = "You"
						numProtected = numProtected + 1;
						sTTProtected = sTTProtected .. "[NEWLINE][ICON_BULLET][COLOR_POSITIVE_TEXT]You[ENDCOLOR]"
					elseif pActiveTeam:IsHasMet(Players[g]:GetTeam()) then
						sName = Players[g]:GetCivilizationShortDescription()
						numProtected = numProtected + 1;
						sTTProtected = sTTProtected .. "[NEWLINE][ICON_BULLET]" .. sName;
					end
				end
			end
		end
		if numProtected > 0 then
			sTTProtected = "Protected by:" .. sTTProtected .. "[NEWLINE][NEWLINE]City-State will not declare war on their protectors (with an exception of scripted events)."
			if numProtected == 1 then
				Controls.ProtectInfo:SetText("[COLOR_PROJECT_TEXT]" .. sName)
			else
				Controls.ProtectInfo:SetText("[COLOR_PROJECT_TEXT]" .. numProtected .. " civilizations")
			end
			Controls.ProtectLabel:SetHide(false)
			Controls.ProtectLabel:SetToolTipString(sTTProtected)
			Controls.ProtectInfo:SetToolTipString(sTTProtected)
		else
			Controls.ProtectLabel:SetHide(true)
		end
	else
		print("OhNo error 125662")	
	end

	-- Body text
	local strText;
	
	-- Active Quests
	UpdateActiveQuests();
	
	UpdateActiveGTQuests();
	-- Peace
	if (not bWar) then
		
		-- Were we sent here because we clicked a notification for a new quest?
		if (m_iLastAction == kiNoAction and m_bNewQuestAvailable) then
			strText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_DIPLO_HELLO_QUEST_MESSAGE");
		
		-- Did we just make peace?
		elseif (m_iLastAction == kiMadePeace) then
			strText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_DIPLO_PEACE_JUST_MADE");
			
		-- Did we just bully gold?
		elseif (m_iLastAction == kiBulliedGold) then
			strText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_DIPLO_JUST_BULLIED");
		
		-- Did we just bully a worker?
		elseif (m_iLastAction == kiBulliedUnit) then
			strText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_DIPLO_JUST_BULLIED_WORKER");
		
		-- Did we just give gold?
		elseif (m_iLastAction == kiGiftedGold) then
			strText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_DIPLO_JUST_SUPPORTED");
		
		-- Did we just PtP?
		elseif (m_iLastAction == kiPledgedToProtect) then
			strText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_PLEDGE_RESPONSE");
		
		-- Did we just revoke a PtP?
		elseif (m_iLastAction == kiRevokedProtection) then
			strText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_DIPLO_JUST_REVOKED_PROTECTION");
		
		-- Normal peaceful hello, with info about active quests
		else
			local iPersonality = pPlayer:GetPersonality();
			
			if (pPlayer:IsProtectedByMajor(iActivePlayer)) then
				strText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_DIPLO_HELLO_PEACE_PROTECTED");
			elseif (iPersonality == MinorCivPersonalityTypes.MINOR_CIV_PERSONALITY_FRIENDLY) then
				strText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_DIPLO_HELLO_PEACE_FRIENDLY");
			elseif (iPersonality == MinorCivPersonalityTypes.MINOR_CIV_PERSONALITY_NEUTRAL) then
				strText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_DIPLO_HELLO_PEACE_NEUTRAL");
			elseif (iPersonality == MinorCivPersonalityTypes.MINOR_CIV_PERSONALITY_HOSTILE) then
				strText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_DIPLO_HELLO_PEACE_HOSTILE");
			elseif (iPersonality == MinorCivPersonalityTypes.MINOR_CIV_PERSONALITY_IRRATIONAL) then
				strText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_DIPLO_HELLO_PEACE_IRRATIONAL");
			end
		
			local strQuestString = "";
			local strWarString = "";
			
			local iNumPlayersAtWar = 0;
			
			-- Loop through all the Majors the active player knows
			for iPlayerLoop = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
				
				pOtherPlayer = Players[iPlayerLoop];
				iOtherTeam = pOtherPlayer:GetTeam();
				
				-- Don't test war with active player!
				if (iPlayerLoop ~= iActivePlayer) then
					if (pOtherPlayer:IsAlive()) then
						if (pTeam:IsAtWar(iOtherTeam)) then
							if (pPlayer:IsMinorWarQuestWithMajorActive(iPlayerLoop)) then
								if (iNumPlayersAtWar ~= 0) then
									strWarString = strWarString .. ", "
								end
								strWarString = strWarString .. Locale.ConvertTextKey(pOtherPlayer:GetNameKey());
								
								iNumPlayersAtWar = iNumPlayersAtWar + 1;
							end
						end
					end
				end
			end
		end
		
		-- Tell the City State to stop gifting us Units (if they are)
		Controls.NoUnitSpawningButton:SetHide(true);
--[[		if (pPlayer:GetMinorCivTrait() == GameInfoTypes.MINOR_TRAIT_MILITARISTIC) then
			if (bFriends) then
				Controls.NoUnitSpawningButton:SetHide(false);
				
				-- Player has said to turn it off
				local strSpawnText;
				if (pPlayer:IsMinorCivUnitSpawningDisabled(iActivePlayer)) then
					strSpawnText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_TURN_SPAWNING_ON");
				else
					strSpawnText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_TURN_SPAWNING_OFF");
				end
				
				Controls.NoUnitSpawningLabel:SetText(strSpawnText);
			end
		end ]]
		
		Controls.GiveButton:SetHide(pPlayer:IsCannotBeInfluencedWithGold());
		Controls.TakeButton:SetHide(pPlayer:IsCannotBeBullied());
		Controls.PeaceButton:SetHide(true);
		Controls.WarButton:SetHide(false);
		
		
		Controls.ProcessITGButton:SetHide(true);
		for i, quest in pairs(gData.Players[iPlayer].CityStateQuests) do
			if quest.Active then
				if quest.ActivePlayers[iActivePlayer] then
					local contribution = (quest.Scale or 1) * quest.ActivePlayers[iActivePlayer].ProjectDonated;
					if quest.Project == 1 then
						Controls.ProcessITGButton:SetHide(false);
						Controls.ProcessITGLabel:SetText("Contribute [ICON_RESEARCH] Science (" .. contribution .. ")");
						Controls.ProcessITGButton:SetToolTipString("Contributing [ICON_RESEARCH] Science will reward you up to 25 points in the eyes of " .. Players[iPlayer]:GetName() .. "'s people. Points earned depends if you outshine your rivals in this area. You have contributed " .. contribution .. " [ICON_RESEARCH] Science so far.");
					elseif quest.Project == 3 then
						Controls.ProcessITGButton:SetHide(false);
						Controls.ProcessITGLabel:SetText("Contribute [ICON_ITG_LEGAL_POWER] Legal Power (" .. contribution .. ")");
						Controls.ProcessITGButton:SetToolTipString("Contributing [ICON_ITG_LEGAL_POWER] Legal Power will reward you up to 25 points in the eyes of " .. Players[iPlayer]:GetName() .. "'s people. Points earned depends if you outshine your rivals in this area. You have contributed " .. contribution .. " [ICON_ITG_LEGAL_POWER] Legal Power so far.");
					elseif quest.Project == 4 then
						Controls.ProcessITGButton:SetHide(false);
						Controls.ProcessITGLabel:SetText("Contribute [ICON_PEACE] Faith (" .. contribution .. ")");
						Controls.ProcessITGButton:SetToolTipString("Contributing [ICON_PEACE] Faith will reward you up to 25 points in the eyes of " .. Players[iPlayer]:GetName() .. "'s people. Points earned depends if you outshine your rivals in this area. You have contributed " .. contribution .. " [ICON_PEACE] Faith so far.");
					elseif quest.Project == 5 then
						Controls.ProcessITGButton:SetHide(false);
						Controls.ProcessITGLabel:SetText("Contribute [ICON_CULTURE] Culture (" .. contribution .. ")");
						Controls.ProcessITGButton:SetToolTipString("Contributing [ICON_CULTURE] Culture will reward you up to 25 points in the eyes of " .. Players[iPlayer]:GetName() .. "'s people. Points earned depends if you outshine your rivals in this area. You have contributed " .. contribution .. " [ICON_CULTURE] Culture so far.");
					elseif quest.Project == 6 then
						Controls.ProcessITGButton:SetHide(false);
						Controls.ProcessITGLabel:SetText("Contribute [ICON_GREAT_ADMIRAL] Great Admiral points (" .. contribution .. ")");
						Controls.ProcessITGButton:SetToolTipString("Contributing [ICON_GREAT_ADMIRAL] Great Admiral points will reward you up to 25 points in the eyes of " .. Players[iPlayer]:GetName() .. "'s people. Points earned depends if you outshine your rivals in this area. You have contributed " .. contribution .. " [ICON_GREAT_ADMIRAL] Great Admiral points so far.");
					elseif quest.Project == 7 then
						Controls.ProcessITGButton:SetHide(false);
						Controls.ProcessITGLabel:SetText("Contribute [ICON_GREAT_GENERAL] Great General points (" .. contribution .. ")");
						Controls.ProcessITGButton:SetToolTipString("Contributing [ICON_GREAT_GENERAL] Great General points will reward you up to 25 points in the eyes of " .. Players[iPlayer]:GetName() .. "'s people. Points earned depends if you outshine your rivals in this area. You have contributed " .. contribution .. " [ICON_GREAT_GENERAL] Great General points so far.");
					elseif quest.Project == 8 then
						Controls.ProcessITGButton:SetHide(false);
						Controls.ProcessITGLabel:SetText("Contribute [ICON_ITG_SETTLER] Settler points (" .. contribution .. ")");
						Controls.ProcessITGButton:SetToolTipString("Contributing [ICON_ITG_SETTLER] Settler points will reward you up to 25 points in the eyes of " .. Players[iPlayer]:GetName() .. "'s people. Points earned depends if you outshine your rivals in this area. You have contributed " .. contribution .. " [ICON_ITG_SETTLER] Settler points so far.");
					elseif quest.Project == 9 then
						Controls.ProcessITGButton:SetHide(false);
						Controls.ProcessITGLabel:SetText("Contribute [ICON_WORKER] Work Rate (" .. contribution .. ")");
						Controls.ProcessITGButton:SetToolTipString("Contributing [ICON_WORKER] Work Rate points will reward you up to 25 points in the eyes of " .. Players[iPlayer]:GetName() .. "'s people. Points earned depends if you outshine your rivals in this area. You have contributed " .. contribution .. " [ICON_WORKER] Work Rate so far.");
					end
				end
			end
		end
		-- ProcessITGButton
		if not pActiveTeam:CanDeclareWar(iTeam) then
			Controls.WarButton:SetHide(true)
		end
	-- War
	else
		
		-- Warmongering player
		if (pPlayer:IsPeaceBlocked(iActiveTeam)) then
			strText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_DIPLO_HELLO_WARMONGER");
			Controls.PeaceButton:SetHide(true);
			
		-- Normal War
		else
			strText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_DIPLO_HELLO_WAR");
			Controls.PeaceButton:SetHide(false);
		end
		
		Controls.GiveButton:SetHide(true);
		Controls.TakeButton:SetHide(true);
		Controls.WarButton:SetHide(true);
		Controls.NoUnitSpawningButton:SetHide(true);
		Controls.ProcessITGButton:SetHide(true);
	end
	
	-- Pledge to Protect
	
	if (Game.IsOption(GameOptionTypes.GAMEOPTION_ALWAYS_WAR)) then
		Controls.PeaceButton:SetHide(true);
	end
	if (Game.IsOption(GameOptionTypes.GAMEOPTION_ALWAYS_PEACE)) then
		Controls.WarButton:SetHide(true);
	end
	if (Game.IsOption(GameOptionTypes.GAMEOPTION_NO_CHANGING_WAR_PEACE)) or (Teams[iActiveTeam]:IsPermanentWarPeace(iTeam)) then
		Controls.PeaceButton:SetHide(true);
		Controls.WarButton:SetHide(true);
	end
	-- Buyout (Austria UA)
	local iBuyoutCost = pPlayer:GetBuyoutCost(iActivePlayer);
	local strButtonLabel = Locale.ConvertTextKey( "TXT_KEY_POP_CSTATE_BUYOUT");
	local strToolTip = Locale.ConvertTextKey( "TXT_KEY_POP_CSTATE_BUYOUT_TT", iBuyoutCost );
	if (pPlayer:CanMajorBuyout(iActivePlayer) and not bWar) then
		Controls.BuyoutButton:SetHide(false);
		Controls.BuyoutAnim:SetHide(false);
	elseif (pActivePlayer:IsAbleToAnnexCityStates() and not bWar) then
		if (pPlayer:GetAlly() == iActivePlayer) then
			local iAllianceTurns = pPlayer:GetAlliedTurns();
			strButtonLabel = "[COLOR_WARNING_TEXT]" .. strButtonLabel .. "[ENDCOLOR]";
			strToolTip = Locale.ConvertTextKey("TXT_KEY_POP_CSTATE_BUYOUT_DISABLED_ALLY_TT", GameDefines.MINOR_CIV_BUYOUT_TURNS, iAllianceTurns, iBuyoutCost);
		else
			strButtonLabel = "[COLOR_WARNING_TEXT]" .. strButtonLabel .. "[ENDCOLOR]";
			strToolTip = Locale.ConvertTextKey("TXT_KEY_POP_CSTATE_BUYOUT_DISABLED_TT", GameDefines.MINOR_CIV_BUYOUT_TURNS, iBuyoutCost);
		end
		--antonjs: todo: disable button entirely, in case bWar doesn't update in time for the callback to disallow buyout in wartime
		Controls.BuyoutButton:SetHide(false);
		Controls.BuyoutAnim:SetHide(true);
	else
		Controls.BuyoutButton:SetHide(true);
	end
	Controls.BuyoutLabel:SetText( strButtonLabel )
	Controls.BuyoutButton:SetToolTipString( strToolTip );
	
	Controls.DescriptionLabel:SetText(strText);
	
	SetButtonSize(Controls.PeaceLabel, Controls.PeaceButton, Controls.PeaceAnim, Controls.PeaceButtonHL);
	SetButtonSize(Controls.GiveLabel, Controls.GiveButton, Controls.GiveAnim, Controls.GiveButtonHL);
	SetButtonSize(Controls.ProcessITGLabel, Controls.ProcessITGButton, Controls.ProcessITGAnim, Controls.ProcessITGButtonHL);
	SetButtonSize(Controls.TakeLabel, Controls.TakeButton, Controls.TakeAnim, Controls.TakeButtonHL);
	SetButtonSize(Controls.WarLabel, Controls.WarButton, Controls.WarAnim, Controls.WarButtonHL);
	SetButtonSize(Controls.PledgeLabel, Controls.PledgeButton, Controls.PledgeAnim, Controls.PledgeButtonHL);
	SetButtonSize(Controls.RevokePledgeLabel, Controls.RevokePledgeButton, Controls.RevokePledgeAnim, Controls.RevokePledgeButtonHL);
	SetButtonSize(Controls.NoUnitSpawningLabel, Controls.NoUnitSpawningButton, Controls.NoUnitSpawningAnim, Controls.NoUnitSpawningButtonHL);
	SetButtonSize(Controls.BuyoutLabel, Controls.BuyoutButton, Controls.BuyoutAnim, Controls.BuyoutButtonHL);
	


	Controls.GiveStack:SetHide(true);
	Controls.ITGProcessStack:SetHide(true);
	Controls.TakeStack:SetHide(true);
	Controls.ButtonStack:SetHide(false);
	
	UpdateButtonStack();
end

function UpdateActiveQuests()
--DoSecondString there if possible
--	local iActivePlayer = Game.GetActivePlayer();
--	local sIconText = GetActiveQuestText(iActivePlayer, g_iMinorCivID);
--	local sToolTipText = GetActiveQuestToolTip(iActivePlayer, g_iMinorCivID);
	
--	Controls.QuestInfo:SetText(sIconText);
--	Controls.QuestInfo:SetToolTipString(sToolTipText);
--	Controls.QuestLabel:SetToolTipString(sToolTipText);
end

function UpdateActiveGTQuests()
	local sIconText = "None";
	local sToolTipText = "This City-State has currently no formulated politics or virtues. It is either too early or too late for that.";
	local sOldText = "";
	local sOldToolTip = "";
	LuaEvents.ITGSG_RequestCityStateQuestData(g_iMinorCivID);
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if gData then
		sToolTipText = gData.OngoingQuest or "";
		sIconText = gData.OngoingQuestIcon or "None";
		if gData.PreviousQuest then
			sOldText = "?"
			sOldToolTip = gData.PreviousQuest;
		end
	end
	Controls.QuestGTInfoOld:SetText(sOldText);
	Controls.QuestGTInfoOld:SetToolTipString(sOldToolTip);
	Controls.QuestGTInfo:SetText(sIconText);
	Controls.QuestGTInfo:SetToolTipString(sToolTipText);
	Controls.QuestGTLabel:SetToolTipString(sToolTipText);
end

-------------------------------------------------
-- On Quest Info Clicked
-------------------------------------------------
function OnQuestInfoClicked()
	local iActivePlayer = Game.GetActivePlayer();
	local pMinor = Players[g_iMinorCivID];
	if (pMinor) then
		if (pMinor:IsMinorCivDisplayedQuestForPlayer(iActivePlayer, MinorCivQuestTypes.MINOR_CIV_QUEST_KILL_CAMP)) then
			local iQuestData1 = pMinor:GetQuestData1(iActivePlayer, MinorCivQuestTypes.MINOR_CIV_QUEST_KILL_CAMP);
			local iQuestData2 = pMinor:GetQuestData2(iActivePlayer, MinorCivQuestTypes.MINOR_CIV_QUEST_KILL_CAMP);
			local pPlot = Map.GetPlot(iQuestData1, iQuestData2);
			if (pPlot) then
				UI.LookAt(pPlot, 0);
				local hex = ToHexFromGrid(Vector2(pPlot:GetX(), pPlot:GetY()));
				Events.GameplayFX(hex.x, hex.y, -1);
			end
		end
	end
end
--Controls.QuestInfo:RegisterCallback( Mouse.eLClick, OnQuestInfoClicked );

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- MAIN MENU
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

----------------------------------------------------------------
-- Pledge
----------------------------------------------------------------
function OnYesProtect()
	Controls.ProtectConfirm:SetHide(true)
	local iActivePlayer = Game.GetActivePlayer();
	if Players[iActivePlayer]:IsTurnActive() then
		LuaEvents.ITGPlayerPledgeToProtect(iActivePlayer, g_iMinorCivID)
		m_iLastAction = kiPledgedToProtect;
		OnDisplay()
	end
end
Controls.YesProtect:RegisterCallback( Mouse.eLClick, OnYesProtect );

function OnNoProtect()
	Controls.ProtectConfirm:SetHide(true)
end
Controls.NoProtect:RegisterCallback( Mouse.eLClick, OnNoProtect)

function OnPledgeButtonClicked ()
	local iActivePlayer = Game.GetActivePlayer();
	 -- g_iMinorCivID;
	if Players[iActivePlayer]:IsTurnActive() then
		Controls.ProtectConfirm:SetHide(false)
	end
end
Controls.PledgeButton:RegisterCallback( Mouse.eLClick, OnPledgeButtonClicked );

----------------------------------------------------------------
-- Revoke Pledge
----------------------------------------------------------------
function OnRevokePledgeButtonClicked ()
	
	local iActivePlayer = Game.GetActivePlayer();
	local pPlayer = Players[g_iMinorCivID];
	
	if (pPlayer:CanMajorWithdrawProtection(iActivePlayer)) then
		Game.DoMinorPledgeProtection(iActivePlayer, g_iMinorCivID, false);
		m_iLastAction = kiRevokedProtection;
	end
end
Controls.RevokePledgeButton:RegisterCallback( Mouse.eLClick, OnRevokePledgeButtonClicked );

----------------------------------------------------------------
-- Buyout
----------------------------------------------------------------
function OnBuyoutButtonClicked()
	local iActivePlayer = Game.GetActivePlayer();
	local pMinor = Players[g_iMinorCivID];
	
	if (pMinor:CanMajorBuyout(iActivePlayer)) then
		UIManager:DequeuePopup( ContextPtr );
		Game.DoMinorBuyout(iActivePlayer, g_iMinorCivID);
	end
end
Controls.BuyoutButton:RegisterCallback( Mouse.eLClick, OnBuyoutButtonClicked );

function OnVisitButtonClicked()
	if Players[Game.GetActivePlayer()]:IsTurnActive() then	
		LuaEvents.UIDiploCallVisitCS(Game.GetActivePlayer(), g_iMinorCivID)
		Controls.VisitCSAnim:SetHide(true)
		OnDisplay()
	end
end
Controls.VisitCS:RegisterCallback( Mouse.eLClick, OnVisitButtonClicked );
----------------------------------------------------------------
-- War
----------------------------------------------------------------
function OnWarButtonClicked ()
	UI.AddPopup{ Type = ButtonPopupTypes.BUTTONPOPUP_DECLAREWARMOVE, Data1 = g_iMinorCivTeamID, Option1 = true};
end
Controls.WarButton:RegisterCallback( Mouse.eLClick, OnWarButtonClicked );

----------------------------------------------------------------
-- Peace
----------------------------------------------------------------
function OnPeaceButtonClicked ()
    
	Network.SendChangeWar(g_iMinorCivTeamID, false);
	m_iLastAction = kiMadePeace;
end
Controls.PeaceButton:RegisterCallback( Mouse.eLClick, OnPeaceButtonClicked );

----------------------------------------------------------------
-- Stop/Start Unit Spawning
----------------------------------------------------------------
function OnStopStartSpawning()
    local pPlayer = Players[g_iMinorCivID];
    local iActivePlayer = Game.GetActivePlayer();
	
	local bSpawningDisabled = pPlayer:IsMinorCivUnitSpawningDisabled(iActivePlayer);
	
	-- Update the text based on what state we're changing to
	local strSpawnText;
	if (bSpawningDisabled) then
		strSpawnText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_TURN_SPAWNING_OFF");
	else
		strSpawnText = Locale.ConvertTextKey("TXT_KEY_CITY_STATE_TURN_SPAWNING_ON");
	end
	
	Controls.NoUnitSpawningLabel:SetText(strSpawnText);
    
	Network.SendMinorNoUnitSpawning(g_iMinorCivID, not bSpawningDisabled);
end
Controls.NoUnitSpawningButton:RegisterCallback( Mouse.eLClick, OnStopStartSpawning );

----------------------------------------------------------------
-- Open Give Submenu
----------------------------------------------------------------
function OnGiveButtonClicked ()
	Controls.GiveStack:SetHide(false);
	Controls.TakeStack:SetHide(true);
	Controls.ButtonStack:SetHide(true);
	Controls.ITGProcessStack:SetHide(true);
	PopulateGiftChoices();
end
Controls.GiveButton:RegisterCallback( Mouse.eLClick, OnGiveButtonClicked );

function PopulateProcessButton ()
	Controls.GiveStack:SetHide(true);
	Controls.TakeStack:SetHide(true);
	Controls.ITGProcessStack:SetHide(false);
	Controls.ButtonStack:SetHide(true);
	PopulateProcessChoices();
end
Controls.ProcessITGButton:RegisterCallback( Mouse.eLClick, PopulateProcessButton );


----------------------------------------------------------------
-- Open Take Submenu
----------------------------------------------------------------
function OnTakeButtonClicked ()
	Controls.GiveStack:SetHide(true);
	Controls.TakeStack:SetHide(false);
	Controls.ITGProcessStack:SetHide(true);
	Controls.ButtonStack:SetHide(true);
	PopulateTakeChoices();
end
Controls.TakeButton:RegisterCallback( Mouse.eLClick, OnTakeButtonClicked );

----------------------------------------------------------------
-- Close
----------------------------------------------------------------
function OnCloseButtonClicked ()
	Controls.ProtectConfirm:SetHide(false)
	UIManager:DequeuePopup( ContextPtr );
end
Controls.CloseButton:RegisterCallback( Mouse.eLClick, OnCloseButtonClicked );

----------------------------------------------------------------
-- Find On Map
----------------------------------------------------------------
function OnFindOnMapButtonClicked ()
	local pPlayer = Players[g_iMinorCivID];
	if (pPlayer) then
		local pCity = pPlayer:GetCapitalCity();
		if (pCity) then
			local pPlot = pCity:Plot();
			if (pPlot) then
				UI.LookAt(pPlot, 0);
			end
		end
	end
end
Controls.FindOnMapButton:RegisterCallback( Mouse.eLClick, OnFindOnMapButtonClicked );

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- GIFT MENU
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
local iGoldGiftLarge = GameDefines["MINOR_GOLD_GIFT_LARGE"];
local iGoldGiftMedium = GameDefines["MINOR_GOLD_GIFT_MEDIUM"];
local iGoldGiftSmall = GameDefines["MINOR_GOLD_GIFT_SMALL"];
local GetITDS = {1, 2, 5, 10, 20}

function PopulateProcessChoices()
	local pPlayer = Players[g_iMinorCivID];
	
	local iActivePlayer = Game.GetActivePlayer();
	local pActivePlayer = Players[iActivePlayer];

	LuaEvents.ITGSG_RequestGT();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;

	Controls.ITGProcessGiveInfo:SetDisabled(true);

	for i, iquest in pairs(gData.Players[g_iMinorCivID].CityStateQuests) do
		if iquest.Active then
			local contribution = iquest.ActivePlayers[iActivePlayer].ProjectDonated * iquest.Scale;
			local avRes = 0;
			local hstring = "";
			local scale = 5 * iquest.Scale;
			if iquest.Project == 1 then
				avRes = gData.Players[iActivePlayer].Science;
				hstring = "[ICON_RESEARCH] Science"
				Controls.ITGProcessGiveInfoLabel:SetText("Currently donated: " .. contribution .. " [ICON_RESEARCH]");
			elseif iquest.Project == 3 then
				avRes = gData.Players[iActivePlayer].LegalPower * 100 + gData.Players[iActivePlayer].LegalPowerPercent;
				hstring = "[ICON_ITG_LEGAL_POWER] Legal Power"
				Controls.ITGProcessGiveInfoLabel:SetText("Currently donated: " .. contribution  .. " [ICON_ITG_LEGAL_POWER]");
			elseif iquest.Project == 4 then
				avRes = Players[iActivePlayer]:GetFaith()
				hstring = "[ICON_PEACE] Faith"
				Controls.ITGProcessGiveInfoLabel:SetText("Currently donated: " .. contribution .. " [ICON_PEACE]");
			elseif iquest.Project == 5 then
				avRes = gData.Players[iActivePlayer].Culture
				hstring = "[ICON_CULTURE] Culture"
				Controls.ITGProcessGiveInfoLabel:SetText("Currently donated: " .. contribution .. " [ICON_CULTURE]");
			elseif iquest.Project == 6 then
				avRes = Players[iActivePlayer]:GetNavalCombatExperience()
				hstring = "[ICON_GREAT_ADMIRAL] Admiral points"
				Controls.ITGProcessGiveInfoLabel:SetText("Currently donated: " .. contribution .. " [ICON_GREAT_ADMIRAL]");
			elseif iquest.Project == 7 then
				avRes = Players[iActivePlayer]:GetCombatExperience() 
				hstring = "[ICON_GREAT_GENERAL] General points"
				Controls.ITGProcessGiveInfoLabel:SetText("Currently donated: " .. contribution .. " [ICON_GREAT_GENERAL]");
			elseif iquest.Project == 8 then
				avRes = gData.Players[iActivePlayer].SettlerGenerationPoints * 5
				hstring = "[ICON_ITG_SETTLER] Settler points"
				Controls.ITGProcessGiveInfoLabel:SetText("Currently donated: " .. contribution .. " [ICON_ITG_SETTLER]");
			elseif iquest.Project == 9 then
				avRes = 0
				for iCity in Players[iActivePlayer]:Cities() do
					local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded();
					avRes = avRes + gData.Cities[cID].WorkRate;
				end
				avRes = math.floor(avRes / 5)
				hstring = "[ICON_WORKER] Work Rate"
				Controls.ITGProcessGiveInfoLabel:SetText("Currently donated: " .. contribution .. " [ICON_WORKER]");
			end
			for i = 1, 5 do
				if avRes < 5 * GetITDS[i] then
					Controls["ITGProcessGive" .. i]:SetDisabled(true)
					Controls["ITGProcessGive" .. i .. "Label"]:SetText("[COLOR_GREY]Grant " .. GetITDS[i] * scale .. " " ..  hstring ..".[ENDCOLOR]");
				else
					Controls["ITGProcessGive" .. i]:SetDisabled(false)
					Controls["ITGProcessGive" .. i .. "Label"]:SetText("Grant " .. GetITDS[i] * scale .. " " ..  hstring ..".");
				end
			end
		end
	end
end

function PopulateGiftChoices()	
	local pPlayer = Players[g_iMinorCivID];
	
	local iActivePlayer = Game.GetActivePlayer();
	local pActivePlayer = Players[iActivePlayer];
	
	-- Small Gold
	local iNumGoldPlayerHas = pActivePlayer:GetGold();
	
	iGold = iGoldGiftSmall;
	iLowestGold = iGold;
	iFriendshipAmount = pPlayer:GetFriendshipFromGoldGift(iActivePlayer, iGold);
	local buttonText = Locale.ConvertTextKey("TXT_KEY_POPUP_MINOR_GOLD_GIFT_AMOUNT", iGold, iFriendshipAmount);
	if (iNumGoldPlayerHas < iGold) then
		buttonText = "[COLOR_WARNING_TEXT]" .. buttonText .. "[ENDCOLOR]";
		Controls.SmallGiftAnim:SetHide(true);
	else
		Controls.SmallGiftAnim:SetHide(false);
	end
	Controls.SmallGift:SetText(buttonText);
	SetButtonSize(Controls.SmallGift, Controls.SmallGiftButton, Controls.SmallGiftAnim, Controls.SmallGiftButtonHL);
	
	-- Medium Gold
	iGold = iGoldGiftMedium;
	iFriendshipAmount = pPlayer:GetFriendshipFromGoldGift(iActivePlayer, iGold);
	local buttonText = Locale.ConvertTextKey("TXT_KEY_POPUP_MINOR_GOLD_GIFT_AMOUNT", iGold, iFriendshipAmount);
	if (iNumGoldPlayerHas < iGold) then
		buttonText = "[COLOR_WARNING_TEXT]" .. buttonText .. "[ENDCOLOR]";
		Controls.MediumGiftAnim:SetHide(true);
	else
		Controls.MediumGiftAnim:SetHide(false);
	end
	Controls.MediumGift:SetText(buttonText);
	SetButtonSize(Controls.MediumGift, Controls.MediumGiftButton, Controls.MediumGiftAnim, Controls.MediumGiftButtonHL);
	
	-- Large Gold
	iGold = iGoldGiftLarge;
	iFriendshipAmount = pPlayer:GetFriendshipFromGoldGift(iActivePlayer, iGold);
	local buttonText = Locale.ConvertTextKey("TXT_KEY_POPUP_MINOR_GOLD_GIFT_AMOUNT", iGold, iFriendshipAmount);
	if (iNumGoldPlayerHas < iGold) then
		buttonText = "[COLOR_WARNING_TEXT]" .. buttonText .. "[ENDCOLOR]";
		Controls.LargeGiftAnim:SetHide(true);
	else
		Controls.LargeGiftAnim:SetHide(false);
	end
	Controls.LargeGift:SetText(buttonText);
	SetButtonSize(Controls.LargeGift, Controls.LargeGiftButton, Controls.LargeGiftAnim, Controls.LargeGiftButtonHL);
	
	-- Unit
	Controls.UnitGiftButton:SetHide(true);
--[[
	local iInfluence = pPlayer:GetFriendshipFromUnitGift(iActivePlayer, false, true);
	local iTravelTurns = GameDefines.MINOR_UNIT_GIFT_TRAVEL_TURNS;
	local buttonText = Locale.ConvertTextKey("TXT_KEY_POP_CSTATE_GIFT_UNIT", iInfluence);
	local tooltipText = Locale.ConvertTextKey("TXT_KEY_POP_CSTATE_GIFT_UNIT_TT", iTravelTurns, iInfluence);
	if (pPlayer:GetIncomingUnitCountdown(iActivePlayer) >= 0) then
		buttonText = "[COLOR_WARNING_TEXT]" .. buttonText .. "[ENDCOLOR]";
		Controls.UnitGiftAnim:SetHide(true);
		Controls.UnitGiftButton:ClearCallback( Mouse.eLClick );
	else
		Controls.UnitGiftAnim:SetHide(false);
		Controls.UnitGiftButton:RegisterCallback( Mouse.eLClick, OnGiftUnit );
	end
	Controls.UnitGift:SetText(buttonText);
	Controls.UnitGiftButton:SetToolTipString(tooltipText);
	SetButtonSize(Controls.UnitGift, Controls.UnitGiftButton, Controls.UnitGiftAnim, Controls.UnitGiftButtonHL);
	]]--



	-- Tile Improvement
	-- Only allowed for allies
	iGold = pPlayer:GetGiftTileImprovementCost(iActivePlayer);
	local buttonText = Locale.ConvertTextKey("TXT_KEY_POPUP_MINOR_GIFT_TILE_IMPROVEMENT", iGold);
	if (not pPlayer:CanMajorGiftTileImprovement(iActivePlayer)) then
		buttonText = "[COLOR_WARNING_TEXT]" .. buttonText .. "[ENDCOLOR]";
		Controls.TileImprovementGiftAnim:SetHide(true);
	else
		Controls.TileImprovementGiftAnim:SetHide(false);
	end
	Controls.TileImprovementGift:SetText(buttonText);
	SetButtonSize(Controls.TileImprovementGift, Controls.TileImprovementGiftButton, Controls.TileImprovementGiftAnim, Controls.TileImprovementGiftButtonHL);
	
	-- Tooltip info
	local iFriendsAmount = GameDefines["FRIENDSHIP_THRESHOLD_FRIENDS"];
	local iAlliesAmount = GameDefines["FRIENDSHIP_THRESHOLD_ALLIES"];
    local iFriendship = pPlayer:GetMinorCivFriendshipWithMajor(iActivePlayer);
	local strInfoTT = Locale.ConvertTextKey("TXT_KEY_POP_CSTATE_GOLD_STATUS_TT", iFriendsAmount, iAlliesAmount, iFriendship);
	strInfoTT = strInfoTT .. "[NEWLINE][NEWLINE]";
	strInfoTT = strInfoTT .. Locale.ConvertTextKey("TXT_KEY_POP_CSTATE_GOLD_TT");
	Controls.SmallGiftButton:SetToolTipString(strInfoTT);
	Controls.MediumGiftButton:SetToolTipString(strInfoTT);
	Controls.LargeGiftButton:SetToolTipString(strInfoTT);
	
	UpdateButtonStack();
end

----------------------------------------------------------------
-- Gold Gifts
----------------------------------------------------------------

function OnProcessITGClicked(val)
	bIsProcess = true;
	LuaEvents.ITGSG_CityStateContribution(g_iMinorCivID , val);
	PopulateProcessChoices()
end
Controls.ITGProcessGive1:RegisterCallback( Mouse.eLClick, OnProcessITGClicked );
Controls.ITGProcessGive2:RegisterCallback( Mouse.eLClick, OnProcessITGClicked );
Controls.ITGProcessGive3:RegisterCallback( Mouse.eLClick, OnProcessITGClicked );
Controls.ITGProcessGive4:RegisterCallback( Mouse.eLClick, OnProcessITGClicked );
Controls.ITGProcessGive5:RegisterCallback( Mouse.eLClick, OnProcessITGClicked );
Controls.ITGProcessGive1:SetVoid1(1);
Controls.ITGProcessGive2:SetVoid1(2);
Controls.ITGProcessGive3:SetVoid1(3);
Controls.ITGProcessGive4:SetVoid1(4);
Controls.ITGProcessGive5:SetVoid1(5);


function OnSmallGold ()
	local iActivePlayer = Game.GetActivePlayer();
	local pActivePlayer = Players[iActivePlayer];
	local iNumGoldPlayerHas = pActivePlayer:GetGold();
	
	if (iNumGoldPlayerHas >= iGoldGiftSmall) then
		Game.DoMinorGoldGift(g_iMinorCivID, iGoldGiftSmall);
		m_iLastAction = kiGiftedGold;
		OnCloseGive();
	end
end
Controls.SmallGiftButton:RegisterCallback( Mouse.eLClick, OnSmallGold );

function OnMediumGold ()
	local iActivePlayer = Game.GetActivePlayer();
	local pActivePlayer = Players[iActivePlayer];
	local iNumGoldPlayerHas = pActivePlayer:GetGold();
	
	if (iNumGoldPlayerHas >= iGoldGiftMedium) then
		Game.DoMinorGoldGift(g_iMinorCivID, iGoldGiftMedium);
		m_iLastAction = kiGiftedGold;
		OnCloseGive();
	end
end
Controls.MediumGiftButton:RegisterCallback( Mouse.eLClick, OnMediumGold );

function OnBigGold ()
	local iActivePlayer = Game.GetActivePlayer();
	local pActivePlayer = Players[iActivePlayer];
	local iNumGoldPlayerHas = pActivePlayer:GetGold();
	
	if (iNumGoldPlayerHas >= iGoldGiftLarge) then
		Game.DoMinorGoldGift(g_iMinorCivID, iGoldGiftLarge);
		m_iLastAction = kiGiftedGold;
		OnCloseGive();
	end
end
Controls.LargeGiftButton:RegisterCallback( Mouse.eLClick, OnBigGold );

----------------------------------------------------------------
-- Gift Unit
----------------------------------------------------------------
function OnGiftUnit()
    UIManager:DequeuePopup( ContextPtr );

	local interfaceModeSelection = InterfaceModeTypes.INTERFACEMODE_GIFT_UNIT;
	
	UI.SetInterfaceMode(interfaceModeSelection);
	UI.SetInterfaceModeValue(g_iMinorCivID);
end
Controls.UnitGiftButton:RegisterCallback( Mouse.eLClick, OnGiftUnit );

----------------------------------------------------------------
-- Gift Improvement
----------------------------------------------------------------
function OnGiftTileImprovement()
	local pMinor = Players[g_iMinorCivID];
	local iActivePlayer = Game.GetActivePlayer();
    
    if (pMinor:CanMajorGiftTileImprovement(iActivePlayer)) then
		UIManager:DequeuePopup( ContextPtr );

		local interfaceModeSelection = InterfaceModeTypes.INTERFACEMODE_GIFT_TILE_IMPROVEMENT;
		
		UI.SetInterfaceMode(interfaceModeSelection);
		UI.SetInterfaceModeValue(g_iMinorCivID);
	end
end
Controls.TileImprovementGiftButton:RegisterCallback( Mouse.eLClick, OnGiftTileImprovement );

----------------------------------------------------------------
-- Close Give Submenu
----------------------------------------------------------------
function OnCloseGive()
	bIsProcess = false;
	Controls.GiveStack:SetHide(true);
	Controls.TakeStack:SetHide(true);
	Controls.ITGProcessStack:SetHide(true);
	Controls.ButtonStack:SetHide(false);
	OnDisplay()
end
Controls.ExitGiveButton:RegisterCallback( Mouse.eLClick, OnCloseGive );
Controls.ExitITGProcessButton:RegisterCallback( Mouse.eLClick, OnCloseGive );

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- TAKE MENU
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
local iBullyGoldInfluenceLost = (GameDefines["MINOR_FRIENDSHIP_DROP_BULLY_GOLD_SUCCESS"] / 100) * -1; -- Since XML value is times 100 for fidelity, and negative
local iBullyUnitInfluenceLost = (GameDefines["MINOR_FRIENDSHIP_DROP_BULLY_WORKER_SUCCESS"] / 100) * -1; -- Since XML value is times 100 for fidelity, and negative
local iBullyUnitMinimumPop = 4; --antonjs: todo: XML

function PopulateTakeChoices()	
	local pPlayer = Players[g_iMinorCivID];
	local iActivePlayer = Game.GetActivePlayer();
	local buttonText = "";
	local ttText = "";
	
	local iBullyGold = pPlayer:GetMinorCivBullyGoldAmount(iActivePlayer);
	buttonText = Locale.Lookup("TXT_KEY_POPUP_MINOR_BULLY_GOLD_AMOUNT", iBullyGold, iBullyGoldInfluenceLost);
	ttText = pPlayer:GetMajorBullyGoldDetails(iActivePlayer);
	ttText = ttText .. "[NEWLINE][NEWLINE]This action prevents you from declaring war on this City-State for next 8 turns.";
	if (not pPlayer:CanMajorBullyGold(iActivePlayer)) then
		buttonText = "[COLOR_WARNING_TEXT]" .. buttonText .. "[ENDCOLOR]";
		Controls.GoldTributeAnim:SetHide(true);
	else
		Controls.GoldTributeAnim:SetHide(false);
	end
	Controls.GoldTributeLabel:SetText(buttonText);
	Controls.GoldTributeButton:SetToolTipString(ttText);
	SetButtonSize(Controls.GoldTributeLabel, Controls.GoldTributeButton, Controls.GoldTributeAnim, Controls.GoldTributeButtonHL);
	
	local sBullyUnit = GameInfo.Units["UNIT_WORKER"].Description; --antonjs: todo: XML or fn
	buttonText = Locale.Lookup("TXT_KEY_POPUP_MINOR_BULLY_UNIT_AMOUNT", sBullyUnit, iBullyUnitInfluenceLost);
	ttText = pPlayer:GetMajorBullyUnitDetails(iActivePlayer);
	if (not pPlayer:CanMajorBullyUnit(iActivePlayer)) then
		buttonText = "[COLOR_WARNING_TEXT]" .. buttonText .. "[ENDCOLOR]";
		Controls.UnitTributeAnim:SetHide(true);
	else
		Controls.UnitTributeAnim:SetHide(false);
	end
	Controls.UnitTributeLabel:SetText(buttonText);
	Controls.UnitTributeButton:SetToolTipString(ttText);
	SetButtonSize(Controls.UnitTributeLabel, Controls.UnitTributeButton, Controls.UnitTributeAnim, Controls.UnitTributeButtonHL);
	
	UpdateButtonStack();
end

----------------------------------------------------------------
-- Bully confirmation
----------------------------------------------------------------
function OnBullyButtonClicked ()

	local listofProtectingCivs = {};
	for iPlayerLoop = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
			
		pOtherPlayer = Players[iPlayerLoop];
			
		-- Don't test protection status with active player!
		if (iPlayerLoop ~= Game.GetActivePlayer()) then
			if (pOtherPlayer:IsAlive()) then
				if (pOtherPlayer:IsProtectingMinor(g_iMinorCivID)) then
					table.insert(listofProtectingCivs, Players[iPlayerLoop]:GetCivilizationShortDescriptionKey()); 
				end
			end
		end
	end
	
	local pMinor = Players[g_iMinorCivID];
	local cityStateName = Locale.Lookup(pMinor:GetCivilizationShortDescriptionKey());
	
	local bullyConfirmString = Locale.ConvertTextKey("TXT_KEY_CONFIRM_BULLY", cityStateName);
	local numProtectingCivs = #listofProtectingCivs;
	if(numProtectingCivs > 0) then
		if(numProtectingCivs == 1) then
			bullyConfirmString = Locale.ConvertTextKey("TXT_KEY_CONFIRM_BULLY_PROTECTED_CITY_STATE", cityStateName, listofProtectingCivs[1]);
		else
			local translatedCivs = {};
			for i,v in ipairs(listofProtectingCivs) do
				translatedCivs[i] = Locale.Lookup(v);
			end
		
			bullyConfirmString = Locale.ConvertTextKey("TXT_KEY_CONFIRM_BULLY_PROTECTED_CITY_STATE_MULTIPLE", cityStateName, table.concat(translatedCivs, ", "));
		end
	end
	
	Controls.BullyConfirmLabel:SetText( bullyConfirmString );
	Controls.BullyConfirm:SetHide(false);
	Controls.BGBlock:SetHide(true);
end

----------------------------------------------------------------
-- Take Gold
----------------------------------------------------------------
function OnGoldTributeButtonClicked()
	local pPlayer = Players[g_iMinorCivID];
	local iActivePlayer = Game.GetActivePlayer();
	
	if (pPlayer:CanMajorBullyGold(iActivePlayer)) then
		m_iPendingAction = kiBulliedGold;
		OnBullyButtonClicked();
		OnCloseTake();
	end
end
Controls.GoldTributeButton:RegisterCallback( Mouse.eLClick, OnGoldTributeButtonClicked );

----------------------------------------------------------------
-- Take Unit
----------------------------------------------------------------
function OnUnitTributeButtonClicked()
	local pPlayer = Players[g_iMinorCivID];
	local iActivePlayer = Game.GetActivePlayer();
	
	if (pPlayer:CanMajorBullyUnit(iActivePlayer)) then
		m_iPendingAction = kiBulliedUnit;
		OnBullyButtonClicked();
		OnCloseTake();
	end
end
Controls.UnitTributeButton:RegisterCallback( Mouse.eLClick, OnUnitTributeButtonClicked );

----------------------------------------------------------------
-- Close Take Submenu
----------------------------------------------------------------
function OnCloseTake()
	Controls.GiveStack:SetHide(true);
	Controls.TakeStack:SetHide(true);
	Controls.ButtonStack:SetHide(false);
	Controls.ITGProcessStack:SetHide(true)
end
Controls.ExitTakeButton:RegisterCallback( Mouse.eLClick, OnCloseTake );

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- BULLY CONFIRMATION POPUP
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function OnYesBully( )
	local iActivePlayer = Game.GetActivePlayer();
	if (m_iPendingAction == kiBulliedGold) then
		Game.DoMinorBullyGold(iActivePlayer, g_iMinorCivID);
		m_iPendingAction = kiNoAction;
		m_iLastAction = kiBulliedGold;
	elseif (m_iPendingAction == kiBulliedUnit) then
		Game.DoMinorBullyUnit(iActivePlayer, g_iMinorCivID);
		m_iPendingAction = kiNoAction;
		m_iLastAction = kiBulliedUnit;
	else
		print("Scripting error - Selected Yes for bully confrirmation dialog, but invalid PendingAction type");
	end

	Controls.BullyConfirm:SetHide(true);
	Controls.BGBlock:SetHide(false);
    UIManager:DequeuePopup( ContextPtr );
end
Controls.YesBully:RegisterCallback( Mouse.eLClick, OnYesBully );

function OnNoBully( )
	Controls.BullyConfirm:SetHide(true);
	Controls.BGBlock:SetHide(false);
end
Controls.NoBully:RegisterCallback( Mouse.eLClick, OnNoBully );

-------------------------------------------------
-- 'Active' (local human) player has changed
-------------------------------------------------
Events.GameplaySetActivePlayer.Add(OnCloseButtonClicked);