-------------------------------------------------
-- Diplomatic
-------------------------------------------------
include("IconSupport");

local thirdName;
local bChecks = {true, true, true, true}
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function ShowHideHandler(bIsHide, bInitState)
	if (not bIsHide) then
		Controls.OptionsPanel:SetHide(true);
		InitMajorCivList();
	end
end
ContextPtr:SetShowHideHandler(ShowHideHandler);

function GetAcronymOf(sText)
	local hS = ""
	for w in sText:gmatch("%S+") do 
		if w ~= "of" then
			hS = hS .. string.upper(string.sub(w, 1, 1))
		end
	end
	return hS;
end

-------------------------------------------------
-------------------------------------------------
function InitMajorCivList()

	local g_iUs = Game.GetActivePlayer();
	local g_pUs = Players[ g_iUs ];
	local g_iUsTeam = g_pUs:GetTeam();
	local g_pUsTeam = Teams[ g_iUsTeam ];
	LuaEvents.ITGSG_RequestGT();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	-- Clear buttons
	Controls.ItemStack:DestroyAllChildren();
	
	local count = 0;
	--------------------------------------------------------------------
	-- Loop through all the Majors the active player knows
	--------------------------------------------------------------------
	for iOtherPlayer = 0, GameDefines.MAX_MAJOR_CIVS - 1 do
		
		local pOtherPlayer = Players[ iOtherPlayer ];
		local iOtherTeam = pOtherPlayer:GetTeam();
		local pOtherTeam = Teams[ iOtherTeam ];
		
		-- Valid player? - Can't be us, and has to be alive
		if (iOtherPlayer ~= g_iUs and pOtherPlayer:IsAlive()) then

			-- Met this player?
			if (g_pUsTeam:IsHasMet(iOtherTeam)) then
			
				count = count + 1;
				
				local controlTable = {};
				ContextPtr:BuildInstanceForControl("MajorInstance", controlTable, Controls.ItemStack);
				
				local primaryColor, secondaryColor = pOtherPlayer:GetPlayerColors();
				local civType = pOtherPlayer:GetCivilizationType();
				local civInfo = GameInfo.Civilizations[ civType ];
				
				local otherLeaderType = pOtherPlayer:GetLeaderType();
				local otherLeaderInfo = GameInfo.Leaders[otherLeaderType];
		
				IconHookup(otherLeaderInfo.PortraitIndex, 128, otherLeaderInfo.IconAtlas, controlTable.LeaderPortrait);
				CivIconHookup(iOtherPlayer, 64, controlTable.CivSymbol, controlTable.CivIconBG, controlTable.CivIconShadow, false, true);

				-- Player name
				local otherName;
				if (pOtherPlayer:IsHuman()) then
					otherName = pOtherPlayer:GetNickName();
				else
					otherName = pOtherPlayer:GetName();
				end
				controlTable.LeaderName:SetText(otherName);
				
				-- Civ name
				controlTable.CivName:LocalizeAndSetText(civInfo.ShortDescription);
				
				-- Civ Era
				local iEra = pOtherPlayer:GetCurrentEra();
				controlTable.EraLabel:SetText(Locale.ConvertTextKey(GameInfo.Eras[iEra].Description));

				controlTable.GovLabel:SetText(Locale.Lookup(gData.Players[iOtherPlayer].GovString))
				controlTable.CultLabel:SetText(gData.Players[iOtherPlayer].CultureString)
				controlTable.ArtEraLabel:SetText(gData.Globals.CultureErasNames[gData.Players[iOtherPlayer].CultureType][gData.Players[iOtherPlayer].CultureEra]);
				
				if gData.Players[iOtherPlayer].OfficialReligion then
					controlTable.ReligionLabel:SetText(GameInfo.Religions[gData.Players[iOtherPlayer].OfficialReligion].IconString .. " " .. Locale.Lookup(Game.GetReligionName(gData.Players[iOtherPlayer].OfficialReligion)))
					controlTable.ReligionLabel:SetHide(false)
				else
					controlTable.ReligionLabel:SetHide(true)
				end
				-- What happens if we click this?
				controlTable.Button:SetVoid1(iOtherPlayer);
				controlTable.Button:RegisterCallback(Mouse.eLClick, LeaderSelected);
				
				-- Policies
				if gData.Players[iOtherPlayer].IdeologyName then
					controlTable.LabelSocialPolicies:SetText("[COLOR_XP_BLUE]" .. GetAcronymOf(gData.Players[iOtherPlayer].IdeologyName));
					controlTable.LabelSocialPolicies:SetToolTipString(gData.Players[iOtherPlayer].IdeologyName);
				else
					controlTable.LabelSocialPolicies:LocalizeAndSetText("Developments");
					controlTable.LabelSocialPolicies:SetToolTipString("");
				end
				LuaEvents.ITG_RequestCultIdeoBonusHelp(iOtherPlayer, true)
				local ciData = MapModData.ITGSG;
				MapModData.ITGSG = nil;
				for i, data in pairs(ciData) do
					local textControls = {};
					ContextPtr:BuildInstanceForControl("LTextEntry", textControls, controlTable.PoliciesStack);
					textControls.Text:SetText(data.Name);
					textControls.Text:SetToolTipString(data.Help);
				end
				
				-- Wonders
				for pBuilding in GameInfo.Buildings() do
					local iBuilding = pBuilding.ID;
					
					local pBuildingClass = GameInfo.BuildingClasses[pBuilding.BuildingClass];
					if (pBuildingClass.MaxGlobalInstances > 0) then
						if (pOtherPlayer:CountNumBuildings(iBuilding) > 0) then
							local textControls = {};
							ContextPtr:BuildInstanceForControl("LTextEntry", textControls, controlTable.WondersStack);
							textControls.Text:LocalizeAndSetText(pBuilding.Description);
						end
					end
				end
								
				local bHasEntry = false;

				local iEmbassy = 0;
				local iEmbargo = 0;

				if gData.Players[g_iUs] then
					if (gData.Players[iOtherPlayer].Embargos[g_iUs]) then
						iEmbargo = 1;
					end
					if (gData.Players[g_iUs].Embargos[iOtherPlayer]) then
						iEmbargo = iEmbargo + 2;
					end
				end

				if (Teams[iOtherTeam]:HasEmbassyAtTeam(g_iUsTeam)) then
					iEmbassy = 1;
				end
				if (g_pUsTeam:HasEmbassyAtTeam(iOtherTeam)) then
					iEmbassy = iEmbassy + 2;
				end

				local isDoFwithUs = (pOtherPlayer:IsDoF(g_iUs)) 
				if isDoFwithUs then
					local hIconString = ""
					if bChecks[3] and (iEmbassy == 3) then
						hIconString = hIconString .. "[ICON_ITG_LEGAL_POWER]"
					end
					if bChecks[2] and (iEmbargo == 3) then
						hIconString = hIconString .. "[ICON_BLOCKADED]"
					end
					if g_pUsTeam:IsDefensivePact(iOtherTeam) then
						hIconString = hIconString .. "[ICON_STRENGTH]"
					end
					local textControls = {};
					ContextPtr:BuildInstanceForControl("TextEntry", textControls, controlTable.PactStack);
					local text = Locale.Lookup("TXT_KEY_DIPLO_FRIENDS_WITH", "TXT_KEY_YOU");
					if hIconString ~= "" then
						text = hIconString .. " " .. text
					end
					textControls.Text:SetText(text);
				else
					if g_pUsTeam:IsDefensivePact(iOtherTeam) then
						local textControls = {};
						ContextPtr:BuildInstanceForControl("TextEntry", textControls, controlTable.PactStack);
						textControls.Text:SetText("[ICON_STRENGTH] Defensive Pact with [COLOR_SELECTED_TEXT]You[ENDCOLOR].");
					end
				end
				
				if bChecks[3] and (iEmbassy > 0) and ((not isDoFwithUs) or iEmbassy ~= 3) then
					controlTable.EmbassyLabel:SetHide(false);
					if iEmbassy == 1 then
						controlTable.EmbassyLabel:SetText("Their Embassy in our Capital")
					elseif iEmbassy == 2 then
						controlTable.EmbassyLabel:SetText("Our Embassy in their Capital")
					elseif iEmbassy == 3 then
						controlTable.EmbassyLabel:SetText("Shared Embassies")
					end
				else
					controlTable.EmbassyLabel:SetHide(true);
				end

				
				if bChecks[2] and (iEmbargo > 0) and ((not isDoFwithUs) or iEmbargo ~= 3) then
					controlTable.EmbargoLabel:SetHide(false);
					if iEmbargo == 1 then
						controlTable.EmbargoLabel:SetText("[COLOR_WARNING_TEXT]Embargo on us[ENDCOLOR]")
					elseif iEmbargo == 2 then
						controlTable.EmbargoLabel:SetText("[COLOR_WARNING_TEXT]Embargo on them[ENDCOLOR]")
					elseif iEmbargo == 3 then
						controlTable.EmbargoLabel:SetText("[COLOR_WARNING_TEXT]Mutual Embargo[ENDCOLOR]")
					end
				else
					controlTable.EmbargoLabel:SetHide(true);
				end

				-- War
				if gData.Players[iOtherPlayer].IsConsideredBarbarian then
					controlTable.WarLabel:SetHide(false);
					controlTable.WarLabel:SetText("[COLOR_WARNING_TEXT]EXILE[ENDCOLOR]");
					bHasEntry = true;
				elseif (g_pUsTeam:IsAtWar(iOtherTeam)) then
					controlTable.WarLabel:SetHide(false);
					controlTable.WarLabel:SetText("[COLOR_WARNING_TEXT]" .. Locale.ConvertTextKey("{TXT_KEY_DO_AT_WAR:upper}") .. "[ENDCOLOR]");
					bHasEntry = true;
				else
					controlTable.WarLabel:SetHide(true);
				end
				
				-- We Denounced Them
				if (Players[g_iUs]:IsDenouncedPlayer(iOtherPlayer)) then
					controlTable.DenounceLabel:SetHide(false);
					
					-- Backstab?
					if (pOtherPlayer:IsFriendDenouncedUs(g_iUs) or pOtherPlayer:IsFriendDeclaredWarOnUs(g_iUs)) then
						controlTable.DenounceLabel:SetText(Locale.ConvertTextKey("TXT_KEY_DIPLO_YOU_HAVE_BACKSTABBED"));
					-- Normal denouncement
					else
						local text = Locale.Lookup("TXT_KEY_DIPLO_YOU_HAVE_DENOUNCED")
						if(Players[g_iUs].GetDenouncedPlayerCounter ~= nil) then
							local turnsLeft = GameDefines.DENUNCIATION_EXPIRATION_TIME - Players[g_iUs]:GetDenouncedPlayerCounter(iOtherPlayer);
							text = text .. " (" .. Locale.Lookup("TXT_KEY_DECLARE_WAR_DEALS_TURNS_LEFT", turnsLeft) .. ")";
						end
					
						controlTable.DenounceLabel:SetText(text);
					end
					bHasEntry = true;
				else
					controlTable.DenounceLabel:SetHide(true);
				end
				
				-- The following goes into the diplo stack on the right
				for iThirdPlayer = 0, GameDefines.MAX_MAJOR_CIVS - 1 do
					if (iThirdPlayer ~= iOtherPlayer) then
						local pThirdPlayer = Players[iThirdPlayer];
						if (pThirdPlayer ~= nil and pThirdPlayer:IsAlive()) then
							local iThirdTeam  = pThirdPlayer:GetTeam();
							
							if (g_pUsTeam:IsHasMet(iThirdTeam) and iThirdPlayer ~= g_iUs) then
								local embassy = 0;
								if (pOtherTeam:HasEmbassyAtTeam(iThirdTeam)) then
									embassy = 1;
								end
								if (Teams[iThirdTeam]:HasEmbassyAtTeam(iOtherTeam)) then
									embassy = embassy + 2;
								end
								iEmbargo = 0;
								if (gData.Players[iOtherPlayer].Embargos[iThirdPlayer]) then
									iEmbargo = 1;
								end
								if (gData.Players[iThirdPlayer].Embargos[iOtherPlayer]) then
									iEmbargo = iEmbargo + 2;
								end
								if (pThirdPlayer:IsHuman()) then
									thirdName = pThirdPlayer:GetNickName();
								-- AI
								else
									thirdName = pThirdPlayer:GetCivilizationShortDescription();
								end
								local isDoFentry = (pOtherPlayer:IsDoF(iThirdPlayer)) 
								if isDoFentry then
									local hIconString = ""
									if bChecks[3] and (embassy == 3) then
										hIconString = hIconString .. "[ICON_ITG_LEGAL_POWER]"
									end
									if bChecks[2] and (embargo == 3) then
										hIconString = hIconString .. "[ICON_BLOCKADED]"
									end
									if pOtherTeam:IsDefensivePact(iThirdTeam) then
										hIconString = hIconString .. "[ICON_STRENGTH]"
									end

									local textControls = {};
									ContextPtr:BuildInstanceForControl("TextEntry", textControls, controlTable.PactStack);
									
									local text = Locale.Lookup("TXT_KEY_DIPLO_FRIENDS_WITH", thirdName);
									if hIconString ~= "" then
										text = hIconString .. " " .. text
									end
									textControls.Text:SetText(text);
								else
									if pOtherTeam:IsDefensivePact(iThirdTeam) then
										local textControls = {};
										ContextPtr:BuildInstanceForControl("TextEntry", textControls, controlTable.PactStack);
										textControls.Text:SetText("[ICON_STRENGTH] Defensive Pact with [COLOR_SELECTED_TEXT]" .. thirdName .. "[ENDCOLOR].");
									end
								end

								if bChecks[3] and (embassy > 0) and ((not isDoFentry) or embassy ~= 3) then
									local textControls = {};
									ContextPtr:BuildInstanceForControl("TextEntry", textControls, controlTable.EmbassyStack);
									if embassy == 3 then
										textControls.Text:LocalizeAndSetText("Shared Embassies with " .. pThirdPlayer:GetCivilizationShortDescription())
									elseif embassy == 2 then
										textControls.Text:LocalizeAndSetText("Embassy from " .. pThirdPlayer:GetCivilizationShortDescription())
									else
										textControls.Text:LocalizeAndSetText("Embassy at " .. pThirdPlayer:GetCivilizationShortDescription())
									end
								end
								if bChecks[2] and (iEmbargo > 0) and ((not isDoFentry) or iEmbargo ~= 3) then
									local textControls = {};
									ContextPtr:BuildInstanceForControl("TextEntry", textControls, controlTable.EmbargoStack);
									if iEmbargo == 3 then
										textControls.Text:LocalizeAndSetText("[COLOR_WARNING_TEXT]Mutual Embargo with " .. pThirdPlayer:GetCivilizationShortDescription() .. "[ENDCOLOR]")
									elseif iEmbargo == 2 then
										textControls.Text:LocalizeAndSetText("[COLOR_WARNING_TEXT]Embargo from " .. pThirdPlayer:GetCivilizationShortDescription() .. "[ENDCOLOR]")
									else
										textControls.Text:LocalizeAndSetText("[COLOR_WARNING_TEXT]Embargo on " .. pThirdPlayer:GetCivilizationShortDescription() .. "[ENDCOLOR]")
									end
								end
							end
						end
					end
				end



				-- War with 3rd party
				if not gData.Players[iOtherPlayer].IsConsideredBarbarian then
					for iThirdPlayer = 0, GameDefines.MAX_MAJOR_CIVS - 1 do
						if (iThirdPlayer ~= iOtherPlayer) then
							local pThirdPlayer = Players[iThirdPlayer];
						
							if (pThirdPlayer ~= nil and pThirdPlayer:IsAlive()) and (not gData.Players[iThirdPlayer].IsConsideredBarbarian) then
								local iThirdTeam  = pThirdPlayer:GetTeam();
							
								if (g_pUsTeam:IsHasMet(iThirdTeam) or iThirdPlayer == g_iUs) then
								
									-- War (NOT us - handled above)
									if (pOtherTeam:IsAtWar(iThirdTeam) and iThirdPlayer ~= g_iUs) then
										bHasEntry = true;
									
										-- Human
										if (pThirdPlayer:IsHuman()) then
											thirdName = pThirdPlayer:GetNickName();
										-- AI
										else
											thirdName = pThirdPlayer:GetCivilizationShortDescription();
										end
									
										local textControls = {};
										ContextPtr:BuildInstanceForControl("TextEntry", textControls, controlTable.PactStack);
										textControls.Text:LocalizeAndSetText("TXT_KEY_AT_WAR_WITH", thirdName);
									end
								end
							end
						end
					end
				end
				
				-- Denouncements
				if bChecks[1] then
					for iThirdPlayer = 0, GameDefines.MAX_MAJOR_CIVS - 1 do
						if (iThirdPlayer ~= iOtherPlayer) then
							local pThirdPlayer = Players[iThirdPlayer];
						
							if (pThirdPlayer ~= nil and pThirdPlayer:IsAlive()) then
								local iThirdTeam  = pThirdPlayer:GetTeam();
							
								if (g_pUsTeam:IsHasMet(iThirdTeam) or iThirdPlayer == g_iUs) then
								
									-- Denounced
									if (pOtherPlayer:IsDenouncedPlayer(iThirdPlayer) or pThirdPlayer:IsFriendDeclaredWarOnUs(iOtherPlayer)) then
										bHasEntry = true;
									
										-- Us
										if (iThirdPlayer == g_iUs) then
											thirdName = "TXT_KEY_YOU";
										-- Human
										elseif (pThirdPlayer:IsHuman()) then
											thirdName = pThirdPlayer:GetNickName();
										-- AI
										else
											thirdName = pThirdPlayer:GetCivilizationShortDescription();
										end
									
										local textControls = {};
										ContextPtr:BuildInstanceForControl("TextEntry", textControls, controlTable.PactStack);
									
										-- Backstab?
										if (pThirdPlayer:IsFriendDenouncedUs(iOtherPlayer) or pThirdPlayer:IsFriendDeclaredWarOnUs(iOtherPlayer)) then
											textControls.Text:LocalizeAndSetText("TXT_KEY_DIPLO_BACKSTABBED", thirdName);
										-- Normal denouncement
										else
											local text = Locale.Lookup("TXT_KEY_DIPLO_DENOUNCED", thirdName);
							--				if(pOtherPlayer.GetDenouncedPlayerCounter ~= nil) then
							--					local turnsLeft = GameDefines.DENUNCIATION_EXPIRATION_TIME - pOtherPlayer:GetDenouncedPlayerCounter(iThirdPlayer);
							--					text = text .. " (" .. Locale.Lookup("TXT_KEY_DECLARE_WAR_DEALS_TURNS_LEFT", turnsLeft) .. ")";
        					--				end
											textControls.Text:SetText(text);
										
										end
									end
								end
							end
						end
					end
				end
				

				-- Allied to Minors
				if bChecks[4] then
					for iThirdPlayer = GameDefines.MAX_MAJOR_CIVS, GameDefines.MAX_CIV_PLAYERS - 1 do
						if (iThirdPlayer ~= iOtherPlayer) then
							local pThirdPlayer = Players[iThirdPlayer];
						
							if (pThirdPlayer ~= nil and pThirdPlayer:IsAlive()) then
								local iThirdTeam  = pThirdPlayer:GetTeam();
							
								if (g_pUsTeam:IsHasMet(iThirdTeam) or iThirdPlayer == g_iUs) then
								
									-- Allied with Minor
									if (pThirdPlayer:IsAllies(iOtherPlayer)) then
										bHasEntry = true;
									
										thirdName = pThirdPlayer:GetCivilizationShortDescription();
									
										local textControls = {};
										ContextPtr:BuildInstanceForControl("TextEntry", textControls, controlTable.PactStack);
										textControls.Text:LocalizeAndSetText("TXT_KEY_ALLIED_WITH", thirdName);
									end
								end
							end
						end
					end
				end

				--controlTable.NothingLabel:SetHide(bHasEntry);
				controlTable.PactStack:CalculateSize();
				controlTable.PoliciesStack:CalculateSize();
				controlTable.WondersStack:CalculateSize();
				controlTable.WarStack:CalculateSize();
				controlTable.EmbassyStack:CalculateSize();
				controlTable.EmbargoStack:CalculateSize();
				controlTable.MinorStack:CalculateSize();
				controlTable.OuterStack:CalculateSize();
				
		--		print("controlTable.OuterStack:GetSizeY()" .. controlTable.OuterStack:GetSizeY());
		--		print("controlTable.PoliciesStack:GetSizeY()" .. controlTable.PoliciesStack:GetSizeY());
		--		print("controlTable.WondersStack:GetSizeY()" .. controlTable.WondersStack:GetSizeY());
				
				local sizeY = math.max(controlTable.OuterStack:GetSizeY(), controlTable.PoliciesStack:GetSizeY() + 25);
				sizeY = math.max(sizeY, controlTable.WondersStack:GetSizeY());
				if (sizeY < 120) then 
					sizeY = 120;
				end
				
				local newSizeY = sizeY + controlTable.OuterStack:GetOffsetY()
				
				controlTable.Button:SetSizeY(newSizeY);
				controlTable.ButtonAnim:SetSizeY(newSizeY);
				controlTable.ButtonAnimGrid:SetSizeY(newSizeY+5);
				controlTable.OuterStack:ReprocessAnchoring();
				
				controlTable.PlayerInfoStack:CalculateSize();
				controlTable.PlayerInfoStack:ReprocessAnchoring();
			end
		end
	end

	if (count > 0) then
		Controls.NoMajorCivs:SetHide(true);
	else
		Controls.NoMajorCivs:SetHide(false);
	end
	
	Controls.ItemStack:CalculateSize();
	Controls.ItemStack:ReprocessAnchoring();
	Controls.MajorCivScrollPanel:CalculateInternalSize();
end



-------------------------------------------------
-- On Leader Selected
-------------------------------------------------
function LeaderSelected(ePlayer)

	if (Players[ ePlayer ]:IsHuman()) then
		Events.OpenPlayerDealScreenEvent(ePlayer);
	else
		
		UI.SetRepeatActionPlayer(ePlayer);
		UI.ChangeStartDiploRepeatCount(1);
		Players[ ePlayer ]:DoBeginDiploWithHuman();	

	end
end


function OnCheck1( bIsChecked )
	bChecks[1] = bIsChecked;
	InitMajorCivList();
end
Controls.CheckDenounce:RegisterCheckHandler( OnCheck1 );
function OnCheck2( bIsChecked )
	bChecks[2] = bIsChecked;
	InitMajorCivList();
end
Controls.CheckEmbargo:RegisterCheckHandler( OnCheck2 );
function OnCheck3( bIsChecked )
	bChecks[3] = bIsChecked;
	InitMajorCivList();
end
Controls.CheckEmbassy:RegisterCheckHandler( OnCheck3 );
function OnCheck4( bIsChecked )
	bChecks[4] = bIsChecked;
	InitMajorCivList();
end
Controls.CheckCS:RegisterCheckHandler( OnCheck4 );

function OnOptions()	
	Controls.OptionsPanel:SetHide( not Controls.OptionsPanel:IsHidden() );
end
Controls.OptionsButton:RegisterCallback( Mouse.eLClick, OnOptions );
Controls.MainOptionStack:CalculateSize();
Controls.OptionsPanel:DoAutoSize();