-------------------------------------------------
-- Espionage Overview Popup
-------------------------------------------------
include( "IconSupport" );
include( "InstanceManager" );
include( "FLuaVector" );
include( "CityStateStatusHelper" );

local g_AgentManager = GenerationalInstanceManager:new( "AgentInstance", "Base", Controls.AgentStack);
--local g_MyCityManager = GenerationalInstanceManager:new( "MyCityInstance", "Base", Controls.MyCityStack);
local g_MyCityButtonManager = GenerationalInstanceManager:new("MyCityButtonInstance", "Base", Controls.MyCityStack);
--use button
--local g_TheirCityManager = GenerationalInstanceManager:new( "TheirCityInstance", "Base", Controls.TheirCityStack);
local g_TheirCityButtonManager = GenerationalInstanceManager:new( "TheirCityButtonInstance", "Base", Controls.TheirCityStack);
local g_IntrigueManager = GenerationalInstanceManager:new( "IntrigueMessageInstance", "Base", Controls.IntrigueMessageStack);
--local g_MapManager = GenerationalInstanceManager:new( "MapInstance", "Base", Controls.MapCont);
local agentSelected = nil;
local g_ConfirmAction = nil;
local g_CitySelected = nil;
local g_Players, g_General, g_GPtype;
local g_PreviousLocationBoxColor = "231,213,0,255";
local bOkayToProcess = true;
local mapID = 1;
local requiresConfirm = false;

local EspionageActions = {}
for row in DB.Query("SELECT * FROM ITG_EspionageActions") do
	local index = row.ID;
	EspionageActions[index] = {}
	for a, b in pairs(row) do
		EspionageActions[index][a] = b
	end
end

function GetEspActionHelp(id)
	local hString = Locale.Lookup(EspionageActions[id].Help)
	if EspionageActions[id].TurnsNeeded then
		hString = hString .. "[NEWLINE][NEWLINE]" .. Locale.ConvertTextKey("TXT_KEY_ITG_ESP_ACTION_TURNS_NEEDED", EspionageActions[id].TurnsNeeded)
	end
	if EspionageActions[id].OnlyOne then
		hString = hString .. "[NEWLINE][NEWLINE]" .. Locale.Lookup("TXT_KEY_ITG_ESP_ACTION_ONLY_ONE")
	end
	return hString;
end

 local g_ProgressBarStates = {{
		IconOffset = {x = 45,y = 45},
		ProgressBarTexture = "MeterBarGreatEspionageGreen.dds",
	}, {
		IconOffset = {x = 45,y = 0},
		ProgressBarTexture = "MeterBarGreatEspionageBlue.dds",
	}, {
		IconOffset = {x = 45, y = 0},
		ProgressBarTexture = "MeterBarGreatPersonGold.dds",
	}, {
		IconOffset = {x = 45,y = 0},
		ProgressBarTexture = "MeterBarGreatPersonGold.dds",
	}, {
		IconOffset = {x = 45, y = 90},
		ProgressBarTexture = "MeterBarGreatPersonGold.dds",
	}, {
		IconOffset = {x = 45,y = 0},
		ProgressBarTexture = "MeterBarGreatEspionageBlue.dds",
	},
}

function OnActivePlayerTurnEnd()
	bOkayToProcess = false;
end
Events.ActivePlayerTurnEnd.Add( OnActivePlayerTurnEnd );

function OnActivePlayerTurnStart()
	bOkayToProcess = true;
end
Events.ActivePlayerTurnStart.Add( OnActivePlayerTurnStart );


g_Tabs = {
	["Overview"] = {
		Panel = Controls.OverviewPanel,
		SelectHighlight = Controls.OverviewSelectHighlight,
	},
	
	["Intrigue"] = {
		Panel = Controls.IntriguePanel,
		SelectHighlight = Controls.IntrigueSelectHighlight,
	},
}
-- Used for Piano Keys
local g_PianoKeys = {
	[0] = {19/255,32/255,46/255,120/255},
	[1] = {12/255,22/255,30/255,120/255},
}

local g_RenaissanceEraType;
local renaissanceEra = GameInfo.Eras["ERA_RENAISSANCE"];
if(renaissanceEra ~= nil) then
	g_RenaissanceEraType = renaissanceEra.ID;
end

-------------------------------------------------
-------------------------------------------------
function OnPopupMessage(popupInfo)
	
	if (not Game.IsOption("GAMEOPTION_NO_ESPIONAGE")) then
		local popupType = popupInfo.Type;
		if popupType ~= ButtonPopupTypes.BUTTONPOPUP_ESPIONAGE_OVERVIEW then
			return;
		end
		
		g_PopupInfo = popupInfo;
		
		if( g_PopupInfo.Data1 == 1 ) then
    		if( ContextPtr:IsHidden() == false ) then
    			OnClose();
			else
        		UIManager:QueuePopup( ContextPtr, PopupPriority.InGameUtmost );
    		end
		else
			UIManager:QueuePopup( ContextPtr, PopupPriority.SocialPolicy );
		end
    end
   	
end
Events.SerialEventGameMessagePopup.Add( OnPopupMessage );

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function OnClose()
	UIManager:DequeuePopup(ContextPtr);
end
Controls.CloseButton:RegisterCallback(Mouse.eLClick, OnClose);

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function InputHandler( uiMsg, wParam, lParam )
    ----------------------------------------------------------------        
    -- Key Down Processing
    ----------------------------------------------------------------        
    if uiMsg == KeyEvents.KeyDown then
        if (wParam == Keys.VK_ESCAPE) then
			if(Controls.ChooseConfirm:IsHidden())then
	            OnClose();
	        else
				Controls.ChooseConfirm:SetHide(true);
           	end
			return true;
        end
        
        -- Do Nothing.
        if(wParam == Keys.VK_RETURN) then
			return true;
        end
    end
end
ContextPtr:SetInputHandler( InputHandler );
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

function ShowHideHandler( bIsHide, bInitState )
    if( not bInitState ) then
        if( not bIsHide ) then
        	UI.incTurnTimerSemaphore(); 
        	
			if(g_RenaissanceEraType ~= nil) then
				local activePlayer = Players[Game.GetActivePlayer()];
				local bUseModern = activePlayer:GetCurrentEra() > g_RenaissanceEraType;
				Controls.ModernBackground:SetHide(not bUseModern);
				Controls.EarlyBackground:SetHide(bUseModern);
			end
        	
        	Events.SerialEventGameMessagePopupShown(g_PopupInfo);
        	TabSelect(g_CurrentTab);
        else
			if(g_PopupInfo ~= nil) then
				Events.SerialEventGameMessagePopupProcessed.CallImmediate(g_PopupInfo.Type, 0);
			end
            UI.decTurnTimerSemaphore();
        end
    end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );

function TabSelect(tab)
	for i,v in pairs(g_Tabs) do
		local bHide = i ~= tab;
		v.Panel:SetHide(bHide);
		v.SelectHighlight:SetHide(bHide);
	end
	g_General = nil;
	g_Players = nil;
	g_GPtype = nil;
	g_ConfirmAction = nil;
	agentSelected = nil;	
	g_CitySelected = nil;
	g_CurrentTab = tab;
	RefreshTheirCities()
	RefreshMyCities()
	Refresh();
end
Controls.TabButtonOverview:RegisterCallback( Mouse.eLClick, function() TabSelect("Overview"); end);
Controls.TabButtonIntrigue:RegisterCallback( Mouse.eLClick, function() TabSelect("Intrigue"); end );

function OnPopupMessage(popupInfo)
	
	local popupType = popupInfo.Type;
	if popupType ~= ButtonPopupTypes.BUTTONPOPUP_ESPIONAGE_OVERVIEW then
		return;
	end
		
	g_PopupInfo = popupInfo;
		
	if( g_PopupInfo.Data1 == 1 ) then
    	if( ContextPtr:IsHidden() == false ) then
    		OnClose();
		else
        	UIManager:QueuePopup( ContextPtr, PopupPriority.InGameUtmost );
    	end
	else
		UIManager:QueuePopup( ContextPtr, PopupPriority.SocialPolicy );
	end
   	
end
Events.SerialEventGameMessagePopup.Add( OnPopupMessage );

function OnClose()
	UIManager:DequeuePopup(ContextPtr);
	g_General = nil;
	g_GPtype = nil;
	g_Players = nil;
	g_ConfirmAction = nil;
	agentSelected = nil;	
	g_CitySelected = nil;
end
Controls.CloseButton:RegisterCallback(Mouse.eLClick, OnClose);

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

function BuildPotentialModifierTT ()

end

function BuildExtraCatchSpiesModifierTT ()

end

function OnConfirmed()
	if bOkayToProcess then
		LuaEvents.ITGSG_EspionageCall(Game.GetActivePlayer(), agentSelected, g_ConfirmAction, g_CitySelected, g_Players, g_General, g_GPtype)
		g_General = nil;
		g_Players = nil;
		g_ConfirmAction = nil;
		agentSelected = nil;	
		g_CitySelected = nil;
		RefreshTheirCities()
		RefreshMyCities()
		Refresh()
	end
end

function ClickedOnAssignAction(it)	
	if bOkayToProcess then
		g_ConfirmAction = nil;
		RefreshTheirCities()
		g_ConfirmAction = it;
		if EspionageActions[it].RequiresTarget then
			RefreshTheirCities(it)
		elseif requiresConfirm then
			Controls.ChooseConfirm:SetHide(false)
		else
			OnConfirmed()
		end
		RefreshMyCities()
	end
end

function RefreshMyCities()
	--actions
	local bTickTock = true;
	g_MyCityButtonManager:ResetInstances();
	if agentSelected then
		LuaEvents.ITGSG_EspionageValidActions(Game.GetActivePlayer(), agentSelected);
		local gData = MapModData.ITGSG;
		MapModData.ITGSG = nil;
		for i, pos in pairs(EspionageActions) do
			if gData[i] then
				bTickTock = not bTickTock;
				local instance = g_MyCityButtonManager:GetInstance();
				if i == g_ConfirmAction then
					instance.ActionName:SetText("[COLOR_POSITIVE_TEXT]" .. Locale.Lookup(pos.Description) .. "[ENDCOLOR]")
				else
					instance.ActionName:SetText(Locale.Lookup(pos.Description))
				end
				instance.ActionName:SetToolTipString(GetEspActionHelp(pos.ID))
				instance.CitySelectButton:SetVoid1(pos.ID)
				instance.Base:SetColorVal(unpack(bTickTock and g_PianoKeys[0] or g_PianoKeys[1]));
				instance.CitySelectButton:RegisterCallback( Mouse.eLClick, ClickedOnAssignAction );
			end
		end
	end
	
	Controls.MyCityStack:CalculateSize();
	Controls.MyCityStack:ReprocessAnchoring();
	Controls.MyCityScrollPanel:CalculateInternalSize();
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

function OnReassign(it)
	if bOkayToProcess then
		agentSelected = it;
		g_ConfirmAction = nil;
		RefreshMyCities()
		RefreshTheirCities()
	end
end

function OnCitySelected(d1, d2, d3, d4)
	if bOkayToProcess then
		g_CitySelected = d1;
		g_Players = d2;
		g_General = d3;
		g_GPtype = d4;
		if requiresConfirm then
			Controls.ChooseConfirm:SetHide(false)
		else
			OnConfirmed()
		end
	end
end

function RefreshTheirCities(type)
	Controls.NoAvailableTargets:SetHide(true)
	local iPlayer = Game.GetActivePlayer();
	g_TheirCityButtonManager:ResetInstances();
	local iTeam = Game.GetActiveTeam();
	local bTickTock = true;
	if g_ConfirmAction then
		local tohide = true;
		LuaEvents.ITGSG_EspionageValidData(type);
		local gData = MapModData.ITGSG;
		MapModData.ITGSG = nil;
		if gData and #gData > 0 then
			tohide = false;
			for g, data in pairs(gData) do
				local instance = g_TheirCityButtonManager:GetInstance()
				bTickTock = not bTickTock;
				instance.CitySelectButton:SetDisabled(data.Already)
				if data.Already then
					instance.CitySelectButton:SetAlpha(0.5)
					instance.CitySelectButton:SetToolTipString("This target is currently taken care of.")
				else
					instance.CitySelectButton:SetAlpha(1.0)
					instance.CitySelectButton:SetToolTipString("")
				end
				local bCapital = false;
				if data.InfoCity and data.TargetCity then
					bCapital = Map.GetPlotByIndex(data.InfoCity):GetPlotCity():IsCapital();
				end
				if bCapital then
					instance.CityName:SetText("[ICON_CAPITAL]" .. data.TargetCity)
				else
					instance.CityName:SetText(data.TargetGPString or data.TargetCity or "")
				end
				instance.CivilizationName:SetText(data.TargetMain or "");
				CivIconHookup(data.PlayerIcon, 32, instance.CivIcon, instance.CivIconBG, instance.CivIconShadow, false, true, instance.CivIconHighlight);
				instance.CitySelectButton:RegisterCallback( Mouse.eLClick, function() OnCitySelected(data.InfoCity, data.InfoPlayer, data.TargetGreatPerson) end );
				instance.Base:SetColorVal(unpack(bTickTock and g_PianoKeys[0] or g_PianoKeys[1]));
				instance.CityPopulation:SetText(data.ExtraInfo or "")
			end
		end
		Controls.NoAvailableTargets:SetHide(not tohide)
	end
	Controls.TheirCityStack:CalculateSize();
	Controls.TheirCityStack:ReprocessAnchoring();
	Controls.TheirCityScrollPanel:CalculateInternalSize();
end

function RefreshAgents()
	local gPlayer = Game.GetActivePlayer();
	LuaEvents.ITGSG_RequestPD(gPlayer);
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	local bTickTock = true;
	g_AgentManager:ResetInstances();
	Controls.NoMoreSpiesLabel:SetHide(#gData.Agents ~= 0)


	for it, data in pairs(gData.Agents) do
		if (not data.IgnoreItForever) then
			local agentEntry = g_AgentManager:GetInstance();
			globalentry = agentEntry;
			agentEntry.AgentName:LocalizeAndSetText(data.Name);
			agentEntry.AgentRank:SetToolTipString(data.XP .. "XP");
			if data.Rank == 1 then
				agentEntry.AgentRank:SetTextureOffset(Vector2(0,42))
			elseif data.Rank == 2 then
				agentEntry.AgentRank:SetTextureOffset(Vector2(0,21))
			else
				agentEntry.AgentRank:SetTextureOffset(Vector2(0,0))
			end
			agentEntry.AgentActivityProgress:SetHide(true);
			agentEntry.AgentActivityProgressFrame:SetHide(true);
			agentEntry.AgentActivityProgressBack:SetHide(true);
			agentEntry.AgentActivityIcon2:SetHide(true)
				agentEntry.AgentIcon:SetToolTipString("");
				agentEntry.AgentIcon:SetHide(false);
				agentEntry.DiplomatIcon:SetHide(true);
				bTickTock = not bTickTock;
		
		
				if not data.IsAlive then
					agentEntry.AgentLocationActionsPanel:SetHide(true);
					agentEntry.AgentActivityPanel:SetHide(true);
					agentEntry.AgentKIAPanel:SetHide(false);
					agentEntry.ReoLabel:SetText("Reorganisation: " .. data.Rekonwalescencja)
					agentEntry.Base:SetColorVal(0,0,0,0);

				else
				
					local gturn = gData.ITGTurn;
					local lastActionTurn = data.TurnsActiveAction or data.LastTimeRelocated;
					local hideCivIcon = data.Mission == -1;
					agentEntry.AgentLocationActionsPanel:SetHide(false);
					agentEntry.AgentActivityPanel:SetHide(hideCivIcon);
					agentEntry.AgentKIAPanel:SetHide(true);
			
					agentEntry.AgentLocation:SetText("Not Assigned");
					agentEntry.RelocateButton:SetText("Assign")
			
					agentEntry.Base:SetColorVal(unpack(bTickTock and g_PianoKeys[0] or g_PianoKeys[1]));
					bTickTock = not bTickTock;
					
					agentEntry.CivIcon:SetHide(hideCivIcon);
					agentEntry.CivIconBG:SetHide(hideCivIcon);
					agentEntry.CivIconShadow:SetHide(hideCivIcon);
					agentEntry.CivIconHighlight:SetHide(hideCivIcon);
					if not hideCivIcon then
						if data.Mission == 0 then
							agentEntry.AgentActivityIcon:SetTextureOffset(Vector2(45,45))
							agentEntry.AgentActivity:SetText("Moving: " .. Locale.Lookup(EspionageActions[data.TempData[1]].Description))
							agentEntry.AgentActivity:SetToolTipString("");
							local fPlayer = data.TempData[3];
							if data.TempData[2] then
								local iCity = Map.GetPlotByIndex(data.TempData[2]):GetPlotCity();
								agentEntry.AgentLocation:SetText(iCity:GetName())
								fPlayer = iCity:GetOwner();
							elseif data.TempData[3] then
								agentEntry.AgentLocation:SetText(Players[data.TempData[3]]:GetCivilizationShortDescription())
							else
								agentEntry.AgentLocation:SetText(Players[gPlayer]:GetCivilizationShortDescription())
							end
							CivIconHookup(fPlayer or gPlayer, 32, agentEntry.CivIcon, agentEntry.CivIconBG, agentEntry.CivIconShadow, false, true, agentEntry.CivIconHighlight);
						else
							agentEntry.AgentActivity:SetText(Locale.Lookup(EspionageActions[data.Mission].Description))
							agentEntry.AgentActivity:SetToolTipString(GetEspActionHelp(data.Mission));
							if not EspionageActions[data.Mission].RequiresTarget then
								CivIconHookup(gPlayer, 32, agentEntry.CivIcon, agentEntry.CivIconBG, agentEntry.CivIconShadow, false, true, agentEntry.CivIconHighlight);
								agentEntry.AgentLocation:SetText(Players[gPlayer]:GetCivilizationShortDescription())
							else
								local fPlayer = data.TargetPlayer;
								if EspionageActions[data.Mission].TurnsNeeded then
									agentEntry.AgentActivityProgress:SetHide(false);
									agentEntry.AgentActivityProgressFrame:SetHide(false);
									agentEntry.AgentActivityProgressBack:SetHide(false);
									agentEntry.AgentActivityProgress:SetTexture(g_ProgressBarStates[2].ProgressBarTexture);
									local progresspct = data.TurnsActiveAction / EspionageActions[data.Mission].TurnsNeeded;
									local nextprogresspct = math.min(1, progresspct + 1 / EspionageActions[data.Mission].TurnsNeeded);
									agentEntry.AgentActivityProgress:SetPercents( progresspct, nextprogresspct );
									agentEntry.AgentActivityProgress:SetToolTipString("The meter shows an amount of preparation into next action (" .. progresspct * 100 .. "%).")
								end
								if data.TargetCity then
									local iCity = Map.GetPlotByIndex(data.TargetCity):GetPlotCity();
									fPlayer = iCity:GetOwner();
									agentEntry.AgentLocation:SetText(iCity:GetName())
								elseif data.TargetGreatPerson then
									LuaEvents.ITGSG_RequestGreatPersonName(data.TargetGreatPerson);
									local gpstring = MapModData.ITGSG;
									MapModData.ITGSG = nil;
									agentEntry.AgentLocation:SetText(gpstring)
								else
									agentEntry.AgentLocation:SetText(Players[data.TargetPlayer]:GetCivilizationShortDescription())
								end
								CivIconHookup(fPlayer, 32, agentEntry.CivIcon, agentEntry.CivIconBG, agentEntry.CivIconShadow, false, true, agentEntry.CivIconHighlight);
							end
							if EspionageActions[data.Mission].TOffsetX then
								agentEntry.AgentActivityIcon:SetTextureOffset(Vector2(EspionageActions[data.Mission].TOffsetX, EspionageActions[data.Mission].TOffsetY))
							end
							if EspionageActions[data.Mission].Diplomacy then
								agentEntry.AgentIcon:SetHide(true);
								agentEntry.DiplomatIcon:SetHide(false);
							end
						end
					end


					local isforce = true;
					agentEntry.ViewCityButton:SetHide(isforce)
					agentEntry.HelpBox:SetHide(not isforce)
					agentEntry.RelocateButton:SetVoid1(it)
					local notAbleToRelocate = ((gturn- data.LastTimeRelocated) < 4) or (gData.Goverment == 1);
					agentEntry.RelocateButton:SetDisabled( notAbleToRelocate)
					if notAbleToRelocate then
						if gData.Goverment == 1 then
							agentEntry.RelocateButton:SetToolTipString("You cannot control your agents during Anarchy.")
						else
							agentEntry.RelocateButton:SetToolTipString("You can relocate an agent once per 4 turns. Try again after " .. (4 - gturn + data.LastTimeRelocated) .. " turn(s).")
						end
					else
						agentEntry.RelocateButton:SetToolTipString("You can relocate this agent right now.")
					end
					agentEntry.RelocateButton:RegisterCallback( Mouse.eLClick, OnReassign );
			
		--[[			agentEntry.AgentActivity:SetToolTipString(strActivityTT);
					agentEntry.AgentActivityProgressFrame:SetHide(bHideActivityInfo);
					agentEntry.AgentActivityProgressFrame:SetToolTipString(strActivityTT);
					agentEntry.AgentActivityProgress:SetHide(bHideActivityInfo);
					agentEntry.AgentActivityProgress:SetToolTipString(strActivityTT);
					agentEntry.AgentActivityProgressBack:SetHide(bHideActivityInfo);
					agentEntry.AgentActivityProgressBack:SetToolTipString(strActivityTT);
					agentEntry.AgentActivityIcon:SetHide(bHideActivityInfo);
					agentEntry.AgentActivityIcon:SetToolTipString(strActivityTT);]]
				
				
					agentEntry.AgentActivitiesStack:CalculateSize();
					agentEntry.AgentActivitiesStack:ReprocessAnchoring();
					--testing
					gEntry = agentEntry;
				end
			end
	end
	
	
	Controls.AgentStack:CalculateSize();
	Controls.AgentStack:ReprocessAnchoring();
	Controls.AgentScrollPanel:CalculateInternalSize();
end


function Refresh()
	local bEspionageStarted = true;
	Controls.LabelEspionageNotStartedYet:SetHide(bEspionageStarted);
	Controls.TabPanel:SetHide(not bEspionageStarted);
	Controls.AgentsListBox:SetHide(not bEspionageStarted);		 
	Controls.AgentMovePanel:SetHide(true);
	Controls.CitiesListBox:SetHide(not bEspionageStarted);
	Controls.CloseButton:SetHide(false);
	
	
	
	LuaEvents.ITGSG_RequestGT("Spy");
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	local gPlayer = Game.GetActivePlayer();
	Controls.BuyAgent:SetHide((not MapModData.GetDBCulturePlayerInfo(gPlayer, "CanBuyAgents")) or (MapModData.DBScenario.GoldBudgetOnly) or (gData.Players[gPlayer].CondemnEspionage > 0))
	local goldCost = gData.Players[gPlayer].EspionagePurchaseCostOfAgent;
	Controls.BuyAgent:SetDisabled(Players[gPlayer]:GetGold() < goldCost)
	Controls.BuyAgentLabel:SetText("Recruit Agent for " .. goldCost .. " [ICON_GOLD]")
	if gData.Players[gPlayer].MasterOfWhispers then
		Controls.MastersOfWhispersLabel:SetText("Master of Whispers: [COLOR_POSITIVE_TEXT]ON[ENDCOLOR]");
		Controls.MastersOfWhispers:SetToolTipString("Master of Whispers is managing your agents if you appoint one. Right now Master of Whispers is active.");
	else
		Controls.MastersOfWhispersLabel:SetText("Master of Whispers: [COLOR_NEGATIVE_TEXT]OFF[ENDCOLOR]");
		Controls.MastersOfWhispers:SetToolTipString("Master of Whispers manages your agents if you appoint one. Right now Master of Whispers is inactive.");
	end

	if(bEspionageStarted) then
		g_SelectedAgentID = nil;
		g_SelectedCity = nil;
		g_CitiesAvailableToRelocate = nil;

		RefreshAgents();
		RefreshIntrigue();
	end
end
Events.SerialEventEspionageScreenDirty.Add(Refresh);

function OnYes( )
	Controls.ChooseConfirm:SetHide(true);

	if(g_ConfirmAction ~= nil) then
		OnConfirmed();
	end	
end
Controls.Yes:RegisterCallback( Mouse.eLClick, OnYes );

function OnNo( )
	Controls.ChooseConfirm:SetHide(true);
end
Controls.No:RegisterCallback( Mouse.eLClick, OnNo );

function OnPlayerMOW( )
	if Players[Game.GetActivePlayer()]:IsTurnActive() then
		LuaEvents.ITGSG_PlayerChangeMOWSettings();
		Refresh()
	end
end
Controls.MastersOfWhispers:RegisterCallback( Mouse.eLClick, OnPlayerMOW );

function OnBuyAgent( )
	if Players[Game.GetActivePlayer()]:IsTurnActive() then
		LuaEvents.ITGSG_PlayerBoughtNewAgent();
		Refresh()
	end
end
Controls.BuyAgent:RegisterCallback( Mouse.eLClick, OnBuyAgent );

----------------------------------------------------------------
function RefreshIntrigue()
--	LuaEvents.ITGSG_RequestGT();
--	local gData = MapModData.ITGSG;
--	MapModData.ITGSG = nil;
--	if not gData then
--		return
--	end
	local iPlayer = Game.GetActivePlayer()
	LuaEvents.ITGSG_RequestEspionageLog(iPlayer)
	local espData = MapModData.ITGSG;
	MapModData.ITGSG = nil;

	g_IntrigueManager:ResetInstances();
	
	Controls.IntrigueMessageHeaders:SetHide(#espData == 0);
	Controls.IntrigueMessageScrollPanel:SetHide(#espData == 0);
	Controls.NoIntrigueAvailable:SetHide(#espData ~= 0);

	local bTickTock = true;
	for i,v in pairs(espData) do
			
		local instance = g_IntrigueManager:GetInstance();
		instance.Base:SetColorVal(unpack(bTickTock and g_PianoKeys[0] or g_PianoKeys[1]));
		instance.TurnNum:SetText(v[1]);
		instance.Message:SetString(v[2]);
		if v[3] then
			instance.Chance:SetText(v[3]/10 .. "%");
		else
			instance.Chance:SetText("");
		end
			
		local width, height = instance.Message:GetSizeVal();
		local baseWidth, baseHeight = instance.Base:GetSizeVal();
		instance.Base:SetSizeVal(baseWidth, height + 22);
		if v[4] then
			instance.CivilizationName:LocalizeAndSetText(v[4]);
			if not v[5] then
				instance.CivIcon:SetHide(true);				
				instance.SpyIcon:SetHide(false);
			else
				local civType = Players[v[5]]:GetCivilizationType();
				IconHookup(GameInfo.Civilizations[civType].PortraitIndex, 32, GameInfo.Civilizations[civType].IconAtlas, instance.CivIcon);
				instance.CivIcon:SetHide(false);
				instance.SpyIcon:SetHide(true);
			end
		else
			instance.CivilizationName:LocalizeAndSetText("");
			instance.CivIcon:SetHide(true);
			instance.SpyIcon:SetHide(true);
		end	
			
		bTickTock = not bTickTock;
	end

	Controls.IntrigueMessageStack:CalculateSize();
	Controls.IntrigueMessageStack:ReprocessAnchoring();
	Controls.IntrigueMessageScrollPanel:CalculateInternalSize();
end

----------------------------------------------------------------
-- 'Active' (local human) player has changed
----------------------------------------------------------------
Events.GameplaySetActivePlayer.Add(OnClose);

-----------------------------------------------------------------
-- Add Espionage Overview to Dropdown


-- Just in case :)
TabSelect("Overview");