--------------------------------------------------------------
-- LeagueOverview.lua
-- Author: Anton Strenger
-- DateCreated: 10/19/2012
--
-- Shows information about the members, resolutions, and votes
-- of Leagues in the game
--------------------------------------------------------------

include( "IconSupport" );
include( "InstanceManager" );
include( "CommonBehaviors" );

local gReasonTimeModifiers = {
"for using Delay option.",
"from using Veto.",
"from ally using Veto.",
"for accepting allied resolution.",
"for supporting allied resolution.",
"from forced allied resolution.",
"from overforcing resolution.",
"from forcing resolution.",
"from passing controversial resolution.",
"for withdrawing controversial resolution.", -- 10
"from denying allied resolution.",
}

-----------------------------
-- Globals --
-----------------------------
-- Used for Piano Keys
local ltBlue = {19/255,32/255,46/255,120/255};
local dkBlue = {12/255,22/255,30/255,120/255};

local bCountCS = false;
local m_PopupInfo = nil;
local m_ActiveEffectsManager = InstanceManager:new("ActiveEffectSummaryEntry", "EffectStack", Controls.ActiveEffectsStack );
local m_ViewAllResolutionsActiveResolutionManager = InstanceManager:new( "ViewAllResolutionsEntry", "Button", Controls.ActiveResolutionStack2 );
local m_ViewAllResolutionsInactiveResolutionManager = InstanceManager:new( "ViewAllResolutionsEntry", "Button", Controls.InactiveResolutionStack2 );
local m_ViewAllResolutionsDisabledResolutionManager = InstanceManager:new( "ViewAllResolutionsEntry", "Button", Controls.DisabledResolutionStack2 );
local m_ProposalManager = InstanceManager:new( "ProposedResolutionInstance", "ResolutionButton", Controls.ProposalStack2 );
local bPVARP1 = true;
local bPVARP2 = false;

local m_MembersManager = InstanceManager:new( "MemberInstance", "MemberButton", Controls.MemberStack);
local membersOffets = {
{446,166},{364,150},{524,150},{394,245},{494,245},{316,77},{572,77},{301,214},{587,214},{244,150},{644,150},{221,238},{667,238},{160,170},{728,170},{192,82},{696,82},{250,32},{638,32},{143,12},{745,12},
{99,102},{789,102},
{38,34},{850,34},
{115,245},{773,245},
{54,177},{834,177},
{3,109},{885,109},
{3,245},{885,245},
}


local m_ActiveResolutionManager = InstanceManager:new( "ResolutionInstance", "ResolutionButton", Controls.ActiveResolutionStack );
local m_InactiveResolutionManager = InstanceManager:new( "ResolutionInstance", "ResolutionButton", Controls.InactiveResolutionStack );
local m_DisabledResolutionManager = InstanceManager:new( "ResolutionInstance", "ResolutionButton", Controls.DisabledResolutionStack );
local m_ResolutionChoiceManager = InstanceManager:new( "ResolutionChoiceInstance", "ResolutionChoiceButton", Controls.ResolutionChoiceStack );

local m_iCurrentLeague = -1;
local m_tResolutionChoices = {};

local bOkayToProcess = Players[Game.GetActivePlayer()]:IsTurnActive();
function OnActivePlayerTurnEnd()
	bOkayToProcess = false;
end
Events.ActivePlayerTurnEnd.Add( OnActivePlayerTurnEnd );

function OnActivePlayerTurnStart()
	bOkayToProcess = true;
end
Events.ActivePlayerTurnStart.Add( OnActivePlayerTurnStart );


-- Constants
local kDebug = false;
local kStateVoteEnact = 100;
local kStateVoteRepeal = 101;
local kStateProposeEnact = 102;
local kStateProposeRepeal = 103;
local kChoiceNone = -1;
local kChoiceNo = 0;
local kChoiceYes = 1;
local kRepealDecision = "RESOLUTION_DECISION_REPEAL";
local kNoDecision = "RESOLUTION_DECISION_NONE";
local kNoPlayer = -1;
local currentInviteLeague;
local iRequiredAnswers = 1;
local answerTab = {}

local hTabOfLeagues = {}
local cUIDl;
local CurrentResID;

g_strEnactPrefix = Locale.Lookup("TXT_KEY_LEAGUE_OVERVIEW_PREFIX_ENACT");
g_strRepealPrefix = Locale.Lookup("TXT_KEY_LEAGUE_OVERVIEW_PREFIX_REPEAL");
g_strProposeButtonCaption = Locale.Lookup("TXT_KEY_LEAGUE_OVERVIEW_PROPOSE_BUTTON");
g_strProposeButtonToolTip = Locale.Lookup("TXT_KEY_LEAGUE_OVERVIEW_PROPOSE_BUTTON_TT");
g_strWorldCongressImage = "WorldCongress.dds";
g_strUnitedNationsImage = "WonderConceptUN_small.dds";


local _, screenSizeY = UIManager:GetScreenSizeVal();
if(screenSizeY <= 768) then
	local offsetX, offsetY = Controls.MainFrame:GetOffsetVal();
	Controls.MainFrame:SetOffsetVal(offsetX, offsetY + 30);
end



-----------------------------
-- General UI Handlers --
-----------------------------
LuaEvents.ITG_CallHumanToChooseStrengthResolution.Add(function()
	LuaEvents.ITGSG_RequestGT()
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;

	local gPlayer= Game.GetActivePlayer();
	for legid, sth in pairs(gData.Leagues) do
		if sth.Members[gPlayer] then
			for hString, pData in pairs(sth.Proposals) do
				if pData.ProposedBy == gPlayer then
					if pData.ProposalStrength == -1 then
						cUIDl = legid;
						break;
					end
				end
			end
		end
	end
	UpdateFull();
	UIManager:QueuePopup( ContextPtr, PopupPriority.VoteResults );
end)

LuaEvents.ITG_CallHumanToAnswerResolution.Add(function()
	LuaEvents.ITGSG_RequestGT()
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;

	local gPlayer= Game.GetActivePlayer();
	for legid, sth in pairs(gData.Leagues) do
		if sth.Members[gPlayer] then
			for hString, pData in pairs(sth.Proposals) do
				if pData.ProposedBy ~= gPlayer then
					if not pData.PlayersAnswers[gPlayer] then
						cUIDl = legid;
						break;
					end
				end
			end
		end
	end
	UpdateFull();
	UIManager:QueuePopup( ContextPtr, PopupPriority.VoteResults );
end)

LuaEvents.ITG_CallHumanToAnswerLeagueInvitation.Add(function()
	CallCheckForInvitation(true)
end)

function CallCheckForInvitation(bForce)
	HandleInviteData(bForce)
	UpdateFull();
	Controls.InviteButton:SetHide(true)
	UIManager:QueuePopup( ContextPtr, PopupPriority.VoteResults );
end

function HandleInviteData(bForce)
	LuaEvents.ITGSG_RequestGT()
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	currentInviteLeague = nil;
	local gPlayer= Game.GetActivePlayer();
	for lig, sth in pairs(gData.Players[gPlayer].InvitationsLeague) do
		if gData.Leagues[lig] then
			currentInviteLeague = lig;
			Controls.ConfirmText:SetText("You have been invited to " .. gData.Leagues[currentInviteLeague].Name .. ". Would you like to join it?")
			Controls.ChooseConfirm:SetHide(false)
			local hStr = "You will join following civilizations:"
			for member, h in pairs(gData.Leagues[lig].Members) do
				if not Players[member]:IsMinorCiv() then
					hStr = hStr .. "[NEWLINE][ICON_BULLET]" .. Players[member]:GetCivilizationShortDescription()
				end
			end
			hStr = hStr .. "[NEWLINE]Creating larger leagues may disband a smaller ones which consist of the same members."
			Controls.ConfirmYes:SetToolTipString(hStr)
			return
		end
	end
	if not currentInviteLeague then
		if bForce then
			LuaEvents.ITG_RequestAllDirtyPlayer();
			print("123/216/412 League invitation no longer valid")
		end
		return;
	end
end

function OnCheckInvitation()
	HandleInviteData(false)
	Controls.InviteButton:SetHide(true)
end
Controls.InviteButton:RegisterCallback( Mouse.eLClick, OnCheckInvitation );

function OnConfirmYes( )
	Controls.ChooseConfirm:SetHide(true);
	LuaEvents.ITGHumanAnswersInvitationToLeague(currentInviteLeague, 1)
	cUIDl = currentInviteLeague;
	UpdateFull();
end
Controls.ConfirmYes:RegisterCallback( Mouse.eLClick, OnConfirmYes );

function OnConfirmNo( )
	Controls.ChooseConfirm:SetHide(true);
	LuaEvents.ITGHumanAnswersInvitationToLeague(currentInviteLeague, 2)
	UpdateFull();
end
Controls.ConfirmNo:RegisterCallback( Mouse.eLClick, OnConfirmNo );

function OnConfirmHide( )
	Controls.ChooseConfirm:SetHide(true);
	--> display button of pending invitation
	Controls.InviteButton:SetHide(false)
end
Controls.ConfirmHide:RegisterCallback( Mouse.eLClick, OnConfirmHide );

LuaEvents.ITG_CallHumanToChooseInitialResolution.Add(function()
	
	--here get nice league and force it
	LuaEvents.ITGSG_RequestGT()
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;

	local gPlayer= Game.GetActivePlayer();
	for legid, sth in pairs(gData.Leagues) do
		if sth.Members[gPlayer] then 
			if ((not sth.Members[gPlayer].InitProposal) and sth.HasInitialProposals) or sth.Members[gPlayer].ForcedResolution == gData.ITGTurn then
				cUIDl = legid;
				break;
			end
		end
	end

	UpdateFull();
	PopulateViewAllResolutionsPopup(true, false)
	Controls.ViewAllResolutionsPopup:SetHide(false);
	Controls.BGBlock:SetHide(true);
	UIManager:QueuePopup( ContextPtr, PopupPriority.VoteResults );

end)

function OnPopup( popupInfo )

	if( popupInfo.Type ~= ButtonPopupTypes.BUTTONPOPUP_LEAGUE_OVERVIEW ) then
		return;
	end
	if not Players[Game.GetActivePlayer()]:IsTurnActive() then
		return;
	end
	m_PopupInfo = popupInfo;
	m_iCurrentLeague = popupInfo.Data1;


	
	UpdateFull();
	CallCheckForInvitation();
	UIManager:QueuePopup( ContextPtr, PopupPriority.VoteResults );
end
Events.SerialEventGameMessagePopup.Add( OnPopup );

function InputHandler( uiMsg, wParam, lParam )
    if uiMsg == KeyEvents.KeyDown then
        if wParam == Keys.VK_ESCAPE or wParam == Keys.VK_RETURN then
        
			if(not Controls.ChooseProposalPopup:IsHidden()) then
				CloseProposeResolutionPopup();
			elseif(not Controls.ViewAllResolutionsPopup:IsHidden()) then
				CloseViewAllResolutionsPopup();
			elseif(not Controls.ResolutionChoicePopup:IsHidden()) then
				CloseResolutionChoicePopup();
			elseif(not Controls.ChangeNamePopup:IsHidden()) then
				CloseRenameLeaguePopup();
			else
				OnClose();
			end
            return true;
        end
    end 
end
ContextPtr:SetInputHandler( InputHandler );

function ShowHideHandler( bIsHide, bInitState )

    if( not bInitState ) then
        if( not bIsHide ) then
        	UI.incTurnTimerSemaphore();
        	Events.SerialEventGameMessagePopupShown(m_PopupInfo);
        else
            UI.decTurnTimerSemaphore();
        end
    end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );

function OnClose()
    Events.SerialEventGameMessagePopupProcessed.CallImmediate(ButtonPopupTypes.BUTTONPOPUP_LEAGUE_OVERVIEW, 0);
    UIManager:DequeuePopup( ContextPtr );
end
Controls.CloseButton:RegisterCallback( Mouse.eLClick, OnClose );

-- 'Active' (local human) player has changed
function HelloThereReset()
	cUIDl = nil;
	m_ActiveResolutionManager:ResetInstances();
	m_InactiveResolutionManager:ResetInstances();
	m_DisabledResolutionManager:ResetInstances();
	m_ResolutionChoiceManager:ResetInstances();
	m_ProposalManager:ResetInstances();
	m_ActiveEffectsManager:ResetInstances();
	ViewLeagueMembers({})
	OnClose()
end
Events.GameplaySetActivePlayer.Add(HelloThereReset);

function SetCurrentLeague(v1)
	if hTabOfLeagues[v1] then
		cUIDl = hTabOfLeagues[v1];
		Update()
	end
end
function ProposeResolution(player, league, proposal, option)
	if bOkayToProcess then
		LuaEvents.ITGSG_ProposeResolutionStrength(player, league, proposal, option)
		Update()
	end
end
function AnswerResolution(player, league, proposal, option)
	if bOkayToProcess then
		LuaEvents.ITGSG_AnswerResolution(player, league, proposal, option)
		Update()
	end
end
function GetNameOrUnknown(iPlayer)
	if Teams[Game.GetActiveTeam()]:IsHasMet(Players[iPlayer]:GetTeam()) then
		return Players[iPlayer]:GetCivilizationShortDescription()
	else
		return "Unknown member"
	end
end

function PrefixLeagueList(lData, iPlayer)
	if not lData.DoFindepedent then
		local iActivePlayerVotes = lData.Members[iPlayer].Votes;
		local votesBh = iActivePlayerVotes;
		for member, sth in pairs(lData.Members) do
			if member ~= iPlayer then
				votesBh = math.min(iActivePlayerVotes - sth.Votes, votesBh);
			end
		end
 		if votesBh >= 5 then
			return "[ICON_ITG_VOTES] "
		end
	end
	return ""
end

function SuffixLeagueList(lData)
	if not lData.DoFindepedent then
		local nmp = 0;
		for a, b in pairs(lData.Members) do
			if not Players[a]:IsMinorCiv() then
				nmp = nmp + 1;
			end
		end
		if nmp > 2 then
			return " (" .. nmp .. ")";
		end
	end
	return "";
end

function Update( bForceViewModel )
	local iActivePlayer = Game.GetActivePlayer();
	local strBackgroundImage = g_strWorldCongressImage;

	LuaEvents.ITGSG_RequestGT()
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return;
	end
	if not gData.Players[iActivePlayer] then
		return;
	end
	hTabOfLeagues = {}
	local automaticPurchasePullDown = Controls.LeaguePullDown;
	function AddToPullDown(text, tooltip, v1, v2)
	-- set up the random slot
		local controlTable = {};
		automaticPurchasePullDown:BuildEntry( "InstanceOne", controlTable );
		
		controlTable.Button:SetVoids(v1, v2);
		controlTable.Button:SetText(text);
		controlTable.Button:SetToolTipString(tooltip);
	end
	automaticPurchasePullDown:ClearEntries();
	for gUID, dt in pairs(gData.Leagues) do
		if dt.Members[iActivePlayer] then
			table.insert(hTabOfLeagues, gUID);
			AddToPullDown(PrefixLeagueList(dt, iActivePlayer) .. dt.Name .. SuffixLeagueList(dt), "", #hTabOfLeagues, 1)
		end
	end
	automaticPurchasePullDown:CalculateInternals();
	automaticPurchasePullDown:RegisterSelectionCallback(function(v1, v2)
		SetCurrentLeague(v1);
	end);
	Controls.LeaguePullDown:SetDisabled(#hTabOfLeagues == 0);

	automaticPurchasePullDown:GetButton():SetText("")

	Controls.RenameButton:SetHide(true);
	Controls.CommitButton:SetHide(true);
	Controls.ResetButton:SetHide(true);
	Controls.ViewAllResolutionsButton:SetHide(true);
	Controls.TurnsUntilVoteFrame:SetDisabled(true);
	if not cUIDl then
		if #hTabOfLeagues > 0 then
			cUIDl = hTabOfLeagues[1];
		end
	end
	local bAnyInvitation = false;
	for lig, sth in pairs(gData.Players[iActivePlayer].InvitationsLeague) do
		if gData.Leagues[lig] then
			bAnyInvitation = true;
			break;
		end
	end

	Controls.InviteButton:SetHide(not bAnyInvitation)

	if cUIDl and gData.Leagues[cUIDl] and gData.Leagues[cUIDl].Members[iActivePlayer] then
		
		if bAnyInvitation then
			Controls.ResetButton:SetDisabled(true)
			Controls.ViewAllResolutionsButton:SetDisabled(true);
			Controls.ResetButton:SetToolTipString("You cannot propose any resolutions or take action while you have pending league invitation.")
			Controls.ViewAllResolutionsButton:SetToolTipString("You cannot propose any resolutions or take action while you have pending league invitation.")
		else
			LuaEvents.ITGSG_RequestResolutionsForProposal(1, iActivePlayer, cUIDl)
			local bb1 = MapModData.ITGSG;
			MapModData.ITGSG = nil;
			LuaEvents.ITGSG_RequestResolutionsForProposal(2, iActivePlayer, cUIDl)
			local bb2 = MapModData.ITGSG;
			MapModData.ITGSG = nil;
			Controls.ResetButton:SetDisabled(#bb1 == 0)
			Controls.ViewAllResolutionsButton:SetDisabled(#bb2 == 0);
			if (#bb1 == 0) and (#bb2 == 0) then
			-- disabled :> display CD info
				local cdforActions = (gData.Leagues[cUIDl].Members[iActivePlayer].LastAction or -1) - gData.ITGTurn;
				if cdforActions > 0 then
					Controls.ResetButton:SetToolTipString("You need to wait " .. cdforActions .. " turn(s) before your next proposal or an action.")
					Controls.ViewAllResolutionsButton:SetToolTipString("You need to wait " .. cdforActions .. " turn(s) before your next proposal or an action.")
				else
					Controls.ResetButton:SetToolTipString("")
					Controls.ViewAllResolutionsButton:SetToolTipString("")
				end
			end
		end
		

		Controls.ViewAllResolutionsButton:SetHide(false);
		Controls.WorldCongressContent:SetHide(false);
		Controls.WorldCongressNotStartedContent:SetHide(true);
		Controls.TurnsUntilVote:SetText(gData.Leagues[cUIDl].TurnFoundString)
		automaticPurchasePullDown:GetButton():SetText(gData.Leagues[cUIDl].Name)
		local hMembers = {}
		for iPlayer, dt in pairs(gData.Leagues[cUIDl].Members) do
			if Players[iPlayer]:IsAlive() then
				local civID = Players[iPlayer]:GetLeaderType();
				local memberTooltip = "[ICON_BULLET][COLOR_POSITIVE_TEXT]" .. gData.Players[iPlayer].LeagueBaseVotes .. "[ENDCOLOR] from membership."
				if gData.Leagues[cUIDl].Host == iPlayer then
					memberTooltip = memberTooltip .. "[NEWLINE][ICON_BULLET][COLOR_POSITIVE_TEXT]" .. 3 .. "[ENDCOLOR] from Host Position.";
				end
				for name, nv in pairs(dt.VotesFromResolution) do
					if nv > 0 then
						memberTooltip = memberTooltip .. "[NEWLINE][ICON_BULLET][COLOR_POSITIVE_TEXT]" .. nv .. "[ENDCOLOR] from " .. name .. ".";
					elseif nv < 0 then
						memberTooltip = memberTooltip .. "[NEWLINE][ICON_BULLET][COLOR_NEGATIVE_TEXT]" .. nv .. "[ENDCOLOR] from " .. name .. ".";
					end
				end
				for name, nv in pairs(dt.VotesTimeModifiers) do
					if nv.Active then
						if nv.Actual > 0 then
							memberTooltip = memberTooltip .. "[NEWLINE][ICON_BULLET][COLOR_POSITIVE_TEXT]" .. nv.Actual .. "[ENDCOLOR] " .. gReasonTimeModifiers[nv.Reason] .. " (" .. (1 + nv.EndTurn - gData.ITGTurn) .. "T)";
						elseif nv.Actual < 0 then
							memberTooltip = memberTooltip .. "[NEWLINE][ICON_BULLET][COLOR_NEGATIVE_TEXT]" .. nv.Actual .. "[ENDCOLOR] " .. gReasonTimeModifiers[nv.Reason] .. " (" .. (1 + nv.EndTurn - gData.ITGTurn) .. "T)";
						end
					end
				end
				for name, nv in pairs(dt.VotesFromScripts) do
					if nv > 0 then
						memberTooltip = memberTooltip .. "[NEWLINE][ICON_BULLET][COLOR_POSITIVE_TEXT]" .. nv .. "[ENDCOLOR] from " .. name .. ".";
					elseif nv < 0 then
						memberTooltip = memberTooltip .. "[NEWLINE][ICON_BULLET][COLOR_NEGATIVE_TEXT]" .. nv .. "[ENDCOLOR] from " .. name .. ".";
					end
				end
				local hTTT = {};
				if iPlayer == gData.Leagues[cUIDl].AuthorityVote then
					table.insert(hTTT, {1, "Authority"})
				end
				if iPlayer == gData.Leagues[cUIDl].MHGVote then
					table.insert(hTTT, {2, "Moral High Ground"})
				end
				if dt.MosqueWonder then
					table.insert(hTTT, {2, "Al-Azhar Mosque"})
				end
				table.insert(hTTT, {gData.Players[iPlayer].NumPrestigeVotes, "Prestige"})
				table.insert(hTTT, {gData.Players[iPlayer].VotesFromBreakingLeague or 0, "being Untrustworthy"})
				table.insert(hTTT, {gData.Players[iPlayer].VotesBribing or 0, "bribed Delegates"})
				table.insert(hTTT, {gData.Leagues[cUIDl].Members[iPlayer].VotesFromIdeologies or 0, "Ideologies"})
				table.insert(hTTT, {gData.Leagues[cUIDl].Members[iPlayer].VotesFromBudget or 0, "supporting Doctrine"})
				table.insert(hTTT, {gData.Leagues[cUIDl].Members[iPlayer].VotesGP, "Great People"})
				table.insert(hTTT, {gData.Players[iPlayer].VotesFromSocials, "Social Developments"})
				table.insert(hTTT, {gData.Players[iPlayer].VotesFromInno, "Innovations"})
				for a, b in pairs(hTTT) do
					if b[1] then
						if b[1] > 0 then
							memberTooltip = memberTooltip .. "[NEWLINE][ICON_BULLET][COLOR_POSITIVE_TEXT]" .. b[1] .. "[ENDCOLOR] from " .. b[2] .. ".";
						elseif b[1] < 0 then
							memberTooltip = memberTooltip .. "[NEWLINE][ICON_BULLET][COLOR_NEGATIVE_TEXT]" .. b[1] .. "[ENDCOLOR] from " .. b[2] .. ".";
						end
					end
				end
				local hVotes = dt.Votes
				local minorVc = 0;
				if Players[iPlayer]:IsMinorCiv() then
					if gData.Leagues[cUIDl].Members[Players[iPlayer]:GetAlly()] then
						if bCountCS then
							hVotes = 0;
						end
						memberTooltip = memberTooltip .. "[NEWLINE][ICON_BULLET]Votes are controlled by " .. GetNameOrUnknown(Players[iPlayer]:GetAlly()) .. ".";
					end
				else
					local votesfromalliances = gData.Leagues[cUIDl].Members[iPlayer].VotesFromAlliedCS or 0;
					if votesfromalliances ~= 0 then
						memberTooltip = memberTooltip .. "[NEWLINE][ICON_BULLET][COLOR_POSITIVE_TEXT]" .. votesfromalliances .. "[ENDCOLOR] from City-States.";
					end
					for mPlayer, bb in pairs(gData.Leagues[cUIDl].Members) do
						if Players[mPlayer]:IsMinorCiv() and Players[mPlayer]:IsAlive() then
							if Players[mPlayer]:GetAlly() == iPlayer then
								minorVc = minorVc + bb.Votes;
							end
						end
					end
					if minorVc > 0 then
						memberTooltip = memberTooltip .. "[NEWLINE][ICON_BULLET]Extra: [COLOR_POSITIVE_TEXT]" .. minorVc .. "[ENDCOLOR] [ICON_ITG_VOTES] Votes from controlling delegates from City-States.";
					end
				end
				-- counting minors?
				if bCountCS then
					hVotes = hVotes + minorVc;
				end
				table.insert(hMembers, {VotesTooltip = memberTooltip, NumVotes = hVotes, Leader = (gData.Leagues[cUIDl].Host == iPlayer),  ShowScroll = not Players[iPlayer]:IsMinorCiv(), ShowSpy = false, ToolTip = "", PlayerId = iPlayer, PortraitIndex = GameInfo.Leaders[civID].PortraitIndex, IconAtlas = GameInfo.Leaders[civID].IconAtlas})
			end
		end
		local nMemb = #hMembers;
		ViewLeagueMembers(hMembers)
		Controls.ResetButton:SetHide(false);

		-- current proposals on
		m_ProposalManager:ResetInstances();
		local voteEntries = {};
		for i,v in pairs(gData.Leagues[cUIDl].Proposals) do
			local instance = m_ProposalManager:GetInstance();
			instance.ResolutionButton:ClearCallback(Mouse.eLClick);
			local prefix = "";
			if(not gData.Leagues[cUIDl].Resolutions[i]) then
				prefix = g_strEnactPrefix;
			else
				prefix = g_strRepealPrefix;
			end
			instance.ResolutionName:SetText(prefix .. v.Name);
			LuaEvents.ITGSG_RequestHelpTextResolution(v.ResolutionID, v.ExtraInfo)
			instance.ResolutionButton:SetToolTipString(MapModData.ITGSG);
			MapModData.ITGSG = nil;
			instance.ResolutionButton:SetDisabled(true);
			if(v.MainIcon ~= nil) then
				IconHookup(v.MainIcon.PortraitIndex, 64, v.MainIcon.IconAtlas, instance.Icon);
			else
				IconHookup(0, 64, "INTL_PROJECT_ATLAS_EXP2", instance.Icon);
			end
		
			instance.SubIconFrame:SetHide(true);
			if (v.ProposedBy ~= nil and v.ProposedBy ~= -1) then
				CivIconHookup(v.ProposedBy, 32, instance.SubIcon, instance.SubIconBG, instance.SubIconShadow, true, true);
				instance.SubIconFrame:SetHide(false);
			end
		
			if(v.AltIcon ~= nil) then
				IconHookup(v.AltIcon.PortraitIndex, 64, v.AltIcon.IconAtlas, instance.AltIcon);
				instance.AltIconFrame:SetHide(false);
			else
				instance.AltIconFrame:SetHide(true);
			end
		
			instance.VoteStatus:SetHide(true);
			instance.VoteBox:SetHide(true);
		
			local width, _ = instance.ResolutionButton:GetSizeVal();
			local _, height = instance.ResolutionName:GetSizeVal();
			local newheight = math.max(64, height + 30);
			instance.ResolutionButton:SetSizeVal(width, newheight);

			
			instance.ResButton1:SetHide(true)
			instance.ResButton2:SetHide(true)
			instance.ResButton3:SetHide(true)
			instance.ResButton4:SetHide(true)
			instance.ResButton5:SetHide(true)
			instance.ResButtonLabel1:SetDisabled("")
			instance.ResButtonLabel2:SetDisabled("")
			instance.ResButtonLabel3:SetDisabled("")
			instance.ResButtonLabel4:SetDisabled("")
			instance.ResButtonLabel5:SetDisabled("")

			instance.AnswerLabel:SetText("")
			if v.Delayed then
				instance.AnswerLabel:SetText("Delayed (" .. v.Delayed - gData.ITGTurn + 1 .. ")")
			elseif v.ProposedBy == iActivePlayer then
				if v.ProposalStrength == -1 then
					instance.ResButton3:SetHide(false)
					instance.ResButton4:SetHide(false)
					instance.ResButton5:SetHide((gData.Leagues[cUIDl].Resolutions[14] ~= nil))
					instance.ResButtonLabel3:SetText("Suggest")
					instance.ResButton3:SetToolTipString("The weakest proposal.[NEWLINE][ICON_BULLET]If other member uses Veto or denies proposal, resolution fails.[NEWLINE][ICON_BULLET]Resolution passes if all other members will accept or support it.")
					instance.ResButtonLabel4:SetText("Insist")
					instance.ResButton4:SetToolTipString("You are backing the proposal with your votes.[NEWLINE][ICON_BULLET]If other member uses Veto, resolution fails and you get +1 Vote for 10 turns.[NEWLINE]If other member denies proposal, the decision is made in debate waged on number of votes. If resolution passes your Votes are reduced by 4 for 12 turns, otherwise you receive +1 Vote for 12 turns.[NEWLINE][ICON_BULLET]Resolution passes if all other members will accept or support it.")
					instance.ResButtonLabel5:SetText("Force")
					instance.ResButton5:SetToolTipString("You are using all your diplomatic power to enforce the resolution.[NEWLINE][ICON_BULLET]If other member uses Veto, resolution fails and you get +2 Vote for 10 turns.[NEWLINE][ICON_BULLET]If other member denies proposal, you receive -8 Votes for 16 turns and resolution still passes.[NEWLINE][ICON_BULLET]Otherwise resolution passes and you receive -1 Vote for 10 turns.")
					instance.ResButton3:SetDisabled(false)
					instance.ResButton4:SetDisabled(false)
					instance.ResButton5:SetDisabled((gData.Leagues[cUIDl].Members[iActivePlayer].Votes < 8))
					
					instance.ResButton3:RegisterCallback( Mouse.eLClick, function() ProposeResolution(iActivePlayer, cUIDl, i, 1); end );
					instance.ResButton4:RegisterCallback( Mouse.eLClick, function() ProposeResolution(iActivePlayer, cUIDl, i, 2); end );
					instance.ResButton5:RegisterCallback( Mouse.eLClick, function() ProposeResolution(iActivePlayer, cUIDl, i, 3); end );
				else
					if v.ProposalStrength == 1 then
						instance.AnswerLabel:SetText("Suggesting")
					elseif v.ProposalStrength == 2 then
						instance.AnswerLabel:SetText("Insisting")
					elseif v.ProposalStrength == 3 then
						instance.AnswerLabel:SetText("Forcing")
					else
						instance.AnswerLabel:SetText("Proposing")
					end
				end
			elseif not v.PlayersAnswers[iActivePlayer] then
				if v.Answers then
					for si, sA in pairs(v.Answers) do
						instance["ResButton" .. si]:SetHide(false)
						instance["ResButtonLabel" .. si]:SetText(sA[2])
						instance["ResButton" .. si]:SetToolTipString(sA[3] or "")
						instance["ResButton" .. si]:SetDisabled(false)
						instance["ResButton" .. si]:RegisterCallback( Mouse.eLClick, function() AnswerResolution(iActivePlayer, cUIDl, i, si); end );
					end
				else
					instance.ResButton1:SetHide(false)
					instance.ResButton2:SetHide(nMemb < 3)
					instance.ResButton3:SetHide((gData.Leagues[cUIDl].Resolutions[13] ~= nil))
					instance.ResButton4:SetHide((gData.Leagues[cUIDl].Resolutions[14] ~= nil))
					instance.ResButton5:SetHide(false)
					if gData.Leagues[cUIDl].NoVotesFromVoting then
						instance.ResButtonLabel1:SetText("Neutral")
						instance.ResButton1:SetToolTipString("Your votes will not be counted.");
						instance.ResButton2:SetToolTipString("Your votes will support the resolution in the debate.");
						instance.ResButton5:SetToolTipString("Your votes will oppose the proposal.")
					else
						instance.ResButtonLabel1:SetText("Accept")
						instance.ResButton1:SetToolTipString("Accepting resolution may result with:[NEWLINE][ICON_BULLET]+1 Vote for 10 turns if proposal is being forced or insisted on.");
						instance.ResButton2:SetToolTipString("Supporting resolution may result with:[NEWLINE][ICON_BULLET]+2 Votes for 10 turns if proposal is being forced or insisted on.[NEWLINE][ICON_BULLET]Your votes will support the resolution in the debate.");
						instance.ResButton5:SetToolTipString("Denying resolution may resulty with.[NEWLINE][ICON_BULLET]+2 Votes for 10 turns if proposal is being forced.[NEWLINE][ICON_BULLET]Resolution fails if being mere suggested.[NEWLINE][ICON_BULLET]Leads to debate if proposal is being insisted on (result is waged on number of votes). If resolution fails you receive -1 Votes for 12 turns.")
					end
					instance.ResButtonLabel2:SetText("Support")
					instance.ResButtonLabel3:SetText("Delay")
					instance.ResButton3:SetToolTipString("Delays the resolution for at least 4 turns (depends on number of members using this option).[NEWLINE][ICON_BULLET]" ..  - (v.DelayCost or 1000) .. " Vote(s) for 10 turns.")
					instance.ResButtonLabel4:SetText("Veto")
					instance.ResButton4:SetToolTipString("Resolution fails without debate.[NEWLINE][ICON_BULLET]-5 Votes for 10 turns.")
					instance.ResButtonLabel5:SetText("Deny")
					instance.ResButton3:SetDisabled(gData.Leagues[cUIDl].Members[iActivePlayer].Votes < (v.DelayCost or 1000))
					instance.ResButton5:SetDisabled(false)
					instance.ResButton4:SetDisabled(gData.Leagues[cUIDl].Members[iActivePlayer].Votes < 5)
					instance.ResButton1:RegisterCallback( Mouse.eLClick, function() AnswerResolution(iActivePlayer, cUIDl, i, 1); end );
					instance.ResButton2:RegisterCallback( Mouse.eLClick, function() AnswerResolution(iActivePlayer, cUIDl, i, 2); end );
					instance.ResButton3:RegisterCallback( Mouse.eLClick, function() AnswerResolution(iActivePlayer, cUIDl, i, 3); end );
					instance.ResButton4:RegisterCallback( Mouse.eLClick, function() AnswerResolution(iActivePlayer, cUIDl, i, 4); end );
					instance.ResButton5:RegisterCallback( Mouse.eLClick, function() AnswerResolution(iActivePlayer, cUIDl, i, 5); end );
				end
			else
				--display your answer
				if v.Special and (not v.NormalAnswer) then
					instance.AnswerLabel:SetText("Waiting")
				else
					if v.PlayersAnswers[iActivePlayer] == 1 then
						instance.AnswerLabel:SetText("Accepting")
					elseif v.PlayersAnswers[iActivePlayer] == 2 then
						instance.AnswerLabel:SetText("Supporting")
					elseif v.PlayersAnswers[iActivePlayer] == 3 then
						instance.AnswerLabel:SetText("Delaying")
					elseif v.PlayersAnswers[iActivePlayer] == 4 then
						instance.AnswerLabel:SetText("Veto!")
					elseif v.PlayersAnswers[iActivePlayer] == 5 then
						instance.AnswerLabel:SetText("Denying")
					end
				end
			end

			vginstance = instance
		end
	
		Controls.ProposalStack2:CalculateSize();
		Controls.ProposalStack2:ReprocessAnchoring();
		Controls.ProposalScrollPanel2:CalculateInternalSize();

		-- current proposals off

		m_ActiveEffectsManager:ResetInstances();
		for id, v in pairs(gData.Leagues[cUIDl].FakeResolutions or {}) do
			local instance = m_ActiveEffectsManager:GetInstance();
			instance.EffectLabel:SetText(v.Name);
			LuaEvents.ITGSG_RequestHelpTextResolution(v.ResID, v.ExtraInfo)
			instance.EffectLabel:SetToolTipString(MapModData.ITGSG);
			MapModData.ITGSG = nil;
			instance.EffectStack:CalculateSize();
			instance.EffectStack:ReprocessAnchoring();
		end
		for id, v in pairs(gData.Leagues[cUIDl].Resolutions) do
			local instance = m_ActiveEffectsManager:GetInstance();
			if v.Project then
				if v.Active then
					instance.EffectLabel:SetText(v.Name .. " (Ongoing)")
				elseif v.ProjectPrepare then
					instance.EffectLabel:SetText(v.Name .. " (Preparations)")
				elseif v.Finished then
					instance.EffectLabel:SetText(v.Name .. " (Completed)")
				else
					print("Errorewr325 52332432")
				end
			elseif v.EndTurn then
				instance.EffectLabel:SetText(v.Name .. " (" .. (1 + v.EndTurn - gData.ITGTurn) .. ")");
			else
				instance.EffectLabel:SetText(v.Name);
			end
			
			LuaEvents.ITGSG_RequestHelpTextResolution(v.ResID, v.ExtraInfo)
			instance.EffectLabel:SetToolTipString(MapModData.ITGSG);
			MapModData.ITGSG = nil;
			instance.EffectStack:CalculateSize();
			instance.EffectStack:ReprocessAnchoring();
		end	
		Controls.ActiveEffectsStack:CalculateSize();
		Controls.ActiveEffectsStack:ReprocessAnchoring();
		Controls.ActiveEffectsScrollPanel:CalculateInternalSize();

	else
		Controls.WorldCongressContent:SetHide(true);
		Controls.WorldCongressNotStartedContent:SetHide(false);
		if #hTabOfLeagues == 0 then
			Controls.LeagueNotFoundedString:LocalizeAndSetText("You are not a member of any union.");
		end
	end
	
	Controls.LeagueImage:SetTexture(strBackgroundImage);
end

function ViewLeagueMembers(members)
	Controls.LeaderButton:SetHide(true);
	m_MembersManager:ResetInstances();
	
	local hindex = 1;
	for jmc = 1, 2 do
		local bMinor = (jmc == 2);
		for i, member in ipairs(members) do
			if Players[member.PlayerId]:IsMinorCiv() == bMinor then
				local buttonControl;
				local iconSize;
				local iconControl; 
				local votesControl;
				local scrollControl;
				local spyControl;
		
				local subIconSize;
				local subIcon;
				local subIconBG;
				local subIconShadow;
				local subIconHighlight;
				-- Assume the first member is the leader.
				if(member.Leader) then
					buttonControl = Controls.LeaderButton;
					iconControl = Controls.LeaderIcon;
					iconSize = 128;
					votesControl = Controls.LeaderVotes;
					scrollControl = Controls.LeaderScroll;
					spyControl = Controls.LeaderSpy;
			
					subIconSize = 45;
					subIcon = Controls.LeaderSubIcon;
					subIconBG = Controls.LeaderSubIconBG;
					subIconShadow = Controls.LeaderSubIconShadow;
					subIconHighlight = Controls.LeaderSubIconHighlight;
			
				elseif membersOffets[hindex] then
					local index = tostring(hindex);
					local controlTable = m_MembersManager:GetInstance();
					controlTable.MemberButton:SetOffsetVal(membersOffets[hindex][1], membersOffets[hindex][2])
					buttonControl = controlTable["MemberButton"];
					iconControl = controlTable["MemberIcon"];
					iconSize = 64;
					votesControl = controlTable["MemberVotes"];
					scrollControl = controlTable["MemberScroll"];
					spyControl = controlTable["MemberSpy"];
			
					subIconSize = 32;
					subIcon = controlTable["MemberSubIcon"];
					subIconBG = controlTable["MemberSubIconBG"];
					subIconShadow = controlTable["MemberSubIconShadow"];
					subIconHighlight = controlTable["MemberSubIconHighlight"];
					hindex = hindex + 1;
				end
				if buttonControl then
					buttonControl:SetHide(false);
		
					if Teams[Game.GetActiveTeam()]:IsHasMet(Players[member.PlayerId]:GetTeam()) then
						CivIconHookup(member.PlayerId, subIconSize, subIcon, subIconBG, subIconShadow, true, true, subIconHighlight);
						if Players[member.PlayerId]:IsMinorCiv() then
							local thisCivType = GameInfo.Civilizations[PreGame.GetCivilization( member.PlayerId )]
							IconHookup(thisCivType.PortraitIndex, iconSize, thisCivType.IconAtlas, iconControl);
						else
							IconHookup(member.PortraitIndex, iconSize, member.IconAtlas, iconControl);
						end
						subIcon:SetToolTipString(Players[member.PlayerId]:GetCivilizationShortDescription())
					else
						subIcon:SetToolTipString("Unknown member")
						CivIconHookup(-1, subIconSize, subIcon, subIconBG, subIconShadow, true, true, subIconHighlight);
						IconHookup(22, iconSize, "LEADER_ATLAS", iconControl);
					end
					votesControl:SetText(member.NumVotes);


					votesControl:SetToolTipString(member.VotesTooltip)
					buttonControl:SetToolTipString(member.ToolTip);
					scrollControl:SetHide(not member.ShowScroll);
					spyControl:SetHide(not member.ShowSpy);
		
					buttonControl:RegisterCallback( Mouse.eLClick, function() OnLeaderSelected(member.PlayerId); end );
				end
			end
		end
	end
end

function IsInAnswerTab(t)
	for i, answer in pairs(answerTab) do
		if answer == t then
			return i;
		end
	end
	return nil;
end

local lastType;
function ExtraChoiceChosen(extrachoice)
	if iRequiredAnswers > 1 then
		local tt = IsInAnswerTab(extrachoice);
		if tt then
			table.remove(answerTab,tt)
			DoPopulateDataForExtraChoice(cUIDl, CurrentResID, lastType)
		else
			table.insert(answerTab, extrachoice)
			if #answerTab >= iRequiredAnswers then
				LuaEvents.HumanPlayerCallResolutionMultiple(cUIDl, CurrentResID, answerTab)
				Update()
				CurrentResID = nil;
				CloseResolutionChoicePopup();
			else
				DoPopulateDataForExtraChoice(cUIDl, CurrentResID, lastType)
			end
		end
	else
		LuaEvents.HumanPlayerCallResolution(cUIDl, CurrentResID, extrachoice)
		Update()
		CurrentResID = nil;
		CloseResolutionChoicePopup();
	end
end

function DoPopulateDataForExtraChoice(leagueID, idR, type)
	LuaEvents.ITGGetExtraInfoForProposal(leagueID, idR, type)
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	iRequiredAnswers = MapModData.ITGSG2 or 1;
	MapModData.ITGSG2 = nil;
	if iRequiredAnswers > 1 then
		Controls.LabelNumLeagueChoices:SetText("Select " .. iRequiredAnswers .. " options.")
	else
		Controls.LabelNumLeagueChoices:SetText("")
	end
	m_ResolutionChoiceManager:ResetInstances();
	
	local tick = true;
	lastType = type;
	
	for _, v in ipairs(gData) do
		local instance = m_ResolutionChoiceManager:GetInstance();
		
		if(v.MainIcon ~= nil) then
			IconHookup(v.MainIcon.PortraitIndex, 64, v.MainIcon.IconAtlas, instance.Icon);
			instance.IconFrame:SetHide(false);
		else
			instance.IconFrame:SetHide(true);
		end
		
		instance.SubIconFrame:SetHide(true);
--[[	if(v.UsePlayerIdForSubIcon) then
			CivIconHookup(v.PlayerId, 32, instance.SubIcon, instance.SubIconBG, instance.SubIconShadow, true, true);
			instance.SubIconHighlight:SetHide(false);
			instance.SubIconFrame:SetHide(false);
		else
			if(v.SubIcon ~= nil) then
				instance.SubIconBG:SetHide(true);
				instance.SubIconShadow:SetHide(true);
				instance.SubIconHighlight:SetHide(true);
				
				IconHookup(v.SubIcon.PortraitIndex, 32, v.SubIcon.IconAtlas, instance.SubIcon);
				instance.SubIconFrame:SetHide(false);
			end
		end]]
		if IsInAnswerTab(v.ExtraInfo) then
			instance.ResolutionChoiceName:SetText("[COLOR_POSITIVE_TEXT]" .. v.Name .. "[ENDCOLOR]");
		elseif type == 2 then
			instance.ResolutionChoiceName:SetText("REPEAL: " .. v.Name);
		else
			instance.ResolutionChoiceName:SetText(v.Name);
		end
		instance.ResolutionChoiceButton:SetToolTipString(v.Tooltip or "");
		instance.ResolutionChoiceButton:SetDisabled(false);
		local color = tick and ltBlue or dkBlue;
		tick = not tick;
		instance.Box:SetColorVal(unpack(color));
		instance.ResolutionChoiceButton:RegisterCallback(Mouse.eLClick, function() 
			ExtraChoiceChosen(v.ExtraInfo)
		end);
	end
	
	Controls.ResolutionChoiceStack:CalculateSize();
	Controls.ResolutionChoiceStack:ReprocessAnchoring();
	Controls.ChoiceScrollPanel:CalculateInternalSize();
end

function DoCallResolution(leagueID, idR, bExtra, type)
	if bOkayToProcess then
		CloseViewAllResolutionsPopup()
		if bExtra then
			CurrentResID = idR;
			answerTab = {}
			DoPopulateDataForExtraChoice(leagueID, idR, type)
			Controls.ResolutionChoicePopup:SetHide(false);
			Controls.BGBlock:SetHide(true);
		else
			LuaEvents.HumanPlayerCallResolution(leagueID, idR)
			Update()
		end
	end
end

function PopulateViewAllResolutionsPopup(bInActive, bActive)
	bPVARP1 = bInActive;
	bPVARP2 = bActive;
	m_ViewAllResolutionsActiveResolutionManager:ResetInstances();
	m_ViewAllResolutionsInactiveResolutionManager:ResetInstances();
	m_ViewAllResolutionsDisabledResolutionManager:ResetInstances();
	
	local tick = true;
	if bActive then
		Controls.ActiveResolutionsHeaderLabel2:SetText("[ICON_MINUS] Enacted Resolutions")
		Controls.ActiveResolutionStack2:SetHide(false)
	else
		Controls.ActiveResolutionsHeaderLabel2:SetText("[ICON_PLUS] Enacted Resolutions")
		Controls.ActiveResolutionStack2:SetHide(true)
	end
	LuaEvents.ITGSG_RequestResolutionsForProposal(2, Game.GetActivePlayer(), cUIDl)
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	for _, v in ipairs(gData) do
		local instance = m_ViewAllResolutionsActiveResolutionManager:GetInstance();
		instance.ResolutionName:SetText("REPEAL: " .. v.Text);
		if v.DisabledText then
			instance.Button:SetToolTipString(v.ToolTip .. "[NEWLINE][NEWLINE][COLOR_NEGATIVE_TEXT]" .. v.DisabledText .. "[ENDCOLOR]")
		else
			instance.Button:SetToolTipString(v.ToolTip);
		end
		instance.Button:SetDisabled(v.Disabled);
		if v.Disabled then
			instance.Button:SetAlpha(0.6);
		else
			instance.Button:SetAlpha(1.0);
		end
		local color = tick and ltBlue or dkBlue;
		tick = not tick;
		instance.ExtraChoices:SetHide(true);
		instance.Button:RegisterCallback(Mouse.eLClick, function() DoCallResolution(cUIDl, v.ID, v.ExtraChoice, 2) end)
		instance.Box:SetColorVal(unpack(color));
	end
	if bInActive then
		Controls.PossibleResolutionsHeaderLabel2:SetText("[ICON_MINUS] Possible Resolutions")
		Controls.InactiveResolutionStack2:SetHide(false)
	else
		Controls.PossibleResolutionsHeaderLabel2:SetText("[ICON_PLUS] Possible Resolutions")
		Controls.InactiveResolutionStack2:SetHide(true)
	end
	LuaEvents.ITGSG_RequestResolutionsForProposal(1, Game.GetActivePlayer(), cUIDl)
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	for _, v in ipairs(gData) do
		local instance = m_ViewAllResolutionsInactiveResolutionManager:GetInstance();
		instance.ResolutionName:SetText(v.Text);
		if v.DisabledText then
			instance.Button:SetToolTipString(v.ToolTip .. "[NEWLINE][NEWLINE][COLOR_NEGATIVE_TEXT]" .. v.DisabledText .. "[ENDCOLOR]")
		else
			instance.Button:SetToolTipString(v.ToolTip);
		end
		instance.Button:SetDisabled(v.Disabled);
		if v.Disabled then
			instance.Button:SetAlpha(0.6);
		else
			instance.Button:SetAlpha(1.0);
		end	
		if v.ExtraChoice then
			instance.ExtraChoices:SetHide(false);
		else
			instance.ExtraChoices:SetHide(true);
		end
		local color = tick and ltBlue or dkBlue;
		tick = not tick;
		instance.Button:RegisterCallback(Mouse.eLClick, function() DoCallResolution(cUIDl, v.ID, v.ExtraChoice, 1) end)
		instance.Box:SetColorVal(unpack(color));
	end
	
	Controls.ActiveResolutionStack2:CalculateSize();
	Controls.ActiveResolutionStack2:ReprocessAnchoring();
	Controls.InactiveResolutionStack2:CalculateSize();
	Controls.InactiveResolutionStack2:ReprocessAnchoring();
	Controls.DisabledResolutionStack2:CalculateSize();
	Controls.DisabledResolutionStack2:ReprocessAnchoring();
	Controls.ViewAllResolutionsScrollPanel:CalculateInternalSize();
end

function ViewCurrentProposals(model)

end

Controls.ResetButton:RegisterCallback(Mouse.eLClick, function()
	PopulateViewAllResolutionsPopup(true, false)
	Controls.ViewAllResolutionsPopup:SetHide(false);
	Controls.BGBlock:SetHide(true);
end);	
Controls.ViewAllResolutionsButton:RegisterCallback(Mouse.eLClick, function()
	PopulateViewAllResolutionsPopup(false, true)
	Controls.ViewAllResolutionsPopup:SetHide(false);
	Controls.BGBlock:SetHide(true);
end);

------------- END OF ITG;
-------------

function UpdateFull()
	Update(true);
end

function UpdatePartial()
	Update(false);
end
--Events.SerialEventLeagueScreenDirty.Add(UpdatePartial);

function GetProposals(league, activePlayerId)
	
	local enactProposals = league:GetEnactProposals();
	local repealProposals = league:GetRepealProposals();
	local enactProposalsOnHold = league:GetEnactProposalsOnHold();
	local repealProposalsOnHold = league:GetRepealProposalsOnHold();
	
	local GetChoicesForResolution = function(resolutionType) 
		
		local resolutionChoices = nil;
		
		local decisionType = GameInfo.Resolutions[resolutionType].VoterDecision;
		if(decisionType ~= "RESOLUTION_DECISION_YES_OR_NO") then
			resolutionChoices = {};
			local decisionId = GameInfo.ResolutionDecisions[decisionType].ID;
			local choices = league:GetChoicesForDecision(decisionId, activePlayerId);
		
			for _, choiceId in ipairs(choices) do
				local choice = {
					Id = choiceId,
					Text = league:GetTextForChoice(decisionId, choiceId),
				}
				
				GatherAdditionalChoiceInfoBasedOnDecisionType(decisionType, choice);
								
				table.insert(resolutionChoices, choice);
			end
		end
		
		return resolutionChoices;
	end	
	
	local GetLeaderIcon = function(proposalPlayer)
		if(proposalPlayer ~= nil and proposalPlayer ~= -1) then
			local player = Players[proposalPlayer];
			local leader = GameInfo.Leaders[player:GetLeaderType()];
			
			return {
				PortraitIndex = leader.PortraitIndex,
				IconAtlas = leader.IconAtlas,
			};
		end
	end
	
	local proposals = {}
	
	return proposals;
end

-- Populates a choice table with additional data based on the decision type.
-- Ie Icon data for resources, or civs, etc
function GatherAdditionalChoiceInfoBasedOnDecisionType(decisionType, choice)
	if(decisionType == "RESOLUTION_DECISION_MAJOR_CIV_MEMBER" or decisionType == "RESOLUTION_DECISION_OTHER_MAJOR_CIV_MEMBER") then
		local player = Players[choice.Id];
		local leader = GameInfo.Leaders[player:GetLeaderType()];
		local civ = GameInfo.Civilizations[player:GetCivilizationType()];
		
		choice.MainIcon = {
			PortraitIndex = leader.PortraitIndex,
			IconAtlas = leader.IconAtlas,
		};
		
		choice.UsePlayerIdForSubIcon = true;
		choice.PlayerId = choice.Id;
	end
	
	if(decisionType == "RESOLUTION_DECISION_ANY_LUXURY_RESOURCE") then
		
		local resource = GameInfo.Resources[choice.Id];
		choice.MainIcon = {
			PortraitIndex = resource.PortraitIndex,
			IconAtlas = resource.IconAtlas
		};
		
	end
	
	if(decisionType == "RESOLUTION_DECISION_RELIGION") then
		local religion = GameInfo.Religions[choice.Id];
		choice.MainIcon = {
			PortraitIndex = religion.PortraitIndex,
			IconAtlas = religion.IconAtlas
		};
	end	
end

-----------------------------
-- View Functions
-----------------------------
function ViewActiveEffectsSummary(activeEffects)
	m_ActiveEffectsManager:ResetInstances();
	
	for _, v in ipairs(activeEffects) do
		local instance = m_ActiveEffectsManager:GetInstance();
		instance.EffectLabel:SetText(v);
		instance.EffectStack:CalculateSize();
		instance.EffectStack:ReprocessAnchoring();
	end
	
	Controls.ActiveEffectsStack:CalculateSize();
	Controls.ActiveEffectsStack:ReprocessAnchoring();
	Controls.ActiveEffectsScrollPanel:CalculateInternalSize();

end


function PopulateProposeResolutionPopup(activeResolutions, inactiveResolutions, currentProposals, selectFunction)

	m_ActiveResolutionManager:ResetInstances();
	m_InactiveResolutionManager:ResetInstances();
	m_DisabledResolutionManager:ResetInstances();
	
	local tick = true;
	
	activeResolutions = activeResolutions or {};
	for _, v in ipairs(activeResolutions) do
		local instance = m_ActiveResolutionManager:GetInstance();
		v.Direction = "Retract";
		instance.ResolutionName:SetText(v.ProposeText);
		instance.ResolutionButton:SetToolTipString(v.ProposeToolTip);
		instance.ResolutionButton:SetDisabled(v.Disabled);
		
		local color = tick and ltBlue or dkBlue;
		tick = not tick;
			
		instance.Box:SetColorVal(unpack(color));
		
		instance.ResolutionButton:RegisterCallback(Mouse.eLClick, function() 
			CloseProposeResolutionPopup();
			selectFunction(v) 
		end);
	end
	
	inactiveResolutions = inactiveResolutions or {};
	for _, v in ipairs(inactiveResolutions) do
		if (not v.Disabled) then
			local instance = m_InactiveResolutionManager:GetInstance();
			v.Direction = "Enact";
			instance.ResolutionName:SetText(v.ProposeText);
			instance.ResolutionButton:SetToolTipString(v.ProposeToolTip);
			instance.ResolutionButton:SetDisabled(v.Disabled);
			
			local color = tick and ltBlue or dkBlue;
			tick = not tick;
				
			instance.Box:SetColorVal(unpack(color));
			
			instance.ResolutionButton:RegisterCallback(Mouse.eLClick, function() 
				CloseProposeResolutionPopup();
				selectFunction(v);
			end);
		end
	end
	for _, v in ipairs(inactiveResolutions) do
		if (v.Disabled) then
			local instance = m_DisabledResolutionManager:GetInstance();
			v.Direction = "Enact";
			instance.ResolutionName:SetText(v.ProposeText);
			instance.ResolutionButton:SetToolTipString(v.ProposeToolTip);
			instance.ResolutionButton:SetDisabled(v.Disabled);
			
			local color = tick and ltBlue or dkBlue;
			tick = not tick;
				
			instance.Box:SetColorVal(unpack(color));
			
			instance.ResolutionButton:RegisterCallback(Mouse.eLClick, function() 
				CloseProposeResolutionPopup();
				selectFunction(v);
			end);
		end
	end
	
	Controls.ActiveResolutionStack:CalculateSize();
	Controls.ActiveResolutionStack:ReprocessAnchoring();
	Controls.InactiveResolutionStack:CalculateSize();
	Controls.InactiveResolutionStack:ReprocessAnchoring();
	Controls.DisabledResolutionStack:CalculateSize();
	Controls.DisabledResolutionStack:ReprocessAnchoring();
	Controls.ChooseProposalStack:CalculateSize();
	Controls.ChooseProposalStack:ReprocessAnchoring();
	Controls.ChooseProposalScrollPanel:CalculateInternalSize();
		
end

function PopulateResolutionChoicePopup(proposerChoices, selectFunction)
	m_ResolutionChoiceManager:ResetInstances();
	
	local tick = true;
	
	for _, v in ipairs(proposerChoices) do
		local instance = m_ResolutionChoiceManager:GetInstance();
		
		if(v.MainIcon ~= nil) then
			IconHookup(v.MainIcon.PortraitIndex, 64, v.MainIcon.IconAtlas, instance.Icon);
			instance.IconFrame:SetHide(false);
		else
			instance.IconFrame:SetHide(true);
		end
		
		instance.SubIconFrame:SetHide(true);
		if(v.UsePlayerIdForSubIcon) then
			
			CivIconHookup(v.PlayerId, 32, instance.SubIcon, instance.SubIconBG, instance.SubIconShadow, true, true);

			instance.SubIconHighlight:SetHide(false);
			instance.SubIconFrame:SetHide(false);
		else
			if(v.SubIcon ~= nil) then
				instance.SubIconBG:SetHide(true);
				instance.SubIconShadow:SetHide(true);
				instance.SubIconHighlight:SetHide(true);
				
				IconHookup(v.SubIcon.PortraitIndex, 32, v.SubIcon.IconAtlas, instance.SubIcon);
				instance.SubIconFrame:SetHide(false);
			end
		end
		
		local sText = v.Text;
		if (v.Disabled) then
			sText = "[COLOR_GREY]" .. sText .. "[ENDCOLOR]";
		end
		instance.ResolutionChoiceName:SetText(sText);
		instance.ResolutionChoiceButton:SetToolTipString(v.ToolTip);
		instance.ResolutionChoiceButton:SetDisabled(v.Disabled);
		
		instance.ResolutionChoiceStack:CalculateSize();
		instance.ResolutionChoiceStack:ReprocessAnchoring();
		
		local color = tick and ltBlue or dkBlue;
		tick = not tick;
			
		instance.Box:SetColorVal(unpack(color));
		
		instance.ResolutionChoiceButton:RegisterCallback(Mouse.eLClick, function() 
			CloseResolutionChoicePopup();
			selectFunction(v.Id) 
		end);
	end
	
	Controls.ResolutionChoiceStack:CalculateSize();
	Controls.ResolutionChoiceStack:ReprocessAnchoring();
	Controls.ChoiceScrollPanel:CalculateInternalSize();
end

function RenameLeague()

	local pLeague = Game.GetLeague(m_iCurrentLeague);
	local defaultName = pLeague:GetName();

	Controls.NewName:SetText(defaultName);
	Controls.ChangeNameError:SetHide(true);
	Controls.ChangeNamePopup:SetHide(false);
end

function OnChangeNameOK()

	local name = Controls.NewName:GetText();
	if(Locale.IsNilOrEmpty(name)) then
		Controls.ChangeNameError:LocalizeAndSetText("TXT_KEY_LEAGUE_OVERVIEW_ERROR_NAME_EMPTY");
		Controls.ChangeNameError:SetHide(false);
	else
		Network.SendLeagueEditName(m_iCurrentLeague, Game.GetActivePlayer(), name);
		Controls.ChangeNameError:SetHide(true);
		Controls.TitleLabel:SetText(name);
		Controls.ChangeNamePopup:SetHide(true);
	end
end
Controls.ChangeNameOKButton:RegisterCallback(Mouse.eLClick, OnChangeNameOK);

function CloseRenameLeaguePopup()
	Controls.ChangeNamePopup:SetHide(true);
end
Controls.ChangeNameCancelButton:RegisterCallback(Mouse.eLClick, CloseRenameLeaguePopup);

function OnLeaderSelected(ePlayer)
	if (not Players[Game.GetActivePlayer()]:IsTurnActive() or Game.IsProcessingMessages()) then
		return;
	end
	
	if (Game.GetActivePlayer() == ePlayer) then
		return;
	end

	if Players[ePlayer]:IsMinorCiv() then
		return;
	end

    if (Players[ePlayer] ~= nil) then
		if (Players[ePlayer]:IsHuman()) then
			OnClose();
			Events.OpenPlayerDealScreenEvent( ePlayer );
		else
			UI.SetRepeatActionPlayer(ePlayer);
			UI.ChangeStartDiploRepeatCount(1);
    		Players[ePlayer]:DoBeginDiploWithHuman();
		end
	end
end


-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function CloseViewAllResolutionsPopup()
	Controls.ViewAllResolutionsPopup:SetHide(true);
	Controls.BGBlock:SetHide(false);	
end
Controls.ViewAllResolutionsCloseButton:RegisterCallback( Mouse.eLClick, CloseViewAllResolutionsPopup);

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function CloseProposeResolutionPopup()
	Controls.ChooseProposalPopup:SetHide(true);
	Controls.BGBlock:SetHide(false);	
end
Controls.ChooseProposalCloseButton:RegisterCallback( Mouse.eLClick, CloseProposeResolutionPopup);

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function CloseResolutionChoicePopup()
	Controls.ResolutionChoicePopup:SetHide(true);
	Controls.BGBlock:SetHide(false);	
	if CurrentResID then
		CurrentResID = nil;
		PopulateViewAllResolutionsPopup(bPVARP1, bPVARP2)
		Controls.ViewAllResolutionsPopup:SetHide(false);
		Controls.BGBlock:SetHide(true);
	end
end
Controls.CloseChoicePopupButton:RegisterCallback( Mouse.eLClick, CloseResolutionChoicePopup );



------------------------------
-- Vote Behavior Controller --
------------------------------
VoteController = {
	TotalVotes = 0,
	VotesAvailable = 0,
	VoteInstances = {},	
};
	
function VoteController:Initialize(data)
	local controller = self;
	
	self.LeagueId = data.LeagueId;
	self.ActivePlayerId = data.ActivePlayerId;
	self.TotalVotes = data.TotalVotes;
	self.VotesAvailable = data.TotalVotes;
	self.VoteEntries = data.VoteEntries;
	self.CommitButton = data.CommitButton;
	self.ResetButton = data.ResetButton;
	self.VotesRemainingLabel = data.VotesRemainingLabel;
	
		
	self.ResetButton:LocalizeAndSetText("TXT_KEY_LEAGUE_OVERVIEW_RESET_VOTES");
	self.ResetButton:RegisterCallback(Mouse.eLClick, function()
		controller:ResetVotes();
	end);
	self.ResetButton:SetDisabled(true);
	self.ResetButton:SetHide(false);
		
	self.CommitButton:LocalizeAndSetText("TXT_KEY_LEAGUE_OVERVIEW_COMMIT_VOTES");
	self.CommitButton:RegisterCallback(Mouse.eLClick, function() 
		controller:CommitVotes();
	end);
	
	self.CommitButton:SetDisabled(true);		
	self.CommitButton:SetHide(false);

	self.VotesRemainingLabel:SetHide(false);	
	
	for _, entry in ipairs(self.VoteEntries) do
	
		if(entry.VoterChoices == nil) then
			entry.VoteController = VoteYesNoController.new(controller, entry);
		else
			entry.VoteController = VoteMajorCivController.new(controller, entry);	
		end
	end

	self:ResetVotes();
end

function VoteController:ResetVotes()
	self.VotesAvailable = self.TotalVotes;
	self.CommitButton:SetHide(false);
	self.ResetButton:SetHide(false);
		
	for _, entry in ipairs(self.VoteEntries) do
		entry.Votes = 0;
		entry.Choice = nil;
		entry.VoteController:UpdateVoteInstance();
	end
		
	self:UpdateVoteState();
end
	
function VoteController:CommitVotes()

	local controller = self;
	local bConfirmUnused = (controller.VotesAvailable > 0);
	local confirmAction = function()
		for _, entry in ipairs(controller.VoteEntries) do
	
			if(entry.Votes ~= 0) then
				local choice = entry.Choice or kChoiceNone;
				local votes = math.abs(entry.Votes);
				
				if(entry.Direction == "Enact") then		
					Network.SendLeagueVoteEnact(controller.LeagueId, entry.ResolutionId, controller.ActivePlayerId, votes, choice);	
				else
					Network.SendLeagueVoteRepeal(controller.LeagueId, entry.ResolutionId, controller.ActivePlayerId, votes, choice);	
				end
			end
		end
		
		if (controller.VotesAvailable > 0) then
			Network.SendLeagueVoteAbstain(controller.LeagueId, controller.ActivePlayerId, controller.VotesAvailable);
		end
	end

	CommitConfirm(confirmAction, bConfirmUnused);
end
	
function VoteController:UpdateVoteState()
	local votesAvailable = self.VotesAvailable or 0;
	self.VotesRemainingLabel:LocalizeAndSetText("TXT_KEY_LEAGUE_OVERVIEW_VOTES_REMAINING", votesAvailable);		
	
	for _, voteEntry in ipairs(self.VoteEntries) do
		voteEntry.VoteController:UpdateVoteState(votesAvailable);
	end
	
	self.ResetButton:SetDisabled(self.VotesAvailable == self.TotalVotes);
	self.CommitButton:SetDisabled(false);
end

-------------------------------------
-- Vote Decision Yes/No Controller --
-------------------------------------
VoteYesNoController = {};
	
function VoteYesNoController.new(voteController, entry)
	
	local entryController = {
		VoteController = voteController,
		Entry = entry,
		
		UpdateVoteInstance = VoteYesNoController.UpdateVoteInstance,
		UpdateVoteState = VoteYesNoController.UpdateVoteState
	};
	
	local instance = entry.Instance;
	
	instance.VoteUpButton:RegisterCallback(Mouse.eLClick, function()
		local votes = entry.Votes;
		if(votes >= 0) then
			voteController.VotesAvailable = voteController.VotesAvailable - 1;
		else
			voteController.VotesAvailable = voteController.VotesAvailable + 1;
		end	
		entry.Votes = votes + 1;	
		
		entryController:UpdateVoteInstance();
		voteController:UpdateVoteState();
	end);
	
	instance.VoteUpButton:RegisterCallback(Mouse.eRClick, function()
		local votes = entry.Votes;
		local availableVotes = voteController.VotesAvailable;
		if(votes >= 0) then
			entry.Votes = votes + availableVotes;
			voteController.VotesAvailable = 0;
		else
			entry.Votes = 0;
			voteController.VotesAvailable = availableVotes - votes;
		end
		
		entryController:UpdateVoteInstance();
		voteController:UpdateVoteState();
	end);
	
	instance.VoteDownButton:RegisterCallback(Mouse.eLClick, function()
		local votes = entry.Votes;
		if(votes > 0) then
			voteController.VotesAvailable = voteController.VotesAvailable + 1;
		else
			voteController.VotesAvailable = voteController.VotesAvailable - 1;
		end	
		entry.Votes = votes - 1;	
		
		entryController:UpdateVoteInstance();
		voteController:UpdateVoteState();
	end);
	
	instance.VoteDownButton:RegisterCallback(Mouse.eRClick, function()
		local votes = entry.Votes;
		local availableVotes = voteController.VotesAvailable;
		if(votes > 0) then
			entry.Votes = 0
			voteController.VotesAvailable = availableVotes + votes;
		else
			entry.Votes = votes - availableVotes;
			voteController.VotesAvailable = 0;
		end
		
		entryController:UpdateVoteInstance();
		voteController:UpdateVoteState();
	end);
	
	return entryController;
end
		
function VoteYesNoController:UpdateVoteInstance()
	local instance = self.Entry.Instance;
	local votes = self.Entry.Votes;

	if(votes > 0) then
		instance.VoteStatus:LocalizeAndSetText("TXT_KEY_LEAGUE_OVERVIEW_VOTE_YAY");
	elseif(votes == 0) then
		instance.VoteStatus:LocalizeAndSetText("TXT_KEY_LEAGUE_OVERVIEW_VOTE_ABSTAIN");
	else
		instance.VoteStatus:LocalizeAndSetText("TXT_KEY_LEAGUE_OVERVIEW_VOTE_NAY");
	end		
	
	instance.VoteCount:SetText(votes);
end

function VoteYesNoController:UpdateVoteState(votesAvailable)
	local entry = self.Entry;
	local votes = entry.Votes;
	entry.Instance.VoteUpButton:SetDisabled(votes >= 0 and votesAvailable == 0);
	entry.Instance.VoteDownButton:SetDisabled(votes <= 0 and votesAvailable == 0);
	
	if(votes > 0) then
		entry.Choice = kChoiceYes;
	elseif(votes < 0) then
		entry.Choice = kChoiceNo;
	end
end

-------------------------------------
-- Vote Major Civ Controller --
-------------------------------------
VoteMajorCivController = {};
	
function VoteMajorCivController.new(voteController, entry)
	
	local entryController = {
		VoteController = voteController,
		Entry = entry,
		UpdateVoteInstance = VoteMajorCivController.UpdateVoteInstance,
		UpdateVoteState = VoteMajorCivController.UpdateVoteState
	};
	
	local instance = entry.Instance;
	
	instance.VoteUpButton:RegisterCallback(Mouse.eLClick, function()
		
		local voteUp = function()
			local votes = entry.Votes;
			if(votes >= 0) then
				voteController.VotesAvailable = voteController.VotesAvailable - 1;
			end	
			entry.Votes = votes + 1;	
			
			entryController:UpdateVoteInstance();
			voteController:UpdateVoteState();
		end
		
		if(entry.Choice == nil) then
			PopulateResolutionChoicePopup(entry.VoterChoices, function(choiceId)			
				entry.Choice = choiceId;
				voteUp(choiceId);	
			end);
			
			Controls.ResolutionChoicePopup:SetHide(false);
			Controls.BGBlock:SetHide(true);
		else
			voteUp();
		end
	end);
	
	instance.VoteUpButton:RegisterCallback(Mouse.eRClick, function()
		
		local voteUp = function()
			local votes = entry.Votes;
			local availableVotes = voteController.VotesAvailable;
			if(votes >= 0) then
				entry.Votes = votes + availableVotes;
				voteController.VotesAvailable = 0;
			end	
			
			entryController:UpdateVoteInstance();
			voteController:UpdateVoteState();
		end
		
		if(entry.Choice == nil) then
			PopulateResolutionChoicePopup(entry.VoterChoices, function(choiceId)			
				entry.Choice = choiceId;
				voteUp(choiceId);	
			end);
			
			Controls.ResolutionChoicePopup:SetHide(false);
			Controls.BGBlock:SetHide(true);
		else
			voteUp();
		end
	end);
	
	instance.VoteDownButton:RegisterCallback(Mouse.eLClick, function()
		local votes = entry.Votes;
		if(votes > 0) then
			voteController.VotesAvailable = voteController.VotesAvailable + 1;
		end	
		entry.Votes = votes - 1;	
		
		if(entry.Votes == 0) then
			entry.Choice = nil;
		end
		
		entryController:UpdateVoteInstance();
		voteController:UpdateVoteState();
	end);
	
	instance.VoteDownButton:RegisterCallback(Mouse.eRClick, function()
		local votes = entry.Votes;
		local availableVotes = voteController.VotesAvailable;
		if(votes > 0) then
			entry.Votes = 0;
			voteController.VotesAvailable = availableVotes + votes;
		end	
		
		if(entry.Votes == 0) then
			entry.Choice = nil;
		end
		
		entryController:UpdateVoteInstance();
		voteController:UpdateVoteState();
	end);
	
	return entryController;
end
		
function VoteMajorCivController:UpdateVoteInstance()
	local instance = self.Entry.Instance;
	local votes = self.Entry.Votes;

	if(votes == 0) then
		instance.VoteStatus:LocalizeAndSetText("TXT_KEY_LEAGUE_OVERVIEW_VOTE_ABSTAIN");
		instance.VoteStatus:SetHide(false);
		instance.AltIconFrame:SetHide(true);
	else
		instance.VoteStatus:SetHide(true);
		instance.AltIconFrame:SetHide(false);
	end		
	
	local choice = self.Entry.Choice;
	if(choice ~= nil) then
		local mainIcon = nil;
		local choicePlayerId;
		for i,v in ipairs(self.Entry.VoterChoices) do
			if(v.Id == choice) then
				mainIcon = v.MainIcon;
				choicePlayerId = v.PlayerId;
			end
		end
		
		IconHookup(mainIcon.PortraitIndex, 64, mainIcon.IconAtlas, instance.AltIcon);
		CivIconHookup(choicePlayerId, 32, instance.AltSubIcon, instance.AltSubIconBG, instance.AltSubIconShadow, true, true);
	end
	
	instance.VoteCount:SetText(votes);
end

function VoteMajorCivController:UpdateVoteState(votesAvailable)
	local entry = self.Entry;
	local votes = entry.Votes;
	entry.Instance.VoteUpButton:SetDisabled(votes >= 0 and votesAvailable == 0);
	entry.Instance.VoteDownButton:SetDisabled(votes <= 0);
end
	
---------------------------------
--Proposal Beahvior Controller --
---------------------------------						
ProposalController = {
}

function ProposalController:Initialize(data)
	
	self.PendingProposals = {};
	self.ActivePlayerId = data.ActivePlayerId;
	self.LeagueId = data.LeagueId;
	self.ResetButton = data.ResetButton;
	self.CommitButton = data.CommitButton;
	self.ActiveResolutions = data.ActiveResolutions;
	self.InactiveResolutions = data.InactiveResolutions;
	self.ProposalMainIcon = data.ProposalMainIcon;
	self.ProposalSubIcon = data.ProposalSubIcon;
	self.ProposalInstances = {};
		
	for i = 1, data.NumProposals, 1 do
		local pendingProposal = {
			Controller = self,
			Index = i,
		};
		
		-- Refer back to the controller to obtain the instance.
		-- This is because the instances may change if a refresh event occurs.
		self.ProposalInstances[i] = data.InstanceManager:GetInstance();
	
		table.insert(self.PendingProposals, pendingProposal);	
		self:UpdatePendingProposalInstance(pendingProposal);
	end
	
	-- Close any popups that are up
	OnConfirmNo();
	CloseResolutionChoicePopup();
	CloseProposeResolutionPopup();
	
	self.ResetButton:SetHide(false);
	self.CommitButton:SetHide(false);
		
	local controller = self;
	self.ResetButton:LocalizeAndSetText("TXT_KEY_LEAGUE_OVERVIEW_RESET_PROPOSALS");
	Controls.ResetButton:RegisterCallback(Mouse.eLClick, function() controller:ResetProposals() end);
	self.ResetButton:SetDisabled(true);
	
	self.CommitButton:LocalizeAndSetText("TXT_KEY_LEAGUE_OVERVIEW_COMMIT_PROPOSALS");
	Controls.CommitButton:RegisterCallback(Mouse.eLClick, function() controller:CommitProposals() end);
	self.CommitButton:SetDisabled(true);
end

function ProposalController:UpdatePendingProposalInstance(pendingProposal)
	
	-- Refer back to the controller to obtain the instance.
	-- This is because the instances may change if a refresh event occurs.
	local instance = pendingProposal.Controller.ProposalInstances[pendingProposal.Index];
	
	local controller = self;
	instance.ResolutionButton:ClearCallback(Mouse.eLClick);
	instance.ResolutionButton:SetDisabled(false);
	instance.ResolutionButton:RegisterCallback(Mouse.eLClick, function()
		PopulateProposeResolutionPopup(self.ActiveResolutions, self.InactiveResolutions, self.PendingProposals, function(proposal)
			
			local SelectProposal = function(choiceId)
				
				
				local league = Game.GetLeague(controller.LeagueId);
				local text;
				local toolTip;
				
				local prefix = "";
				if(proposal.Direction == "Enact") then
					prefix = g_strEnactPrefix;
					text = league:GetResolutionName(proposal.Type, -1, choiceId, false);
					toolTip = league:GetResolutionDetails(proposal.Type, controller.ActivePlayerId, -1, choiceId); 				
				else
					prefix = g_strRepealPrefix;
					text = league:GetResolutionName(proposal.Type, -1, proposal.ProposerDecision, false);
					toolTip = league:GetResolutionDetails(proposal.Type, controller.ActivePlayerId, -1, proposal.ProposerDecision); 	
				end
						
				
				pendingProposal.SelectedProposal = {
					Id = proposal.Id,
					Type = proposal.Type,
					ChoiceId = choiceId,
					Text = prefix .. text,
					ToolTip = toolTip,
					Direction = proposal.Direction,
				};
				
				local canCommit = true;
			
				for i,v in ipairs(controller.PendingProposals) do
					if(v.SelectedProposal == nil) then
						canCommit = false;
						break;
					end
				end
				
				controller.CommitButton:SetDisabled(not canCommit);
				controller.ResetButton:SetDisabled(false);
			
				controller:UpdatePendingProposalInstance(pendingProposal);
			end
			
			if(proposal.ProposerChoices ~= nil and #proposal.ProposerChoices > 0) then
				PopulateResolutionChoicePopup(proposal.ProposerChoices, function(choiceId)
					SelectProposal(choiceId);
				end);
				
				Controls.ResolutionChoicePopup:SetHide(false);
				Controls.BGBlock:SetHide(true);
			else
				SelectProposal(kChoiceNone)
			end		
		end);
	
		Controls.ChooseProposalPopup:SetHide(false);
		Controls.BGBlock:SetHide(true);
	end);	
			
	if(pendingProposal.SelectedProposal ~= nil) then
		instance.ResolutionName:SetText(pendingProposal.SelectedProposal.Text);
		instance.ResolutionButton:SetToolTipString(pendingProposal.SelectedProposal.ToolTip);	
	else
		instance.ResolutionName:SetText(g_strProposeButtonCaption);
		instance.ResolutionButton:SetToolTipString(g_strProposeButtonToolTip);
	end
	
	
	instance.SubIconFrame:SetHide(false);
	if(self.ProposalMainIcon ~= nil) then
		IconHookup(self.ProposalMainIcon.PortraitIndex, 64, self.ProposalMainIcon.IconAtlas, instance.Icon);
	else
		IconHookup(0, 64, "INTL_PROJECT_ATLAS_EXP2", instance.Icon);
	end
	
	CivIconHookup(self.ActivePlayerId, 32, instance.SubIcon, instance.SubIconBG, instance.SubIconShadow, true, true);
		
	instance.AltIconFrame:SetHide(true);
	instance.VoteBox:SetHide(true);
end

function ProposalController:ResetProposals()
	for i, proposal in ipairs(self.PendingProposals) do
		proposal.SelectedProposal = nil;
		self:UpdatePendingProposalInstance(proposal);
	end
	
	self.ResetButton:SetDisabled(true);
	self.CommitButton:SetDisabled(true);
end

function ProposalController:CommitProposals()

	local controller = self;
	local confirmAction = function()
		for i, pendingProposal in ipairs(controller.PendingProposals) do
			local proposal = pendingProposal.SelectedProposal;
			if(proposal.Direction == "Enact") then
				local choice = proposal.ChoiceId or kChoiceNone;		
				Network.SendLeagueProposeEnact(controller.LeagueId, proposal.Type, controller.ActivePlayerId, choice);
			else
				Network.SendLeagueProposeRepeal(controller.LeagueId, proposal.Id, controller.ActivePlayerId);
			end
		end	
	end
	
	CommitConfirm(confirmAction);
end

-------------------------------
-- Collapse/Expand Behaviors --
-------------------------------
function OnCollapseExpand()
	Controls.ActiveResolutionStack:CalculateSize();
	Controls.ActiveResolutionStack:ReprocessAnchoring();
	Controls.InactiveResolutionStack:CalculateSize();
	Controls.InactiveResolutionStack:ReprocessAnchoring();
	Controls.DisabledResolutionStack:CalculateSize();
	Controls.DisabledResolutionStack:ReprocessAnchoring();
	Controls.ChooseProposalStack:CalculateSize();
	Controls.ChooseProposalStack:ReprocessAnchoring();
	Controls.ChooseProposalScrollPanel:CalculateInternalSize();
end

function OnCollapseExpand2()
	Controls.ActiveResolutionStack2:CalculateSize();
	Controls.ActiveResolutionStack2:ReprocessAnchoring();
	Controls.InactiveResolutionStack2:CalculateSize();
	Controls.InactiveResolutionStack2:ReprocessAnchoring();
	Controls.DisabledResolutionStack2:CalculateSize();
	Controls.DisabledResolutionStack2:ReprocessAnchoring();
	Controls.ViewAllResolutionsScrollPanel:CalculateInternalSize();
end

local activeResolutionsText = Locale.Lookup("TXT_KEY_LEAGUE_OVERVIEW_ACTIVE_RESOLUTIONS");
RegisterCollapseBehavior{	
	Header = Controls.ActiveResolutionsHeader, 
	HeaderLabel = Controls.ActiveResolutionsHeaderLabel, 
	HeaderExpandedLabel = "[ICON_MINUS] " .. activeResolutionsText,
	HeaderCollapsedLabel = "[ICON_PLUS] " .. activeResolutionsText,
	Panel = Controls.ActiveResolutionStack,
	Collapsed = false,
	OnCollapse = OnCollapseExpand,
	OnExpand = OnCollapseExpand,
};

RegisterCollapseBehavior{	
	Header = Controls.ActiveResolutionsHeader2, 
	HeaderLabel = Controls.ActiveResolutionsHeaderLabel2, 
	HeaderExpandedLabel = "[ICON_MINUS] " .. activeResolutionsText,
	HeaderCollapsedLabel = "[ICON_PLUS] " .. activeResolutionsText,
	Panel = Controls.ActiveResolutionStack2,
	Collapsed = false,
	OnCollapse = OnCollapseExpand2,
	OnExpand = OnCollapseExpand2,
};
							
local possibleResolutionsText = Locale.Lookup("TXT_KEY_LEAGUE_OVERVIEW_INACTIVE_RESOLUTIONS");
RegisterCollapseBehavior{
	Header = Controls.PossibleResolutionsHeader,
	HeaderLabel = Controls.PossibleResolutionsHeaderLabel,
	HeaderExpandedLabel = "[ICON_MINUS] " .. possibleResolutionsText,
	HeaderCollapsedLabel = "[ICON_PLUS] " .. possibleResolutionsText,
	Panel = Controls.InactiveResolutionStack,
	Collapsed = false,
	OnCollapse = OnCollapseExpand,
	OnExpand = OnCollapseExpand,
};

RegisterCollapseBehavior{
	Header = Controls.PossibleResolutionsHeader2,
	HeaderLabel = Controls.PossibleResolutionsHeaderLabel2,
	HeaderExpandedLabel = "[ICON_MINUS] " .. possibleResolutionsText,
	HeaderCollapsedLabel = "[ICON_PLUS] " .. possibleResolutionsText,
	Panel = Controls.InactiveResolutionStack2,
	Collapsed = false,
	OnCollapse = OnCollapseExpand2,
	OnExpand = OnCollapseExpand2,
};

local disabledResolutionsText = Locale.Lookup("TXT_KEY_LEAGUE_OVERVIEW_OTHER_RESOLUTIONS");
RegisterCollapseBehavior{
	Header = Controls.DisabledResolutionsHeader,
	HeaderLabel = Controls.DisabledResolutionsHeaderLabel,
	HeaderExpandedLabel = "[ICON_MINUS] " .. disabledResolutionsText,
	HeaderCollapsedLabel = "[ICON_PLUS] " .. disabledResolutionsText,
	Panel = Controls.DisabledResolutionStack,
	Collapsed = false,
	OnCollapse = OnCollapseExpand,
	OnExpand = OnCollapseExpand,
};

RegisterCollapseBehavior{
	Header = Controls.DisabledResolutionsHeader2,
	HeaderLabel = Controls.DisabledResolutionsHeaderLabel2,
	HeaderExpandedLabel = "[ICON_MINUS] " .. disabledResolutionsText,
	HeaderCollapsedLabel = "[ICON_PLUS] " .. disabledResolutionsText,
	Panel = Controls.DisabledResolutionStack2,
	Collapsed = false,
	OnCollapse = OnCollapseExpand2,
	OnExpand = OnCollapseExpand2,
};
-----------------------------------------------------------------
-- Add Espionage Overview to Dropdown
-----------------------------------------------------------------
LuaEvents.AdditionalInformationDropdownGatherEntries.Add(function(entries)
	if (not Game.IsOption("GAMEOPTION_NO_LEAGUES")) then
		table.insert(entries, {
			text = Locale.Lookup("TXT_KEY_LEAGUE_OVERVIEW"),
			call = function() 
				Events.SerialEventGameMessagePopup{ 
					Type = ButtonPopupTypes.BUTTONPOPUP_LEAGUE_OVERVIEW,
				};
			end,
		});
	end
end);

-- Just in case :)
LuaEvents.RequestRefreshAdditionalInformationDropdownEntries();

Controls.DisabledResolutionsHeader2:SetHide(true);
Controls.TitleLabel:SetText("Leagues and Unions")
Controls.ViewAllResolutionsButton:SetText("Renact Resolution")
Controls.ResetButton:SetText("Make Proposal")

function OnCSIncludeAlive( bIsChecked )
	bCountCS = bIsChecked;
	Update()
end
Controls.CheckIncludeCS:RegisterCheckHandler( OnCSIncludeAlive );