include( "InstanceManager" );
include( "IconSupport" );


local isVisible = false;
local currentID;
local dataTable = {}

-----------------------------
-- Globals --
-----------------------------
local m_bDebug = false;
local m_PopupInfo = nil;
local m_iLeague = nil;
local m_iProject = nil;

local cultureStrings = {};
for row in DB.Query("SELECT * FROM ITG_Cultures") do
	cultureStrings[row.ID] = row.Description;
end

local m_PlayerRankInstanceManager = InstanceManager:new( "PlayerEntryInstance", "PlayerEntryBox", Controls.PlayerListStack );
local m_RewardTierInstanceManager = InstanceManager:new( "RewardTierInstance", "RewardTierBox", Controls.PlayerListStack );

local m_tContributions = {}; --[iLeague][iProject][iPlayer]


-------------------------------------------------
-- General UI Handlers
-------------------------------------------------
function OnPopup( popupInfo )

	if( popupInfo.Type ~= ButtonPopupTypes.BUTTONPOPUP_LEAGUE_PROJECT_COMPLETED ) then
		return;
	end
	
	m_PopupInfo = popupInfo;
	m_iLeague = popupInfo.Data1;
	m_iProject = popupInfo.Data2;
	
	UpdateContributions();
	UpdateAll();

	UIManager:QueuePopup( ContextPtr, PopupPriority.WhosWinning );
end
Events.SerialEventGameMessagePopup.Add( OnPopup );

function UpdateAll()
	print("Well LeagueProjectPopup forgotten")
end

function OnClose ()
	table.remove(dataTable, 1)
	if dataTable[1] then
		UpdateITGData()
	else
		 isVisible = false;
		 UIManager:DequeuePopup( ContextPtr );
	end
end
Controls.CloseButton:RegisterCallback( Mouse.eLClick, OnClose );
Events.GameplaySetActivePlayer.Add(OnClose);

function InputHandler( uiMsg, wParam, lParam )
    if uiMsg == KeyEvents.KeyDown then
        if wParam == Keys.VK_ESCAPE or wParam == Keys.VK_RETURN then
            OnClose();
            return true;
        end
    end
end
ContextPtr:SetInputHandler( InputHandler );

function ShowHideHandler( bIsHide, bInitState )
    if( not bInitState ) then
        if( not bIsHide ) then
        	UI.incTurnTimerSemaphore();
        	Events.SerialEventGameMessagePopupShown(m_PopupInfo);
        else
            UI.decTurnTimerSemaphore();
            Events.SerialEventGameMessagePopupProcessed.CallImmediate(ButtonPopupTypes.BUTTONPOPUP_LEAGUE_PROJECT_COMPLETED, 0);
        end
    end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );

-------------------------------------------------
-- AddRewardTier
-------------------------------------------------
function AddRewardTier (iRewardTier)
	local tProject = GameInfo.LeagueProjects[m_iProject];
	if (tProject ~= nil) then
		local controlTable = m_RewardTierInstanceManager:GetInstance();
		local sText = "";
		local sTooltip = "";
		
		if (iRewardTier > 3 or iRewardTier < 0) then
			return;
		elseif (iRewardTier == 0) then
			sText = Locale.Lookup("TXT_KEY_LEAGUE_PROJECT_REWARD_NONE");
			sTooltip = Locale.Lookup("TXT_KEY_LEAGUE_PROJECT_REWARD_NONE_HELP");
		else
			sText = Locale.Lookup(GameInfo.LeagueProjectRewards[tProject["RewardTier" .. iRewardTier]].Description);
			sTooltip = Locale.Lookup(GameInfo.LeagueProjectRewards[tProject["RewardTier" .. iRewardTier]].Description);
			-- Includes all previous tiers
			for i = 1, iRewardTier, 1 do
				sTooltip = sTooltip .. "[NEWLINE][ICON_BULLET]";
				sTooltip = sTooltip .. Locale.Lookup(GameInfo.LeagueProjectRewards[tProject["RewardTier" .. i]].Help);
			end
		end
		
		controlTable.RewardTierLabel:SetText(sText);
		controlTable.RewardTierBox:SetToolTipString(sTooltip);
	end
end

-------------------------------------------------
-- AddPlayerEntry
-------------------------------------------------
function AddPlayerEntry (iPlayerID, iScore, iTier, iRank)
	if (m_iLeague ~= nil and m_iProject ~= nil) then
		local pLeague = Game.GetLeague(m_iLeague);
		if (pLeague ~= nil) then
			local pPlayer = Players[iPlayerID];
			local iTeam = pPlayer:GetTeam();
			local pTeam = Teams[iTeam];

			local controlTable = m_PlayerRankInstanceManager:GetInstance();
			controlTable.CultTypeLabel:SetText("")
			local leader = GameInfo.Leaders[pPlayer:GetLeaderType()];
			local strName = "";
			
			-- Active player
			if (pPlayer:GetID() == Game.GetActivePlayer()) then
				strName = Locale.ConvertTextKey( "TXT_KEY_POP_VOTE_RESULTS_YOU" );
				CivIconHookup( iPlayerID, 32, controlTable.CivIcon, controlTable.CivIconBG, controlTable.CivIconShadow, true, true, controlTable.CivIconHighlight);  
				IconHookup( leader.PortraitIndex, 64, leader.IconAtlas, controlTable.Portrait );
			-- Haven't yet met this player
			elseif (not pTeam:IsHasMet(Game.GetActiveTeam())) then
				strName = Locale.ConvertTextKey( "TXT_KEY_POP_VOTE_RESULTS_UNMET_PLAYER" );
				CivIconHookup( -1, 32, controlTable.CivIcon, controlTable.CivIconBG, controlTable.CivIconShadow, true, true, controlTable.CivIconHighlight);
				IconHookup( 22, 64, "LEADER_ATLAS", controlTable.Portrait );
			-- Met players
			else
				strName = Locale.ConvertTextKey(pPlayer:GetNameKey());
				CivIconHookup( iPlayerID, 32, controlTable.CivIcon, controlTable.CivIconBG, controlTable.CivIconShadow, true, true, controlTable.CivIconHighlight);  
				IconHookup( leader.PortraitIndex, 64, leader.IconAtlas, controlTable.Portrait );
			end 
			
			controlTable.PlayerNameText:SetText(strName);
			
			-- Formats the # and reduces it to X.XX
			local strPoints = Locale.ConvertTextKey("TXT_KEY_FORMAT_NUMBER", iScore) .. "[ICON_PRODUCTION]";
			controlTable.ScoreText:SetText(strPoints);
			
			-- Reward tier icons
			local strRewardIcons = "";
			local strRewardTT = "";
			if (iTier >= 1) then
				strRewardIcons = strRewardIcons .. "[ICON_TROPHY_BRONZE]";
				strRewardTT = pLeague:GetProjectRewardTierDetails(1, m_iProject) .. strRewardTT;
			end
			if (iTier >= 2) then
				strRewardIcons = strRewardIcons .. "[ICON_TROPHY_SILVER]";
				strRewardTT = pLeague:GetProjectRewardTierDetails(2, m_iProject) .. "[NEWLINE][NEWLINE]" .. strRewardTT;
			end
			if (iTier >= 3) then
				strRewardIcons = strRewardIcons .. "[ICON_TROPHY_GOLD]";
				strRewardTT = pLeague:GetProjectRewardTierDetails(3, m_iProject) .. "[NEWLINE][NEWLINE]" .. strRewardTT;
			end
			controlTable.RewardIconsText:SetText(strRewardIcons);
			controlTable.RewardIconsText:SetToolTipString(strRewardTT);
		end
	end
end

function DistributeInfoData(data)
	table.insert(dataTable, data)
	if not isVisible then
		isVisible = true;
		currentID = 1;
		UpdateITGData()
		UIManager:QueuePopup( ContextPtr, PopupPriority.WhosWinning );
	end
	UpdateLowButton()
end
LuaEvents.ITGSG_CallRaceInfo.Add(DistributeInfoData)

local gWonderRaceHelpCTText = "The very best people of your culture group competed to come up with something astonishing that will stand the Test of Time and make anyone who look back in past on it, to call it a Wonder.[NEWLINE][NEWLINE]Rewards are based on contribution:[NEWLINE][NEWLINE][ICON_TROPHY_GOLD]Wonder Builder (Highest Contributor and at least ([ICON_GREAT_ENGINEER]"
local gWonderRaceHelpCTText2 ="): The right to construct a new Wonder and exclusion from future Wonder Races until the right is earned back. In case of tie the Wonder is not granted.[NEWLINE][NEWLINE][ICON_TROPHY_BRONZE]Seeker ([ICON_GREAT_ENGINEER] Any) 20% of contributed [ICON_GREAT_ENGINEER] Production is lost and converted into [ICON_GOLD] Gold.[NEWLINE][NEWLINE]15% of [ICON_GREAT_ENGINEER] Wonder production lost is also added to [ICON_GREAT_PEOPLE] Great People points.";


local gWonderRaceHelpText = "The very best people of the times competed to come up with something astonishing that will stand the Test of Time and make anyone who look back in past on it, to call it a Wonder.[NEWLINE][NEWLINE]This race does not require a total amount of Production to be completed. The better contribution earn the better reward:[NEWLINE][NEWLINE]"
local gWonderRaceHelpText2 = "[ICON_TROPHY_GOLD]Wonder Builder (Highest Contributor and at least "
local gWonderRaceHelpText25 = "): The right to construct a new Wonder and exclusion from future Wonder Races until the right is earned back. In case of tie the Wonder is not granted.[NEWLINE][NEWLINE]"
local gWonderRaceHelpText3 = "[ICON_TROPHY_SILVER]Capable of Wondrous Things ([ICON_GREAT_ENGINEER]"
local gWonderRaceHelpText4 = " ) 25% of contributed [ICON_GREAT_ENGINEER] Production is lost and converted into [ICON_GOLD] Gold.[NEWLINE][NEWLINE][ICON_TROPHY_BRONZE]Mere Mortal ([ICON_GREAT_ENGINEER] Any) 50% of contributed [ICON_GREAT_ENGINEER] Production is lost and converted into [ICON_CULTURE] Culture.[NEWLINE][NEWLINE]15% of [ICON_GREAT_ENGINEER] Wonder production lost is also added to [ICON_GREAT_PEOPLE] Great People points.";
local gHelpTextForArtPeriod = "Once again the most talented artist around the globe have clashed in the battle for immortality. The rewards are based on generated [ICON_GREAT_ARTIST] Art points by you and the art style you are currently representing:[NEWLINE][NEWLINE][ICON_TROPHY_GOLD]+2 [ICON_GREAT_WORK] Great Works for the greatest contributor in this art style, +1 [ICON_GREAT_WORK] Great Work for every representant. One [ICON_GREAT_WORK] Great Work for every "
local gHelpTextForArtPeriod2 = " [ICON_GREAT_ARTIST] Art points generated.[NEWLINE][NEWLINE][ICON_TROPHY_SILVER]+2 [ICON_GREAT_WORK] Great Works for the greatest contributor in this art style. One [ICON_GREAT_WORK] Great Work for every "
local gHelpTextForArtPeriod3 = " [ICON_GREAT_ARTIST] Art points generated.[NEWLINE][NEWLINE][ICON_TROPHY_BRONZE]+1 [ICON_GREAT_WORK] Great Work for the greatest contributor in this art style. One [ICON_GREAT_WORK] Great Work for every "
local gHelpTextForArtPeriod4 = " [ICON_GREAT_ARTIST] Art points generated.[NEWLINE][NEWLINE]Other art styles receive 1 [ICON_GREAT_WORK] Great Work per "
local gHelpTextForArtPeriod5 = " [ICON_GREAT_ARTIST] Art points generated. This value is also modified by the number of cities and other factors. All players receive [ICON_GREAT_PEOPLE] Great People points based on their Art Style."
local gDominationOverText = "The War has ended! Let it be a lesson for humanity not to forget. All members of surviving leagues are considered winners and receive:[NEWLINE][NEWLINE][ICON_TROPHY_GOLD]Winner: +5 [ICON_HAPPINESS_1] Happiness in all Cities. +30 [ICON_ITG_MERITOCRACY] Meritocracy.[NEWLINE][NEWLINE]All Total War resolutions were removed, the path to peace is open once more."

function UpdateITGData()
	m_PlayerRankInstanceManager:ResetInstances();
	m_RewardTierInstanceManager:ResetInstances();
	LuaEvents.ITGSG_RequestGT();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if dataTable[1].Type == "BasicRace" then
		local tProject = GameInfo.LeagueProjects[1];
		if (tProject.ProjectSplashImage ~= nil) then
			Controls.BackgroundImage:SetTextureAndResize(tProject.ProjectSplashImage);
		end
		Controls.PresentsLabel:LocalizeAndSetText(dataTable[1].Title);
		Controls.ListNameLabel:LocalizeAndSetText("TXT_KEY_LEAGUE_PROJECT_POPUP_COMPLETE");
		Controls.ProjectInfoLabel:SetText(dataTable[1].Help)
		for i, hh in pairs(dataTable[1].Players) do
			AddPlayerEntryBasic(hh[1], hh[2], hh[3], i, dataTable[1].OnlyOne, dataTable[1].SpecIcon);
		end
	elseif dataTable[1].Type == "DomOver" then
		local tProject = GameInfo.LeagueProjects[1];
		if (tProject.ProjectSplashImage ~= nil) then
			Controls.BackgroundImage:SetTextureAndResize(tProject.ProjectSplashImage);
		end
		if dataTable[1].Ideology then
			Controls.ListNameLabel:LocalizeAndSetText("Victory");
		else
			Controls.ListNameLabel:LocalizeAndSetText("Defeat");
		end
		Controls.PresentsLabel:LocalizeAndSetText("The War");
		Controls.ProjectInfoLabel:SetText(gDominationOverText)
		for i, bbb in pairs(dataTable[1].Data) do
			AddDominationEntry(bbb[1], bbb[3], bbb[4], bbb[2])
		end
	elseif dataTable[1].Type == "WonderRace" then
		local tProject = GameInfo.LeagueProjects[1];
		if (tProject.ProjectSplashImage ~= nil) then
			Controls.BackgroundImage:SetTextureAndResize(tProject.ProjectSplashImage);
		end
		local treshold = dataTable[1].CTreshold;
		Controls.PresentsLabel:LocalizeAndSetText("Wonder Race");
		Controls.ListNameLabel:LocalizeAndSetText("TXT_KEY_LEAGUE_PROJECT_POPUP_COMPLETE");
		Controls.ProjectInfoLabel:SetText(gWonderRaceHelpText .. gWonderRaceHelpText2 .. treshold .. gWonderRaceHelpText25 .. gWonderRaceHelpText3 .. treshold .. gWonderRaceHelpText4)
		local lWinner = dataTable[1].Winner;
		if dataTable[1].Wonder then
			Controls.ListNameLabel:SetText("You have designed " .. Locale.Lookup(GameInfo.Buildings[dataTable[1].Wonder].Description));
		end
	
		if lWinner then
			for i, value in pairs(dataTable[1].Data[2]) do
				for j, player in pairs(dataTable[1].Data[1][value]) do
					if player == lWinner then
						AddPlayerEntryWonderRace(player, value/100, 1, i, dataTable[1].ExtraYieldData[player], dataTable[1].GPPointsT[player], dataTable[1].LeagueTag[player]);
						break;
					end
				end
			end
		end
		for i, value in pairs(dataTable[1].Data[2]) do
			for j, player in pairs(dataTable[1].Data[1][value]) do
				if player ~= lWinner then
					local iContribution = value/100;
					local iTier = 0;
					if iContribution >= treshold then
						iTier = 2;
					elseif value > 0 then
						iTier = 3;
					end
					AddPlayerEntryWonderRace(player, iContribution, iTier, i, dataTable[1].ExtraYieldData[player], dataTable[1].GPPointsT[player], dataTable[1].LeagueTag[player]);
				end
			end
		end
	elseif dataTable[1].Type == "WonderRaceCT" then
		local tProject = GameInfo.LeagueProjects[1];
		if (tProject.ProjectSplashImage ~= nil) then
			Controls.BackgroundImage:SetTextureAndResize(tProject.ProjectSplashImage);
		end
		local treshold = dataTable[1].CPTreshold;
		local newtreshold = dataTable[1].CTreshold;
		Controls.PresentsLabel:LocalizeAndSetText("Wonder Race");
		Controls.ListNameLabel:LocalizeAndSetText("TXT_KEY_LEAGUE_PROJECT_POPUP_COMPLETE");
		local sText = gWonderRaceHelpCTText .. treshold .. gWonderRaceHelpCTText2;
--		Controls.ProjectInfoLabel:SetText(gWonderRaceHelpText .. gWonderRaceHelpText2 .. gWonderRaceHelpText3 .. treshold .. gWonderRaceHelpText4)
		if newtreshold ~= treshold then
			sText = sText .. "[NEWLINE][NEWLINE]Wonder Treshold for your Culture Group is increased to " .. newtreshold .. " now."
		end
		if dataTable[1].OnlyOne then
			sText = sText .. "[NEWLINE][NEWLINE]You can design only one World Wonder."
		end
		Controls.ProjectInfoLabel:SetText(sText)
		if dataTable[1].Wonder then
			Controls.ListNameLabel:SetText("You have designed " .. Locale.Lookup(GameInfo.Buildings[dataTable[1].Wonder].Description));
		end
	
		local ismax = true;
		for i, htt in pairs(dataTable[1].Data) do
			local iContribution = htt[2]/100;
			local iTier = 3;
			if dataTable[1].Winners[htt[1]] then
				iTier = 1;
			end
			AddPlayerEntryWonderRace(htt[1], iContribution, iTier, i, dataTable[1].ExtraYieldData[htt[1]], dataTable[1].GPPointsT[htt[1]], nil, htt[3]);
		end
	elseif dataTable[1].Type == "GreatWorks" then
		local tProject = GameInfo.LeagueProjects[1];
		if (tProject.ProjectSplashImage ~= nil) then
			Controls.BackgroundImage:SetTextureAndResize(tProject.ProjectSplashImage);
		end
		Controls.PresentsLabel:LocalizeAndSetText("Art Period");
		Controls.ListNameLabel:LocalizeAndSetText("TXT_KEY_LEAGUE_PROJECT_POPUP_COMPLETE");
		local gBase = dataTable[1].CurrentBaseCost;
		local gBetter = dataTable[1].GreaterBaseCost;
		Controls.ProjectInfoLabel:SetText(gHelpTextForArtPeriod .. gBetter .. gHelpTextForArtPeriod2 .. gBetter .. gHelpTextForArtPeriod3 .. gBetter .. gHelpTextForArtPeriod4 .. gBase .. gHelpTextForArtPeriod5)
	
	
	--HERE Edit
		for i, vsorted in pairs(dataTable[1].Sorted) do
			local gEraTypeTier = dataTable[1].GID[vsorted[1]][3];
			local gEraContribution = dataTable[1].GID[vsorted[1]][2];
			for j, playerdata in pairs(dataTable[1].EraP[vsorted[1]]) do
				local player = playerdata[1];
				local pPlayerLoop = Players[player];
				local iContribution = playerdata[2];
				local iTier = playerdata[3];
				local gType = gData.Players[player].CultureType;
				local gEraType = gData.Players[player].CultureEra;
				AddPlayerEntryGreatWorks(player, iContribution, iTier, gData.Globals.CultureErasNames[gType][gEraType], gEraContribution, gEraTypeTier, dataTable[1].NumGreatWorks[player], dataTable[1].CostTable[player], dataTable[1].GPPoints[player]);
			end
			ismax = false;
		end
	
	end
	Controls.PlayerListStack:CalculateSize();
	Controls.PlayerListStack:ReprocessAnchoring();
	Controls.PlayerListScrollPanel:CalculateInternalSize();
	UpdateLowButton()
end

function AddPlayerEntryGreatWorks(iPlayerID, iScore, iPlayerTier, eraName, eScore, eTier, numgW, tt1, gpstring)
	local pPlayer = Players[iPlayerID];
	local iTeam = pPlayer:GetTeam();
	local pTeam = Teams[iTeam];

	local controlTable = m_PlayerRankInstanceManager:GetInstance();
	controlTable.CultTypeLabel:SetText("")
	local leader = GameInfo.Leaders[pPlayer:GetLeaderType()];
	local strName = "";
			
	-- Active player
	if (pPlayer:GetID() == Game.GetActivePlayer()) then
		strName = Locale.ConvertTextKey( "TXT_KEY_POP_VOTE_RESULTS_YOU" );
		CivIconHookup( iPlayerID, 32, controlTable.CivIcon, controlTable.CivIconBG, controlTable.CivIconShadow, true, true, controlTable.CivIconHighlight);  
		IconHookup( leader.PortraitIndex, 64, leader.IconAtlas, controlTable.Portrait );
	-- Haven't yet met this player
	elseif (not pTeam:IsHasMet(Game.GetActiveTeam())) then
		strName = Locale.ConvertTextKey( "TXT_KEY_POP_VOTE_RESULTS_UNMET_PLAYER" );
		CivIconHookup( -1, 32, controlTable.CivIcon, controlTable.CivIconBG, controlTable.CivIconShadow, true, true, controlTable.CivIconHighlight);
		IconHookup( 22, 64, "LEADER_ATLAS", controlTable.Portrait );
	-- Met players
	else
		strName = Locale.ConvertTextKey(pPlayer:GetNameKey());
		CivIconHookup( iPlayerID, 32, controlTable.CivIcon, controlTable.CivIconBG, controlTable.CivIconShadow, true, true, controlTable.CivIconHighlight);  
		IconHookup( leader.PortraitIndex, 64, leader.IconAtlas, controlTable.Portrait );
	end 
	controlTable.PlayerNameText:SetText(strName .. "[NEWLINE](" .. eraName .. ")");
	controlTable.PlayerNameText:SetToolTipString("");
	controlTable.CivIconFrame:SetHide(false)
	controlTable.IconFrame:SetHide(false)
	local strPoints = Locale.ConvertTextKey("TXT_KEY_FORMAT_NUMBER", iScore) .. "[ICON_GREAT_ARTIST]";
	if eScore == 1 then
		controlTable.ScoreText:SetText(strPoints .. "([ICON_TROPHY_GOLD])");
	elseif eScore == 2 then
		controlTable.ScoreText:SetText(strPoints .. "([ICON_TROPHY_SILVER])");
	elseif eScore == 3 then
		controlTable.ScoreText:SetText(strPoints .. "([ICON_TROPHY_BRONZE])");
	else
		controlTable.ScoreText:SetText(strPoints);
	end
	local strRewardIcons = "";
	local strRewardTT = "One [ICON_GREAT_WORK] Great Work per " .. tt1 .. " points.";
	if (eTier == 3) then
		strRewardIcons = strRewardIcons .. "[ICON_TROPHY_BRONZE]";
	elseif (eTier == 2) then
		strRewardIcons = strRewardIcons .. "[ICON_TROPHY_SILVER]";
	elseif (eTier == 1) then
		strRewardIcons = strRewardIcons .. "[ICON_TROPHY_GOLD]";
	end
	if iPlayerTier == 1 then
		strRewardIcons = strRewardIcons .. "[ICON_TROPHY_GOLD]";
	end
	local sstringhh = "(" .. numgW .. "[ICON_GREAT_WORK]";
	if gpstring then
		sstringhh = sstringhh .. ", " .. gpstring;
	end
	sstringhh = sstringhh .. ")";
	controlTable.RewardIconsText:SetText(strRewardIcons .. sstringhh);
	controlTable.RewardIconsText:SetToolTipString(strRewardTT);
end

function AddDominationEntry(leagueAcro, leagueTT, bActive, score)
	local controlTable = m_PlayerRankInstanceManager:GetInstance();
	controlTable.CultTypeLabel:SetText("")
	controlTable.IconFrame:SetHide(true)
	controlTable.PlayerNameText:SetText(leagueAcro);
	controlTable.PlayerNameText:SetToolTipString(leagueTT);

	controlTable.ScoreText:SetText(score);
	if bActive then
		controlTable.RewardIconsText:SetText("[ICON_TROPHY_GOLD]");
	else
		controlTable.RewardIconsText:SetText("");
	end
	controlTable.RewardIconsText:SetToolTipString("");
end

function AddPlayerEntryWonderRace(iPlayerID, iScore, iTier, iRank, yield, gpString, leagueTag, cultureType)
	local pPlayer = Players[iPlayerID];
	local iTeam = pPlayer:GetTeam();
	local pTeam = Teams[iTeam];

	local controlTable = m_PlayerRankInstanceManager:GetInstance();
	controlTable.CultTypeLabel:SetText("")
	local leader = GameInfo.Leaders[pPlayer:GetLeaderType()];
	local strName = "";
			
	-- Active player
	if (pPlayer:GetID() == Game.GetActivePlayer()) then
		strName = Locale.ConvertTextKey( "TXT_KEY_POP_VOTE_RESULTS_YOU" );
		CivIconHookup( iPlayerID, 32, controlTable.CivIcon, controlTable.CivIconBG, controlTable.CivIconShadow, true, true, controlTable.CivIconHighlight);  
		IconHookup( leader.PortraitIndex, 64, leader.IconAtlas, controlTable.Portrait );
		if cultureType then
			controlTable.CultTypeLabel:SetText("(" .. Locale.Lookup(cultureStrings[cultureType]) .. ")")
		end
	-- Haven't yet met this player
	elseif (not pTeam:IsHasMet(Game.GetActiveTeam())) then
		strName = Locale.ConvertTextKey( "TXT_KEY_POP_VOTE_RESULTS_UNMET_PLAYER" );
		CivIconHookup( -1, 32, controlTable.CivIcon, controlTable.CivIconBG, controlTable.CivIconShadow, true, true, controlTable.CivIconHighlight);
		IconHookup( 22, 64, "LEADER_ATLAS", controlTable.Portrait );
	-- Met players
	else
		strName = Locale.ConvertTextKey(pPlayer:GetNameKey());
		CivIconHookup( iPlayerID, 32, controlTable.CivIcon, controlTable.CivIconBG, controlTable.CivIconShadow, true, true, controlTable.CivIconHighlight);  
		IconHookup( leader.PortraitIndex, 64, leader.IconAtlas, controlTable.Portrait );
		if cultureType then
			controlTable.CultTypeLabel:SetText("(" .. Locale.Lookup(cultureStrings[cultureType]) .. ")")
		end
	end
	if leagueTag then
		strName = strName .. " (League)"
	end
			
	controlTable.PlayerNameText:SetText(strName);
	controlTable.PlayerNameText:SetToolTipString("");
	controlTable.CivIconFrame:SetHide(false)
	controlTable.IconFrame:SetHide(false) 
			
	-- Formats the # and reduces it to X.XX
	local strPoints = Locale.ConvertTextKey("TXT_KEY_FORMAT_NUMBER", iScore) .. "[ICON_GREAT_ENGINEER]";
	controlTable.ScoreText:SetText(strPoints);
			
	-- Reward tier icons
	local strRewardIcons = "";
	local strRewardTT = "";
	if (iTier == 1) then
		strRewardIcons = strRewardIcons .. "[ICON_TROPHY_GOLD]";
	elseif (iTier == 2) then
		strRewardIcons = strRewardIcons .. "[ICON_TROPHY_SILVER]";
	elseif (iTier == 3) then
		strRewardIcons = strRewardIcons .. "[ICON_TROPHY_BRONZE]";
	end
	local bAny = false;
	if yield then
		bAny = true;
		strRewardIcons = strRewardIcons .. " (" .. yield;
		if gpString then
			strRewardIcons = strRewardIcons .. ", " .. gpString;
		end
	else
		if gpString then
			bAny = true;
			strRewardIcons = strRewardIcons .. " (" .. gpString;
		end
	end
	if bAny then
		strRewardIcons = strRewardIcons .. ")"
		strRewardTT = strRewardIcons;
	end 
	controlTable.RewardIconsText:SetText(strRewardIcons);
	controlTable.RewardIconsText:SetToolTipString(strRewardTT);
end

function UpdateLowButton()
	if #dataTable > 1 then
		Controls.CloseButton:SetText("Next")
	else
		Controls.CloseButton:SetText("Close")
	end
end

function AddPlayerEntryBasic(iPlayerID, iScore, iTier, iRank, bOnlyOne, sIcon)
	local pPlayer = Players[iPlayerID];
	local iTeam = pPlayer:GetTeam();
	local pTeam = Teams[iTeam];

	local controlTable = m_PlayerRankInstanceManager:GetInstance();
	controlTable.CultTypeLabel:SetText("");
	local leader = GameInfo.Leaders[pPlayer:GetLeaderType()];
	local strName = "";
			
	-- Active player
	if (pPlayer:GetID() == Game.GetActivePlayer()) then
		strName = Locale.ConvertTextKey( "TXT_KEY_POP_VOTE_RESULTS_YOU" );
		CivIconHookup( iPlayerID, 32, controlTable.CivIcon, controlTable.CivIconBG, controlTable.CivIconShadow, true, true, controlTable.CivIconHighlight);  
		IconHookup( leader.PortraitIndex, 64, leader.IconAtlas, controlTable.Portrait );
	-- Haven't yet met this player
	elseif (not pTeam:IsHasMet(Game.GetActiveTeam())) then
		strName = Locale.ConvertTextKey( "TXT_KEY_POP_VOTE_RESULTS_UNMET_PLAYER" );
		CivIconHookup( -1, 32, controlTable.CivIcon, controlTable.CivIconBG, controlTable.CivIconShadow, true, true, controlTable.CivIconHighlight);
		IconHookup( 22, 64, "LEADER_ATLAS", controlTable.Portrait );
	-- Met players
	elseif pPlayer:IsMinorCiv() then
		strName = Locale.ConvertTextKey(pPlayer:GetNameKey());
		CivIconHookup( iPlayerID, 32, controlTable.CivIcon, controlTable.CivIconBG, controlTable.CivIconShadow, true, true, controlTable.CivIconHighlight);  
		IconHookup( 22, 64, "LEADER_ATLAS", controlTable.Portrait );
	else
		strName = Locale.ConvertTextKey(pPlayer:GetNameKey());
		CivIconHookup( iPlayerID, 32, controlTable.CivIcon, controlTable.CivIconBG, controlTable.CivIconShadow, true, true, controlTable.CivIconHighlight);  
		IconHookup( leader.PortraitIndex, 64, leader.IconAtlas, controlTable.Portrait );
	end
	controlTable.PlayerNameText:SetText(strName);
	controlTable.PlayerNameText:SetToolTipString("");
	controlTable.CivIconFrame:SetHide(false)
	controlTable.IconFrame:SetHide(false)
			
	-- Formats the # and reduces it to X.XX
	local strPoints = Locale.ConvertTextKey("TXT_KEY_FORMAT_NUMBER", iScore) .. (sIcon or "[ICON_PRODUCTION]");
	controlTable.ScoreText:SetText(strPoints);
			
	-- Reward tier icons
	local strRewardIcons = "";
	local strRewardTT = "";
	if bOnlyOne then
		if (iTier == 1) then
			strRewardIcons = strRewardIcons .. "[ICON_TROPHY_GOLD]";
		elseif (iTier == 2) then
			strRewardIcons = strRewardIcons .. "[ICON_TROPHY_SILVER]";
		elseif (iTier == 3) then
			strRewardIcons = strRewardIcons .. "[ICON_TROPHY_BRONZE]";
		end
	else
		if (iTier == 1) then
			strRewardIcons = strRewardIcons .. "[ICON_TROPHY_GOLD][ICON_TROPHY_SILVER][ICON_TROPHY_BRONZE]";
		elseif (iTier == 2) then
			strRewardIcons = strRewardIcons .. "[ICON_TROPHY_SILVER][ICON_TROPHY_BRONZE]";
		elseif (iTier == 3) then
			strRewardIcons = strRewardIcons .. "[ICON_TROPHY_BRONZE]";
		end
	end
	controlTable.RewardIconsText:SetText(strRewardIcons);
	controlTable.RewardIconsText:SetToolTipString(strRewardTT);
end