include( "IconSupport" );
include( "UniqueBonuses" );
include( "InstanceManager" );

local vStringC = "08.2022[NEWLINE]v24"
local iCivID = -1;
local g_bLoadComplete;
local m_MilitaryIM = InstanceManager:new( "UnitInstance", "Quote", Controls.MissionTextStack );
local m_WondersIM = InstanceManager:new( "CircleWonderUUInstance", "Root", Controls.SubStack );
local wonderTClass = {}
local TabOfPieceNames = {"ITGLoadSetCPiece", "ITGLoadSetSPiece", "ITGLoadSetWPiece", "ITGLoadSetLPiece", "ITGLoadSetMPiece", "ITGLoadSetNWPiece", "ITGLoadSetAPiece"}
local bPiecesGenerated = false;
--local bIsReloadingNeeded = false;

function ShowHide( isHide, isInit )
	if ( not isInit ) then
		if ( isHide == true ) then
			UIManager:SetUICursor( 0 );
			Controls.Image:UnloadTexture();
			--print("Texture is unloaded");
			if (iCivID ~= -1) then
				Events.SerialEventDawnOfManHide(iCivID);
			end
		else
			OnInitScreen();
			UIManager:SetUICursor( 1 );
			if (iCivID ~= -1) then
--				Events.SerialEventDawnOfManShow(iCivID);        
			end
		end
	end
	
	if(not isHide) then
		UI.SetDontShowPopups(true);
	end
end
ContextPtr:SetShowHideHandler( ShowHide );

Controls.ProgressBar:SetPercent( 1 );

function OnInitScreen()
	g_bLoadComplete = false;
	
    Controls.AlphaAnim:SetToBeginning();
	Controls.ActivateButton:SetHide(true);
	HideTheDoM()
	
	local civIndex = PreGame.GetCivilization( Game:GetActivePlayer() );
    
    local civ = GameInfo.Civilizations[civIndex];
    
    if(civ == nil) then
		PreGame.SetCivilization(0, -1);
	end
	
    if ( not PreGame.IsMultiplayerGame() ) then
		-- Force some settings off when loading a HotSeat game.
        PreGame.SetGameOption("GAMEOPTION_DYNAMIC_TURNS", false);
        PreGame.SetGameOption("GAMEOPTION_SIMULTANEOUS_TURNS", false);
        PreGame.SetGameOption("GAMEOPTION_PITBOSS", false);
    end	

	
end      

function OnActivateButtonClicked()
	--print("Activate button clicked!");
	LuaEvents.ITGSGPlayerClosedLoadScreen()
	Events.LoadScreenClose();
	if (not PreGame.IsMultiplayerGame() and not PreGame.IsHotSeatGame()) then
		Game.SetPausePlayer(-1);
	end
	UI.SetDontShowPopups(false);
	LuaEvents.ITGDoMRemovedCheckIfEndPanelOn()
end
Controls.ActivateButton:RegisterCallback( Mouse.eLClick, OnActivateButtonClicked );


----------------------------------------------------------------        
-- Key Down Processing
----------------------------------------------------------------        
function InputHandler( uiMsg, wParam, lParam )
    if( uiMsg == KeyEvents.KeyDown )
    then
        if( wParam == Keys.VK_ESCAPE or wParam == Keys.VK_RETURN ) then
			if (g_bLoadComplete) then
				OnActivateButtonClicked();
			end
        end
    end
    return true;
end
ContextPtr:SetInputHandler( InputHandler );

function HideBackgrounds ()
	GenerateTheDoM()
	Controls.AlphaAnim:Play();
end

function OnSequenceGameInitComplete ()
	print("SGIC", os.clock())
	g_bLoadComplete = true;	
	
	if (PreGame.IsMultiplayerGame() or PreGame.IsHotSeatGame()) then
		OnActivateButtonClicked();
	else
		Game.SetPausePlayer(Game.GetActivePlayer());
		local strGameButtonName;
		if (not UI:IsLoadedGame()) then
			strGameButtonName = Locale.ConvertTextKey("TXT_KEY_BEGIN_GAME_BUTTON");
		else
			strGameButtonName = Locale.ConvertTextKey("TXT_KEY_BEGIN_GAME_BUTTON_CONTINUE");
		end
	
		Controls.ActivateButtonText:SetText(strGameButtonName);
		Controls.ActivateButton:SetHide(false);

		GenerateTheDoM()
		HideBackgrounds();
        UIManager:SetUICursor( 0 );	
	end
end

Events.SequenceGameInitComplete.Add( OnSequenceGameInitComplete );

function GenerateTheDoM()	
	g_bLoadComplete = false;
	
	m_MilitaryIM:ResetInstances();
	local civIndex = PreGame.GetCivilization( Game:GetActivePlayer() );
    
    local civ = GameInfo.Civilizations[civIndex];
    -- Sets up Selected Civ Slot
    if ( civ ~= nil ) then
		
        -- Use the Civilization_Leaders table to cross reference from this civ to the Leaders table
        local leader = GameInfo.Leaders[GameInfo.Civilization_Leaders( "CivilizationType = '" .. civ.Type .. "'" )().LeaderheadType];
        local leaderDescription = leader.Description;

		-- Set Leader & Civ Text
		Controls.Civilization:LocalizeAndSetText( Locale.Lookup(civ.ShortDescription) );
		Controls.Leader:SetText( Locale.Lookup(leaderDescription) .. "," );
        
        -- Set Civ Leader Icon
	--	it does not seem to be used even in unmodded game?
--		IconHookup( leader.PortraitIndex, 128, leader.IconAtlas, Controls.Portrait );
		
		-- Set Civ Icon
		SimpleCivIconHookup( Game.GetActivePlayer(), 80, Controls.IconShadow );
		
         -- Set Text
		LuaEvents.ITGSG_RequestGT();
		local gData = MapModData.ITGSG;
		local gActivePlayer = Game.GetActivePlayer()
		MapModData.ITGSG = nil;
		if gData and gData.Players[gActivePlayer] then
			LuaEvents.ITGSG_RequestDoMinfo(gActivePlayer)
			local domTab = MapModData.ITGSG;
			MapModData.ITGSG = nil
			for i, tt in pairs(domTab or {}) do
				local instance;
				instance = m_MilitaryIM:GetInstance();
				instance.Quote:SetText( tt[1]);
				instance.Quote:SetToolTipString( tt[2] or "");
			end
		end
        Controls.VersionString:SetText(vStringC)
        -- Sets Dawn of Man Image
		if not bPiecesGenerated then
	        Controls.Image:SetTexture("ITGLoadScreenBackground.dds");
			for i = 1, 16 do
				Controls["MiniImage" .. i]:SetTexture(TabOfPieceNames[math.random(1,#TabOfPieceNames)] .. i .. ".dds")
			end
			bPiecesGenerated = true;
		end
		Controls.Image:SetHide(false);
		--iCivID = civ.ID;
        --print("iCivID: " .. iCivID);
	end
	
    Controls.MissionTextStack:CalculateSize();
	Controls.MissionScroll:SetSizeY(500);
    Controls.MissionScroll:CalculateInternalSize();
    Controls.MissionScroll:ReprocessAnchoring();
	--new stuff
	--to make it look nice...
--	if bIsReloadingNeeded then
--		Controls.ActivateButtonText:SetText("Start the reloading.")
--	else
		Controls.ActivateButtonText:LocalizeAndSetText("TXT_KEY_BEGIN_GAME_BUTTON")
--	end
	Controls.BlackGrid:SetHide(false)
end

function HideTheDoM()
	GenerateTheDoM()
    Controls.BlackGrid:SetHide(true)
end

LuaEvents.ITGCallNewReloadMethod.Add(function()
--	bIsReloadingNeeded = true;
	Controls.BGReload:SetHide(false)
--	Controls.ActivateButtonText:SetText("Start the reloading.")

--Is it needed:?
--	OnActivateButtonClicked()
	Events.LoadScreenClose()
--	UI.SetDontShowPopups(false);
end)

LuaEvents.ITG_FailedOnInitialScript.Add(function(stringT)
	local errMessage = "Please report following error:"
	for i, str in pairs(stringT) do
		errMessage = errMessage .. "[NEWLINE]" .. str;
	end
	Controls.DescErrLabel:SetText(errMessage)
	Controls.ErrorGrid:SetSizeY(math.max(270, 160 + Controls.DescErrLabel:GetSizeY()));
	Controls.ErrorBox:SetHide(false)
end)

function OnMainMenuButtonClicked()
	Events.ExitToMainMenu();
end
Controls.MainMenuButton:RegisterCallback( Mouse.eLClick, OnMainMenuButtonClicked );

function OnExitGameButtonClicked()
	UI.ExitGame();
end
Controls.ErrExitButton:RegisterCallback( Mouse.eLClick, OnExitGameButtonClicked );