-------------------------------------------------
-- Help text for techs
-------------------------------------------------
local scenarioType = 1;
local ScenarioTable = {}
local bFirst = true;

function OnFirstTechInclude()
	LuaEvents.ITGSG_RequestGT();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if gData then
		scenarioType = gData.Globals.ScenarioType;
		ScenarioTable = {}
		for v, s in pairs(gData.Globals.GoodEras) do
			ScenarioTable[GameInfoTypes[v]] = 1
		end
		bFirst = false;
	end
end

LuaEvents.ITGSG_DoEraConversion.Add(function()
	bFirst = true;
end)

function GetHelpTextForTech( iTechID )
	local pTechInfo = GameInfo.Technologies[iTechID];
	
	local pActiveTeam = Teams[Game.GetActiveTeam()];
	local pActivePlayer = Players[Game.GetActivePlayer()];
	local pTeamTechs = pActiveTeam:GetTeamTechs();
	local iTechCost = pActivePlayer:GetResearchCost(iTechID);
	
	local strHelpText = "";

	-- Name
	strHelpText = strHelpText .. Locale.ToUpper(Locale.ConvertTextKey( pTechInfo.Description ));

	-- Do we have this tech?
	if (pTeamTechs:HasTech(iTechID)) then
		strHelpText = strHelpText .. " [COLOR_POSITIVE_TEXT](" .. Locale.ConvertTextKey("TXT_KEY_RESEARCHED") .. ")[ENDCOLOR]";
	end

	-- Cost/Progress
	strHelpText = strHelpText .. "[NEWLINE]-------------------------[NEWLINE]";
	
	local iProgress = pActivePlayer:GetResearchProgress(iTechID);
	local bShowProgress = true;
	
	-- Don't show progres if we have 0 or we're done with the tech
	if (iProgress == 0 or pTeamTechs:HasTech(iTechID)) then
		bShowProgress = false;
	end
	
	-- Leads to...
	local strLeadsTo = "";
	local bFirstLeadsTo = true;

	if bFirst then
		OnFirstTechInclude()
	end
	for row in GameInfo.Technology_PrereqTechs() do
		local pPrereqTech = GameInfo.Technologies[row.PrereqTech];
		local pLeadsToTech = GameInfo.Technologies[row.TechType];
		
		if (pPrereqTech and pLeadsToTech) then
			if pLeadsToTech.IsInnovation or ScenarioTable[GameInfoTypes[pLeadsToTech.Era]] then
				if (pTechInfo.ID == pPrereqTech.ID) then
				
					-- If this isn't the first leads-to tech, then add a comma to separate
					if (bFirstLeadsTo) then
						bFirstLeadsTo = false;
					else
						strLeadsTo = strLeadsTo .. ", ";
					end
				
					strLeadsTo = strLeadsTo .. "[COLOR_POSITIVE_TEXT]" .. Locale.ConvertTextKey( pLeadsToTech.Description ) .. "[ENDCOLOR]";
				end
			end
		end
	end
	
	if (strLeadsTo ~= "") then
		strHelpText = strHelpText .. "[NEWLINE]";
		strHelpText = strHelpText .. Locale.ConvertTextKey("TXT_KEY_TECH_HELP_LEADS_TO", strLeadsTo);
	end

	local bFirstRequired = true;
	local strRequiresSth = "";
	if pTechInfo.IsInnovation then
		for row in GameInfo.Technology_PrereqTechs() do
			local pPrereqTech = GameInfo.Technologies[row.PrereqTech];
			local pLeadsToTech = GameInfo.Technologies[row.TechType];
		
			if (pPrereqTech and pLeadsToTech) then
				if (pTechInfo.ID == pLeadsToTech.ID) then
					if (bFirstRequired) then
						bFirstRequired = false;
					else
						strRequiresSth = strRequiresSth .. ", ";
					end
					strRequiresSth = strRequiresSth .. "[COLOR_POSITIVE_TEXT]" .. Locale.ConvertTextKey( pPrereqTech.Description ) .. "[ENDCOLOR]";
				
				end
			end
		end
	end

	if (strRequiresSth ~= "") then
		strHelpText = strHelpText .. "[NEWLINE]";
		strHelpText = strHelpText .. "[ICON_BULLET]Requires: " .. strRequiresSth;
	end

	local strExclusiveWith = "";
	local pairID = GetExclusivePairID(iTechID)
	if pairID then
		local sTech = GetSecondTech(pairID, iTechID)
		if sTech then

			strHelpText = strHelpText .. "[NEWLINE][ICON_BULLET]Is mutually exclusive with [COLOR_NEGATIVE_TEXT]" .. Locale.Lookup(GameInfo.Technologies[sTech].Description) .. "[ENDCOLOR].";
		end
	end

	-- Pre-written help text
	strHelpText = strHelpText .. "[NEWLINE]-------------------------[NEWLINE]";
	strHelpText = strHelpText .. Locale.ConvertTextKey( pTechInfo.Help );
	
	return strHelpText;
end

--exclusive tech tab

local ExclusiveTechITGtable = {
{ID = 1, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_EUROPE_1, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_EUROPE_3}, AIHelp = InnoAIHelpFuncPair1},
{ID = 2, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_EUROPE_2, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_EUROPE_4}, AIHelp = InnoAIHelpFuncPair2},
{ID = 3, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ASIA_1, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ASIA_2}, AIHelp = InnoAIHelpFuncPair3},
{ID = 4, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ASIA_3, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ASIA_4}, AIHelp = InnoAIHelpFuncPair4},
{ID = 5, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_MESOAMERICAN_2, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_MESOAMERICAN_3}, AIHelp = InnoAIHelpFuncPair5},
{ID = 6, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_MESOAMERICAN_1, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_MESOAMERICAN_4}, AIHelp = InnoAIHelpFuncPair6},
{ID = 7, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_EUROPE_5, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_EUROPE_6}, AIHelp = InnoAIHelpFuncPair7},
{ID = 8, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_MESOAMERICAN_5, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_MESOAMERICAN_6}, AIHelp = InnoAIHelpFuncPair8},
{ID = 9, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ASIA_5, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ASIA_6}, AIHelp = InnoAIHelpFuncPair9},
{ID = 10, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ASIA_7, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ASIA_8}, AIHelp = InnoAIHelpFuncPair10},
{ID = 11, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_MESOAMERICAN_7, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_MESOAMERICAN_8}, AIHelp = InnoAIHelpFuncPair11},
{ID = 12, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_EUROPE_7, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_EUROPE_8}, AIHelp = InnoAIHelpFuncPair12},
{ID = 13, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ARAB_1, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ARAB_2}, AIHelp = InnoAIHelpFuncPair13},
{ID = 14, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ARAB_3, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ARAB_4}, AIHelp = InnoAIHelpFuncPair14},
{ID = 15, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ARAB_5, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ARAB_6}, AIHelp = InnoAIHelpFuncPair15},
{ID = 16, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ARAB_7, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_ARAB_8}, AIHelp = InnoAIHelpFuncPair16},
{ID = 17, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_AFR_1, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_AFR_2}, AIHelp = InnoAIHelpFuncPair17},
{ID = 18, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_AFR_3, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_AFR_4}, AIHelp = InnoAIHelpFuncPair18},
{ID = 19, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_AFR_5, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_AFR_6}, AIHelp = InnoAIHelpFuncPair19},
{ID = 20, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_AFR_7, GameInfoTypes.TECH_UNIQUE_ITG_NEW_WORLD_AFR_8}, AIHelp = InnoAIHelpFuncPair20},
{ID = 21, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_EURO_1, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_EURO_2}, AIHelp = InnoAIHelpFuncPair21},
{ID = 22, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_EURO_3, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_EURO_4}, AIHelp = InnoAIHelpFuncPair22},
{ID = 23, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_EURO_5, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_EURO_6}, AIHelp = InnoAIHelpFuncPair23},
{ID = 24, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_EURO_7, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_EURO_8}, AIHelp = InnoAIHelpFuncPair24},
{ID = 25, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ARABIA_1, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ARABIA_2}, AIHelp = InnoAIHelpFuncPair25},
{ID = 26, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ARABIA_3, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ARABIA_4}, AIHelp = InnoAIHelpFuncPair26},
{ID = 27, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ARABIA_5, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ARABIA_6}, AIHelp = InnoAIHelpFuncPair27},
{ID = 28, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ARABIA_7, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ARABIA_8}, AIHelp = InnoAIHelpFuncPair28},
{ID = 29, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ASIA_1, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ASIA_2}, AIHelp = InnoAIHelpFuncPair29},
{ID = 30, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ASIA_3, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ASIA_4}, AIHelp = InnoAIHelpFuncPair30},
{ID = 31, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ASIA_5, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ASIA_6}, AIHelp = InnoAIHelpFuncPair31},
{ID = 32, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ASIA_7, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_ASIA_8}, AIHelp = InnoAIHelpFuncPair32},
{ID = 33, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_PAGAN_1, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_PAGAN_2}, AIHelp = InnoAIHelpFuncPair33},
{ID = 34, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_PAGAN_3, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_PAGAN_4}, AIHelp = InnoAIHelpFuncPair34},
{ID = 35, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_PAGAN_5, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_PAGAN_6}, AIHelp = InnoAIHelpFuncPair35},
{ID = 36, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_PAGAN_7, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_PAGAN_8}, AIHelp = InnoAIHelpFuncPair36},
{ID = 37, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_HORDE_1, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_HORDE_2}, AIHelp = InnoAIHelpFuncPair37},
{ID = 38, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_HORDE_3, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_HORDE_4}, AIHelp = InnoAIHelpFuncPair38},
{ID = 39, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_HORDE_5, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_HORDE_6}, AIHelp = InnoAIHelpFuncPair39},
{ID = 40, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_HORDE_7, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_HORDE_8}, AIHelp = InnoAIHelpFuncPair40},
{ID = 41, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_VIKING_1, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_VIKING_2}, AIHelp = InnoAIHelpFuncPair41},
{ID = 42, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_VIKING_3, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_VIKING_4}, AIHelp = InnoAIHelpFuncPair42},
{ID = 43, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_VIKING_5, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_VIKING_6}, AIHelp = InnoAIHelpFuncPair43},
{ID = 44, Techs = {GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_VIKING_7, GameInfoTypes.TECH_UNIQUE_ITG_TRIAL_VIKING_8}, AIHelp = InnoAIHelpFuncPair44},
}

function GetExclusivePairID(tInno)
	for i, data in pairs(ExclusiveTechITGtable) do
		for j, tech in pairs(data.Techs) do
			if tech == tInno then
				return i;
			end
		end
	end
	return nil;
end

function GetSecondTech(pairID, iTechID)
	for j, tech in pairs(ExclusiveTechITGtable[pairID].Techs) do
		if tech ~= iTechID then
			return tech;
		end
	end
	print("Error logical 124127755")
end