-------------------------------------------------
-- Tech Tree Popup
-------------------------------------------------
include( "InstanceManager" );

g_UseSmallIcons = true;

include( "TechButtonInclude" );
include( "TechHelpInclude" );

local scenarioType = 1;
local ScenarioTable = {[GameInfoTypes.ERA_MEDIEVAL] = 1, [GameInfoTypes.ERA_RENAISSANCE] = 1}
local BadTechs = {}
local bVertFirstTime = true;
local xBack = 15;
local innowidth = 2;
function DoTheInitialData()
	for row in GameInfo.Technologies() do
		if row.Show and (not row.IsInnovation) then
			if ScenarioTable[GameInfoTypes[row.Era]] then
				if not BadTechs[row.ID] then
					if row.GridX < xBack then
						xBack = row.GridX
					end
				end
			end
		end
	end
end

local innovationoffset = 0;
local m_PopupInfo = nil;
local stealingTechTargetPlayerID = -1;

local g_PipeManager = InstanceManager:new( "TechPipeInstance", "TechPipeIcon", Controls.TechTreeScrollPanel );
local g_EraManager = InstanceManager:new( "EraBlockInstance", "EraBlock", Controls.EraStack );
local g_TechInstanceManager = InstanceManager:new( "TechButtonInstance", "TechButton", Controls.TechTreeScrollPanel );

local g_NeedsFullRefreshOnOpen = false;
local g_NeedsFullRefresh = false;
LuaEvents.ITG_TechTreeDirty.Add(function()
	g_NeedsFullRefreshOnOpen = true;
	g_NeedsFullRefresh = true;
end)

local maxSmallButtons = 5;

-- I'll need these before I'm done
local playerID = Game.GetActivePlayer();	
local player = Players[playerID];
local civType = GameInfo.Civilizations[player:GetCivilizationType()].Type;
local activeTeamID = Game.GetActiveTeam();
local activeTeam = Teams[activeTeamID];

-- textures I'll be using
local right1Texture = "TechBranchH.dds"
local right2Texture = "TechBranches.dds"
local right2BTexture = "TechBranches.dds"
local right2TTexture = "TechBranches.dds"
local right3Texture = "TechBranches.dds"

local left1Texture = "TechBranchH.dds"
local left2Texture = "TechBranches.dds"
local left2TTexture = "TechBranches.dds"
local left2BTexture = "TechBranches.dds"

local topRightTexture = "TechBranches.dds"
local topLeftTexture = "TechBranches.dds"
local bottomRightTexture = "TechBranches.dds"
local bottomLeftTexture = "TechBranches.dds"

local hTexture = "TechBranchH.dds";
local vTexture = "TechBranchV.dds";

local connectorSize = { x = 32; y = 42; };

local pinkColor = {x = 2, y = 0, z = 2, w = 1};

local blockSpacingX = 270 + 96;
local blockSizeX = 270;
local blockSpacingY = 68;
local extraYOffset = 32;


local maxTechNameLength = 22 -- /- Locale.Length(Locale.Lookup("TXT_KEY_TURNS"));

-------------------------------------------------
-- Do initial setup stuff here
-------------------------------------------------

local techButtons = {};
local eraBlocks = {};
local eraColumns = {};



function InitialSetup()
	LuaEvents.ITGSG_RequestGT("TechTree");
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	
	scenarioType = gData.Globals.ScenarioType;
	ScenarioTable = {}
	if gData.Players[Game.GetActivePlayer()] then
		LuaEvents.ITGSG_RequestDisabledTechs(Game.GetActivePlayer())
		BadTechs = MapModData.ITGSG;
		MapModData.ITGSG = nil;
	else
		BadTechs = {}
	end
	for v, s in pairs(gData.Globals.GoodEras) do
		ScenarioTable[GameInfoTypes[v]] = 1
	end
	if gData.Globals.ScenarioType == 5 then
		innowidth = 3;
	else
		innowidth = 2;
	end
	DoTheInitialData()
	-- make the scroll bar the correct size for the display size
	Controls.TechTreeScrollBar:SetSizeX( Controls.TechTreeScrollPanel:GetSize().x - 150 );
	
	-- gather info about this player's unique units and buldings
	GatherInfoAboutUniqueStuff( civType, GetProperPlayerForGatheringUniques(playerID) );

	-- add the Era panels to the background
	AddEraPanels();

	-- add the pipes
	local techPipes = {};
	for row in GameInfo.Technologies() do
		if row.Show then
			techPipes[row.Type] = 
			{
				leftConnectionUp = false;
				leftConnectionDown = false;
				leftConnectionCenter = false;
				leftConnectionType = 0;
				rightConnectionUp = false;
				rightConnectionDown = false;
				rightConnectionCenter = false;
				rightConnectionType = 0;
				xOffset = 0;
				techType = row.Type;
			};
		end
	end
	
	local cnxCenter = 1
	local cnxUp = 2
	local cnxDown = 4
	
	-- Figure out which left and right adapters we need
	if not MapModData.DBScenario.IgnorePrereqTech then
		for row in GameInfo.Technology_PrereqTechs() do
			local prereq = GameInfo.Technologies[row.PrereqTech];
			local tech = GameInfo.Technologies[row.TechType];
			if tech and prereq and tech.Era then
				if ScenarioTable[GameInfoTypes[prereq.Era]] and ScenarioTable[GameInfoTypes[tech.Era]] then
					if (not BadTechs[GameInfoTypes[prereq.Type]]) and (not BadTechs[GameInfoTypes[tech.Type]]) then
						if tech.GridY < prereq.GridY then
							techPipes[tech.Type].leftConnectionDown = true;
							techPipes[prereq.Type].rightConnectionUp = true;
						elseif tech.GridY > prereq.GridY then
							techPipes[tech.Type].leftConnectionUp = true;
							techPipes[prereq.Type].rightConnectionDown = true;
						else -- tech.GridY == prereq.GridY
							techPipes[tech.Type].leftConnectionCenter = true;
							techPipes[prereq.Type].rightConnectionCenter = true;
						end
			
						local xOffset = ((tech.GridX - xBack) - (prereq.GridX - xBack)) - 1;
						if xOffset > techPipes[prereq.Type].xOffset then
							techPipes[prereq.Type].xOffset = xOffset;
						end
					end
				end
			end
		end
	for pipeIndex, thisPipe in pairs(techPipes) do
		if thisPipe.leftConnectionDown then
			thisPipe.leftConnectionType = thisPipe.leftConnectionType + cnxDown;
		end 
		if thisPipe.leftConnectionUp then
			thisPipe.leftConnectionType = thisPipe.leftConnectionType + cnxUp;
		end 
		if thisPipe.leftConnectionCenter then
			thisPipe.leftConnectionType = thisPipe.leftConnectionType + cnxCenter;
		end 
		if thisPipe.rightConnectionDown then
			thisPipe.rightConnectionType = thisPipe.rightConnectionType + cnxDown;
		end 
		if thisPipe.rightConnectionUp then
			thisPipe.rightConnectionType = thisPipe.rightConnectionType + cnxUp;
		end 
		if thisPipe.rightConnectionCenter then
			thisPipe.rightConnectionType = thisPipe.rightConnectionType + cnxCenter;
		end 
	end
	
	for row in GameInfo.Technology_PrereqTechs() do
		local prereq = GameInfo.Technologies[row.PrereqTech];
		local tech = GameInfo.Technologies[row.TechType];
		if tech and prereq and tech.Era then
			if ScenarioTable[GameInfoTypes[prereq.Era]] and ScenarioTable[GameInfoTypes[tech.Era]] then
				if (not BadTechs[GameInfoTypes[prereq.Type]]) and (not BadTechs[GameInfoTypes[tech.Type]]) then
					if (tech.GridX - xBack) - (prereq.GridX - xBack) > 1 then
						local hConnection = g_PipeManager:GetInstance();
						hConnection.TechPipeIcon:SetOffsetVal((prereq.GridX - xBack)*blockSpacingX + blockSizeX + 96, (prereq.GridY-5)*blockSpacingY + 12 + extraYOffset);
						hConnection.TechPipeIcon:SetTexture(hTexture);
						local size = { x = ((tech.GridX - xBack)-(prereq.GridX - xBack) - 1)*blockSpacingX + 4; y = 42; };
						hConnection.TechPipeIcon:SetSize(size);
					end
			
					if tech.GridY - prereq.GridY == 1 or tech.GridY - prereq.GridY == -1 then
						local vConnection = g_PipeManager:GetInstance();
						vConnection.TechPipeIcon:SetOffsetVal(((tech.GridX - xBack)-1)*blockSpacingX + blockSizeX + 96, ((tech.GridY-5)*blockSpacingY) - (((tech.GridY-prereq.GridY) * blockSpacingY) / 2) + extraYOffset);
						vConnection.TechPipeIcon:SetTexture(vTexture);
						local size = { x = 32; y = (blockSpacingY / 2) + 8; };
						vConnection.TechPipeIcon:SetSize(size);
					end

					if tech.GridY - prereq.GridY == 2 or tech.GridY - prereq.GridY == -2 then
						local vConnection = g_PipeManager:GetInstance();
						vConnection.TechPipeIcon:SetOffsetVal(((tech.GridX - xBack)-1)*blockSpacingX + blockSizeX + 96, (tech.GridY-5)*blockSpacingY - (((tech.GridY-prereq.GridY) * blockSpacingY) / 2) + extraYOffset);
						vConnection.TechPipeIcon:SetTexture(vTexture);
						local size = { x = 32; y = (blockSpacingY * 3 / 2) + 8; };
						vConnection.TechPipeIcon:SetSize(size);
					end
			
					if tech.GridY - prereq.GridY == 3 or tech.GridY - prereq.GridY == -3 then
						local vConnection = g_PipeManager:GetInstance();
						vConnection.TechPipeIcon:SetOffsetVal(((tech.GridX - xBack)-1)*blockSpacingX + blockSizeX + 96, ((tech.GridY-5)*blockSpacingY) - (((tech.GridY-prereq.GridY) * blockSpacingY) / 2) + extraYOffset);
						vConnection.TechPipeIcon:SetTexture(vTexture);
						local size = { x = 32; y = blockSpacingY * 2 + 20; };
						vConnection.TechPipeIcon:SetSize(size);
					end
			
					if tech.GridY - prereq.GridY == 4 or tech.GridY - prereq.GridY == -4 then
						local vConnection = g_PipeManager:GetInstance();
						vConnection.TechPipeIcon:SetOffsetVal(((tech.GridX - xBack)-1)*blockSpacingX + blockSizeX + 96, ((tech.GridY-5)*blockSpacingY) - (((tech.GridY-prereq.GridY) * blockSpacingY) / 2) + extraYOffset);
						vConnection.TechPipeIcon:SetTexture(vTexture);
						local size = { x = 32; y = blockSpacingY * 3 + 8; };
						vConnection.TechPipeIcon:SetSize(size);
					end
				end
			end
		end
	end
	
	for pipeIndex, thisPipe in pairs(techPipes) do
	
		local tech = GameInfo.Technologies[thisPipe.techType];
		
		local yOffset = (tech.GridY-5)*blockSpacingY + 12 + extraYOffset;
		
		if thisPipe.rightConnectionType >= 1 then
			
			local startPipe = g_PipeManager:GetInstance();
			startPipe.TechPipeIcon:SetOffsetVal( (tech.GridX - xBack)*blockSpacingX + blockSizeX + 64, yOffset );
			startPipe.TechPipeIcon:SetTexture(right1Texture);
			startPipe.TechPipeIcon:SetSize(connectorSize);
			
			local pipe = g_PipeManager:GetInstance();			
			if thisPipe.rightConnectionType == 1 then
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX + blockSizeX + 96, yOffset );
				pipe.TechPipeIcon:SetTexture(right1Texture);
			elseif thisPipe.rightConnectionType == 2 then
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack)+thisPipe.xOffset)*blockSpacingX + blockSizeX + 96 - 12, yOffset - 15 );
				pipe.TechPipeIcon:SetTexture(bottomRightTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(72,72));
			elseif thisPipe.rightConnectionType == 3 then
				--pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack)+thisPipe.xOffset)*blockSpacingX + blockSizeX + 96 - 6, yOffset - 15 );
				--pipe.TechPipeIcon:SetTexture(right2BTexture);
				--pipe.TechPipeIcon:SetTextureOffset(Vector2(36,72));
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX + blockSizeX + 96, yOffset );
				pipe.TechPipeIcon:SetTexture(right1Texture);
				pipe = g_PipeManager:GetInstance();			
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack)+thisPipe.xOffset)*blockSpacingX + blockSizeX + 96 - 12, yOffset - 15 );
				pipe.TechPipeIcon:SetTexture(bottomRightTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(72,72));
			elseif thisPipe.rightConnectionType == 4 then
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack)+thisPipe.xOffset)*blockSpacingX + blockSizeX + 96 - 12, yOffset );
				pipe.TechPipeIcon:SetTexture(topRightTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(72,0));
			elseif thisPipe.rightConnectionType == 5 then
				--pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX + blockSizeX + 96 - 6, yOffset );
				--pipe.TechPipeIcon:SetTexture(right2TTexture);
				--pipe.TechPipeIcon:SetTextureOffset(Vector2(36,0));
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX + blockSizeX + 96, yOffset );
				pipe.TechPipeIcon:SetTexture(right1Texture);
				pipe = g_PipeManager:GetInstance();			
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX + blockSizeX + 96 - 12, yOffset );
				pipe.TechPipeIcon:SetTexture(topRightTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(72,0));
			elseif thisPipe.rightConnectionType == 6 then
				--pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack)+thisPipe.xOffset)*blockSpacingX + blockSizeX + 96 - 12, yOffset - 6 );
				--pipe.TechPipeIcon:SetTexture(right2Texture);
				--pipe.TechPipeIcon:SetTextureOffset(Vector2(72,36));
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX + blockSizeX + 96 - 12, yOffset );
				pipe.TechPipeIcon:SetTexture(topRightTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(72,0));
				pipe = g_PipeManager:GetInstance();			
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack)+thisPipe.xOffset)*blockSpacingX + blockSizeX + 96 - 12, yOffset - 15 );
				pipe.TechPipeIcon:SetTexture(bottomRightTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(72,72));
			else-- thisPipe.rightConnectionType == 7 then
				--pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX + blockSizeX + 96 - 6, yOffset - 6 );
				--pipe.TechPipeIcon:SetTexture(right3Texture);
				--pipe.TechPipeIcon:SetTextureOffset(Vector2(36,36));
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX + blockSizeX + 96, yOffset );
				pipe.TechPipeIcon:SetTexture(right1Texture);
				pipe = g_PipeManager:GetInstance();			
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX + blockSizeX + 96 - 12, yOffset );
				pipe.TechPipeIcon:SetTexture(topRightTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(72,0));
				pipe = g_PipeManager:GetInstance();			
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack)+thisPipe.xOffset)*blockSpacingX + blockSizeX + 96 - 12, yOffset - 15 );
				pipe.TechPipeIcon:SetTexture(bottomRightTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(72,72));
			end
		end
		
		if thisPipe.leftConnectionType >= 1 then
			
			local startPipe = g_PipeManager:GetInstance();
			startPipe.TechPipeIcon:SetOffsetVal( (tech.GridX - xBack)*blockSpacingX + 26, yOffset );
			startPipe.TechPipeIcon:SetTexture(left1Texture);
			startPipe.TechPipeIcon:SetSize(	Vector2(40, 42) );
			
			local pipe = g_PipeManager:GetInstance();			
			if thisPipe.leftConnectionType == 1 then
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX, yOffset );
				pipe.TechPipeIcon:SetTexture(left1Texture);
			elseif thisPipe.leftConnectionType == 2 then
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX, yOffset - 15);
				pipe.TechPipeIcon:SetTexture(bottomLeftTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(0,72));
			elseif thisPipe.leftConnectionType == 3 then
				--pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX - 6, yOffset - 15 );
				--pipe.TechPipeIcon:SetTexture(left2BTexture);
				--pipe.TechPipeIcon:SetTextureOffset(Vector2(36,72));
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX, yOffset );
				pipe.TechPipeIcon:SetTexture(left1Texture);
				pipe = g_PipeManager:GetInstance();	
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX, yOffset - 15);
				pipe.TechPipeIcon:SetTexture(bottomLeftTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(0,72));
			elseif thisPipe.leftConnectionType == 4 then
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX, yOffset );
				pipe.TechPipeIcon:SetTexture(topLeftTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(0,0));
			elseif thisPipe.leftConnectionType == 5 then
				--pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX - 6, yOffset );
				--pipe.TechPipeIcon:SetTexture(left2TTexture);
				--pipe.TechPipeIcon:SetTextureOffset(Vector2(36,0));
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX, yOffset );
				pipe.TechPipeIcon:SetTexture(left1Texture);
				pipe = g_PipeManager:GetInstance();	
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX, yOffset );
				pipe.TechPipeIcon:SetTexture(topLeftTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(0,0));
			elseif thisPipe.leftConnectionType == 6 then
				--pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX, yOffset - 6 );
				--pipe.TechPipeIcon:SetTexture(left2Texture);
				--pipe.TechPipeIcon:SetTextureOffset(Vector2(0,36));
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX, yOffset );
				pipe.TechPipeIcon:SetTexture(topLeftTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(0,0));
				pipe = g_PipeManager:GetInstance();	
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX, yOffset - 15);
				pipe.TechPipeIcon:SetTexture(bottomLeftTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(0,72));
			else-- thisPipe.rightConnectionType == 7 then
				--pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX - 6, yOffset - 6 );
				--pipe.TechPipeIcon:SetTexture(left2Texture);
				--pipe.TechPipeIcon:SetTextureOffset(Vector2(36,36));
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX, yOffset );
				pipe.TechPipeIcon:SetTexture(left1Texture);
				pipe = g_PipeManager:GetInstance();	
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX, yOffset );
				pipe.TechPipeIcon:SetTexture(topLeftTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(0,0));
				pipe = g_PipeManager:GetInstance();	
				pipe.TechPipeIcon:SetOffsetVal( ((tech.GridX - xBack))*blockSpacingX, yOffset - 15);
				pipe.TechPipeIcon:SetTexture(bottomLeftTexture);
				pipe.TechPipeIcon:SetTextureOffset(Vector2(0,72));
			end
		end
			
	end
	
	end
	--here ends pipes

	techPediaSearchStrings = {};

	-- add the instances of the tech panels
	for tech in GameInfo.Technologies() do
		if tech.Era and tech.Show and (not tech.IsInnovation) then
			if ScenarioTable[GameInfoTypes[tech.Era]] then
				if (not BadTechs[tech.ID]) then
					AddTechButton( tech );
				end
			end
		end
	end
	-- resize the panel to fit the contents
	Controls.EraStack:CalculateSize();
	Controls.EraStack:ReprocessAnchoring();
    Controls.TechTreeScrollPanel:CalculateInternalSize();
    
    --initialized = true;		
end

function AddEraPanels()
	-- find the range of columns that each era takes
	for tech in GameInfo.Technologies() do
		if tech.Era and tech.Show and (not tech.IsInnovation) then
			local eraID = GameInfo.Eras[tech.Era].ID;
			if ScenarioTable[eraID] then
				if not eraColumns[eraID] then
					eraColumns[eraID] = { minGridX = (tech.GridX - xBack); maxGridX = (tech.GridX - xBack); researched = false; };
				else
					if (tech.GridX - xBack) < eraColumns[eraID].minGridX then
						eraColumns[eraID].minGridX = (tech.GridX - xBack);
					end
					if (tech.GridX - xBack) > eraColumns[eraID].maxGridX then
						eraColumns[eraID].maxGridX = (tech.GridX - xBack);
					end
				end
			end
		end
	end

	-- add the era panels
	for era in GameInfo.Eras() do
		if ScenarioTable[era.ID] then
			local thisEraBlockInstance = g_EraManager:GetInstance();
			-- store this panel off for later
			eraBlocks[era.ID] = thisEraBlockInstance;
		
			-- add the correct text for this era panel
			local textString = era.Description;
			local localizedLabel = Locale.ConvertTextKey( textString );
			thisEraBlockInstance.OldLabel:SetText( localizedLabel );
			thisEraBlockInstance.CurrentLabel:SetText( localizedLabel );
			thisEraBlockInstance.FutureLabel:SetText( localizedLabel );
		
			-- adjust the sizes of the era panels
			local blockWidth;
			if (eraColumns[era.ID] ~= nil) then
				blockWidth = (eraColumns[era.ID].maxGridX - eraColumns[era.ID].minGridX + 1);
			else
				blockWidth = 1;
			end
			innovationoffset = innovationoffset + blockWidth;

			blockWidth = (blockWidth * blockSpacingX);
			if era.ID == 0 then
				blockWidth = blockWidth + 32;
			end
			local blockSize = thisEraBlockInstance.EraBlock:GetSize();
			blockSize.x = blockWidth;
			thisEraBlockInstance.EraBlock:SetSize( blockSize );
			blockSize = thisEraBlockInstance.FrameBottom:GetSize();
			blockSize.x = blockWidth;
			thisEraBlockInstance.FrameBottom:SetSize( blockSize );	
		
			blockSize = thisEraBlockInstance.OldBar:GetSize();
			blockSize.x = blockWidth;
			thisEraBlockInstance.OldBar:SetSize( blockSize );
			blockSize = thisEraBlockInstance.OldBlock:GetSize();
			blockSize.x = blockWidth;
			thisEraBlockInstance.OldBlock:SetSize( blockSize );
		
			blockSize = thisEraBlockInstance.CurrentBlock:GetSize();
			blockSize.x = blockWidth;
			thisEraBlockInstance.CurrentBlock:SetSize( blockSize );
			blockSize = thisEraBlockInstance.CurrentBlock1:GetSize();
			blockSize.x = blockWidth;
			thisEraBlockInstance.CurrentBlock1:SetSize( blockSize );
			blockSize = thisEraBlockInstance.CurrentBlock2:GetSize();
			blockSize.x = blockWidth;
			thisEraBlockInstance.CurrentBlock2:SetSize( blockSize );
				
			blockSize = thisEraBlockInstance.CurrentTop:GetSize();
			blockSize.x = blockWidth;
			thisEraBlockInstance.CurrentTop:SetSize( blockSize );
			blockSize = thisEraBlockInstance.CurrentTop1:GetSize();
			blockSize.x = blockWidth;
			thisEraBlockInstance.CurrentTop1:SetSize( blockSize );
			blockSize = thisEraBlockInstance.CurrentTop2:GetSize();
			blockSize.x = blockWidth;
			thisEraBlockInstance.CurrentTop2:SetSize( blockSize );
				
			blockSize = thisEraBlockInstance.FutureBlock:GetSize();
			blockSize.x = blockWidth;
			thisEraBlockInstance.FutureBlock:SetSize( blockSize );
		end
	end

	--innovations

	local thisEraBlockInstance = g_EraManager:GetInstance();
	-- store this panel off for later
	eraBlocks["Innovations"] = thisEraBlockInstance;
		
	-- add the correct text for this era panel
	local textString = "Innovations";
	local localizedLabel = "Innovations";
	thisEraBlockInstance.OldLabel:SetText( localizedLabel );
	thisEraBlockInstance.CurrentLabel:SetText( localizedLabel );
	thisEraBlockInstance.FutureLabel:SetText( localizedLabel );
	local blockWidth = innowidth;
	blockWidth = (blockWidth * blockSpacingX);
			
	local blockSize = thisEraBlockInstance.EraBlock:GetSize();
	blockSize.x = blockWidth;
	thisEraBlockInstance.EraBlock:SetSize( blockSize );
	blockSize = thisEraBlockInstance.FrameBottom:GetSize();
	blockSize.x = blockWidth;
	thisEraBlockInstance.FrameBottom:SetSize( blockSize );	
		
	blockSize = thisEraBlockInstance.OldBar:GetSize();
	blockSize.x = blockWidth;
	thisEraBlockInstance.OldBar:SetSize( blockSize );
	blockSize = thisEraBlockInstance.OldBlock:GetSize();
	blockSize.x = blockWidth;
	thisEraBlockInstance.OldBlock:SetSize( blockSize );
		
	blockSize = thisEraBlockInstance.CurrentBlock:GetSize();
	blockSize.x = blockWidth;
	thisEraBlockInstance.CurrentBlock:SetSize( blockSize );
	blockSize = thisEraBlockInstance.CurrentBlock1:GetSize();
	blockSize.x = blockWidth;
	thisEraBlockInstance.CurrentBlock1:SetSize( blockSize );
	blockSize = thisEraBlockInstance.CurrentBlock2:GetSize();
	blockSize.x = blockWidth;
	thisEraBlockInstance.CurrentBlock2:SetSize( blockSize );
				
	blockSize = thisEraBlockInstance.CurrentTop:GetSize();
	blockSize.x = blockWidth;
	thisEraBlockInstance.CurrentTop:SetSize( blockSize );
	blockSize = thisEraBlockInstance.CurrentTop1:GetSize();
	blockSize.x = blockWidth;
	thisEraBlockInstance.CurrentTop1:SetSize( blockSize );
	blockSize = thisEraBlockInstance.CurrentTop2:GetSize();
	blockSize.x = blockWidth;
	thisEraBlockInstance.CurrentTop2:SetSize( blockSize );
				
	blockSize = thisEraBlockInstance.FutureBlock:GetSize();
	blockSize.x = blockWidth;
	thisEraBlockInstance.FutureBlock:SetSize( blockSize );
end

function TechSelected( eTech, iDiscover)
	--print("eTech:"..tostring(eTech));
	print("stealingTechTargetPlayerID: " .. stealingTechTargetPlayerID);
	print("player:GetNumFreeTechs(): " ..  player:GetNumFreeTechs());
	if eTech > -1 then
		if (stealingTechTargetPlayerID ~= -1) then
			Network.SendResearch(eTech, 0, stealingTechTargetPlayerID, UIManager:GetShift());
		else
	   		Network.SendResearch(eTech, player:GetNumFreeTechs(), -1, UIManager:GetShift());
		end
   	end
end

function AddITGTechButton( tech, xGrid, yGrid )
	local thisTechButtonInstance = g_TechInstanceManager:GetInstance();
	if thisTechButtonInstance then
		
		-- store this instance off for later
		techButtons[tech.ID] = thisTechButtonInstance;
		
		-- position this instance
		thisTechButtonInstance.TechButton:SetOffsetVal( ((xGrid or tech.GridX ) + innovationoffset)*blockSpacingX + 64, ((yGrid or tech.GridY) -5)*blockSpacingY + extraYOffset);
		
		-- update the name of this instance
		local techName = Locale.ConvertTextKey( tech.Description );
		
		techName = Locale.TruncateString(techName, maxTechNameLength, true);
		thisTechButtonInstance.AlreadyResearchedTechName:SetText( techName );
		thisTechButtonInstance.CurrentlyResearchingTechName:SetText( techName );
		thisTechButtonInstance.AvailableTechName:SetText( techName );
		thisTechButtonInstance.UnavailableTechName:SetText( techName );
		thisTechButtonInstance.LockedTechName:SetText( techName );
		thisTechButtonInstance.FreeTechName:SetText( techName );
		
		thisTechButtonInstance.TechButton:SetToolTipString( GetHelpTextForTech(tech.ID) );
		
		-- update the picture
		if IconHookup( tech.PortraitIndex, 64, tech.IconAtlas, thisTechButtonInstance.TechPortrait ) then
			thisTechButtonInstance.TechPortrait:SetHide( false );
		else
			thisTechButtonInstance.TechPortrait:SetHide( true );
		end
		
		-- add the small pictures and their tooltips
		AddSmallButtonsToTechButton( thisTechButtonInstance, tech, maxSmallButtons, 45 );
		
	end
end

function AddTechButton( tech )
	local thisTechButtonInstance = g_TechInstanceManager:GetInstance();
	if thisTechButtonInstance then
		
		-- store this instance off for later
		techButtons[tech.ID] = thisTechButtonInstance;
		
		-- add the input handler to this button
		thisTechButtonInstance.TechButton:SetVoid1( tech.ID ); -- indicates tech to add to queue
		thisTechButtonInstance.TechButton:SetVoid2( 0 ); -- how many free techs
		techPediaSearchStrings[tostring(thisTechButtonInstance.TechButton)] = tech.Description;
		thisTechButtonInstance.TechButton:RegisterCallback( Mouse.eRClick, GetTechPedia );

		local scienceDisabled = Game.IsOption(GameOptionTypes.GAMEOPTION_NO_SCIENCE);
 		if (not scienceDisabled) then
			thisTechButtonInstance.TechButton:RegisterCallback( Mouse.eLClick, TechSelected );
		end
		
		-- position this instance
		thisTechButtonInstance.TechButton:SetOffsetVal( (tech.GridX - xBack)*blockSpacingX + 64, (tech.GridY-5)*blockSpacingY + extraYOffset);
		
		-- update the name of this instance
		local techName = Locale.ConvertTextKey( tech.Description );
		
		techName = Locale.TruncateString(techName, maxTechNameLength, true);
		thisTechButtonInstance.AlreadyResearchedTechName:SetText( techName );
		thisTechButtonInstance.CurrentlyResearchingTechName:SetText( techName );
		thisTechButtonInstance.AvailableTechName:SetText( techName );
		thisTechButtonInstance.UnavailableTechName:SetText( techName );
		thisTechButtonInstance.LockedTechName:SetText( techName );
		thisTechButtonInstance.FreeTechName:SetText( techName );
		
		thisTechButtonInstance.TechButton:SetToolTipString( GetHelpTextForTech(tech.ID) );
		
		-- update the picture
		if IconHookup( tech.PortraitIndex, 64, tech.IconAtlas, thisTechButtonInstance.TechPortrait ) then
			thisTechButtonInstance.TechPortrait:SetHide( false );
		else
			thisTechButtonInstance.TechPortrait:SetHide( true );
		end
		
		-- add the small pictures and their tooltips
		AddSmallButtonsToTechButton( thisTechButtonInstance, tech, maxSmallButtons, 45 );
		
	end
end


-------------------------------------------------
-- On Display
-------------------------------------------------
local g_isOpen = false;

function OnDisplay( popupInfo )

	if popupInfo.Type ~= ButtonPopupTypes.BUTTONPOPUP_TECH_TREE then
		return;
	end

	m_PopupInfo = popupInfo;

    g_isOpen = true;
	if g_NeedsFullRefresh or g_NeedsFullRefreshOnOpen then
		g_PipeManager:ResetInstances();
		g_TechInstanceManager:ResetInstances();
		g_EraManager:ResetInstances();
		bVertFirstTime = true;
		techButtons = {};
		eraBlocks = {};
		eraColumns = {};
		innovationoffset = 0;
		InitialSetup()
	end
    if not g_NeedsFullRefresh then
		g_NeedsFullRefresh = g_NeedsFullRefreshOnOpen;
	end
	g_NeedsFullRefreshOnOpen = false;

	LuaEvents.ITGSG_RequestGT("TechTree");
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if gData and gData.Players[Game.GetActivePlayer()] then
		Controls.InnovationLabel:SetText("Chance for innovation: " .. gData.Players[Game.GetActivePlayer()].TPInnovationChance / 10 .. "%.")
		if bVertFirstTime then
			bVertFirstTime = false;
			local innoTech = gData.Players[Game.GetActivePlayer()].Innovations;
			for i, techID in pairs(innoTech) do
		--		print("Is adding innovation", techID, xGrid, yGrid, innovationoffset)
				AddITGTechButton( GameInfo.Technologies[techID], xGrid, yGrid )
			end
		end
	end

	if( m_PopupInfo.Data1 == 1 ) then
    	if( ContextPtr:IsHidden() == false ) then
    	    OnCloseButtonClicked();
    	    return;
    	else
        	UIManager:QueuePopup( ContextPtr, PopupPriority.eUtmost );
    	end
	else
        UIManager:QueuePopup( ContextPtr, PopupPriority.TechTree );
    end
    
    stealingTechTargetPlayerID = popupInfo.Data2;
    
	Events.SerialEventGameMessagePopupShown(m_PopupInfo);
		
  	RefreshDisplay();
  	
end
Events.SerialEventGameMessagePopup.Add( OnDisplay );

function RefreshDisplay()
	LuaEvents.ITGSG_RequestGT();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	local iActivePlayer = Game.GetActivePlayer();
	for tech in GameInfo.Technologies() do
		if tech.Era and tech.Show and (not tech.IsInnovation) then
			if ScenarioTable[GameInfoTypes[tech.Era]] then
				if not BadTechs[tech.ID] then
					local bFirst = false;
					if gData then
						bFirst = (gData.Globals.TechFirstTeams[tech.ID] == iActivePlayer)
					end
					RefreshDisplayOfSpecificTech( tech, nil, bFirst);
				end
			end
		end
	end
	if gData and gData.Players[iActivePlayer] then
		local innoTech = gData.Players[Game.GetActivePlayer()].Innovations;
		for i, techID in pairs(innoTech) do
			RefreshDisplayOfSpecificTech( GameInfo.Technologies[techID] , gData.Globals.ExcludedTechs[techID], (gData.Globals.TechFirstTeams[techID] == iActivePlayer));
		end
	end
	
	-- update the era panels
	local highestEra = 0;
	for thisEra, sth in pairs(eraBlocks)  do
		if eraColumns[thisEra] then
			if eraColumns[thisEra].researched == true then
				highestEra = thisEra;
			end
		end
	end
	for thisEra, sth in pairs(eraBlocks)  do
		if thisEra ~= "Innovations" then
			local thisEraBlockInstance = eraBlocks[thisEra];
			if thisEra < highestEra then
				thisEraBlockInstance.OldBar:SetHide( false );
				thisEraBlockInstance.CurrentBlock:SetHide( true );
				thisEraBlockInstance.CurrentTop:SetHide( true );
				thisEraBlockInstance.FutureBlock:SetHide( true );
			elseif thisEra == highestEra then
				thisEraBlockInstance.OldBar:SetHide( true );
				thisEraBlockInstance.CurrentBlock:SetHide( false );
				thisEraBlockInstance.CurrentTop:SetHide( false );
				thisEraBlockInstance.FutureBlock:SetHide( true );
			else
				thisEraBlockInstance.OldBar:SetHide( true );
				thisEraBlockInstance.CurrentBlock:SetHide( true );
				thisEraBlockInstance.CurrentTop:SetHide( true );
				thisEraBlockInstance.FutureBlock:SetHide( false );
			end
		end
	end
	
	g_NeedsFullRefresh = false;
end

function RefreshDisplayOfSpecificTech( tech , iExcluded, bFirstShow)
	local techID = tech.ID;
	local thisTechButton = techButtons[techID];
  	local numFreeTechs = player:GetNumFreeTechs();
 	local researchTurnsLeft = player:GetResearchTurnsLeft( techID, true );
 	local turnText = tostring( researchTurnsLeft ).." "..turnsString;
	local isAllowedToStealTech = false;
	local isAllowedToGetTechFree = false;
	
	-- Espionage - stealing a tech!
 	if stealingTechTargetPlayerID ~= -1 then
 		if player:CanResearch( techID ) then
			opponentPlayer = Players[stealingTechTargetPlayerID];
			local opponentTeam = Teams[opponentPlayer:GetTeam()];
			if (opponentTeam:IsHasTech(techID)) then
				isAllowedToStealTech = true;
			end
		end
	end
	
	-- Choosing a free tech - extra conditions may apply
	if (numFreeTechs > 0) then
		if (player:CanResearchForFree(techID)) then
			isAllowedToGetTechFree = true;
		end
	end
 	
 	local potentiallyBlockedFromStealing = (stealingTechTargetPlayerID ~= -1) and ((not isAllowedToStealTech) or player:GetNumTechsToSteal(stealingTechTargetPlayerID) <= 0);
 	
 	-- Rebuild the small buttons if needed
 	if (g_NeedsFullRefresh) then
		AddSmallButtonsToTechButton( thisTechButton, tech, maxSmallButtons, 45 );
 	end
 	
 	thisTechButton.TechButton:SetToolTipString( GetHelpTextForTech(techID) );
 	
 	local scienceDisabled = Game.IsOption(GameOptionTypes.GAMEOPTION_NO_SCIENCE);
 	if (scienceDisabled) then
 		turnText = "";
 	end
 	
 	
	if(not scienceDisabled) then
		thisTechButton.TechButton:SetVoid1( techID ); -- indicates tech to add to queue
		thisTechButton.TechButton:SetVoid2( numFreeTechs ); -- how many free techs
		AddCallbackToSmallButtons( thisTechButton, maxSmallButtons, techID, numFreeTechs, Mouse.eLClick, TechSelected );
	end
	
 	if activeTeam:GetTeamTechs():HasTech(techID) then -- the player (or more accurately his team) has already researched this one
 		thisTechButton.AlreadyResearched:SetHide( false );
		thisTechButton.AlreadyResearchedFirst:SetHide(not bFirstShow);

 		thisTechButton.FreeTech:SetHide( true );
 		thisTechButton.CurrentlyResearching:SetHide( true );
 		thisTechButton.Available:SetHide( true );
 		thisTechButton.Unavailable:SetHide( true );
		thisTechButton.Locked:SetHide( true );
 
 		-- figure out if we need the first place dingus
 
 		-- update the era marker for this tech
		if tech.Era then
 			local eraID = GameInfo.Eras[tech.Era].ID;
			if eraColumns[eraID] then
				eraColumns[eraID].researched = true;
			end
		end
		
  		-- hide advisor icon
 		--thisTechButton.AdvisorIcon:SetHide( true );
 				
		if(not scienceDisabled) then
			thisTechButton.TechQueue:SetHide( true );
			thisTechButton.TechButton:SetVoid2( 0 ); -- num free techs
			thisTechButton.TechButton:SetVoid1( -1 ); -- indicates tech is invalid
			AddCallbackToSmallButtons( thisTechButton, maxSmallButtons, -1, 0, Mouse.eLClick, TechSelected );
 		end
 	 elseif (iExcluded) or (not player:CanEverResearch( techID ) or isAllowedToGetTechFree or stealingTechTargetPlayerID ~= -1) then
 		thisTechButton.AlreadyResearched:SetHide( true );
 		thisTechButton.CurrentlyResearching:SetHide( true );
 		thisTechButton.Available:SetHide( true );
 		thisTechButton.Unavailable:SetHide( true );
		thisTechButton.Locked:SetHide( false );
  		thisTechButton.FreeTech:SetHide( true );
		-- have queue number say "LOCKED"
		thisTechButton.TechQueueLabel:SetText( lockedString );
		thisTechButton.TechQueue:SetHide( false );
		-- hide the advisor icon
 		--thisTechButton.AdvisorIcon:SetHide( true );
		if(not scienceDisabled) then
			thisTechButton.TechButton:SetVoid1( -1 ); 
			thisTechButton.TechButton:SetVoid2( 0 ); -- num free techs
			AddCallbackToSmallButtons( thisTechButton, maxSmallButtons, -1, 0, Mouse.eLClick, TechSelected );
 		end	
 	elseif player:GetCurrentResearch() == techID and (not potentiallyBlockedFromStealing) then -- the player is currently researching this one
 		thisTechButton.AlreadyResearched:SetHide( true );
 		thisTechButton.Available:SetHide( true );
 		thisTechButton.Unavailable:SetHide( true );
		thisTechButton.Locked:SetHide( true );
		-- deal with free tech
		if (isAllowedToGetTechFree) or (stealingTechTargetPlayerID ~= -1 and isAllowedToStealTech) then
  			thisTechButton.FreeTech:SetHide( false );
 			thisTechButton.CurrentlyResearching:SetHide( true );
			-- update number of turns to research
 			if 	player:GetScience() > 0 and stealingTechTargetPlayerID == -1 then
  				thisTechButton.FreeTurns:SetText( turnText );
  				thisTechButton.FreeTurns:SetHide( false );
  			else
  				thisTechButton.FreeTurns:SetHide( true );
  			end
			thisTechButton.TechQueueLabel:SetText( freeString );
			thisTechButton.TechQueue:SetHide( false );
		else
  			thisTechButton.FreeTech:SetHide( true );
 			thisTechButton.CurrentlyResearching:SetHide( false );
			-- update number of turns to research
 			if 	player:GetScience() > 0 then
  				thisTechButton.CurrentlyResearchingTurns:SetText( turnText );
  				thisTechButton.CurrentlyResearchingTurns:SetHide( false );
  			else
  				thisTechButton.CurrentlyResearchingTurns:SetHide( true );
  			end
			thisTechButton.TechQueue:SetHide( true );
		end
 		-- turn on the meter
		local teamTechs = activeTeam:GetTeamTechs();
		local researchProgressPercent = 0;
		local researchProgressPlusThisTurnPercent = 0;
		local researchTurnsLeft = player:GetResearchTurnsLeft(techID, true);
		local currentResearchProgress = player:GetResearchProgress(techID);
		local researchNeeded = player:GetResearchCost(techID);
		local researchPerTurn = player:GetScience();
		local currentResearchPlusThisTurn = currentResearchProgress + researchPerTurn;		
		researchProgressPercent = currentResearchProgress / researchNeeded;
		researchProgressPlusThisTurnPercent = currentResearchPlusThisTurn / researchNeeded;		
		if (researchProgressPlusThisTurnPercent > 1) then
			researchProgressPlusThisTurnPercent = 1
		end
 		-- update advisor icon if needed
 		--thisTechButton.AdvisorIcon:SetHide( true );
 	elseif (player:CanResearch( techID ) and not scienceDisabled and (not potentiallyBlockedFromStealing)) then -- the player research this one right now if he wants
 		thisTechButton.AlreadyResearched:SetHide( true );
 		thisTechButton.CurrentlyResearching:SetHide( true );
 		thisTechButton.Unavailable:SetHide( true );
		thisTechButton.Locked:SetHide( true );
 		-- deal with free 		
		if (isAllowedToGetTechFree)  or (stealingTechTargetPlayerID ~= -1 and isAllowedToStealTech) then
 			thisTechButton.FreeTech:SetHide( false );
 			thisTechButton.Available:SetHide( true );
			-- update number of turns to research
 			if 	player:GetScience() > 0 and stealingTechTargetPlayerID == -1 then
  				thisTechButton.FreeTurns:SetText( turnText );
  				thisTechButton.FreeTurns:SetHide( false );
  			else
  				thisTechButton.FreeTurns:SetHide( true );
  			end
			-- update queue number to say "FREE"
			thisTechButton.TechQueueLabel:SetText( freeString );
			thisTechButton.TechQueue:SetHide( false );
		else
 			thisTechButton.FreeTech:SetHide( true );
 			thisTechButton.Available:SetHide( false );
			-- update number of turns to research
 			if 	player:GetScience() > 0 then
  				thisTechButton.AvailableTurns:SetText( turnText );
  				thisTechButton.AvailableTurns:SetHide( false );
  			else
  				thisTechButton.AvailableTurns:SetHide( true );
  			end
			-- update queue number if needed
			local queuePosition = player:GetQueuePosition( techID );
			if queuePosition == -1 then
				thisTechButton.TechQueue:SetHide( true );
			else
				thisTechButton.TechQueueLabel:SetText( tostring( queuePosition-1 ) );
				thisTechButton.TechQueue:SetHide( false );
			end
		end
  		-- update advisor icon if needed
 		--thisTechButton.AdvisorIcon:SetHide( true );
 	else -- currently unavailable
 		thisTechButton.AlreadyResearched:SetHide( true );
 		thisTechButton.CurrentlyResearching:SetHide( true );
 		thisTechButton.Available:SetHide( true );
 		thisTechButton.Unavailable:SetHide( false );
		thisTechButton.Locked:SetHide( true );
  		thisTechButton.FreeTech:SetHide( true );
 		-- update number of turns to research
 		if 	player:GetScience() > 0 then
  			thisTechButton.UnavailableTurns:SetText( turnText );
  			thisTechButton.UnavailableTurns:SetHide( false );
  		else
  			thisTechButton.UnavailableTurns:SetHide( true );
  		end
  		
		-- update queue number if needed
		local queuePosition = player:GetQueuePosition( techID );
		if queuePosition == -1 then
			thisTechButton.TechQueue:SetHide( true );
		else
			thisTechButton.TechQueueLabel:SetText( tostring( queuePosition-1 ) );
			thisTechButton.TechQueue:SetHide( false );
		end
		
 		-- update advisor icon if needed
 		--thisTechButton.AdvisorIcon:SetHide( true );

		if (isAllowedToGetTechFree) then
			thisTechButton.TechButton:SetVoid1( -1 ); 
			AddCallbackToSmallButtons( thisTechButton, maxSmallButtons, -1, 0, Mouse.eLClick, OnTechnologyButtonClicked );
		else
			if(not scienceDisabled) then
				thisTechButton.TechButton:SetVoid1( tech.ID );
				AddCallbackToSmallButtons( thisTechButton, maxSmallButtons, techID, numFreeTechs, Mouse.eLClick, TechSelected );
			end
		end
	end
end

----------------------------------------------------------------        
-- Input processing
----------------------------------------------------------------        

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function OnCloseButtonClicked ()
	UIManager:DequeuePopup( ContextPtr );
    Events.SerialEventGameMessagePopupProcessed.CallImmediate(ButtonPopupTypes.BUTTONPOPUP_TECH_TREE, 0);
    g_isOpen = false;	
end
Controls.CloseButton:RegisterCallback( Mouse.eLClick, OnCloseButtonClicked );

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

function InputHandler( uiMsg, wParam, lParam )
    if g_isOpen and uiMsg == KeyEvents.KeyDown then
        if wParam == Keys.VK_ESCAPE or wParam == Keys.VK_RETURN then
            OnCloseButtonClicked();
            return true;
        end
    end
end
ContextPtr:SetInputHandler( InputHandler );


function ShowHideHandler( bIsHide, bIsInit )
    if( not bIsInit ) then
        if( not bIsHide ) then
        	UI.incTurnTimerSemaphore();
        else
        	UI.decTurnTimerSemaphore();
        end
    end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );

----------------------------------------------------------------
-- 'Active' (local human) player has changed
----------------------------------------------------------------
function GetProperPlayerForGatheringUniques(playerID)
	if PreGame.GetGameOption("GAMEOPTION_ITG_AI_GAME") == 1 then
		for i = 0, 61 do
			if Players[i]:IsAlive() then
				return i;
			end
		end
	end
	return playerID
end

function OnTechTreeActivePlayerChanged( iActivePlayer, iPrevActivePlayer )
	
	playerID = Game.GetActivePlayer();	
	player = Players[playerID];
	civType = GameInfo.Civilizations[player:GetCivilizationType()].Type;
	activeTeamID = Game.GetActiveTeam();
	activeTeam = Teams[activeTeamID];	
	-- Rebuild some tables	
	GatherInfoAboutUniqueStuff( civType, GetProperPlayerForGatheringUniques(playerID));	
	
	-- So some extra stuff gets re-built on the refresh call
	if not g_isOpen then
		g_NeedsFullRefreshOnOpen = true;	
	else
		g_NeedsFullRefresh = true;
	end
	
	-- Close it, so the next player does not have to.
	OnCloseButtonClicked();
end
Events.GameplaySetActivePlayer.Add(OnTechTreeActivePlayerChanged);

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function OnEventResearchDirty()
	if (g_isOpen) then
		RefreshDisplay();
	end
end
Events.SerialEventResearchDirty.Add(OnEventResearchDirty);

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

--One time initialization
LuaEvents.ITG_GameLoaded.Add(function()
	InitialSetup()
end)
LuaEvents.ITG_IsGameLoaded()
local gData = MapModData.ITGSG;
MapModData.ITGSG = nil;
if gData then
	InitialSetup()
end
LuaEvents.ITG_ForceUniqueChangeData.Add(function()
	GatherInfoAboutUniqueStuff(civType, GetProperPlayerForGatheringUniques(playerID));
end)

LuaEvents.ITG_Civilopedia_TechDetails.Add(function(techID)
	MapModData.ITGSG = GetCivilopediaTechDetails(techID)
end)