-------------------------------
-- TopPanel.lua
-------------------------------

include("ITGSG_HelpFunctions.lua")

local isAImode = PreGame.GetGameOption("GAMEOPTION_ITG_AI_GAME") == 1
local tWt = Controls.TopPanelScrollPanel:GetSizeX();
local shouldNotUpdateData = (not isAImode);
local ggctt = 0;

function GetColorTopPanelBasedOnVals(val, above, belowRed, belowYellow, bAddPlus)
	local str = "";
	if val > above then
		str = "[COLOR_POSITIVE_TEXT]";
	elseif val < belowRed then
		str = "[COLOR_NEGATIVE_TEXT]";
	elseif val < belowYellow then
		str = "[COLOR_YELLOW]";
	end
	if bAddPlus then
		if val > 0 then
			str = str .. "+"
		end
	end
	return str .. val;
end

function doprint()
	print(ggctt)
end

LuaEvents.ITGTurnOffTopPanel.Add(function(bValue, bDoNotUpdate)
	if (not isAImode) then
		shouldNotUpdateData = bValue;
	end
	if not bDoNotUpdate then
		UpdateData()
	end
end)

LuaEvents.ITGSGGameLoaded.Add(function(bValue)
	if bValue then
		if Players[Game.GetActivePlayer()]:IsTurnActive() then
			shouldNotUpdateData = false;
			UpdateData()
		end
	else
		shouldNotUpdateData = true;
	end
end)

function UpdateData()
	if shouldNotUpdateData then
		ggctt = ggctt + 1;
		return;
	end
	LuaEvents.ITGSG_RequestGT(1);
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	-- Update turn counter
	local turn = Locale.ConvertTextKey("TXT_KEY_TP_TURN_COUNTER", gData.ITGTurn);
	Controls.CurrentTurn:SetText(turn);
	--DATE
	Controls.CurrentDate:SetText(GetTextDate(gData.ITGTurn, gData.Globals.StartDate));
	local iPlayerID = Game.GetActivePlayer();
	if (not Players[iPlayerID]:IsAlive()) then
		Controls.TopPanelScrollPanel:SetHide(true);
		return;
	else
		Controls.TopPanelScrollPanel:SetHide(false);
	end

	if( iPlayerID >= 0 ) then
		local pPlayer = Players[iPlayerID];
		local pTeam = Teams[pPlayer:GetTeam()];
		local pCity = UI.GetHeadSelectedCity();
		
		--1.Gold
		Controls.SciencePerTurn:SetHide(true)

		if (pPlayer:GetNumCities() > 0) then
			
			if (pCity ~= nil and UI.IsCityScreenUp()) then		
				Controls.MenuButton:SetText(Locale.ToUpper(Locale.ConvertTextKey("TXT_KEY_RETURN")));
				Controls.MenuButton:SetToolTipString(Locale.ConvertTextKey("TXT_KEY_CITY_SCREEN_EXIT_TOOLTIP"));
			else
				Controls.MenuButton:SetText(Locale.ToUpper(Locale.ConvertTextKey("TXT_KEY_MENU")));
				Controls.MenuButton:SetToolTipString(Locale.ConvertTextKey("TXT_KEY_MENU_TOOLTIP"));
			end
			
		end

		-- Update gold stats
			-----------------------------
			local iTotalGold = pPlayer:GetGold();
			local iGoldPerTurn = math.floor(gData.Players[iPlayerID].TPRealGoldRate);

		--	local strGoldStr = Locale.ConvertTextKey("TXT_KEY_TOP_PANEL_GOLD", iTotalGold, iGoldPerTurn);
			
			if (iGoldPerTurn <= 0) then
				Controls.GoldPerTurn:SetText("[COLOR:255:60:60:255](" .. iGoldPerTurn .. ")[/COLOR][COLOR:255:235:0:255]" .. iTotalGold .. " [ICON_GOLD][/COLOR]");
			else
				Controls.GoldPerTurn:SetText("[COLOR:255:235:0:255](+" .. iGoldPerTurn .. ") " .. iTotalGold .. " [ICON_GOLD][ENDCOLOR]");
			end	
			Controls.InvestCurrency:SetText("[COLOR:255:235:0:255]" .. gData.Players[iPlayerID].MintBonus .. "% [ICON_INVEST][ENDCOLOR]");
			-----------------------------
		
		-- Update date
		Controls.Morale:SetHide(true)
		Controls.CultureString:SetHide(true)
		Controls.FaithString:SetHide(true)
		--Happiness
		if gData.Players[iPlayerID].TPHap.NumCities == 0 then
			Controls.HappinessStack:SetHide(true)
		else
			Controls.HappinessStack:SetHide(false)
			if gData.Players[iPlayerID].TPHap.Average < 0 then
				Controls.HappinessString2:SetText("[ICON_HAPPINESS_3]")
				Controls.HappinessString1:SetText("[COLOR_NEGATIVE_TEXT]" .. gData.Players[iPlayerID].TPHap.Average .. "[ENDCOLOR]")
			else
				Controls.HappinessString2:SetText("[ICON_HAPPINESS_1]")
				Controls.HappinessString1:SetText("[COLOR_POSITIVE_TEXT]" .. gData.Players[iPlayerID].TPHap.Average .. "[ENDCOLOR]")
			end
			if (gData.Players[iPlayerID].TPHap.NumCities == 1) or (gData.Players[iPlayerID].TPHap.Max == gData.Players[iPlayerID].TPHap.Min) then
				Controls.HapinessMiniStack:SetHide(true)
			else
				Controls.HapinessMiniStack:SetHide(false)
				if gData.Players[iPlayerID].TPHap.Max < 0 then
					Controls.HappinessString3:SetText("[COLOR_NEGATIVE_TEXT]" .. gData.Players[iPlayerID].TPHap.Max .. "[ENDCOLOR]")
				else
					Controls.HappinessString3:SetText("[COLOR_POSITIVE_TEXT]" .. gData.Players[iPlayerID].TPHap.Max .. "[ENDCOLOR]")
				end
				if gData.Players[iPlayerID].TPHap.Min < 0 then
					Controls.HappinessString4:SetText("[COLOR_NEGATIVE_TEXT]" .. gData.Players[iPlayerID].TPHap.Min .. "[ENDCOLOR]")
				else
					Controls.HappinessString4:SetText("[COLOR_POSITIVE_TEXT]" .. gData.Players[iPlayerID].TPHap.Min .. "[ENDCOLOR]")
				end
			end
		end
		--SCIENCE
		-----------------------------
			-- Update science stats
			-----------------------------
			local strScienceText;
			local sciencePerTurn = gData.Players[iPlayerID].TPExtraSciencePerTurn;
			
			-- No Science
			if (sciencePerTurn <= 0) then
				strScienceText = string.format("[COLOR:255:60:60:255]" .. Locale.ConvertTextKey("TXT_KEY_NO_SCIENCE") .. "[/COLOR]");
			-- We have science
			else
				strScienceText = string.format("+%i", sciencePerTurn);

				local iGoldPerTurn = gData.Players[iPlayerID].TPRealGoldRate;
					
				-- Gold being deducted from our Science
				if (pPlayer:GetGold() + iGoldPerTurn < 0) then
					strScienceText = "[COLOR:255:60:0:255]" .. strScienceText .. "[/COLOR]";
				-- Normal Science state
				else
					strScienceText = "[COLOR:33:190:247:255]" .. strScienceText .. "[/COLOR]";
				end
			end
			strScienceText = "(" .. strScienceText .. ")";
			Controls.ScienceString1:SetText(strScienceText);
			Controls.ScienceString3:SetText("[COLOR:33:190:247:255]" .. math.floor(gData.Players[iPlayerID].Science) .. "[/COLOR]")
			local getNextCost = gData.Players[iPlayerID].TPNextScienceCost;
			Controls.ScienceString4:SetText("[COLOR:33:190:247:255]" .. getNextCost .. "[/COLOR]")
			Controls.SciencePerTurn:SetHide(false)

			-----------------------------
			-- Update Culture
			-----------------------------

			local strCultureStr;
			strCultureStr = string.format("(+%i)", gData.Players[iPlayerID].TPTotalCulturePerTurn );
			
			strCultureStr = "[COLOR:255:0:255:255]" .. strCultureStr .. "[/COLOR]";
			Controls.CultureString1:SetText(strCultureStr);
			Controls.CultureString3:SetText("[COLOR:255:0:255:255]" .. gData.Players[iPlayerID].Culture .. "[/COLOR]")
			local getNextCost = gData.Players[iPlayerID].TPNextCultureCost;
			Controls.CultureString4:SetText("[COLOR:255:0:255:255]" .. getNextCost .. "[/COLOR]")
			Controls.CultureString:SetHide(false)

			-- Update Faith
			-----------------------------
			if MapModData.DBScenario.NoFaithGeneration then
				Controls.FaithString:SetHide(true)
			else
				local strFaithStr;
				local tFaith = gData.Players[iPlayerID].TPTotalFaith
				if tFaith > 0 then
					strFaithStr = string.format("(+%i)", tFaith);
				else
					strFaithStr = string.format("(%i)", tFaith);
				end
				if gData.Players[iPlayerID].TPNextFaithCost then
					Controls.FaithString3:SetText(pPlayer:GetFaith());
					Controls.FaithString4:SetText(gData.Players[iPlayerID].TPNextFaithCost)
					Controls.FaithMiniStack:SetHide(false)
				else
					strFaithStr = strFaithStr .. pPlayer:GetFaith()
					Controls.FaithMiniStack:SetHide(true)
				end
				Controls.FaithString:SetHide(false)
				Controls.FaithString1:SetText(strFaithStr);
			end

			-- update settler
			if MapModData.DBScenario.SettlersOff then
				Controls.SettlerString:SetHide(true);
			else
				Controls.SettlerString:SetHide(false);
				if gData.Players[iPlayerID].TPSettlerPT >= 0 then
					Controls.SettlerString1:SetText("[COLOR_FONT_GREEN]+" .. (math.floor(gData.Players[iPlayerID].TPSettlerPT * 10) / 10 ))
				else
					Controls.SettlerString1:SetText("[COLOR_FONT_GREEN]" .. (math.floor(gData.Players[iPlayerID].TPSettlerPT * 10) / 10))
				end
				Controls.SettlerString3:SetText("[COLOR_FONT_GREEN]" .. math.floor(gData.Players[iPlayerID].SettlerGenerationPoints))
				Controls.SettlerString4:SetText("[COLOR_FONT_GREEN]" .. gData.Players[iPlayerID].SettlerTreshold)
			end
			--UPDATEGreatPerson

			if MapModData.DBScenario.Archaeology then
				Controls.ArchaeologyString:SetHide(false)
				Controls.ArchaeologyString1:SetText("[COLOR_SELECTED_TEXT]" .. string.format("(+%i)", math.floor(gData.Players[iPlayerID].PredictedArchaeologyNewTurn)))
				Controls.ArchaeologyString3:SetText("[COLOR_SELECTED_TEXT]" .. math.floor(gData.CurrentArchaelogyData.Players[iPlayerID]))
				Controls.ArchaeologyString4:SetText("[COLOR_SELECTED_TEXT]" .. gData.Players[iPlayerID].TotalArchaeologyGlobal)
			else
				Controls.ArchaeologyString:SetHide(true)
			end

			Controls.AuthorityString3:SetText("[COLOR_GREAT_PEOPLE_STORED]" .. gData.Players[iPlayerID].Authority)
			Controls.AuthorityString4:SetText("[COLOR_FONT_GREEN]" .. gData.Players[iPlayerID].AuthorityRank)

			Controls.MeritocracyString3:SetText(GetColorTopPanelBasedOnVals(gData.Players[iPlayerID].MeritocracyLevel, 50, 10, 25, false))
			Controls.MeritocracyString4:SetText(GetColorTopPanelBasedOnVals(gData.Players[iPlayerID].TPMeritocracyPT, 0, 0, 0, true))

			local gpPT;
			if gData.Players[iPlayerID].TPGPCivilPerTurn > 0 then
				gpPT = string.format("[COLOR_GREAT_PEOPLE_STORED](+%i)", math.floor(gData.Players[iPlayerID].TPGPCivilPerTurn));
			else
				gpPT = "[COLOR_GREAT_PEOPLE_STORED]" .. math.floor(gData.Players[iPlayerID].TPGPCivilPerTurn);
			end
			Controls.GreatPeopleString1:SetText("[COLOR_GREAT_PEOPLE_STORED]" .. gpPT);
			Controls.GreatPeopleString3:SetText("[COLOR_GREAT_PEOPLE_STORED]" .. math.floor(gData.Players[iPlayerID].GreatPeoplePoints));
			Controls.GreatPeopleString4:SetText("[COLOR_GREAT_PEOPLE_STORED]" .. gData.Players[iPlayerID].TPGPCivilTreshold)
			Controls.GreatPeopleString:SetHide(false)
			--UPDATE colony

			Controls.ColonyString1:SetText("[COLOR:255:235:0:255]" .. string.format("(+%i)", math.floor(gData.Players[iPlayerID].TPGPColonyPerTurn)));
			Controls.ColonyString3:SetText("[COLOR:255:235:0:255]" .. math.floor(gData.Players[iPlayerID].ColonyPoints)) ;
			Controls.ColonyString4:SetText("[COLOR:255:235:0:255]" .. gData.Players[iPlayerID].TPColonyTreshold) ;

			--UPDATE mercenary
			if MapModData.DBScenario.NoMercenaries then
				Controls.MercenaryString:SetHide(true);
			else
				Controls.MercenaryString:SetHide(false);
				local kBm = 0;
				for h, v in pairs(gData.Mercenaries) do
					kBm = kBm + v;
				end
				Controls.MercenaryString3:SetText("[COLOR:252:148:41:255]" .. kBm) ;
				Controls.MercenaryString4:SetText("[COLOR:252:148:41:255]" .. (100 + Players[iPlayerID]:GetMercenaryModifier() ));
			end

			--UDPDATE GA POINTS

			Controls.GreatAristString3:SetText("[COLOR:255:0:255:255]+" .. math.floor(gData.Players[iPlayerID].TPArtPointsPerTurn) .. "[/COLOR]")
			Controls.GreatAristString4:SetText("[COLOR:255:0:255:255]" .. math.floor(gData.Players[iPlayerID].GreatArtistPoints) .. "[/COLOR]")

		--supply
		-- Update Unit Supply
		local supplied = pPlayer:GetNumUnitsSupplied()
		local tosupply = pPlayer:GetNumUnitsToSupply()
		local coloursupply = "[COLOR_YELLOW]"
		if supplied > tosupply + 4 then
			coloursupply = "[COLOR_POSITIVE_TEXT]"
		elseif supplied < tosupply then
			coloursupply = "[COLOR_NEGATIVE_TEXT]"
		end
		Controls.UnitSupplyString3:SetText(coloursupply .. tosupply);
		Controls.UnitSupplyString4:SetText(coloursupply .. supplied);

		Controls.NumGreatWorks:SetText("[COLOR:255:0:255:255]" .. gData.Players[iPlayerID].NumGreatWorks .. "[ICON_GREAT_WORK][ENDCOLOR] ")
		
		--if wonder race then
		Controls.WonderRaceString3:SetText("+" .. math.floor(gData.Players[iPlayerID].TPWonderPointsPerTurn))
		Controls.WonderRaceString4:SetText(math.floor(gData.Players[iPlayerID].WonderProduction))
		Controls.WonderRace:SetHide(not gData.Players[iPlayerID].CanWonderRace)
		
		if gData.Players[iPlayerID].TPLegalPerTurn > 0 then
			Controls.LegalString:SetText("[COLOR_SELECTED_TEXT](+" .. gData.Players[iPlayerID].TPLegalPerTurn/100 .. ") " .. gData.Players[iPlayerID].LegalPower + gData.Players[iPlayerID].LegalPowerPercent / 100 .. "[ICON_ITG_LEGAL_POWER]");
		else
			Controls.LegalString:SetText("[COLOR_SELECTED_TEXT](" .. gData.Players[iPlayerID].TPLegalPerTurn/100 .. ") " .. gData.Players[iPlayerID].LegalPower + gData.Players[iPlayerID].LegalPowerPercent / 100 .. "[ICON_ITG_LEGAL_POWER]");
		end
		if gData.Players[iPlayerID].NatUnity >= 60 then
			Controls.CultureIndicator:SetText("[COLOR_POSITIVE_TEXT]" .. gData.Players[iPlayerID].NatUnity .. "%[ICON_ITG_NAT_UNITY]")
		elseif gData.Players[iPlayerID].NatUnity >= 20 then
			Controls.CultureIndicator:SetText("[COLOR_YELLOW]" .. gData.Players[iPlayerID].NatUnity .. "%[ICON_ITG_NAT_UNITY]")
		else
			Controls.CultureIndicator:SetText("[COLOR_NEGATIVE_TEXT]" .. gData.Players[iPlayerID].NatUnity .. "%[ICON_ITG_NAT_UNITY]")
		end
		
		-- anarchy
		Controls.AnarchyBox:SetHide(true)
		if (gData.Players[iPlayerID].TPAnarchyLegalLeft) and (not isAImode) then
			Controls.AnarchyBox:SetHide(false)
			Controls.AnarchyLegalLeft:SetText(gData.Players[iPlayerID].TPAnarchyLegalLeft)
		end
		
		--horse and iron // resources
		local pResource;
		local bShowResource;
		local iNumAvailable;
		local iNumUsed;
--		local iNumTotal;
			
		local strResourceText = "";
		local strTempText = "";
			
		for iRes, i in pairs(gData.Globals.ResourcesInScenario) do
			local pResource = GameInfo.Resources[iRes]
			local iResourceLoop = pResource.ID;
			bShowResource = true;
			if pResource.TechReveal then
				bShowResource = false;
				if (pTeam:GetTeamTechs():HasTech(GameInfoTypes[pResource.TechReveal])) then
					bShowResource = true;
				end
			end
			if pResource.TechCityTrade then
				bShowResource = false;
				if (pTeam:GetTeamTechs():HasTech(GameInfoTypes[pResource.TechCityTrade])) then
					bShowResource = true;
				end
			end
			if pResource.PolicyReveal then
				bShowResource = false;
				if pPlayer:HasPolicy(GameInfo.Policies[pResource.PolicyReveal].ID) then
					bShowResource = true;
				end
			end
			
							
			if MapModData.DBScenario.StrategicResStorage then	
				if (bShowResource) then
					local text = Locale.ConvertTextKey(pResource.IconString);
					iNumStored = pPlayer:GetResourceStored(iResourceLoop) - pPlayer:GetNumResourceUsed(iResourceLoop);
					iNumSource = pPlayer:GetNumResourceTotal(iResourceLoop, true) - (gData.Players[iPlayerID].TPResConsumedPT[iResourceLoop] or 0)
					if iNumSource ~= 0 then
						if iNumStored == 0 then
							if iNumSource < 0 then
								strTempText = string.format("%i %s   ", iNumSource, text);
							else
								strTempText = string.format("+%i %s   ", iNumSource, text);
							end
						elseif iNumSource < 0 then
							strTempText = string.format("%i(%i) %s   ", iNumStored, iNumSource, text);
						else
							strTempText = string.format("%i(+%i) %s   ", iNumStored, iNumSource, text);
						end
					else
						strTempText = string.format("%i %s   ", iNumStored, text);
					end
					if (iNumSource > 0) then
						strTempText = "[COLOR_POSITIVE_TEXT]" .. strTempText .. "[ENDCOLOR]";
					elseif (iNumSource < 0) then
						strTempText = "[COLOR_WARNING_TEXT]" .. strTempText .. "[ENDCOLOR]";
					end
					strResourceText = strResourceText .. strTempText;
				end
			else
				iNumAvailable = pPlayer:GetNumStrategicResourceAvailable(iResourceLoop, true);
				iNumUsed = pPlayer:GetNumResourceUsed(iResourceLoop);
				if (iNumUsed > 0) then
					bShowResource = true;
				end
							
				if (bShowResource) then
					local text = Locale.ConvertTextKey(pResource.IconString);
					strTempText = string.format("%i %s   ", iNumAvailable, text);
						
					-- Colorize for amount available
					if (iNumAvailable > 0) then
						strTempText = "[COLOR_POSITIVE_TEXT]" .. strTempText .. "[ENDCOLOR]";
					elseif (iNumAvailable < 0) then
						strTempText = "[COLOR_WARNING_TEXT]" .. strTempText .. "[ENDCOLOR]";
					end
						
					strResourceText = strResourceText .. strTempText;
				end
			end
		end
		Controls.HorseString:SetText(strResourceText);

		Controls.OlympicGamesString:SetHide(true)
		if gData.Globals.OlympicGames then
			if gData.Globals.OlympicGames.Players[iPlayerID] then
				if not gData.Globals.OlympicGames.OnGoing then
					Controls.OlympicGamesString:SetText(gData.Globals.OlympicGames.Next - gData.ITGTurn .. "T[ICON_IDEOLOGY_FREEDOM]")
				else
					Controls.OlympicGamesString:SetText("OG![ICON_IDEOLOGY_FREEDOM]");
				end
				Controls.OlympicGamesString:SetHide(false)
			end
		end


		-----------------------------
		-- Update Golden Age Info
		-----------------------------
		local strGoldenAgeStr;

	
		if (pPlayer:GetGoldenAgeTurns() > 0) then
			strGoldenAgeStr = string.format(Locale.ToUpper(Locale.ConvertTextKey("TXT_KEY_GOLDEN_AGE_ANNOUNCE")) .. " (%i)", pPlayer:GetGoldenAgeTurns());
		else
	--			strGoldenAgeStr = string.format("%i/%i", pPlayer:GetGoldenAgeProgressMeter(), pPlayer:GetGoldenAgeProgressThreshold());
			if gData.Players[iPlayerID].HadGoldenAge then
				strGoldenAgeStr = "OVER";
			else
				strGoldenAgeStr = string.format("%i/%i", gData.Players[iPlayerID].TotalGoldenAgePoints, gData.Players[iPlayerID].TPGoldenAgeCost);
			end
		end
		strGoldenAgeStr = "[COLOR:255:255:255:255]" .. strGoldenAgeStr .. "[/COLOR][ICON_GOLDEN_AGE]";
		Controls.GoldenAgeString:SetText(strGoldenAgeStr);
		-- new
		if gData.Players[iPlayerID].Morale >= 0 then
			Controls.MoraleString:SetText("[COLOR_POSITIVE_TEXT]" .. gData.Players[iPlayerID].Morale .. "%[ENDCOLOR][ICON_ITG_MORALE]")
		else
			Controls.MoraleString:SetText("[COLOR_WARNING_TEXT]" .. gData.Players[iPlayerID].Morale .. "%[ENDCOLOR][ICON_ITG_MORALE]")
		end
		Controls.FestivalString:SetText(gData.Players[iPlayerID].TPFestivalChance .. "%[ICON_ITG_FESTIVAL]")

		if gData.Players[iPlayerID].TPMissionaryGrowth then
			Controls.MissionaryString1:SetText("+" .. gData.Players[iPlayerID].TPMissionaryGrowth)
			Controls.MissionaryString3:SetText(gData.Players[iPlayerID].MissionaryGrowth)
			Controls.MissionaryString4:SetText(gData.Players[iPlayerID].TPMissionaryTreshold)
			Controls.MissionaryString:SetHide(false)
		else
			Controls.MissionaryString:SetHide(true)
		end

		local bHideHeresy = true;
		if MapModData.DBScenario.HeresyOn then
			if gData.Players[iPlayerID].TPHeresy > 0 then
				bHideHeresy = false;
				Controls.HeresyString:SetText(gData.Players[iPlayerID].TPHeresy .. "%[ICON_PROPHET]")
			end
		end
		Controls.HeresyString:SetHide(bHideHeresy)

		if gData.Players[iPlayerID].OccupyLimit then
			local lTlimit = gData.Players[iPlayerID].OccupyLimit;
			for iCity in Players[iPlayerID]:Cities() do
				if iCity:IsOccupied() then
					lTlimit = lTlimit - 1;
				end
			end
			if lTlimit > 0 then
				Controls.OccupyString:SetText("[COLOR_POSITIVE_TEXT]" .. lTlimit .. "[ICON_OCCUPIED]")
			else
				Controls.OccupyString:SetText("[COLOR_NEGATIVE_TEXT]" .. lTlimit .. "[ICON_OCCUPIED]")
			end
			Controls.OccupyString:SetHide(false)
		else
			Controls.OccupyString:SetHide(true)
		end

		if gData.Players[iPlayerID].NumPirateMods then
			Controls.PirateString:SetText(gData.Players[iPlayerID].NumPirateMods .. "[ICON_PIRATE]")
			Controls.PirateString:SetHide(false)
		else
			Controls.PirateString:SetHide(true)
		end
		

		if gData.Players[iPlayerID].TPSuppFishBoats > 0 then
			Controls.FishString:SetText( pPlayer:GetUnitClassCountPlusMaking(GameInfoTypes.UNITCLASS_WORKBOAT) .. "/" .. gData.Players[iPlayerID].TPSuppFishBoats .. "[ICON_RES_FISH]");
		else
			Controls.FishString:SetText(strFishAva);
		end
	end

	Controls.TopPanelScrollPanel:SetSizeX(tWt - 20 - Controls.TopPanelLeft:GetSizeX())
	Controls.TopPanelInfoStack:SetSizeX(tWt - 20 - Controls.TopPanelLeft:GetSizeX())
	Controls.TopPanelInfoStack:CalculateSize();
	Controls.TopPanelInfoStack:ReprocessAnchoring();
	Controls.TopPanelScrollPanel:CalculateInternalSize()
end



function OnTopPanelDirty()
	UpdateData();
end

-------------------------------------------------
-------------------------------------------------
function OnCivilopedia()	
	-- In City View, return to main game
	--if (UI.GetHeadSelectedCity() ~= nil) then
		--Events.SerialEventExitCityScreen();
	--end
	--
	-- opens the Civilopedia without changing its current state
	Events.SearchForPediaEntry("");
end
Controls.CivilopediaButton:RegisterCallback( Mouse.eLClick, OnCivilopedia );

-------------------------------------------------
-------------------------------------------------
function OnMenu()
	
	-- In City View, return to main game
	if (UI.GetHeadSelectedCity() ~= nil) then
		Events.SerialEventExitCityScreen();
		--UI.SetInterfaceMode(InterfaceModeTypes.INTERFACEMODE_SELECTION);
	-- In Main View, open Menu Popup
	else
	    UIManager:QueuePopup( LookUpControl( "/InGame/GameMenu" ), PopupPriority.InGameMenu );
	end
end
Controls.MenuButton:RegisterCallback( Mouse.eLClick, OnMenu );

-------------------------------------------------
-- TOOLTIPS
-------------------------------------------------


-- Tooltip init
function DoInitTooltips()
	Controls.ScienceString1:SetToolTipCallback( ScienceTipHandler );
	Controls.ScienceString2:SetToolTipCallback( ScienceTipHandler );
	Controls.ScienceString3:SetToolTipCallback( ScienceTipHandler );
	Controls.ScienceString4:SetToolTipCallback( ScienceTipHandler );
	Controls.GoldPerTurn:SetToolTipCallback( GoldTipHandler );
	Controls.InvestCurrency:SetToolTipCallback( CurrencyTipHandler );
	
	Controls.HappinessString1:SetToolTipCallback( HappinessTipHandler );
	Controls.HappinessString2:SetToolTipCallback( HappinessTipHandler );
	Controls.HappinessString3:SetToolTipCallback( HappinessTipHandler );
	Controls.HappinessString4:SetToolTipCallback( HappinessTipHandler );

	Controls.MissionaryString1:SetToolTipCallback( MissionaryTipHandler );
	Controls.MissionaryString2:SetToolTipCallback( MissionaryTipHandler );
	Controls.MissionaryString3:SetToolTipCallback( MissionaryTipHandler );
	Controls.MissionaryString4:SetToolTipCallback( MissionaryTipHandler );
	

	Controls.GoldenAgeString:SetToolTipCallback( GoldenAgeTipHandler );
	Controls.MoraleString:SetToolTipCallback( MoraleTipHandler );
	Controls.FestivalString:SetToolTipCallback( FestivalTipHandler );
	Controls.HeresyString:SetToolTipCallback( HeresyTipHandler );
	Controls.OccupyString:SetToolTipCallback( OccupyTipHandler );
	Controls.PirateString:SetToolTipCallback( PirateTipHandler ) ;
	Controls.CultureString1:SetToolTipCallback( CultureTipHandler );
	Controls.CultureString2:SetToolTipCallback( CultureTipHandler );
	Controls.CultureString3:SetToolTipCallback( CultureTipHandler );
	Controls.CultureString4:SetToolTipCallback( CultureTipHandler );
--	Controls.TourismString:SetToolTipCallback( TourismTipHandler );
	Controls.FaithString1:SetToolTipCallback( FaithTipHandler );
	Controls.FaithString2:SetToolTipCallback( FaithTipHandler );
	Controls.FaithString3:SetToolTipCallback( FaithTipHandler );
	Controls.FaithString4:SetToolTipCallback( FaithTipHandler );
	Controls.SettlerString1:SetToolTipCallback( SettlerTipHandler );
	Controls.SettlerString2:SetToolTipCallback( SettlerTipHandler );
	Controls.SettlerString3:SetToolTipCallback( SettlerTipHandler );
	Controls.SettlerString4:SetToolTipCallback( SettlerTipHandler );
	Controls.ArchaeologyString1:SetToolTipCallback( ArchaeologyTipHandler );
	Controls.ArchaeologyString2:SetToolTipCallback( ArchaeologyTipHandler );
	Controls.ArchaeologyString3:SetToolTipCallback( ArchaeologyTipHandler );
	Controls.ArchaeologyString4:SetToolTipCallback( ArchaeologyTipHandler );
	Controls.AuthorityString2:SetToolTipCallback( AuthorityTipHandler );
	Controls.AuthorityString3:SetToolTipCallback( AuthorityTipHandler );
	Controls.AuthorityString4:SetToolTipCallback( AuthorityTipHandler );
	Controls.MeritocracyString2:SetToolTipCallback( MeritocracyTipHandler );
	Controls.MeritocracyString3:SetToolTipCallback( MeritocracyTipHandler );
	Controls.MeritocracyString4:SetToolTipCallback( MeritocracyTipHandler );
	
	Controls.GreatPeopleString1:SetToolTipCallback( GPTipHandler );
	Controls.GreatPeopleString2:SetToolTipCallback( GPTipHandler );
	Controls.GreatPeopleString3:SetToolTipCallback( GPTipHandler );
	Controls.GreatPeopleString4:SetToolTipCallback( GPTipHandler );
	
	Controls.UnitSupplyString2:SetToolTipCallback( USupplyTipHandler );
	Controls.UnitSupplyString3:SetToolTipCallback( USupplyTipHandler );
	Controls.UnitSupplyString4:SetToolTipCallback( USupplyTipHandler );
	Controls.ColonyString1:SetToolTipCallback( ColonyTipHandler );
	Controls.ColonyString2:SetToolTipCallback( ColonyTipHandler );
	Controls.ColonyString3:SetToolTipCallback( ColonyTipHandler );
	Controls.ColonyString4:SetToolTipCallback( ColonyTipHandler );
	Controls.MercenaryString2:SetToolTipCallback( MercenaryTipHandler );
	Controls.MercenaryString3:SetToolTipCallback( MercenaryTipHandler );
	Controls.MercenaryString4:SetToolTipCallback( MercenaryTipHandler );
	Controls.LegalString:SetToolTipCallback( LegalTipHandler );
	Controls.GreatAristString3:SetToolTipCallback( GreatArtistTipHandler );
	Controls.GreatAristString2:SetToolTipCallback( GreatArtistTipHandler );
	Controls.GreatAristString4:SetToolTipCallback( GreatArtistTipHandler );
	Controls.NumGreatWorks:SetToolTipCallback( GreatWorksTipHandler );
	Controls.WonderRaceString2:SetToolTipCallback( WonderRaceTipHandler );
	Controls.WonderRaceString3:SetToolTipCallback( WonderRaceTipHandler );
	Controls.WonderRaceString4:SetToolTipCallback( WonderRaceTipHandler );
	Controls.CultureIndicator:SetToolTipCallback( CultureIndicatorTipHandler );
	Controls.HorseString:SetToolTipCallback( ResourcesTipHandler );
	Controls.OlympicGamesString:SetToolTipCallback( OlympicTipHandler) ;
	Controls.FishString:SetToolTipCallback( FishSupTipHandler );
--	Controls.InternationalTradeRoutes:SetToolTipCallback( InternationalTradeRoutesTipHandler );]]
end

-- Science Tooltip
local tipControlTable = {};
TTManager:GetTypeControlTable( "TooltipTypeTopPanel", tipControlTable );
function FishSupTipHandler( control )
	CallBasicTopPanelTTHandler( control, "FishSup")
end

function WonderRaceTipHandler( control )
	CallBasicTopPanelTTHandler( control, "WonderRace")
end

function CultureIndicatorTipHandler( control)
	CallBasicTopPanelTTHandler( control, "CultureIndicator")
end

function LegalTipHandler( control )
	CallBasicTopPanelTTHandler( control, "Legal")
end

function GreatWorksTipHandler( control )
	CallBasicTopPanelTTHandler( control, "GreatWork")
end

function MercenaryTipHandler( control)
	CallBasicTopPanelTTHandler( control, "Mercenary")
end

function ColonyTipHandler( control )
	CallBasicTopPanelTTHandler( control, "Colony")
end

function USupplyTipHandler(control)
	CallBasicTopPanelTTHandler( control, "USupply")
end

function GreatArtistTipHandler( control )
	CallBasicTopPanelTTHandler( control, "GreatArtist")
end

function ScienceTipHandler( control )
	CallBasicTopPanelTTHandler( control, "Science")
end

-- Gold Tooltip
function GoldTipHandler( control )
	CallBasicTopPanelTTHandler( control, "Gold")
end

function CurrencyTipHandler(control)
	CallBasicTopPanelTTHandler( control, "Currency")
end

-- Happiness Tooltip
function HappinessTipHandler( control )
	CallBasicTopPanelTTHandler( control, "Happiness")
end

function MissionaryTipHandler( control )
	CallBasicTopPanelTTHandler( control, "Missionary")
end

-- Golden Age Tooltip
function GoldenAgeTipHandler( control )
	CallBasicTopPanelTTHandler( control, "GoldenAge")
end

function FestivalTipHandler( control)
	CallBasicTopPanelTTHandler( control, "Festival")
end

function CallBasicTopPanelTTHandler( control, tstring)
	LuaEvents.ITGSG_RequestTopPanelTooltip(tstring);
	local strText = MapModData.ITGSG;
	MapModData.ITGSG = nil;

	tipControlTable.TooltipLabel:SetText( strText or "" );
	tipControlTable.TopPanelMouseover:SetHide(false);
    -- Autosize tooltip
    tipControlTable.TopPanelMouseover:DoAutoSize();	
end

function HeresyTipHandler( control)
	CallBasicTopPanelTTHandler( control, "Heresy")
end

function OccupyTipHandler(control)
	CallBasicTopPanelTTHandler( control, "OccupyLimit")
end

function PirateTipHandler( control)
	CallBasicTopPanelTTHandler( control, "Pirate")
end

function MoraleTipHandler( control)
	CallBasicTopPanelTTHandler( control, "Morale")
end

-- Culture Tooltip
function CultureTipHandler( control )
	CallBasicTopPanelTTHandler( control, "Culture")
end

-- Tourism Tooltip
function TourismTipHandler( control )
--obsolete
end

function GPTipHandler( control )
	CallBasicTopPanelTTHandler( control, "GreatPeople")
end

function AuthorityTipHandler(control)
	CallBasicTopPanelTTHandler( control, "Authority")
end

function MeritocracyTipHandler(control)
	CallBasicTopPanelTTHandler( control, "Meritocracy")
end

function ArchaeologyTipHandler(control)
	CallBasicTopPanelTTHandler( control, "Archaeology")
end

function SettlerTipHandler( control )
	CallBasicTopPanelTTHandler( control, "Settler")
end

-- FaithTooltip
function FaithTipHandler( control )
	CallBasicTopPanelTTHandler( control, "Faith")
end

function OlympicTipHandler( control )
	CallBasicTopPanelTTHandler( control, "OlympicGames")
end
-- Resources Tooltip
function ResourcesTipHandler( control )
	CallBasicTopPanelTTHandler( control, "Resource")
end

-- International Trade Route Tooltip
function InternationalTradeRoutesTipHandler( control )
--obsolete
end

-------------------------------------------------
-- On Top Panel mouseover exited
-------------------------------------------------
--function HelpClose()
	---- Hide the help text box
	--Controls.HelpTextBox:SetHide( true );
--end

-- Register Events
Events.SerialEventGameDataDirty.Add(OnTopPanelDirty);
Events.SerialEventTurnTimerDirty.Add(OnTopPanelDirty);
Events.SerialEventCityInfoDirty.Add(OnTopPanelDirty);
function ITGSGTopPanelCall(bInitial)
	OnTopPanelDirty()
	if bInitial then
		ShuffleTheStack(nil, true)
	end
end
LuaEvents.ITGSGTopPanelDirty.Add(ITGSGTopPanelCall);

--Register Links to overviews
function OnEconomicClicked()
	LuaEvents.ITG_CallGoldTPClicked();
end
function OnExchangeClicked()
	Events.SerialEventGameMessagePopup( { Type = ButtonPopupTypes.BUTTONPOPUP_ECONOMIC_OVERVIEW, Data2 = 3} );
end
function OnGreatPeopleClicked()
	LuaEvents.ITG_CallWonderOverview(2);
end
function OnTechClicked()
	Events.SerialEventGameMessagePopup( { Type = ButtonPopupTypes.BUTTONPOPUP_TECH_TREE, Data2 = -1} );
end

function OnRelClicked()
	Events.SerialEventGameMessagePopup( { Type = ButtonPopupTypes.BUTTONPOPUP_RELIGION_OVERVIEW, Data2 = -1} );
end

function OnArchaClicked()
	LuaEvents.ITG_CallSeeNextArchaeologySite()
end

function OnHapClicked()
	LuaEvents.ITG_CallGovermentOverview()
end

function OnMeritClicked()
	LuaEvents.ITG_CallGovermentOverview(3)
end
Controls.ScienceString1:RegisterCallback( Mouse.eLClick, OnTechClicked );
Controls.ScienceString2:RegisterCallback( Mouse.eLClick, OnTechClicked );
Controls.ScienceString3:RegisterCallback( Mouse.eLClick, OnTechClicked );
Controls.ScienceString4:RegisterCallback( Mouse.eLClick, OnTechClicked );
Controls.GoldPerTurn:RegisterCallback( Mouse.eLClick, OnEconomicClicked );
Controls.InvestCurrency:RegisterCallback( Mouse.eLClick, OnExchangeClicked );
Controls.GreatPeopleString1:RegisterCallback( Mouse.eLClick, OnGreatPeopleClicked );
Controls.GreatPeopleString2:RegisterCallback( Mouse.eLClick, OnGreatPeopleClicked );
Controls.GreatPeopleString3:RegisterCallback( Mouse.eLClick, OnGreatPeopleClicked );
Controls.GreatPeopleString4:RegisterCallback( Mouse.eLClick, OnGreatPeopleClicked );
--Controls.CultureString:RegisterCallback( Mouse.eLClick, OnTechClicked );
--Controls.CultureString1:RegisterCallback( Mouse.eLClick, OnTechClicked );
--Controls.CultureString2:RegisterCallback( Mouse.eLClick, OnTechClicked );
--Controls.CultureString3:RegisterCallback( Mouse.eLClick, OnTechClicked );
--Controls.CultureString4:RegisterCallback( Mouse.eLClick, OnTechClicked );
Controls.FaithString1:RegisterCallback( Mouse.eLClick, OnRelClicked );
Controls.FaithString2:RegisterCallback( Mouse.eLClick, OnRelClicked );
Controls.FaithString3:RegisterCallback( Mouse.eLClick, OnRelClicked );
Controls.FaithString4:RegisterCallback( Mouse.eLClick, OnRelClicked );
Controls.ArchaeologyString1:RegisterCallback( Mouse.eLClick, OnArchaClicked );
Controls.ArchaeologyString2:RegisterCallback( Mouse.eLClick, OnArchaClicked );
Controls.ArchaeologyString3:RegisterCallback( Mouse.eLClick, OnArchaClicked );
Controls.ArchaeologyString4:RegisterCallback( Mouse.eLClick, OnArchaClicked );

Controls.HappinessString1:RegisterCallback( Mouse.eLClick, OnHapClicked );
Controls.HappinessString2:RegisterCallback( Mouse.eLClick, OnHapClicked );
Controls.HappinessString3:RegisterCallback( Mouse.eLClick, OnHapClicked );
Controls.HappinessString4:RegisterCallback( Mouse.eLClick, OnHapClicked );
Controls.LegalString:RegisterCallback( Mouse.eLClick, OnHapClicked );
Controls.MeritocracyString2:RegisterCallback( Mouse.eLClick, OnMeritClicked );
Controls.MeritocracyString3:RegisterCallback( Mouse.eLClick, OnMeritClicked );
Controls.MeritocracyString4:RegisterCallback( Mouse.eLClick, OnMeritClicked );
--NumGreatWorks
--WonderRace
--ColonyString2/3/4
--GreatAristString
--HorseString
--UnitSupplyString
--CultureIndicator

local helpIDMovingTab = {
Controls.SciencePerTurn,
Controls.GoldPerTurn,
Controls.InvestCurrency,
Controls.CultureString,
Controls.FaithString,
Controls.HappinessStack,
Controls.LegalString,
Controls.NumGreatWorks,
Controls.WonderRace,
Controls.ColonyString,
Controls.GreatAristString,
Controls.CultureIndicator,
Controls.HorseString,
Controls.FishString,
Controls.GoldenAgeString,
Controls.GreatPeopleString,
Controls.UnitSupplyString,
Controls.MoraleString,
Controls.SettlerString,
Controls.FestivalString,
Controls.MercenaryString,
Controls.OlympicGamesString,
Controls.ArchaeologyString,
Controls.AuthorityString,
Controls.HeresyString,
Controls.PirateString,
Controls.MissionaryString,
Controls.MeritocracyString,
Controls.OccupyString
} -- add always to last position

local helpPreMadePositionTab = {
{Controls.SciencePerTurn, 1},
{Controls.GoldPerTurn, 2},
{Controls.InvestCurrency,3      },
{Controls.CultureString,  4     },
{Controls.FaithString,     5    },
{Controls.MissionaryString,     27    },
{Controls.GreatPeopleString,  16  },
{Controls.HappinessStack,   6   },
{Controls.LegalString,       7  },
{Controls.NumGreatWorks,    8   },
{Controls.WonderRace,      9    },
{Controls.ColonyString,    10    },
{Controls.GreatAristString, 11   },
{Controls.CultureIndicator, 12   },
{Controls.FestivalString, 20},
{Controls.MoraleString, 18},
{Controls.SettlerString, 19},
{Controls.HorseString, 13 },
{Controls.FishString, 14   },
{Controls.GoldenAgeString, 15 },
{Controls.UnitSupplyString, 17 },
{Controls.MercenaryString, 21},
{Controls.OlympicGamesString, 22},
{Controls.ArchaeologyString, 23},
{Controls.AuthorityString, 24},
{Controls.MeritocracyString, 28},
{Controls.HeresyString, 25},
{Controls.PirateString, 26},
{Controls.OccupyString, 29},
}

function ShuffleTheStack(tFirst, bInit)
	LuaEvents.ITGSG_RequestGT();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	for i, control in pairs(helpIDMovingTab) do
		control:ChangeParent(Controls.TopPanelLeft)
	end
	Controls.DummyAtTheEnd:ChangeParent(Controls.TopPanelLeft)
	local hTab = {}
	local bFirstIsLast = false;
	local gPlayer = Game.GetActivePlayer();
	if tFirst then
		if gData.Players[gPlayer] and gData.Players[gPlayer].TopPanelQueueStack then
			if gData.Players[gPlayer].TopPanelQueueStack[1] == tFirst then
				bFirstIsLast = true;
			end
		elseif tFirst == helpPreMadePositionTab[1][2] then
			bFirstIsLast = true;
		end
		if not bFirstIsLast then
			table.insert(hTab, tFirst)
			helpIDMovingTab[tFirst]:ChangeParent(Controls.TopPanelInfoStack)
		end
	end
	if gData.Players[gPlayer] and gData.Players[gPlayer].TopPanelQueueStack then
		for i, cid in pairs(gData.Players[gPlayer].TopPanelQueueStack) do
			if cid ~= tFirst then
				helpIDMovingTab[cid]:ChangeParent(Controls.TopPanelInfoStack)
				table.insert(hTab, cid)
			end
		end
	else
		for i, controld in pairs(helpPreMadePositionTab) do
			if controld[2] ~= tFirst then
				controld[1]:ChangeParent(Controls.TopPanelInfoStack)
				table.insert(hTab, controld[2])
			end
		end
	end
	if bFirstIsLast then
		helpIDMovingTab[tFirst]:ChangeParent(Controls.TopPanelInfoStack)
		table.insert(hTab, tFirst)
	end
	Controls.DummyAtTheEnd:ChangeParent(Controls.TopPanelInfoStack)
	if not bInit then
		MapModData.ITGSG = hTab;
		LuaEvents.ITGSG_CallTheTopPanelQueue();
	end
end

Controls.ScienceString1:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(1) end );
Controls.ScienceString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(1) end );
Controls.ScienceString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(1) end );
Controls.ScienceString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(1) end );
Controls.GoldPerTurn:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(2) end );
Controls.InvestCurrency:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(3) end );
Controls.CultureString1:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(4) end );
Controls.CultureString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(4) end );
Controls.CultureString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(4) end );
Controls.CultureString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(4) end );
Controls.FaithString1:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(5) end );
Controls.FaithString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(5) end );
Controls.FaithString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(5) end );
Controls.FaithString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(5) end );
Controls.HappinessString1:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(6) end );
Controls.HappinessString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(6) end );
Controls.HappinessString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(6) end );
Controls.HappinessString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(6) end );
Controls.LegalString:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(7) end );
Controls.NumGreatWorks:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(8) end );
Controls.WonderRaceString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(9) end );
Controls.WonderRaceString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(9) end );
Controls.WonderRaceString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(9) end );
Controls.ColonyString1:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(10) end );
Controls.ColonyString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(10) end );
Controls.ColonyString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(10) end );
Controls.ColonyString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(10) end );
Controls.GreatAristString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(11) end );
Controls.GreatAristString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(11) end );
Controls.GreatAristString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(11) end );
Controls.CultureIndicator:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(12) end );
Controls.HorseString:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(13) end );
Controls.FishString:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(14) end );
Controls.GoldenAgeString:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(15) end );
Controls.GreatPeopleString1:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(16) end );
Controls.GreatPeopleString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(16) end );
Controls.GreatPeopleString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(16) end );
Controls.GreatPeopleString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(16) end );
Controls.UnitSupplyString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(17) end );
Controls.UnitSupplyString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(17) end );
Controls.UnitSupplyString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(17) end );
Controls.MoraleString:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(18) end );
Controls.SettlerString1:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(19) end );
Controls.SettlerString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(19) end );
Controls.SettlerString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(19) end );
Controls.SettlerString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(19) end );
Controls.FestivalString:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(20) end );
Controls.MercenaryString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(21) end );
Controls.MercenaryString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(21) end );
Controls.MercenaryString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(21) end );
Controls.OlympicGamesString:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(22) end );
Controls.ArchaeologyString1:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(23) end );
Controls.ArchaeologyString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(23) end );
Controls.ArchaeologyString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(23) end );
Controls.ArchaeologyString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(23) end );
Controls.AuthorityString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(24) end );
Controls.AuthorityString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(24) end );
Controls.AuthorityString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(24) end );
Controls.HeresyString:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(25) end );
Controls.PirateString:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(26) end );
Controls.MissionaryString1:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(27) end );
Controls.MissionaryString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(27) end );
Controls.MissionaryString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(27) end );
Controls.MissionaryString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(27) end );
Controls.MeritocracyString2:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(28) end );
Controls.MeritocracyString3:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(28) end );
Controls.MeritocracyString4:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(28) end );
Controls.OccupyString:RegisterCallback( Mouse.eRClick, function() ShuffleTheStack(29) end );


function OpenGovermentOverview()
	LuaEvents.ITG_CallGovermentOverview()
end
Controls.AnarchyLegalLeft:RegisterCallback( Mouse.eLClick, OpenGovermentOverview)
Controls.AnarchyMain:RegisterCallback( Mouse.eLClick, OpenGovermentOverview)
-- Update data at initialization
UpdateData();
DoInitTooltips();