-------------------------------------------------
-- ITG CustomsHouse
-------------------------------------------------
include( "IconSupport" );
include( "InstanceManager" );

-------------------------------------------------
-------------------------------------------------
local ltBlue = {19/255,32/255,46/255,120/255};
local dkBlue = {12/255,22/255,30/255,120/255};
local g_MoC;
local bTickTock = false;
local g_CounselorList = GenerationalInstanceManager:new( "CounselorInstance", "Base", Controls.CounselorStack);
local MoEducationT = {
{Name = "Manual", Type = -1, Help = "Support management is inactive."},
{Name = "Always Random", Type = 1, Help = "If you have at least 125 [ICON_GOLD] Gold at the turn start then academies (in undamaged cities) will be supported (random branch for 100 [ICON_GOLD] Gold)."},
{Name = "Keystone or Random", Type = 2, Help = "If you have at least 125 [ICON_GOLD] Gold at the turn start then academies (in undamaged cities) will be supported in following manner: branches needing one point for next keystone are gonna be prioritized, otherwise random branch for 100 [ICON_GOLD] Gold will be supported."},
{Name = "Always Highest", Type = 3, Help = "If you have at least 125 [ICON_GOLD] Gold at the turn start then academies (in undamaged cities) will be supported (branch with the highest amount of points)."},
}
local LuxuryTradeRegT = {
{Name = "Manual", Type = -1, Help = "Offer management is inactive."},
{Name = "Everything", Type = 1, Help = "Every single resource available will be put up for sale."},
{Name = "Everything 200+", Type = 9, Help = "Every single resource available that is priced at least 200 [ICON_GOLD] Gold will be put up for sale."},
{Name = "Keep a copy", Type = 4, Help = "Offer management will leave one copy of every luxury, rest will be put up for sale."},
{Name = "Keep a copy 200+", Type = 3, Help = "Offer management will leave one copy of every luxury, rest will be put up for sale. Resources with price lesser than 200 [ICON_GOLD] Gold will be ignored."},
{Name = "Low-key", Type = 5, Help = "Offer management will leave one copy of every luxury, and will put only one copy of certain luxury up for sale at a time."},
{Name = "Really low-key", Type = 6, Help = "Offer management will leave one copy of every luxury, and will put only one copy of certain luxury up for sale at a time. Resources with price lesser than 200 [ICON_GOLD] Gold will be ignored."},
{Name = "Variety", Type = 8, Help = "Offer management will put one copy of each luxury up for sale at a time."},
{Name = "Variety 200+", Type = 7, Help = "Offer management will put one copy of each luxury up for sale at a time. Resources with price lesser than 200 [ICON_GOLD] Gold will be ignored."},
}
local TaxCollectorT = {
{Name = "Manual", Type = -1, Help = "Tax collecting is inactive."},
{Name = "Below Anchor", Type = 1, Help = "Bully any City-State with [ICON_INFLUENCE] Influence lower or equal its anchor."},
{Name = "Non-friendly", Type = 2, Help = "Bully any non-friendly City-State."},
{Name = "Non-allied", Type = 3, Help = "Bully any non-allied City-State."},
}
		
function GetIDFromType(tab, ah)
	for hh, bb in pairs(tab) do
		if bb.Type == ah then
			return hh;
		end
	end
	return 1;
end

function OnMasterOfConstruction()
	local iPlayer = Game.GetActivePlayer();
	LuaEvents.ITGSG_RequestMasterOfContruction(Game.GetActivePlayer());
	g_MoC = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not g_MoC then
		return
	end
	Controls.MasterOConstructionStack:DestroyAllChildren(); 
	for iClass, val in pairs(g_MoC) do
		local iBuilding = Players[iPlayer]:GetPlayerUniqueBuilding(iClass);
		if iBuilding ~= -1 then
			local instance = {};
			ContextPtr:BuildInstanceForControl( "MoCBuildingInstance", instance, Controls.MasterOConstructionStack );
			instance.NameText:SetText(Locale.Lookup(GameInfo.Buildings[iBuilding].Description))
			IconHookup( GameInfo.Buildings[iBuilding].PortraitIndex, 64, GameInfo.Buildings[iBuilding].IconAtlas, instance.Portrait );				
			instance.CheckMoC:SetCheck(val);
			instance.CheckMoC:RegisterCheckHandler( function(bCheck)
				g_MoC[iClass] = not g_MoC[iClass];
			end);
		end
	end
    Controls.MasterOConstructionStack:CalculateSize();
    Controls.MasterOConstructionStack:ReprocessAnchoring();
    Controls.MasterOConstructionScrollPanel:CalculateInternalSize();
	Controls.MasterOConstruction:SetHide(false);
end

function OnConfirmMoC()
	LuaEvents.ITGHumanChooseAIConstructionManagement(Game.GetActivePlayer(), g_MoC)
	Controls.MasterOConstruction:SetHide(true);
	UpdateScreen()
end
Controls.ConfirmMoC:RegisterCallback( Mouse.eLClick, OnConfirmMoC )

function OnCancelMoC()
	Controls.MasterOConstruction:SetHide(true);
end
Controls.CancelMoC:RegisterCallback( Mouse.eLClick, OnCancelMoC )

function OnTaxCollector(v1, v2)
	LuaEvents.ITGHumanPlayerChooseAIBullying(Game.GetActivePlayer(), TaxCollectorT[v1].Type);
	UpdateScreen()
end

function OnLuxuryTradeRegulation(v1, v2)
	LuaEvents.ITGHumanPlayerChooseAIAuctionManagement(Game.GetActivePlayer(), LuxuryTradeRegT[v1].Type);
	UpdateScreen()
end

function OnMasterOfEducation(v1, v2)
	LuaEvents.ITGHumanPlayerChooseAIAcademyManagement(Game.GetActivePlayer(), MoEducationT[v1].Type);
	UpdateScreen()
end

function OnGrandArchitect()
	LuaEvents.ITGSG_PlayerChangeGASetting();
	UpdateScreen()
end

function OnMoW()
	LuaEvents.ITGSG_PlayerChangeMOWSettings();
	UpdateScreen()
end

function OnAutomaticMissionaries()
	LuaEvents.ITGSG_SetAutomaticMissionaries()
	UpdateScreen()
end

function OnAutomaticFaithPurchase()
	LuaEvents.ITGSG_SetAutomaticFaithPurchase()
	UpdateScreen()
end

function OnMasterOfArms()
	LuaEvents.ITGSG_HumanChangeVeteranSettings();
	UpdateScreen()
end

function OnMasterOfRailways()
	LuaEvents.ITGHumanPlayerChooseAIRailroad()
	UpdateScreen()
end

function OnAutomaticBombardment()
	LuaEvents.ITGHumanPlayerChooseAIBombardment()
	UpdateScreen()
end

function InitialCounselorInstance(itemInstance)
	itemInstance.ActiveButton:SetHide(true)
	itemInstance.BoxPD:SetHide(true);
end

function FinalCounselorInstance(itemInstance)
	if(bTickTock == false) then
		itemInstance.Base:SetColorVal(unpack(ltBlue));
	else
		itemInstance.Base:SetColorVal(unpack(dkBlue));
	end
	bTickTock = not bTickTock;
	local buttonWidth, buttonHeight = itemInstance.Base:GetSizeVal();
	local descWidth, descHeight = itemInstance.Help:GetSizeVal();
	local newHeight = descHeight + 50;	
	itemInstance.Base:SetSizeVal(buttonWidth, newHeight);
end

function SetCounselorButtonBasic(itemInstance, bool)
	itemInstance.ActiveButton:SetHide(false)
	if bool then
		itemInstance.ActiveLabel:SetText("[COLOR_POSITIVE_TEXT]ON[ENDCOLOR]")
	else
		itemInstance.ActiveLabel:SetText("[COLOR_NEGATIVE_TEXT]OFF[ENDCOLOR]")
	end
end

function AddToPullDown(tPulldown, text, tooltip, v1, v2)
	local controlTable = {};
	tPulldown:BuildEntry( "InstanceOne", controlTable );
	controlTable.Button:SetVoids(v1, v2);
	controlTable.Button:SetText(text);
	controlTable.Button:SetToolTipString(tooltip);
end

function UpdateScreen()
	LuaEvents.ITGSG_RequestGT();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	local iTeam = Game.GetActiveTeam();
	local iPlayer = Game.GetActivePlayer()
	if not gData.Players[iPlayer] then
		return;
	end
	bTickTock = false;
	g_CounselorList:ResetInstances();

	--Grand Architect
	local itemInstance = g_CounselorList:GetInstance();
	InitialCounselorInstance(itemInstance)
	itemInstance.Name:SetText("Grand Architect")
	itemInstance.Help:SetText("Automatic management of Palaces' commisions.")
	SetCounselorButtonBasic(itemInstance, gData.Players[iPlayer].GrandArchitect )
	itemInstance.ActiveButton:RegisterCallback( Mouse.eLClick, OnGrandArchitect )
	FinalCounselorInstance(itemInstance)

	--Master of Army
	local itemInstance = g_CounselorList:GetInstance();
	InitialCounselorInstance(itemInstance)
	itemInstance.Name:SetText("Master of Arms")
	itemInstance.Help:SetText("Automatic management of Veterancy promotions.")
	SetCounselorButtonBasic(itemInstance, gData.Players[iPlayer].MasterOfArmy )
	itemInstance.ActiveButton:RegisterCallback( Mouse.eLClick, OnMasterOfArms )
	FinalCounselorInstance(itemInstance)

	--Master of Construction
	local itemInstance = g_CounselorList:GetInstance();
	InitialCounselorInstance(itemInstance)
	itemInstance.Name:SetText("Master of Construction")
	local mocText = "Allows you to define a set of buildings that will be automatically built or rebuilt in all Cities (if City has no production order or is during non-competive process)."
	local bAnyMoc = false;
	for iClass, val in pairs(gData.Players[iPlayer].AIConstructionManager or {}) do
		if val then
			local iBuilding = Players[iPlayer]:GetPlayerUniqueBuilding(iClass);
			if iBuilding ~= -1 then
				if not bAnyMoc then
					mocText = mocText .. " Currently chosen buildings:"
					bAnyMoc = true;
				end
				mocText = mocText .. "[NEWLINE][ICON_BULLET][COLOR_POSITIVE_TEXT]" .. Locale.Lookup(GameInfo.Buildings[iBuilding].Description) .. "[ENDCOLOR]"
			end
		end
	end
	if not bAnyMoc then
		mocText = mocText .. " Currently no building is selected.";
	end
	itemInstance.Help:SetText(mocText)
	itemInstance.ActiveButton:SetHide(false)
	itemInstance.ActiveLabel:SetText("ADJUST")
	itemInstance.ActiveButton:RegisterCallback( Mouse.eLClick, OnMasterOfConstruction )
	FinalCounselorInstance(itemInstance)

	--Master of Defense

--	secretary of defense
	local itemInstance = g_CounselorList:GetInstance();
	InitialCounselorInstance(itemInstance)
	itemInstance.Name:SetText("Secretary of Defense")
	itemInstance.Help:SetText("Approach towards automatic City bombardment (happens only at the turn start).")
	SetCounselorButtonBasic(itemInstance, gData.Players[iPlayer].AutomaticBombardment)
	itemInstance.ActiveButton:RegisterCallback( Mouse.eLClick, OnAutomaticBombardment )
	FinalCounselorInstance(itemInstance)

	--Master of Education
	if (gData.Globals.ScenarioType ~= 5) then
		local itemInstance = g_CounselorList:GetInstance();
		InitialCounselorInstance(itemInstance)
		itemInstance.Name:SetText("Master of Education")
		itemInstance.Help:SetText("Approach towards automatic supporting of your [ICON_ITG_ACADEMY_1] Academies.")
		itemInstance.BasicSupportPD:ClearEntries();
		for a, b in pairs(MoEducationT) do
			AddToPullDown(itemInstance.BasicSupportPD, b.Name, b.Help, a, 1)
		end
		local cOption = GetIDFromType(MoEducationT, gData.Players[iPlayer].AIAcademyManager);
		itemInstance.BasicSupportPD:GetButton():LocalizeAndSetText(MoEducationT[cOption].Name);		
		itemInstance.BasicSupportPD:GetButton():SetToolTipString(MoEducationT[cOption].Help);
		itemInstance.BasicSupportPD:CalculateInternals();
		itemInstance.BasicSupportPD:RegisterSelectionCallback(OnMasterOfEducation); 
		itemInstance.BoxPD:SetHide(false)
		FinalCounselorInstance(itemInstance)
	end

	--Luxury Resources
	local itemInstance = g_CounselorList:GetInstance();
	InitialCounselorInstance(itemInstance)
	itemInstance.Name:SetText("Luxury Trade Regulations")
	itemInstance.Help:SetText("Automatic management of your offer in Auction House.")
	itemInstance.BasicSupportPD:ClearEntries();
	for a, b in pairs(LuxuryTradeRegT) do
		AddToPullDown(itemInstance.BasicSupportPD, b.Name, b.Help, a, 1)
	end
	local cOption = GetIDFromType(LuxuryTradeRegT, gData.Players[iPlayer].AIAuctionManager);
	itemInstance.BasicSupportPD:GetButton():LocalizeAndSetText(LuxuryTradeRegT[cOption].Name);		
	itemInstance.BasicSupportPD:GetButton():SetToolTipString(LuxuryTradeRegT[cOption].Help);
	itemInstance.BasicSupportPD:CalculateInternals();
	itemInstance.BasicSupportPD:RegisterSelectionCallback(OnLuxuryTradeRegulation); 
	itemInstance.BoxPD:SetHide(false)
	FinalCounselorInstance(itemInstance)
	
	--Master of Proselytism
	if (gData.Globals.ScenarioType ~= 5) then
		local itemInstance = g_CounselorList:GetInstance();
		InitialCounselorInstance(itemInstance)
		itemInstance.Name:SetText("Master of Proselytism")
		itemInstance.Help:SetText("Automatic management of your Missionaries.")
		SetCounselorButtonBasic(itemInstance, gData.Players[iPlayer].AutomaticMissionaries)
		itemInstance.ActiveButton:RegisterCallback( Mouse.eLClick, OnAutomaticMissionaries )
		FinalCounselorInstance(itemInstance)
	end
	
	--Minister of Railroads
	if (gData.Globals.ScenarioType == 4) or (gData.Globals.ScenarioType == 5) then
		local itemInstance = g_CounselorList:GetInstance();
		InitialCounselorInstance(itemInstance)
		itemInstance.Name:SetText("Minister of Railways")
		itemInstance.Help:SetText("Automatic construction of Railroads.")
		SetCounselorButtonBasic(itemInstance, gData.Players[iPlayer].AIRailroad )
		itemInstance.ActiveButton:RegisterCallback( Mouse.eLClick, OnMasterOfRailways )
		FinalCounselorInstance(itemInstance)
	end

	--Tax Collector
	if gData.Globals.ScenarioType ~= 5 then
		local itemInstance = g_CounselorList:GetInstance();
		InitialCounselorInstance(itemInstance)
		itemInstance.Name:SetText("Tax Collector")
		itemInstance.Help:SetText("Automatic management of bullying [ICON_GOLD] Gold from City-States (happens only at the turn start).")
		itemInstance.BasicSupportPD:ClearEntries();
		for a, b in pairs(TaxCollectorT) do
			AddToPullDown(itemInstance.BasicSupportPD, b.Name, b.Help, a, 1)
		end
		local cOption = GetIDFromType(TaxCollectorT, gData.Players[iPlayer].AIBullying);
		itemInstance.BasicSupportPD:GetButton():LocalizeAndSetText(TaxCollectorT[cOption].Name);		
		itemInstance.BasicSupportPD:GetButton():SetToolTipString(TaxCollectorT[cOption].Help);
		itemInstance.BasicSupportPD:CalculateInternals();
		itemInstance.BasicSupportPD:RegisterSelectionCallback(OnTaxCollector); 
		itemInstance.BoxPD:SetHide(false)
		FinalCounselorInstance(itemInstance)
	end

	--Master of Whispers
	local itemInstance = g_CounselorList:GetInstance();
	InitialCounselorInstance(itemInstance)
	itemInstance.Name:SetText("Master of Whispers")
	itemInstance.Help:SetText("Automatic management of your Agents.")
	SetCounselorButtonBasic(itemInstance, gData.Players[iPlayer].MasterOfWhispers)
	itemInstance.ActiveButton:RegisterCallback( Mouse.eLClick, OnMoW )
	FinalCounselorInstance(itemInstance)
	
	if (gData.Globals.ScenarioType ~= 5) then
		--Master of Worship
		local itemInstance = g_CounselorList:GetInstance();
		InitialCounselorInstance(itemInstance)
		itemInstance.Name:SetText("Master of Worship")
		itemInstance.Help:SetText("Automatic purchasing of buildings with [ICON_PEACE] Faith.")
		SetCounselorButtonBasic(itemInstance, gData.Players[iPlayer].AutomaticFaithPurchase)
		itemInstance.ActiveButton:RegisterCallback( Mouse.eLClick, OnAutomaticFaithPurchase )
		FinalCounselorInstance(itemInstance)
	end


	
    Controls.CounselorStack:CalculateSize();
    Controls.CounselorStack:ReprocessAnchoring();
    Controls.CounselorScroll:CalculateInternalSize();
end
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function ShowHideHandler( bIsHide, bInitState )
    if( not bIsHide ) then
    	UpdateScreen();
	end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );
