-------------------------------------------------
-- ITG Governors
-------------------------------------------------
include( "IconSupport" );
include( "InstanceManager" );

-------------------------------------------------
-------------------------------------------------
local ltBlue = {19/255,32/255,46/255,120/255};
local dkBlue = {12/255,22/255,30/255,120/255};
local g_PianoKeys = {
	[0] = {19/255,32/255,46/255,120/255},
	[1] = {12/255,22/255,30/255,120/255},
}
local bTickTock = false;
local cPanelHelpID = 0;
local cPanelExtraInfo = -1;
local m_BasicLegalInfo_IM = InstanceManager:new("BasicLegalInfoInstance", "BasicLegalInfoBox", Controls.InfoGovStack)
local g_TabList = GenerationalInstanceManager:new( "TabPanelInstance", "TabButton", Controls.PanelsTabStack);
local g_CausesActive = GenerationalInstanceManager:new( "CauseInstance", "Base", Controls.CauseStackActive);
local g_CausesDisactive = GenerationalInstanceManager:new( "CauseInstance", "Base", Controls.CauseStackDisactive);
local g_GovernorsInfo = InstanceManager:new( "GovernorInstance", "Button", Controls.MainStack );
local g_ForeignInfo = InstanceManager:new( "ForeignGovernorInstance", "Button", Controls.ForeignStack );
local g_LogManager = GenerationalInstanceManager:new( "LogInstance", "Base", Controls.LogStack);
local g_DetailLogManager = GenerationalInstanceManager:new( "LogInstance", "Base", Controls.DetailLogStack);

local CauseTypeStrings = {
[0] = {"Stackable", "A cause which importance is based on amount of certain actions/events that happened."},
[1] = {"Permanent", "Once active, this cause will be active for entire scenario."},
[2] = {"Situational", "A cause which is active if you meet criteria (tested every turn)."}, 
}
local g_SortFunction;
local l_SortOptions = {
	{
		Button = Controls.CauseSortByType,
		ImageControl = Controls.CauseSortByImage,
		Column = 1,
		DefaultDirection = "asc",
		CurrentDirection = "asc",
	},
	{
		Button = Controls.CauseSortByName,
		ImageControl = Controls.CauseSortByNameImage,
		Column = 2,
		DefaultDirection = "desc",
		CurrentDirection = nil,
	},
	{
		Button = Controls.CauseSortByStacks,
		ImageControl = Controls.CauseSortbyStacksImage,
		Column = 3,
		DefaultDirection = "desc",
		CurrentDirection = nil,
		SortType = "numeric",
	},
};
--[[
0. Overview, 1. Governors, 2. Causes, 3. Foreign Relations
]]
function OnTabSelect(id, eInfo)
	if (id ~= cPanelHelpID) or (eInfo ~= cPanelExtraInfo) then
		cPanelExtraInfo = eInfo;
		cPanelHelpID = id;
		UpdateScreen()
	end
end
function GenerateTabInstance(str, helpID, extraInfo)
	ginstance = g_TabList:GetInstance();
	ginstance.TabButton:SetVoid1(helpID)
	ginstance.TabButton:SetVoid2(extraInfo)
	ginstance.TabLabel:SetText(str);
	ginstance.SelectHighlight:SetHide((helpID ~= cPanelHelpID) or (extraInfo ~= cPanelExtraInfo));
	local sizeY = math.max(24, ginstance.TabLabel:GetSizeY() + 12);
	ginstance.TabButton:SetSizeY(sizeY)
	ginstance.SelectHighlight:SetSizeY(sizeY + 2)
	ginstance.HL1:SetSizeY(sizeY + 3)
	ginstance.HL2:SetSizeY(sizeY + 3)
	ginstance.TabButton:RegisterCallback( Mouse.eLClick, OnTabSelect )
end

function UpdateScreen()
	LuaEvents.ITGSG_RequestGT();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	local iTeam = Game.GetActiveTeam();
	local iPlayer = Game.GetActivePlayer()
	if not gData.Players[iPlayer] then
		return;
	end
	bTickTock = false;
	g_TabList:ResetInstances()
	GenerateTabInstance("Overview", 0, -1)
	for gvrn, sth in pairs(gData.Players[iPlayer].Governors) do
		GenerateTabInstance("[ICON_ITG_GOVERNOR]" .. Locale.Lookup(gData.Globals.Governors[gvrn].Name), 1, gvrn)
	end
	GenerateTabInstance("Causes", 2, -1)
	GenerateTabInstance("Foreign Relations", 3, -1)
	GenerateTabInstance("Event Log", 4, -1)
	
	Controls.PanelsTabStack:CalculateSize();
	Controls.PanelsTabStack:ReprocessAnchoring();
	Controls.OptionTab:SetSizeY(90 + Controls.PanelsTabStack:GetSizeY())
	Controls.OptionTabBg:SetSizeY(Controls.OptionTab:GetSizeY())
	if cPanelHelpID == 0 then
		DoOverviewPanel()
	elseif cPanelHelpID == 1 then
		DoDetailPanel()
	elseif cPanelHelpID == 2 then
		DoCausesPanel()
	elseif cPanelHelpID == 3 then
		DoForeignPanel()
	elseif cPanelHelpID == 4 then
		DoLogPanel()
	end
	Controls.MainOverviewPanel:SetHide(cPanelHelpID ~= 0);
	Controls.DetailPanel:SetHide(cPanelHelpID ~= 1);
	Controls.CausePanel:SetHide(cPanelHelpID ~= 2);
	Controls.ForeignPanel:SetHide(cPanelHelpID ~= 3);
	Controls.LogPanel:SetHide(cPanelHelpID ~= 4);

	
end

function GetColorCause(iLevel)
	if iLevel == 1 then
		return "[COLOR_YELLOW]"
	elseif iLevel == 2 then
		return "[COLOR_NEGATIVE_TEXT]"
	end
end

function GetNumCausesWithColour(iLevel)
	if iLevel > 8 then
		return "[COLOR_NEGATIVE_TEXT]" .. iLevel;
	elseif iLevel > 4 then
		return "[COLOR_YELLOW]" .. iLevel;
	end
	return iLevel;
end

function AddSmallGovInfo(a, b, c)
	local controlTable = m_BasicLegalInfo_IM:GetInstance();
	controlTable.MainInfo:SetText(a or "")
	controlTable.DataInfo:SetText(b or "")
	controlTable.BasicLegalInfoBox:SetToolTipString(c or "")
end

function OnReassignGovernor(govID)
	if Players[Game.GetActivePlayer()]:IsTurnActive() then
		LuaEvents.ITG_HumanRequestGovernorReassign(Game.GetActivePlayer(), govID);
	end
end
function OnNominateGovernor(govID)
	if Players[Game.GetActivePlayer()]:IsTurnActive() then
		LuaEvents.ITG_HumanOpensNominateGovernor(govID);
	end
end
function OnExecuteGovernor(govID)
	if Players[Game.GetActivePlayer()]:IsTurnActive() then
		LuaEvents.ITG_HumanOpensExecuteGovernor(govID);
	end
end
Controls.GovAction1:RegisterCallback( Mouse.eLClick, OnReassignGovernor )
Controls.GovAction2:RegisterCallback( Mouse.eLClick, OnNominateGovernor )
Controls.GovAction3:RegisterCallback( Mouse.eLClick, OnExecuteGovernor )

local h1, h2 = 130, 155;
function SetSizesHH(a, b)
	h1, h2 = a, b
	DoOverviewPanel()
end

function DoDetailPanel()
	local govID = cPanelExtraInfo;
	local iPlayer = Game.GetActivePlayer()
	LuaEvents.ITGSG_RequestGT();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	if not gData.Players[iPlayer].Governors[govID] then
		OnTabSelect(0, -1)
		return;
	end
	LuaEvents.ITG_RequestForeignGovernorData(iPlayer, govID, true);
	local tab = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	local govinfo = gData.Globals.Governors[govID];

	Controls.PowerLabel:SetText(tab.PowerTT);
	Controls.CitiesManagedLabel:SetText(tab.CitiesTT[1]);
	Controls.CitiesChallengedLabel:SetText(tab.CitiesTT[2]);
	Controls.CitiesForeignLabel:SetText(tab.CitiesTT[3]);
	Controls.CitiesManaged:SetHide(tab.CitiesTT[1] == "")
	Controls.CitiesManagedLabel:SetHide(tab.CitiesTT[1] == "")
	Controls.CitiesChallenged:SetHide(tab.CitiesTT[2] == "")
	Controls.CitiesChallengedLabel:SetHide(tab.CitiesTT[2] == "")
	Controls.CitiesForeign:SetHide(tab.CitiesTT[3] == "")
	Controls.CitiesForeignLabel:SetHide(tab.CitiesTT[3] == "")
	if tab.InfoTT ~= "" then
		Controls.ExtraInfoBox:SetToolTipString(tab.InfoTT)
		Controls.ExtraInfoBox:SetHide(false)
	else
		Controls.ExtraInfoBox:SetHide(true)
	end
	local hstrgg = ""
	for jgb, gbr in pairs(tab.GrudgeTT) do
		if hstrgg ~= "" then
			hstrgg = hstrgg .. "[NEWLINE]"
		end
		hstrgg = hstrgg .. "[ICON_BULLET]" .. gbr[1] .. "[NEWLINE]" .. gbr[2];
	end
	Controls.GrudgeLabel:SetText(hstrgg)
	Controls.GrudgeLabel:SetHide(hstrgg == "")
	Controls.GrudgesTitle:SetHide(hstrgg == "")
	Controls.RelationsLabel:SetText(tab.RelationTT)
	Controls.RelationsLabel:SetHide(tab.RelationTT == "")
	Controls.RelationsTitle:SetHide(tab.RelationTT == "")
	
	Controls.GovAction1:SetVoid1(govID)
	Controls.GovAction2:SetVoid1(govID)
	Controls.GovAction3:SetVoid1(govID)
	local disabledActionTT = "";
	if tab.ActionBlock == 1 then
		disabledActionTT = "All actions are disabled during Anarchy."
	elseif tab.ActionBlock == 2 then
		disabledActionTT = "All actions while there is an active diplo action."
	end
	if govinfo.City then
		Controls.GovAction1Text:SetText("Reassign")
		if govinfo.LastTAssign + tab.ReassignCD > gData.ITGTurn then
			Controls.GovAction1:SetToolTipString("This [ICON_ITG_GOVERNOR] Governor cannot be reassigned for another " .. govinfo.LastTAssign + tab.ReassignCD - gData.ITGTurn .. " turn(s).")
			Controls.GovAction1:SetDisabled(true)
		elseif tab.ActionBlock then
			Controls.GovAction1:SetDisabled(true)
			Controls.GovAction1:SetToolTipString(disabledActionTT)
		elseif Map.GetPlotByIndex(govinfo.City):GetPlotCity():GetDamage() > 0 then
			Controls.GovAction1:SetDisabled(true)
			Controls.GovAction1:SetToolTipString("[ICON_ITG_GOVERNOR] Governor cannot be reassigned from damaged City.")
		else
			Controls.GovAction1:SetToolTipString("Opens a panel to change [ICON_ITG_GOVERNOR] Governor location. [ICON_ITG_GOVERNOR] Governors can be reassigned only once every " .. tab.ReassignCD .. " turns.")
			Controls.GovAction1:SetDisabled(not tab.HasFreeLocation)
		end
	elseif tab.ActionBlock then
		Controls.GovAction1:SetDisabled(true)
		Controls.GovAction1:SetToolTipString(disabledActionTT)
	else
		Controls.GovAction1Text:SetText("Assign")
		Controls.GovAction1:SetToolTipString("Opens a panel to change [ICON_ITG_GOVERNOR] Governor location. [ICON_ITG_GOVERNOR] Governors can be reassigned only once every " .. tab.ReassignCD .. " turns.")
		Controls.GovAction1:SetDisabled(not tab.HasFreeLocation)
	end
	Controls.GovAction2:SetDisabled(tab.ActionBlock or tab.IsPuppet or tab.NoValidPosition or tab.HasStrongPosition)
	Controls.GovAction3:SetDisabled(tab.ActionBlock or tab.IsPuppet)

	Controls.GovTypeInfo:SetText(tab.GovType[1])
	Controls.GovTypeBox:SetToolTipString(tab.GovType[2])
	Controls.GovLevelInfo:SetText(govinfo.CityLevel)
	Controls.Name:SetText("[ICON_ITG_GOVERNOR] " .. Locale.Lookup(govinfo.Name));
	IconHookup( govinfo.PIndex, 64, govinfo.IconAtlas, Controls.GovIcon );
	if govinfo.City then
		Controls.LocationInfo:SetText(Map.GetPlotByIndex(govinfo.City):GetPlotCity():GetName())
	else
		Controls.LocationInfo:SetText("Unassigned")
	end		
	Controls.LocationInfo:SetToolTipString("Contests cities within " .. tab.GRange .. " radius for governorship.")
	Controls.PowerInfo:SetText(govinfo.Power .. "[ICON_ITG_POWER]")
	Controls.CityTraitInfo:SetText("[ICON_PLUS] " .. tab.CityTraitData[1])
	Controls.CityTraitBox:SetToolTipString("Bonus for managed Cities:[NEWLINE]" .. Locale.Lookup(tab.CityTraitData[2]))
	if govinfo.CityReligion then
		Controls.ReligionInfo:SetText(GameInfo.Religions[govinfo.CityReligion].IconString .. " " .. Locale.Lookup(Game.GetReligionName(govinfo.CityReligion)))
		Controls.ReligionBox:SetHide(false)
	else
		Controls.ReligionBox:SetHide(true)
	end

	LuaEvents.ITG_RequestGovActionLog(iPlayer, govID);
	local tLogData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	g_DetailLogManager:ResetInstances()
	local bTickTock = true;
	for a, v in pairs(tLogData) do
		local instance = g_DetailLogManager:GetInstance();
		instance.Base:SetColorVal(unpack(bTickTock and g_PianoKeys[0] or g_PianoKeys[1]));
		instance.TurnNum:SetText(v[1]);
		instance.Message:SetString(v[2]);
		local width, height = instance.Message:GetSizeVal();
		local baseWidth, baseHeight = instance.Base:GetSizeVal();
		instance.Base:SetSizeVal(baseWidth, height + 22);
			
		bTickTock = not bTickTock;
	end
	Controls.DetailLogStack:CalculateSize();
	Controls.DetailLogStack:ReprocessAnchoring();
    Controls.DetailLogScroll:CalculateInternalSize();
	Controls.DetailCitiesLogStack:CalculateSize();
	Controls.DetailCitiesLogStack:ReprocessAnchoring();
    Controls.DetailCitiesScroll:CalculateInternalSize();
	Controls.DetailGrudgesLogStack:CalculateSize();
	Controls.DetailGrudgesLogStack:ReprocessAnchoring();
    Controls.DetailGrudgesScroll:CalculateInternalSize();
end

function DoOverviewPanel()
	local iTeam = Game.GetActiveTeam();
	local iPlayer = Game.GetActivePlayer()
	LuaEvents.ITGSG_RequestAllGovernorRelatedData(iPlayer);
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	m_BasicLegalInfo_IM:ResetInstances()
	local numCities = Players[iPlayer]:GetNumCities();
	local disabledActionTT = "";
	if gData.ActionBlock == 1 then
		disabledActionTT = "All actions are disabled during Anarchy."
	elseif gData.ActionBlock == 2 then
		disabledActionTT = "All actions while there is an active diplo action."
	end

	LuaEvents.ITG_BasicGovLegalRequest(34)
	local tabBuyNew = MapModData.ITGSG
	MapModData.ITGSG = nil;
	if tabBuyNew[1] then
		Controls.AppointGovLabel:SetText("2 [ICON_ITG_LEGAL_POWER] Appoint Governor")
		Controls.AppointGovernor:SetDisabled(not tabBuyNew[2])
		Controls.AppointGovernor:SetHide(false)
	else
		Controls.AppointGovernor:SetHide(true)
	end

	--sm0all infos
	AddSmallGovInfo("Power:", gData.GovLike[-1].Power .. " [ICON_ITG_POWER]", gData.FullPlayerPowerTT)
	AddSmallGovInfo("Governors:", gData.NumGovs .. "/" .. gData.MaxGovs .. " [ICON_ITG_GOVERNOR]", "Amount of [ICON_ITG_GOVERNOR] Governor slots occupied.")
	AddSmallGovInfo("Governed Cities:", gData.CityUnderGovPerc .. "%", "Capital and each city with placed [ICON_ITG_GOVERNOR] Governor counts as being governed. Once this percentage drops below 25% your empire will loose an extra 1 [ICON_ITG_MERITOCRACY] Meritocracy per turn.")
	AddSmallGovInfo("Causes:", GetNumCausesWithColour(gData.Causes), "Amount of active Causes' types that may affect [ICON_ITG_GOVERNOR] Governors' behaviour.")
	AddSmallGovInfo("Units:", gData.NumUnits, "Amount of alive military units.")
	AddSmallGovInfo("City-States:", gData.NumAlliedCS, "Amount of allied City-States.")
	AddSmallGovInfo("Foreign Contacts:", gData.NumForeign, "Amount of foreign [ICON_ITG_GOVERNOR] Governors you have established relations with.")
	
	bTickTock = false;
	local function PopulateSmallGovernorInfo(govID, tab, bLeader)
		local controlTable = g_GovernorsInfo:GetInstance();
		--resize if needed
		controlTable.GovAction1:SetVoid1(govID)
		controlTable.GovAction3:SetVoid1(govID)
		controlTable.GovAction4:SetVoid1(govID)
		controlTable.GovAction1:RegisterCallback( Mouse.eLClick, OnReassignGovernor )
		controlTable.GovAction3:RegisterCallback( Mouse.eLClick, OnExecuteGovernor )
		controlTable.GovAction4:RegisterCallback( Mouse.eLClick, OnNominateGovernor )
		controlTable.PowerInfo:SetText(tab.Power .. "[ICON_ITG_POWER] (" .. math.floor(tab.Power / gData.TotalPower * 100) .. "%)")
		controlTable.PowerBox:SetToolTipString(tab.PowerTT)
		if numCities > 0 then
			controlTable.CitiesInfo:SetText(#tab.Cities .. " (" .. math.floor(#tab.Cities / numCities * 100) .. "%)")
		else
			controlTable.CitiesInfo:SetText(0)
		end
		local citiesTT = ""
		for i, cityID in pairs(tab.Cities) do
			if citiesTT ~= "" then
				citiesTT = citiesTT .. "[NEWLINE]"
			end
			citiesTT = citiesTT .. Players[iPlayer]:GetCityByID(cityID):GetName();
		end
		controlTable.CitiesBox:SetToolTipString(citiesTT);
		controlTable.MilitaryInfo:SetText(gData.MilitaryShare[govID] .. "%")
		if tab.Religion then
			controlTable.ReligionInfo:SetText(GameInfo.Religions[tab.Religion].IconString .. " " .. Locale.Lookup(Game.GetReligionName(tab.Religion)))
			controlTable.ReligionBox:SetHide(false)
		else
			controlTable.ReligionBox:SetHide(true)
		end
		controlTable.GovTypeBox:SetHide(bLeader)
		controlTable.GovLevelBox:SetHide(bLeader)
		controlTable.GrudgesBox:SetHide(bLeader)
		if bLeader then
			controlTable.LocationInfo:SetToolTipString("You contest all cities for governorship.")
			controlTable.Name:SetText(tab.Name);
			IconHookup( GameInfo.Leaders[Players[iPlayer]:GetLeaderType()].PortraitIndex, 64, GameInfo.Leaders[Players[iPlayer]:GetLeaderType()].IconAtlas , controlTable.GovIcon );
			if numCities > 0 then
				controlTable.LocationInfo:SetText(Players[iPlayer]:GetCapitalCity():GetName())
			else
				controlTable.LocationInfo:SetText("Exile")
			end
			controlTable.ButtonStack:SetHide(true)
			controlTable.Button:SetSizeY(h1);
			controlTable.ExtraInfoBox:SetHide(true)
			controlTable.CityTraitBox:SetHide(true)
		else
			controlTable.LocationInfo:SetToolTipString("Contests cities within " .. tab.GRange .. " radius for governorship.")
			controlTable.CityTraitBox:SetHide(false)
			controlTable.CityTraitInfo:SetText("[ICON_PLUS] " .. tab.CityTraitData[1])
			controlTable.CityTraitBox:SetToolTipString("Bonus for managed Cities:[NEWLINE]" .. Locale.Lookup(tab.CityTraitData[2]))
			controlTable.GovAction2:SetVoid1(1)
			controlTable.GovAction2:SetVoid2(govID)
			controlTable.GovAction2:RegisterCallback( Mouse.eLClick, OnTabSelect )
			controlTable.GovLevelInfo:SetText(tab.Level)
			controlTable.GovTypeInfo:SetText(tab.GovType[1])
			controlTable.GrudgesInfo:SetText(#tab.GrudgeTT)
			if tab.InfoTT and (tab.InfoTT ~= "") then
				controlTable.ExtraInfoBox:SetToolTipString(tab.InfoTT)
				controlTable.ExtraInfoBox:SetHide(false)
			else
				controlTable.ExtraInfoBox:SetHide(true)
			end
			if #tab.GrudgeTT == 0 then
				controlTable.GrudgesBox:SetToolTipString("Has no grudges towards you.")
			else
				local hstrgg = ""
				for jgb, gbr in pairs(tab.GrudgeTT) do
					if hstrgg ~= "" then
						hstrgg = hstrgg .. "[NEWLINE]"
					end
					hstrgg = hstrgg .. "[ICON_BULLET]" .. gbr[1] .. "[NEWLINE]" .. gbr[2];
				end
				controlTable.GrudgesBox:SetToolTipString(hstrgg)
			end
			controlTable.GovTypeBox:SetToolTipString(tab.GovType[2])
			controlTable.Name:SetText("[ICON_ITG_GOVERNOR] " .. Locale.Lookup(tab.Name));
			IconHookup( tab.PortIndex, 64, tab.IconAtlas, controlTable.GovIcon );
			if tab.Location then
				controlTable.GovAction1Text:SetText("Reassign")
				controlTable.LocationInfo:SetText(Map.GetPlotByIndex(tab.Location):GetPlotCity():GetName())
				if tab.LastTAssign + tab.ReassignCD > gData.Turn then
					controlTable.GovAction1:SetToolTipString("This [ICON_ITG_GOVERNOR] Governor cannot be reassigned for another " .. tab.LastTAssign + tab.ReassignCD - gData.Turn .. " turn(s).")
					controlTable.GovAction1:SetDisabled(true)
				elseif gData.ActionBlock then
					controlTable.GovAction1:SetDisabled(true)
					controlTable.GovAction1:SetToolTipString(disabledActionTT)
				elseif Map.GetPlotByIndex(tab.Location):GetPlotCity():GetDamage() > 0 then
					controlTable.GovAction1:SetDisabled(true)
					controlTable.GovAction1:SetToolTipString("[ICON_ITG_GOVERNOR] Governor cannot be reassigned from damaged City.")
				else
					controlTable.GovAction1:SetToolTipString("Opens a panel to change [ICON_ITG_GOVERNOR] Governor location. [ICON_ITG_GOVERNOR] Governors can be reassigned only once every " .. tab.ReassignCD .. " turns.")
					controlTable.GovAction1:SetDisabled(not gData.HasFreeLocation)
				end
			elseif gData.ActionBlock then
				controlTable.GovAction1:SetDisabled(true)
				controlTable.GovAction1:SetToolTipString(disabledActionTT)
			else
				controlTable.GovAction1Text:SetText("Assign")
				controlTable.GovAction1:SetToolTipString("Opens a panel to change [ICON_ITG_GOVERNOR] Governor location. [ICON_ITG_GOVERNOR] Governors can be reassigned only once every " .. tab.ReassignCD .. " turns.")
				controlTable.GovAction1:SetDisabled(not gData.HasFreeLocation)
				controlTable.LocationInfo:SetText("Unassigned")
			end
			controlTable.ButtonStack:SetHide(false)
			--reassign button
			controlTable.GovAction3:SetDisabled(gData.ActionBlock or gData.IsPuppet)
			controlTable.GovAction4:SetDisabled(gData.ActionBlock or gData.IsPupper or gData.NoValidPosition or tab.HasStrongPosition)
			controlTable.Button:SetSizeY(h2);
		end
		
		if(bTickTock == false) then
			controlTable.Button:SetColorVal(unpack(ltBlue));
		else
			controlTable.Button:SetColorVal(unpack(dkBlue));
		end
		bTickTock = not bTickTock;
		ghhs = controlTable
	end
	g_GovernorsInfo:ResetInstances()
	--leader
	PopulateSmallGovernorInfo(-1, gData.GovLike[-1], true)
	--governors
	for hhid, data in pairs(gData.GovLike) do
		if hhid ~= -1 then
			PopulateSmallGovernorInfo(hhid, data)
		end
	end
	
	Controls.MainStack:CalculateSize();
	Controls.MainStack:ReprocessAnchoring();
    Controls.ScrollPanel:CalculateInternalSize();
end

local bShowActive = true;
local bShowInActive = true;
function DoCausesPanel()
	local iTeam = Game.GetActiveTeam();
	local iPlayer = Game.GetActivePlayer()
	LuaEvents.ITG_RequestGovCauses(iPlayer);
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	g_CausesActive:ResetInstances();
	g_CausesDisactive:ResetInstances();

	local preData = {}
	for a, b in pairs(gData) do
		table.insert(preData, {Locale.Lookup(CauseTypeStrings[b.Type][1]), Locale.Lookup(b.Name), a})
	end
	table.sort(preData, g_SortFunction);
	for a, b in pairs(preData) do
		local data = gData[b[3]];
		local instance;
		if data.On then
			if bShowActive then
				instance = g_CausesActive:GetInstance();
				instance.Base:SetDisabled(false)
			end
		elseif bShowInActive then
			instance = g_CausesDisactive:GetInstance();
			instance.Base:SetDisabled(true)
		end
		if instance then
			instance.Type:SetText(CauseTypeStrings[data.Type][1])
			instance.Type:SetToolTipString(CauseTypeStrings[data.Type][2])
			instance.Name:SetText(GetColorCause(data.Level) .. Locale.Lookup(data.Name))
			instance.Name:SetToolTipString(Locale.Lookup(data.Help))
			local sStacks = "";
			if (data.Type == 0) and (data.On) then
				sStacks = data.On;
			end
			instance.Stacks:SetText(sStacks)
		end
	end


	Controls.CauseStackActive:CalculateSize();
	Controls.CauseStackActive:ReprocessAnchoring();
	Controls.CauseStackDisactive:CalculateSize();
	Controls.CauseStackDisactive:ReprocessAnchoring();
	Controls.CauseStack:CalculateSize();
	Controls.CauseStack:ReprocessAnchoring();
    Controls.CauseScroll:CalculateInternalSize();
end

function DoForeignPanel()
	local iPlayer = Game.GetActivePlayer()
	local iTeam = Game.GetActiveTeam()
	LuaEvents.ITGSG_RequestGT();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	g_ForeignInfo:ResetInstances()
	local bHasAny = false;
	bTickTock = true;
	for govID, sth in pairs(gData.Players[iPlayer].ForeignGovernors) do
		bHasAny = true;
		LuaEvents.ITG_RequestForeignGovernorData(iPlayer, govID);
		local tab = MapModData.ITGSG;
		MapModData.ITGSG = nil;
		local govinfo = gData.Globals.Governors[govID];
		controlTable = g_ForeignInfo:GetInstance();

		if govinfo.Player == -1 then
			controlTable.LeaderSubIconFrame:SetHide(true)
			controlTable.CivilizationInfo:SetText("Exile")
		else
			controlTable.CivilizationInfo:SetText(Players[govinfo.Player]:GetCivilizationShortDescription())
			controlTable.LeaderSubIconFrame:SetHide(false)
			CivIconHookup(govinfo.Player, 45, controlTable.LeaderSubIcon, controlTable.LeaderSubIconBG, controlTable.LeaderSubIconShadow, true, true, controlTable.LeaderSubIconHighlight);
		end
		controlTable.GovTypeInfo:SetText(tab.GovType[1])
		controlTable.GovTypeBox:SetToolTipString(tab.GovType[2])
		controlTable.GovLevelInfo:SetText(govinfo.CityLevel)
		controlTable.Name:SetText("[ICON_ITG_GOVERNOR] " .. Locale.Lookup(govinfo.Name));
		IconHookup( govinfo.PIndex, 64, govinfo.IconAtlas, controlTable.GovIcon );
		if govinfo.City then
			if Map.GetPlotByIndex(govinfo.City):IsRevealed(iTeam) then
				controlTable.LocationInfo:SetText(Map.GetPlotByIndex(govinfo.City):GetPlotCity():GetName())
			else
				controlTable.LocationInfo:SetText("Unknown City")
			end
		else
			controlTable.LocationInfo:SetText("Unassigned")
		end		
		controlTable.PowerInfo:SetText(govinfo.Power .. "[ICON_ITG_POWER]")
		controlTable.TurnInfo:SetText(govinfo.ForeignContacts[iPlayer].Established .. "T")
		controlTable.FavourInfo:SetText(govinfo.ForeignContacts[iPlayer].Favour or 0 .. "[ICON_ITG_POWER]")
		controlTable.CityTraitInfo:SetText(tab.CityTraitData[1])
		controlTable.CityTraitBox:SetToolTipString("Bonus for managed Cities:[NEWLINE]" .. Locale.Lookup(tab.CityTraitData[2]))
		if govinfo.CityReligion then
			controlTable.ReligionInfo:SetText(GameInfo.Religions[govinfo.CityReligion].IconString .. " " .. Locale.Lookup(Game.GetReligionName(govinfo.CityReligion)))
			controlTable.ReligionBox:SetHide(false)
		else
			controlTable.ReligionBox:SetHide(true)
		end
		if govinfo.ForeignContacts[iPlayer].SecretPact then
			IconHookup(13, 45, "TECH_ITG_ATLAS_1", controlTable.TechIcon );
			controlTable.SecretFrame:SetHide(false);
		else
			controlTable.SecretFrame:SetHide(true);
		end

		if(bTickTock == false) then
			controlTable.Button:SetColorVal(unpack(ltBlue));
		else
			controlTable.Button:SetColorVal(unpack(dkBlue));
		end
		bTickTock = not bTickTock;
	end
	Controls.NoForeignAvailable:SetHide(bHasAny)
	
	Controls.ForeignStack:CalculateSize();
	Controls.ForeignStack:ReprocessAnchoring();
    Controls.ScrollForeign:CalculateInternalSize();
end

function DoLogPanel()
	local iPlayer = Game.GetActivePlayer()
	LuaEvents.ITG_RequestGovActionLog(iPlayer);
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	g_LogManager:ResetInstances()
	local bTickTock = true;
	for a, v in pairs(gData) do
		local instance = g_LogManager:GetInstance();
		instance.Base:SetColorVal(unpack(bTickTock and g_PianoKeys[0] or g_PianoKeys[1]));
		instance.TurnNum:SetText(v[1]);
		instance.Message:SetString(v[2]);
			
		local width, height = instance.Message:GetSizeVal();
		local baseWidth, baseHeight = instance.Base:GetSizeVal();
		instance.Base:SetSizeVal(baseWidth, height + 22);
			
		bTickTock = not bTickTock;
	end
	Controls.LogStack:CalculateSize();
	Controls.LogStack:ReprocessAnchoring();
    Controls.ScrollLog:CalculateInternalSize();
end

function AppointGovernorFunc()
	if not Players[Game.GetActivePlayer()]:IsTurnActive() then
		return;
	end
	LuaEvents.ITG_PushLegalActionIfPossible(Game.GetActivePlayer(), 34)
	DoOverviewPanel()
end
Controls.AppointGovernor:RegisterCallback(Mouse.eLClick, AppointGovernorFunc)

function OnHeadingActive()
	bShowActive = (not bShowActive);
	if bShowActive then
		Controls.ListHeadingActiveLabel:SetText("- Active")
	else
		Controls.ListHeadingActiveLabel:SetText("+ Active")
	end
	DoCausesPanel()
end
Controls.ListHeadingActiveButton:RegisterCallback( Mouse.eLClick, OnHeadingActive )

function OnHeadingDisActive()
	bShowInActive = (not bShowInActive);
	if bShowInActive then
		Controls.ListHeadingDisactiveLabel:SetText("- Inactive")
	else
		Controls.ListHeadingDisactiveLabel:SetText("+ Inactive")
	end
	DoCausesPanel()
end
Controls.ListHeadingDisactiveButton:RegisterCallback( Mouse.eLClick, OnHeadingDisActive )
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function ShowHideHandler( bIsHide, bInitState )
    if( not bIsHide ) then
    	UpdateScreen();
	end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );

LuaEvents.ITGSG_UpdateGovernorInfoIfOpen.Add(function()
	if (not ContextPtr:IsHidden()) then
    	UpdateScreen();
	end
end)

-- SORTING

function SortOptionSelected(option)
	UpdateSortOptionState(l_SortOptions, option);
	UpdateSortOptionsDisplay(l_SortOptions);
	g_SortFunction = GetSortFunction(l_SortOptions);
	
	DoCausesPanel();
end

function GetSortFunction(sortOptions)
	local orderBy = nil;
	for i,v in ipairs(sortOptions) do
		if(v.CurrentDirection ~= nil) then
			if(v.SortType == "numeric") then
				return NumericSortFunction(v.Column, v.CurrentDirection);
			else
				return AlphabeticalSortFunction(v.Column, v.CurrentDirection);
			end
		end
	end
	
	return nil;
end

function UpdateSortOptionState(sortOptions, selectedOption)
	for i,v in ipairs(sortOptions) do
		if(v == selectedOption) then
			if(v.CurrentDirection == nil) then			
				v.CurrentDirection = v.DefaultDirection;
			else
				if(v.CurrentDirection == "asc") then
					v.CurrentDirection = "desc";
				else
					v.CurrentDirection = "asc";
				end
			end
		else
			v.CurrentDirection = nil;
		end
	end
end

function AlphabeticalSortFunction(field, direction)
	if(direction == "asc") then
		return function(a,b)
			return Locale.Compare(a[field], b[field]) == -1;
		end
	elseif(direction == "desc") then
		return function(a,b)
			return Locale.Compare(a[field], b[field]) == 1;
		end
	end
end

function NumericSortFunction(field, direction)
	if(direction == "asc") then
		return function(a,b)
			return a[field] <= b[field];
		end
	elseif(direction == "desc") then
		return function(a,b)
			return a[field] > b[field];
		end
	end
end

function UpdateSortOptionsDisplay(sortOptions)
	for i,v in ipairs(sortOptions) do
		local imageControl = v.ImageControl;
		if(imageControl ~= nil) then
			if(v.CurrentDirection == nil) then
				imageControl:SetHide(true);
			else
				local imageToUse = "SelectedUp.dds";
				if(v.CurrentDirection == "desc") then
					imageToUse = "SelectedDown.dds";
				end
				imageControl:SetTexture(imageToUse);
				
				imageControl:SetHide(false);
			end
		end
	end
end

function RegisterSortOptions()
	for i,v in ipairs(l_SortOptions) do
		if(v.Button ~= nil) then
			v.Button:RegisterCallback(Mouse.eLClick, function() SortOptionSelected(v); end);
		end
	end
	UpdateSortOptionsDisplay(l_SortOptions);
	g_SortFunction = GetSortFunction(l_SortOptions);
end
RegisterSortOptions()