include( "InstanceManager" );
include( "IconSupport" );
include( "SupportFunctions" );
include( "ITGSG_ModUserData.lua" )

local g_InstanceManager = InstanceManager:new("ItemInstance", "RootButton", Controls.Stack );
local scEraNames = {"The New World", "The Trial", "Age of Titans","Light of Hope", "Sleep of Reason"}
local cEra = -1;
local cType = 1; --1: only 20; 2 all recorded
local sPDT;

---------------------------------------------------------------- 
function SortFunctionForGreaterInteger2(a, b)
	local va = (a ~= nil and a[2] ~= nil) and a[2] or 999;
	local vb = (b ~= nil and b[2] ~= nil) and b[2] or 999;
	return tonumber(va) > tonumber(vb);
end

function DoScrollPDEntry(control, name, void)
	local controlTable = {};
	control:BuildEntry( "InstanceOne", controlTable );
	controlTable.Button:SetVoid1(void);
	controlTable.Button:SetText(name);
end

function ScoreTypePullDown(gType)
	if gType == 1 then
		Controls.ScoreTypePullDown:GetButton():SetText("Top 20")
	else
		Controls.ScoreTypePullDown:GetButton():SetText("All")
	end
	cType = gType;
	UpdateTheList()
end
Controls.ScoreTypePullDown:RegisterSelectionCallback(ScoreTypePullDown); 

function ChooseEraToDisplay(gEra)
	if not scEraNames[gEra] then
		Controls.ScoreEraPullDown:GetButton():SetText("Full Run")
	else
		Controls.ScoreEraPullDown:GetButton():SetText(scEraNames[gEra])
	end
	cEra = gEra;
	UpdateTheList()
end
Controls.ScoreEraPullDown:RegisterSelectionCallback(ChooseEraToDisplay); 

function OnBack()
    UIManager:DequeuePopup( ContextPtr );
end
Controls.BackButton:RegisterCallback( Mouse.eLClick, OnBack );

----------------------------------------------------------------        
----------------------------------------------------------------        

function UpdateTheList()
	g_InstanceManager:ResetInstances();
	for i, v in pairs(sPDT[cEra]) do
		if (cType == 2) and (i > 20) then
			break;
		end
		local controlTable = g_InstanceManager:GetInstance();
			local civType = MUD_Get("VIC_SAVE_CIV_" .. v[1])
			local leaderType = MUD_Get("VIC_SAVE_LEADER_" .. v[1])
			local bHuman = (MUD_Get("VIC_SAVE_ISHUMAN_" .. v[1]) == 1)
			local cultType = MUD_Get("VIC_SAVE_CULT_TYPE_" .. v[1])
			local date = MUD_Get("VIC_SAVE_DATE_" .. v[1])
			local version = "v" ..  MUD_Get("VIC_SAVE_MODVERSION_" .. v[1])
			local handicap = MUD_Get("VIC_SAVE_HANDICAP_" .. v[1])
			local mapsize = MUD_Get("VIC_SAVE_MAPSIZE_" .. v[1]) .. " (" .. MUD_Get("VIC_SAVE_NUMPLAYERS_" .. v[1]) .. ")"
			local score = v[2];

		IconHookup( GameInfo.Leaders[leaderType].PortraitIndex, 128, GameInfo.Leaders[leaderType].IconAtlas, controlTable.Portrait );
		IconHookup( GameInfo.Civilizations[civType].PortraitIndex, 32, GameInfo.Civilizations[civType].IconAtlas, controlTable.CivIcon );
		
		controlTable.Place:SetText(i .. ".")
		controlTable.LeaderName:SetText(Locale.Lookup(GameInfo.Leaders[leaderType].Description))
		if bHuman then
			controlTable.CivName:SetText(Locale.Lookup(GameInfo.Civilizations[civType].Description))
		else
			controlTable.CivName:SetText(Locale.Lookup(GameInfo.Civilizations[civType].Description) .. " (AI)")
		end
		controlTable.Score:SetText(score)
		if cEra ~= -1 then
			controlTable.CulturalType:SetText(cultType)
		end
		controlTable.CulturalType:SetHide(cEra == -1);

		controlTable.MapInfo:SetText(mapsize)
		controlTable.HandicapInfo:SetText(handicap)
		controlTable.DateInfo:SetText(date)
		controlTable.VersionInfo:SetText(version)
		controlTable.StatusStack:CalculateSize();
		controlTable.StatusStack:ReprocessAnchoring();

		gggh = controlTable;
	end
	Controls.Stack:CalculateSize();
	Controls.ScrollPanel:CalculateInternalSize();
	Controls.ScrollPanel:ReprocessAnchoring();
end

function CallsPDT()
	sPDT = {}
	for i, v in pairs(scEraNames) do
		sPDT[i] = {}
	end
	sPDT[-1] = {}
	local tITT = MUD_Get("VIC_INDEX") or 0;
	for i = 1, tITT do
		table.insert(sPDT[MUD_Get("VIC_SAVE_SC_TYPE_" .. i)], {i, MUD_Get("VIC_SAVE_SCORE_" .. i)})
	end
	for i, v in pairs(scEraNames) do
		table.sort(sPDT[i], SortFunctionForGreaterInteger2)
	end
	table.sort(sPDT[-1], SortFunctionForGreaterInteger2)
end
----------------------------------------------------------------        
---------------------------------------------------------------- 
----------------------------------------------------------------        
----------------------------------------------------------------        
function SetupFileButtonList()
	CallsPDT()

	Controls.ScoreEraPullDown:ClearEntries();
	for i, v in pairs({3, 2, 1, 4, 5}) do
		DoScrollPDEntry(Controls.ScoreEraPullDown,scEraNames[v], v)
	end
	DoScrollPDEntry(Controls.ScoreEraPullDown, "Full Run", -1)
	Controls.ScoreEraPullDown:CalculateInternals();
	Controls.ScoreEraPullDown:GetButton():SetText("Full Run")

	Controls.ScoreTypePullDown:ClearEntries();
	DoScrollPDEntry(Controls.ScoreTypePullDown, "Top 20", 1)
	DoScrollPDEntry(Controls.ScoreTypePullDown, "All", 2)
	Controls.ScoreTypePullDown:CalculateInternals();
	Controls.ScoreTypePullDown:GetButton():SetText("Top 20")
	cType = 1;
	cEra = -1;



	UpdateTheList()
end

----------------------------------------------------------------        
-- Key Down Processing
----------------------------------------------------------------        
function InputHandler( uiMsg, wParam, lParam )
    if uiMsg == KeyEvents.KeyDown then
        if wParam == Keys.VK_ESCAPE then
	        OnBack();
        end
    end
    return true;
end
ContextPtr:SetInputHandler( InputHandler );

----------------------------------------------------------------        
----------------------------------------------------------------        
function ShowHideHandler(isHide)
    if( not isHide ) then
		SetupFileButtonList();
	end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );