-------------------------------------------------
-- ITG CustomsHouse
-------------------------------------------------
include( "IconSupport" );
include( "InstanceManager" );

-------------------------------------------------
-------------------------------------------------
local lastHolySite = nil;
local ltBlue = {19/255,32/255,46/255,120/255};
local dkBlue = {12/255,22/255,30/255,120/255};
local bTickTock = false;
local bShowOnlyYours = true;
local bShowOnlyReligion = false;
local g_ListHolySites = GenerationalInstanceManager:new( "HSListInstance", "Base", Controls.HolySitesStack);

function OnFindOnMapButtonClicked(void)
	UI.LookAt(Map.GetPlotByIndex(void), 0);
end
-- Controls.FindOnMapButton:RegisterCallback( Mouse.eLClick, OnFindOnMapButtonClicked );

function OnSanctifyButtonClicked()
	if Players[Game.GetActivePlayer()]:IsTurnActive() then
		LuaEvents.ITG_PushLegalActionIfPossible(Game.GetActivePlayer(), 32, lastHolySite);
		UpdateScreen(true)
	end
end
Controls.SanctifyButton:RegisterCallback( Mouse.eLClick, OnSanctifyButtonClicked )

function OnDetailedHolySite(void1)
	if void1 then
		Controls.HolySiteDetailBox:SetHide(false)
		lastHolySite = void1;
	else
		Controls.HolySiteDetailBox:SetHide(true)
		return;
	end
	LuaEvents.ITGSG_RequestGT("HolySite", void1);
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	local iOwner = Map.GetPlotByIndex(void1):GetOwner();
	local bDisableAll = iOwner ~= Game.GetActivePlayer();

	Controls.BIRfaith:SetText(gData.HolySites[void1].BasicBonuses[1] .. "[ICON_PEACE]")
	Controls.BIRbasepressure:SetText(gData.HolySites[void1].BasicBonuses[2] .. "[ICON_RELIGION]")
	Controls.BIRmissionary:SetText(gData.HolySites[void1].BasicBonuses[3] .. "[ICON_MISSIONARY]")
	Controls.BIRvisitLog:SetText(#gData.HolySites[void1].VisitLog .. "[ICON_ITG_PILGRIM]")
	local visLogTT = "Amount of Pilgrims that visited this site."
	for a, b in pairs(gData.HolySites[void1].VisitLog) do
		visLogTT = visLogTT .. "[NEWLINE][ICON_BULLET]" .. b[2] .. "T [ICON_PILGRIM] from " .. Players[b[1]]:GetCivilizationShortDescription(); 
	end
	Controls.BIRvisitLog:SetToolTipString(visLogTT)
	local presText = ""
	local hsitesTT = ""
	for rel, val in pairs(gData.HolySites[void1].ActiveReligions) do
		presText = presText .. "+" .. val .. GameInfo.Religions[rel].IconString .. "   ";
		hsitesTT = hsitesTT .. "[NEWLINE]+" .. val .. GameInfo.Religions[rel].IconString
	end
	Controls.BIRpressure:SetText(presText)
	Controls.BIRpressure:SetToolTipString("Religious pressures after modifiers (from beliefs or Wonders). If owner has official religion all other religion have their pressure reduced by 50%." .. hsitesTT)
	local name = "[ICON_RELIGION] Holy Site of " .. gData.HolySites[void1].Name;
	Controls.HolySiteDetailName:SetText(name)

	LuaEvents.ITGSG_RequestDetailsOnHolySite(void1);
	local hsData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	Controls.SanctifyButton:SetHide(true)
	if (not bDisableAll) and hsData.CanSanctify then
		Controls.SanctifyButton:SetDisabled(not hsData.CanSanctifyRightNow)
		Controls.SanctifyButton:SetHide(false)
	end

	Controls.CitiesLabel:SetText(hsData.CityLabel)

	local treshold = hsData.Treshold;
	Controls.GPMeter:SetPercent( gData.HolySites[void1].XP / treshold )
	Controls.MeterImage:SetToolTipString("Progress towards next upgrade bonus: " .. gData.HolySites[void1].XP .. "/" .. treshold .. "[ICON_RELIGION].[NEWLINE][NEWLINE]Each upgrade increases [ICON_PEACE] Faith and base [ICON_RELIGION] Pressure by 1.[NEWLINE][NEWLINE]Progress is earned from:[NEWLINE][ICON_BULLET]+10 each time Pilgrim visits Holy Site.")


    Controls.CitiesAffectedStack:CalculateSize();
    Controls.CitiesAffectedStack:ReprocessAnchoring();
    Controls.CitiesAffectedScroll:CalculateInternalSize();
end

function UpdateScreen(bNotInit)
	local iTeam = Game.GetActiveTeam();
	local iPlayer = Game.GetActivePlayer();
	
	if lastHolySite then
		if not Map.GetPlotByIndex(lastHolySite):IsRevealed(Game.GetActiveTeam()) then
			lastHolySite = nil;
		end
	end
	g_ListHolySites:ResetInstances();
	
	local pRel = Players[iPlayer]:GetReligionCreatedByPlayer();
	Controls.HideShowOnlyReligion:SetHide(pRel < 1);

	LuaEvents.RequestAllHolySiteData(iPlayer, iTeam, bShowOnlyYours, bShowOnlyReligion, pRel)
	local tabAlph = MapModData.ITGSG
	MapModData.ITGSG = nil;
	if not tabAlph then
		return;
	end

	for i, v in pairs(tabAlph) do
		local index = v[1]
		local gPlot = Map.GetPlotByIndex(index)
		local gPlayer = gPlot:GetOwner();
    	local itemInstance = g_ListHolySites:GetInstance();
		local dn = v[2];
		if v[3] then
			dn = dn .. " ([ICON_RAZING])"
		end
		dn = dn .. v[4];
		itemInstance.Base:SetDisabled(false)
		itemInstance.Description:SetText(dn);

		itemInstance.SelectPlotButton:SetVoid1(index)
		itemInstance.SelectPlotButton:RegisterCallback( Mouse.eLClick, OnFindOnMapButtonClicked );

		if(bTickTock == false) then
			itemInstance.Box:SetColorVal(unpack(ltBlue));
		else
			itemInstance.Box:SetColorVal(unpack(dkBlue));
		end
		CivIconHookup( gPlayer, 32, itemInstance.Icon, itemInstance.CivIconBG, itemInstance.CivIconShadow, false, true);  

		itemInstance.Base:SetVoid1(index)
		itemInstance.Base:RegisterCallback(Mouse.eLClick, OnDetailedHolySite);

		itemInstance.HolySiteTraitLabel:SetText(string.upper(Locale.Lookup(v[5])))
		itemInstance.TraitText:SetText(Locale.ConvertTextKey(v[6], v[7]))



		local buttonWidth, buttonHeight = itemInstance.Base:GetSizeVal();
		local sizeY = 45 + itemInstance.TraitText:GetSizeY();
		itemInstance.Base:SetSizeVal(buttonWidth, sizeY);
		itemInstance.Box:SetSizeVal(buttonWidth + 20, sizeY);
		itemInstance.BounceAnim:SetSizeVal(buttonWidth + 20, sizeY + 3);
		itemInstance.BounceGrid:SetSizeVal(buttonWidth + 20, sizeY + 3);
		bTickTock = not bTickTock;

		ghsinfo = itemInstance;
	end

    Controls.HolySitesStack:CalculateSize();
    Controls.HolySitesStack:ReprocessAnchoring();
    Controls.HolySitesScroll:CalculateInternalSize();

	OnDetailedHolySite(lastHolySite)
end

function SetDetailedHolySite(index)
	lastHolySite = index;
end
LuaEvents.ITG_SetHolySiteUI.Add(SetDetailedHolySite)

function OnShowOnlyYoursCH( bIsChecked )
	bShowOnlyYours = bIsChecked;
	UpdateScreen();
end
Controls.HideShowOnlyYours:RegisterCheckHandler( OnShowOnlyYoursCH );

function OnHideShowOnlyReligion( bIsChecked )
	bShowOnlyReligion = bIsChecked;
	UpdateScreen();
end
Controls.HideShowOnlyReligion:RegisterCheckHandler( OnHideShowOnlyReligion );

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function ShowHideHandler( bIsHide, bInitState )
    if( not bIsHide ) then
    	UpdateScreen();
	end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );
