-------------------------------------------------
-- ITG Palaces
-------------------------------------------------
include( "IconSupport" );
include( "InstanceManager" );

local g_PolicyUpgrade = InstanceManager:new( "CityPalaceUpgrade", "Button", Controls.UpgradeStack );
local g_TabInfoInstance = InstanceManager:new( "TabInfoInstance", "InstanceLabel", Controls.TabInfoStack );
local g_tabInstance = {}
local g_CityID;
local g_pType;
local bShowTabInfo = true;
local lastLoadedPalaceID = -1;
local tabInfoTabData = {
{Color = "[COLOR_GREAT_PEOPLE_STORED]", Type = "Authority", Icon = "[ICON_ITG_AUTHORITY]", Tooltip = ""},
{Color = "[COLOR:33:190:247:255]", Type = "Science", Icon = "[ICON_RESEARCH]", Tooltip = ""},
{Color = "[COLOR:255:235:0:255]", Type = "Gold", Icon = "[ICON_GOLD]", Tooltip = ""},
{Color = "[COLOR:255:0:255:255]", Type = "Culture", Icon = "[ICON_CULTURE]", Tooltip = ""},
{Color = "", Type = "Production", Icon = "[ICON_PRODUCTION]", Tooltip = ""},
{Color = "[COLOR_SELECTED_TEXT]", Type = "WorkRate", Icon = "[ICON_WORKER]", Tooltip = ""},
{Color = "[COLOR_GREAT_PEOPLE_STORED]", Type = "GreatPeople", Icon = "[ICON_GREAT_PEOPLE]", Tooltip = ""},
{Color = "[COLOR:255:0:255:255]", Type = "Art", Icon = "[ICON_GREAT_ARTIST]", Tooltip = ""},
{Color = "", Type = "Presence", Icon = "[ICON_ITG_PRESENCE]", Tooltip = ""},
{Color = "[COLOR_POSITIVE_TEXT]", Type = "Happiness", Icon = "[ICON_HAPPINESS_1]", Tooltip = "", Suffix = "%"},
{Color = "", Type = "Legal", Icon = "[ICON_ITG_LEGAL_POWER]", Tooltip = "", Scale = 0.01},
{Color = "", Type = "Defense", Icon = "[ICON_STRENGTH]", Tooltip = "", Scale = 0.01},
{Color = "", Type = "Faith", Icon = "[ICON_PEACE]", Tooltip = ""},
{Color = "", Type = "Intrigue", Icon = "[ICON_ITG_POWER]", Tooltip = ""},
{Color = "", Type = "TradeAppeal", Icon = "[ICON_TRADE_WHITE]", Tooltip = ""},
}

function GetUpgradeInstance(i)
	return g_tabInstance[i];
end

local palaceTab = {}
local ITG_PalaceTypes = {}
local ITG_PalaceUpgradeTypes = {}
for row in DB.Query("SELECT * FROM ITG_Palaces") do
	ITG_PalaceTypes[row.Type] = row.ID;
	palaceTab[row.ID] = {DisplayItems = {}, AnimTexture = row.AnimTexture}
end
for row in DB.Query("SELECT * FROM ITG_PalaceUpgrades") do
	ITG_PalaceUpgradeTypes[row.Type] = row.ID;
end
local sBtC = {Upgrade = 1, Upgrade2 = 1, Upgrade3 = 1, NoUpgrade = 1, NoUpgrade2 = 1};
for row in DB.Query("SELECT * FROM ITG_Palace_Sprites") do
	local rowid = ITG_PalaceTypes[row.PalaceType];
	local rTab = {};
	for a, b in pairs(row) do
		if sBtC[a] then
			rTab[a] = ITG_PalaceUpgradeTypes[b];
		else
			rTab[a] = b;
		end
	end
	if row.AfterAnim then
		if not palaceTab[rowid].DisplayItemsPastAnim then
			palaceTab[rowid].DisplayItemsPastAnim = {};
		end
		table.insert(palaceTab[rowid].DisplayItemsPastAnim, rTab)
	else
		table.insert(palaceTab[rowid].DisplayItems, rTab)
	end
end

function GetITGPalaceDBData()
	return palaceTab;
end

-------------------------------------------------
-------------------------------------------------
function HideShowPalacesBack(val)
	Controls.ITG_Palace_Panel:SetHide(val == -1)
end

function LoadPalaceAssets(g_pType)
	lastLoadedPalaceID = g_pType
	for i = 1, 500 do
		if Controls["ITGPalace_Comp_" .. i] then
			Controls["ITGPalace_Comp_" .. i]:SetHide(true)
			if palaceTab[g_pType].DisplayItems[i] then
				Controls["ITGPalace_Comp_" .. i]:UnloadTexture()
				Controls["ITGPalace_Comp_" .. i]:SetTextureAndResize(palaceTab[g_pType].DisplayItems[i].Texture)
				Controls["ITGPalace_Comp_" .. i]:SetOffsetVal(palaceTab[g_pType].DisplayItems[i].OffsetX,palaceTab[g_pType].DisplayItems[i].OffsetY)
			end
		else
			break;
		end	
	end
	for i = 1, 500 do
		if Controls["ITGPalace_AfterAnim_" .. i] then
			Controls["ITGPalace_AfterAnim_" .. i]:SetHide(true)
			if palaceTab[g_pType].DisplayItemsPastAnim and palaceTab[g_pType].DisplayItemsPastAnim[i] then
				Controls["ITGPalace_AfterAnim_" .. i]:UnloadTexture()
				Controls["ITGPalace_AfterAnim_" .. i]:SetTextureAndResize(palaceTab[g_pType].DisplayItemsPastAnim[i].Texture)
				Controls["ITGPalace_AfterAnim_" .. i]:SetOffsetVal(palaceTab[g_pType].DisplayItemsPastAnim[i].OffsetX,palaceTab[g_pType].DisplayItemsPastAnim[i].OffsetY)
			end
		else
			break;
		end	
	end
	Controls.ITGPalace_AlphaAnimImage:SetTextureAndResize(palaceTab[g_pType].AnimTexture)
end

function SetToEndAllAnims()
	Controls.ITGPalace_AlphaAnim:SetToEnd()
end

function PalaceUpgradeChosen(v)
	if Players[Game.GetActivePlayer()]:IsTurnActive() then
	--animations here!
		LuaEvents.ITG_CallPalaceUpgrade(g_CityID, v)
		Controls.ITGPalace_AlphaAnim:SetHide(false)
		Controls.ITGPalace_AlphaAnim:SetToBeginning();
		Controls.ITGPalace_AlphaAnim:Play();
		UpdateScreen(true)
	end
end

function UpdateScreen(bNotInit)
	if not bNotInit then
		HideShowPalacesBack(-1)
		g_CityID = nil;
	end
	g_PolicyUpgrade:ResetInstances();
	local iPlayer = Game.GetActivePlayer();
	LuaEvents.ITGSG_RequestGT();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end

	local chID;
	Controls.TabInfoStack:SetHide(true);
	Controls.InfoLog:SetHide(true);
	Controls.PalacesScroll:ClearEntries();
	for iCity in Players[iPlayer]:Cities() do
		local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded()
		if gData.Cities[cID].Palace then
			local text = iCity:GetName() .. " (" .. gData.Cities[cID].Palace.NumUpgrades .. ")"
			if gData.Cities[cID].PalaceUpgrades > 0 then
				text = text .. " [COLOR_POSITIVE_TEXT]+" .. gData.Cities[cID].PalaceUpgrades .. "[ENDCOLOR]"
				chID = iCity:GetID();
			end
			local controlTable = {};
			Controls.PalacesScroll:BuildEntry( "InstanceOne", controlTable );
			controlTable.Button:SetVoid1(iCity:GetID());
			controlTable.Button:SetText(text);
			if not chID then
				chID = iCity:GetID()
			end
		end
	end
	Controls.PalacesScroll:CalculateInternals();

	Controls.AnimFreeUpgrades:SetHide(gData.Players[iPlayer].FreePalacePoints <= 0)
	Controls.FreeUpgradesLabel:SetText("You have " .. gData.Players[iPlayer].FreePalacePoints .. " free upgrades.")

	if g_CityID or chID then
		ChooseCityIDPalace(g_CityID or chID)
	end
end

function ChooseCityIDPalace(v1)
	local iPlayer = Game.GetActivePlayer();
	LuaEvents.ITGSG_RequestGT();
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	g_CityID = v1;
	local iCity = Players[iPlayer]:GetCityByID(v1);
	local cID = iCity:Plot():GetPlotIndex() .. "," .. iCity:GetGameTurnFounded()
	g_pType = gData.Cities[cID].Palace.Type;
	local text = iCity:GetName() .. " (" .. gData.Cities[cID].Palace.NumUpgrades .. ")"
	if gData.Cities[cID].PalaceUpgrades > 0 then
		text = text .. " [COLOR_POSITIVE_TEXT]+" .. gData.Cities[cID].PalaceUpgrades .. "[ENDCOLOR]"
	end
	Controls.PalacesScroll:GetButton():SetText(text)
	if lastLoadedPalaceID ~= g_pType then
		LoadPalaceAssets(g_pType)
	end
	for i, v in pairs(palaceTab[g_pType].DisplayItems) do
		local bHide = false;
		if v.Upgrade and (not gData.Cities[cID].Palace.Upgrades[v.Upgrade]) then
			bHide = true;
		end
		if v.Upgrade2 and (not gData.Cities[cID].Palace.Upgrades[v.Upgrade2]) then
			bHide = true;
		end
		if v.Upgrade3 and (not gData.Cities[cID].Palace.Upgrades[v.Upgrade3]) then
			bHide = true;
		end
		if v.NoUpgrade and (gData.Cities[cID].Palace.Upgrades[v.NoUpgrade]) then
			bHide = true;
		end
		if v.NoUpgrade2 and (gData.Cities[cID].Palace.Upgrades[v.NoUpgrade2]) then
			bHide = true;
		end			
		Controls["ITGPalace_Comp_" .. i]:SetHide(bHide)
	end
	for i, v in pairs(palaceTab[g_pType].DisplayItemsPastAnim or {}) do
		local bHide = false;
		if v.Upgrade and (not gData.Cities[cID].Palace.Upgrades[v.Upgrade]) then
			bHide = true;
		end
		if v.Upgrade2 and (not gData.Cities[cID].Palace.Upgrades[v.Upgrade2]) then
			bHide = true;
		end
		if v.Upgrade3 and (not gData.Cities[cID].Palace.Upgrades[v.Upgrade3]) then
			bHide = true;
		end
		if v.NoUpgrade and (gData.Cities[cID].Palace.Upgrades[v.NoUpgrade]) then
			bHide = true;
		end
		if v.NoUpgrade2 and (gData.Cities[cID].Palace.Upgrades[v.NoUpgrade2]) then
			bHide = true;
		end			
		Controls["ITGPalace_AfterAnim_" .. i]:SetHide(bHide)
	end
	g_PolicyUpgrade:ResetInstances();

	g_tabInstance = {}

	bHideTabInfo = not bShowTabInfo
	if (gData.Cities[cID].Palace.NumUpgrades < 10) and ((gData.Cities[cID].PalaceUpgrades > 0) or (gData.Players[iPlayer].FreePalacePoints > 0)) then
		LuaEvents.ITGSG_RequestEditedPalaceUpgrades(v1);
		local ndata = MapModData.ITGSG;
		MapModData.ITGSG = nil;
		if ndata then
			for i, data in pairs(ndata) do
				local controlTable = g_PolicyUpgrade:GetInstance();

				controlTable.Button:SetOffsetVal(data.OffsetX, data.OffsetY);
				controlTable.Button:SetVoid1( data.ID );
				controlTable.Button:RegisterCallback( Mouse.eLClick, PalaceUpgradeChosen );
				local tt = "[COLOR_POSITIVE_TEXT]" .. data.Name .. "[ENDCOLOR][NEWLINE]----------------[NEWLINE]" .. data.Yields .. "[ENDCOLOR]----------------"
				if data.Help ~= "" then
					tt = tt .. "[NEWLINE]" .. data.Help;
				end
				controlTable.Button:SetToolTipString(tt);
				table.insert(g_tabInstance, controlTable)

			end
		end
	end
	if gData.Players[iPlayer].FreePalacePoints > 0 then
		bHideTabInfo = true;
	end
	Controls.TabInfoStack:SetHide(bHideTabInfo);
	Controls.InfoLog:SetHide(bHideTabInfo)
	if not bHideTabInfo then
		g_TabInfoInstance:ResetInstances();
		for i, data in pairs(tabInfoTabData) do
			local controlTable = g_TabInfoInstance:GetInstance();
			controlTable.InstanceLabel:SetText((data.Color or "") .. gData.Cities[cID].Palace[data.Type] * (data.Scale or 1) .. (data.Suffix or "") .. data.Icon)
			controlTable.InstanceLabel:SetToolTipString(data.Tooltip or "")
		end
		Controls.TabInfoStack:CalculateSize()
		Controls.TabInfoStack:ReprocessAnchoring()
		-- info log
		local sToolTip = ""
		for jkj, bjb in pairs(gData.Cities[cID].PalaceLog) do
			if jkj ~= 1 then
				sToolTip = sToolTip .. "[NEWLINE]"
			end
			sToolTip = sToolTip .. "[ICON_BULLET]" .. bjb[1] .. "T: "
			if bjb[2] == 1 then
				if jkj == 1 then
					sToolTip = sToolTip .. "Created by " .. GetPlayerNiceName(bjb[3]) .. "."
				else
					sToolTip = sToolTip .. "Rebuilt by " .. GetPlayerNiceName(bjb[3]) .. "."
				end
			elseif bjb[2] == 2 then
				sToolTip = sToolTip .. "Inherited by " .. GetPlayerNiceName(bjb[3]) .. "."
			elseif bjb[2] == 3 then
				sToolTip = sToolTip .. "Expanded by " .. GetPlayerNiceName(bjb[3]) .. "."
			elseif bjb[2] == 4 then
				sToolTip = sToolTip .. "Destroyed by " .. GetPlayerNiceName(bjb[3]) .. "'s troops." 
			elseif bjb[2] == 5 then
				sToolTip = sToolTip .. "Damaged by " .. GetPlayerNiceName(bjb[3]) .. "'s troops (-" .. bjb[4] .. ")." 
			elseif bjb[2] == 6 then
				sToolTip = sToolTip .. "Damaged by rioting peasants (-" .. bjb[3] .. ")." 
			elseif bjb[2] == 7 then
				sToolTip = sToolTip .. "Damaged by " .. GetPlayerNiceName(bjb[3]) .. "'s bombardment (-1)." 
			elseif bjb[2] == 8 then
				sToolTip = sToolTip .. "Destroyed during coup started by " .. Locale.Lookup(gData.Globals.Governors[bjb[3]].Name) .. "."
			elseif bjb[2] == 9 then
				sToolTip = sToolTip .. "Damaged during coup started by " .. Locale.Lookup(gData.Globals.Governors[bjb[3]].Name) .. " (-1)."
			end
		end
		Controls.InfoLog:SetToolTipString(sToolTip)
	end
--	Upgrades
	HideShowPalacesBack(g_pType)
end
Controls.PalacesScroll:RegisterSelectionCallback(ChooseCityIDPalace); 
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function ShowHideHandler( bIsHide, bInitState )
    if( not bIsHide ) then
		SetToEndAllAnims()
    	UpdateScreen();
	end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );

function OnTabShow( bIsChecked )
	bShowTabInfo = bIsChecked;
	if g_CityID then
		ChooseCityIDPalace(g_CityID)
	end
end
Controls.ShowInfoCheck:RegisterCheckHandler( OnTabShow );

function GetPlayerNiceName(i)
	if Game.GetActivePlayer() == i then
		return "You"
	elseif Teams[Game.GetActiveTeam()]:IsHasMet(Players[i]:GetTeam()) then
		return Players[i]:GetName()
	end
	return "Unknown Player"
end
