-------------------------------------------------
-- Happiness Info
-------------------------------------------------
include( "IconSupport" );
include( "InstanceManager" );

local g_StockPaperManager1 = GenerationalInstanceManager:new( "StockPaperInstance", "Base", Controls.StackVertical1);
local g_StockPaperManager2 = GenerationalInstanceManager:new( "StockPaperInstance", "Base", Controls.StackVertical2);
local g_LogStockManager = GenerationalInstanceManager:new( "TransactionMessageInstance", "Base", Controls.IntrigueMessageStack);


--g_LineSegmentInstanceManager = InstanceManager:new("GraphLineInstance","LineSegment", Controls.HelpID);

local m_CurrentStock = 1;
local cYourGold;
local cOtherGold;
local ltBlue = {19/255,32/255,46/255,120/255};
local dkBlue = {12/255,22/255,30/255,120/255};
local hStockInstanceTab = {}
local awS1, awS2 = 2, 2;
local bCheckRelativeGraph = false;

function SetVariables(a, b)
	awS1, awS2 = a, b;
end

function GetTabsHelp()
	return hStockInstanceTab;
end
-------------------------------------------------
-------------------------------------------------

function UpdateScreen()
	g_StockPaperManager1:ResetInstances();
	g_StockPaperManager2:ResetInstances();
	g_LogStockManager:ResetInstances()
	ShowHideLogControls(true)
	LuaEvents.ITG_CanPlayerEnterStockExchange()
	local hCan = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if hCan then
		UpdateStockActiveNow()
		if m_CurrentStock == 1 then
			CallStockPaper("StockExchange")
		elseif m_CurrentStock == 2 then
			CallStockPaper("StockCurrency")
		elseif  m_CurrentStock == 3 then
			CallLogMessages()
		end
	else
		Controls.TurnsLeft:SetText("");
	end
	Controls.MastersOfCoin:SetHide(not hCan)
	Controls.CheckRelativeGraph:SetHide(not hCan)
	Controls.DisabledStock:SetHide(hCan)
	Controls.StockButton:SetDisabled(not hCan)
	Controls.CurrencyInvestButton:SetDisabled(not hCan)
	Controls.TransactionsButton:SetDisabled(not hCan)
end

function StockPaperInstanceIncrease(i)
	ChangeInvestOfPaper(i, 1)
end

function StockPaperInstanceRemove(i)
	ChangeInvestOfPaper(i, -1)
end

function ChangeInvestOfPaper(i, change)
	local iPlayer = Game.GetActivePlayer() ;
	if not Players[iPlayer]:IsTurnActive() then
		return;
	end
	if m_CurrentStock == 1 then
		LuaEvents.ITGSG_PlayerChangeInvestStock(iPlayer, i, change, "StockExchange")
	else
		LuaEvents.ITGSG_PlayerChangeInvestStock(iPlayer, i, change, "StockCurrency")
	end
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then -- changed nothing
		return;
	end
	if gData[1] == 0 then
		hStockInstanceTab[i].Text1:SetText("N/A")
	else
		hStockInstanceTab[i].Text1:SetText(gData[1] .. "[ICON_INVEST]")
	end
	Controls.TurnsLeft:SetText("Available investing sources: " .. gData[2] .. " [ICON_INVEST]")
end

function UpdateStockActiveNow()
	LuaEvents.ITGSG_RequestGT()
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	local iPlayer = Game.GetActivePlayer()
	if gData.Globals.StockIsOver then
		Controls.TurnsLeft:SetText("No more sessions during this Era.")
	elseif gData.Globals.StockActive then
		Controls.TurnsLeft:SetText("Available investment left: " .. gData.Players[iPlayer].StockInvestAvailable .. " [ICON_INVEST].")
	else
		local hTT = 15 - gData.ITGTurn % 14
		if hTT > 14 and (gData.ITGTurn > 2) then
			hTT = 1;
		end
		Controls.TurnsLeft:SetText("Next session starts in " .. hTT .. " turns.")
	end
	UpdateMasterOfCoin(gData.Players[iPlayer].MasterOfCoin)
end

function UpdateMasterOfCoin(moc)
	if moc then
		Controls.MastersOfCoinLabel:SetText("Master of Coin: [COLOR_POSITIVE_TEXT]ON[ENDCOLOR]");
		Controls.MastersOfCoin:SetToolTipString("Master of Coin is currently managing your investments.");
	else
		Controls.MastersOfCoinLabel:SetText("Master of Coin: [COLOR_NEGATIVE_TEXT]OFF[ENDCOLOR]");
		Controls.MastersOfCoin:SetToolTipString("Master of Coin can manage your investments if you appoint one.");
	end
end

function OnPlayerMOW( )
	if Players[Game.GetActivePlayer()]:IsTurnActive() then
		LuaEvents.ITGSG_PlayerChangeMOCSettings();
		UpdateScreen()
	end
end
Controls.MastersOfCoin:RegisterCallback( Mouse.eLClick, OnPlayerMOW );

function ShowHideLogControls(bHide)
	Controls.LogMessageHeaders:SetHide(bHide)
	Controls.LogMessageScrollPanel:SetHide(bHide)
	Controls.NoInvestAvailable:SetHide(bHide)
end

function CallLogMessages()
	ShowHideLogControls(false)
	local iPlayer = Game.GetActivePlayer() ;
	LuaEvents.ITGSG_PrepareStockLogData(iPlayer)
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return;
	end
	g_LogStockManager:ResetInstances()
	if #gData > 0 then
		for a, b in pairs(gData) do
			local itemInstance = g_LogStockManager:GetInstance();
			itemInstance.TurnNum:SetText(b[1])
			itemInstance.CurrencyPower:SetText(b[2])
			itemInstance.GoldChange:SetText(b[3])
			itemInstance.Message:SetText(b[4])
		end
		Controls.NoInvestAvailable:SetHide(true)
	else
		Controls.NoInvestAvailable:SetHide(false)
	end
    Controls.IntrigueMessageStack:CalculateSize();
    Controls.IntrigueMessageStack:ReprocessAnchoring();
    Controls.LogMessageScrollPanel:CalculateInternalSize();
end

function OnCheckClicked( )
	bCheckRelativeGraph = not bCheckRelativeGraph
	if m_CurrentStock == 1 then
		CallStockPaper("StockExchange")
	elseif m_CurrentStock == 2 then
		CallStockPaper("StockCurrency")
	end
end
Controls.CheckRelativeGraph:RegisterCallback( Mouse.eLClick, OnCheckClicked );
		
function CallStockPaper(type)
	local iPlayer = Game.GetActivePlayer() ;
    local pPlayer = Players[iPlayer];
	LuaEvents.ITGSG_RequestGT()
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return;
	end
	
	g_StockPaperManager1:ResetInstances();
	g_StockPaperManager2:ResetInstances();
	LuaEvents.ITGSG_RequestStockBasicData()
	local sbData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	
	local bStockActiveNow = gData.Globals.StockActive;

	local vFinC = 0;

	for i, stockD in pairs(gData[type]) do
		local bContinue = true;
		if stockD.Ignore then
			bContinue = false;
		end
		if type == "StockCurrency" then
			if i == iPlayer then
				bContinue = false;
			end
			if (i ~= iPlayer) and (not Teams[Players[i]:GetTeam()]:IsHasMet(Players[iPlayer]:GetTeam())) then
				bContinue = false;
			end
			if gData.Players[i].MintPower <= 0 then
				bContinue = false;
			end
		end
		if bContinue then
			local itemInstance;
			vFinC = vFinC + 1;
			if vFinC % 2 == 1 then
				itemInstance = g_StockPaperManager1:GetInstance();
			else
				itemInstance = g_StockPaperManager2:GetInstance();
			end
			if stockD.PlayerInvest[iPlayer] then
				itemInstance.Text1:SetText(stockD.PlayerInvest[iPlayer] .. "[ICON_INVEST]")
			else
				itemInstance.Text1:SetText("N/A")
			end
			itemInstance.BoxGraph:DestroyAllChildren();

			itemInstance.TextLastInvest:SetHide(true)
			if bStockActiveNow or (gData.ITGTurn > 98) then
				for aaa, bbb in pairs(gData.Players[iPlayer].InvestLog) do
					if bbb[7] == m_CurrentStock then
						if i == bbb[1] then
							if gData.ITGTurn - bbb[6] < 3 then 
								local tString =  bbb[2] .. "[ICON_INVEST] ("
								if bbb[3] >= 0 then
									tString = tString .. "[COLOR_POSITIVE_TEXT]+" .. bbb[3] .. "%[ENDCOLOR])" 
								else
									tString = tString .. "[COLOR_NEGATIVE_TEXT]" .. bbb[3] .. "%[ENDCOLOR])" 
								end

								itemInstance.TextLastInvest:SetText(tString);
								itemInstance.TextLastInvest:SetHide(false)
								itemInstance.TextLastInvest:SetAlpha(0.75)
								break;
							end
						end
					end
				end
			end

			local mtt;
			local baseVal = gData.Globals.CurrencyBaseVal;
			if type == "StockCurrency" then
				itemInstance.HnHH1:SetHide(false)
				sName = Players[i]:GetCivilizationAdjective() .. " Currency"
				mtt = "Reward from investing in [ICON_INVEST] Currency is based on rise of [ICON_INVEST] Currency Power within next session. Currently [COLOR_SELECTED_TEXT]" .. Players[i]:GetCivilizationAdjective() .. " Currency[ENDCOLOR] has " .. gData.Players[i].MintPower .. " [ICON_INVEST] Currency Power.[NEWLINE][NEWLINE]Base value: [COLOR_SELECTED_TEXT]" .. gData.Globals.CurrencyBaseVal .. "[ENDCOLOR]" ;
				CivIconHookup( i, 32, itemInstance.Icon, itemInstance.CivIconBG, itemInstance.CivIconShadow, false, true);  
			else
				itemInstance.HnHH1:SetHide(true)
				sName = sbData[i][1];
				baseVal = sbData[i][3]
				mtt = sbData[i][2] .. "[NEWLINE][NEWLINE]Base value: [COLOR_SELECTED_TEXT]" .. sbData[i][3] .. "[ENDCOLOR]" ;
			end
			itemInstance.Name:SetText(sName)
			local sLength = string.len(sName)
			local gridTopSizeX = 100;
			if sLength > 18 then
				gridTopSizeX = math.min(160, math.max(100, sLength * 7))
				itemInstance.Name:SetFontByName("TwCenMT14")
			elseif sLength > 10 then
				gridTopSizeX = math.min(160, math.max(100, sLength * 9))
				itemInstance.Name:SetFontByName("TwCenMT18")
			else
				gridTopSizeX = math.min(160, math.max(100, sLength * 12))
				itemInstance.Name:SetFontByName("TwCenMT22")
			end
			itemInstance.ExGrid7:SetSizeX(gridTopSizeX)

			local tt = ""
			local minV, maxV;
			for gj, ddj in pairs(stockD.DataRates) do
				if (not minV) or (minV > ddj) then
					minV = ddj;
				end
				if (not maxV) or (maxV < ddj) then
					maxV = ddj;
				end
			end
			if minV then
				minV = minV + baseVal;
				maxV = maxV + baseVal;
			end
			local hs0, hsM = 0, 120;
			local vs0, vsM = 0, 120;
			local vExS = 5;
			local xDif = 20;
			local grandYoverRide;
			if minV == maxV then
				grandYoverRide = (vs0 + vsM) / 2;
			elseif not bCheckRelativeGraph then
				minV = -baseVal;
			end
			for gj, ddj in pairs(stockD.DataRates) do
				local instance = {};
				ContextPtr:BuildInstanceForControl( "GraphLineInstance", instance, itemInstance.BoxGraph );
				local startX, startY =  xDif * (gj - 1), grandYoverRide or (vsM - math.floor(vExS + (ddj + baseVal - minV) / (maxV - minV) * (vsM - vs0 - 2 * vExS)));
				local endX, endY = startX + 1, startY
				instance.LineSegment:SetWidth(awS1)
				if gj == 1 then
					tt = tt .. "[NEWLINE][ICON_BULLET]" .. ddj;
				else
					local pcsd = math.ceil((ddj + baseVal) / (stockD.DataRates[gj - 1] + baseVal)* 100) - 100;
					if pcsd > 0 then
						tt = tt .. "[NEWLINE][ICON_BULLET]" .. ddj .. " ([COLOR_POSITIVE_TEXT]+" .. pcsd .. "%[ENDCOLOR])"
					else
						tt = tt .. "[NEWLINE][ICON_BULLET]" .. ddj .. " ([COLOR_NEGATIVE_TEXT]" .. pcsd .. "%[ENDCOLOR])"
					end
				end
				if gj < #stockD.DataRates then
					endX = endX + xDif - 1;
					endY = grandYoverRide or (vsM - math.floor(vExS + (stockD.DataRates[gj + 1] + baseVal - minV) / (maxV - minV) * (vsM - vs0 - 2 * vExS)));
					if ddj > stockD.DataRates[gj + 1] then
						instance.LineSegment:SetColorVal(1.0, 0.25, 0.25, 0.8);
					elseif ddj == stockD.DataRates[gj + 1] then
						instance.LineSegment:SetColorVal(0.75, 0.75, 0, 0.8);
					else
						instance.LineSegment:SetColorVal(0.25, 1.0, 0.25, 0.8);
					end
				else
					instance.LineSegment:SetWidth(awS2)
					if gj > 1 then
						if ddj > stockD.DataRates[gj - 1] then
							instance.LineSegment:SetColorVal(0.25, 1.0, 0.25, 0.8);
						elseif ddj == stockD.DataRates[gj - 1] then
							instance.LineSegment:SetColorVal(0.75, 0.75, 0, 0.8);
						else
							instance.LineSegment:SetColorVal(1.0, 0.25, 0.25, 0.8);
						end
					else
						instance.LineSegment:SetColorVal(0.75, 0.75, 0, 0.8);
					end
				end
				instance.LineSegment:SetStartVal(startX, startY);
				instance.LineSegment:SetEndVal(endX, endY);
				instance.LineSegment:SetHide(false);	
			end
			if tt == "" then
				mtt = mtt .. "[NEWLINE][NEWLINE]There is no data about rates yet."
			else
				mtt = mtt .. "[NEWLINE][NEWLINE]Data Rates:" .. tt;
			end
			itemInstance.AddInvest:SetDisabled((not bStockActiveNow) or ((type ~= "StockCurrency") and  (not stockD.Active)))
			itemInstance.AddInvest:SetVoid1(i)
			itemInstance.RemoveInvest:SetDisabled((not bStockActiveNow) or ((type ~= "StockCurrency") and  (not stockD.Active)))
			itemInstance.RemoveInvest:SetVoid1(i)
			itemInstance.AddInvest:RegisterCallback( Mouse.eLClick, StockPaperInstanceIncrease );
			itemInstance.RemoveInvest:RegisterCallback( Mouse.eLClick, StockPaperInstanceRemove );

			if (type ~= "StockCurrency") and (not stockD.Active) then
				itemInstance.Base:SetAlpha(0.7)
			else
				itemInstance.Base:SetAlpha(1.0)
			end
			itemInstance.Base:SetToolTipString(mtt);
			hStockInstanceTab[i] = itemInstance
		end
	end
    Controls.StackVertical1:CalculateSize();
    Controls.StackVertical2:CalculateSize();
    Controls.StackVertical1:ReprocessAnchoring();
    Controls.StackVertical2:ReprocessAnchoring();
    Controls.MainScrollVertical:CalculateInternalSize();
end

function OnStockPaperButton()
	Controls.StockSelectHighlight:SetHide(false);
	Controls.CurrencyInvestSelectHighlight:SetHide(true);
	Controls.TransactionsSelectHighlight:SetHide(true);
    m_CurrentStock = 1;
	UpdateScreen()
end
Controls.StockButton:RegisterCallback( Mouse.eLClick, OnStockPaperButton );

function OnCurrencyStockButton()
	Controls.StockSelectHighlight:SetHide(true);
	Controls.CurrencyInvestSelectHighlight:SetHide(false);
	Controls.TransactionsSelectHighlight:SetHide(true);
    m_CurrentStock = 2;
	UpdateScreen()
end
Controls.CurrencyInvestButton:RegisterCallback(Mouse.elClick, OnCurrencyStockButton);


function OnTransactionLogExchangeButton()
	Controls.StockSelectHighlight:SetHide(true);
	Controls.CurrencyInvestSelectHighlight:SetHide(true);
	Controls.TransactionsSelectHighlight:SetHide(false);
    m_CurrentStock = 3;
	UpdateScreen()
end
Controls.TransactionsButton:RegisterCallback(Mouse.elClick, OnTransactionLogExchangeButton);
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function ShowHideHandler( bIsHide, bInitState )
    if( not bIsHide ) then
    	UpdateScreen();
	end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );
Controls.HelpImageId:SetAlpha(0.33)