-------------------------------------------------
-- City Trade Info
-------------------------------------------------
include( "IconSupport" );
include( "InstanceManager" );

local g_TrTargets = GenerationalInstanceManager:new( "DestinationInstance", "Base", Controls.TargetStack);
local gc_Tab = {};
local gc_City;
local m_iDomain = DomainTypes.DOMAIN_LAND;
local ltBlue = {19/255,32/255,46/255,120/255};
local dkBlue = {12/255,22/255,30/255,120/255};
local m_BasicInfo_IM = InstanceManager:new("TradeInfoInstance", "TradeStackInfo", Controls.TInfoStack)

function GenerateCityList(iPlayer)
	gc_Tab = {};
	gc_City = nil;
	for iCity in Players[iPlayer]:Cities() do
		table.insert(gc_Tab, iCity:GetID());
	end
end

function GetCurrentCity()
	if not gc_City then
		gc_City = gc_Tab[1];
	end
end

function CenterUnitOnMap(void)
	if void then
		local iUnit = Players[Game.GetActivePlayer()]:GetUnitByID(void);
		if iUnit then
			UI.LookAt(Map.GetPlot(iUnit:GetX(), iUnit:GetY()))
		end
	end
end

function OnNextCityButton()
	if gc_Tab[#gc_Tab] == gc_City then
		gc_City = gc_Tab[1];
	else
		for i, v in pairs(gc_Tab) do
			if v == gc_City then
				gc_City = gc_Tab[i + 1];
				break
			end
		end
	end
	NewCitySelected()
end
Controls.NextCityButton:RegisterCallback( Mouse.eLClick, OnNextCityButton );

-------------------------------------------------
-------------------------------------------------
function OnPrevCityButton()
	if gc_Tab[1] == gc_City then
		gc_City = gc_Tab[#gc_Tab];
	else
		for i, v in pairs(gc_Tab) do
			if v == gc_City then
				gc_City = gc_Tab[i - 1];
				break
			end
		end
	end
	NewCitySelected()
end
Controls.PrevCityButton:RegisterCallback( Mouse.eLClick, OnPrevCityButton );

function NewCitySelected()
	DoBasicInfoCity()
	PrepareDestinationInfo()
end

function DoBasicInfoCity()
	local iPlayer = Game.GetActivePlayer();
	local gCity = Players[iPlayer]:GetCityByID(gc_City);
	LuaEvents.ITGSG_RequestCityTradeInfo(iPlayer, gc_City);
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	local percent = gData.StoredLandTradeRoute / gData.LandTradeRouteCost;
	Controls.TradeLandBox:SetToolTipString(gData.LandTradeRouteChangeTT);
	Controls.TradeLandMeter:SetPercent( percent );
	percent = gData.StoredNavalTradeRoute / gData.NavalTradeRouteCost;
	Controls.TradeNavalBox:SetToolTipString(gData.NavalTradeRouteChangeTT);
	Controls.TradeNavalMeter:SetPercent( percent );
	if(gCity:IsCapital())then
		Controls.CityNameTitleBarLabel:SetText("[ICON_CAPITAL]" .. gCity:GetName());
	else
		Controls.CityNameTitleBarLabel:SetText(gCity:GetName());
	end
	if m_iDomain == DomainTypes.DOMAIN_LAND then
		Controls.TradeLandBox:SetAlpha(1);
		Controls.TradeNavalBox:SetAlpha(0.5);
	else
		Controls.TradeNavalBox:SetAlpha(1);
		Controls.TradeLandBox:SetAlpha(0.5);
	end
end

function OnLandSelected()
	if m_iDomain ~= DomainTypes.DOMAIN_LAND then
		m_iDomain = DomainTypes.DOMAIN_LAND;
		Controls.TradeLandBox:SetAlpha(1);
		Controls.TradeNavalBox:SetAlpha(0.5);
		PrepareDestinationInfo()
	end
end
function OnNavalSelected()
	if m_iDomain ~= DomainTypes.DOMAIN_SEA then
		m_iDomain = DomainTypes.DOMAIN_SEA;
		Controls.TradeNavalBox:SetAlpha(1);
		Controls.TradeLandBox:SetAlpha(0.5);
		PrepareDestinationInfo()
	end
end
Controls.TradeLandBox:RegisterCallback( Mouse.eLClick, OnLandSelected );
Controls.TradeNavalBox:RegisterCallback( Mouse.eLClick, OnNavalSelected );
-------------------------------------------------
-------------------------------------------------
function UpdateTradeScreen()
	local iPlayer = Game.GetActivePlayer();
	local numCities = Players[iPlayer]:GetNumCities()
	Controls.NoCitiesAvailable:SetHide(numCities ~= 0)
	Controls.CityPanel:SetHide(numCities == 0);
	if numCities == 0 then
		return;
	end
	GenerateCityList(iPlayer);
	GetCurrentCity()
	DoBasicInfoCity()
	Controls.NextCityButton:SetDisabled(numCities == 1);
	Controls.PrevCityButton:SetDisabled(numCities == 1);
	PrepareDestinationInfo()
	PrepareTradeInfo(iPlayer)
end

function PrepareDestinationInfo()
	g_TrTargets:ResetInstances();
	local iPlayer = Game.GetActivePlayer();
	local gCity = Players[iPlayer]:GetCityByID(gc_City);
	LuaEvents.ITGSG_RequestCityTradeTargets(iPlayer, gc_City, m_iDomain);
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if not gData then
		return
	end
	if #gData == 0 then
		Controls.NoTargetsAvailable:SetHide(false)
	else
		Controls.NoTargetsAvailable:SetHide(true)
		for i, v in pairs(gData) do
			local itemInstance = g_TrTargets:GetInstance();
			local tCity = Map.GetPlotByIndex(v[1]):GetPlotCity();
			CivIconHookup( tCity:GetOwner(), 32, itemInstance.Icon, itemInstance.CivIconBG, itemInstance.CivIconShadow, false, true);
			itemInstance.CityName:SetText(tCity:GetName());
			local sColour = ""
			if tCity:IsPlague() then
				sColour = "[COLOR_CITY_GREEN]"
			elseif Teams[Game.GetActiveTeam()]:IsAtWar(Players[tCity:GetOwner()]:GetTeam()) then
				sColour = "[COLOR_NEGATIVE_TEXT]"
			end
			itemInstance.TrAttr:SetText(sColour .. v[2] .. "[ICON_TRADE_WHITE]")
			itemInstance.TrAttr:SetToolTipString(v[8]);
			if v[4] ~= 0 then
				itemInstance.BaseGold:SetText(v[3] .. "(" .. v[4] .. ")[ICON_GOLD]")
			else
				itemInstance.BaseGold:SetText(v[3] .. "[ICON_GOLD]")
			end
			local exString = ""
			if v[5] then
				exString = exString .. "[ICON_CULTURE]"
			end
			if v[6] then
				exString = exString .. "[ICON_RESEARCH]"
			end
			if v[7] then
				exString = exString .. "[ICON_INFLUENCE]"
			end
			itemInstance.TrOther:SetText(exString)
			if(bTickTock == false) then
				itemInstance.Base:SetColorVal(unpack(ltBlue));
			else
				itemInstance.Base:SetColorVal(unpack(dkBlue));
			end
			bTickTock = not bTickTock;
		end
	end

    Controls.TargetStack:CalculateSize();
    Controls.TargetStack:ReprocessAnchoring();
    Controls.TargetScroll:CalculateInternalSize();
end

function AddTradeInfo(sText, sToolTip)
	local instance = m_BasicInfo_IM:GetInstance();
	instance.InfoLabel:SetText(sText);
	instance.TradeStackInfo:SetToolTipString(sToolTip);
end

function PrepareTradeInfo(iPlayer)
	m_BasicInfo_IM:ResetInstances();
	for i = 1, 20 do
		local sText = Locale.Lookup("TXT_KEY_ITG_UI_TRADE_INFO_" .. i)
		if sText == "TXT_KEY_ITG_UI_TRADE_INFO_" .. i then
			break;
		else
			AddTradeInfo(sText, Locale.Lookup("TXT_KEY_ITG_UI_TRADE_INFO_" .. i .. "_TT"))
		end
	end
	LuaEvents.ITGSG_RequestTradeModifiers(iPlayer);
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	if gData then
		AddTradeInfo("[ICON_GOLD]% Gold modifiers:", "This list includes only player's modifiers, invidual Cities may be affected by more specific ones.")
		for a, b in pairs(gData) do
			AddTradeInfo(b[1], b[2]);
		end
	end
    Controls.TInfoStack:CalculateSize();
    Controls.TInfoStack:ReprocessAnchoring();
    Controls.TInfoScroll:CalculateInternalSize();
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function ShowHideHandler( bIsHide, bInitState )
    if( not bIsHide ) then
    	UpdateTradeScreen();
	end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );

IconHookup( 3, 64, "EXPANSION2_UNIT_ATLAS", Controls.TradeLandImage )
IconHookup( 4, 64, "EXPANSION2_UNIT_ATLAS", Controls.TradeNavalImage )