-------------------------------------------------
-- Religion Overview Popup
-------------------------------------------------
include( "IconSupport" );
include( "InstanceManager" );
include( "InfoTooltipInclude" );
-------------------------------------------------
-- Global Constants
-------------------------------------------------

local bShowWonderStatus = {
["Forgotten"] = true,
["Ruined"] = true,
["Designed"] = true,
["Existing"] = true,
}
local m_Wonders_IM = InstanceManager:new("WonderInstance", "Button", Controls.WonderMainStack);
local ltBlue = {19/255,32/255,46/255,120/255};
local dkBlue = {12/255,22/255,30/255,120/255};
local fWonderSortFunction;
function GetNiceShortCityDescription(index)
	local iPlot = Map.GetPlotByIndex(index)
	if iPlot:IsRevealed(Game.GetActiveTeam()) or Teams[Game.GetActiveTeam()]:IsHasMet(Players[iPlot:GetOwner()]:GetTeam()) then
		return iPlot:GetPlotCity():GetName()
	end
	return "Unknown";
end

function GetNiceShortDescription(i)
	if i == Game.GetActivePlayer() or Teams[Game.GetActiveTeam()]:IsHasMet(Players[i]:GetTeam()) then
		return Players[i]:GetCivilizationShortDescription()
	else
		return "Unknown";
	end
end
function GetNicePlayerID(i)
	if i == Game.GetActivePlayer() or Teams[Game.GetActiveTeam()]:IsHasMet(Players[i]:GetTeam()) then
		return i;
	else
		return -1;
	end
end


function OnClose()
	UIManager:DequeuePopup(ContextPtr);
end
Controls.CloseButton:RegisterCallback(Mouse.eLClick, OnClose);

--make database 2 include

g_Tabs = {
	["GreatPeople"] = {
		Panel = Controls.GreatPeopleInfo,
		SelectHighlight = Controls.BeliefsSelectHighlight,
	},

	["Wonders"] = {
		Panel = Controls.WonderPanel,
		SelectHighlight = Controls.WondersSelectHighlight,
	},

	["Inspirations"] = {
		Panel = Controls.InspirationsInfo,
		SelectHighlight = Controls.InspirationsSelectHighlight,
	},
}

-------------------------------------------------
-- Global Variables
-------------------------------------------------
g_CurrentTab = nil;		-- The currently selected Tab.
	
-------------------------------------------------
-------------------------------------------------
function OnPopupMessage(popupInfo)
	
	local popupType = popupInfo.Type;
	if popupType ~= 4326463 then
		return;
	end
	g_PopupInfo = popupInfo;
	if g_PopupInfo.Data2 then
		if g_PopupInfo.Data2 == 3 then
	        TabSelect("GreatPeople");
		else
	        TabSelect("Wonders");
		end
	end
	
	if( g_PopupInfo.Data1 == 1 ) then
    	if( ContextPtr:IsHidden() == false ) then
    		OnClose();
		else
        	UIManager:QueuePopup( ContextPtr, PopupPriority.InGameUtmost );
    	end
	else
		UIManager:QueuePopup( ContextPtr, PopupPriority.SocialPolicy );
	end   	
end
Events.SerialEventGameMessagePopup.Add( OnPopupMessage );

function CalledGovOver(val1, val2)
   	if( ContextPtr:IsHidden() == false ) then
   		OnClose();
	else
		if not val1 then
			TabSelect("GreatPeople");
		elseif val1 == 2 then
			TabSelect("GreatPeople");
		end
	   	UIManager:QueuePopup( ContextPtr, PopupPriority.InGameUtmost );
   	end
end
LuaEvents.ITG_CallWonderOverview.Add(CalledGovOver)

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function InputHandler( uiMsg, wParam, lParam )
    ----------------------------------------------------------------        
    -- Key Down Processing
    ----------------------------------------------------------------        
    if(uiMsg == KeyEvents.KeyDown) then
        if (wParam == Keys.VK_ESCAPE) then
			OnClose();
			return true;
        end
        
        -- Do Nothing.
        if(wParam == Keys.VK_RETURN) then
			return true;
        end
    end
end
ContextPtr:SetInputHandler( InputHandler );

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function TabSelect(tab)

	for i,v in pairs(g_Tabs) do
		local bHide = i ~= tab;
		v.Panel:SetHide(bHide);
		v.SelectHighlight:SetHide(bHide);
	end
	g_CurrentTab = tab;
	g_Tabs[tab].RefreshContent();	
end
Controls.TabButtonInspirations:RegisterCallback( Mouse.eLClick, function() TabSelect("Inspirations"); end );
Controls.TabButtonBeliefs:RegisterCallback( Mouse.eLClick, function() TabSelect("GreatPeople"); end );
Controls.TabButtonWonder:RegisterCallback( Mouse.eLClick, function() TabSelect("Wonders"); end );

function RefreshInspirations()

end
g_Tabs["Inspirations"].RefreshContent = RefreshInspirations;

function RefreshBeliefs()

end
g_Tabs["GreatPeople"].RefreshContent = RefreshBeliefs;

function OnMapWonderClick(index)
	if index ~= -1 then
		if Map.GetPlotByIndex(index):IsRevealed(Game.GetActiveTeam()) then
			UI.LookAt(Map.GetPlotByIndex(index));
		end
	end
end

function RefreshWonders()
	LuaEvents.ITGSG_RequestWonderUIData()
	local gData = MapModData.ITGSG;
	MapModData.ITGSG = nil;
	m_Wonders_IM:ResetInstances()
	if not gData then
		return
	end
	local gActiveTeam = Game.GetActiveTeam();
	local totot = {}
	for i, data in pairs(gData) do
		local tab = {Name = Locale.Lookup(GameInfo.Buildings[data.Wonder].Description), Wonder = data.Wonder, ToolTip = (data.Tooltip or "")}
		if data.City then
			if Teams[gActiveTeam]:IsHasMet(Players[data.City:GetOwner()]:GetTeam()) then
				tab.CityPlayer = data.City:GetOwner()
			else
				tab.CityPlayer = -1
			end
			if data.City:Plot():IsRevealed(gActiveTeam) then
				tab.CityName = data.City:GetName();
			else
				tab.CityName = "Unknown"
			end
			tab.CityPlotIndex = data.City:Plot():GetPlotIndex();
		else
			tab.CityPlotIndex = -1;
			tab.CityName = ""
			tab.CityPlayer = ""
		end
		if data.Dust then
			tab.Status = "Forgotten"
		elseif data.Ruin then
			tab.Status = "Ruined"
		elseif data.Planned then
			tab.Status = "Designed"
		else
			tab.Status = "Existing"
		end
		if Teams[gActiveTeam]:IsHasMet(Players[data.Constructor]:GetTeam()) then
			tab.Constructor = data.Constructor
			tab.ConstructorText = Players[data.Constructor]:GetCivilizationShortDescription()
		else
			tab.Constructor = -1
			tab.ConstructorText = "Unknown"
		end
		table.insert(totot, tab)
	end
	table.sort(totot, fWonderSortFunction)
	for i, data in pairs(totot) do
		if bShowWonderStatus[data.Status] then
			local controlTable = m_Wonders_IM:GetInstance();
			gcontrolTable = controlTable
			IconHookup( GameInfo.Buildings[data.Wonder].PortraitIndex, 45, GameInfo.Buildings[data.Wonder].IconAtlas, controlTable.Icon1 );
			controlTable.WonderName:SetText(data.Name)
			controlTable.Box1:SetToolTipString(data.ToolTip)
			controlTable.BoxImageID1:SetToolTipString(data.ToolTip)
			controlTable.WonderStatus:SetText(data.Status)
			controlTable.CivIconBG:SetHide(data.CityPlayer == "")
			controlTable.CivIconShadow:SetHide(data.CityPlayer == "")
			controlTable.Icon:SetHide(data.CityPlayer == "")
			if data.CityPlayer ~= "" then
				CivIconHookup( data.CityPlayer, 32, controlTable.Icon, controlTable.CivIconBG, controlTable.CivIconShadow, false, true);  
			end
			if data.Status == "Ruined" or data.Status == "Forgotten" then
				controlTable.Icon1:SetColor({x = 0.8, y = 0.8, z = 0.5, w = 1})
			else
				controlTable.Icon1:SetColor({x = 1, y = 1, z = 1, w = 1})
			end
			if data.Status == "Forgotten" then
				controlTable.Button:SetAlpha(0.5)
			else
				controlTable.Button:SetAlpha(1)
			end
			controlTable.WonderCity:SetText(data.CityName)
			CivIconHookup( data.Constructor, 32, controlTable.Icon2, controlTable.CivIconBG2, controlTable.CivIconShadow2, false, true);  
			controlTable.BoxWonderConstructor:SetText(data.ConstructorText)

			controlTable.Button:SetVoid1(data.CityPlotIndex)
			controlTable.Button:RegisterCallback( Mouse.eLClick, OnMapWonderClick );
		end
	end	
    Controls.WonderMainStack:CalculateSize();
    Controls.WonderMainStack:ReprocessAnchoring();
    Controls.WonderScrollPanel:CalculateInternalSize();
end
g_Tabs["Wonders"].RefreshContent = RefreshWonders;

-- Updates the sort option structure

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function ShowHideHandler( bIsHide, bInitState )
	CivIconHookup( 0, 64, Controls.Icon, Controls.CivIconBG, Controls.CivIconShadow, false, true );
    if( not bInitState ) then
        if( not bIsHide ) then
        	UI.incTurnTimerSemaphore();  
        	Events.SerialEventGameMessagePopupShown(g_PopupInfo);
        	
        	TabSelect(g_CurrentTab);
        else
			if(g_PopupInfo ~= nil) then
				Events.SerialEventGameMessagePopupProcessed.CallImmediate(4326463, 0);
            end
            UI.decTurnTimerSemaphore();
        end
    end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );

----------------------------------------------------------------
-- 'Active' (local human) player has changed
----------------------------------------------------------------
function OnActivePlayerChanged()
	--if (not Controls.ChooseConfirm:IsHidden()) then
	--	Controls.ChooseConfirm:SetHide(true);
	--end
end
Events.GameplaySetActivePlayer.Add(OnActivePlayerChanged);

-----------------------------------------------------------------
ContextPtr:SetHide(true)
TabSelect("GreatPeople");

local cTofSorty = "name"
--fWonderSortFunction
function OnSortWnameClicked()
	if cTofSorty == "name" then
		fWonderSortFunction = AlphabeticalSortFunction("Name", "desc")
		cTofSorty = nil;
	else
		fWonderSortFunction = AlphabeticalSortFunction("Name", "asc")
		cTofSorty = "name"
	end
	RefreshWonders()
end
function OnSortWstatusClicked()
	if cTofSorty == "status" then
		fWonderSortFunction = AlphabeticalSortFunction("Status", "desc")
		cTofSorty = nil;
	else
		fWonderSortFunction = AlphabeticalSortFunction("Status", "asc")
		cTofSorty = "status"
	end
	RefreshWonders()
end
function OnSortWlocationClicked()
	if cTofSorty == "CityName" then
		fWonderSortFunction = AlphabeticalSortFunction("CityName", "desc")
		cTofSorty = nil;
	else
		fWonderSortFunction = AlphabeticalSortFunction("CityName", "asc")
		cTofSorty = "CityName"
	end
	RefreshWonders()
end
function OnSortWconstructorClicked()
	if cTofSorty == "ConstructorText" then
		fWonderSortFunction = AlphabeticalSortFunction("ConstructorText", "desc")
		cTofSorty = nil;
	else
		fWonderSortFunction = AlphabeticalSortFunction("ConstructorText", "asc")
		cTofSorty = "ConstructorText"
	end
	RefreshWonders()
end
function AlphabeticalSortFunction(field, direction)
	if(direction == "asc") then
		return function(a,b)
			return Locale.Compare(a[field], b[field]) == -1;
		end
	elseif(direction == "desc") then
		return function(a,b)
			return Locale.Compare(a[field], b[field]) == 1;
		end
	end
end
function NumericSortFunction(field, direction)
	if(direction == "asc") then
		return function(a,b)
			return a[field] <= b[field];
		end
	elseif(direction == "desc") then
		return function(a,b)
			return a[field] > b[field];
		end
	end
end
fWonderSortFunction = AlphabeticalSortFunction("Name", "asc")

function OnCheckOption1( bIsChecked )
	bShowWonderStatus["Forgotten"] = bIsChecked;
	RefreshWonders()
end
Controls.CheckOption1:RegisterCheckHandler( OnCheckOption1 );

function OnCheckOption2( bIsChecked )
	bShowWonderStatus["Ruined"] = bIsChecked;
	RefreshWonders()
end
Controls.CheckOption2:RegisterCheckHandler( OnCheckOption2 );

function OnCheckOption3( bIsChecked )
	bShowWonderStatus["Designed"] = bIsChecked;
	RefreshWonders()
end
Controls.CheckOption3:RegisterCheckHandler( OnCheckOption3 );

function OnCheckOption4( bIsChecked )
	bShowWonderStatus["Existing"] = bIsChecked;
	RefreshWonders()
end
Controls.CheckOption4:RegisterCheckHandler( OnCheckOption4 );

function OnOptions()	
	Controls.OptionsPanel:SetHide( not Controls.OptionsPanel:IsHidden() );
end
Controls.OptionsButton:RegisterCallback( Mouse.eLClick, OnOptions );
Controls.MainOptionStack:CalculateSize();
Controls.OptionsPanel:DoAutoSize();

Controls.WSname:RegisterCallback( Mouse.eLClick, OnSortWnameClicked );
Controls.WSstatus:RegisterCallback( Mouse.eLClick, OnSortWstatusClicked );
Controls.WSlocation:RegisterCallback( Mouse.eLClick, OnSortWlocationClicked );
Controls.WSconstructor:RegisterCallback( Mouse.eLClick, OnSortWconstructorClicked );
Controls.HelpImageId2:SetAlpha(0.5)